
/* --- FILE GENERATED BY GENOM, DO NOT EDIT BY HAND ------------------- */

/* 
 *      Autonomous Systems Lab, Swiss Federal Institute of Technology.
 * Copyright (c) 1993-2005 LAAS/CNRS
 * All rights reserved.
 *
 * Redistribution and use  in source  and binary  forms,  with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *   1. Redistributions of  source  code must retain the  above copyright
 *      notice, this list of conditions and the following disclaimer.
 *   2. Redistributions in binary form must reproduce the above copyright
 *      notice,  this list of  conditions and the following disclaimer in
 *      the  documentation  and/or  other   materials provided  with  the
 *      distribution.
 *
 * THIS  SOFTWARE IS PROVIDED BY  THE  COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND  ANY  EXPRESS OR IMPLIED  WARRANTIES,  INCLUDING,  BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES  OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR  PURPOSE ARE DISCLAIMED. IN  NO EVENT SHALL THE COPYRIGHT
 * HOLDERS OR      CONTRIBUTORS  BE LIABLE FOR   ANY    DIRECT, INDIRECT,
 * INCIDENTAL,  SPECIAL,  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF  SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN  CONTRACT, STRICT LIABILITY, OR
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE   OF THIS SOFTWARE, EVEN   IF ADVISED OF   THE POSSIBILITY OF SUCH
 * DAMAGE.
 */

/*
 * Control task body
 */

#if defined(__RTAI__) && defined(__KERNEL__)
# define exit(x)	taskDelete(0)
#else
# include <stdlib.h>
# include <unistd.h>
# include <signal.h>
#endif

#include <portLib.h>

#include "icuHeader.h"

#include <taskLib.h> 
#include <errnoLib.h>
#include <commonStructLib.h>
#include <h2evnLib.h>
#include <h2timerLib.h>

#include "icuMsgLib.h"
#include "genom/moduleEvents.h"

/* Print debugging information */
#define GENOM_DEBUG_CNTRLTASK

#ifdef GENOM_DEBUG_CNTRLTASK
# define LOGDBG(x)	logMsg x
#else
# define LOGDBG(x)
#endif

/*-------------------- VARIABLES IMPORTEES ---------------------------------*/

/* Semaphores d'initialisation et de debug */
extern SEM_ID semicuCntrlTaskInit;   

extern char pidFilePath[];

/*-------------------- VARIABLES GLOBALES ---------------------------------*/

/* Nombre d'activites ZOMBIE */
/*XXXXX Voila une variable qu'il serait judicieux de mettre de la SDI de controle a l'occasion XXXXX*/
static int icuNbZombie=0;
static POSTER_ID icuCntrlPosterId;            /* Poster de controle */
static int icuLastActivityNum=-1;
static int icuLastAbsolutActivityNum=-1;

static char *icuExecTaskNameTab[] = {
"RunTask"
};
static BOOL icuSignalAbort;

#define TIMEOUT_CNTRL_TASK 2000 /* 10 sec (pas encore utilise) */

static MODULE_EVENT_STR moduleEventCntrl;

/*---------------- PROTOTYPES DES FONCTIONS LOCALES ------------------------*/

static void   icuCntrlIntEvnExec   (SERV_ID ServId);
static void   icuCntrlExtEvnExec   (SERV_ID ServId);
static void   icuSpawnActivities   (SERV_ID servId);
static void   icuCntrlPosterWrite  (void);
static BOOL   icuAbortActivity     (SERV_ID servId, int activity);
static STATUS icuCntrlInitTask     (SERV_ID *ServId);
static int    allocActivity             (int rqstId, 
					 int requestNum, 
					 int execTaskNum, 
					 int reentrantFlag,
					 SERV_ID servId);
static void   freeActivity              (int activity);
static BOOL   controlExecTaskStatus(SERV_ID servId, int rqstId);
static BOOL   controlExecTaskStatusAndInitRqst(SERV_ID servId, int rqstId);
static void   icuSendFinalReply    (SERV_ID servId, int activity);
static void   icuRqstAbortActivity (SERV_ID ServId, int rqstId);
static void   icuCntrlTaskSuspend  (BOOL giveFlag);
static void   icuReplyAndSuspend   (SERV_ID servId, int rqstId, 
					 BOOL giveFlag);      
static void   icuSignalHandler(int);
static void   icuSignalEnd(void);

/*---------------------- FONCTION EXPORTEE ---------------------------------*/

/*****************************************************************************
 *
 *   icuCntrlTask - Tache de controle du module ICU
 *
 *   Description:
 *   Cette tache attend suspendue l'arrivee de messages dans sa boite de
 *   reception ou d'evenements internes. A la reception d'une lettre,
 *   la requete correspondante est traitee. Les changement d'etat internes 
 *   sont pris en compte et les repliques finales sont emises au client.
 *   Cette tache boucle a jamais.
 */

void
icuCntrlTask()
{
  static SERV_ID icuServId;              /* Id du serveur */

  /* Routine d'initialisation */
  CNTRL_TASK_STATUS = icuCntrlInitTask (&icuServId);
  if (CNTRL_TASK_STATUS == ERROR)
     CNTRL_TASK_BILAN = errnoGet();
  else
     CNTRL_TASK_BILAN = OK;
  
  /* Donner le sem de fin d'initialisation */
  semGive (semicuCntrlTaskInit) ;

  /* Se suspendre en cas de probleme */
  if (CNTRL_TASK_STATUS == ERROR) {
     icuCntrlTaskSuspend(FALSE);
  }

  moduleEventCntrl.moduleNum = 7480;

  /* Record a signal handler */
  icuSignalAbort = FALSE;
  signal(SIGTERM, icuSignalHandler);
  /* Boucler indefiniment */
  FOREVER
    {
      /* Bloquer dans l'attente d'evenements internes et externes */
      if (h2evnSusp (0) != TRUE)
	 icuCntrlTaskSuspend (FALSE);

      /* Prendre l'acces aux SDI */
      commonStructTake ((void *) icuCntrlStrId);
      commonStructTake ((void *) icuDataStrId);
 
      if (icuSignalAbort) {
	      icuSignalEnd();
      }
      /* Traiter les evenements internes */
      icuCntrlIntEvnExec (icuServId);

      /* Traiter les evenements externes */
      icuCntrlExtEvnExec (icuServId);

      /* Lancer les activites en attentes */
      icuSpawnActivities(icuServId);

      /* Mettre a jour le poster de controle */
      icuCntrlPosterWrite();

      CNTRL_TASK_BILAN = errnoGet();

     /* Liberer l'acces aux SDI */
      commonStructGive ((void *) icuDataStrId);
      commonStructGive ((void *) icuCntrlStrId);

    }
}



/*-------------------- ROUTINES LOCALES ------------------------------------*/


 
/*****************************************************************************
 *
 *  icuCntrlIntEvnExec  -  Traiter les evenements internes produits
 *                              par les taches d'execution
 */

static void icuCntrlIntEvnExec(SERV_ID servId)
{
  int activity;

  /* Activites terminees ou activite a reveiller */
  for (activity = 0;  activity < MAX_ACTIVITIES; activity++) {

    /* Activite terminee */
    if (ACTIVITY_EVN(activity) == ETHER) {
      
      /* Est-ce l'activit d'init ? */
      if (INIT_RQST != -1)
	if (ACTIVITY_RQST_TYPE(activity) == INIT_RQST)
	  if (ACTIVITY_BILAN(activity) == OK)
	    INIT_RQST = -1;

      /* On envoie la replique finale */
      icuSendFinalReply (servId, activity);
      
      /* On libere l'activite */
      freeActivity (activity);
    }
    
    /* Activite suspendue */
    else if (ACTIVITY_EVN(activity) == ZOMBIE) {
      
      /* On envoie la replique finale */
      icuSendFinalReply (servId, activity);

      /* On bloque l'activite */
      icuNbZombie++;
      ACTIVITY_STATUS(activity) = ZOMBIE;
      ACTIVITY_EVN(activity) = NO_EVENT;
    }

    /* Activite en attente */
    else if (ACTIVITY_EVN(activity) == SLEEP) {
      ACTIVITY_STATUS(activity) = SLEEP;
      ACTIVITY_EVN(activity) = NO_EVENT;
    }

    /* XXX Eveil systematique - le traitement des requetes est fait apres -> si l'une d'elle met le flag EXEC_TASK_WAKE_UP_FLAG on ne le voie pas ici */
/*    else if (ACTIVITY_STATUS(activity) == SLEEP) {*/
/*      && EXEC_TASK_WAKE_UP_FLAG(ACTIVITY_TASK_NUM(activity))) {*/
/*      ACTIVITY_EVN(activity) = EXEC;
      h2evnSignal(EXEC_TASK_ID(ACTIVITY_TASK_NUM(activity)));
    }*/

  } /* for */

  /* Reset demande d'eveil (pas utilise actuelement) */
/*  for (i=0; i<ICU_NB_EXEC_TASK; i++)
    EXEC_TASK_WAKE_UP_FLAG(i) = FALSE;*/
}


/*****************************************************************************
 *
 *  icuCntrlExtEvnExec  -  Traiter les evenements externes: arrivee
 *                              de requetes
 */

static void icuCntrlExtEvnExec (SERV_ID icuServId)
     
{
  int mboxStatus;                    /* Etat de la boite aux lettres */
  int status;
  int i;
  BOOL wakeUp=FALSE;
  
  /* Obtenir l'etat de la boite aux lettres de reception de requetes */
  status = (mboxStatus = csMboxStatus (RCV_MBOX));
 
  /* Execution de la requete */
  if (mboxStatus & RCV_MBOX)  {
    status = csServRqstExec (icuServId);
    wakeUp = TRUE;
  }

  /* Suspendre la tache, en cas d'erreur */
  if (status == ERROR)
    icuCntrlTaskSuspend (TRUE);

  /* On eveil toutes les taches d'exec (non periodique) un petit coup */
  if (wakeUp)
    for (i=0; i<ICU_NB_EXEC_TASK; i++)
      if (!EXEC_TASK_PERIOD(i)) h2evnSignal(EXEC_TASK_ID(i));
}

/*****************************************************************************
 *
 *  icuSpawnActivities  -  Lancer les activitees pretes
 *
 *  Description:  Pour chaque activite en attente, regarde si les activites
 *                incompatibles ont ete arretees. En ce cas lance l'activite.
 */

static void icuSpawnActivities (SERV_ID servId)
     
{
  int activity;
  int i, j;
  int nbIncomp;

  /* Pour chaque activite */
  for (activity = 0; activity < MAX_ACTIVITIES; activity++) 

    /* Activite en attente */
    if (ACTIVITY_STATUS(activity) == INIT 
	&& ACTIVITY_EVN(activity) == NO_EVENT) {
      
      /* Activite incompatibles */
      nbIncomp = ACTIVITY_NB_INCOMP(activity);
      
      /* Liste des activites encore incompatibles */
      for (i=0, j=0; i<nbIncomp; i++)
	if (ACTIVITY_STATUS(ACTIVITY_TAB_INCOMP(activity)[i]) == ETHER
	    || ACTIVITY_STATUS(ACTIVITY_TAB_INCOMP(activity)[i]) == ZOMBIE)
	  ACTIVITY_NB_INCOMP(activity)--;
	else
	  ACTIVITY_TAB_INCOMP(activity)[j++]
	    = ACTIVITY_TAB_INCOMP(activity)[i]; 
      
      /* Il n'y a plus d'activite incompatible */
      if (ACTIVITY_NB_INCOMP(activity) == 0) {
    
	/* Declencher l'execution */
	ACTIVITY_EVN(activity) = START;
	h2evnSignal(EXEC_TASK_ID(ACTIVITY_TASK_NUM(activity)));
	
	/* Envoyer la replique intermediaire */
	if (csServReplySend (servId, ACTIVITY_RQST_ID(activity), INTERMED_REPLY, 
			     OK, (void *) &ACTIVITY_ID(activity), sizeof(int), 
			     (FUNCPTR) NULL) != OK)
	  icuCntrlTaskSuspend (TRUE);
      }      /* Activite a lancer */
    }        /* Activite en attente */
}


/*****************************************************************************
 *
 *  icuCntrlPosterWrite  - Met a jour le poster de controle
 *
 *  Description: Se suspend en cas d'erreur
 */

static void icuCntrlPosterWrite ()
{
  if (posterWrite (icuCntrlPosterId, 0, (void *) icuCntrlStrId,
		   sizeof (ICU_CNTRL_STR))
      != sizeof (ICU_CNTRL_STR)) {
    h2perror("icuCntrlPosterWrite");
    icuCntrlTaskSuspend (TRUE);
  }
}

/*-------------------- FONCTIONS AUXILIAIRES -------------------------------*/

static void icuCntrlInitDisplay(SERV_ID servId, int rqstId);
static void icuCntrlPomTagging(SERV_ID servId, int rqstId);
static void icuCntrlAutoMode(SERV_ID servId, int rqstId);
static void icuCntrlSaveFace(SERV_ID servId, int rqstId);
static void icuCntrlCaptureImage(SERV_ID servId, int rqstId);
static void icuCntrlCaptureVideo(SERV_ID servId, int rqstId);
static void icuCntrlSetViamPosterName(SERV_ID servId, int rqstId);
static void icuCntrlEnableDisplay(SERV_ID servId, int rqstId);
static void icuCntrlSetCameraPos(SERV_ID servId, int rqstId);
static void icuCntrlTrackIt(SERV_ID servId, int rqstId);
static void icuCntrlWaitForSomebody(SERV_ID servId, int rqstId);
static void icuCntrlWaitForSomeone(SERV_ID servId, int rqstId);
static void icuCntrlWaitForNobody(SERV_ID servId, int rqstId);
static void icuCntrlWaitForNone(SERV_ID servId, int rqstId);
static void icuCntrlWaitForNobodyKnown(SERV_ID servId, int rqstId);
static void icuCntrlWaitForPersonListChange(SERV_ID servId, int rqstId);
static void icuCntrlProcessSetModality(SERV_ID servId, int rqstId);
static void icuCntrlFacerecSetWorkingPath(SERV_ID servId, int rqstId);
static void icuCntrlFacerecSetParams(SERV_ID servId, int rqstId);
static void icuCntrlFacerecInit(SERV_ID servId, int rqstId);
static void icuCntrlFacerecAddSomebody(SERV_ID servId, int rqstId);
static void icuCntrlFacerecSetPeople(SERV_ID servId, int rqstId);
static void icuCntrlFacerecRecognizeAll(SERV_ID servId, int rqstId);
static void icuCntrlFacerecLoadSomebody(SERV_ID servId, int rqstId);
static void icuCntrlFacerecUnloadSomebody(SERV_ID servId, int rqstId);
static void icuCntrlFacerecLoadEverybody(SERV_ID servId, int rqstId);
static void icuCntrlFacerecUnloadEverybody(SERV_ID servId, int rqstId);
static void icuCntrlWaitForModalityChange(SERV_ID servId, int rqstId);
static void icuCntrlLaunchPlatine(SERV_ID servId, int rqstId);
static void icuCntrlCmdPtu(SERV_ID servId, int rqstId);
static void icuCntrlIcuInitRFID(SERV_ID servId, int rqstId);
static void icuCntrlIcuCloseRFID(SERV_ID servId, int rqstId);
static void icuCntrlIcuAcquireRFID(SERV_ID servId, int rqstId);
static void icuCntrlIcuAcquireTag(SERV_ID servId, int rqstId);

static void (*icuTabRequestFunc[])() = {
icuCntrlInitDisplay,
icuCntrlPomTagging,
icuCntrlAutoMode,
icuCntrlSaveFace,
icuCntrlCaptureImage,
icuCntrlCaptureVideo,
icuCntrlSetViamPosterName,
icuCntrlEnableDisplay,
icuCntrlSetCameraPos,
icuCntrlTrackIt,
icuCntrlWaitForSomebody,
icuCntrlWaitForSomeone,
icuCntrlWaitForNobody,
icuCntrlWaitForNone,
icuCntrlWaitForNobodyKnown,
icuCntrlWaitForPersonListChange,
icuCntrlProcessSetModality,
icuCntrlFacerecSetWorkingPath,
icuCntrlFacerecSetParams,
icuCntrlFacerecInit,
icuCntrlFacerecAddSomebody,
icuCntrlFacerecSetPeople,
icuCntrlFacerecRecognizeAll,
icuCntrlFacerecLoadSomebody,
icuCntrlFacerecUnloadSomebody,
icuCntrlFacerecLoadEverybody,
icuCntrlFacerecUnloadEverybody,
icuCntrlWaitForModalityChange,
icuCntrlLaunchPlatine,
icuCntrlCmdPtu,
icuCntrlIcuInitRFID,
icuCntrlIcuCloseRFID,
icuCntrlIcuAcquireRFID,
icuCntrlIcuAcquireTag
};
static int icuTabRequestNum[] = {
0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20, 21, 22, 23, 24, 25, 26, 27, 28, 29, 30, 31, 32, 33};



/*****************************************************************************
 *
 *  icuCntrlInitTask - Routine d'initialisation de la tache de controle
 */


static STATUS
icuCntrlInitTask(SERV_ID *icuServId)
{
  int i;

  CNTRL_SDI_F = SDI_F;

  /* Creation de la boite aux lettres de reception des requetes */
  if (csMboxInit (ICU_MBOX_NAME, 
		  ICU_MBOX_RQST_SIZE, 0)
      != OK) {
    logMsg("icuCntrlTask: csMboxInit: ");
    h2printErrno(errnoGet());
    return (ERROR);
  }
  LOGDBG(("icuCntrlInitTask: created mailbox\n"));
  
  /* S'initialiser comme serveur */
  if (csServInitN (ICU_MAX_RQST_SIZE, 
		  ICU_MAX_REPLY_SIZE >= ICU_MAX_INTERMED_REPLY_SIZE ?
		  ICU_MAX_REPLY_SIZE : ICU_MAX_INTERMED_REPLY_SIZE, 
		  NB_RQST_TYPE,
		  icuServId) 
      != OK) { 
    logMsg("icuCntrlTask: csServInit: ");
    h2printErrno(errnoGet());
    return (ERROR);
  }
  LOGDBG(("icuCntrlInitTask: initialized mailbox as a server\n"));
  
  /* Installer les routines de traitement des requetes */
  for (i=0; i<NB_RQST_TYPE-1; i++) {
    if (csServFuncInstall (*icuServId, icuTabRequestNum[i], 
			   (FUNCPTR) icuTabRequestFunc[i]) != OK) {
      logMsg("icuCntrlTask: csServFuncInstall: ");
      h2printErrno(errnoGet());
      return (ERROR);
    }
  }
  LOGDBG(("icuCntrlInitTask: installed requests\n"));

  /* Installer la requete abort
     (c'est le parser qui attribue le premier numero libre de requete ) */
  if (csServFuncInstall (*icuServId, 34, 
			 (FUNCPTR) icuRqstAbortActivity) != OK) {
    logMsg("icuCntrlTask: csServFuncInstall: ");
    h2printErrno(errnoGet());
    return (ERROR);
  }
  LOGDBG(("icuCntrlInitTask: installed abort request\n"));
  
  /* Initialiser la structure de controle */
  STOP_MODULE_FLAG = FALSE;
  for (i=0; i<MAX_ACTIVITIES; i++) {
    ACTIVITY_STATUS(i) = ETHER;
    ACTIVITY_EVN(i) = NO_EVENT;
    ACTIVITY_TASK_NUM(i) = -1;
  }
  for (i=0; i<ICU_NB_EXEC_TASK; i++)
    EXEC_TASK_NB_ACTI(i) = 0;

  CNTRL_NB_EXEC_TASKS = ICU_NB_EXEC_TASK;

  /* La requete d'init */
  INIT_RQST = 0;

  /* Creer le poster de controle */
  if (posterCreate (ICU_CNTRL_POSTER_NAME, sizeof (ICU_CNTRL_STR), 
                    &icuCntrlPosterId) != OK) {
    logMsg("icuCntrlTask: posterCreate: ");
    h2printErrno(errnoGet());
    return (ERROR);
  }
  LOGDBG(("icuCntrlInitTask: created control poster\n"));
 
  /* Obtenir son propre identificateur de tache */
  CNTRL_TASK_ID = taskIdSelf ();
  NB_ACTIVITIES = 0;

  /* Record errors */
  icuRecordH2errMsgs();

  /* C'est OK */
  return (OK);
}

/*****************************************************************************
 *
 *  icuAbortActivity -  Arrete une activite
 *
 *  Description : Demande l'arret de l'activite si celle ci n'est pas
 *                deja arretee. Si elle est deja arretee, libere l'activite.
 *
 *  Retourne : TRUE si activite terminee, FALSE sinon
 */
 
static BOOL icuAbortActivity (SERV_ID servId, int activity)
{
  
  switch (ACTIVITY_STATUS(activity)) {
    
    /* Activite inexistante */
  case ETHER:
    return(TRUE);

    /* Activite plantee */
  case ZOMBIE:
    freeActivity(activity);
    return(TRUE);
    
    /* Activite pas encore demarree */
  case INIT:
    icuSendFinalReply(servId, activity);
    freeActivity (activity);
    return(TRUE);

    /* L'acitivite roupille */
  case SLEEP:
    ACTIVITY_EVN(activity) = INTER;
    h2evnSignal(EXEC_TASK_ID(ACTIVITY_TASK_NUM(activity)));
    return(FALSE);
    
    /* Activite en cours d'exec: START, EXEC, END, FAIL, INTER , SLEEP */
  default:
    switch (ACTIVITY_EVN(activity)) {
      
      /* Activite terminee */
    case ETHER:
      icuSendFinalReply(servId, activity);
      freeActivity (activity);
      return(TRUE);
      
      /* Activite va se planter */
    case FAIL:
      return(FALSE);

      /* Interruption deja demandee */
    case INTER:
      return FALSE;

      /* Demande d'interruption END, START, EXEC, SLEEP? */
    default:
      ACTIVITY_EVN(activity) = INTER;
      h2evnSignal(EXEC_TASK_ID(ACTIVITY_TASK_NUM(activity)));
      return(FALSE);
    }
  }
}

/****************************************************************************
 *
 *  allocActivity  -  Allocation/Initialisation d'une activite
 *
 *  Retourne : Numero de l'activite ou -1
 */

static int allocActivity (int rqstId,     /* Id de la requete */
			  int requestNum, /* Type d'activite */
			  int execTaskNum,
			  int reentrantFlag,
			  SERV_ID servId)
{
  int activity, i;
  
  /* Il y a une activite ZOMBIE */
  if (icuNbZombie != 0) {
    if (csServReplySend (servId, rqstId, FINAL_REPLY, 
			 S_icu_stdGenoM_WAIT_ABORT_ZOMBIE_ACTIVITY, 
			 (void *) NULL, 0, (FUNCPTR) NULL) != OK)
      icuCntrlTaskSuspend (TRUE);
    return -1;
  }

  /* Recherche d'une activite libre */
  /*  activity = 0;
  while(ACTIVITY_STATUS(activity) != ETHER && activity < MAX_ACTIVITIES) 
    activity++;*/

  
  for(i=0; i<MAX_ACTIVITIES; i++) {
    activity = (icuLastActivityNum+1+i)%MAX_ACTIVITIES;
    if (ACTIVITY_STATUS(activity) == ETHER)
      break;
  }

  /* Pas d'activite libre */
  if (/*activity*/ i == MAX_ACTIVITIES) {
    if (csServReplySend (servId, rqstId, FINAL_REPLY, 
			 S_icu_stdGenoM_TOO_MANY_ACTIVITIES, 
			 (void *) NULL, 0, (FUNCPTR) NULL) != OK)
      icuCntrlTaskSuspend (TRUE);
    return -1;
  }

  icuLastActivityNum = activity;
  icuLastAbsolutActivityNum++;

  /* Cree l'activite */
  ACTIVITY_STATUS(activity) = INIT;
  ACTIVITY_EVN(activity) = NO_EVENT;
  ACTIVITY_RQST_TYPE(activity) = requestNum;
  ACTIVITY_RQST_ID(activity) = rqstId;
  ACTIVITY_ID(activity) = icuLastAbsolutActivityNum;
  ACTIVITY_BILAN(activity) = OK;
  ACTIVITY_NB_INCOMP(activity) = 0;
  ACTIVITY_TASK_NUM(activity) = execTaskNum;
  ACTIVITY_REENTRANCE(activity) = reentrantFlag;

  /* Initialise les parametres */
  ACTIVITY_INPUT_ID(activity) = NULL;
  ACTIVITY_INPUT_SIZE(activity) = 0;
  ACTIVITY_OUTPUT_ID(activity) = NULL;
  ACTIVITY_OUTPUT_SIZE(activity) = 0;

  /* Une activite de plus */
  EXEC_TASK_NB_ACTI(execTaskNum)++;
  NB_ACTIVITIES++;

  return(activity);
}

/****************************************************************************
 *
 *  controlExecTaskStatus  -  Controle l'tat des tches d'exec
 *
 */
static BOOL controlExecTaskStatus(SERV_ID servId, int rqstId)
{
  int i;

  /* Exec tasks status */
  for (i=0; i<ICU_NB_EXEC_TASK; i++)
    if (EXEC_TASK_STATUS(i) == ERROR) {
      /* XXXXX Il faudrait definir un bilan spcifique:
	 S_icu_stdGenoM_EXEC_TASK_SUSPENDED */
      csServReplySend (servId, rqstId, FINAL_REPLY, 
		       S_icu_stdGenoM_SYSTEM_ERROR,
		       (void *) NULL, 0, (FUNCPTR) NULL);
      return FALSE;
    }

  return TRUE;
}

/****************************************************************************
 *
 *  controlExecTaskStatusAndInitRqst  -  Controle l'tat des tches d'exec
 *					 et la requete d'init
 *
 */
static BOOL controlExecTaskStatusAndInitRqst(SERV_ID servId, int rqstId)
{
  int i;

  /* Exec tasks status */
  for (i=0; i<ICU_NB_EXEC_TASK; i++)
    if (EXEC_TASK_STATUS(i) == ERROR) {
      /* XXXXX Il faudrait definir un bilan spcifique:
	 S_icu_stdGenoM_EXEC_TASK_SUSPENDED */
      csServReplySend (servId, rqstId, FINAL_REPLY, 
		       S_icu_stdGenoM_SYSTEM_ERROR,
		       (void *) NULL, 0, (FUNCPTR) NULL);
      return FALSE;
    }

  /* Init request */
  if (INIT_RQST != -1) {
    csServReplySend (servId, rqstId, FINAL_REPLY, 
		     S_icu_stdGenoM_WAIT_INIT_RQST, 
		     (void *) NULL, 0, (FUNCPTR) NULL);
    return FALSE;
  }    

  return TRUE;
}

/****************************************************************************
 *
 *  freeActivity  -  Retire l'activite du tableau d'activite
 */

static void freeActivity (int activity)
{
  int execTaskNum;
  
  /* C'etait une activite ZOMBIE */
  if (ACTIVITY_STATUS(activity) == ZOMBIE) icuNbZombie--;

  /* Petit netoyage */
  execTaskNum = ACTIVITY_TASK_NUM(activity);
  ACTIVITY_STATUS(activity) = ETHER;
  ACTIVITY_EVN(activity) = NO_EVENT;
  ACTIVITY_TASK_NUM(activity) = -1;
  EXEC_TASK_NB_ACTI(execTaskNum)--;
  NB_ACTIVITIES --;
}


/****************************************************************************
 *
 *  icuSendFinalReply  -  Envoie la replique finale
 *
 *  Description : Envoie la replique finale avec le bilan d'execution si 
 *                tout c'est bien termine' (dernier etat END), ou avec un 
 *                message d'interruption (dernier etat INTER ou INIT)
 *                ou d'echec (dernier etat FAIL).
 */

static void icuSendFinalReply (SERV_ID servId,
				    int activity)

{
  int bilan;

  /* 
   * Select bilan according to the last event 
   */
  switch(ACTIVITY_EVN(activity)) {

    /* Normal end or "clean" interruption */
  case ETHER:
    /* Clean interruption or not yet started  */
    if (ACTIVITY_STATUS(activity) == INIT ||
	ACTIVITY_STATUS(activity) == INTER) {
      bilan = S_icu_stdGenoM_ACTIVITY_INTERRUPTED;
      ACTIVITY_BILAN(activity) = bilan;
    }
    
    /* Normal end */
    else {
      bilan = ACTIVITY_BILAN(activity);
      if (bilan != OK && !H2_MODULE_ERR_FLAG(bilan)) {
	CNTRL_TASK_BILAN = bilan;
	bilan = S_icu_stdGenoM_SYSTEM_ERROR;
      }
    }
    break;
    
    /* Activity not yet started */
  case START:
    if (ACTIVITY_STATUS(activity) == INIT) {
      bilan = S_icu_stdGenoM_ACTIVITY_INTERRUPTED;
      ACTIVITY_BILAN(activity) = bilan;      
    }
    break;

    /* Echec */
  case ZOMBIE:
    bilan = S_icu_stdGenoM_ACTIVITY_FAILED;
    break;

    /* Interruption before starting (state == INIT) */
  case NO_EVENT:
    bilan = S_icu_stdGenoM_ACTIVITY_INTERRUPTED;
    ACTIVITY_BILAN(activity) = bilan;
    break;

    /* Impossible termination event */
  default:
    logMsg("icuCntrlTask: activity %d state %d event %d !?! \n", 
	   activity, ACTIVITY_STATUS(activity), ACTIVITY_EVN(activity));
    errnoSet(S_icu_stdGenoM_FORBIDDEN_ACTIVITY_TRANSITION);
    bilan = S_icu_stdGenoM_FORBIDDEN_ACTIVITY_TRANSITION;
    ACTIVITY_BILAN(activity) = bilan;
  }
  
  /* 
   * Send the final reply 
   */
  if (csServReplySend (servId, ACTIVITY_RQST_ID(activity), FINAL_REPLY, 
		       bilan, (void *) ACTIVITY_OUTPUT_ID(activity),
		       ACTIVITY_OUTPUT_SIZE(activity), (FUNCPTR) NULL) != OK)
    icuCntrlTaskSuspend (TRUE);

  moduleEventCntrl.eventType = STATE_END_EVENT;
  moduleEventCntrl.activityNum = activity;
  moduleEventCntrl.activityState = ACTIVITY_EVN(activity);
  moduleEventCntrl.rqstType = ACTIVITY_RQST_TYPE(activity);
  moduleEventCntrl.taskNum = ACTIVITY_TASK_NUM(activity);
  sendModuleEvent(&moduleEventCntrl);
}


/****************************************************************************
 *
 *  icuCntrlTaskSuspend  -  Suspension de la tache de controle
 *
 *  Description : recupere le bilan, puis suspend la tache de controle
 *
 */

static void icuCntrlTaskSuspend (BOOL giveFlag)

{
  char strerr[64];

  /* Indiquer qu'une erreur a ete detectee */
  CNTRL_TASK_STATUS = ERROR;
  CNTRL_TASK_BILAN = errnoGet();

  logMsg("icuCntrlTaskSuspend: %s\n", h2getErrMsg(CNTRL_TASK_BILAN, strerr, 64));

  /* Mettre a jour le poster de controle */
  if (posterWrite (icuCntrlPosterId, 0, (void *) icuCntrlStrId,
		   sizeof (ICU_CNTRL_STR)) != sizeof (ICU_CNTRL_STR)) {
    h2perror("icuCntrlTaskSuspend: posterWrite");
  }

   /* Liberer les SDIs */
  if (giveFlag) {
    commonStructGive ((void *) icuDataStrId);
    commonStructGive ((void *) icuCntrlStrId);    
  }

  /* Suspendre la tache */
  taskSuspend (0);
}



/****************************************************************************
 *
 *  icuReplyAndSuspend  -  Repliquer et suspendre
 *
 *  Description : envoie la replique puis se suspend
 */

static void icuReplyAndSuspend (SERV_ID servId,  
				     int rqstId,
				     BOOL giveFlag)
{
  char strerr[64];

  /* Indiquer qu'une erreur a ete detectee */
  CNTRL_TASK_STATUS = ERROR;
  CNTRL_TASK_BILAN = errnoGet();

  logMsg("icuCntrlTaskReplyAndSuspend: %s\n", 
	 h2getErrMsg(CNTRL_TASK_BILAN, strerr, 64));

  /* Mettre a jour le poster de controle */
  if (posterWrite (icuCntrlPosterId, 0, (void *) icuCntrlStrId,
		   sizeof (ICU_CNTRL_STR)) != sizeof (ICU_CNTRL_STR)) {
    h2perror ("icuReplyAndSuspend: posterWrite");
  }

/*XXX  icuCntrlPosterShow();*/


  /* Liberer les SDIs */
  if (giveFlag) {
    commonStructGive ((void *) icuDataStrId);
    commonStructGive ((void *) icuCntrlStrId);    
  }

  /* Envoyer le code d'erreur vers le client */
  (void) csServReplySend (servId, rqstId, FINAL_REPLY, CNTRL_TASK_BILAN, 
			  (void *) NULL, 0, (FUNCPTR) NULL);

  /* Suspendre la tache */
  taskSuspend (0);

}

/*------------------- TRAITEMENT DES REQUETES  -----------------------------*/


/****************************************************************************
 *
 *  icuRqstAbortActivity  -  Traitement de la requete Abort
 */

static void icuRqstAbortActivity (SERV_ID servId, int rqstId)

{
  int activityId; /* Absolut activity id */
  int activity; /* Local activity Id */
  int bilan = OK;
  int i;

  /* Get activity to abort */
  if (csServRqstParamsGet (servId, rqstId, (void *) &activityId, 
			   sizeof(int), (FUNCPTR) NULL) != OK)
    icuReplyAndSuspend (servId, rqstId, TRUE);

  /* Activity identification */
  if (activityId < 0) {

    switch (activityId) 
      {
	/* Activity -88: restart suspended exec tasks */
      case GENOM_RESUME_EXEC_TASK:
	for (i=0; i<ICU_NB_EXEC_TASK; i++) {
	  taskResume(EXEC_TASK_ID(i));
	  EXEC_TASK_STATUS(i) = OK;
	}
	break;
    
      case GENOM_PRINT_TIME_RQST:
	if (GENOM_PRINT_TIME_FLAG) {
	  GENOM_PRINT_TIME_FLAG = 0;
	  /* reset time max for next display */
	  for (i=0; i<ICU_NB_EXEC_TASK; i++) {
	    EXEC_TASK_MAX_PERIOD(i) = 0;
	  }
	}
	else {
	  GENOM_PRINT_TIME_FLAG = 1;
	}
	break;

      case GENOM_VERBOSE_RQST:
	if (GENOM_VERBOSE_LEVEL)
	  GENOM_VERBOSE_LEVEL = 0;
	else
	  GENOM_VERBOSE_LEVEL = 1;
	break;

	/* Activity -99: stop the module */
      case GENOM_END_MODULE:

	/* Test if there is on going activities */
	if (NB_ACTIVITIES != 0) {
	  bilan = S_icu_stdGenoM_WAIT_ABORT_ZOMBIE_ACTIVITY;
	  /* bilan = S_stdGenoM_ACTIVITIES_stdGenoM_REMAINED; */
	  break;
	}

	/* End the module */
	else {

	  STOP_MODULE_FLAG = TRUE;
	  for (i=ICU_NB_EXEC_TASK-1; i > -1; i--) 
	    EXEC_TASK_WAKE_UP_FLAG(i) = TRUE;

	  /* give back Internal Data Structures */
	  commonStructGive ((void *) icuDataStrId);
	  commonStructGive ((void *) icuCntrlStrId);

	  /* Interrupt exec task */
	  for (i=ICU_NB_EXEC_TASK-1; i > -1; i--) {
	    logMsg("Killing task %s ... \n", icuExecTaskNameTab[i]);
	    taskResume(EXEC_TASK_ID(i));
	    h2evnSignal(EXEC_TASK_ID(i));
	  }

	  /* Wait end */
	  for (i=ICU_NB_EXEC_TASK-1; i > -1; i--) {
	    while (EXEC_TASK_WAKE_UP_FLAG(i))
	      h2evnSusp(0);
	    logMsg("    ... task %s killed\n", icuExecTaskNameTab[i]);
	  }

	  /* send final reply */
	  csServReplySend (servId, rqstId, FINAL_REPLY, bilan, 
			   (void *) NULL, 0, (FUNCPTR) NULL);

	  
	  /* Destruction of mboxes, SDI and posters and BYE ! */
	  csMboxEnd();
	  commonStructDelete ((void *) icuDataStrId);
	  commonStructDelete ((void *) icuCntrlStrId);
	  posterDelete(icuCntrlPosterId);
	  logMsg("icuCntrlTask ended\n");
	  /* Remove PID file */
	  unlink(pidFilePath);
	  exit(0);
	}
	break;

      default:
	bilan = S_icu_stdGenoM_UNKNOWN_ACTIVITY;
	break;
      } /* switch */
  }
  
  /* Numro d'activit positive inconnu */
  else if(activityId > icuLastAbsolutActivityNum) {
    bilan = S_icu_stdGenoM_UNKNOWN_ACTIVITY;
  }
    
  /* Numro valide */
  else {

    /* Recherche de l'activit */
    for (activity = 0; activity < MAX_ACTIVITIES; activity++)
      if (ACTIVITY_ID(activity) == activityId)
	break;

    /* Activit dj termine */
    if (activity == MAX_ACTIVITIES || ACTIVITY_STATUS(activity) == ETHER)
      bilan = S_icu_stdGenoM_ACTIVITY_ALREADY_ENDED;

    /* Activit trouve: interruption */
    else 
      icuAbortActivity(servId, activity);
  }

  /* Send final reply */
  if (csServReplySend (servId, rqstId, FINAL_REPLY, bilan, 
		       (void *) NULL, 0, (FUNCPTR) NULL) != OK)
    icuCntrlTaskSuspend (TRUE); 
}

/****************************************************************************
 * icuSignalHandler - Handle signals in a module
 */
void
icuSignalHandler(int sig)
{
	printf("Signal received %d\n", sig);
	icuSignalAbort = TRUE;	/* indicate that we wanna end */
	h2evnSignal(CNTRL_TASK_ID);	/* wake up control task */
}

/****************************************************************************
 *
 * icuEnd - terminate a module upon signal reception
 */
static void
icuSignalEnd(void)
{
	int i;

	STOP_MODULE_FLAG = TRUE;
	for (i=ICU_NB_EXEC_TASK-1; i > -1; i--) 
		EXEC_TASK_WAKE_UP_FLAG(i) = TRUE;
	
	/* give back Internal Data Structures */
	commonStructGive ((void *) icuDataStrId);
	commonStructGive ((void *) icuCntrlStrId);
	
	/* Interrupt exec task */
	for (i=ICU_NB_EXEC_TASK-1; i > -1; i--) {
		logMsg("Killing task %s ... \n", icuExecTaskNameTab[i]);
		taskResume(EXEC_TASK_ID(i));
		h2evnSignal(EXEC_TASK_ID(i));
	}
	
	/* Wait end */
	for (i=ICU_NB_EXEC_TASK-1; i > -1; i--) {
		while (EXEC_TASK_WAKE_UP_FLAG(i))
			h2evnSusp(0);
		logMsg("    ... task %s killed\n", icuExecTaskNameTab[i]);
	}
	
	
	/* Destruction of mboxes, SDI and posters and BYE ! */
	csMboxEnd();
	commonStructDelete ((void *) icuDataStrId);
	commonStructDelete ((void *) icuCntrlStrId);
	posterDelete(icuCntrlPosterId);
	logMsg("icuCntrlTask ended\n");
	  /* Remove PID file */
	  unlink(pidFilePath);
	  exit(0);
}

/* Requetes de type controle */
/* Requetes de type execution */



/*****************************************************************************
 *
 *  icuCntrlInitDisplay - Execution request
 */

#if (0 /* control func */)
  extern int dummy ();
#endif 

static void icuCntrlInitDisplay (SERV_ID servId, int rqstId)

{
  static int icuInitDisplayCompatibility[] = { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 };
  int activity;
  int i;
  int *compatibilityTab = icuInitDisplayCompatibility;
#if ((1) && 0 /* Inputs + control func */)
  ICU_AFFICHAGE display;
#endif

  /*--------------------------------------------------------------
   * Control init done and exec tasks status 
   */
#if (0 /* not init rqst */)
  if (!controlExecTaskStatusAndInitRqst(servId, rqstId))
    return;  
#endif
 
  /*--------------------------------------------------------------
   * Activity allocation
   */
  activity = allocActivity(rqstId, 0, 0, 0, servId);
  if (activity == -1) return;

  /*--------------------------------------------------------------
   * Call control func
   */
#if (0 /* control func */)
  {
    STATUS status;
    int bilan;

#if ((1)  /* Input */)
    /* Get inputs */
    if (csServRqstParamsGet (servId, rqstId, (void *) &display, 
			     sizeof((*icuDataStrId).display), (FUNCPTR) NULL) != OK)
      icuReplyAndSuspend (servId, rqstId, TRUE);
    status = dummy (&display, &bilan);
#else
    status = dummy (&bilan);
#endif

    /* Control not OK: stop there */
    if (status != OK) {
      freeActivity(activity);
      if (bilan == OK)
	bilan = S_icu_stdGenoM_CONTROL_CODEL_ERROR;
      if (csServReplySend (servId, rqstId, FINAL_REPLY, bilan, 
			   (void *) NULL, 0, (FUNCPTR) NULL) != OK)
	icuCntrlTaskSuspend (TRUE);
      return;
    }
  }
#endif

 /*-------------------------------------------------------------
  * Record inputs
  *       XXXXXXXX NON C'EST TROP TOT -> INTERROMPRE LES ACTIVITES AVANT !
  */
#if ((1) /* Input */)

  /* Get inputs address */
#if (0 /* multiple request  */)
  ACTIVITY_INPUT_ID(activity) = &(icuDataStrId->InitDisplayInput)[activity];
#else
  ACTIVITY_INPUT_ID(activity) = &((*icuDataStrId).display);
#endif                
  ACTIVITY_INPUT_SIZE(activity) = sizeof((*icuDataStrId).display);

  /* Record inputs */
#if (0 /* control func */)
  memcpy ((void *) ACTIVITY_INPUT_ID(activity), (void *) &display, 
	  ACTIVITY_INPUT_SIZE(activity));
#else
  if (csServRqstParamsGet (servId, rqstId, ACTIVITY_INPUT_ID(activity), 
			   sizeof((*icuDataStrId).display), (FUNCPTR) NULL) != OK)
    icuReplyAndSuspend (servId, rqstId, TRUE);
#endif
#endif          
  
 /*-------------------------------------------------------------
  * Get outputs address
  */
#if ((0) /* Output */)
#if (0 /* Multiple request */)
  ACTIVITY_OUTPUT_ID(activity) = 
    &(icuDataStrId->InitDisplayOutput)[activity];
#else
  ACTIVITY_OUTPUT_ID(activity) = NULL;
#endif            
  ACTIVITY_OUTPUT_SIZE(activity) = 0;
#endif          

  moduleEventCntrl.eventType = STATE_START_EVENT;
  moduleEventCntrl.activityNum = activity;
  moduleEventCntrl.activityState = INIT;
  moduleEventCntrl.rqstType = 0;
  moduleEventCntrl.taskNum = 0;
  sendModuleEvent(&moduleEventCntrl);

  /*-------------------------------------------------------------
   * Interruption of the incompatible activities
   */
  for (i = 0; i < MAX_ACTIVITIES; i++) 
    if (i != activity && ACTIVITY_STATUS(i) != ETHER 
	&& ! compatibilityTab[ACTIVITY_RQST_TYPE(i)]) 
      
      /* Record remaining incompatible activities */
      if (! icuAbortActivity (servId, i)) {
	ACTIVITY_TAB_INCOMP(activity)[ACTIVITY_NB_INCOMP(activity)] = i;
	ACTIVITY_NB_INCOMP(activity)++;
      }  
}




/*****************************************************************************
 *
 *  icuCntrlPomTagging - Control request
 *
 */

#if (1)
  extern int icuPomTaggingCntrl ();
#endif 

static void icuCntrlPomTagging (SERV_ID servId, int rqstId)

{
  int i;
  static int icuPomTaggingCompatibility[] = { 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1 };
  int *compatibilityTab = icuPomTaggingCompatibility;
  int bilan=OK;
#if ((1) && 1)
  int tag_on_off;
#endif

  /*--------------------------------------------------------------
   * Control init done and exec tasks status 
   */
  if (!controlExecTaskStatus(servId, rqstId))
    return;

  /*--------------------------------------------------------------
   * Interruption of the incompatible activities
   */
  for (i = 0; i < MAX_ACTIVITIES; i++) 
    if (ACTIVITY_STATUS(i) != ETHER && ! compatibilityTab[ACTIVITY_RQST_TYPE(i)]) 
      icuAbortActivity (servId, i);

  /* 
   * Send the associated event
   */
  moduleEventCntrl.eventType = STATE_START_EVENT;
  moduleEventCntrl.activityState = EXEC;
  moduleEventCntrl.rqstType = 1;
  moduleEventCntrl.taskNum = -1;
  sendModuleEvent(&moduleEventCntrl);

  /*--------------------------------------------------------------
   * Call control func
   */
#if (1 /* Control func */)
  {
    BOOL status;

#if ((1) /* Input */)
    /* Get inputs */
    if (csServRqstParamsGet (servId, rqstId, (void *) &tag_on_off, 
			     sizeof((*icuDataStrId).tag_on_off), (FUNCPTR) NULL) != OK)
      icuReplyAndSuspend (servId, rqstId, TRUE);
    
    /* Call control func */
    status = icuPomTaggingCntrl (&tag_on_off, &bilan);
#else
    status = icuPomTaggingCntrl (&bilan);
#endif
    if (status != OK) {
      if (bilan == OK)
	bilan =  S_icu_stdGenoM_CONTROL_CODEL_ERROR;
      if (csServReplySend (servId, rqstId, FINAL_REPLY, bilan, 
			   (void *) NULL, 0, (FUNCPTR) NULL) != OK)
	icuCntrlTaskSuspend (TRUE);
      return;
    }
  }
#endif

 /*-------------------------------------------------------------
  * Record inputs
  */
#if ((1) /* Input */) 
#if (1 /* Control func */)
  memcpy ((void *) &((*icuDataStrId).tag_on_off), (void *) &tag_on_off, sizeof((*icuDataStrId).tag_on_off));
#else
  if (csServRqstParamsGet (servId, rqstId, (void *) &((*icuDataStrId).tag_on_off),
			   sizeof((*icuDataStrId).tag_on_off), (FUNCPTR) NULL) != OK)
    icuReplyAndSuspend (servId, rqstId, TRUE);
#endif
#endif

 /*-------------------------------------------------------------
  * Send final reply
  */
  if (csServReplySend (servId, rqstId, FINAL_REPLY, bilan, 
		       (void *) NULL, 0,
		       (FUNCPTR) NULL) != OK)
    icuCntrlTaskSuspend (TRUE);

  moduleEventCntrl.eventType = STATE_END_EVENT;
  moduleEventCntrl.activityState = ETHER;
  sendModuleEvent(&moduleEventCntrl);
}




/*****************************************************************************
 *
 *  icuCntrlAutoMode - Control request
 *
 */

#if (1)
  extern int icuAutoModeCntrl ();
#endif 

static void icuCntrlAutoMode (SERV_ID servId, int rqstId)

{
  int i;
  static int icuAutoModeCompatibility[] = { 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1 };
  int *compatibilityTab = icuAutoModeCompatibility;
  int bilan=OK;
#if ((1) && 1)
  ICU_AUTOSWITCH_CONTROL autoSwitchControl;
#endif

  /*--------------------------------------------------------------
   * Control init done and exec tasks status 
   */
  if (!controlExecTaskStatus(servId, rqstId))
    return;

  /*--------------------------------------------------------------
   * Interruption of the incompatible activities
   */
  for (i = 0; i < MAX_ACTIVITIES; i++) 
    if (ACTIVITY_STATUS(i) != ETHER && ! compatibilityTab[ACTIVITY_RQST_TYPE(i)]) 
      icuAbortActivity (servId, i);

  /* 
   * Send the associated event
   */
  moduleEventCntrl.eventType = STATE_START_EVENT;
  moduleEventCntrl.activityState = EXEC;
  moduleEventCntrl.rqstType = 2;
  moduleEventCntrl.taskNum = -1;
  sendModuleEvent(&moduleEventCntrl);

  /*--------------------------------------------------------------
   * Call control func
   */
#if (1 /* Control func */)
  {
    BOOL status;

#if ((1) /* Input */)
    /* Get inputs */
    if (csServRqstParamsGet (servId, rqstId, (void *) &autoSwitchControl, 
			     sizeof((*icuDataStrId).autoSwitchControl), (FUNCPTR) NULL) != OK)
      icuReplyAndSuspend (servId, rqstId, TRUE);
    
    /* Call control func */
    status = icuAutoModeCntrl (&autoSwitchControl, &bilan);
#else
    status = icuAutoModeCntrl (&bilan);
#endif
    if (status != OK) {
      if (bilan == OK)
	bilan =  S_icu_stdGenoM_CONTROL_CODEL_ERROR;
      if (csServReplySend (servId, rqstId, FINAL_REPLY, bilan, 
			   (void *) NULL, 0, (FUNCPTR) NULL) != OK)
	icuCntrlTaskSuspend (TRUE);
      return;
    }
  }
#endif

 /*-------------------------------------------------------------
  * Record inputs
  */
#if ((1) /* Input */) 
#if (1 /* Control func */)
  memcpy ((void *) &((*icuDataStrId).autoSwitchControl), (void *) &autoSwitchControl, sizeof((*icuDataStrId).autoSwitchControl));
#else
  if (csServRqstParamsGet (servId, rqstId, (void *) &((*icuDataStrId).autoSwitchControl),
			   sizeof((*icuDataStrId).autoSwitchControl), (FUNCPTR) NULL) != OK)
    icuReplyAndSuspend (servId, rqstId, TRUE);
#endif
#endif

 /*-------------------------------------------------------------
  * Send final reply
  */
  if (csServReplySend (servId, rqstId, FINAL_REPLY, bilan, 
		       (void *) NULL, 0,
		       (FUNCPTR) NULL) != OK)
    icuCntrlTaskSuspend (TRUE);

  moduleEventCntrl.eventType = STATE_END_EVENT;
  moduleEventCntrl.activityState = ETHER;
  sendModuleEvent(&moduleEventCntrl);
}




/*****************************************************************************
 *
 *  icuCntrlSaveFace - Control request
 *
 */

#if (1)
  extern int icuSaveFaceCntrl ();
#endif 

static void icuCntrlSaveFace (SERV_ID servId, int rqstId)

{
  int i;
  static int icuSaveFaceCompatibility[] = { 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1 };
  int *compatibilityTab = icuSaveFaceCompatibility;
  int bilan=OK;
#if ((1) && 1)
  int save_face;
#endif

  /*--------------------------------------------------------------
   * Control init done and exec tasks status 
   */
  if (!controlExecTaskStatus(servId, rqstId))
    return;

  /*--------------------------------------------------------------
   * Interruption of the incompatible activities
   */
  for (i = 0; i < MAX_ACTIVITIES; i++) 
    if (ACTIVITY_STATUS(i) != ETHER && ! compatibilityTab[ACTIVITY_RQST_TYPE(i)]) 
      icuAbortActivity (servId, i);

  /* 
   * Send the associated event
   */
  moduleEventCntrl.eventType = STATE_START_EVENT;
  moduleEventCntrl.activityState = EXEC;
  moduleEventCntrl.rqstType = 3;
  moduleEventCntrl.taskNum = -1;
  sendModuleEvent(&moduleEventCntrl);

  /*--------------------------------------------------------------
   * Call control func
   */
#if (1 /* Control func */)
  {
    BOOL status;

#if ((1) /* Input */)
    /* Get inputs */
    if (csServRqstParamsGet (servId, rqstId, (void *) &save_face, 
			     sizeof((*icuDataStrId).save_face), (FUNCPTR) NULL) != OK)
      icuReplyAndSuspend (servId, rqstId, TRUE);
    
    /* Call control func */
    status = icuSaveFaceCntrl (&save_face, &bilan);
#else
    status = icuSaveFaceCntrl (&bilan);
#endif
    if (status != OK) {
      if (bilan == OK)
	bilan =  S_icu_stdGenoM_CONTROL_CODEL_ERROR;
      if (csServReplySend (servId, rqstId, FINAL_REPLY, bilan, 
			   (void *) NULL, 0, (FUNCPTR) NULL) != OK)
	icuCntrlTaskSuspend (TRUE);
      return;
    }
  }
#endif

 /*-------------------------------------------------------------
  * Record inputs
  */
#if ((1) /* Input */) 
#if (1 /* Control func */)
  memcpy ((void *) &((*icuDataStrId).save_face), (void *) &save_face, sizeof((*icuDataStrId).save_face));
#else
  if (csServRqstParamsGet (servId, rqstId, (void *) &((*icuDataStrId).save_face),
			   sizeof((*icuDataStrId).save_face), (FUNCPTR) NULL) != OK)
    icuReplyAndSuspend (servId, rqstId, TRUE);
#endif
#endif

 /*-------------------------------------------------------------
  * Send final reply
  */
  if (csServReplySend (servId, rqstId, FINAL_REPLY, bilan, 
		       (void *) NULL, 0,
		       (FUNCPTR) NULL) != OK)
    icuCntrlTaskSuspend (TRUE);

  moduleEventCntrl.eventType = STATE_END_EVENT;
  moduleEventCntrl.activityState = ETHER;
  sendModuleEvent(&moduleEventCntrl);
}



/*****************************************************************************
 *
 *  icuCntrlCaptureImage - Execution request
 */

#if (1 /* control func */)
  extern int icuCaptureImageCntrl ();
#endif 

static void icuCntrlCaptureImage (SERV_ID servId, int rqstId)

{
  static int icuCaptureImageCompatibility[] = { 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1 };
  int activity;
  int i;
  int *compatibilityTab = icuCaptureImageCompatibility;
#if ((1) && 1 /* Inputs + control func */)
  int captureImage;
#endif

  /*--------------------------------------------------------------
   * Control init done and exec tasks status 
   */
#if (1 /* not init rqst */)
  if (!controlExecTaskStatusAndInitRqst(servId, rqstId))
    return;  
#endif
 
  /*--------------------------------------------------------------
   * Activity allocation
   */
  activity = allocActivity(rqstId, 4, 0, 1, servId);
  if (activity == -1) return;

  /*--------------------------------------------------------------
   * Call control func
   */
#if (1 /* control func */)
  {
    STATUS status;
    int bilan;

#if ((1)  /* Input */)
    /* Get inputs */
    if (csServRqstParamsGet (servId, rqstId, (void *) &captureImage, 
			     sizeof((*icuDataStrId).captureImage), (FUNCPTR) NULL) != OK)
      icuReplyAndSuspend (servId, rqstId, TRUE);
    status = icuCaptureImageCntrl (&captureImage, &bilan);
#else
    status = icuCaptureImageCntrl (&bilan);
#endif

    /* Control not OK: stop there */
    if (status != OK) {
      freeActivity(activity);
      if (bilan == OK)
	bilan = S_icu_stdGenoM_CONTROL_CODEL_ERROR;
      if (csServReplySend (servId, rqstId, FINAL_REPLY, bilan, 
			   (void *) NULL, 0, (FUNCPTR) NULL) != OK)
	icuCntrlTaskSuspend (TRUE);
      return;
    }
  }
#endif

 /*-------------------------------------------------------------
  * Record inputs
  *       XXXXXXXX NON C'EST TROP TOT -> INTERROMPRE LES ACTIVITES AVANT !
  */
#if ((1) /* Input */)

  /* Get inputs address */
#if (1 /* multiple request  */)
  ACTIVITY_INPUT_ID(activity) = &(icuDataStrId->CaptureImageInput)[activity];
#else
  ACTIVITY_INPUT_ID(activity) = &((*icuDataStrId).captureImage);
#endif                
  ACTIVITY_INPUT_SIZE(activity) = sizeof((*icuDataStrId).captureImage);

  /* Record inputs */
#if (1 /* control func */)
  memcpy ((void *) ACTIVITY_INPUT_ID(activity), (void *) &captureImage, 
	  ACTIVITY_INPUT_SIZE(activity));
#else
  if (csServRqstParamsGet (servId, rqstId, ACTIVITY_INPUT_ID(activity), 
			   sizeof((*icuDataStrId).captureImage), (FUNCPTR) NULL) != OK)
    icuReplyAndSuspend (servId, rqstId, TRUE);
#endif
#endif          
  
 /*-------------------------------------------------------------
  * Get outputs address
  */
#if ((0) /* Output */)
#if (1 /* Multiple request */)
  ACTIVITY_OUTPUT_ID(activity) = 
    &(icuDataStrId->CaptureImageOutput)[activity];
#else
  ACTIVITY_OUTPUT_ID(activity) = NULL;
#endif            
  ACTIVITY_OUTPUT_SIZE(activity) = 0;
#endif          

  moduleEventCntrl.eventType = STATE_START_EVENT;
  moduleEventCntrl.activityNum = activity;
  moduleEventCntrl.activityState = INIT;
  moduleEventCntrl.rqstType = 4;
  moduleEventCntrl.taskNum = 0;
  sendModuleEvent(&moduleEventCntrl);

  /*-------------------------------------------------------------
   * Interruption of the incompatible activities
   */
  for (i = 0; i < MAX_ACTIVITIES; i++) 
    if (i != activity && ACTIVITY_STATUS(i) != ETHER 
	&& ! compatibilityTab[ACTIVITY_RQST_TYPE(i)]) 
      
      /* Record remaining incompatible activities */
      if (! icuAbortActivity (servId, i)) {
	ACTIVITY_TAB_INCOMP(activity)[ACTIVITY_NB_INCOMP(activity)] = i;
	ACTIVITY_NB_INCOMP(activity)++;
      }  
}




/*****************************************************************************
 *
 *  icuCntrlCaptureVideo - Control request
 *
 */

#if (1)
  extern int icuCaptureVideoCntrl ();
#endif 

static void icuCntrlCaptureVideo (SERV_ID servId, int rqstId)

{
  int i;
  static int icuCaptureVideoCompatibility[] = { 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1 };
  int *compatibilityTab = icuCaptureVideoCompatibility;
  int bilan=OK;
#if ((1) && 1)
  int captureVideo;
#endif

  /*--------------------------------------------------------------
   * Control init done and exec tasks status 
   */
  if (!controlExecTaskStatus(servId, rqstId))
    return;

  /*--------------------------------------------------------------
   * Interruption of the incompatible activities
   */
  for (i = 0; i < MAX_ACTIVITIES; i++) 
    if (ACTIVITY_STATUS(i) != ETHER && ! compatibilityTab[ACTIVITY_RQST_TYPE(i)]) 
      icuAbortActivity (servId, i);

  /* 
   * Send the associated event
   */
  moduleEventCntrl.eventType = STATE_START_EVENT;
  moduleEventCntrl.activityState = EXEC;
  moduleEventCntrl.rqstType = 5;
  moduleEventCntrl.taskNum = -1;
  sendModuleEvent(&moduleEventCntrl);

  /*--------------------------------------------------------------
   * Call control func
   */
#if (1 /* Control func */)
  {
    BOOL status;

#if ((1) /* Input */)
    /* Get inputs */
    if (csServRqstParamsGet (servId, rqstId, (void *) &captureVideo, 
			     sizeof((*icuDataStrId).captureVideo), (FUNCPTR) NULL) != OK)
      icuReplyAndSuspend (servId, rqstId, TRUE);
    
    /* Call control func */
    status = icuCaptureVideoCntrl (&captureVideo, &bilan);
#else
    status = icuCaptureVideoCntrl (&bilan);
#endif
    if (status != OK) {
      if (bilan == OK)
	bilan =  S_icu_stdGenoM_CONTROL_CODEL_ERROR;
      if (csServReplySend (servId, rqstId, FINAL_REPLY, bilan, 
			   (void *) NULL, 0, (FUNCPTR) NULL) != OK)
	icuCntrlTaskSuspend (TRUE);
      return;
    }
  }
#endif

 /*-------------------------------------------------------------
  * Record inputs
  */
#if ((1) /* Input */) 
#if (1 /* Control func */)
  memcpy ((void *) &((*icuDataStrId).captureVideo), (void *) &captureVideo, sizeof((*icuDataStrId).captureVideo));
#else
  if (csServRqstParamsGet (servId, rqstId, (void *) &((*icuDataStrId).captureVideo),
			   sizeof((*icuDataStrId).captureVideo), (FUNCPTR) NULL) != OK)
    icuReplyAndSuspend (servId, rqstId, TRUE);
#endif
#endif

 /*-------------------------------------------------------------
  * Send final reply
  */
  if (csServReplySend (servId, rqstId, FINAL_REPLY, bilan, 
		       (void *) NULL, 0,
		       (FUNCPTR) NULL) != OK)
    icuCntrlTaskSuspend (TRUE);

  moduleEventCntrl.eventType = STATE_END_EVENT;
  moduleEventCntrl.activityState = ETHER;
  sendModuleEvent(&moduleEventCntrl);
}




/*****************************************************************************
 *
 *  icuCntrlSetViamPosterName - Control request
 *
 */

#if (1)
  extern int icuSetViamPosterCntrl ();
#endif 

static void icuCntrlSetViamPosterName (SERV_ID servId, int rqstId)

{
  int i;
  static int icuSetViamPosterNameCompatibility[] = { 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1 };
  int *compatibilityTab = icuSetViamPosterNameCompatibility;
  int bilan=OK;
#if ((1) && 1)
  str viamPosterName;
#endif

  /*--------------------------------------------------------------
   * Control init done and exec tasks status 
   */
  if (!controlExecTaskStatus(servId, rqstId))
    return;

  /*--------------------------------------------------------------
   * Interruption of the incompatible activities
   */
  for (i = 0; i < MAX_ACTIVITIES; i++) 
    if (ACTIVITY_STATUS(i) != ETHER && ! compatibilityTab[ACTIVITY_RQST_TYPE(i)]) 
      icuAbortActivity (servId, i);

  /* 
   * Send the associated event
   */
  moduleEventCntrl.eventType = STATE_START_EVENT;
  moduleEventCntrl.activityState = EXEC;
  moduleEventCntrl.rqstType = 6;
  moduleEventCntrl.taskNum = -1;
  sendModuleEvent(&moduleEventCntrl);

  /*--------------------------------------------------------------
   * Call control func
   */
#if (1 /* Control func */)
  {
    BOOL status;

#if ((1) /* Input */)
    /* Get inputs */
    if (csServRqstParamsGet (servId, rqstId, (void *) &viamPosterName, 
			     sizeof((*icuDataStrId).viamPosterName), (FUNCPTR) NULL) != OK)
      icuReplyAndSuspend (servId, rqstId, TRUE);
    
    /* Call control func */
    status = icuSetViamPosterCntrl (&viamPosterName, &bilan);
#else
    status = icuSetViamPosterCntrl (&bilan);
#endif
    if (status != OK) {
      if (bilan == OK)
	bilan =  S_icu_stdGenoM_CONTROL_CODEL_ERROR;
      if (csServReplySend (servId, rqstId, FINAL_REPLY, bilan, 
			   (void *) NULL, 0, (FUNCPTR) NULL) != OK)
	icuCntrlTaskSuspend (TRUE);
      return;
    }
  }
#endif

 /*-------------------------------------------------------------
  * Record inputs
  */
#if ((1) /* Input */) 
#if (1 /* Control func */)
  memcpy ((void *) &((*icuDataStrId).viamPosterName), (void *) &viamPosterName, sizeof((*icuDataStrId).viamPosterName));
#else
  if (csServRqstParamsGet (servId, rqstId, (void *) &((*icuDataStrId).viamPosterName),
			   sizeof((*icuDataStrId).viamPosterName), (FUNCPTR) NULL) != OK)
    icuReplyAndSuspend (servId, rqstId, TRUE);
#endif
#endif

 /*-------------------------------------------------------------
  * Send final reply
  */
  if (csServReplySend (servId, rqstId, FINAL_REPLY, bilan, 
		       (void *) NULL, 0,
		       (FUNCPTR) NULL) != OK)
    icuCntrlTaskSuspend (TRUE);

  moduleEventCntrl.eventType = STATE_END_EVENT;
  moduleEventCntrl.activityState = ETHER;
  sendModuleEvent(&moduleEventCntrl);
}



/*****************************************************************************
 *
 *  icuCntrlEnableDisplay - Execution request
 */

#if (1 /* control func */)
  extern int icuEnableDisplayCntrl ();
#endif 

static void icuCntrlEnableDisplay (SERV_ID servId, int rqstId)

{
  static int icuEnableDisplayCompatibility[] = { 1, 1, 1, 1, 1, 1, 1, 0, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1 };
  int activity;
  int i;
  int *compatibilityTab = icuEnableDisplayCompatibility;
#if ((1) && 1 /* Inputs + control func */)
  ICU_AFFICHAGE display;
#endif

  /*--------------------------------------------------------------
   * Control init done and exec tasks status 
   */
#if (1 /* not init rqst */)
  if (!controlExecTaskStatusAndInitRqst(servId, rqstId))
    return;  
#endif
 
  /*--------------------------------------------------------------
   * Activity allocation
   */
  activity = allocActivity(rqstId, 7, 0, 0, servId);
  if (activity == -1) return;

  /*--------------------------------------------------------------
   * Call control func
   */
#if (1 /* control func */)
  {
    STATUS status;
    int bilan;

#if ((1)  /* Input */)
    /* Get inputs */
    if (csServRqstParamsGet (servId, rqstId, (void *) &display, 
			     sizeof((*icuDataStrId).display), (FUNCPTR) NULL) != OK)
      icuReplyAndSuspend (servId, rqstId, TRUE);
    status = icuEnableDisplayCntrl (&display, &bilan);
#else
    status = icuEnableDisplayCntrl (&bilan);
#endif

    /* Control not OK: stop there */
    if (status != OK) {
      freeActivity(activity);
      if (bilan == OK)
	bilan = S_icu_stdGenoM_CONTROL_CODEL_ERROR;
      if (csServReplySend (servId, rqstId, FINAL_REPLY, bilan, 
			   (void *) NULL, 0, (FUNCPTR) NULL) != OK)
	icuCntrlTaskSuspend (TRUE);
      return;
    }
  }
#endif

 /*-------------------------------------------------------------
  * Record inputs
  *       XXXXXXXX NON C'EST TROP TOT -> INTERROMPRE LES ACTIVITES AVANT !
  */
#if ((1) /* Input */)

  /* Get inputs address */
#if (0 /* multiple request  */)
  ACTIVITY_INPUT_ID(activity) = &(icuDataStrId->EnableDisplayInput)[activity];
#else
  ACTIVITY_INPUT_ID(activity) = &((*icuDataStrId).display);
#endif                
  ACTIVITY_INPUT_SIZE(activity) = sizeof((*icuDataStrId).display);

  /* Record inputs */
#if (1 /* control func */)
  memcpy ((void *) ACTIVITY_INPUT_ID(activity), (void *) &display, 
	  ACTIVITY_INPUT_SIZE(activity));
#else
  if (csServRqstParamsGet (servId, rqstId, ACTIVITY_INPUT_ID(activity), 
			   sizeof((*icuDataStrId).display), (FUNCPTR) NULL) != OK)
    icuReplyAndSuspend (servId, rqstId, TRUE);
#endif
#endif          
  
 /*-------------------------------------------------------------
  * Get outputs address
  */
#if ((0) /* Output */)
#if (0 /* Multiple request */)
  ACTIVITY_OUTPUT_ID(activity) = 
    &(icuDataStrId->EnableDisplayOutput)[activity];
#else
  ACTIVITY_OUTPUT_ID(activity) = NULL;
#endif            
  ACTIVITY_OUTPUT_SIZE(activity) = 0;
#endif          

  moduleEventCntrl.eventType = STATE_START_EVENT;
  moduleEventCntrl.activityNum = activity;
  moduleEventCntrl.activityState = INIT;
  moduleEventCntrl.rqstType = 7;
  moduleEventCntrl.taskNum = 0;
  sendModuleEvent(&moduleEventCntrl);

  /*-------------------------------------------------------------
   * Interruption of the incompatible activities
   */
  for (i = 0; i < MAX_ACTIVITIES; i++) 
    if (i != activity && ACTIVITY_STATUS(i) != ETHER 
	&& ! compatibilityTab[ACTIVITY_RQST_TYPE(i)]) 
      
      /* Record remaining incompatible activities */
      if (! icuAbortActivity (servId, i)) {
	ACTIVITY_TAB_INCOMP(activity)[ACTIVITY_NB_INCOMP(activity)] = i;
	ACTIVITY_NB_INCOMP(activity)++;
      }  
}



/*****************************************************************************
 *
 *  icuCntrlSetCameraPos - Execution request
 */

#if (1 /* control func */)
  extern int icuSetCameraPosCntrl ();
#endif 

static void icuCntrlSetCameraPos (SERV_ID servId, int rqstId)

{
  static int icuSetCameraPosCompatibility[] = { 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1 };
  int activity;
  int i;
  int *compatibilityTab = icuSetCameraPosCompatibility;
#if ((1) && 1 /* Inputs + control func */)
  ICU_CAMERA_POS cameraPos;
#endif

  /*--------------------------------------------------------------
   * Control init done and exec tasks status 
   */
#if (1 /* not init rqst */)
  if (!controlExecTaskStatusAndInitRqst(servId, rqstId))
    return;  
#endif
 
  /*--------------------------------------------------------------
   * Activity allocation
   */
  activity = allocActivity(rqstId, 8, 0, 1, servId);
  if (activity == -1) return;

  /*--------------------------------------------------------------
   * Call control func
   */
#if (1 /* control func */)
  {
    STATUS status;
    int bilan;

#if ((1)  /* Input */)
    /* Get inputs */
    if (csServRqstParamsGet (servId, rqstId, (void *) &cameraPos, 
			     sizeof((*icuDataStrId).cameraPos), (FUNCPTR) NULL) != OK)
      icuReplyAndSuspend (servId, rqstId, TRUE);
    status = icuSetCameraPosCntrl (&cameraPos, &bilan);
#else
    status = icuSetCameraPosCntrl (&bilan);
#endif

    /* Control not OK: stop there */
    if (status != OK) {
      freeActivity(activity);
      if (bilan == OK)
	bilan = S_icu_stdGenoM_CONTROL_CODEL_ERROR;
      if (csServReplySend (servId, rqstId, FINAL_REPLY, bilan, 
			   (void *) NULL, 0, (FUNCPTR) NULL) != OK)
	icuCntrlTaskSuspend (TRUE);
      return;
    }
  }
#endif

 /*-------------------------------------------------------------
  * Record inputs
  *       XXXXXXXX NON C'EST TROP TOT -> INTERROMPRE LES ACTIVITES AVANT !
  */
#if ((1) /* Input */)

  /* Get inputs address */
#if (1 /* multiple request  */)
  ACTIVITY_INPUT_ID(activity) = &(icuDataStrId->SetCameraPosInput)[activity];
#else
  ACTIVITY_INPUT_ID(activity) = &((*icuDataStrId).cameraPos);
#endif                
  ACTIVITY_INPUT_SIZE(activity) = sizeof((*icuDataStrId).cameraPos);

  /* Record inputs */
#if (1 /* control func */)
  memcpy ((void *) ACTIVITY_INPUT_ID(activity), (void *) &cameraPos, 
	  ACTIVITY_INPUT_SIZE(activity));
#else
  if (csServRqstParamsGet (servId, rqstId, ACTIVITY_INPUT_ID(activity), 
			   sizeof((*icuDataStrId).cameraPos), (FUNCPTR) NULL) != OK)
    icuReplyAndSuspend (servId, rqstId, TRUE);
#endif
#endif          
  
 /*-------------------------------------------------------------
  * Get outputs address
  */
#if ((0) /* Output */)
#if (1 /* Multiple request */)
  ACTIVITY_OUTPUT_ID(activity) = 
    &(icuDataStrId->SetCameraPosOutput)[activity];
#else
  ACTIVITY_OUTPUT_ID(activity) = NULL;
#endif            
  ACTIVITY_OUTPUT_SIZE(activity) = 0;
#endif          

  moduleEventCntrl.eventType = STATE_START_EVENT;
  moduleEventCntrl.activityNum = activity;
  moduleEventCntrl.activityState = INIT;
  moduleEventCntrl.rqstType = 8;
  moduleEventCntrl.taskNum = 0;
  sendModuleEvent(&moduleEventCntrl);

  /*-------------------------------------------------------------
   * Interruption of the incompatible activities
   */
  for (i = 0; i < MAX_ACTIVITIES; i++) 
    if (i != activity && ACTIVITY_STATUS(i) != ETHER 
	&& ! compatibilityTab[ACTIVITY_RQST_TYPE(i)]) 
      
      /* Record remaining incompatible activities */
      if (! icuAbortActivity (servId, i)) {
	ACTIVITY_TAB_INCOMP(activity)[ACTIVITY_NB_INCOMP(activity)] = i;
	ACTIVITY_NB_INCOMP(activity)++;
      }  
}



/*****************************************************************************
 *
 *  icuCntrlTrackIt - Execution request
 */

#if (0 /* control func */)
  extern int dummy ();
#endif 

static void icuCntrlTrackIt (SERV_ID servId, int rqstId)

{
  static int icuTrackItCompatibility[] = { 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1 };
  int activity;
  int i;
  int *compatibilityTab = icuTrackItCompatibility;
#if ((1) && 0 /* Inputs + control func */)
  int track_it;
#endif

  /*--------------------------------------------------------------
   * Control init done and exec tasks status 
   */
#if (1 /* not init rqst */)
  if (!controlExecTaskStatusAndInitRqst(servId, rqstId))
    return;  
#endif
 
  /*--------------------------------------------------------------
   * Activity allocation
   */
  activity = allocActivity(rqstId, 9, 0, 1, servId);
  if (activity == -1) return;

  /*--------------------------------------------------------------
   * Call control func
   */
#if (0 /* control func */)
  {
    STATUS status;
    int bilan;

#if ((1)  /* Input */)
    /* Get inputs */
    if (csServRqstParamsGet (servId, rqstId, (void *) &track_it, 
			     sizeof((*icuDataStrId).track_it), (FUNCPTR) NULL) != OK)
      icuReplyAndSuspend (servId, rqstId, TRUE);
    status = dummy (&track_it, &bilan);
#else
    status = dummy (&bilan);
#endif

    /* Control not OK: stop there */
    if (status != OK) {
      freeActivity(activity);
      if (bilan == OK)
	bilan = S_icu_stdGenoM_CONTROL_CODEL_ERROR;
      if (csServReplySend (servId, rqstId, FINAL_REPLY, bilan, 
			   (void *) NULL, 0, (FUNCPTR) NULL) != OK)
	icuCntrlTaskSuspend (TRUE);
      return;
    }
  }
#endif

 /*-------------------------------------------------------------
  * Record inputs
  *       XXXXXXXX NON C'EST TROP TOT -> INTERROMPRE LES ACTIVITES AVANT !
  */
#if ((1) /* Input */)

  /* Get inputs address */
#if (1 /* multiple request  */)
  ACTIVITY_INPUT_ID(activity) = &(icuDataStrId->TrackItInput)[activity];
#else
  ACTIVITY_INPUT_ID(activity) = &((*icuDataStrId).track_it);
#endif                
  ACTIVITY_INPUT_SIZE(activity) = sizeof((*icuDataStrId).track_it);

  /* Record inputs */
#if (0 /* control func */)
  memcpy ((void *) ACTIVITY_INPUT_ID(activity), (void *) &track_it, 
	  ACTIVITY_INPUT_SIZE(activity));
#else
  if (csServRqstParamsGet (servId, rqstId, ACTIVITY_INPUT_ID(activity), 
			   sizeof((*icuDataStrId).track_it), (FUNCPTR) NULL) != OK)
    icuReplyAndSuspend (servId, rqstId, TRUE);
#endif
#endif          
  
 /*-------------------------------------------------------------
  * Get outputs address
  */
#if ((0) /* Output */)
#if (1 /* Multiple request */)
  ACTIVITY_OUTPUT_ID(activity) = 
    &(icuDataStrId->TrackItOutput)[activity];
#else
  ACTIVITY_OUTPUT_ID(activity) = NULL;
#endif            
  ACTIVITY_OUTPUT_SIZE(activity) = 0;
#endif          

  moduleEventCntrl.eventType = STATE_START_EVENT;
  moduleEventCntrl.activityNum = activity;
  moduleEventCntrl.activityState = INIT;
  moduleEventCntrl.rqstType = 9;
  moduleEventCntrl.taskNum = 0;
  sendModuleEvent(&moduleEventCntrl);

  /*-------------------------------------------------------------
   * Interruption of the incompatible activities
   */
  for (i = 0; i < MAX_ACTIVITIES; i++) 
    if (i != activity && ACTIVITY_STATUS(i) != ETHER 
	&& ! compatibilityTab[ACTIVITY_RQST_TYPE(i)]) 
      
      /* Record remaining incompatible activities */
      if (! icuAbortActivity (servId, i)) {
	ACTIVITY_TAB_INCOMP(activity)[ACTIVITY_NB_INCOMP(activity)] = i;
	ACTIVITY_NB_INCOMP(activity)++;
      }  
}



/*****************************************************************************
 *
 *  icuCntrlWaitForSomebody - Execution request
 */

#if (1 /* control func */)
  extern int icuWaitForSomebodyCntrl ();
#endif 

static void icuCntrlWaitForSomebody (SERV_ID servId, int rqstId)

{
  static int icuWaitForSomebodyCompatibility[] = { 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1 };
  int activity;
  int i;
  int *compatibilityTab = icuWaitForSomebodyCompatibility;
#if ((1) && 1 /* Inputs + control func */)
  int nbImageMaxNobody;
#endif

  /*--------------------------------------------------------------
   * Control init done and exec tasks status 
   */
#if (1 /* not init rqst */)
  if (!controlExecTaskStatusAndInitRqst(servId, rqstId))
    return;  
#endif
 
  /*--------------------------------------------------------------
   * Activity allocation
   */
  activity = allocActivity(rqstId, 10, 0, 1, servId);
  if (activity == -1) return;

  /*--------------------------------------------------------------
   * Call control func
   */
#if (1 /* control func */)
  {
    STATUS status;
    int bilan;

#if ((1)  /* Input */)
    /* Get inputs */
    if (csServRqstParamsGet (servId, rqstId, (void *) &nbImageMaxNobody, 
			     sizeof((*icuDataStrId).nbImageMaxNobody), (FUNCPTR) NULL) != OK)
      icuReplyAndSuspend (servId, rqstId, TRUE);
    status = icuWaitForSomebodyCntrl (&nbImageMaxNobody, &bilan);
#else
    status = icuWaitForSomebodyCntrl (&bilan);
#endif

    /* Control not OK: stop there */
    if (status != OK) {
      freeActivity(activity);
      if (bilan == OK)
	bilan = S_icu_stdGenoM_CONTROL_CODEL_ERROR;
      if (csServReplySend (servId, rqstId, FINAL_REPLY, bilan, 
			   (void *) NULL, 0, (FUNCPTR) NULL) != OK)
	icuCntrlTaskSuspend (TRUE);
      return;
    }
  }
#endif

 /*-------------------------------------------------------------
  * Record inputs
  *       XXXXXXXX NON C'EST TROP TOT -> INTERROMPRE LES ACTIVITES AVANT !
  */
#if ((1) /* Input */)

  /* Get inputs address */
#if (1 /* multiple request  */)
  ACTIVITY_INPUT_ID(activity) = &(icuDataStrId->WaitForSomebodyInput)[activity];
#else
  ACTIVITY_INPUT_ID(activity) = &((*icuDataStrId).nbImageMaxNobody);
#endif                
  ACTIVITY_INPUT_SIZE(activity) = sizeof((*icuDataStrId).nbImageMaxNobody);

  /* Record inputs */
#if (1 /* control func */)
  memcpy ((void *) ACTIVITY_INPUT_ID(activity), (void *) &nbImageMaxNobody, 
	  ACTIVITY_INPUT_SIZE(activity));
#else
  if (csServRqstParamsGet (servId, rqstId, ACTIVITY_INPUT_ID(activity), 
			   sizeof((*icuDataStrId).nbImageMaxNobody), (FUNCPTR) NULL) != OK)
    icuReplyAndSuspend (servId, rqstId, TRUE);
#endif
#endif          
  
 /*-------------------------------------------------------------
  * Get outputs address
  */
#if ((1) /* Output */)
#if (1 /* Multiple request */)
  ACTIVITY_OUTPUT_ID(activity) = 
    &(icuDataStrId->WaitForSomebodyOutput)[activity];
#else
  ACTIVITY_OUTPUT_ID(activity) = &((*icuDataStrId).waitFor);
#endif            
  ACTIVITY_OUTPUT_SIZE(activity) = sizeof((*icuDataStrId).waitFor);
#endif          

  moduleEventCntrl.eventType = STATE_START_EVENT;
  moduleEventCntrl.activityNum = activity;
  moduleEventCntrl.activityState = INIT;
  moduleEventCntrl.rqstType = 10;
  moduleEventCntrl.taskNum = 0;
  sendModuleEvent(&moduleEventCntrl);

  /*-------------------------------------------------------------
   * Interruption of the incompatible activities
   */
  for (i = 0; i < MAX_ACTIVITIES; i++) 
    if (i != activity && ACTIVITY_STATUS(i) != ETHER 
	&& ! compatibilityTab[ACTIVITY_RQST_TYPE(i)]) 
      
      /* Record remaining incompatible activities */
      if (! icuAbortActivity (servId, i)) {
	ACTIVITY_TAB_INCOMP(activity)[ACTIVITY_NB_INCOMP(activity)] = i;
	ACTIVITY_NB_INCOMP(activity)++;
      }  
}



/*****************************************************************************
 *
 *  icuCntrlWaitForSomeone - Execution request
 */

#if (1 /* control func */)
  extern int icuWaitForSomeoneCntrl ();
#endif 

static void icuCntrlWaitForSomeone (SERV_ID servId, int rqstId)

{
  static int icuWaitForSomeoneCompatibility[] = { 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1 };
  int activity;
  int i;
  int *compatibilityTab = icuWaitForSomeoneCompatibility;
#if ((1) && 1 /* Inputs + control func */)
  ICU_WAITFOR waitFor;
#endif

  /*--------------------------------------------------------------
   * Control init done and exec tasks status 
   */
#if (1 /* not init rqst */)
  if (!controlExecTaskStatusAndInitRqst(servId, rqstId))
    return;  
#endif
 
  /*--------------------------------------------------------------
   * Activity allocation
   */
  activity = allocActivity(rqstId, 11, 0, 1, servId);
  if (activity == -1) return;

  /*--------------------------------------------------------------
   * Call control func
   */
#if (1 /* control func */)
  {
    STATUS status;
    int bilan;

#if ((1)  /* Input */)
    /* Get inputs */
    if (csServRqstParamsGet (servId, rqstId, (void *) &waitFor, 
			     sizeof((*icuDataStrId).waitFor), (FUNCPTR) NULL) != OK)
      icuReplyAndSuspend (servId, rqstId, TRUE);
    status = icuWaitForSomeoneCntrl (&waitFor, &bilan);
#else
    status = icuWaitForSomeoneCntrl (&bilan);
#endif

    /* Control not OK: stop there */
    if (status != OK) {
      freeActivity(activity);
      if (bilan == OK)
	bilan = S_icu_stdGenoM_CONTROL_CODEL_ERROR;
      if (csServReplySend (servId, rqstId, FINAL_REPLY, bilan, 
			   (void *) NULL, 0, (FUNCPTR) NULL) != OK)
	icuCntrlTaskSuspend (TRUE);
      return;
    }
  }
#endif

 /*-------------------------------------------------------------
  * Record inputs
  *       XXXXXXXX NON C'EST TROP TOT -> INTERROMPRE LES ACTIVITES AVANT !
  */
#if ((1) /* Input */)

  /* Get inputs address */
#if (1 /* multiple request  */)
  ACTIVITY_INPUT_ID(activity) = &(icuDataStrId->WaitForSomeoneInput)[activity];
#else
  ACTIVITY_INPUT_ID(activity) = &((*icuDataStrId).waitFor);
#endif                
  ACTIVITY_INPUT_SIZE(activity) = sizeof((*icuDataStrId).waitFor);

  /* Record inputs */
#if (1 /* control func */)
  memcpy ((void *) ACTIVITY_INPUT_ID(activity), (void *) &waitFor, 
	  ACTIVITY_INPUT_SIZE(activity));
#else
  if (csServRqstParamsGet (servId, rqstId, ACTIVITY_INPUT_ID(activity), 
			   sizeof((*icuDataStrId).waitFor), (FUNCPTR) NULL) != OK)
    icuReplyAndSuspend (servId, rqstId, TRUE);
#endif
#endif          
  
 /*-------------------------------------------------------------
  * Get outputs address
  */
#if ((0) /* Output */)
#if (1 /* Multiple request */)
  ACTIVITY_OUTPUT_ID(activity) = 
    &(icuDataStrId->WaitForSomeoneOutput)[activity];
#else
  ACTIVITY_OUTPUT_ID(activity) = NULL;
#endif            
  ACTIVITY_OUTPUT_SIZE(activity) = 0;
#endif          

  moduleEventCntrl.eventType = STATE_START_EVENT;
  moduleEventCntrl.activityNum = activity;
  moduleEventCntrl.activityState = INIT;
  moduleEventCntrl.rqstType = 11;
  moduleEventCntrl.taskNum = 0;
  sendModuleEvent(&moduleEventCntrl);

  /*-------------------------------------------------------------
   * Interruption of the incompatible activities
   */
  for (i = 0; i < MAX_ACTIVITIES; i++) 
    if (i != activity && ACTIVITY_STATUS(i) != ETHER 
	&& ! compatibilityTab[ACTIVITY_RQST_TYPE(i)]) 
      
      /* Record remaining incompatible activities */
      if (! icuAbortActivity (servId, i)) {
	ACTIVITY_TAB_INCOMP(activity)[ACTIVITY_NB_INCOMP(activity)] = i;
	ACTIVITY_NB_INCOMP(activity)++;
      }  
}



/*****************************************************************************
 *
 *  icuCntrlWaitForNobody - Execution request
 */

#if (1 /* control func */)
  extern int icuWaitForNobodyCntrl ();
#endif 

static void icuCntrlWaitForNobody (SERV_ID servId, int rqstId)

{
  static int icuWaitForNobodyCompatibility[] = { 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1 };
  int activity;
  int i;
  int *compatibilityTab = icuWaitForNobodyCompatibility;
#if ((1) && 1 /* Inputs + control func */)
  int nbImageMaxNobody;
#endif

  /*--------------------------------------------------------------
   * Control init done and exec tasks status 
   */
#if (1 /* not init rqst */)
  if (!controlExecTaskStatusAndInitRqst(servId, rqstId))
    return;  
#endif
 
  /*--------------------------------------------------------------
   * Activity allocation
   */
  activity = allocActivity(rqstId, 12, 0, 1, servId);
  if (activity == -1) return;

  /*--------------------------------------------------------------
   * Call control func
   */
#if (1 /* control func */)
  {
    STATUS status;
    int bilan;

#if ((1)  /* Input */)
    /* Get inputs */
    if (csServRqstParamsGet (servId, rqstId, (void *) &nbImageMaxNobody, 
			     sizeof((*icuDataStrId).nbImageMaxNobody), (FUNCPTR) NULL) != OK)
      icuReplyAndSuspend (servId, rqstId, TRUE);
    status = icuWaitForNobodyCntrl (&nbImageMaxNobody, &bilan);
#else
    status = icuWaitForNobodyCntrl (&bilan);
#endif

    /* Control not OK: stop there */
    if (status != OK) {
      freeActivity(activity);
      if (bilan == OK)
	bilan = S_icu_stdGenoM_CONTROL_CODEL_ERROR;
      if (csServReplySend (servId, rqstId, FINAL_REPLY, bilan, 
			   (void *) NULL, 0, (FUNCPTR) NULL) != OK)
	icuCntrlTaskSuspend (TRUE);
      return;
    }
  }
#endif

 /*-------------------------------------------------------------
  * Record inputs
  *       XXXXXXXX NON C'EST TROP TOT -> INTERROMPRE LES ACTIVITES AVANT !
  */
#if ((1) /* Input */)

  /* Get inputs address */
#if (1 /* multiple request  */)
  ACTIVITY_INPUT_ID(activity) = &(icuDataStrId->WaitForNobodyInput)[activity];
#else
  ACTIVITY_INPUT_ID(activity) = &((*icuDataStrId).nbImageMaxNobody);
#endif                
  ACTIVITY_INPUT_SIZE(activity) = sizeof((*icuDataStrId).nbImageMaxNobody);

  /* Record inputs */
#if (1 /* control func */)
  memcpy ((void *) ACTIVITY_INPUT_ID(activity), (void *) &nbImageMaxNobody, 
	  ACTIVITY_INPUT_SIZE(activity));
#else
  if (csServRqstParamsGet (servId, rqstId, ACTIVITY_INPUT_ID(activity), 
			   sizeof((*icuDataStrId).nbImageMaxNobody), (FUNCPTR) NULL) != OK)
    icuReplyAndSuspend (servId, rqstId, TRUE);
#endif
#endif          
  
 /*-------------------------------------------------------------
  * Get outputs address
  */
#if ((0) /* Output */)
#if (1 /* Multiple request */)
  ACTIVITY_OUTPUT_ID(activity) = 
    &(icuDataStrId->WaitForNobodyOutput)[activity];
#else
  ACTIVITY_OUTPUT_ID(activity) = NULL;
#endif            
  ACTIVITY_OUTPUT_SIZE(activity) = 0;
#endif          

  moduleEventCntrl.eventType = STATE_START_EVENT;
  moduleEventCntrl.activityNum = activity;
  moduleEventCntrl.activityState = INIT;
  moduleEventCntrl.rqstType = 12;
  moduleEventCntrl.taskNum = 0;
  sendModuleEvent(&moduleEventCntrl);

  /*-------------------------------------------------------------
   * Interruption of the incompatible activities
   */
  for (i = 0; i < MAX_ACTIVITIES; i++) 
    if (i != activity && ACTIVITY_STATUS(i) != ETHER 
	&& ! compatibilityTab[ACTIVITY_RQST_TYPE(i)]) 
      
      /* Record remaining incompatible activities */
      if (! icuAbortActivity (servId, i)) {
	ACTIVITY_TAB_INCOMP(activity)[ACTIVITY_NB_INCOMP(activity)] = i;
	ACTIVITY_NB_INCOMP(activity)++;
      }  
}



/*****************************************************************************
 *
 *  icuCntrlWaitForNone - Execution request
 */

#if (1 /* control func */)
  extern int icuWaitForNoneCntrl ();
#endif 

static void icuCntrlWaitForNone (SERV_ID servId, int rqstId)

{
  static int icuWaitForNoneCompatibility[] = { 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1 };
  int activity;
  int i;
  int *compatibilityTab = icuWaitForNoneCompatibility;
#if ((1) && 1 /* Inputs + control func */)
  ICU_WAITFOR waitFor;
#endif

  /*--------------------------------------------------------------
   * Control init done and exec tasks status 
   */
#if (1 /* not init rqst */)
  if (!controlExecTaskStatusAndInitRqst(servId, rqstId))
    return;  
#endif
 
  /*--------------------------------------------------------------
   * Activity allocation
   */
  activity = allocActivity(rqstId, 13, 0, 1, servId);
  if (activity == -1) return;

  /*--------------------------------------------------------------
   * Call control func
   */
#if (1 /* control func */)
  {
    STATUS status;
    int bilan;

#if ((1)  /* Input */)
    /* Get inputs */
    if (csServRqstParamsGet (servId, rqstId, (void *) &waitFor, 
			     sizeof((*icuDataStrId).waitFor), (FUNCPTR) NULL) != OK)
      icuReplyAndSuspend (servId, rqstId, TRUE);
    status = icuWaitForNoneCntrl (&waitFor, &bilan);
#else
    status = icuWaitForNoneCntrl (&bilan);
#endif

    /* Control not OK: stop there */
    if (status != OK) {
      freeActivity(activity);
      if (bilan == OK)
	bilan = S_icu_stdGenoM_CONTROL_CODEL_ERROR;
      if (csServReplySend (servId, rqstId, FINAL_REPLY, bilan, 
			   (void *) NULL, 0, (FUNCPTR) NULL) != OK)
	icuCntrlTaskSuspend (TRUE);
      return;
    }
  }
#endif

 /*-------------------------------------------------------------
  * Record inputs
  *       XXXXXXXX NON C'EST TROP TOT -> INTERROMPRE LES ACTIVITES AVANT !
  */
#if ((1) /* Input */)

  /* Get inputs address */
#if (1 /* multiple request  */)
  ACTIVITY_INPUT_ID(activity) = &(icuDataStrId->WaitForNoneInput)[activity];
#else
  ACTIVITY_INPUT_ID(activity) = &((*icuDataStrId).waitFor);
#endif                
  ACTIVITY_INPUT_SIZE(activity) = sizeof((*icuDataStrId).waitFor);

  /* Record inputs */
#if (1 /* control func */)
  memcpy ((void *) ACTIVITY_INPUT_ID(activity), (void *) &waitFor, 
	  ACTIVITY_INPUT_SIZE(activity));
#else
  if (csServRqstParamsGet (servId, rqstId, ACTIVITY_INPUT_ID(activity), 
			   sizeof((*icuDataStrId).waitFor), (FUNCPTR) NULL) != OK)
    icuReplyAndSuspend (servId, rqstId, TRUE);
#endif
#endif          
  
 /*-------------------------------------------------------------
  * Get outputs address
  */
#if ((0) /* Output */)
#if (1 /* Multiple request */)
  ACTIVITY_OUTPUT_ID(activity) = 
    &(icuDataStrId->WaitForNoneOutput)[activity];
#else
  ACTIVITY_OUTPUT_ID(activity) = NULL;
#endif            
  ACTIVITY_OUTPUT_SIZE(activity) = 0;
#endif          

  moduleEventCntrl.eventType = STATE_START_EVENT;
  moduleEventCntrl.activityNum = activity;
  moduleEventCntrl.activityState = INIT;
  moduleEventCntrl.rqstType = 13;
  moduleEventCntrl.taskNum = 0;
  sendModuleEvent(&moduleEventCntrl);

  /*-------------------------------------------------------------
   * Interruption of the incompatible activities
   */
  for (i = 0; i < MAX_ACTIVITIES; i++) 
    if (i != activity && ACTIVITY_STATUS(i) != ETHER 
	&& ! compatibilityTab[ACTIVITY_RQST_TYPE(i)]) 
      
      /* Record remaining incompatible activities */
      if (! icuAbortActivity (servId, i)) {
	ACTIVITY_TAB_INCOMP(activity)[ACTIVITY_NB_INCOMP(activity)] = i;
	ACTIVITY_NB_INCOMP(activity)++;
      }  
}



/*****************************************************************************
 *
 *  icuCntrlWaitForNobodyKnown - Execution request
 */

#if (1 /* control func */)
  extern int icuWaitForNobodyKnownCntrl ();
#endif 

static void icuCntrlWaitForNobodyKnown (SERV_ID servId, int rqstId)

{
  static int icuWaitForNobodyKnownCompatibility[] = { 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1 };
  int activity;
  int i;
  int *compatibilityTab = icuWaitForNobodyKnownCompatibility;
#if ((1) && 1 /* Inputs + control func */)
  int nbImageMaxNobody;
#endif

  /*--------------------------------------------------------------
   * Control init done and exec tasks status 
   */
#if (1 /* not init rqst */)
  if (!controlExecTaskStatusAndInitRqst(servId, rqstId))
    return;  
#endif
 
  /*--------------------------------------------------------------
   * Activity allocation
   */
  activity = allocActivity(rqstId, 14, 0, 1, servId);
  if (activity == -1) return;

  /*--------------------------------------------------------------
   * Call control func
   */
#if (1 /* control func */)
  {
    STATUS status;
    int bilan;

#if ((1)  /* Input */)
    /* Get inputs */
    if (csServRqstParamsGet (servId, rqstId, (void *) &nbImageMaxNobody, 
			     sizeof((*icuDataStrId).nbImageMaxNobody), (FUNCPTR) NULL) != OK)
      icuReplyAndSuspend (servId, rqstId, TRUE);
    status = icuWaitForNobodyKnownCntrl (&nbImageMaxNobody, &bilan);
#else
    status = icuWaitForNobodyKnownCntrl (&bilan);
#endif

    /* Control not OK: stop there */
    if (status != OK) {
      freeActivity(activity);
      if (bilan == OK)
	bilan = S_icu_stdGenoM_CONTROL_CODEL_ERROR;
      if (csServReplySend (servId, rqstId, FINAL_REPLY, bilan, 
			   (void *) NULL, 0, (FUNCPTR) NULL) != OK)
	icuCntrlTaskSuspend (TRUE);
      return;
    }
  }
#endif

 /*-------------------------------------------------------------
  * Record inputs
  *       XXXXXXXX NON C'EST TROP TOT -> INTERROMPRE LES ACTIVITES AVANT !
  */
#if ((1) /* Input */)

  /* Get inputs address */
#if (1 /* multiple request  */)
  ACTIVITY_INPUT_ID(activity) = &(icuDataStrId->WaitForNobodyKnownInput)[activity];
#else
  ACTIVITY_INPUT_ID(activity) = &((*icuDataStrId).nbImageMaxNobody);
#endif                
  ACTIVITY_INPUT_SIZE(activity) = sizeof((*icuDataStrId).nbImageMaxNobody);

  /* Record inputs */
#if (1 /* control func */)
  memcpy ((void *) ACTIVITY_INPUT_ID(activity), (void *) &nbImageMaxNobody, 
	  ACTIVITY_INPUT_SIZE(activity));
#else
  if (csServRqstParamsGet (servId, rqstId, ACTIVITY_INPUT_ID(activity), 
			   sizeof((*icuDataStrId).nbImageMaxNobody), (FUNCPTR) NULL) != OK)
    icuReplyAndSuspend (servId, rqstId, TRUE);
#endif
#endif          
  
 /*-------------------------------------------------------------
  * Get outputs address
  */
#if ((0) /* Output */)
#if (1 /* Multiple request */)
  ACTIVITY_OUTPUT_ID(activity) = 
    &(icuDataStrId->WaitForNobodyKnownOutput)[activity];
#else
  ACTIVITY_OUTPUT_ID(activity) = NULL;
#endif            
  ACTIVITY_OUTPUT_SIZE(activity) = 0;
#endif          

  moduleEventCntrl.eventType = STATE_START_EVENT;
  moduleEventCntrl.activityNum = activity;
  moduleEventCntrl.activityState = INIT;
  moduleEventCntrl.rqstType = 14;
  moduleEventCntrl.taskNum = 0;
  sendModuleEvent(&moduleEventCntrl);

  /*-------------------------------------------------------------
   * Interruption of the incompatible activities
   */
  for (i = 0; i < MAX_ACTIVITIES; i++) 
    if (i != activity && ACTIVITY_STATUS(i) != ETHER 
	&& ! compatibilityTab[ACTIVITY_RQST_TYPE(i)]) 
      
      /* Record remaining incompatible activities */
      if (! icuAbortActivity (servId, i)) {
	ACTIVITY_TAB_INCOMP(activity)[ACTIVITY_NB_INCOMP(activity)] = i;
	ACTIVITY_NB_INCOMP(activity)++;
      }  
}



/*****************************************************************************
 *
 *  icuCntrlWaitForPersonListChange - Execution request
 */

#if (1 /* control func */)
  extern int icuWaitForPersonListChangeCntrl ();
#endif 

static void icuCntrlWaitForPersonListChange (SERV_ID servId, int rqstId)

{
  static int icuWaitForPersonListChangeCompatibility[] = { 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 0, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1 };
  int activity;
  int i;
  int *compatibilityTab = icuWaitForPersonListChangeCompatibility;
#if ((1) && 1 /* Inputs + control func */)
  ICU_PERSON_LIST_PARAMS paramsPersonList;
#endif

  /*--------------------------------------------------------------
   * Control init done and exec tasks status 
   */
#if (1 /* not init rqst */)
  if (!controlExecTaskStatusAndInitRqst(servId, rqstId))
    return;  
#endif
 
  /*--------------------------------------------------------------
   * Activity allocation
   */
  activity = allocActivity(rqstId, 15, 0, 0, servId);
  if (activity == -1) return;

  /*--------------------------------------------------------------
   * Call control func
   */
#if (1 /* control func */)
  {
    STATUS status;
    int bilan;

#if ((1)  /* Input */)
    /* Get inputs */
    if (csServRqstParamsGet (servId, rqstId, (void *) &paramsPersonList, 
			     sizeof((*icuDataStrId).paramsPersonList), (FUNCPTR) NULL) != OK)
      icuReplyAndSuspend (servId, rqstId, TRUE);
    status = icuWaitForPersonListChangeCntrl (&paramsPersonList, &bilan);
#else
    status = icuWaitForPersonListChangeCntrl (&bilan);
#endif

    /* Control not OK: stop there */
    if (status != OK) {
      freeActivity(activity);
      if (bilan == OK)
	bilan = S_icu_stdGenoM_CONTROL_CODEL_ERROR;
      if (csServReplySend (servId, rqstId, FINAL_REPLY, bilan, 
			   (void *) NULL, 0, (FUNCPTR) NULL) != OK)
	icuCntrlTaskSuspend (TRUE);
      return;
    }
  }
#endif

 /*-------------------------------------------------------------
  * Record inputs
  *       XXXXXXXX NON C'EST TROP TOT -> INTERROMPRE LES ACTIVITES AVANT !
  */
#if ((1) /* Input */)

  /* Get inputs address */
#if (0 /* multiple request  */)
  ACTIVITY_INPUT_ID(activity) = &(icuDataStrId->WaitForPersonListChangeInput)[activity];
#else
  ACTIVITY_INPUT_ID(activity) = &((*icuDataStrId).paramsPersonList);
#endif                
  ACTIVITY_INPUT_SIZE(activity) = sizeof((*icuDataStrId).paramsPersonList);

  /* Record inputs */
#if (1 /* control func */)
  memcpy ((void *) ACTIVITY_INPUT_ID(activity), (void *) &paramsPersonList, 
	  ACTIVITY_INPUT_SIZE(activity));
#else
  if (csServRqstParamsGet (servId, rqstId, ACTIVITY_INPUT_ID(activity), 
			   sizeof((*icuDataStrId).paramsPersonList), (FUNCPTR) NULL) != OK)
    icuReplyAndSuspend (servId, rqstId, TRUE);
#endif
#endif          
  
 /*-------------------------------------------------------------
  * Get outputs address
  */
#if ((1) /* Output */)
#if (0 /* Multiple request */)
  ACTIVITY_OUTPUT_ID(activity) = 
    &(icuDataStrId->WaitForPersonListChangeOutput)[activity];
#else
  ACTIVITY_OUTPUT_ID(activity) = &((*icuDataStrId).waitFor);
#endif            
  ACTIVITY_OUTPUT_SIZE(activity) = sizeof((*icuDataStrId).waitFor);
#endif          

  moduleEventCntrl.eventType = STATE_START_EVENT;
  moduleEventCntrl.activityNum = activity;
  moduleEventCntrl.activityState = INIT;
  moduleEventCntrl.rqstType = 15;
  moduleEventCntrl.taskNum = 0;
  sendModuleEvent(&moduleEventCntrl);

  /*-------------------------------------------------------------
   * Interruption of the incompatible activities
   */
  for (i = 0; i < MAX_ACTIVITIES; i++) 
    if (i != activity && ACTIVITY_STATUS(i) != ETHER 
	&& ! compatibilityTab[ACTIVITY_RQST_TYPE(i)]) 
      
      /* Record remaining incompatible activities */
      if (! icuAbortActivity (servId, i)) {
	ACTIVITY_TAB_INCOMP(activity)[ACTIVITY_NB_INCOMP(activity)] = i;
	ACTIVITY_NB_INCOMP(activity)++;
      }  
}



/*****************************************************************************
 *
 *  icuCntrlProcessSetModality - Execution request
 */

#if (1 /* control func */)
  extern int icuProcessSetModalityCntrl ();
#endif 

static void icuCntrlProcessSetModality (SERV_ID servId, int rqstId)

{
  static int icuProcessSetModalityCompatibility[] = { 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 0, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1 };
  int activity;
  int i;
  int *compatibilityTab = icuProcessSetModalityCompatibility;
#if ((1) && 1 /* Inputs + control func */)
  ICU_MODALITIES setModality;
#endif

  /*--------------------------------------------------------------
   * Control init done and exec tasks status 
   */
#if (1 /* not init rqst */)
  if (!controlExecTaskStatusAndInitRqst(servId, rqstId))
    return;  
#endif
 
  /*--------------------------------------------------------------
   * Activity allocation
   */
  activity = allocActivity(rqstId, 16, 0, 0, servId);
  if (activity == -1) return;

  /*--------------------------------------------------------------
   * Call control func
   */
#if (1 /* control func */)
  {
    STATUS status;
    int bilan;

#if ((1)  /* Input */)
    /* Get inputs */
    if (csServRqstParamsGet (servId, rqstId, (void *) &setModality, 
			     sizeof((*icuDataStrId).setModality), (FUNCPTR) NULL) != OK)
      icuReplyAndSuspend (servId, rqstId, TRUE);
    status = icuProcessSetModalityCntrl (&setModality, &bilan);
#else
    status = icuProcessSetModalityCntrl (&bilan);
#endif

    /* Control not OK: stop there */
    if (status != OK) {
      freeActivity(activity);
      if (bilan == OK)
	bilan = S_icu_stdGenoM_CONTROL_CODEL_ERROR;
      if (csServReplySend (servId, rqstId, FINAL_REPLY, bilan, 
			   (void *) NULL, 0, (FUNCPTR) NULL) != OK)
	icuCntrlTaskSuspend (TRUE);
      return;
    }
  }
#endif

 /*-------------------------------------------------------------
  * Record inputs
  *       XXXXXXXX NON C'EST TROP TOT -> INTERROMPRE LES ACTIVITES AVANT !
  */
#if ((1) /* Input */)

  /* Get inputs address */
#if (0 /* multiple request  */)
  ACTIVITY_INPUT_ID(activity) = &(icuDataStrId->ProcessSetModalityInput)[activity];
#else
  ACTIVITY_INPUT_ID(activity) = &((*icuDataStrId).setModality);
#endif                
  ACTIVITY_INPUT_SIZE(activity) = sizeof((*icuDataStrId).setModality);

  /* Record inputs */
#if (1 /* control func */)
  memcpy ((void *) ACTIVITY_INPUT_ID(activity), (void *) &setModality, 
	  ACTIVITY_INPUT_SIZE(activity));
#else
  if (csServRqstParamsGet (servId, rqstId, ACTIVITY_INPUT_ID(activity), 
			   sizeof((*icuDataStrId).setModality), (FUNCPTR) NULL) != OK)
    icuReplyAndSuspend (servId, rqstId, TRUE);
#endif
#endif          
  
 /*-------------------------------------------------------------
  * Get outputs address
  */
#if ((0) /* Output */)
#if (0 /* Multiple request */)
  ACTIVITY_OUTPUT_ID(activity) = 
    &(icuDataStrId->ProcessSetModalityOutput)[activity];
#else
  ACTIVITY_OUTPUT_ID(activity) = NULL;
#endif            
  ACTIVITY_OUTPUT_SIZE(activity) = 0;
#endif          

  moduleEventCntrl.eventType = STATE_START_EVENT;
  moduleEventCntrl.activityNum = activity;
  moduleEventCntrl.activityState = INIT;
  moduleEventCntrl.rqstType = 16;
  moduleEventCntrl.taskNum = 0;
  sendModuleEvent(&moduleEventCntrl);

  /*-------------------------------------------------------------
   * Interruption of the incompatible activities
   */
  for (i = 0; i < MAX_ACTIVITIES; i++) 
    if (i != activity && ACTIVITY_STATUS(i) != ETHER 
	&& ! compatibilityTab[ACTIVITY_RQST_TYPE(i)]) 
      
      /* Record remaining incompatible activities */
      if (! icuAbortActivity (servId, i)) {
	ACTIVITY_TAB_INCOMP(activity)[ACTIVITY_NB_INCOMP(activity)] = i;
	ACTIVITY_NB_INCOMP(activity)++;
      }  
}



/*****************************************************************************
 *
 *  icuCntrlFacerecSetWorkingPath - Execution request
 */

#if (1 /* control func */)
  extern int icuFacerecSetWorkingPathCntrl ();
#endif 

static void icuCntrlFacerecSetWorkingPath (SERV_ID servId, int rqstId)

{
  static int icuFacerecSetWorkingPathCompatibility[] = { 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 0, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1 };
  int activity;
  int i;
  int *compatibilityTab = icuFacerecSetWorkingPathCompatibility;
#if ((1) && 1 /* Inputs + control func */)
  GEN_STRING256 facerecWorkingPath;
#endif

  /*--------------------------------------------------------------
   * Control init done and exec tasks status 
   */
#if (1 /* not init rqst */)
  if (!controlExecTaskStatusAndInitRqst(servId, rqstId))
    return;  
#endif
 
  /*--------------------------------------------------------------
   * Activity allocation
   */
  activity = allocActivity(rqstId, 17, 0, 0, servId);
  if (activity == -1) return;

  /*--------------------------------------------------------------
   * Call control func
   */
#if (1 /* control func */)
  {
    STATUS status;
    int bilan;

#if ((1)  /* Input */)
    /* Get inputs */
    if (csServRqstParamsGet (servId, rqstId, (void *) &facerecWorkingPath, 
			     sizeof((*icuDataStrId).facerecWorkingPath), (FUNCPTR) NULL) != OK)
      icuReplyAndSuspend (servId, rqstId, TRUE);
    status = icuFacerecSetWorkingPathCntrl (&facerecWorkingPath, &bilan);
#else
    status = icuFacerecSetWorkingPathCntrl (&bilan);
#endif

    /* Control not OK: stop there */
    if (status != OK) {
      freeActivity(activity);
      if (bilan == OK)
	bilan = S_icu_stdGenoM_CONTROL_CODEL_ERROR;
      if (csServReplySend (servId, rqstId, FINAL_REPLY, bilan, 
			   (void *) NULL, 0, (FUNCPTR) NULL) != OK)
	icuCntrlTaskSuspend (TRUE);
      return;
    }
  }
#endif

 /*-------------------------------------------------------------
  * Record inputs
  *       XXXXXXXX NON C'EST TROP TOT -> INTERROMPRE LES ACTIVITES AVANT !
  */
#if ((1) /* Input */)

  /* Get inputs address */
#if (0 /* multiple request  */)
  ACTIVITY_INPUT_ID(activity) = &(icuDataStrId->FacerecSetWorkingPathInput)[activity];
#else
  ACTIVITY_INPUT_ID(activity) = &((*icuDataStrId).facerecWorkingPath);
#endif                
  ACTIVITY_INPUT_SIZE(activity) = sizeof((*icuDataStrId).facerecWorkingPath);

  /* Record inputs */
#if (1 /* control func */)
  memcpy ((void *) ACTIVITY_INPUT_ID(activity), (void *) &facerecWorkingPath, 
	  ACTIVITY_INPUT_SIZE(activity));
#else
  if (csServRqstParamsGet (servId, rqstId, ACTIVITY_INPUT_ID(activity), 
			   sizeof((*icuDataStrId).facerecWorkingPath), (FUNCPTR) NULL) != OK)
    icuReplyAndSuspend (servId, rqstId, TRUE);
#endif
#endif          
  
 /*-------------------------------------------------------------
  * Get outputs address
  */
#if ((0) /* Output */)
#if (0 /* Multiple request */)
  ACTIVITY_OUTPUT_ID(activity) = 
    &(icuDataStrId->FacerecSetWorkingPathOutput)[activity];
#else
  ACTIVITY_OUTPUT_ID(activity) = NULL;
#endif            
  ACTIVITY_OUTPUT_SIZE(activity) = 0;
#endif          

  moduleEventCntrl.eventType = STATE_START_EVENT;
  moduleEventCntrl.activityNum = activity;
  moduleEventCntrl.activityState = INIT;
  moduleEventCntrl.rqstType = 17;
  moduleEventCntrl.taskNum = 0;
  sendModuleEvent(&moduleEventCntrl);

  /*-------------------------------------------------------------
   * Interruption of the incompatible activities
   */
  for (i = 0; i < MAX_ACTIVITIES; i++) 
    if (i != activity && ACTIVITY_STATUS(i) != ETHER 
	&& ! compatibilityTab[ACTIVITY_RQST_TYPE(i)]) 
      
      /* Record remaining incompatible activities */
      if (! icuAbortActivity (servId, i)) {
	ACTIVITY_TAB_INCOMP(activity)[ACTIVITY_NB_INCOMP(activity)] = i;
	ACTIVITY_NB_INCOMP(activity)++;
      }  
}



/*****************************************************************************
 *
 *  icuCntrlFacerecSetParams - Execution request
 */

#if (1 /* control func */)
  extern int icuFacerecSetParamsCntrl ();
#endif 

static void icuCntrlFacerecSetParams (SERV_ID servId, int rqstId)

{
  static int icuFacerecSetParamsCompatibility[] = { 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 0, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1 };
  int activity;
  int i;
  int *compatibilityTab = icuFacerecSetParamsCompatibility;
#if ((1) && 1 /* Inputs + control func */)
  ICU_PERSON_LIST_PARAMS paramsPersonList;
#endif

  /*--------------------------------------------------------------
   * Control init done and exec tasks status 
   */
#if (1 /* not init rqst */)
  if (!controlExecTaskStatusAndInitRqst(servId, rqstId))
    return;  
#endif
 
  /*--------------------------------------------------------------
   * Activity allocation
   */
  activity = allocActivity(rqstId, 18, 0, 0, servId);
  if (activity == -1) return;

  /*--------------------------------------------------------------
   * Call control func
   */
#if (1 /* control func */)
  {
    STATUS status;
    int bilan;

#if ((1)  /* Input */)
    /* Get inputs */
    if (csServRqstParamsGet (servId, rqstId, (void *) &paramsPersonList, 
			     sizeof((*icuDataStrId).paramsPersonList), (FUNCPTR) NULL) != OK)
      icuReplyAndSuspend (servId, rqstId, TRUE);
    status = icuFacerecSetParamsCntrl (&paramsPersonList, &bilan);
#else
    status = icuFacerecSetParamsCntrl (&bilan);
#endif

    /* Control not OK: stop there */
    if (status != OK) {
      freeActivity(activity);
      if (bilan == OK)
	bilan = S_icu_stdGenoM_CONTROL_CODEL_ERROR;
      if (csServReplySend (servId, rqstId, FINAL_REPLY, bilan, 
			   (void *) NULL, 0, (FUNCPTR) NULL) != OK)
	icuCntrlTaskSuspend (TRUE);
      return;
    }
  }
#endif

 /*-------------------------------------------------------------
  * Record inputs
  *       XXXXXXXX NON C'EST TROP TOT -> INTERROMPRE LES ACTIVITES AVANT !
  */
#if ((1) /* Input */)

  /* Get inputs address */
#if (0 /* multiple request  */)
  ACTIVITY_INPUT_ID(activity) = &(icuDataStrId->FacerecSetParamsInput)[activity];
#else
  ACTIVITY_INPUT_ID(activity) = &((*icuDataStrId).paramsPersonList);
#endif                
  ACTIVITY_INPUT_SIZE(activity) = sizeof((*icuDataStrId).paramsPersonList);

  /* Record inputs */
#if (1 /* control func */)
  memcpy ((void *) ACTIVITY_INPUT_ID(activity), (void *) &paramsPersonList, 
	  ACTIVITY_INPUT_SIZE(activity));
#else
  if (csServRqstParamsGet (servId, rqstId, ACTIVITY_INPUT_ID(activity), 
			   sizeof((*icuDataStrId).paramsPersonList), (FUNCPTR) NULL) != OK)
    icuReplyAndSuspend (servId, rqstId, TRUE);
#endif
#endif          
  
 /*-------------------------------------------------------------
  * Get outputs address
  */
#if ((0) /* Output */)
#if (0 /* Multiple request */)
  ACTIVITY_OUTPUT_ID(activity) = 
    &(icuDataStrId->FacerecSetParamsOutput)[activity];
#else
  ACTIVITY_OUTPUT_ID(activity) = NULL;
#endif            
  ACTIVITY_OUTPUT_SIZE(activity) = 0;
#endif          

  moduleEventCntrl.eventType = STATE_START_EVENT;
  moduleEventCntrl.activityNum = activity;
  moduleEventCntrl.activityState = INIT;
  moduleEventCntrl.rqstType = 18;
  moduleEventCntrl.taskNum = 0;
  sendModuleEvent(&moduleEventCntrl);

  /*-------------------------------------------------------------
   * Interruption of the incompatible activities
   */
  for (i = 0; i < MAX_ACTIVITIES; i++) 
    if (i != activity && ACTIVITY_STATUS(i) != ETHER 
	&& ! compatibilityTab[ACTIVITY_RQST_TYPE(i)]) 
      
      /* Record remaining incompatible activities */
      if (! icuAbortActivity (servId, i)) {
	ACTIVITY_TAB_INCOMP(activity)[ACTIVITY_NB_INCOMP(activity)] = i;
	ACTIVITY_NB_INCOMP(activity)++;
      }  
}



/*****************************************************************************
 *
 *  icuCntrlFacerecInit - Execution request
 */

#if (1 /* control func */)
  extern int icuFacerecInitCntrl ();
#endif 

static void icuCntrlFacerecInit (SERV_ID servId, int rqstId)

{
  static int icuFacerecInitCompatibility[] = { 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 0, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1 };
  int activity;
  int i;
  int *compatibilityTab = icuFacerecInitCompatibility;
#if ((0) && 1 /* Inputs + control func */)
  
#endif

  /*--------------------------------------------------------------
   * Control init done and exec tasks status 
   */
#if (1 /* not init rqst */)
  if (!controlExecTaskStatusAndInitRqst(servId, rqstId))
    return;  
#endif
 
  /*--------------------------------------------------------------
   * Activity allocation
   */
  activity = allocActivity(rqstId, 19, 0, 0, servId);
  if (activity == -1) return;

  /*--------------------------------------------------------------
   * Call control func
   */
#if (1 /* control func */)
  {
    STATUS status;
    int bilan;

#if ((0)  /* Input */)
    /* Get inputs */
    if (csServRqstParamsGet (servId, rqstId, (void *) NULL, 
			     0, (FUNCPTR) NULL) != OK)
      icuReplyAndSuspend (servId, rqstId, TRUE);
    status = icuFacerecInitCntrl (NULL, &bilan);
#else
    status = icuFacerecInitCntrl (&bilan);
#endif

    /* Control not OK: stop there */
    if (status != OK) {
      freeActivity(activity);
      if (bilan == OK)
	bilan = S_icu_stdGenoM_CONTROL_CODEL_ERROR;
      if (csServReplySend (servId, rqstId, FINAL_REPLY, bilan, 
			   (void *) NULL, 0, (FUNCPTR) NULL) != OK)
	icuCntrlTaskSuspend (TRUE);
      return;
    }
  }
#endif

 /*-------------------------------------------------------------
  * Record inputs
  *       XXXXXXXX NON C'EST TROP TOT -> INTERROMPRE LES ACTIVITES AVANT !
  */
#if ((0) /* Input */)

  /* Get inputs address */
#if (0 /* multiple request  */)
  ACTIVITY_INPUT_ID(activity) = &(icuDataStrId->FacerecInitInput)[activity];
#else
  ACTIVITY_INPUT_ID(activity) = NULL;
#endif                
  ACTIVITY_INPUT_SIZE(activity) = 0;

  /* Record inputs */
#if (1 /* control func */)
  memcpy ((void *) ACTIVITY_INPUT_ID(activity), (void *) NULL, 
	  ACTIVITY_INPUT_SIZE(activity));
#else
  if (csServRqstParamsGet (servId, rqstId, ACTIVITY_INPUT_ID(activity), 
			   0, (FUNCPTR) NULL) != OK)
    icuReplyAndSuspend (servId, rqstId, TRUE);
#endif
#endif          
  
 /*-------------------------------------------------------------
  * Get outputs address
  */
#if ((0) /* Output */)
#if (0 /* Multiple request */)
  ACTIVITY_OUTPUT_ID(activity) = 
    &(icuDataStrId->FacerecInitOutput)[activity];
#else
  ACTIVITY_OUTPUT_ID(activity) = NULL;
#endif            
  ACTIVITY_OUTPUT_SIZE(activity) = 0;
#endif          

  moduleEventCntrl.eventType = STATE_START_EVENT;
  moduleEventCntrl.activityNum = activity;
  moduleEventCntrl.activityState = INIT;
  moduleEventCntrl.rqstType = 19;
  moduleEventCntrl.taskNum = 0;
  sendModuleEvent(&moduleEventCntrl);

  /*-------------------------------------------------------------
   * Interruption of the incompatible activities
   */
  for (i = 0; i < MAX_ACTIVITIES; i++) 
    if (i != activity && ACTIVITY_STATUS(i) != ETHER 
	&& ! compatibilityTab[ACTIVITY_RQST_TYPE(i)]) 
      
      /* Record remaining incompatible activities */
      if (! icuAbortActivity (servId, i)) {
	ACTIVITY_TAB_INCOMP(activity)[ACTIVITY_NB_INCOMP(activity)] = i;
	ACTIVITY_NB_INCOMP(activity)++;
      }  
}



/*****************************************************************************
 *
 *  icuCntrlFacerecAddSomebody - Execution request
 */

#if (1 /* control func */)
  extern int icuFacerecAddSomebodyCntrl ();
#endif 

static void icuCntrlFacerecAddSomebody (SERV_ID servId, int rqstId)

{
  static int icuFacerecAddSomebodyCompatibility[] = { 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 0, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1 };
  int activity;
  int i;
  int *compatibilityTab = icuFacerecAddSomebodyCompatibility;
#if ((1) && 1 /* Inputs + control func */)
  GEN_STRING256 recoAddName;
#endif

  /*--------------------------------------------------------------
   * Control init done and exec tasks status 
   */
#if (1 /* not init rqst */)
  if (!controlExecTaskStatusAndInitRqst(servId, rqstId))
    return;  
#endif
 
  /*--------------------------------------------------------------
   * Activity allocation
   */
  activity = allocActivity(rqstId, 20, 0, 0, servId);
  if (activity == -1) return;

  /*--------------------------------------------------------------
   * Call control func
   */
#if (1 /* control func */)
  {
    STATUS status;
    int bilan;

#if ((1)  /* Input */)
    /* Get inputs */
    if (csServRqstParamsGet (servId, rqstId, (void *) &recoAddName, 
			     sizeof((*icuDataStrId).recoAddName), (FUNCPTR) NULL) != OK)
      icuReplyAndSuspend (servId, rqstId, TRUE);
    status = icuFacerecAddSomebodyCntrl (&recoAddName, &bilan);
#else
    status = icuFacerecAddSomebodyCntrl (&bilan);
#endif

    /* Control not OK: stop there */
    if (status != OK) {
      freeActivity(activity);
      if (bilan == OK)
	bilan = S_icu_stdGenoM_CONTROL_CODEL_ERROR;
      if (csServReplySend (servId, rqstId, FINAL_REPLY, bilan, 
			   (void *) NULL, 0, (FUNCPTR) NULL) != OK)
	icuCntrlTaskSuspend (TRUE);
      return;
    }
  }
#endif

 /*-------------------------------------------------------------
  * Record inputs
  *       XXXXXXXX NON C'EST TROP TOT -> INTERROMPRE LES ACTIVITES AVANT !
  */
#if ((1) /* Input */)

  /* Get inputs address */
#if (0 /* multiple request  */)
  ACTIVITY_INPUT_ID(activity) = &(icuDataStrId->FacerecAddSomebodyInput)[activity];
#else
  ACTIVITY_INPUT_ID(activity) = &((*icuDataStrId).recoAddName);
#endif                
  ACTIVITY_INPUT_SIZE(activity) = sizeof((*icuDataStrId).recoAddName);

  /* Record inputs */
#if (1 /* control func */)
  memcpy ((void *) ACTIVITY_INPUT_ID(activity), (void *) &recoAddName, 
	  ACTIVITY_INPUT_SIZE(activity));
#else
  if (csServRqstParamsGet (servId, rqstId, ACTIVITY_INPUT_ID(activity), 
			   sizeof((*icuDataStrId).recoAddName), (FUNCPTR) NULL) != OK)
    icuReplyAndSuspend (servId, rqstId, TRUE);
#endif
#endif          
  
 /*-------------------------------------------------------------
  * Get outputs address
  */
#if ((0) /* Output */)
#if (0 /* Multiple request */)
  ACTIVITY_OUTPUT_ID(activity) = 
    &(icuDataStrId->FacerecAddSomebodyOutput)[activity];
#else
  ACTIVITY_OUTPUT_ID(activity) = NULL;
#endif            
  ACTIVITY_OUTPUT_SIZE(activity) = 0;
#endif          

  moduleEventCntrl.eventType = STATE_START_EVENT;
  moduleEventCntrl.activityNum = activity;
  moduleEventCntrl.activityState = INIT;
  moduleEventCntrl.rqstType = 20;
  moduleEventCntrl.taskNum = 0;
  sendModuleEvent(&moduleEventCntrl);

  /*-------------------------------------------------------------
   * Interruption of the incompatible activities
   */
  for (i = 0; i < MAX_ACTIVITIES; i++) 
    if (i != activity && ACTIVITY_STATUS(i) != ETHER 
	&& ! compatibilityTab[ACTIVITY_RQST_TYPE(i)]) 
      
      /* Record remaining incompatible activities */
      if (! icuAbortActivity (servId, i)) {
	ACTIVITY_TAB_INCOMP(activity)[ACTIVITY_NB_INCOMP(activity)] = i;
	ACTIVITY_NB_INCOMP(activity)++;
      }  
}



/*****************************************************************************
 *
 *  icuCntrlFacerecSetPeople - Execution request
 */

#if (1 /* control func */)
  extern int icuFacerecSetPeopleCntrl ();
#endif 

static void icuCntrlFacerecSetPeople (SERV_ID servId, int rqstId)

{
  static int icuFacerecSetPeopleCompatibility[] = { 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 0, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1 };
  int activity;
  int i;
  int *compatibilityTab = icuFacerecSetPeopleCompatibility;
#if ((1) && 1 /* Inputs + control func */)
  GEN_STRING256 recoName;
#endif

  /*--------------------------------------------------------------
   * Control init done and exec tasks status 
   */
#if (1 /* not init rqst */)
  if (!controlExecTaskStatusAndInitRqst(servId, rqstId))
    return;  
#endif
 
  /*--------------------------------------------------------------
   * Activity allocation
   */
  activity = allocActivity(rqstId, 21, 0, 0, servId);
  if (activity == -1) return;

  /*--------------------------------------------------------------
   * Call control func
   */
#if (1 /* control func */)
  {
    STATUS status;
    int bilan;

#if ((1)  /* Input */)
    /* Get inputs */
    if (csServRqstParamsGet (servId, rqstId, (void *) &recoName, 
			     sizeof((*icuDataStrId).recoName), (FUNCPTR) NULL) != OK)
      icuReplyAndSuspend (servId, rqstId, TRUE);
    status = icuFacerecSetPeopleCntrl (&recoName, &bilan);
#else
    status = icuFacerecSetPeopleCntrl (&bilan);
#endif

    /* Control not OK: stop there */
    if (status != OK) {
      freeActivity(activity);
      if (bilan == OK)
	bilan = S_icu_stdGenoM_CONTROL_CODEL_ERROR;
      if (csServReplySend (servId, rqstId, FINAL_REPLY, bilan, 
			   (void *) NULL, 0, (FUNCPTR) NULL) != OK)
	icuCntrlTaskSuspend (TRUE);
      return;
    }
  }
#endif

 /*-------------------------------------------------------------
  * Record inputs
  *       XXXXXXXX NON C'EST TROP TOT -> INTERROMPRE LES ACTIVITES AVANT !
  */
#if ((1) /* Input */)

  /* Get inputs address */
#if (0 /* multiple request  */)
  ACTIVITY_INPUT_ID(activity) = &(icuDataStrId->FacerecSetPeopleInput)[activity];
#else
  ACTIVITY_INPUT_ID(activity) = &((*icuDataStrId).recoName);
#endif                
  ACTIVITY_INPUT_SIZE(activity) = sizeof((*icuDataStrId).recoName);

  /* Record inputs */
#if (1 /* control func */)
  memcpy ((void *) ACTIVITY_INPUT_ID(activity), (void *) &recoName, 
	  ACTIVITY_INPUT_SIZE(activity));
#else
  if (csServRqstParamsGet (servId, rqstId, ACTIVITY_INPUT_ID(activity), 
			   sizeof((*icuDataStrId).recoName), (FUNCPTR) NULL) != OK)
    icuReplyAndSuspend (servId, rqstId, TRUE);
#endif
#endif          
  
 /*-------------------------------------------------------------
  * Get outputs address
  */
#if ((0) /* Output */)
#if (0 /* Multiple request */)
  ACTIVITY_OUTPUT_ID(activity) = 
    &(icuDataStrId->FacerecSetPeopleOutput)[activity];
#else
  ACTIVITY_OUTPUT_ID(activity) = NULL;
#endif            
  ACTIVITY_OUTPUT_SIZE(activity) = 0;
#endif          

  moduleEventCntrl.eventType = STATE_START_EVENT;
  moduleEventCntrl.activityNum = activity;
  moduleEventCntrl.activityState = INIT;
  moduleEventCntrl.rqstType = 21;
  moduleEventCntrl.taskNum = 0;
  sendModuleEvent(&moduleEventCntrl);

  /*-------------------------------------------------------------
   * Interruption of the incompatible activities
   */
  for (i = 0; i < MAX_ACTIVITIES; i++) 
    if (i != activity && ACTIVITY_STATUS(i) != ETHER 
	&& ! compatibilityTab[ACTIVITY_RQST_TYPE(i)]) 
      
      /* Record remaining incompatible activities */
      if (! icuAbortActivity (servId, i)) {
	ACTIVITY_TAB_INCOMP(activity)[ACTIVITY_NB_INCOMP(activity)] = i;
	ACTIVITY_NB_INCOMP(activity)++;
      }  
}



/*****************************************************************************
 *
 *  icuCntrlFacerecRecognizeAll - Execution request
 */

#if (1 /* control func */)
  extern int icuFacerecRecognizeAllCntrl ();
#endif 

static void icuCntrlFacerecRecognizeAll (SERV_ID servId, int rqstId)

{
  static int icuFacerecRecognizeAllCompatibility[] = { 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 0, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1 };
  int activity;
  int i;
  int *compatibilityTab = icuFacerecRecognizeAllCompatibility;
#if ((0) && 1 /* Inputs + control func */)
  
#endif

  /*--------------------------------------------------------------
   * Control init done and exec tasks status 
   */
#if (1 /* not init rqst */)
  if (!controlExecTaskStatusAndInitRqst(servId, rqstId))
    return;  
#endif
 
  /*--------------------------------------------------------------
   * Activity allocation
   */
  activity = allocActivity(rqstId, 22, 0, 0, servId);
  if (activity == -1) return;

  /*--------------------------------------------------------------
   * Call control func
   */
#if (1 /* control func */)
  {
    STATUS status;
    int bilan;

#if ((0)  /* Input */)
    /* Get inputs */
    if (csServRqstParamsGet (servId, rqstId, (void *) NULL, 
			     0, (FUNCPTR) NULL) != OK)
      icuReplyAndSuspend (servId, rqstId, TRUE);
    status = icuFacerecRecognizeAllCntrl (NULL, &bilan);
#else
    status = icuFacerecRecognizeAllCntrl (&bilan);
#endif

    /* Control not OK: stop there */
    if (status != OK) {
      freeActivity(activity);
      if (bilan == OK)
	bilan = S_icu_stdGenoM_CONTROL_CODEL_ERROR;
      if (csServReplySend (servId, rqstId, FINAL_REPLY, bilan, 
			   (void *) NULL, 0, (FUNCPTR) NULL) != OK)
	icuCntrlTaskSuspend (TRUE);
      return;
    }
  }
#endif

 /*-------------------------------------------------------------
  * Record inputs
  *       XXXXXXXX NON C'EST TROP TOT -> INTERROMPRE LES ACTIVITES AVANT !
  */
#if ((0) /* Input */)

  /* Get inputs address */
#if (0 /* multiple request  */)
  ACTIVITY_INPUT_ID(activity) = &(icuDataStrId->FacerecRecognizeAllInput)[activity];
#else
  ACTIVITY_INPUT_ID(activity) = NULL;
#endif                
  ACTIVITY_INPUT_SIZE(activity) = 0;

  /* Record inputs */
#if (1 /* control func */)
  memcpy ((void *) ACTIVITY_INPUT_ID(activity), (void *) NULL, 
	  ACTIVITY_INPUT_SIZE(activity));
#else
  if (csServRqstParamsGet (servId, rqstId, ACTIVITY_INPUT_ID(activity), 
			   0, (FUNCPTR) NULL) != OK)
    icuReplyAndSuspend (servId, rqstId, TRUE);
#endif
#endif          
  
 /*-------------------------------------------------------------
  * Get outputs address
  */
#if ((0) /* Output */)
#if (0 /* Multiple request */)
  ACTIVITY_OUTPUT_ID(activity) = 
    &(icuDataStrId->FacerecRecognizeAllOutput)[activity];
#else
  ACTIVITY_OUTPUT_ID(activity) = NULL;
#endif            
  ACTIVITY_OUTPUT_SIZE(activity) = 0;
#endif          

  moduleEventCntrl.eventType = STATE_START_EVENT;
  moduleEventCntrl.activityNum = activity;
  moduleEventCntrl.activityState = INIT;
  moduleEventCntrl.rqstType = 22;
  moduleEventCntrl.taskNum = 0;
  sendModuleEvent(&moduleEventCntrl);

  /*-------------------------------------------------------------
   * Interruption of the incompatible activities
   */
  for (i = 0; i < MAX_ACTIVITIES; i++) 
    if (i != activity && ACTIVITY_STATUS(i) != ETHER 
	&& ! compatibilityTab[ACTIVITY_RQST_TYPE(i)]) 
      
      /* Record remaining incompatible activities */
      if (! icuAbortActivity (servId, i)) {
	ACTIVITY_TAB_INCOMP(activity)[ACTIVITY_NB_INCOMP(activity)] = i;
	ACTIVITY_NB_INCOMP(activity)++;
      }  
}



/*****************************************************************************
 *
 *  icuCntrlFacerecLoadSomebody - Execution request
 */

#if (1 /* control func */)
  extern int icuFacerecLoadSomebodyCntrl ();
#endif 

static void icuCntrlFacerecLoadSomebody (SERV_ID servId, int rqstId)

{
  static int icuFacerecLoadSomebodyCompatibility[] = { 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 0, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1 };
  int activity;
  int i;
  int *compatibilityTab = icuFacerecLoadSomebodyCompatibility;
#if ((1) && 1 /* Inputs + control func */)
  GEN_STRING256 recoName;
#endif

  /*--------------------------------------------------------------
   * Control init done and exec tasks status 
   */
#if (1 /* not init rqst */)
  if (!controlExecTaskStatusAndInitRqst(servId, rqstId))
    return;  
#endif
 
  /*--------------------------------------------------------------
   * Activity allocation
   */
  activity = allocActivity(rqstId, 23, 0, 0, servId);
  if (activity == -1) return;

  /*--------------------------------------------------------------
   * Call control func
   */
#if (1 /* control func */)
  {
    STATUS status;
    int bilan;

#if ((1)  /* Input */)
    /* Get inputs */
    if (csServRqstParamsGet (servId, rqstId, (void *) &recoName, 
			     sizeof((*icuDataStrId).recoName), (FUNCPTR) NULL) != OK)
      icuReplyAndSuspend (servId, rqstId, TRUE);
    status = icuFacerecLoadSomebodyCntrl (&recoName, &bilan);
#else
    status = icuFacerecLoadSomebodyCntrl (&bilan);
#endif

    /* Control not OK: stop there */
    if (status != OK) {
      freeActivity(activity);
      if (bilan == OK)
	bilan = S_icu_stdGenoM_CONTROL_CODEL_ERROR;
      if (csServReplySend (servId, rqstId, FINAL_REPLY, bilan, 
			   (void *) NULL, 0, (FUNCPTR) NULL) != OK)
	icuCntrlTaskSuspend (TRUE);
      return;
    }
  }
#endif

 /*-------------------------------------------------------------
  * Record inputs
  *       XXXXXXXX NON C'EST TROP TOT -> INTERROMPRE LES ACTIVITES AVANT !
  */
#if ((1) /* Input */)

  /* Get inputs address */
#if (0 /* multiple request  */)
  ACTIVITY_INPUT_ID(activity) = &(icuDataStrId->FacerecLoadSomebodyInput)[activity];
#else
  ACTIVITY_INPUT_ID(activity) = &((*icuDataStrId).recoName);
#endif                
  ACTIVITY_INPUT_SIZE(activity) = sizeof((*icuDataStrId).recoName);

  /* Record inputs */
#if (1 /* control func */)
  memcpy ((void *) ACTIVITY_INPUT_ID(activity), (void *) &recoName, 
	  ACTIVITY_INPUT_SIZE(activity));
#else
  if (csServRqstParamsGet (servId, rqstId, ACTIVITY_INPUT_ID(activity), 
			   sizeof((*icuDataStrId).recoName), (FUNCPTR) NULL) != OK)
    icuReplyAndSuspend (servId, rqstId, TRUE);
#endif
#endif          
  
 /*-------------------------------------------------------------
  * Get outputs address
  */
#if ((0) /* Output */)
#if (0 /* Multiple request */)
  ACTIVITY_OUTPUT_ID(activity) = 
    &(icuDataStrId->FacerecLoadSomebodyOutput)[activity];
#else
  ACTIVITY_OUTPUT_ID(activity) = NULL;
#endif            
  ACTIVITY_OUTPUT_SIZE(activity) = 0;
#endif          

  moduleEventCntrl.eventType = STATE_START_EVENT;
  moduleEventCntrl.activityNum = activity;
  moduleEventCntrl.activityState = INIT;
  moduleEventCntrl.rqstType = 23;
  moduleEventCntrl.taskNum = 0;
  sendModuleEvent(&moduleEventCntrl);

  /*-------------------------------------------------------------
   * Interruption of the incompatible activities
   */
  for (i = 0; i < MAX_ACTIVITIES; i++) 
    if (i != activity && ACTIVITY_STATUS(i) != ETHER 
	&& ! compatibilityTab[ACTIVITY_RQST_TYPE(i)]) 
      
      /* Record remaining incompatible activities */
      if (! icuAbortActivity (servId, i)) {
	ACTIVITY_TAB_INCOMP(activity)[ACTIVITY_NB_INCOMP(activity)] = i;
	ACTIVITY_NB_INCOMP(activity)++;
      }  
}



/*****************************************************************************
 *
 *  icuCntrlFacerecUnloadSomebody - Execution request
 */

#if (1 /* control func */)
  extern int icuFacerecUnloadSomebodyCntrl ();
#endif 

static void icuCntrlFacerecUnloadSomebody (SERV_ID servId, int rqstId)

{
  static int icuFacerecUnloadSomebodyCompatibility[] = { 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 0, 1, 1, 1, 1, 1, 1, 1, 1, 1 };
  int activity;
  int i;
  int *compatibilityTab = icuFacerecUnloadSomebodyCompatibility;
#if ((1) && 1 /* Inputs + control func */)
  GEN_STRING256 recoName;
#endif

  /*--------------------------------------------------------------
   * Control init done and exec tasks status 
   */
#if (1 /* not init rqst */)
  if (!controlExecTaskStatusAndInitRqst(servId, rqstId))
    return;  
#endif
 
  /*--------------------------------------------------------------
   * Activity allocation
   */
  activity = allocActivity(rqstId, 24, 0, 0, servId);
  if (activity == -1) return;

  /*--------------------------------------------------------------
   * Call control func
   */
#if (1 /* control func */)
  {
    STATUS status;
    int bilan;

#if ((1)  /* Input */)
    /* Get inputs */
    if (csServRqstParamsGet (servId, rqstId, (void *) &recoName, 
			     sizeof((*icuDataStrId).recoName), (FUNCPTR) NULL) != OK)
      icuReplyAndSuspend (servId, rqstId, TRUE);
    status = icuFacerecUnloadSomebodyCntrl (&recoName, &bilan);
#else
    status = icuFacerecUnloadSomebodyCntrl (&bilan);
#endif

    /* Control not OK: stop there */
    if (status != OK) {
      freeActivity(activity);
      if (bilan == OK)
	bilan = S_icu_stdGenoM_CONTROL_CODEL_ERROR;
      if (csServReplySend (servId, rqstId, FINAL_REPLY, bilan, 
			   (void *) NULL, 0, (FUNCPTR) NULL) != OK)
	icuCntrlTaskSuspend (TRUE);
      return;
    }
  }
#endif

 /*-------------------------------------------------------------
  * Record inputs
  *       XXXXXXXX NON C'EST TROP TOT -> INTERROMPRE LES ACTIVITES AVANT !
  */
#if ((1) /* Input */)

  /* Get inputs address */
#if (0 /* multiple request  */)
  ACTIVITY_INPUT_ID(activity) = &(icuDataStrId->FacerecUnloadSomebodyInput)[activity];
#else
  ACTIVITY_INPUT_ID(activity) = &((*icuDataStrId).recoName);
#endif                
  ACTIVITY_INPUT_SIZE(activity) = sizeof((*icuDataStrId).recoName);

  /* Record inputs */
#if (1 /* control func */)
  memcpy ((void *) ACTIVITY_INPUT_ID(activity), (void *) &recoName, 
	  ACTIVITY_INPUT_SIZE(activity));
#else
  if (csServRqstParamsGet (servId, rqstId, ACTIVITY_INPUT_ID(activity), 
			   sizeof((*icuDataStrId).recoName), (FUNCPTR) NULL) != OK)
    icuReplyAndSuspend (servId, rqstId, TRUE);
#endif
#endif          
  
 /*-------------------------------------------------------------
  * Get outputs address
  */
#if ((0) /* Output */)
#if (0 /* Multiple request */)
  ACTIVITY_OUTPUT_ID(activity) = 
    &(icuDataStrId->FacerecUnloadSomebodyOutput)[activity];
#else
  ACTIVITY_OUTPUT_ID(activity) = NULL;
#endif            
  ACTIVITY_OUTPUT_SIZE(activity) = 0;
#endif          

  moduleEventCntrl.eventType = STATE_START_EVENT;
  moduleEventCntrl.activityNum = activity;
  moduleEventCntrl.activityState = INIT;
  moduleEventCntrl.rqstType = 24;
  moduleEventCntrl.taskNum = 0;
  sendModuleEvent(&moduleEventCntrl);

  /*-------------------------------------------------------------
   * Interruption of the incompatible activities
   */
  for (i = 0; i < MAX_ACTIVITIES; i++) 
    if (i != activity && ACTIVITY_STATUS(i) != ETHER 
	&& ! compatibilityTab[ACTIVITY_RQST_TYPE(i)]) 
      
      /* Record remaining incompatible activities */
      if (! icuAbortActivity (servId, i)) {
	ACTIVITY_TAB_INCOMP(activity)[ACTIVITY_NB_INCOMP(activity)] = i;
	ACTIVITY_NB_INCOMP(activity)++;
      }  
}



/*****************************************************************************
 *
 *  icuCntrlFacerecLoadEverybody - Execution request
 */

#if (1 /* control func */)
  extern int icuFacerecLoadEverybodyCntrl ();
#endif 

static void icuCntrlFacerecLoadEverybody (SERV_ID servId, int rqstId)

{
  static int icuFacerecLoadEverybodyCompatibility[] = { 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 0, 1, 1, 1, 1, 1, 1, 1, 1 };
  int activity;
  int i;
  int *compatibilityTab = icuFacerecLoadEverybodyCompatibility;
#if ((0) && 1 /* Inputs + control func */)
  
#endif

  /*--------------------------------------------------------------
   * Control init done and exec tasks status 
   */
#if (1 /* not init rqst */)
  if (!controlExecTaskStatusAndInitRqst(servId, rqstId))
    return;  
#endif
 
  /*--------------------------------------------------------------
   * Activity allocation
   */
  activity = allocActivity(rqstId, 25, 0, 0, servId);
  if (activity == -1) return;

  /*--------------------------------------------------------------
   * Call control func
   */
#if (1 /* control func */)
  {
    STATUS status;
    int bilan;

#if ((0)  /* Input */)
    /* Get inputs */
    if (csServRqstParamsGet (servId, rqstId, (void *) NULL, 
			     0, (FUNCPTR) NULL) != OK)
      icuReplyAndSuspend (servId, rqstId, TRUE);
    status = icuFacerecLoadEverybodyCntrl (NULL, &bilan);
#else
    status = icuFacerecLoadEverybodyCntrl (&bilan);
#endif

    /* Control not OK: stop there */
    if (status != OK) {
      freeActivity(activity);
      if (bilan == OK)
	bilan = S_icu_stdGenoM_CONTROL_CODEL_ERROR;
      if (csServReplySend (servId, rqstId, FINAL_REPLY, bilan, 
			   (void *) NULL, 0, (FUNCPTR) NULL) != OK)
	icuCntrlTaskSuspend (TRUE);
      return;
    }
  }
#endif

 /*-------------------------------------------------------------
  * Record inputs
  *       XXXXXXXX NON C'EST TROP TOT -> INTERROMPRE LES ACTIVITES AVANT !
  */
#if ((0) /* Input */)

  /* Get inputs address */
#if (0 /* multiple request  */)
  ACTIVITY_INPUT_ID(activity) = &(icuDataStrId->FacerecLoadEverybodyInput)[activity];
#else
  ACTIVITY_INPUT_ID(activity) = NULL;
#endif                
  ACTIVITY_INPUT_SIZE(activity) = 0;

  /* Record inputs */
#if (1 /* control func */)
  memcpy ((void *) ACTIVITY_INPUT_ID(activity), (void *) NULL, 
	  ACTIVITY_INPUT_SIZE(activity));
#else
  if (csServRqstParamsGet (servId, rqstId, ACTIVITY_INPUT_ID(activity), 
			   0, (FUNCPTR) NULL) != OK)
    icuReplyAndSuspend (servId, rqstId, TRUE);
#endif
#endif          
  
 /*-------------------------------------------------------------
  * Get outputs address
  */
#if ((0) /* Output */)
#if (0 /* Multiple request */)
  ACTIVITY_OUTPUT_ID(activity) = 
    &(icuDataStrId->FacerecLoadEverybodyOutput)[activity];
#else
  ACTIVITY_OUTPUT_ID(activity) = NULL;
#endif            
  ACTIVITY_OUTPUT_SIZE(activity) = 0;
#endif          

  moduleEventCntrl.eventType = STATE_START_EVENT;
  moduleEventCntrl.activityNum = activity;
  moduleEventCntrl.activityState = INIT;
  moduleEventCntrl.rqstType = 25;
  moduleEventCntrl.taskNum = 0;
  sendModuleEvent(&moduleEventCntrl);

  /*-------------------------------------------------------------
   * Interruption of the incompatible activities
   */
  for (i = 0; i < MAX_ACTIVITIES; i++) 
    if (i != activity && ACTIVITY_STATUS(i) != ETHER 
	&& ! compatibilityTab[ACTIVITY_RQST_TYPE(i)]) 
      
      /* Record remaining incompatible activities */
      if (! icuAbortActivity (servId, i)) {
	ACTIVITY_TAB_INCOMP(activity)[ACTIVITY_NB_INCOMP(activity)] = i;
	ACTIVITY_NB_INCOMP(activity)++;
      }  
}



/*****************************************************************************
 *
 *  icuCntrlFacerecUnloadEverybody - Execution request
 */

#if (1 /* control func */)
  extern int icuFacerecUnloadEverybodyCntrl ();
#endif 

static void icuCntrlFacerecUnloadEverybody (SERV_ID servId, int rqstId)

{
  static int icuFacerecUnloadEverybodyCompatibility[] = { 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 0, 1, 1, 1, 1, 1, 1, 1 };
  int activity;
  int i;
  int *compatibilityTab = icuFacerecUnloadEverybodyCompatibility;
#if ((0) && 1 /* Inputs + control func */)
  
#endif

  /*--------------------------------------------------------------
   * Control init done and exec tasks status 
   */
#if (1 /* not init rqst */)
  if (!controlExecTaskStatusAndInitRqst(servId, rqstId))
    return;  
#endif
 
  /*--------------------------------------------------------------
   * Activity allocation
   */
  activity = allocActivity(rqstId, 26, 0, 0, servId);
  if (activity == -1) return;

  /*--------------------------------------------------------------
   * Call control func
   */
#if (1 /* control func */)
  {
    STATUS status;
    int bilan;

#if ((0)  /* Input */)
    /* Get inputs */
    if (csServRqstParamsGet (servId, rqstId, (void *) NULL, 
			     0, (FUNCPTR) NULL) != OK)
      icuReplyAndSuspend (servId, rqstId, TRUE);
    status = icuFacerecUnloadEverybodyCntrl (NULL, &bilan);
#else
    status = icuFacerecUnloadEverybodyCntrl (&bilan);
#endif

    /* Control not OK: stop there */
    if (status != OK) {
      freeActivity(activity);
      if (bilan == OK)
	bilan = S_icu_stdGenoM_CONTROL_CODEL_ERROR;
      if (csServReplySend (servId, rqstId, FINAL_REPLY, bilan, 
			   (void *) NULL, 0, (FUNCPTR) NULL) != OK)
	icuCntrlTaskSuspend (TRUE);
      return;
    }
  }
#endif

 /*-------------------------------------------------------------
  * Record inputs
  *       XXXXXXXX NON C'EST TROP TOT -> INTERROMPRE LES ACTIVITES AVANT !
  */
#if ((0) /* Input */)

  /* Get inputs address */
#if (0 /* multiple request  */)
  ACTIVITY_INPUT_ID(activity) = &(icuDataStrId->FacerecUnloadEverybodyInput)[activity];
#else
  ACTIVITY_INPUT_ID(activity) = NULL;
#endif                
  ACTIVITY_INPUT_SIZE(activity) = 0;

  /* Record inputs */
#if (1 /* control func */)
  memcpy ((void *) ACTIVITY_INPUT_ID(activity), (void *) NULL, 
	  ACTIVITY_INPUT_SIZE(activity));
#else
  if (csServRqstParamsGet (servId, rqstId, ACTIVITY_INPUT_ID(activity), 
			   0, (FUNCPTR) NULL) != OK)
    icuReplyAndSuspend (servId, rqstId, TRUE);
#endif
#endif          
  
 /*-------------------------------------------------------------
  * Get outputs address
  */
#if ((0) /* Output */)
#if (0 /* Multiple request */)
  ACTIVITY_OUTPUT_ID(activity) = 
    &(icuDataStrId->FacerecUnloadEverybodyOutput)[activity];
#else
  ACTIVITY_OUTPUT_ID(activity) = NULL;
#endif            
  ACTIVITY_OUTPUT_SIZE(activity) = 0;
#endif          

  moduleEventCntrl.eventType = STATE_START_EVENT;
  moduleEventCntrl.activityNum = activity;
  moduleEventCntrl.activityState = INIT;
  moduleEventCntrl.rqstType = 26;
  moduleEventCntrl.taskNum = 0;
  sendModuleEvent(&moduleEventCntrl);

  /*-------------------------------------------------------------
   * Interruption of the incompatible activities
   */
  for (i = 0; i < MAX_ACTIVITIES; i++) 
    if (i != activity && ACTIVITY_STATUS(i) != ETHER 
	&& ! compatibilityTab[ACTIVITY_RQST_TYPE(i)]) 
      
      /* Record remaining incompatible activities */
      if (! icuAbortActivity (servId, i)) {
	ACTIVITY_TAB_INCOMP(activity)[ACTIVITY_NB_INCOMP(activity)] = i;
	ACTIVITY_NB_INCOMP(activity)++;
      }  
}



/*****************************************************************************
 *
 *  icuCntrlWaitForModalityChange - Execution request
 */

#if (1 /* control func */)
  extern int icuWaitForModalityChangeCntrl ();
#endif 

static void icuCntrlWaitForModalityChange (SERV_ID servId, int rqstId)

{
  static int icuWaitForModalityChangeCompatibility[] = { 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 0, 1, 1, 1, 1, 1, 1 };
  int activity;
  int i;
  int *compatibilityTab = icuWaitForModalityChangeCompatibility;
#if ((0) && 1 /* Inputs + control func */)
  
#endif

  /*--------------------------------------------------------------
   * Control init done and exec tasks status 
   */
#if (1 /* not init rqst */)
  if (!controlExecTaskStatusAndInitRqst(servId, rqstId))
    return;  
#endif
 
  /*--------------------------------------------------------------
   * Activity allocation
   */
  activity = allocActivity(rqstId, 27, 0, 0, servId);
  if (activity == -1) return;

  /*--------------------------------------------------------------
   * Call control func
   */
#if (1 /* control func */)
  {
    STATUS status;
    int bilan;

#if ((0)  /* Input */)
    /* Get inputs */
    if (csServRqstParamsGet (servId, rqstId, (void *) NULL, 
			     0, (FUNCPTR) NULL) != OK)
      icuReplyAndSuspend (servId, rqstId, TRUE);
    status = icuWaitForModalityChangeCntrl (NULL, &bilan);
#else
    status = icuWaitForModalityChangeCntrl (&bilan);
#endif

    /* Control not OK: stop there */
    if (status != OK) {
      freeActivity(activity);
      if (bilan == OK)
	bilan = S_icu_stdGenoM_CONTROL_CODEL_ERROR;
      if (csServReplySend (servId, rqstId, FINAL_REPLY, bilan, 
			   (void *) NULL, 0, (FUNCPTR) NULL) != OK)
	icuCntrlTaskSuspend (TRUE);
      return;
    }
  }
#endif

 /*-------------------------------------------------------------
  * Record inputs
  *       XXXXXXXX NON C'EST TROP TOT -> INTERROMPRE LES ACTIVITES AVANT !
  */
#if ((0) /* Input */)

  /* Get inputs address */
#if (0 /* multiple request  */)
  ACTIVITY_INPUT_ID(activity) = &(icuDataStrId->WaitForModalityChangeInput)[activity];
#else
  ACTIVITY_INPUT_ID(activity) = NULL;
#endif                
  ACTIVITY_INPUT_SIZE(activity) = 0;

  /* Record inputs */
#if (1 /* control func */)
  memcpy ((void *) ACTIVITY_INPUT_ID(activity), (void *) NULL, 
	  ACTIVITY_INPUT_SIZE(activity));
#else
  if (csServRqstParamsGet (servId, rqstId, ACTIVITY_INPUT_ID(activity), 
			   0, (FUNCPTR) NULL) != OK)
    icuReplyAndSuspend (servId, rqstId, TRUE);
#endif
#endif          
  
 /*-------------------------------------------------------------
  * Get outputs address
  */
#if ((0) /* Output */)
#if (0 /* Multiple request */)
  ACTIVITY_OUTPUT_ID(activity) = 
    &(icuDataStrId->WaitForModalityChangeOutput)[activity];
#else
  ACTIVITY_OUTPUT_ID(activity) = NULL;
#endif            
  ACTIVITY_OUTPUT_SIZE(activity) = 0;
#endif          

  moduleEventCntrl.eventType = STATE_START_EVENT;
  moduleEventCntrl.activityNum = activity;
  moduleEventCntrl.activityState = INIT;
  moduleEventCntrl.rqstType = 27;
  moduleEventCntrl.taskNum = 0;
  sendModuleEvent(&moduleEventCntrl);

  /*-------------------------------------------------------------
   * Interruption of the incompatible activities
   */
  for (i = 0; i < MAX_ACTIVITIES; i++) 
    if (i != activity && ACTIVITY_STATUS(i) != ETHER 
	&& ! compatibilityTab[ACTIVITY_RQST_TYPE(i)]) 
      
      /* Record remaining incompatible activities */
      if (! icuAbortActivity (servId, i)) {
	ACTIVITY_TAB_INCOMP(activity)[ACTIVITY_NB_INCOMP(activity)] = i;
	ACTIVITY_NB_INCOMP(activity)++;
      }  
}



/*****************************************************************************
 *
 *  icuCntrlLaunchPlatine - Execution request
 */

#if (1 /* control func */)
  extern int icuLaunchPlatineCntrl ();
#endif 

static void icuCntrlLaunchPlatine (SERV_ID servId, int rqstId)

{
  static int icuLaunchPlatineCompatibility[] = { 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 0, 1, 1, 1, 1, 1 };
  int activity;
  int i;
  int *compatibilityTab = icuLaunchPlatineCompatibility;
#if ((1) && 1 /* Inputs + control func */)
  ICU_PLATINE_PARAMS platineParams;
#endif

  /*--------------------------------------------------------------
   * Control init done and exec tasks status 
   */
#if (1 /* not init rqst */)
  if (!controlExecTaskStatusAndInitRqst(servId, rqstId))
    return;  
#endif
 
  /*--------------------------------------------------------------
   * Activity allocation
   */
  activity = allocActivity(rqstId, 28, 0, 0, servId);
  if (activity == -1) return;

  /*--------------------------------------------------------------
   * Call control func
   */
#if (1 /* control func */)
  {
    STATUS status;
    int bilan;

#if ((1)  /* Input */)
    /* Get inputs */
    if (csServRqstParamsGet (servId, rqstId, (void *) &platineParams, 
			     sizeof((*icuDataStrId).platineParams), (FUNCPTR) NULL) != OK)
      icuReplyAndSuspend (servId, rqstId, TRUE);
    status = icuLaunchPlatineCntrl (&platineParams, &bilan);
#else
    status = icuLaunchPlatineCntrl (&bilan);
#endif

    /* Control not OK: stop there */
    if (status != OK) {
      freeActivity(activity);
      if (bilan == OK)
	bilan = S_icu_stdGenoM_CONTROL_CODEL_ERROR;
      if (csServReplySend (servId, rqstId, FINAL_REPLY, bilan, 
			   (void *) NULL, 0, (FUNCPTR) NULL) != OK)
	icuCntrlTaskSuspend (TRUE);
      return;
    }
  }
#endif

 /*-------------------------------------------------------------
  * Record inputs
  *       XXXXXXXX NON C'EST TROP TOT -> INTERROMPRE LES ACTIVITES AVANT !
  */
#if ((1) /* Input */)

  /* Get inputs address */
#if (0 /* multiple request  */)
  ACTIVITY_INPUT_ID(activity) = &(icuDataStrId->LaunchPlatineInput)[activity];
#else
  ACTIVITY_INPUT_ID(activity) = &((*icuDataStrId).platineParams);
#endif                
  ACTIVITY_INPUT_SIZE(activity) = sizeof((*icuDataStrId).platineParams);

  /* Record inputs */
#if (1 /* control func */)
  memcpy ((void *) ACTIVITY_INPUT_ID(activity), (void *) &platineParams, 
	  ACTIVITY_INPUT_SIZE(activity));
#else
  if (csServRqstParamsGet (servId, rqstId, ACTIVITY_INPUT_ID(activity), 
			   sizeof((*icuDataStrId).platineParams), (FUNCPTR) NULL) != OK)
    icuReplyAndSuspend (servId, rqstId, TRUE);
#endif
#endif          
  
 /*-------------------------------------------------------------
  * Get outputs address
  */
#if ((0) /* Output */)
#if (0 /* Multiple request */)
  ACTIVITY_OUTPUT_ID(activity) = 
    &(icuDataStrId->LaunchPlatineOutput)[activity];
#else
  ACTIVITY_OUTPUT_ID(activity) = NULL;
#endif            
  ACTIVITY_OUTPUT_SIZE(activity) = 0;
#endif          

  moduleEventCntrl.eventType = STATE_START_EVENT;
  moduleEventCntrl.activityNum = activity;
  moduleEventCntrl.activityState = INIT;
  moduleEventCntrl.rqstType = 28;
  moduleEventCntrl.taskNum = 0;
  sendModuleEvent(&moduleEventCntrl);

  /*-------------------------------------------------------------
   * Interruption of the incompatible activities
   */
  for (i = 0; i < MAX_ACTIVITIES; i++) 
    if (i != activity && ACTIVITY_STATUS(i) != ETHER 
	&& ! compatibilityTab[ACTIVITY_RQST_TYPE(i)]) 
      
      /* Record remaining incompatible activities */
      if (! icuAbortActivity (servId, i)) {
	ACTIVITY_TAB_INCOMP(activity)[ACTIVITY_NB_INCOMP(activity)] = i;
	ACTIVITY_NB_INCOMP(activity)++;
      }  
}



/*****************************************************************************
 *
 *  icuCntrlCmdPtu - Execution request
 */

#if (0 /* control func */)
  extern int dummy ();
#endif 

static void icuCntrlCmdPtu (SERV_ID servId, int rqstId)

{
  static int icuCmdPtuCompatibility[] = { 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 0, 1, 1, 1, 1 };
  int activity;
  int i;
  int *compatibilityTab = icuCmdPtuCompatibility;
#if ((0) && 0 /* Inputs + control func */)
  
#endif

  /*--------------------------------------------------------------
   * Control init done and exec tasks status 
   */
#if (1 /* not init rqst */)
  if (!controlExecTaskStatusAndInitRqst(servId, rqstId))
    return;  
#endif
 
  /*--------------------------------------------------------------
   * Activity allocation
   */
  activity = allocActivity(rqstId, 29, 0, 0, servId);
  if (activity == -1) return;

  /*--------------------------------------------------------------
   * Call control func
   */
#if (0 /* control func */)
  {
    STATUS status;
    int bilan;

#if ((0)  /* Input */)
    /* Get inputs */
    if (csServRqstParamsGet (servId, rqstId, (void *) NULL, 
			     0, (FUNCPTR) NULL) != OK)
      icuReplyAndSuspend (servId, rqstId, TRUE);
    status = dummy (NULL, &bilan);
#else
    status = dummy (&bilan);
#endif

    /* Control not OK: stop there */
    if (status != OK) {
      freeActivity(activity);
      if (bilan == OK)
	bilan = S_icu_stdGenoM_CONTROL_CODEL_ERROR;
      if (csServReplySend (servId, rqstId, FINAL_REPLY, bilan, 
			   (void *) NULL, 0, (FUNCPTR) NULL) != OK)
	icuCntrlTaskSuspend (TRUE);
      return;
    }
  }
#endif

 /*-------------------------------------------------------------
  * Record inputs
  *       XXXXXXXX NON C'EST TROP TOT -> INTERROMPRE LES ACTIVITES AVANT !
  */
#if ((0) /* Input */)

  /* Get inputs address */
#if (0 /* multiple request  */)
  ACTIVITY_INPUT_ID(activity) = &(icuDataStrId->CmdPtuInput)[activity];
#else
  ACTIVITY_INPUT_ID(activity) = NULL;
#endif                
  ACTIVITY_INPUT_SIZE(activity) = 0;

  /* Record inputs */
#if (0 /* control func */)
  memcpy ((void *) ACTIVITY_INPUT_ID(activity), (void *) NULL, 
	  ACTIVITY_INPUT_SIZE(activity));
#else
  if (csServRqstParamsGet (servId, rqstId, ACTIVITY_INPUT_ID(activity), 
			   0, (FUNCPTR) NULL) != OK)
    icuReplyAndSuspend (servId, rqstId, TRUE);
#endif
#endif          
  
 /*-------------------------------------------------------------
  * Get outputs address
  */
#if ((0) /* Output */)
#if (0 /* Multiple request */)
  ACTIVITY_OUTPUT_ID(activity) = 
    &(icuDataStrId->CmdPtuOutput)[activity];
#else
  ACTIVITY_OUTPUT_ID(activity) = NULL;
#endif            
  ACTIVITY_OUTPUT_SIZE(activity) = 0;
#endif          

  moduleEventCntrl.eventType = STATE_START_EVENT;
  moduleEventCntrl.activityNum = activity;
  moduleEventCntrl.activityState = INIT;
  moduleEventCntrl.rqstType = 29;
  moduleEventCntrl.taskNum = 0;
  sendModuleEvent(&moduleEventCntrl);

  /*-------------------------------------------------------------
   * Interruption of the incompatible activities
   */
  for (i = 0; i < MAX_ACTIVITIES; i++) 
    if (i != activity && ACTIVITY_STATUS(i) != ETHER 
	&& ! compatibilityTab[ACTIVITY_RQST_TYPE(i)]) 
      
      /* Record remaining incompatible activities */
      if (! icuAbortActivity (servId, i)) {
	ACTIVITY_TAB_INCOMP(activity)[ACTIVITY_NB_INCOMP(activity)] = i;
	ACTIVITY_NB_INCOMP(activity)++;
      }  
}



/*****************************************************************************
 *
 *  icuCntrlIcuInitRFID - Execution request
 */

#if (1 /* control func */)
  extern int icuInitRFIDCntrl ();
#endif 

static void icuCntrlIcuInitRFID (SERV_ID servId, int rqstId)

{
  static int icuIcuInitRFIDCompatibility[] = { 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 0, 1, 1, 1 };
  int activity;
  int i;
  int *compatibilityTab = icuIcuInitRFIDCompatibility;
#if ((0) && 1 /* Inputs + control func */)
  
#endif

  /*--------------------------------------------------------------
   * Control init done and exec tasks status 
   */
#if (1 /* not init rqst */)
  if (!controlExecTaskStatusAndInitRqst(servId, rqstId))
    return;  
#endif
 
  /*--------------------------------------------------------------
   * Activity allocation
   */
  activity = allocActivity(rqstId, 30, 0, 0, servId);
  if (activity == -1) return;

  /*--------------------------------------------------------------
   * Call control func
   */
#if (1 /* control func */)
  {
    STATUS status;
    int bilan;

#if ((0)  /* Input */)
    /* Get inputs */
    if (csServRqstParamsGet (servId, rqstId, (void *) NULL, 
			     0, (FUNCPTR) NULL) != OK)
      icuReplyAndSuspend (servId, rqstId, TRUE);
    status = icuInitRFIDCntrl (NULL, &bilan);
#else
    status = icuInitRFIDCntrl (&bilan);
#endif

    /* Control not OK: stop there */
    if (status != OK) {
      freeActivity(activity);
      if (bilan == OK)
	bilan = S_icu_stdGenoM_CONTROL_CODEL_ERROR;
      if (csServReplySend (servId, rqstId, FINAL_REPLY, bilan, 
			   (void *) NULL, 0, (FUNCPTR) NULL) != OK)
	icuCntrlTaskSuspend (TRUE);
      return;
    }
  }
#endif

 /*-------------------------------------------------------------
  * Record inputs
  *       XXXXXXXX NON C'EST TROP TOT -> INTERROMPRE LES ACTIVITES AVANT !
  */
#if ((0) /* Input */)

  /* Get inputs address */
#if (0 /* multiple request  */)
  ACTIVITY_INPUT_ID(activity) = &(icuDataStrId->IcuInitRFIDInput)[activity];
#else
  ACTIVITY_INPUT_ID(activity) = NULL;
#endif                
  ACTIVITY_INPUT_SIZE(activity) = 0;

  /* Record inputs */
#if (1 /* control func */)
  memcpy ((void *) ACTIVITY_INPUT_ID(activity), (void *) NULL, 
	  ACTIVITY_INPUT_SIZE(activity));
#else
  if (csServRqstParamsGet (servId, rqstId, ACTIVITY_INPUT_ID(activity), 
			   0, (FUNCPTR) NULL) != OK)
    icuReplyAndSuspend (servId, rqstId, TRUE);
#endif
#endif          
  
 /*-------------------------------------------------------------
  * Get outputs address
  */
#if ((0) /* Output */)
#if (0 /* Multiple request */)
  ACTIVITY_OUTPUT_ID(activity) = 
    &(icuDataStrId->IcuInitRFIDOutput)[activity];
#else
  ACTIVITY_OUTPUT_ID(activity) = NULL;
#endif            
  ACTIVITY_OUTPUT_SIZE(activity) = 0;
#endif          

  moduleEventCntrl.eventType = STATE_START_EVENT;
  moduleEventCntrl.activityNum = activity;
  moduleEventCntrl.activityState = INIT;
  moduleEventCntrl.rqstType = 30;
  moduleEventCntrl.taskNum = 0;
  sendModuleEvent(&moduleEventCntrl);

  /*-------------------------------------------------------------
   * Interruption of the incompatible activities
   */
  for (i = 0; i < MAX_ACTIVITIES; i++) 
    if (i != activity && ACTIVITY_STATUS(i) != ETHER 
	&& ! compatibilityTab[ACTIVITY_RQST_TYPE(i)]) 
      
      /* Record remaining incompatible activities */
      if (! icuAbortActivity (servId, i)) {
	ACTIVITY_TAB_INCOMP(activity)[ACTIVITY_NB_INCOMP(activity)] = i;
	ACTIVITY_NB_INCOMP(activity)++;
      }  
}



/*****************************************************************************
 *
 *  icuCntrlIcuCloseRFID - Execution request
 */

#if (1 /* control func */)
  extern int icuCloseRFIDCntrl ();
#endif 

static void icuCntrlIcuCloseRFID (SERV_ID servId, int rqstId)

{
  static int icuIcuCloseRFIDCompatibility[] = { 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 0, 0, 0 };
  int activity;
  int i;
  int *compatibilityTab = icuIcuCloseRFIDCompatibility;
#if ((0) && 1 /* Inputs + control func */)
  
#endif

  /*--------------------------------------------------------------
   * Control init done and exec tasks status 
   */
#if (1 /* not init rqst */)
  if (!controlExecTaskStatusAndInitRqst(servId, rqstId))
    return;  
#endif
 
  /*--------------------------------------------------------------
   * Activity allocation
   */
  activity = allocActivity(rqstId, 31, 0, 0, servId);
  if (activity == -1) return;

  /*--------------------------------------------------------------
   * Call control func
   */
#if (1 /* control func */)
  {
    STATUS status;
    int bilan;

#if ((0)  /* Input */)
    /* Get inputs */
    if (csServRqstParamsGet (servId, rqstId, (void *) NULL, 
			     0, (FUNCPTR) NULL) != OK)
      icuReplyAndSuspend (servId, rqstId, TRUE);
    status = icuCloseRFIDCntrl (NULL, &bilan);
#else
    status = icuCloseRFIDCntrl (&bilan);
#endif

    /* Control not OK: stop there */
    if (status != OK) {
      freeActivity(activity);
      if (bilan == OK)
	bilan = S_icu_stdGenoM_CONTROL_CODEL_ERROR;
      if (csServReplySend (servId, rqstId, FINAL_REPLY, bilan, 
			   (void *) NULL, 0, (FUNCPTR) NULL) != OK)
	icuCntrlTaskSuspend (TRUE);
      return;
    }
  }
#endif

 /*-------------------------------------------------------------
  * Record inputs
  *       XXXXXXXX NON C'EST TROP TOT -> INTERROMPRE LES ACTIVITES AVANT !
  */
#if ((0) /* Input */)

  /* Get inputs address */
#if (0 /* multiple request  */)
  ACTIVITY_INPUT_ID(activity) = &(icuDataStrId->IcuCloseRFIDInput)[activity];
#else
  ACTIVITY_INPUT_ID(activity) = NULL;
#endif                
  ACTIVITY_INPUT_SIZE(activity) = 0;

  /* Record inputs */
#if (1 /* control func */)
  memcpy ((void *) ACTIVITY_INPUT_ID(activity), (void *) NULL, 
	  ACTIVITY_INPUT_SIZE(activity));
#else
  if (csServRqstParamsGet (servId, rqstId, ACTIVITY_INPUT_ID(activity), 
			   0, (FUNCPTR) NULL) != OK)
    icuReplyAndSuspend (servId, rqstId, TRUE);
#endif
#endif          
  
 /*-------------------------------------------------------------
  * Get outputs address
  */
#if ((0) /* Output */)
#if (0 /* Multiple request */)
  ACTIVITY_OUTPUT_ID(activity) = 
    &(icuDataStrId->IcuCloseRFIDOutput)[activity];
#else
  ACTIVITY_OUTPUT_ID(activity) = NULL;
#endif            
  ACTIVITY_OUTPUT_SIZE(activity) = 0;
#endif          

  moduleEventCntrl.eventType = STATE_START_EVENT;
  moduleEventCntrl.activityNum = activity;
  moduleEventCntrl.activityState = INIT;
  moduleEventCntrl.rqstType = 31;
  moduleEventCntrl.taskNum = 0;
  sendModuleEvent(&moduleEventCntrl);

  /*-------------------------------------------------------------
   * Interruption of the incompatible activities
   */
  for (i = 0; i < MAX_ACTIVITIES; i++) 
    if (i != activity && ACTIVITY_STATUS(i) != ETHER 
	&& ! compatibilityTab[ACTIVITY_RQST_TYPE(i)]) 
      
      /* Record remaining incompatible activities */
      if (! icuAbortActivity (servId, i)) {
	ACTIVITY_TAB_INCOMP(activity)[ACTIVITY_NB_INCOMP(activity)] = i;
	ACTIVITY_NB_INCOMP(activity)++;
      }  
}



/*****************************************************************************
 *
 *  icuCntrlIcuAcquireRFID - Execution request
 */

#if (1 /* control func */)
  extern int icuAcquireRFIDCntrl ();
#endif 

static void icuCntrlIcuAcquireRFID (SERV_ID servId, int rqstId)

{
  static int icuIcuAcquireRFIDCompatibility[] = { 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 0, 0 };
  int activity;
  int i;
  int *compatibilityTab = icuIcuAcquireRFIDCompatibility;
#if ((0) && 1 /* Inputs + control func */)
  
#endif

  /*--------------------------------------------------------------
   * Control init done and exec tasks status 
   */
#if (1 /* not init rqst */)
  if (!controlExecTaskStatusAndInitRqst(servId, rqstId))
    return;  
#endif
 
  /*--------------------------------------------------------------
   * Activity allocation
   */
  activity = allocActivity(rqstId, 32, 0, 0, servId);
  if (activity == -1) return;

  /*--------------------------------------------------------------
   * Call control func
   */
#if (1 /* control func */)
  {
    STATUS status;
    int bilan;

#if ((0)  /* Input */)
    /* Get inputs */
    if (csServRqstParamsGet (servId, rqstId, (void *) NULL, 
			     0, (FUNCPTR) NULL) != OK)
      icuReplyAndSuspend (servId, rqstId, TRUE);
    status = icuAcquireRFIDCntrl (NULL, &bilan);
#else
    status = icuAcquireRFIDCntrl (&bilan);
#endif

    /* Control not OK: stop there */
    if (status != OK) {
      freeActivity(activity);
      if (bilan == OK)
	bilan = S_icu_stdGenoM_CONTROL_CODEL_ERROR;
      if (csServReplySend (servId, rqstId, FINAL_REPLY, bilan, 
			   (void *) NULL, 0, (FUNCPTR) NULL) != OK)
	icuCntrlTaskSuspend (TRUE);
      return;
    }
  }
#endif

 /*-------------------------------------------------------------
  * Record inputs
  *       XXXXXXXX NON C'EST TROP TOT -> INTERROMPRE LES ACTIVITES AVANT !
  */
#if ((0) /* Input */)

  /* Get inputs address */
#if (0 /* multiple request  */)
  ACTIVITY_INPUT_ID(activity) = &(icuDataStrId->IcuAcquireRFIDInput)[activity];
#else
  ACTIVITY_INPUT_ID(activity) = NULL;
#endif                
  ACTIVITY_INPUT_SIZE(activity) = 0;

  /* Record inputs */
#if (1 /* control func */)
  memcpy ((void *) ACTIVITY_INPUT_ID(activity), (void *) NULL, 
	  ACTIVITY_INPUT_SIZE(activity));
#else
  if (csServRqstParamsGet (servId, rqstId, ACTIVITY_INPUT_ID(activity), 
			   0, (FUNCPTR) NULL) != OK)
    icuReplyAndSuspend (servId, rqstId, TRUE);
#endif
#endif          
  
 /*-------------------------------------------------------------
  * Get outputs address
  */
#if ((0) /* Output */)
#if (0 /* Multiple request */)
  ACTIVITY_OUTPUT_ID(activity) = 
    &(icuDataStrId->IcuAcquireRFIDOutput)[activity];
#else
  ACTIVITY_OUTPUT_ID(activity) = NULL;
#endif            
  ACTIVITY_OUTPUT_SIZE(activity) = 0;
#endif          

  moduleEventCntrl.eventType = STATE_START_EVENT;
  moduleEventCntrl.activityNum = activity;
  moduleEventCntrl.activityState = INIT;
  moduleEventCntrl.rqstType = 32;
  moduleEventCntrl.taskNum = 0;
  sendModuleEvent(&moduleEventCntrl);

  /*-------------------------------------------------------------
   * Interruption of the incompatible activities
   */
  for (i = 0; i < MAX_ACTIVITIES; i++) 
    if (i != activity && ACTIVITY_STATUS(i) != ETHER 
	&& ! compatibilityTab[ACTIVITY_RQST_TYPE(i)]) 
      
      /* Record remaining incompatible activities */
      if (! icuAbortActivity (servId, i)) {
	ACTIVITY_TAB_INCOMP(activity)[ACTIVITY_NB_INCOMP(activity)] = i;
	ACTIVITY_NB_INCOMP(activity)++;
      }  
}



/*****************************************************************************
 *
 *  icuCntrlIcuAcquireTag - Execution request
 */

#if (1 /* control func */)
  extern int icuAcquireTagCntrl ();
#endif 

static void icuCntrlIcuAcquireTag (SERV_ID servId, int rqstId)

{
  static int icuIcuAcquireTagCompatibility[] = { 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 0, 0 };
  int activity;
  int i;
  int *compatibilityTab = icuIcuAcquireTagCompatibility;
#if ((1) && 1 /* Inputs + control func */)
  GEN_STRING256 recoTag;
#endif

  /*--------------------------------------------------------------
   * Control init done and exec tasks status 
   */
#if (1 /* not init rqst */)
  if (!controlExecTaskStatusAndInitRqst(servId, rqstId))
    return;  
#endif
 
  /*--------------------------------------------------------------
   * Activity allocation
   */
  activity = allocActivity(rqstId, 33, 0, 0, servId);
  if (activity == -1) return;

  /*--------------------------------------------------------------
   * Call control func
   */
#if (1 /* control func */)
  {
    STATUS status;
    int bilan;

#if ((1)  /* Input */)
    /* Get inputs */
    if (csServRqstParamsGet (servId, rqstId, (void *) &recoTag, 
			     sizeof((*icuDataStrId).recoTag), (FUNCPTR) NULL) != OK)
      icuReplyAndSuspend (servId, rqstId, TRUE);
    status = icuAcquireTagCntrl (&recoTag, &bilan);
#else
    status = icuAcquireTagCntrl (&bilan);
#endif

    /* Control not OK: stop there */
    if (status != OK) {
      freeActivity(activity);
      if (bilan == OK)
	bilan = S_icu_stdGenoM_CONTROL_CODEL_ERROR;
      if (csServReplySend (servId, rqstId, FINAL_REPLY, bilan, 
			   (void *) NULL, 0, (FUNCPTR) NULL) != OK)
	icuCntrlTaskSuspend (TRUE);
      return;
    }
  }
#endif

 /*-------------------------------------------------------------
  * Record inputs
  *       XXXXXXXX NON C'EST TROP TOT -> INTERROMPRE LES ACTIVITES AVANT !
  */
#if ((1) /* Input */)

  /* Get inputs address */
#if (0 /* multiple request  */)
  ACTIVITY_INPUT_ID(activity) = &(icuDataStrId->IcuAcquireTagInput)[activity];
#else
  ACTIVITY_INPUT_ID(activity) = &((*icuDataStrId).recoTag);
#endif                
  ACTIVITY_INPUT_SIZE(activity) = sizeof((*icuDataStrId).recoTag);

  /* Record inputs */
#if (1 /* control func */)
  memcpy ((void *) ACTIVITY_INPUT_ID(activity), (void *) &recoTag, 
	  ACTIVITY_INPUT_SIZE(activity));
#else
  if (csServRqstParamsGet (servId, rqstId, ACTIVITY_INPUT_ID(activity), 
			   sizeof((*icuDataStrId).recoTag), (FUNCPTR) NULL) != OK)
    icuReplyAndSuspend (servId, rqstId, TRUE);
#endif
#endif          
  
 /*-------------------------------------------------------------
  * Get outputs address
  */
#if ((0) /* Output */)
#if (0 /* Multiple request */)
  ACTIVITY_OUTPUT_ID(activity) = 
    &(icuDataStrId->IcuAcquireTagOutput)[activity];
#else
  ACTIVITY_OUTPUT_ID(activity) = NULL;
#endif            
  ACTIVITY_OUTPUT_SIZE(activity) = 0;
#endif          

  moduleEventCntrl.eventType = STATE_START_EVENT;
  moduleEventCntrl.activityNum = activity;
  moduleEventCntrl.activityState = INIT;
  moduleEventCntrl.rqstType = 33;
  moduleEventCntrl.taskNum = 0;
  sendModuleEvent(&moduleEventCntrl);

  /*-------------------------------------------------------------
   * Interruption of the incompatible activities
   */
  for (i = 0; i < MAX_ACTIVITIES; i++) 
    if (i != activity && ACTIVITY_STATUS(i) != ETHER 
	&& ! compatibilityTab[ACTIVITY_RQST_TYPE(i)]) 
      
      /* Record remaining incompatible activities */
      if (! icuAbortActivity (servId, i)) {
	ACTIVITY_TAB_INCOMP(activity)[ACTIVITY_NB_INCOMP(activity)] = i;
	ACTIVITY_NB_INCOMP(activity)++;
      }  
}

