
/* --- FILE GENERATED WITH GENOM, DO NOT EDIT BY HAND ------------------ */

/* 
 * Copyright (c) 2004-2005
 *      Autonomous Systems Lab, Swiss Federal Institute of Technology.
 * Copyright (c) 1993-2005 LAAS/CNRS
 * All rights reserved.
 *
 * Redistribution and use  in source  and binary  forms,  with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *   1. Redistributions of  source  code must retain the  above copyright
 *      notice, this list of conditions and the following disclaimer.
 *   2. Redistributions in binary form must reproduce the above copyright
 *      notice,  this list of  conditions and the following disclaimer in
 *      the  documentation  and/or  other   materials provided  with  the
 *      distribution.
 *
 * THIS  SOFTWARE IS PROVIDED BY  THE  COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND  ANY  EXPRESS OR IMPLIED  WARRANTIES,  INCLUDING,  BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES  OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR  PURPOSE ARE DISCLAIMED. IN  NO EVENT SHALL THE COPYRIGHT
 * HOLDERS OR      CONTRIBUTORS  BE LIABLE FOR   ANY    DIRECT, INDIRECT,
 * INCIDENTAL,  SPECIAL,  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF  SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN  CONTRACT, STRICT LIABILITY, OR
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE   OF THIS SOFTWARE, EVEN   IF ADVISED OF   THE POSSIBILITY OF SUCH
 * DAMAGE.
 */

/*
 * Execution task body
 */

#if defined(__RTAI__) && defined(__KERNEL__)
# include <linux/time.h>
# define gettimeofday(x,y)	do_gettimeofday(x)
#else
# include <sys/time.h>
# include <stdio.h>
# include <stdlib.h>
# include <string.h>
#endif /* not RTAI && KERNEL */

#include <taskLib.h>
#include <errnoLib.h>

#include <commonStructLib.h>
#include <h2evnLib.h>
#include <h2timeLib.h>
#include <h2timerLib.h>

#include <genom/modules.h>
#include <genom/moduleEvents.h>

#include "icuHeader.h"


/* Print debugging information */
#define GENOM_DEBUG_EXECTASK

#ifdef GENOM_DEBUG_EXECTASK
# define LOGDBG(x)	logMsg x
#else
# define LOGDBG(x)
#endif

/* Size of the mailbox receiving replies from servers */
#define ICU_RUNTASK_MBOX_REPLY_SIZE 


/*--------------------------- EXTERNAL VARIABLES -------------------------*/
/* WARNING: use unique names! */

/* Initialization semaphore */
extern SEM_ID semicuInitExecTab[];

/*---------------- PROTOTYPES OF INTERNAL FUNCTIONS ----------------------*/

/* Execution task initialization */
static STATUS icuRunTaskInitTaskFunc (H2TIMER_ID *execTimerId);

/* Suspend the the execution task */
static void icuRunTaskSuspend (BOOL giveFlag);

/* Call of user functions */
static ACTIVITY_EVENT execTaskCallUserFunc (ACTIVITY_STATE state, 
					    int activityId);

/* Signal a transition after handling it */
static BOOL filterAndSendEvn (ACTIVITY_STATE state,
			     ACTIVITY_EVENT evn);

/* Returns a string representing the h2 event */
char const * h2GetEvnStateString(int num);

/* Compute micro-seconds difference between 2 H2TIMESPEC */
static long h2timespec_udiff(const H2TIMESPEC *, const H2TIMESPEC *);

/*--------------------------- LOCAL VARIABLES -------------------------*/
/* WARNING: use unique names */

/*---------------- User function prototypes -------------------*/

/* Permanent activity functions */
#if (1)
STATUS icuRunTaskExec (int *bilan);
#endif
#if (0)
STATUS dummy (int *bilan);
#endif

/* Initialisation function */
#if (1)
int icuRunTaskInit ();
#endif

/*---------------------- EXPORTED PROCEDURES --------------------------------*/

/*****************************************************************************
 *
 *  icuRunTask  -  Exec task
 *
 *  Description : Execute fonctions in the module
 *
 *
 *  Returns: never
 */

void icuRunTask (void)
{
  static H2TIMER_ID icuRunTaskTimerId;
  int i, nb, nbActi;
  int prevExecTaskBilan;
  int wakeUpCntrlTask;
#if (0) /* client task */
  int extEvn;
#endif
  int periodOverShot=0;

  MODULE_EVENT_STR moduleEvent;
  H2TIMESPEC tpBegin, tpBeginPrev, tpEnd; /* time measure */

  unsigned long  meanDuration, nbIter=1;
  int firstChrono=TRUE;
  int computeMeanFlag = FALSE; /* try to compute meanDuration only if main exec loop and not abnormal overshot */

  /* Initialization of task */
  errnoSet(0);
  EXEC_TASK_STATUS(0) = 
    icuRunTaskInitTaskFunc (&icuRunTaskTimerId);
  prevExecTaskBilan = EXEC_TASK_BILAN(0) = errnoGet();
  
  /* Release the initialization semaphore */
  semGive (semicuInitExecTab[0]);
  
  /* suspend ourselves in case of problems */
  if(EXEC_TASK_STATUS(0) != OK)
    icuRunTaskSuspend (FALSE);
  moduleEvent.moduleNum = 7480;
  moduleEvent.taskNum = 0;

  tpBeginPrev.tv_sec = 0;
  tpBeginPrev.tv_nsec = 0;

  /* main loop */
  FOREVER {
#if (1) /* Periodic task */
    if (h2timerPause (icuRunTaskTimerId) != OK) {
      logMsg("icuRunTask: h2timerPause error\n");
      icuRunTaskSuspend (FALSE);
    }
#elif ($task_wait_codel_flag$)
    if ($task_wait_codel$() != OK) {
      logMsh("icuRunTask: $task_wait_codel$ error\n");
      icuRunTaskSuspend(FALSE);
    }
#else /* wait for external events */
    if (h2evnSusp(0) != TRUE) {
      printf ("icuRunTask: h2evnSusp error\n");
      icuRunTaskSuspend (FALSE);
    }
#endif

    /* Get time */
    moduleEvent.eventType = EXEC_START_EVENT;
    sendModuleEvent(&moduleEvent);

    h2GetTimeSpec(&tpBegin);

    if (firstChrono) {
	    firstChrono=FALSE; 
	    memcpy(&tpBeginPrev, &tpBegin, sizeof(H2TIMESPEC));
    }

    computeMeanFlag = 1;
#if (1) /* periodic task */
    EXEC_TASK_ON_PERIOD(0) = 
	h2timespec_udiff(&tpBegin, &tpBeginPrev);
    if (EXEC_TASK_ON_PERIOD(0) 
	> EXEC_TASK_MAX_PERIOD(0)) {
      EXEC_TASK_MAX_PERIOD(0) = EXEC_TASK_ON_PERIOD(0);
      computeMeanFlag = 0;
    }
    memcpy(&tpBeginPrev, &tpBegin, sizeof(H2TIMESPEC));

    /* Previous computation overshot */
    if (EXEC_TASK_DURATION_LOOP(0) > (1000000.0*EXEC_TASK_PERIOD(0))) {
      periodOverShot = (int) ((EXEC_TASK_DURATION_LOOP(0)
			       /(1000000.0*EXEC_TASK_PERIOD(0))));
      
      if (GENOM_PRINT_TIME_FLAG) 
	printf("icuRunTask overshot of %d periods (d=%ldus mean=%ldus p=%ldus max=%ldus th=%dus)\n",
	       periodOverShot,
	       EXEC_TASK_DURATION_LOOP(0),
	       meanDuration,
	       EXEC_TASK_ON_PERIOD(0),
	       EXEC_TASK_MAX_PERIOD(0),
	       (int)(1000000*EXEC_TASK_PERIOD(0)));
    }
    else
      periodOverShot = 0;
#endif

    wakeUpCntrlTask = FALSE;

    /* Interruption requested */
    if (STOP_MODULE_FLAG) {
      
#if (1)
      /* Execute end codel */
      icuRunTaskEnd ();
#endif
      
      /* free posters, clients, mailboxes */
        if (EXEC_TASK_POSTER_ID(0)[0] != (POSTER_ID)NULL)
    posterDelete(EXEC_TASK_POSTER_ID(0)[0]);
  if (EXEC_TASK_POSTER_ID(0)[1] != (POSTER_ID)NULL)
    posterDelete(EXEC_TASK_POSTER_ID(0)[1]);
  if (EXEC_TASK_POSTER_ID(0)[2] != (POSTER_ID)NULL)
    posterDelete(EXEC_TASK_POSTER_ID(0)[2]);
  if (EXEC_TASK_POSTER_ID(0)[3] != (POSTER_ID)NULL)
    posterDelete(EXEC_TASK_POSTER_ID(0)[3]);

#if (0) /* client task */
      
      csMboxEnd();
#else
#if (!1) /* non-periodic task */
      /* free device created to manage h2evn required to aperiodic tasks */
      mboxEnd(0);
#endif
#endif
      /* free semaphore */
      EXEC_TASK_WAKE_UP_FLAG(0) = FALSE;
      h2evnSignal(CNTRL_TASK_ID);

      return;
    }

    /* take IDS access */
      if (commonStructTake (icuCntrlStrId) != OK ||
	  commonStructTake (icuDataStrId) != OK) {
	logMsg("icuRunTask: commonStructTake error\n");
	icuRunTaskSuspend (FALSE);
      }
    
    /* Check for a received reply (XXX: see remark down) */
#if (0) /* client task */
    extEvn = FALSE;
    if (csMboxStatus(REPLY_MBOX) & REPLY_MBOX)
      extEvn = TRUE;
#endif


    /* If last period overshot, does not execute activities this time */
    /* XXXXXX NOT YET: AFTER COGNIRON, SEEMS TO HAVE SIDES EFFECTS
       if (!periodOverShot) { */
    
    /* permanent activity 1 */
#if (1) 
    moduleEvent.eventType = STATE_START_EVENT;
    moduleEvent.activityNum = -1;
    moduleEvent.activityState = EXEC;
    sendModuleEvent(&moduleEvent);    

    CURRENT_ACTIVITY_NUM(0) = -1;
    EXEC_TASK_BILAN(0) = OK;
    if (icuRunTaskExec (&EXEC_TASK_BILAN(0)) != OK) {
      logMsg("icuRunTask: permanent activity error\n");
      icuRunTaskSuspend (TRUE);
    }

    moduleEvent.eventType = STATE_END_EVENT;
    sendModuleEvent(&moduleEvent);   
#endif

    /* Look for activities */
    nbActi = EXEC_TASK_NB_ACTI(0);
    for (i = 0, nb = 0; nb < nbActi && i < MAX_ACTIVITIES; i++) 
      if (ACTIVITY_TASK_NUM(i) == 0) {
	
	nb++;

	/* Wake up sleeping activities */
	/* XXX pb: on peut louper l'info extEvn: si la lecture est effectuer par une autre activite ou par la fonction permanente et que la "reply" arrive apres le test extEvn et avant la lecture !!!
1ere solution: eveiller systematiquement les activites SLEEP ?!? 
2nd solution: la fonction qui recoit la replique execute EXEC_TASK_WAKE_UP */
	if (ACTIVITY_STATUS(i) == SLEEP && ACTIVITY_EVN(i) == NO_EVENT)
	  ACTIVITY_EVN(i) = EXEC;

	/* Execute, depending on the event */
	switch (ACTIVITY_EVN(i)) {
	  
	  /* Nothing (ZOMBIE, SLEEP activity) */
	case NO_EVENT:
	  break;
	  
	  /* Handled by the control task */
	case ETHER:
	case ZOMBIE:
	case INIT:
	case SLEEP:
	  break;
	  
	  /* This is for us */
	case START:
	  EXEC_TASK_MAX_PERIOD(0) = 0;
	case END:
	case FAIL:
	case INTER:
	  computeMeanFlag = 0;
	case EXEC:
	  moduleEvent.eventType = STATE_START_EVENT;
	  moduleEvent.activityNum = i;
	  moduleEvent.activityState = ACTIVITY_EVN(i);
	  moduleEvent.rqstType = ACTIVITY_RQST_TYPE(i);
	  sendModuleEvent(&moduleEvent);

	  /* record new state */
	  ACTIVITY_STATUS(i) = (ACTIVITY_STATE) ACTIVITY_EVN(i);
	  
	  /* Call the processing function */
	  CURRENT_ACTIVITY_NUM(0) = i;
	  ACTIVITY_EVN(i) = execTaskCallUserFunc(ACTIVITY_STATUS(i), i);
	  
	  /* Check event validity */
	  if(filterAndSendEvn(ACTIVITY_STATUS(i), ACTIVITY_EVN(i)))
	    wakeUpCntrlTask = TRUE;

	  moduleEvent.eventType = STATE_END_EVENT;
	  sendModuleEvent(&moduleEvent);
	  break;
	  
	default:
	  logMsg("icuRunTask, activity %d: incoherent evn %s\n",
		  i, h2GetEvnStateString (ACTIVITY_EVN(i)));
	  icuRunTaskSuspend (TRUE);
	}           /* switch evn */ 
      }        /* while there are activities */ 
    
    /*****XXXX Temporary test */
    if (nb != nbActi)
      logMsg("icuRunTask: invalid number of activities %d (expected %d) !\n", nb, nbActi);

/*XXXXXXXX     }  *//* period overshot test */

    /* permanent activity 2 */
#if (0) 
    moduleEvent.eventType = STATE_START_EVENT;
    moduleEvent.activityNum = -1;
    moduleEvent.activityState = EXEC;
    sendModuleEvent(&moduleEvent);    

    CURRENT_ACTIVITY_NUM(0) = -1;
    EXEC_TASK_BILAN(0) = OK;
    if (dummy (&EXEC_TASK_BILAN(0)) != OK) {
      logMsg("icuRunTask: permanent activity 2 error\n");
      icuRunTaskSuspend (TRUE);
    }

    moduleEvent.eventType = STATE_END_EVENT;
    sendModuleEvent(&moduleEvent);   
#endif

    /* no more activity */
    CURRENT_ACTIVITY_NUM(0) = -2;

    /* update "auto" posters */
      {
  ICU_RESULTS_POSTER_STR x;
  int offset, size;

  offset = (int)&(x.humanPos) - (int)&(x);
  size = sizeof(x.humanPos);
  if (posterWrite(ICU_RESULTS_POSTER_ID, offset, (char *)&(icuDataStrId->humanPos), size) != size) icuRunTaskSuspend (TRUE);
  }  {
  ICU_HUMANPRESENCE_POSTER_STR x;
  int offset, size;

  offset = (int)&(x.loadedPersonList) - (int)&(x);
  size = sizeof(x.loadedPersonList);
  if (posterWrite(ICU_HUMANPRESENCE_POSTER_ID, offset, (char *)&(icuDataStrId->loadedPersonList), size) != size) icuRunTaskSuspend (TRUE);
  }  {
  ICU_PLATINECMD_POSTER_STR x;
  int offset, size;

  offset = (int)&(x.cmdPlatine) - (int)&(x);
  size = sizeof(x.cmdPlatine);
  if (posterWrite(ICU_PLATINECMD_POSTER_ID, offset, (char *)&(icuDataStrId->cmdPlatine), size) != size) icuRunTaskSuspend (TRUE);
  }  {
  ICU_VISUALDATA_POSTER_STR x;
  int offset, size;

  offset = (int)&(x.Data) - (int)&(x);
  size = sizeof(x.Data);
  if (posterWrite(ICU_VISUALDATA_POSTER_ID, offset, (char *)&(icuDataStrId->Data), size) != size) icuRunTaskSuspend (TRUE);
  }
    
    /* Time elapsed since previous read */
    moduleEvent.eventType = EXEC_END_EVENT;
    sendModuleEvent(&moduleEvent);

    h2GetTimeSpec(&tpEnd);

#if (!1)
    EXEC_TASK_ON_PERIOD(0) = h2timespec_udiff(&tpEnd, &tpBegin);
    if (EXEC_TASK_ON_PERIOD(0) > EXEC_TASK_MAX_PERIOD(0)) {
      EXEC_TASK_MAX_PERIOD(0) = EXEC_TASK_ON_PERIOD(0);
      computeMeanFlag = 0;
    }
#endif

    EXEC_TASK_DURATION_LOOP(0) = 
	h2timespec_udiff(&tpEnd, &tpBegin);
    if (computeMeanFlag) {
      meanDuration = (int)
	(((double)((nbIter-1)*meanDuration + EXEC_TASK_DURATION_LOOP(0))
	  /((double)nbIter)));
      nbIter++;
    }

    /* display about time */
    if (GENOM_PRINT_TIME_FLAG) {
      printf("icuRunTask: d %4ld mean %4ld p %4ld max %4ld th %4d\n",
	     EXEC_TASK_DURATION_LOOP(0),
	     meanDuration,
	     EXEC_TASK_ON_PERIOD(0),
	     EXEC_TASK_MAX_PERIOD(0),
	     (int)(EXEC_TASK_PERIOD(0)*1000000.0)); 
    }

    /* The result changed: inform the control task */
    if (prevExecTaskBilan != EXEC_TASK_BILAN(0)) {
      prevExecTaskBilan =  EXEC_TASK_BILAN(0);
      wakeUpCntrlTask = TRUE;
    }

    if(wakeUpCntrlTask)
      h2evnSignal(CNTRL_TASK_ID);

    /* Wake-up others tasks ? */
    for (i=0; i<CNTRL_NB_EXEC_TASKS; i++) {
      if (EXEC_TASK_WAKE_UP_FLAG(i)) {
	h2evnSignal(EXEC_TASK_ID(i));
	EXEC_TASK_WAKE_UP_FLAG(i) = FALSE;
      }
    }

    if (commonStructGive (icuDataStrId) != OK ||
	commonStructGive (icuCntrlStrId) != OK) {
      logMsg("icuRunTask: commonStructGive error\n");
      icuRunTaskSuspend (FALSE);
    }

    /* relinquish CPU to give a chance to other tasks to run in heavily
     * contended situations (small period) */
    taskDelay(0);
  }     /* FOREVER */

}



/*----------------------  ROUTINES LOCALES ---------------------------------*/



/*****************************************************************************
*
*  icuRunTaskInitTaskFunc  -  Routine initialisation
*
*  Description:
*  Cette fonction alloue et initialise tous les objets utilises par la tache
*  d'execution.
*
*  Retourne : OK ou ERROR
*/

static STATUS icuRunTaskInitTaskFunc (H2TIMER_ID *execTimerId)

{
#if (1)
  int bilan;
#endif

    /* Enregistrer l'id de la tache */
  EXEC_TASK_ID(0) = taskIdSelf ();

#if (0) /* Client */
  /* Creation de la boite aux lettres de reception des repliques */
  if (csMboxInit ("icuRunTask", 0,  
                  ICU_RUNTASK_MBOX_REPLY_SIZE) != OK) {
    h2perror("icuRunTaskInitTaskFunc: csMboxInit");
    return (ERROR); 
  }

  /* S'initialiser comme client */
  
#endif

#if (1) /* Periodic */

    /*** XXXX
      Passer la periode en milliseconde. Verifier que c'est un
      nombre entier de tic:
      Utiliser sysClkRateGet au lieu de NTICKS_PER_SEC
      Sinon sortir 
      XXX */
  /* Allouer un timer h2 */
  if ((*execTimerId = h2timerAlloc ()) == NULL) {
    h2perror("icuRunTaskInitTaskFunc: h2timerAlloc");
    return (ERROR);
  }
  LOGDBG(("icuRunTaskInitTaskFunc: timer allocated\n"));

  /* Demarrer le timer d'asservissement */
  if (h2timerStart (*execTimerId, 20, 10) != OK) {
    h2perror("icuRunTaskInitTaskFunc: h2timerStart");
    return (ERROR);
  }
  LOGDBG(("icuRunTaskInitTaskFunc: timer started\n"));
  
  /* Obtenir la periode d'asservissement */
  EXEC_TASK_PERIOD(0) = 
    ((double) 20 / (double) NTICKS_PER_SEC);
#else
  EXEC_TASK_PERIOD(0) = 0;
#endif /* Periodic */

  EXEC_TASK_MAX_PERIOD(0) = 0;
  EXEC_TASK_ON_PERIOD(0) = 0;
  EXEC_TASK_WAKE_UP_FLAG(0) = FALSE;

  /* Creer le poster */
    if (posterCreate(ICU_RESULTS_POSTER_NAME,
    sizeof(ICU_RESULTS_POSTER_STR),
    &(EXEC_TASK_POSTER_ID(0)[0])) != OK) {
     logMsg("icuRunTaskInitTaskFunc: cannot create poster Results\n");
     return(ERROR);
    }
    {
       int size = sizeof(ICU_RESULTS_POSTER_STR);
       char *tmp = malloc(size);
       if (tmp == NULL) {
          fprintf (stderr, "icuRunTaskInitTaskFunc : not enough mem to init poster Results\n");
       }
       else {
          memset(tmp, 0, size);
          if (posterWrite(EXEC_TASK_POSTER_ID(0)[0], 0, tmp, size) != size) {
             fprintf (stderr, "icuRunTaskInitTaskFunc : cannot init poster Results\n");
             free(tmp);
             return(ERROR);
          }
          free(tmp);
       }
    }
  if (posterCreate(ICU_HUMANPRESENCE_POSTER_NAME,
    sizeof(ICU_HUMANPRESENCE_POSTER_STR),
    &(EXEC_TASK_POSTER_ID(0)[1])) != OK) {
     logMsg("icuRunTaskInitTaskFunc: cannot create poster HumanPresence\n");
     return(ERROR);
    }
    {
       int size = sizeof(ICU_HUMANPRESENCE_POSTER_STR);
       char *tmp = malloc(size);
       if (tmp == NULL) {
          fprintf (stderr, "icuRunTaskInitTaskFunc : not enough mem to init poster HumanPresence\n");
       }
       else {
          memset(tmp, 0, size);
          if (posterWrite(EXEC_TASK_POSTER_ID(0)[1], 0, tmp, size) != size) {
             fprintf (stderr, "icuRunTaskInitTaskFunc : cannot init poster HumanPresence\n");
             free(tmp);
             return(ERROR);
          }
          free(tmp);
       }
    }
  if (posterCreate(ICU_PLATINECMD_POSTER_NAME,
    sizeof(ICU_PLATINECMD_POSTER_STR),
    &(EXEC_TASK_POSTER_ID(0)[2])) != OK) {
     logMsg("icuRunTaskInitTaskFunc: cannot create poster PlatineCmd\n");
     return(ERROR);
    }
    {
       int size = sizeof(ICU_PLATINECMD_POSTER_STR);
       char *tmp = malloc(size);
       if (tmp == NULL) {
          fprintf (stderr, "icuRunTaskInitTaskFunc : not enough mem to init poster PlatineCmd\n");
       }
       else {
          memset(tmp, 0, size);
          if (posterWrite(EXEC_TASK_POSTER_ID(0)[2], 0, tmp, size) != size) {
             fprintf (stderr, "icuRunTaskInitTaskFunc : cannot init poster PlatineCmd\n");
             free(tmp);
             return(ERROR);
          }
          free(tmp);
       }
    }
  if (posterCreate(ICU_VISUALDATA_POSTER_NAME,
    sizeof(ICU_VISUALDATA_POSTER_STR),
    &(EXEC_TASK_POSTER_ID(0)[3])) != OK) {
     logMsg("icuRunTaskInitTaskFunc: cannot create poster VisualData\n");
     return(ERROR);
    }
    {
       int size = sizeof(ICU_VISUALDATA_POSTER_STR);
       char *tmp = malloc(size);
       if (tmp == NULL) {
          fprintf (stderr, "icuRunTaskInitTaskFunc : not enough mem to init poster VisualData\n");
       }
       else {
          memset(tmp, 0, size);
          if (posterWrite(EXEC_TASK_POSTER_ID(0)[3], 0, tmp, size) != size) {
             fprintf (stderr, "icuRunTaskInitTaskFunc : cannot init poster VisualData\n");
             free(tmp);
             return(ERROR);
          }
          free(tmp);
       }
    }

  LOGDBG(("icuRunTaskInitTaskFunc: posters created\n"));
      
  /* S'initialiser comme client des Posters */
  
  LOGDBG(("icuRunTaskInitTaskFunc: client posters initialized\n"));

  /* Enregister le nom de la tache */
/*  strcpy (EXEC_TASK_NAME(0), "execTaskName"); */

#if (1)
    /* Execution de la fonction d'initialisation */
  if (icuRunTaskInit (&bilan) != OK) {
    errnoSet(bilan);
    h2perror("icuRunTaskInitTaskFunc: icuRunTaskInit");
    return (ERROR);
  }
#endif

  /* Record errors */
/*   icuRecordH2errMsgs(); */

   /* Donner le sem de fin d'initialisation */
  LOGDBG(("icuRunTaskInitTaskFunc: ok\n"));
  return (OK);
}

/*****************************************************************************
*
*  icuRunTaskSuspend  -  Suspension de la tache d'asservissement
*
*  Description :
*  Cette fonction signale qu'une erreur a ete retrouvee a l'interieur de 
*  la tache et se suspend par la suite
*
*  Retourne: Neant
*/

static void icuRunTaskSuspend (BOOL giveFlag)

{
  char str[64];

  /* Indiquer qu'une erreur a ete detectee */
  EXEC_TASK_STATUS(0) = ERROR;
  if (EXEC_TASK_BILAN(0) == OK)
    EXEC_TASK_BILAN(0) = errnoGet();

  logMsg("Suspend icuRunTask: %s\n", 
	  h2getErrMsg(EXEC_TASK_BILAN(0), str, 64));

  /* Eveiller la tache de controle */
  h2evnSignal(CNTRL_TASK_ID);

  /* Liberer les SDIs */
  if (giveFlag) {
    commonStructGive ((void *) icuDataStrId);
    commonStructGive ((void *) icuCntrlStrId);    
  }

  /* Suspendre la tache */
  taskSuspend (0);
}

/*****************************************************************************
*
*  execTaskCallUserFunc  -  Appele la fonction utilisateur correspondant au 
*                           numero d'activite et on etat. 
*
*  Description : 
*
*  Retourne: L'etat retourne par la fonction utilisateur 
*/

static ACTIVITY_EVENT execTaskCallUserFunc (ACTIVITY_STATE state, 
					    int activityId)

{
  void *inputId;          /* Adresse input */
  void *outputId;         /* Adresse output */
  int inputFlag;          /* Flag input */
  int outputFlag;         /* Flag output */
  int *bilan;             /* Adresse bilan */
  int activityNum;        /* Numero d'activite */

  /* Tableau des fonction d'execution (user) */
  static ACTIVITY_EVENT (*icuRunTaskExecFuncTab[])() = {
  icuInitDisplayExec,
  NULL,
  NULL,
  NULL,
  icuCaptureImageExec,
  NULL,
  NULL,
  icuEnableDisplayExec,
  icuSetCameraPosExec,
  icuTrackItExec,
  icuWaitForSomebodyExec,
  icuWaitForSomeoneExec,
  icuWaitForNobodyExec,
  icuWaitForNoneExec,
  icuWaitForNobodyKnownExec,
  icuWaitForPersonListChangeExec,
  icuProcessSetModalityExec,
  icuFacerecSetWorkingPathExec,
  NULL,
  icuFacerecInitExec,
  icuFacerecAddSomebodyExec,
  icuFacerecSetPeopleExec,
  icuFacerecRecognizeAllExec,
  icuFacerecLoadSomebodyExec,
  icuFacerecUnloadSomebodyExec,
  icuFacerecLoadEverybodyExec,
  icuFacerecUnloadEverybodyExec,
  icuWaitForModalityChangeExec,
  icuLaunchPlatineExec,
  icuCmdPtu,
  icuInitRFIDExec,
  icuCloseRFIDExec,
  icuAcquireRFIDExec,
  icuAcquireTagExec,
  NULL};

  /* Tableau des fonctions de demarrage (user)*/
  static ACTIVITY_EVENT (*icuRunTaskExecFuncStartTab[])() = {
  NULL,
  NULL,
  NULL,
  NULL,
  NULL,
  NULL,
  NULL,
  NULL,
  NULL,
  NULL,
  NULL,
  icuWaitForSomeoneStart,
  NULL,
  icuWaitForNoneStart,
  NULL,
  NULL,
  NULL,
  NULL,
  NULL,
  NULL,
  NULL,
  NULL,
  NULL,
  NULL,
  NULL,
  NULL,
  NULL,
  NULL,
  icuLaunchPlatineExecStart,
  NULL,
  NULL,
  NULL,
  NULL,
  NULL,
  NULL};

  /* Tableau des fonctions de terminaison (user) */
  static ACTIVITY_EVENT (*icuRunTaskExecFuncEndTab[])() = {
  NULL,
  NULL,
  NULL,
  NULL,
  NULL,
  NULL,
  NULL,
  NULL,
  NULL,
  NULL,
  NULL,
  NULL,
  NULL,
  NULL,
  NULL,
  NULL,
  NULL,
  NULL,
  NULL,
  NULL,
  NULL,
  NULL,
  NULL,
  NULL,
  NULL,
  NULL,
  NULL,
  NULL,
  NULL,
  NULL,
  NULL,
  NULL,
  NULL,
  NULL,
  NULL};

  /* Tableau des fonctions de terminaison en cas d'interruption (user) */
  static ACTIVITY_EVENT (*icuRunTaskExecFuncInterTab[])() = {
  NULL,
  NULL,
  NULL,
  NULL,
  NULL,
  NULL,
  NULL,
  NULL,
  NULL,
  NULL,
  NULL,
  NULL,
  NULL,
  NULL,
  NULL,
  NULL,
  NULL,
  NULL,
  NULL,
  NULL,
  NULL,
  NULL,
  NULL,
  NULL,
  NULL,
  NULL,
  NULL,
  NULL,
  NULL,
  NULL,
  NULL,
  NULL,
  NULL,
  NULL,
  NULL};
 
  /* Tableau des fonctions de terminaison en cas d'echec (user) */
  static ACTIVITY_EVENT (*icuRunTaskExecFuncFailTab[])() = {
  NULL,
  NULL,
  NULL,
  NULL,
  NULL,
  NULL,
  NULL,
  NULL,
  NULL,
  NULL,
  NULL,
  NULL,
  NULL,
  NULL,
  NULL,
  NULL,
  NULL,
  NULL,
  NULL,
  NULL,
  NULL,
  NULL,
  NULL,
  NULL,
  NULL,
  NULL,
  NULL,
  NULL,
  NULL,
  NULL,
  NULL,
  NULL,
  NULL,
  NULL,
  NULL};
 

  /* Numero de requete */
  activityNum = ACTIVITY_RQST_TYPE(activityId);

  /* Adresse des parametres */
  inputId = ACTIVITY_INPUT_ID(activityId);
  outputId = ACTIVITY_OUTPUT_ID(activityId);
  bilan =  &ACTIVITY_BILAN(activityId);

  /* Flag parametres */
  inputFlag = inputId == NULL ? FALSE : TRUE;
  outputFlag = outputId == NULL ? FALSE : TRUE;

  switch(state) {
    /* Fonction de demarage */
  case START:
    if (icuRunTaskExecFuncStartTab [activityNum] == NULL)
      return (EXEC);
    if (inputFlag && outputFlag)
      return((*icuRunTaskExecFuncStartTab [activityNum])
	     (inputId, outputId, bilan));
    if (inputFlag)
      return((*icuRunTaskExecFuncStartTab [activityNum])
	     (inputId, bilan));
    if (outputFlag)
      return((*icuRunTaskExecFuncStartTab [activityNum])
	     (outputId, bilan));
    return((*icuRunTaskExecFuncStartTab [activityNum])
	   (bilan));
    
    /* Fonction d'execution */
  case EXEC:
    if (icuRunTaskExecFuncTab [activityNum] == NULL)
      return (END);
    if (inputFlag && outputFlag)
      return((*icuRunTaskExecFuncTab [activityNum])
	     (inputId, outputId, bilan));
    if (inputFlag)
      return((*icuRunTaskExecFuncTab [activityNum])
	     (inputId, bilan));
    if (outputFlag)
      return((*icuRunTaskExecFuncTab [activityNum])
	     (outputId, bilan));
    return((*icuRunTaskExecFuncTab [activityNum])
	   (bilan));
    
    /* Fonction de terminaison normale, sur pb ou sur interruption */
  case END:
    if (icuRunTaskExecFuncEndTab [activityNum] == NULL)
      return(ETHER);
    if (inputFlag && outputFlag)
      return((*icuRunTaskExecFuncEndTab [activityNum])
	     (inputId, outputId, bilan));
    if (inputFlag)
      return((*icuRunTaskExecFuncEndTab [activityNum])
	     (inputId, bilan));
    if (outputFlag)
      return((*icuRunTaskExecFuncEndTab [activityNum])
	     (outputId, bilan));
    return((*icuRunTaskExecFuncEndTab [activityNum])
	   (bilan));

    /* Fonction a appeler en cas d'echec */
  case FAIL:
    if (icuRunTaskExecFuncFailTab [activityNum] == NULL)
      return(ZOMBIE);
    if (inputFlag && outputFlag)
      return((*icuRunTaskExecFuncFailTab [activityNum])
	     (inputId, outputId, bilan));
    if (inputFlag)
      return((*icuRunTaskExecFuncFailTab [activityNum])
	     (inputId, bilan));
    if (outputFlag)
      return((*icuRunTaskExecFuncFailTab [activityNum])
	     (outputId, bilan));
    return((*icuRunTaskExecFuncFailTab [activityNum])
	   (bilan));

    /* Fonction a appeler en cas d'interruption */
  case INTER:
    if (icuRunTaskExecFuncInterTab [activityNum] == NULL)
      return(ETHER);
    if (inputFlag && outputFlag)
      return((*icuRunTaskExecFuncInterTab [activityNum])
	     (inputId, outputId, bilan));
    if (inputFlag)
      return((*icuRunTaskExecFuncInterTab [activityNum])
	     (inputId, bilan));
    if (outputFlag)
      return((*icuRunTaskExecFuncInterTab [activityNum])
	     (outputId, bilan));
    return((*icuRunTaskExecFuncInterTab [activityNum])
	   (bilan));

    /* Etats impossibles: ZOMBIE ETHER INIT et autres */
  default:
    logMsg("icuRunTask: Activity %d status %s incoherent\n",
	    activityId, h2GetEvnStateString(state));
    icuRunTaskSuspend (TRUE);
    return(ZOMBIE);
  }  /* Switch state */
}
    
/*****************************************************************************
*
*  filterAndSendEvn  -  Teste et signal les transitions
*
*  Description : Cette fonction teste si une transition d'une activite
*                est valide. Sinon la tache est suspendue.
*
*  Retourne: TRUE s'il faut eveiller la tache de controle.
*/
	
static BOOL filterAndSendEvn (ACTIVITY_STATE state,
			      ACTIVITY_EVENT evn)
{
  BOOL transition=FALSE;
  BOOL wakeUpCntrlTask=FALSE;

  /* Transitions autorisees */
  switch (state) {
  case START:
    if (evn == EXEC || evn == END || evn == FAIL || 
	evn == ETHER || evn == ZOMBIE || evn == SLEEP || evn == START)
      transition = TRUE;
    break;
  case EXEC: 
    if (evn == EXEC || evn == END || evn == FAIL || 
	evn == ETHER || evn == ZOMBIE || evn == SLEEP)
      transition = TRUE;
    break;
  case END:
    if (evn == ZOMBIE || evn == ETHER || evn == END || evn == FAIL)
      transition = TRUE;
    break;
  case FAIL:
    if (evn == ZOMBIE)
      transition = TRUE;
    break;
  case INTER:
    if (evn == ZOMBIE || evn == ETHER || evn == FAIL || evn == INTER)
      transition = TRUE;
    break;
  default:
    logMsg("icuRunTask: status %s incoherent\n",
	    h2GetEvnStateString(state));
    icuRunTaskSuspend (TRUE);  
  } 

  /* Transition autorisee */
  if (transition) {

#if (!1) /* tache non periodique */
    /* Tache non periodique: Auto reveille sauf evn SLEEP */
    if (evn != SLEEP)
      h2evnSignal(EXEC_TASK_ID(0));
/*    semGive (EXEC_TASK_SEM_ID(0));*/
#endif

    /* Changement d'etat en vu : il faudra prevenir la tache de controle */
    if (state != (ACTIVITY_STATE) evn) 
      wakeUpCntrlTask = TRUE;
  }

  /* Transition interdite */
  else {
    /* Ne pas appeler 2 fois la fonction h2GetEvnStateString dans le logMsg */
    logMsg("icuRunTask: event %s ", h2GetEvnStateString(evn));
    logMsg("from state %s not allowed\n", h2GetEvnStateString(state)); 
    icuRunTaskSuspend (TRUE);
  }
  return wakeUpCntrlTask;
}

static long
h2timespec_udiff(const H2TIMESPEC *x, const H2TIMESPEC *y)
{
	H2TIMESPEC yy;

	memcpy(&yy, y, sizeof(H2TIMESPEC));

	/* Carry */
	if (x->tv_nsec < y->tv_nsec) {
		long sec = (y->tv_nsec - x->tv_nsec) / 1000000000 + 1;
		yy.tv_nsec -= 1000000000 * sec;
		yy.tv_sec += sec;
	}
	if (x->tv_nsec - y->tv_nsec > 1000000000) {
		int sec = (x->tv_nsec - y->tv_nsec) / 1000000000;
		yy.tv_nsec += 1000000000 * sec;
		yy.tv_sec -= sec;
	}

	return 1000000*(x->tv_sec - yy.tv_sec) 
	    + (x->tv_nsec - yy.tv_nsec)/1000;
}
