
/* 
 * Copyright (c) 1993-2003 LAAS/CNRS
 * All rights reserved.
 *
 * Redistribution and use  in source  and binary  forms,  with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *   1. Redistributions of  source  code must retain the  above copyright
 *      notice, this list of conditions and the following disclaimer.
 *   2. Redistributions in binary form must reproduce the above copyright
 *      notice,  this list of  conditions and the following disclaimer in
 *      the  documentation  and/or  other   materials provided  with  the
 *      distribution.
 *
 * THIS  SOFTWARE IS PROVIDED BY  THE  COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND  ANY  EXPRESS OR IMPLIED  WARRANTIES,  INCLUDING,  BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES  OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR  PURPOSE ARE DISCLAIMED. IN  NO EVENT SHALL THE COPYRIGHT
 * HOLDERS OR      CONTRIBUTORS  BE LIABLE FOR   ANY    DIRECT, INDIRECT,
 * INCIDENTAL,  SPECIAL,  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF  SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN  CONTRACT, STRICT LIABILITY, OR
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE   OF THIS SOFTWARE, EVEN   IF ADVISED OF   THE POSSIBILITY OF SUCH
 * DAMAGE.
 */

/*------------------  Fichier gnr automatiquement ------------------*/
/*------------------  Ne pas diter manuellement !!! ------------------*/
 
/****************************************************************************
 *   LABORATOIRE D'AUTOMATIQUE ET D'ANALYSE DE SYSTEMES - LAAS / CNRS       
 *   PROJET HILARE II - ROUTINE D'TEST INTERACTIF       
 *   FICHIER SOURCE: icuTest.c                                          
 ****************************************************************************/

/* VERSION ACTUELLE / HISTORIQUE DES MODIFICATIONS :
 */

/* DESCRIPTION :
   Cette routine doit etre appelee dans le shell de vxWorks ou UNIX.
   */

/*----------------------------- INCLUSIONS ---------------------------------*/

/* Inclusions generales */

#include <stdio.h>
#include <stdarg.h>
#include <stdlib.h>
#include <unistd.h>

#include <portLib.h>
#include <h2initGlob.h>
#include <taskLib.h>

/* Inclusions pour acceder au module */
#include "icuMsgLib.h"
#include "icuPosterShowLib.h" 
#include "icuPrint.h"
#include "icuScan.h"

/* #include "icuReports.h" */

#include "genom/testLib.h"


void icuTest (int testNumber);

/*------------------- PROTOTYPES DES FONCTIONS LOCALES --------------------*/

static void icuTestInitTask (TEST_STR* testStr);

static BOOL icuTestInitDisplay(TEST_STR *id, int rq, int ac, BOOL ch);
static BOOL icuTestPomTagging(TEST_STR *id, int rq, int ac, BOOL ch);
static BOOL icuTestAutoMode(TEST_STR *id, int rq, int ac, BOOL ch);
static BOOL icuTestSaveFace(TEST_STR *id, int rq, int ac, BOOL ch);
static BOOL icuTestCaptureImage(TEST_STR *id, int rq, int ac, BOOL ch);
static BOOL icuTestCaptureVideo(TEST_STR *id, int rq, int ac, BOOL ch);
static BOOL icuTestSetViamPosterName(TEST_STR *id, int rq, int ac, BOOL ch);
static BOOL icuTestEnableDisplay(TEST_STR *id, int rq, int ac, BOOL ch);
static BOOL icuTestSetCameraPos(TEST_STR *id, int rq, int ac, BOOL ch);
static BOOL icuTestTrackIt(TEST_STR *id, int rq, int ac, BOOL ch);
static BOOL icuTestWaitForSomebody(TEST_STR *id, int rq, int ac, BOOL ch);
static BOOL icuTestWaitForSomeone(TEST_STR *id, int rq, int ac, BOOL ch);
static BOOL icuTestWaitForNobody(TEST_STR *id, int rq, int ac, BOOL ch);
static BOOL icuTestWaitForNone(TEST_STR *id, int rq, int ac, BOOL ch);
static BOOL icuTestWaitForNobodyKnown(TEST_STR *id, int rq, int ac, BOOL ch);
static BOOL icuTestWaitForPersonListChange(TEST_STR *id, int rq, int ac, BOOL ch);
static BOOL icuTestProcessSetModality(TEST_STR *id, int rq, int ac, BOOL ch);
static BOOL icuTestFacerecSetWorkingPath(TEST_STR *id, int rq, int ac, BOOL ch);
static BOOL icuTestFacerecSetParams(TEST_STR *id, int rq, int ac, BOOL ch);
static BOOL icuTestFacerecInit(TEST_STR *id, int rq, int ac, BOOL ch);
static BOOL icuTestFacerecAddSomebody(TEST_STR *id, int rq, int ac, BOOL ch);
static BOOL icuTestFacerecSetPeople(TEST_STR *id, int rq, int ac, BOOL ch);
static BOOL icuTestFacerecRecognizeAll(TEST_STR *id, int rq, int ac, BOOL ch);
static BOOL icuTestFacerecLoadSomebody(TEST_STR *id, int rq, int ac, BOOL ch);
static BOOL icuTestFacerecUnloadSomebody(TEST_STR *id, int rq, int ac, BOOL ch);
static BOOL icuTestFacerecLoadEverybody(TEST_STR *id, int rq, int ac, BOOL ch);
static BOOL icuTestFacerecUnloadEverybody(TEST_STR *id, int rq, int ac, BOOL ch);
static BOOL icuTestWaitForModalityChange(TEST_STR *id, int rq, int ac, BOOL ch);
static BOOL icuTestLaunchPlatine(TEST_STR *id, int rq, int ac, BOOL ch);
static BOOL icuTestCmdPtu(TEST_STR *id, int rq, int ac, BOOL ch);
static BOOL icuTestIcuInitRFID(TEST_STR *id, int rq, int ac, BOOL ch);
static BOOL icuTestIcuCloseRFID(TEST_STR *id, int rq, int ac, BOOL ch);
static BOOL icuTestIcuAcquireRFID(TEST_STR *id, int rq, int ac, BOOL ch);
static BOOL icuTestIcuAcquireTag(TEST_STR *id, int rq, int ac, BOOL ch);

static  TEST_RQST_DESC_STR icuTestRqstFuncTab[] = {
  {ICU_INITDISPLAY_RQST, icuTestInitDisplay, sizeof(ICU_AFFICHAGE), 0},
  {ICU_POMTAGGING_RQST, icuTestPomTagging, sizeof(int), 0},
  {ICU_AUTOMODE_RQST, icuTestAutoMode, sizeof(ICU_AUTOSWITCH_CONTROL), 0},
  {ICU_SAVEFACE_RQST, icuTestSaveFace, sizeof(int), 0},
  {ICU_CAPTUREIMAGE_RQST, icuTestCaptureImage, sizeof(int), 0},
  {ICU_CAPTUREVIDEO_RQST, icuTestCaptureVideo, sizeof(int), 0},
  {ICU_SETVIAMPOSTERNAME_RQST, icuTestSetViamPosterName, sizeof(str), 0},
  {ICU_ENABLEDISPLAY_RQST, icuTestEnableDisplay, sizeof(ICU_AFFICHAGE), 0},
  {ICU_SETCAMERAPOS_RQST, icuTestSetCameraPos, sizeof(ICU_CAMERA_POS), 0},
  {ICU_TRACKIT_RQST, icuTestTrackIt, sizeof(int), 0},
  {ICU_WAITFORSOMEBODY_RQST, icuTestWaitForSomebody, sizeof(int), sizeof(ICU_WAITFOR)},
  {ICU_WAITFORSOMEONE_RQST, icuTestWaitForSomeone, sizeof(ICU_WAITFOR), 0},
  {ICU_WAITFORNOBODY_RQST, icuTestWaitForNobody, sizeof(int), 0},
  {ICU_WAITFORNONE_RQST, icuTestWaitForNone, sizeof(ICU_WAITFOR), 0},
  {ICU_WAITFORNOBODYKNOWN_RQST, icuTestWaitForNobodyKnown, sizeof(int), 0},
  {ICU_WAITFORPERSONLISTCHANGE_RQST, icuTestWaitForPersonListChange, sizeof(ICU_PERSON_LIST_PARAMS), sizeof(ICU_WAITFOR)},
  {ICU_PROCESSSETMODALITY_RQST, icuTestProcessSetModality, sizeof(ICU_MODALITIES), 0},
  {ICU_FACERECSETWORKINGPATH_RQST, icuTestFacerecSetWorkingPath, sizeof(GEN_STRING256), 0},
  {ICU_FACERECSETPARAMS_RQST, icuTestFacerecSetParams, sizeof(ICU_PERSON_LIST_PARAMS), 0},
  {ICU_FACERECINIT_RQST, icuTestFacerecInit, 0, 0},
  {ICU_FACERECADDSOMEBODY_RQST, icuTestFacerecAddSomebody, sizeof(GEN_STRING256), 0},
  {ICU_FACERECSETPEOPLE_RQST, icuTestFacerecSetPeople, sizeof(GEN_STRING256), 0},
  {ICU_FACERECRECOGNIZEALL_RQST, icuTestFacerecRecognizeAll, 0, 0},
  {ICU_FACERECLOADSOMEBODY_RQST, icuTestFacerecLoadSomebody, sizeof(GEN_STRING256), 0},
  {ICU_FACERECUNLOADSOMEBODY_RQST, icuTestFacerecUnloadSomebody, sizeof(GEN_STRING256), 0},
  {ICU_FACERECLOADEVERYBODY_RQST, icuTestFacerecLoadEverybody, 0, 0},
  {ICU_FACERECUNLOADEVERYBODY_RQST, icuTestFacerecUnloadEverybody, 0, 0},
  {ICU_WAITFORMODALITYCHANGE_RQST, icuTestWaitForModalityChange, 0, 0},
  {ICU_LAUNCHPLATINE_RQST, icuTestLaunchPlatine, sizeof(ICU_PLATINE_PARAMS), 0},
  {ICU_CMDPTU_RQST, icuTestCmdPtu, 0, 0},
  {ICU_ICUINITRFID_RQST, icuTestIcuInitRFID, 0, 0},
  {ICU_ICUCLOSERFID_RQST, icuTestIcuCloseRFID, 0, 0},
  {ICU_ICUACQUIRERFID_RQST, icuTestIcuAcquireRFID, 0, 0},
  {ICU_ICUACQUIRETAG_RQST, icuTestIcuAcquireTag, sizeof(GEN_STRING256), 0}
};
  
static char *icuTestRequestNameTab[] = {
"InitDisplay (I)",
"PomTagging",
"AutoMode",
"SaveFace",
"CaptureImage (nE)",
"CaptureVideo",
"SetViamPosterName",
"EnableDisplay (E)",
"SetCameraPos (nE)",
"TrackIt (nE)",
"WaitForSomebody (nE)",
"WaitForSomeone (nE)",
"WaitForNobody (nE)",
"WaitForNone (nE)",
"WaitForNobodyKnown (nE)",
"WaitForPersonListChange (E)",
"ProcessSetModality (E)",
"FacerecSetWorkingPath (E)",
"FacerecSetParams (E)",
"FacerecInit (E)",
"FacerecAddSomebody (E)",
"FacerecSetPeople (E)",
"FacerecRecognizeAll (E)",
"FacerecLoadSomebody (E)",
"FacerecUnloadSomebody (E)",
"FacerecLoadEverybody (E)",
"FacerecUnloadEverybody (E)",
"WaitForModalityChange (E)",
"LaunchPlatine (E)",
"CmdPtu (E)",
"IcuInitRFID (E)",
"IcuCloseRFID (E)",
"IcuAcquireRFID (E)",
"IcuAcquireTag (E)"
};

  
static char *icuTestPosterNameTab[] = {
"Results",
"HumanPresence",
"PlatineCmd",
"VisualData"
};
  
static STATUS (*icuTestPosterShowFuncTab[])() = {
icuResultsPosterShow,
icuHumanPresencePosterShow,
icuPlatineCmdPosterShow,
icuVisualDataPosterShow,
icuCntrlPosterShow,
icuCntrlPosterActivityShow
};


/*--------------------------------------------------------------------------*/

int
main(int argc, char **argv)
{
  if (argc != 2) {
    fprintf(stderr, "usage: icuTest <numMBox>\n");
    exit(1);
  }
  if (h2initGlob(0) == ERROR) {
      exit(2);
  }
  icuTest(atoi(argv[1]));
  /*NOTREACHED*/
  return 0;
} /* main */

/**
 **   icuTest - Fonction de test du module icu
 **/

void
icuTest (int testNumber)     
{
  TEST_STR *testStr;
  
  /* Allocation de la structure */
  if ((testStr = testInit(testNumber, "icu",
			  ICU_CLIENT_MBOX_REPLY_SIZE,
			  ICU_ABORT_RQST,
			  34,
			  icuTestRequestNameTab,
			  icuTestRqstFuncTab,
			  4, 
			  icuTestPosterNameTab,
			  icuTestPosterShowFuncTab)) == NULL)
    return;

  /* Init specifiques */
  icuTestInitTask (testStr);
  
  /* Fonction principale */
  testMain(testStr);
}

/*----------------------- ROUTINES LOCALES ---------------------------------*/


/**
 **  icuTestInitTask - Routine d'initialisation de la tache d'essai
 **/

static void icuTestInitTask (TEST_STR *testStr)
     
{
  /* S'initialiser comme client */
  printf ("client init ...");
  if (csClientInit (ICU_MBOX_NAME, ICU_MAX_RQST_SIZE,
		    ICU_MAX_INTERMED_REPLY_SIZE, 
		    ICU_MAX_REPLY_SIZE, &TEST_CID(testStr)) != OK) {
      (void) h2perror("Client init failed");
      testEnd(testStr);
  }
  
  /* S'initialiser comme client des posters */
  printf ("ok.   Poster init ...");
  if (icuPosterInit () != OK)
    h2perror ("Warning: not found all icu's posters");

  printf ("ok.");
}
 


/* 
 * Copyright (c) 2005 LAAS/CNRS
 * All rights reserved.
 *
 * Redistribution and use  in source  and binary  forms,  with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *   1. Redistributions of  source  code must retain the  above copyright
 *      notice, this list of conditions and the following disclaimer.
 *   2. Redistributions in binary form must reproduce the above copyright
 *      notice,  this list of  conditions and the following disclaimer in
 *      the  documentation  and/or  other   materials provided  with  the
 *      distribution.
 *
 * THIS  SOFTWARE IS PROVIDED BY  THE  COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND  ANY  EXPRESS OR IMPLIED  WARRANTIES,  INCLUDING,  BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES  OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR  PURPOSE ARE DISCLAIMED. IN  NO EVENT SHALL THE COPYRIGHT
 * HOLDERS OR      CONTRIBUTORS  BE LIABLE FOR   ANY    DIRECT, INDIRECT,
 * INCIDENTAL,  SPECIAL,  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF  SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN  CONTRACT, STRICT LIABILITY, OR
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE   OF THIS SOFTWARE, EVEN   IF ADVISED OF   THE POSSIBILITY OF SUCH
 * DAMAGE.
 */


/**
 **  Emission/reception de la requete d'execution icuTestInitDisplay
 **/

static BOOL icuTestInitDisplay (TEST_STR *testId, int rqstNum, 
				    int acti, BOOL silent)
{
  int *inputDims = NULL;
  

  /* Saisie donnes */
  if (!TEST_ACTIVITY_ON(testId, acti)) {
    printf ("-- Enter ICU_AFFICHAGE display:\n");
    scan_struct_ICU_AFFICHAGE(stdin, stdout, (ICU_AFFICHAGE *)TEST_RQST_INPUT(testId,rqstNum), 1, 0, inputDims);
  }
  
  /* Emission d'une requete + replique intermediaire 
     ET/OU reception replique finale (selon TEST_ACTIVITY_RQST_ID) */
  if (!testSendAndOrRcvExecRqst(testId, rqstNum, acti, silent))
    return FALSE;

  
  return TRUE;
}


/* 
 * Copyright (c) 2005 LAAS/CNRS
 * All rights reserved.
 *
 * Redistribution and use  in source  and binary  forms,  with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *   1. Redistributions of  source  code must retain the  above copyright
 *      notice, this list of conditions and the following disclaimer.
 *   2. Redistributions in binary form must reproduce the above copyright
 *      notice,  this list of  conditions and the following disclaimer in
 *      the  documentation  and/or  other   materials provided  with  the
 *      distribution.
 *
 * THIS  SOFTWARE IS PROVIDED BY  THE  COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND  ANY  EXPRESS OR IMPLIED  WARRANTIES,  INCLUDING,  BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES  OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR  PURPOSE ARE DISCLAIMED. IN  NO EVENT SHALL THE COPYRIGHT
 * HOLDERS OR      CONTRIBUTORS  BE LIABLE FOR   ANY    DIRECT, INDIRECT,
 * INCIDENTAL,  SPECIAL,  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF  SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN  CONTRACT, STRICT LIABILITY, OR
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE   OF THIS SOFTWARE, EVEN   IF ADVISED OF   THE POSSIBILITY OF SUCH
 * DAMAGE.
 */

/**
 **  Emission et reception de la requete de controle icuTestPomTagging
 **/

static BOOL icuTestPomTagging (TEST_STR *testId, int rqstNum, 
				    int acti, BOOL silent)

{
  int *inputDims = NULL;
  
  printf ("-- Enter int tag_on_off: ");
    scan_int(stdin, stdout, (int *)TEST_RQST_INPUT(testId,rqstNum), 0, 0, inputDims);
  if (!testSendAndRcvCntrlRqst(testId, rqstNum, acti, silent))
     return FALSE;

  
  return TRUE;
}


/* 
 * Copyright (c) 2005 LAAS/CNRS
 * All rights reserved.
 *
 * Redistribution and use  in source  and binary  forms,  with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *   1. Redistributions of  source  code must retain the  above copyright
 *      notice, this list of conditions and the following disclaimer.
 *   2. Redistributions in binary form must reproduce the above copyright
 *      notice,  this list of  conditions and the following disclaimer in
 *      the  documentation  and/or  other   materials provided  with  the
 *      distribution.
 *
 * THIS  SOFTWARE IS PROVIDED BY  THE  COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND  ANY  EXPRESS OR IMPLIED  WARRANTIES,  INCLUDING,  BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES  OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR  PURPOSE ARE DISCLAIMED. IN  NO EVENT SHALL THE COPYRIGHT
 * HOLDERS OR      CONTRIBUTORS  BE LIABLE FOR   ANY    DIRECT, INDIRECT,
 * INCIDENTAL,  SPECIAL,  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF  SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN  CONTRACT, STRICT LIABILITY, OR
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE   OF THIS SOFTWARE, EVEN   IF ADVISED OF   THE POSSIBILITY OF SUCH
 * DAMAGE.
 */

/**
 **  Emission et reception de la requete de controle icuTestAutoMode
 **/

static BOOL icuTestAutoMode (TEST_STR *testId, int rqstNum, 
				    int acti, BOOL silent)

{
  int *inputDims = NULL;
  
  printf ("-- Enter ICU_AUTOSWITCH_CONTROL autoSwitchControl:\n");
    scan_struct_ICU_AUTOSWITCH_CONTROL(stdin, stdout, (ICU_AUTOSWITCH_CONTROL *)TEST_RQST_INPUT(testId,rqstNum), 1, 0, inputDims);
  if (!testSendAndRcvCntrlRqst(testId, rqstNum, acti, silent))
     return FALSE;

  
  return TRUE;
}


/* 
 * Copyright (c) 2005 LAAS/CNRS
 * All rights reserved.
 *
 * Redistribution and use  in source  and binary  forms,  with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *   1. Redistributions of  source  code must retain the  above copyright
 *      notice, this list of conditions and the following disclaimer.
 *   2. Redistributions in binary form must reproduce the above copyright
 *      notice,  this list of  conditions and the following disclaimer in
 *      the  documentation  and/or  other   materials provided  with  the
 *      distribution.
 *
 * THIS  SOFTWARE IS PROVIDED BY  THE  COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND  ANY  EXPRESS OR IMPLIED  WARRANTIES,  INCLUDING,  BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES  OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR  PURPOSE ARE DISCLAIMED. IN  NO EVENT SHALL THE COPYRIGHT
 * HOLDERS OR      CONTRIBUTORS  BE LIABLE FOR   ANY    DIRECT, INDIRECT,
 * INCIDENTAL,  SPECIAL,  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF  SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN  CONTRACT, STRICT LIABILITY, OR
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE   OF THIS SOFTWARE, EVEN   IF ADVISED OF   THE POSSIBILITY OF SUCH
 * DAMAGE.
 */

/**
 **  Emission et reception de la requete de controle icuTestSaveFace
 **/

static BOOL icuTestSaveFace (TEST_STR *testId, int rqstNum, 
				    int acti, BOOL silent)

{
  int *inputDims = NULL;
  
  printf ("-- Enter int save_face: ");
    scan_int(stdin, stdout, (int *)TEST_RQST_INPUT(testId,rqstNum), 0, 0, inputDims);
  if (!testSendAndRcvCntrlRqst(testId, rqstNum, acti, silent))
     return FALSE;

  
  return TRUE;
}



/* 
 * Copyright (c) 2005 LAAS/CNRS
 * All rights reserved.
 *
 * Redistribution and use  in source  and binary  forms,  with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *   1. Redistributions of  source  code must retain the  above copyright
 *      notice, this list of conditions and the following disclaimer.
 *   2. Redistributions in binary form must reproduce the above copyright
 *      notice,  this list of  conditions and the following disclaimer in
 *      the  documentation  and/or  other   materials provided  with  the
 *      distribution.
 *
 * THIS  SOFTWARE IS PROVIDED BY  THE  COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND  ANY  EXPRESS OR IMPLIED  WARRANTIES,  INCLUDING,  BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES  OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR  PURPOSE ARE DISCLAIMED. IN  NO EVENT SHALL THE COPYRIGHT
 * HOLDERS OR      CONTRIBUTORS  BE LIABLE FOR   ANY    DIRECT, INDIRECT,
 * INCIDENTAL,  SPECIAL,  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF  SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN  CONTRACT, STRICT LIABILITY, OR
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE   OF THIS SOFTWARE, EVEN   IF ADVISED OF   THE POSSIBILITY OF SUCH
 * DAMAGE.
 */


/**
 **  Emission/reception de la requete d'execution icuTestCaptureImage
 **/

static BOOL icuTestCaptureImage (TEST_STR *testId, int rqstNum, 
				    int acti, BOOL silent)
{
  int *inputDims = NULL;
  

  /* Saisie donnes */
  if (!TEST_ACTIVITY_ON(testId, acti)) {
    printf ("-- Enter int captureImage: ");
    scan_int(stdin, stdout, (int *)TEST_RQST_INPUT(testId,rqstNum), 0, 0, inputDims);
  }
  
  /* Emission d'une requete + replique intermediaire 
     ET/OU reception replique finale (selon TEST_ACTIVITY_RQST_ID) */
  if (!testSendAndOrRcvExecRqst(testId, rqstNum, acti, silent))
    return FALSE;

  
  return TRUE;
}


/* 
 * Copyright (c) 2005 LAAS/CNRS
 * All rights reserved.
 *
 * Redistribution and use  in source  and binary  forms,  with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *   1. Redistributions of  source  code must retain the  above copyright
 *      notice, this list of conditions and the following disclaimer.
 *   2. Redistributions in binary form must reproduce the above copyright
 *      notice,  this list of  conditions and the following disclaimer in
 *      the  documentation  and/or  other   materials provided  with  the
 *      distribution.
 *
 * THIS  SOFTWARE IS PROVIDED BY  THE  COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND  ANY  EXPRESS OR IMPLIED  WARRANTIES,  INCLUDING,  BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES  OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR  PURPOSE ARE DISCLAIMED. IN  NO EVENT SHALL THE COPYRIGHT
 * HOLDERS OR      CONTRIBUTORS  BE LIABLE FOR   ANY    DIRECT, INDIRECT,
 * INCIDENTAL,  SPECIAL,  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF  SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN  CONTRACT, STRICT LIABILITY, OR
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE   OF THIS SOFTWARE, EVEN   IF ADVISED OF   THE POSSIBILITY OF SUCH
 * DAMAGE.
 */

/**
 **  Emission et reception de la requete de controle icuTestCaptureVideo
 **/

static BOOL icuTestCaptureVideo (TEST_STR *testId, int rqstNum, 
				    int acti, BOOL silent)

{
  int *inputDims = NULL;
  
  printf ("-- Enter int captureVideo: ");
    scan_int(stdin, stdout, (int *)TEST_RQST_INPUT(testId,rqstNum), 0, 0, inputDims);
  if (!testSendAndRcvCntrlRqst(testId, rqstNum, acti, silent))
     return FALSE;

  
  return TRUE;
}


/* 
 * Copyright (c) 2005 LAAS/CNRS
 * All rights reserved.
 *
 * Redistribution and use  in source  and binary  forms,  with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *   1. Redistributions of  source  code must retain the  above copyright
 *      notice, this list of conditions and the following disclaimer.
 *   2. Redistributions in binary form must reproduce the above copyright
 *      notice,  this list of  conditions and the following disclaimer in
 *      the  documentation  and/or  other   materials provided  with  the
 *      distribution.
 *
 * THIS  SOFTWARE IS PROVIDED BY  THE  COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND  ANY  EXPRESS OR IMPLIED  WARRANTIES,  INCLUDING,  BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES  OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR  PURPOSE ARE DISCLAIMED. IN  NO EVENT SHALL THE COPYRIGHT
 * HOLDERS OR      CONTRIBUTORS  BE LIABLE FOR   ANY    DIRECT, INDIRECT,
 * INCIDENTAL,  SPECIAL,  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF  SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN  CONTRACT, STRICT LIABILITY, OR
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE   OF THIS SOFTWARE, EVEN   IF ADVISED OF   THE POSSIBILITY OF SUCH
 * DAMAGE.
 */

/**
 **  Emission et reception de la requete de controle icuTestSetViamPosterName
 **/

static BOOL icuTestSetViamPosterName (TEST_STR *testId, int rqstNum, 
				    int acti, BOOL silent)

{
  int *inputDims = NULL;
  
  printf ("-- Enter str viamPosterName:\n");
    scan_struct_str(stdin, stdout, (str *)TEST_RQST_INPUT(testId,rqstNum), 1, 0, inputDims);
  if (!testSendAndRcvCntrlRqst(testId, rqstNum, acti, silent))
     return FALSE;

  
  return TRUE;
}



/* 
 * Copyright (c) 2005 LAAS/CNRS
 * All rights reserved.
 *
 * Redistribution and use  in source  and binary  forms,  with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *   1. Redistributions of  source  code must retain the  above copyright
 *      notice, this list of conditions and the following disclaimer.
 *   2. Redistributions in binary form must reproduce the above copyright
 *      notice,  this list of  conditions and the following disclaimer in
 *      the  documentation  and/or  other   materials provided  with  the
 *      distribution.
 *
 * THIS  SOFTWARE IS PROVIDED BY  THE  COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND  ANY  EXPRESS OR IMPLIED  WARRANTIES,  INCLUDING,  BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES  OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR  PURPOSE ARE DISCLAIMED. IN  NO EVENT SHALL THE COPYRIGHT
 * HOLDERS OR      CONTRIBUTORS  BE LIABLE FOR   ANY    DIRECT, INDIRECT,
 * INCIDENTAL,  SPECIAL,  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF  SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN  CONTRACT, STRICT LIABILITY, OR
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE   OF THIS SOFTWARE, EVEN   IF ADVISED OF   THE POSSIBILITY OF SUCH
 * DAMAGE.
 */


/**
 **  Emission/reception de la requete d'execution icuTestEnableDisplay
 **/

static BOOL icuTestEnableDisplay (TEST_STR *testId, int rqstNum, 
				    int acti, BOOL silent)
{
  int *inputDims = NULL;
  

  /* Saisie donnes */
  if (!TEST_ACTIVITY_ON(testId, acti)) {
    printf ("-- Enter ICU_AFFICHAGE display:\n");
    scan_struct_ICU_AFFICHAGE(stdin, stdout, (ICU_AFFICHAGE *)TEST_RQST_INPUT(testId,rqstNum), 1, 0, inputDims);
  }
  
  /* Emission d'une requete + replique intermediaire 
     ET/OU reception replique finale (selon TEST_ACTIVITY_RQST_ID) */
  if (!testSendAndOrRcvExecRqst(testId, rqstNum, acti, silent))
    return FALSE;

  
  return TRUE;
}



/* 
 * Copyright (c) 2005 LAAS/CNRS
 * All rights reserved.
 *
 * Redistribution and use  in source  and binary  forms,  with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *   1. Redistributions of  source  code must retain the  above copyright
 *      notice, this list of conditions and the following disclaimer.
 *   2. Redistributions in binary form must reproduce the above copyright
 *      notice,  this list of  conditions and the following disclaimer in
 *      the  documentation  and/or  other   materials provided  with  the
 *      distribution.
 *
 * THIS  SOFTWARE IS PROVIDED BY  THE  COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND  ANY  EXPRESS OR IMPLIED  WARRANTIES,  INCLUDING,  BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES  OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR  PURPOSE ARE DISCLAIMED. IN  NO EVENT SHALL THE COPYRIGHT
 * HOLDERS OR      CONTRIBUTORS  BE LIABLE FOR   ANY    DIRECT, INDIRECT,
 * INCIDENTAL,  SPECIAL,  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF  SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN  CONTRACT, STRICT LIABILITY, OR
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE   OF THIS SOFTWARE, EVEN   IF ADVISED OF   THE POSSIBILITY OF SUCH
 * DAMAGE.
 */


/**
 **  Emission/reception de la requete d'execution icuTestSetCameraPos
 **/

static BOOL icuTestSetCameraPos (TEST_STR *testId, int rqstNum, 
				    int acti, BOOL silent)
{
  int *inputDims = NULL;
  

  /* Saisie donnes */
  if (!TEST_ACTIVITY_ON(testId, acti)) {
    printf ("-- Enter ICU_CAMERA_POS cameraPos:\n");
    scan_struct_ICU_CAMERA_POS(stdin, stdout, (ICU_CAMERA_POS *)TEST_RQST_INPUT(testId,rqstNum), 1, 0, inputDims);
  }
  
  /* Emission d'une requete + replique intermediaire 
     ET/OU reception replique finale (selon TEST_ACTIVITY_RQST_ID) */
  if (!testSendAndOrRcvExecRqst(testId, rqstNum, acti, silent))
    return FALSE;

  
  return TRUE;
}



/* 
 * Copyright (c) 2005 LAAS/CNRS
 * All rights reserved.
 *
 * Redistribution and use  in source  and binary  forms,  with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *   1. Redistributions of  source  code must retain the  above copyright
 *      notice, this list of conditions and the following disclaimer.
 *   2. Redistributions in binary form must reproduce the above copyright
 *      notice,  this list of  conditions and the following disclaimer in
 *      the  documentation  and/or  other   materials provided  with  the
 *      distribution.
 *
 * THIS  SOFTWARE IS PROVIDED BY  THE  COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND  ANY  EXPRESS OR IMPLIED  WARRANTIES,  INCLUDING,  BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES  OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR  PURPOSE ARE DISCLAIMED. IN  NO EVENT SHALL THE COPYRIGHT
 * HOLDERS OR      CONTRIBUTORS  BE LIABLE FOR   ANY    DIRECT, INDIRECT,
 * INCIDENTAL,  SPECIAL,  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF  SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN  CONTRACT, STRICT LIABILITY, OR
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE   OF THIS SOFTWARE, EVEN   IF ADVISED OF   THE POSSIBILITY OF SUCH
 * DAMAGE.
 */


/**
 **  Emission/reception de la requete d'execution icuTestTrackIt
 **/

static BOOL icuTestTrackIt (TEST_STR *testId, int rqstNum, 
				    int acti, BOOL silent)
{
  int *inputDims = NULL;
  

  /* Saisie donnes */
  if (!TEST_ACTIVITY_ON(testId, acti)) {
    printf ("-- Enter int track_it: ");
    scan_int(stdin, stdout, (int *)TEST_RQST_INPUT(testId,rqstNum), 0, 0, inputDims);
  }
  
  /* Emission d'une requete + replique intermediaire 
     ET/OU reception replique finale (selon TEST_ACTIVITY_RQST_ID) */
  if (!testSendAndOrRcvExecRqst(testId, rqstNum, acti, silent))
    return FALSE;

  
  return TRUE;
}



/* 
 * Copyright (c) 2005 LAAS/CNRS
 * All rights reserved.
 *
 * Redistribution and use  in source  and binary  forms,  with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *   1. Redistributions of  source  code must retain the  above copyright
 *      notice, this list of conditions and the following disclaimer.
 *   2. Redistributions in binary form must reproduce the above copyright
 *      notice,  this list of  conditions and the following disclaimer in
 *      the  documentation  and/or  other   materials provided  with  the
 *      distribution.
 *
 * THIS  SOFTWARE IS PROVIDED BY  THE  COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND  ANY  EXPRESS OR IMPLIED  WARRANTIES,  INCLUDING,  BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES  OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR  PURPOSE ARE DISCLAIMED. IN  NO EVENT SHALL THE COPYRIGHT
 * HOLDERS OR      CONTRIBUTORS  BE LIABLE FOR   ANY    DIRECT, INDIRECT,
 * INCIDENTAL,  SPECIAL,  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF  SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN  CONTRACT, STRICT LIABILITY, OR
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE   OF THIS SOFTWARE, EVEN   IF ADVISED OF   THE POSSIBILITY OF SUCH
 * DAMAGE.
 */


/**
 **  Emission/reception de la requete d'execution icuTestWaitForSomebody
 **/

static BOOL icuTestWaitForSomebody (TEST_STR *testId, int rqstNum, 
				    int acti, BOOL silent)
{
  int *inputDims = NULL;
  int *outputDims = NULL;

  /* Saisie donnes */
  if (!TEST_ACTIVITY_ON(testId, acti)) {
    printf ("-- Enter int nbImageMaxNobody: ");
    scan_int(stdin, stdout, (int *)TEST_RQST_INPUT(testId,rqstNum), 0, 0, inputDims);
  }
  
  /* Emission d'une requete + replique intermediaire 
     ET/OU reception replique finale (selon TEST_ACTIVITY_RQST_ID) */
  if (!testSendAndOrRcvExecRqst(testId, rqstNum, acti, silent))
    return FALSE;

  if (!silent) {
    printf ("waitFor:\n");
    print_struct_ICU_WAITFOR(stdout, (ICU_WAITFOR *)TEST_RQST_OUTPUT(testId,rqstNum), 1, 0, outputDims, stdin);
  }

  return TRUE;
}



/* 
 * Copyright (c) 2005 LAAS/CNRS
 * All rights reserved.
 *
 * Redistribution and use  in source  and binary  forms,  with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *   1. Redistributions of  source  code must retain the  above copyright
 *      notice, this list of conditions and the following disclaimer.
 *   2. Redistributions in binary form must reproduce the above copyright
 *      notice,  this list of  conditions and the following disclaimer in
 *      the  documentation  and/or  other   materials provided  with  the
 *      distribution.
 *
 * THIS  SOFTWARE IS PROVIDED BY  THE  COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND  ANY  EXPRESS OR IMPLIED  WARRANTIES,  INCLUDING,  BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES  OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR  PURPOSE ARE DISCLAIMED. IN  NO EVENT SHALL THE COPYRIGHT
 * HOLDERS OR      CONTRIBUTORS  BE LIABLE FOR   ANY    DIRECT, INDIRECT,
 * INCIDENTAL,  SPECIAL,  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF  SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN  CONTRACT, STRICT LIABILITY, OR
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE   OF THIS SOFTWARE, EVEN   IF ADVISED OF   THE POSSIBILITY OF SUCH
 * DAMAGE.
 */


/**
 **  Emission/reception de la requete d'execution icuTestWaitForSomeone
 **/

static BOOL icuTestWaitForSomeone (TEST_STR *testId, int rqstNum, 
				    int acti, BOOL silent)
{
  int *inputDims = NULL;
  

  /* Saisie donnes */
  if (!TEST_ACTIVITY_ON(testId, acti)) {
      const char *menu[] = {"waitFor", "request WaitForSomebody (nE) - needs input -", "request WaitForPersonListChange (E) - needs input -"};
  if(testPrintGetInputMenu(testId, 2, menu, rqstNum)) {
  switch(TEST_RQST_GET_CMD(testId,rqstNum)) {
    case 0:
    if (!testSendAndGetInput(testId, ICU_WAITFORSOMEBODY_RQST, acti, TEST_RQST_INPUT(testId,rqstNum)))
       return FALSE;
    break;
    case 1:
    if (!testSendAndGetInput(testId, ICU_WAITFORPERSONLISTCHANGE_RQST, acti, TEST_RQST_INPUT(testId,rqstNum)))
       return FALSE;
    break;
    default:
    break;
   }
  }
  printf("\n");printf ("-- Enter ICU_WAITFOR waitFor:\n");
    scan_struct_ICU_WAITFOR(stdin, stdout, (ICU_WAITFOR *)TEST_RQST_INPUT(testId,rqstNum), 1, 0, inputDims);
  }
  
  /* Emission d'une requete + replique intermediaire 
     ET/OU reception replique finale (selon TEST_ACTIVITY_RQST_ID) */
  if (!testSendAndOrRcvExecRqst(testId, rqstNum, acti, silent))
    return FALSE;

  
  return TRUE;
}



/* 
 * Copyright (c) 2005 LAAS/CNRS
 * All rights reserved.
 *
 * Redistribution and use  in source  and binary  forms,  with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *   1. Redistributions of  source  code must retain the  above copyright
 *      notice, this list of conditions and the following disclaimer.
 *   2. Redistributions in binary form must reproduce the above copyright
 *      notice,  this list of  conditions and the following disclaimer in
 *      the  documentation  and/or  other   materials provided  with  the
 *      distribution.
 *
 * THIS  SOFTWARE IS PROVIDED BY  THE  COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND  ANY  EXPRESS OR IMPLIED  WARRANTIES,  INCLUDING,  BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES  OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR  PURPOSE ARE DISCLAIMED. IN  NO EVENT SHALL THE COPYRIGHT
 * HOLDERS OR      CONTRIBUTORS  BE LIABLE FOR   ANY    DIRECT, INDIRECT,
 * INCIDENTAL,  SPECIAL,  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF  SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN  CONTRACT, STRICT LIABILITY, OR
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE   OF THIS SOFTWARE, EVEN   IF ADVISED OF   THE POSSIBILITY OF SUCH
 * DAMAGE.
 */


/**
 **  Emission/reception de la requete d'execution icuTestWaitForNobody
 **/

static BOOL icuTestWaitForNobody (TEST_STR *testId, int rqstNum, 
				    int acti, BOOL silent)
{
  int *inputDims = NULL;
  

  /* Saisie donnes */
  if (!TEST_ACTIVITY_ON(testId, acti)) {
    printf ("-- Enter int nbImageMaxNobody: ");
    scan_int(stdin, stdout, (int *)TEST_RQST_INPUT(testId,rqstNum), 0, 0, inputDims);
  }
  
  /* Emission d'une requete + replique intermediaire 
     ET/OU reception replique finale (selon TEST_ACTIVITY_RQST_ID) */
  if (!testSendAndOrRcvExecRqst(testId, rqstNum, acti, silent))
    return FALSE;

  
  return TRUE;
}



/* 
 * Copyright (c) 2005 LAAS/CNRS
 * All rights reserved.
 *
 * Redistribution and use  in source  and binary  forms,  with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *   1. Redistributions of  source  code must retain the  above copyright
 *      notice, this list of conditions and the following disclaimer.
 *   2. Redistributions in binary form must reproduce the above copyright
 *      notice,  this list of  conditions and the following disclaimer in
 *      the  documentation  and/or  other   materials provided  with  the
 *      distribution.
 *
 * THIS  SOFTWARE IS PROVIDED BY  THE  COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND  ANY  EXPRESS OR IMPLIED  WARRANTIES,  INCLUDING,  BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES  OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR  PURPOSE ARE DISCLAIMED. IN  NO EVENT SHALL THE COPYRIGHT
 * HOLDERS OR      CONTRIBUTORS  BE LIABLE FOR   ANY    DIRECT, INDIRECT,
 * INCIDENTAL,  SPECIAL,  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF  SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN  CONTRACT, STRICT LIABILITY, OR
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE   OF THIS SOFTWARE, EVEN   IF ADVISED OF   THE POSSIBILITY OF SUCH
 * DAMAGE.
 */


/**
 **  Emission/reception de la requete d'execution icuTestWaitForNone
 **/

static BOOL icuTestWaitForNone (TEST_STR *testId, int rqstNum, 
				    int acti, BOOL silent)
{
  int *inputDims = NULL;
  

  /* Saisie donnes */
  if (!TEST_ACTIVITY_ON(testId, acti)) {
      const char *menu[] = {"waitFor", "request WaitForSomebody (nE) - needs input -", "request WaitForPersonListChange (E) - needs input -"};
  if(testPrintGetInputMenu(testId, 2, menu, rqstNum)) {
  switch(TEST_RQST_GET_CMD(testId,rqstNum)) {
    case 0:
    if (!testSendAndGetInput(testId, ICU_WAITFORSOMEBODY_RQST, acti, TEST_RQST_INPUT(testId,rqstNum)))
       return FALSE;
    break;
    case 1:
    if (!testSendAndGetInput(testId, ICU_WAITFORPERSONLISTCHANGE_RQST, acti, TEST_RQST_INPUT(testId,rqstNum)))
       return FALSE;
    break;
    default:
    break;
   }
  }
  printf("\n");printf ("-- Enter ICU_WAITFOR waitFor:\n");
    scan_struct_ICU_WAITFOR(stdin, stdout, (ICU_WAITFOR *)TEST_RQST_INPUT(testId,rqstNum), 1, 0, inputDims);
  }
  
  /* Emission d'une requete + replique intermediaire 
     ET/OU reception replique finale (selon TEST_ACTIVITY_RQST_ID) */
  if (!testSendAndOrRcvExecRqst(testId, rqstNum, acti, silent))
    return FALSE;

  
  return TRUE;
}



/* 
 * Copyright (c) 2005 LAAS/CNRS
 * All rights reserved.
 *
 * Redistribution and use  in source  and binary  forms,  with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *   1. Redistributions of  source  code must retain the  above copyright
 *      notice, this list of conditions and the following disclaimer.
 *   2. Redistributions in binary form must reproduce the above copyright
 *      notice,  this list of  conditions and the following disclaimer in
 *      the  documentation  and/or  other   materials provided  with  the
 *      distribution.
 *
 * THIS  SOFTWARE IS PROVIDED BY  THE  COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND  ANY  EXPRESS OR IMPLIED  WARRANTIES,  INCLUDING,  BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES  OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR  PURPOSE ARE DISCLAIMED. IN  NO EVENT SHALL THE COPYRIGHT
 * HOLDERS OR      CONTRIBUTORS  BE LIABLE FOR   ANY    DIRECT, INDIRECT,
 * INCIDENTAL,  SPECIAL,  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF  SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN  CONTRACT, STRICT LIABILITY, OR
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE   OF THIS SOFTWARE, EVEN   IF ADVISED OF   THE POSSIBILITY OF SUCH
 * DAMAGE.
 */


/**
 **  Emission/reception de la requete d'execution icuTestWaitForNobodyKnown
 **/

static BOOL icuTestWaitForNobodyKnown (TEST_STR *testId, int rqstNum, 
				    int acti, BOOL silent)
{
  int *inputDims = NULL;
  

  /* Saisie donnes */
  if (!TEST_ACTIVITY_ON(testId, acti)) {
    printf ("-- Enter int nbImageMaxNobody: ");
    scan_int(stdin, stdout, (int *)TEST_RQST_INPUT(testId,rqstNum), 0, 0, inputDims);
  }
  
  /* Emission d'une requete + replique intermediaire 
     ET/OU reception replique finale (selon TEST_ACTIVITY_RQST_ID) */
  if (!testSendAndOrRcvExecRqst(testId, rqstNum, acti, silent))
    return FALSE;

  
  return TRUE;
}



/* 
 * Copyright (c) 2005 LAAS/CNRS
 * All rights reserved.
 *
 * Redistribution and use  in source  and binary  forms,  with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *   1. Redistributions of  source  code must retain the  above copyright
 *      notice, this list of conditions and the following disclaimer.
 *   2. Redistributions in binary form must reproduce the above copyright
 *      notice,  this list of  conditions and the following disclaimer in
 *      the  documentation  and/or  other   materials provided  with  the
 *      distribution.
 *
 * THIS  SOFTWARE IS PROVIDED BY  THE  COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND  ANY  EXPRESS OR IMPLIED  WARRANTIES,  INCLUDING,  BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES  OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR  PURPOSE ARE DISCLAIMED. IN  NO EVENT SHALL THE COPYRIGHT
 * HOLDERS OR      CONTRIBUTORS  BE LIABLE FOR   ANY    DIRECT, INDIRECT,
 * INCIDENTAL,  SPECIAL,  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF  SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN  CONTRACT, STRICT LIABILITY, OR
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE   OF THIS SOFTWARE, EVEN   IF ADVISED OF   THE POSSIBILITY OF SUCH
 * DAMAGE.
 */


/**
 **  Emission/reception de la requete d'execution icuTestWaitForPersonListChange
 **/

static BOOL icuTestWaitForPersonListChange (TEST_STR *testId, int rqstNum, 
				    int acti, BOOL silent)
{
  int *inputDims = NULL;
  int *outputDims = NULL;

  /* Saisie donnes */
  if (!TEST_ACTIVITY_ON(testId, acti)) {
    printf ("-- Enter ICU_PERSON_LIST_PARAMS paramsPersonList:\n");
    scan_struct_ICU_PERSON_LIST_PARAMS(stdin, stdout, (ICU_PERSON_LIST_PARAMS *)TEST_RQST_INPUT(testId,rqstNum), 1, 0, inputDims);
  }
  
  /* Emission d'une requete + replique intermediaire 
     ET/OU reception replique finale (selon TEST_ACTIVITY_RQST_ID) */
  if (!testSendAndOrRcvExecRqst(testId, rqstNum, acti, silent))
    return FALSE;

  if (!silent) {
    printf ("waitFor:\n");
    print_struct_ICU_WAITFOR(stdout, (ICU_WAITFOR *)TEST_RQST_OUTPUT(testId,rqstNum), 1, 0, outputDims, stdin);
  }

  return TRUE;
}



/* 
 * Copyright (c) 2005 LAAS/CNRS
 * All rights reserved.
 *
 * Redistribution and use  in source  and binary  forms,  with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *   1. Redistributions of  source  code must retain the  above copyright
 *      notice, this list of conditions and the following disclaimer.
 *   2. Redistributions in binary form must reproduce the above copyright
 *      notice,  this list of  conditions and the following disclaimer in
 *      the  documentation  and/or  other   materials provided  with  the
 *      distribution.
 *
 * THIS  SOFTWARE IS PROVIDED BY  THE  COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND  ANY  EXPRESS OR IMPLIED  WARRANTIES,  INCLUDING,  BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES  OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR  PURPOSE ARE DISCLAIMED. IN  NO EVENT SHALL THE COPYRIGHT
 * HOLDERS OR      CONTRIBUTORS  BE LIABLE FOR   ANY    DIRECT, INDIRECT,
 * INCIDENTAL,  SPECIAL,  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF  SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN  CONTRACT, STRICT LIABILITY, OR
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE   OF THIS SOFTWARE, EVEN   IF ADVISED OF   THE POSSIBILITY OF SUCH
 * DAMAGE.
 */


/**
 **  Emission/reception de la requete d'execution icuTestProcessSetModality
 **/

static BOOL icuTestProcessSetModality (TEST_STR *testId, int rqstNum, 
				    int acti, BOOL silent)
{
  int *inputDims = NULL;
  

  /* Saisie donnes */
  if (!TEST_ACTIVITY_ON(testId, acti)) {
    printf ("-- Enter ICU_MODALITIES setModality: ");
    scan_enum_ICU_MODALITIES(stdin, stdout, (ICU_MODALITIES *)TEST_RQST_INPUT(testId,rqstNum), 0, 0, inputDims);
  }
  
  /* Emission d'une requete + replique intermediaire 
     ET/OU reception replique finale (selon TEST_ACTIVITY_RQST_ID) */
  if (!testSendAndOrRcvExecRqst(testId, rqstNum, acti, silent))
    return FALSE;

  
  return TRUE;
}



/* 
 * Copyright (c) 2005 LAAS/CNRS
 * All rights reserved.
 *
 * Redistribution and use  in source  and binary  forms,  with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *   1. Redistributions of  source  code must retain the  above copyright
 *      notice, this list of conditions and the following disclaimer.
 *   2. Redistributions in binary form must reproduce the above copyright
 *      notice,  this list of  conditions and the following disclaimer in
 *      the  documentation  and/or  other   materials provided  with  the
 *      distribution.
 *
 * THIS  SOFTWARE IS PROVIDED BY  THE  COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND  ANY  EXPRESS OR IMPLIED  WARRANTIES,  INCLUDING,  BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES  OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR  PURPOSE ARE DISCLAIMED. IN  NO EVENT SHALL THE COPYRIGHT
 * HOLDERS OR      CONTRIBUTORS  BE LIABLE FOR   ANY    DIRECT, INDIRECT,
 * INCIDENTAL,  SPECIAL,  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF  SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN  CONTRACT, STRICT LIABILITY, OR
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE   OF THIS SOFTWARE, EVEN   IF ADVISED OF   THE POSSIBILITY OF SUCH
 * DAMAGE.
 */


/**
 **  Emission/reception de la requete d'execution icuTestFacerecSetWorkingPath
 **/

static BOOL icuTestFacerecSetWorkingPath (TEST_STR *testId, int rqstNum, 
				    int acti, BOOL silent)
{
  int *inputDims = NULL;
  

  /* Saisie donnes */
  if (!TEST_ACTIVITY_ON(testId, acti)) {
    printf ("-- Enter GEN_STRING256 facerecWorkingPath:\n");
    scan_struct_GEN_STRING256(stdin, stdout, (GEN_STRING256 *)TEST_RQST_INPUT(testId,rqstNum), 1, 0, inputDims);
  }
  
  /* Emission d'une requete + replique intermediaire 
     ET/OU reception replique finale (selon TEST_ACTIVITY_RQST_ID) */
  if (!testSendAndOrRcvExecRqst(testId, rqstNum, acti, silent))
    return FALSE;

  
  return TRUE;
}



/* 
 * Copyright (c) 2005 LAAS/CNRS
 * All rights reserved.
 *
 * Redistribution and use  in source  and binary  forms,  with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *   1. Redistributions of  source  code must retain the  above copyright
 *      notice, this list of conditions and the following disclaimer.
 *   2. Redistributions in binary form must reproduce the above copyright
 *      notice,  this list of  conditions and the following disclaimer in
 *      the  documentation  and/or  other   materials provided  with  the
 *      distribution.
 *
 * THIS  SOFTWARE IS PROVIDED BY  THE  COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND  ANY  EXPRESS OR IMPLIED  WARRANTIES,  INCLUDING,  BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES  OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR  PURPOSE ARE DISCLAIMED. IN  NO EVENT SHALL THE COPYRIGHT
 * HOLDERS OR      CONTRIBUTORS  BE LIABLE FOR   ANY    DIRECT, INDIRECT,
 * INCIDENTAL,  SPECIAL,  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF  SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN  CONTRACT, STRICT LIABILITY, OR
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE   OF THIS SOFTWARE, EVEN   IF ADVISED OF   THE POSSIBILITY OF SUCH
 * DAMAGE.
 */


/**
 **  Emission/reception de la requete d'execution icuTestFacerecSetParams
 **/

static BOOL icuTestFacerecSetParams (TEST_STR *testId, int rqstNum, 
				    int acti, BOOL silent)
{
  int *inputDims = NULL;
  

  /* Saisie donnes */
  if (!TEST_ACTIVITY_ON(testId, acti)) {
    printf ("-- Enter ICU_PERSON_LIST_PARAMS paramsPersonList:\n");
    scan_struct_ICU_PERSON_LIST_PARAMS(stdin, stdout, (ICU_PERSON_LIST_PARAMS *)TEST_RQST_INPUT(testId,rqstNum), 1, 0, inputDims);
  }
  
  /* Emission d'une requete + replique intermediaire 
     ET/OU reception replique finale (selon TEST_ACTIVITY_RQST_ID) */
  if (!testSendAndOrRcvExecRqst(testId, rqstNum, acti, silent))
    return FALSE;

  
  return TRUE;
}



/* 
 * Copyright (c) 2005 LAAS/CNRS
 * All rights reserved.
 *
 * Redistribution and use  in source  and binary  forms,  with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *   1. Redistributions of  source  code must retain the  above copyright
 *      notice, this list of conditions and the following disclaimer.
 *   2. Redistributions in binary form must reproduce the above copyright
 *      notice,  this list of  conditions and the following disclaimer in
 *      the  documentation  and/or  other   materials provided  with  the
 *      distribution.
 *
 * THIS  SOFTWARE IS PROVIDED BY  THE  COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND  ANY  EXPRESS OR IMPLIED  WARRANTIES,  INCLUDING,  BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES  OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR  PURPOSE ARE DISCLAIMED. IN  NO EVENT SHALL THE COPYRIGHT
 * HOLDERS OR      CONTRIBUTORS  BE LIABLE FOR   ANY    DIRECT, INDIRECT,
 * INCIDENTAL,  SPECIAL,  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF  SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN  CONTRACT, STRICT LIABILITY, OR
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE   OF THIS SOFTWARE, EVEN   IF ADVISED OF   THE POSSIBILITY OF SUCH
 * DAMAGE.
 */


/**
 **  Emission/reception de la requete d'execution icuTestFacerecInit
 **/

static BOOL icuTestFacerecInit (TEST_STR *testId, int rqstNum, 
				    int acti, BOOL silent)
{
  
  

  /* Saisie donnes */
  if (!TEST_ACTIVITY_ON(testId, acti)) {
    
  }
  
  /* Emission d'une requete + replique intermediaire 
     ET/OU reception replique finale (selon TEST_ACTIVITY_RQST_ID) */
  if (!testSendAndOrRcvExecRqst(testId, rqstNum, acti, silent))
    return FALSE;

  
  return TRUE;
}



/* 
 * Copyright (c) 2005 LAAS/CNRS
 * All rights reserved.
 *
 * Redistribution and use  in source  and binary  forms,  with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *   1. Redistributions of  source  code must retain the  above copyright
 *      notice, this list of conditions and the following disclaimer.
 *   2. Redistributions in binary form must reproduce the above copyright
 *      notice,  this list of  conditions and the following disclaimer in
 *      the  documentation  and/or  other   materials provided  with  the
 *      distribution.
 *
 * THIS  SOFTWARE IS PROVIDED BY  THE  COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND  ANY  EXPRESS OR IMPLIED  WARRANTIES,  INCLUDING,  BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES  OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR  PURPOSE ARE DISCLAIMED. IN  NO EVENT SHALL THE COPYRIGHT
 * HOLDERS OR      CONTRIBUTORS  BE LIABLE FOR   ANY    DIRECT, INDIRECT,
 * INCIDENTAL,  SPECIAL,  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF  SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN  CONTRACT, STRICT LIABILITY, OR
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE   OF THIS SOFTWARE, EVEN   IF ADVISED OF   THE POSSIBILITY OF SUCH
 * DAMAGE.
 */


/**
 **  Emission/reception de la requete d'execution icuTestFacerecAddSomebody
 **/

static BOOL icuTestFacerecAddSomebody (TEST_STR *testId, int rqstNum, 
				    int acti, BOOL silent)
{
  int *inputDims = NULL;
  

  /* Saisie donnes */
  if (!TEST_ACTIVITY_ON(testId, acti)) {
    printf ("-- Enter GEN_STRING256 recoAddName:\n");
    scan_struct_GEN_STRING256(stdin, stdout, (GEN_STRING256 *)TEST_RQST_INPUT(testId,rqstNum), 1, 0, inputDims);
  }
  
  /* Emission d'une requete + replique intermediaire 
     ET/OU reception replique finale (selon TEST_ACTIVITY_RQST_ID) */
  if (!testSendAndOrRcvExecRqst(testId, rqstNum, acti, silent))
    return FALSE;

  
  return TRUE;
}



/* 
 * Copyright (c) 2005 LAAS/CNRS
 * All rights reserved.
 *
 * Redistribution and use  in source  and binary  forms,  with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *   1. Redistributions of  source  code must retain the  above copyright
 *      notice, this list of conditions and the following disclaimer.
 *   2. Redistributions in binary form must reproduce the above copyright
 *      notice,  this list of  conditions and the following disclaimer in
 *      the  documentation  and/or  other   materials provided  with  the
 *      distribution.
 *
 * THIS  SOFTWARE IS PROVIDED BY  THE  COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND  ANY  EXPRESS OR IMPLIED  WARRANTIES,  INCLUDING,  BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES  OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR  PURPOSE ARE DISCLAIMED. IN  NO EVENT SHALL THE COPYRIGHT
 * HOLDERS OR      CONTRIBUTORS  BE LIABLE FOR   ANY    DIRECT, INDIRECT,
 * INCIDENTAL,  SPECIAL,  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF  SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN  CONTRACT, STRICT LIABILITY, OR
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE   OF THIS SOFTWARE, EVEN   IF ADVISED OF   THE POSSIBILITY OF SUCH
 * DAMAGE.
 */


/**
 **  Emission/reception de la requete d'execution icuTestFacerecSetPeople
 **/

static BOOL icuTestFacerecSetPeople (TEST_STR *testId, int rqstNum, 
				    int acti, BOOL silent)
{
  int *inputDims = NULL;
  

  /* Saisie donnes */
  if (!TEST_ACTIVITY_ON(testId, acti)) {
    printf ("-- Enter GEN_STRING256 recoName:\n");
    scan_struct_GEN_STRING256(stdin, stdout, (GEN_STRING256 *)TEST_RQST_INPUT(testId,rqstNum), 1, 0, inputDims);
  }
  
  /* Emission d'une requete + replique intermediaire 
     ET/OU reception replique finale (selon TEST_ACTIVITY_RQST_ID) */
  if (!testSendAndOrRcvExecRqst(testId, rqstNum, acti, silent))
    return FALSE;

  
  return TRUE;
}



/* 
 * Copyright (c) 2005 LAAS/CNRS
 * All rights reserved.
 *
 * Redistribution and use  in source  and binary  forms,  with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *   1. Redistributions of  source  code must retain the  above copyright
 *      notice, this list of conditions and the following disclaimer.
 *   2. Redistributions in binary form must reproduce the above copyright
 *      notice,  this list of  conditions and the following disclaimer in
 *      the  documentation  and/or  other   materials provided  with  the
 *      distribution.
 *
 * THIS  SOFTWARE IS PROVIDED BY  THE  COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND  ANY  EXPRESS OR IMPLIED  WARRANTIES,  INCLUDING,  BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES  OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR  PURPOSE ARE DISCLAIMED. IN  NO EVENT SHALL THE COPYRIGHT
 * HOLDERS OR      CONTRIBUTORS  BE LIABLE FOR   ANY    DIRECT, INDIRECT,
 * INCIDENTAL,  SPECIAL,  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF  SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN  CONTRACT, STRICT LIABILITY, OR
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE   OF THIS SOFTWARE, EVEN   IF ADVISED OF   THE POSSIBILITY OF SUCH
 * DAMAGE.
 */


/**
 **  Emission/reception de la requete d'execution icuTestFacerecRecognizeAll
 **/

static BOOL icuTestFacerecRecognizeAll (TEST_STR *testId, int rqstNum, 
				    int acti, BOOL silent)
{
  
  

  /* Saisie donnes */
  if (!TEST_ACTIVITY_ON(testId, acti)) {
    
  }
  
  /* Emission d'une requete + replique intermediaire 
     ET/OU reception replique finale (selon TEST_ACTIVITY_RQST_ID) */
  if (!testSendAndOrRcvExecRqst(testId, rqstNum, acti, silent))
    return FALSE;

  
  return TRUE;
}



/* 
 * Copyright (c) 2005 LAAS/CNRS
 * All rights reserved.
 *
 * Redistribution and use  in source  and binary  forms,  with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *   1. Redistributions of  source  code must retain the  above copyright
 *      notice, this list of conditions and the following disclaimer.
 *   2. Redistributions in binary form must reproduce the above copyright
 *      notice,  this list of  conditions and the following disclaimer in
 *      the  documentation  and/or  other   materials provided  with  the
 *      distribution.
 *
 * THIS  SOFTWARE IS PROVIDED BY  THE  COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND  ANY  EXPRESS OR IMPLIED  WARRANTIES,  INCLUDING,  BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES  OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR  PURPOSE ARE DISCLAIMED. IN  NO EVENT SHALL THE COPYRIGHT
 * HOLDERS OR      CONTRIBUTORS  BE LIABLE FOR   ANY    DIRECT, INDIRECT,
 * INCIDENTAL,  SPECIAL,  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF  SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN  CONTRACT, STRICT LIABILITY, OR
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE   OF THIS SOFTWARE, EVEN   IF ADVISED OF   THE POSSIBILITY OF SUCH
 * DAMAGE.
 */


/**
 **  Emission/reception de la requete d'execution icuTestFacerecLoadSomebody
 **/

static BOOL icuTestFacerecLoadSomebody (TEST_STR *testId, int rqstNum, 
				    int acti, BOOL silent)
{
  int *inputDims = NULL;
  

  /* Saisie donnes */
  if (!TEST_ACTIVITY_ON(testId, acti)) {
    printf ("-- Enter GEN_STRING256 recoName:\n");
    scan_struct_GEN_STRING256(stdin, stdout, (GEN_STRING256 *)TEST_RQST_INPUT(testId,rqstNum), 1, 0, inputDims);
  }
  
  /* Emission d'une requete + replique intermediaire 
     ET/OU reception replique finale (selon TEST_ACTIVITY_RQST_ID) */
  if (!testSendAndOrRcvExecRqst(testId, rqstNum, acti, silent))
    return FALSE;

  
  return TRUE;
}



/* 
 * Copyright (c) 2005 LAAS/CNRS
 * All rights reserved.
 *
 * Redistribution and use  in source  and binary  forms,  with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *   1. Redistributions of  source  code must retain the  above copyright
 *      notice, this list of conditions and the following disclaimer.
 *   2. Redistributions in binary form must reproduce the above copyright
 *      notice,  this list of  conditions and the following disclaimer in
 *      the  documentation  and/or  other   materials provided  with  the
 *      distribution.
 *
 * THIS  SOFTWARE IS PROVIDED BY  THE  COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND  ANY  EXPRESS OR IMPLIED  WARRANTIES,  INCLUDING,  BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES  OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR  PURPOSE ARE DISCLAIMED. IN  NO EVENT SHALL THE COPYRIGHT
 * HOLDERS OR      CONTRIBUTORS  BE LIABLE FOR   ANY    DIRECT, INDIRECT,
 * INCIDENTAL,  SPECIAL,  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF  SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN  CONTRACT, STRICT LIABILITY, OR
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE   OF THIS SOFTWARE, EVEN   IF ADVISED OF   THE POSSIBILITY OF SUCH
 * DAMAGE.
 */


/**
 **  Emission/reception de la requete d'execution icuTestFacerecUnloadSomebody
 **/

static BOOL icuTestFacerecUnloadSomebody (TEST_STR *testId, int rqstNum, 
				    int acti, BOOL silent)
{
  int *inputDims = NULL;
  

  /* Saisie donnes */
  if (!TEST_ACTIVITY_ON(testId, acti)) {
    printf ("-- Enter GEN_STRING256 recoName:\n");
    scan_struct_GEN_STRING256(stdin, stdout, (GEN_STRING256 *)TEST_RQST_INPUT(testId,rqstNum), 1, 0, inputDims);
  }
  
  /* Emission d'une requete + replique intermediaire 
     ET/OU reception replique finale (selon TEST_ACTIVITY_RQST_ID) */
  if (!testSendAndOrRcvExecRqst(testId, rqstNum, acti, silent))
    return FALSE;

  
  return TRUE;
}



/* 
 * Copyright (c) 2005 LAAS/CNRS
 * All rights reserved.
 *
 * Redistribution and use  in source  and binary  forms,  with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *   1. Redistributions of  source  code must retain the  above copyright
 *      notice, this list of conditions and the following disclaimer.
 *   2. Redistributions in binary form must reproduce the above copyright
 *      notice,  this list of  conditions and the following disclaimer in
 *      the  documentation  and/or  other   materials provided  with  the
 *      distribution.
 *
 * THIS  SOFTWARE IS PROVIDED BY  THE  COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND  ANY  EXPRESS OR IMPLIED  WARRANTIES,  INCLUDING,  BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES  OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR  PURPOSE ARE DISCLAIMED. IN  NO EVENT SHALL THE COPYRIGHT
 * HOLDERS OR      CONTRIBUTORS  BE LIABLE FOR   ANY    DIRECT, INDIRECT,
 * INCIDENTAL,  SPECIAL,  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF  SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN  CONTRACT, STRICT LIABILITY, OR
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE   OF THIS SOFTWARE, EVEN   IF ADVISED OF   THE POSSIBILITY OF SUCH
 * DAMAGE.
 */


/**
 **  Emission/reception de la requete d'execution icuTestFacerecLoadEverybody
 **/

static BOOL icuTestFacerecLoadEverybody (TEST_STR *testId, int rqstNum, 
				    int acti, BOOL silent)
{
  
  

  /* Saisie donnes */
  if (!TEST_ACTIVITY_ON(testId, acti)) {
    
  }
  
  /* Emission d'une requete + replique intermediaire 
     ET/OU reception replique finale (selon TEST_ACTIVITY_RQST_ID) */
  if (!testSendAndOrRcvExecRqst(testId, rqstNum, acti, silent))
    return FALSE;

  
  return TRUE;
}



/* 
 * Copyright (c) 2005 LAAS/CNRS
 * All rights reserved.
 *
 * Redistribution and use  in source  and binary  forms,  with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *   1. Redistributions of  source  code must retain the  above copyright
 *      notice, this list of conditions and the following disclaimer.
 *   2. Redistributions in binary form must reproduce the above copyright
 *      notice,  this list of  conditions and the following disclaimer in
 *      the  documentation  and/or  other   materials provided  with  the
 *      distribution.
 *
 * THIS  SOFTWARE IS PROVIDED BY  THE  COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND  ANY  EXPRESS OR IMPLIED  WARRANTIES,  INCLUDING,  BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES  OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR  PURPOSE ARE DISCLAIMED. IN  NO EVENT SHALL THE COPYRIGHT
 * HOLDERS OR      CONTRIBUTORS  BE LIABLE FOR   ANY    DIRECT, INDIRECT,
 * INCIDENTAL,  SPECIAL,  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF  SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN  CONTRACT, STRICT LIABILITY, OR
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE   OF THIS SOFTWARE, EVEN   IF ADVISED OF   THE POSSIBILITY OF SUCH
 * DAMAGE.
 */


/**
 **  Emission/reception de la requete d'execution icuTestFacerecUnloadEverybody
 **/

static BOOL icuTestFacerecUnloadEverybody (TEST_STR *testId, int rqstNum, 
				    int acti, BOOL silent)
{
  
  

  /* Saisie donnes */
  if (!TEST_ACTIVITY_ON(testId, acti)) {
    
  }
  
  /* Emission d'une requete + replique intermediaire 
     ET/OU reception replique finale (selon TEST_ACTIVITY_RQST_ID) */
  if (!testSendAndOrRcvExecRqst(testId, rqstNum, acti, silent))
    return FALSE;

  
  return TRUE;
}



/* 
 * Copyright (c) 2005 LAAS/CNRS
 * All rights reserved.
 *
 * Redistribution and use  in source  and binary  forms,  with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *   1. Redistributions of  source  code must retain the  above copyright
 *      notice, this list of conditions and the following disclaimer.
 *   2. Redistributions in binary form must reproduce the above copyright
 *      notice,  this list of  conditions and the following disclaimer in
 *      the  documentation  and/or  other   materials provided  with  the
 *      distribution.
 *
 * THIS  SOFTWARE IS PROVIDED BY  THE  COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND  ANY  EXPRESS OR IMPLIED  WARRANTIES,  INCLUDING,  BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES  OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR  PURPOSE ARE DISCLAIMED. IN  NO EVENT SHALL THE COPYRIGHT
 * HOLDERS OR      CONTRIBUTORS  BE LIABLE FOR   ANY    DIRECT, INDIRECT,
 * INCIDENTAL,  SPECIAL,  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF  SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN  CONTRACT, STRICT LIABILITY, OR
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE   OF THIS SOFTWARE, EVEN   IF ADVISED OF   THE POSSIBILITY OF SUCH
 * DAMAGE.
 */


/**
 **  Emission/reception de la requete d'execution icuTestWaitForModalityChange
 **/

static BOOL icuTestWaitForModalityChange (TEST_STR *testId, int rqstNum, 
				    int acti, BOOL silent)
{
  
  

  /* Saisie donnes */
  if (!TEST_ACTIVITY_ON(testId, acti)) {
    
  }
  
  /* Emission d'une requete + replique intermediaire 
     ET/OU reception replique finale (selon TEST_ACTIVITY_RQST_ID) */
  if (!testSendAndOrRcvExecRqst(testId, rqstNum, acti, silent))
    return FALSE;

  
  return TRUE;
}



/* 
 * Copyright (c) 2005 LAAS/CNRS
 * All rights reserved.
 *
 * Redistribution and use  in source  and binary  forms,  with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *   1. Redistributions of  source  code must retain the  above copyright
 *      notice, this list of conditions and the following disclaimer.
 *   2. Redistributions in binary form must reproduce the above copyright
 *      notice,  this list of  conditions and the following disclaimer in
 *      the  documentation  and/or  other   materials provided  with  the
 *      distribution.
 *
 * THIS  SOFTWARE IS PROVIDED BY  THE  COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND  ANY  EXPRESS OR IMPLIED  WARRANTIES,  INCLUDING,  BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES  OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR  PURPOSE ARE DISCLAIMED. IN  NO EVENT SHALL THE COPYRIGHT
 * HOLDERS OR      CONTRIBUTORS  BE LIABLE FOR   ANY    DIRECT, INDIRECT,
 * INCIDENTAL,  SPECIAL,  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF  SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN  CONTRACT, STRICT LIABILITY, OR
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE   OF THIS SOFTWARE, EVEN   IF ADVISED OF   THE POSSIBILITY OF SUCH
 * DAMAGE.
 */


/**
 **  Emission/reception de la requete d'execution icuTestLaunchPlatine
 **/

static BOOL icuTestLaunchPlatine (TEST_STR *testId, int rqstNum, 
				    int acti, BOOL silent)
{
  int *inputDims = NULL;
  

  /* Saisie donnes */
  if (!TEST_ACTIVITY_ON(testId, acti)) {
    printf ("-- Enter ICU_PLATINE_PARAMS platineParams:\n");
    scan_struct_ICU_PLATINE_PARAMS(stdin, stdout, (ICU_PLATINE_PARAMS *)TEST_RQST_INPUT(testId,rqstNum), 1, 0, inputDims);
  }
  
  /* Emission d'une requete + replique intermediaire 
     ET/OU reception replique finale (selon TEST_ACTIVITY_RQST_ID) */
  if (!testSendAndOrRcvExecRqst(testId, rqstNum, acti, silent))
    return FALSE;

  
  return TRUE;
}



/* 
 * Copyright (c) 2005 LAAS/CNRS
 * All rights reserved.
 *
 * Redistribution and use  in source  and binary  forms,  with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *   1. Redistributions of  source  code must retain the  above copyright
 *      notice, this list of conditions and the following disclaimer.
 *   2. Redistributions in binary form must reproduce the above copyright
 *      notice,  this list of  conditions and the following disclaimer in
 *      the  documentation  and/or  other   materials provided  with  the
 *      distribution.
 *
 * THIS  SOFTWARE IS PROVIDED BY  THE  COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND  ANY  EXPRESS OR IMPLIED  WARRANTIES,  INCLUDING,  BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES  OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR  PURPOSE ARE DISCLAIMED. IN  NO EVENT SHALL THE COPYRIGHT
 * HOLDERS OR      CONTRIBUTORS  BE LIABLE FOR   ANY    DIRECT, INDIRECT,
 * INCIDENTAL,  SPECIAL,  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF  SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN  CONTRACT, STRICT LIABILITY, OR
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE   OF THIS SOFTWARE, EVEN   IF ADVISED OF   THE POSSIBILITY OF SUCH
 * DAMAGE.
 */


/**
 **  Emission/reception de la requete d'execution icuTestCmdPtu
 **/

static BOOL icuTestCmdPtu (TEST_STR *testId, int rqstNum, 
				    int acti, BOOL silent)
{
  
  

  /* Saisie donnes */
  if (!TEST_ACTIVITY_ON(testId, acti)) {
    
  }
  
  /* Emission d'une requete + replique intermediaire 
     ET/OU reception replique finale (selon TEST_ACTIVITY_RQST_ID) */
  if (!testSendAndOrRcvExecRqst(testId, rqstNum, acti, silent))
    return FALSE;

  
  return TRUE;
}



/* 
 * Copyright (c) 2005 LAAS/CNRS
 * All rights reserved.
 *
 * Redistribution and use  in source  and binary  forms,  with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *   1. Redistributions of  source  code must retain the  above copyright
 *      notice, this list of conditions and the following disclaimer.
 *   2. Redistributions in binary form must reproduce the above copyright
 *      notice,  this list of  conditions and the following disclaimer in
 *      the  documentation  and/or  other   materials provided  with  the
 *      distribution.
 *
 * THIS  SOFTWARE IS PROVIDED BY  THE  COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND  ANY  EXPRESS OR IMPLIED  WARRANTIES,  INCLUDING,  BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES  OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR  PURPOSE ARE DISCLAIMED. IN  NO EVENT SHALL THE COPYRIGHT
 * HOLDERS OR      CONTRIBUTORS  BE LIABLE FOR   ANY    DIRECT, INDIRECT,
 * INCIDENTAL,  SPECIAL,  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF  SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN  CONTRACT, STRICT LIABILITY, OR
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE   OF THIS SOFTWARE, EVEN   IF ADVISED OF   THE POSSIBILITY OF SUCH
 * DAMAGE.
 */


/**
 **  Emission/reception de la requete d'execution icuTestIcuInitRFID
 **/

static BOOL icuTestIcuInitRFID (TEST_STR *testId, int rqstNum, 
				    int acti, BOOL silent)
{
  
  

  /* Saisie donnes */
  if (!TEST_ACTIVITY_ON(testId, acti)) {
    
  }
  
  /* Emission d'une requete + replique intermediaire 
     ET/OU reception replique finale (selon TEST_ACTIVITY_RQST_ID) */
  if (!testSendAndOrRcvExecRqst(testId, rqstNum, acti, silent))
    return FALSE;

  
  return TRUE;
}



/* 
 * Copyright (c) 2005 LAAS/CNRS
 * All rights reserved.
 *
 * Redistribution and use  in source  and binary  forms,  with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *   1. Redistributions of  source  code must retain the  above copyright
 *      notice, this list of conditions and the following disclaimer.
 *   2. Redistributions in binary form must reproduce the above copyright
 *      notice,  this list of  conditions and the following disclaimer in
 *      the  documentation  and/or  other   materials provided  with  the
 *      distribution.
 *
 * THIS  SOFTWARE IS PROVIDED BY  THE  COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND  ANY  EXPRESS OR IMPLIED  WARRANTIES,  INCLUDING,  BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES  OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR  PURPOSE ARE DISCLAIMED. IN  NO EVENT SHALL THE COPYRIGHT
 * HOLDERS OR      CONTRIBUTORS  BE LIABLE FOR   ANY    DIRECT, INDIRECT,
 * INCIDENTAL,  SPECIAL,  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF  SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN  CONTRACT, STRICT LIABILITY, OR
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE   OF THIS SOFTWARE, EVEN   IF ADVISED OF   THE POSSIBILITY OF SUCH
 * DAMAGE.
 */


/**
 **  Emission/reception de la requete d'execution icuTestIcuCloseRFID
 **/

static BOOL icuTestIcuCloseRFID (TEST_STR *testId, int rqstNum, 
				    int acti, BOOL silent)
{
  
  

  /* Saisie donnes */
  if (!TEST_ACTIVITY_ON(testId, acti)) {
    
  }
  
  /* Emission d'une requete + replique intermediaire 
     ET/OU reception replique finale (selon TEST_ACTIVITY_RQST_ID) */
  if (!testSendAndOrRcvExecRqst(testId, rqstNum, acti, silent))
    return FALSE;

  
  return TRUE;
}



/* 
 * Copyright (c) 2005 LAAS/CNRS
 * All rights reserved.
 *
 * Redistribution and use  in source  and binary  forms,  with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *   1. Redistributions of  source  code must retain the  above copyright
 *      notice, this list of conditions and the following disclaimer.
 *   2. Redistributions in binary form must reproduce the above copyright
 *      notice,  this list of  conditions and the following disclaimer in
 *      the  documentation  and/or  other   materials provided  with  the
 *      distribution.
 *
 * THIS  SOFTWARE IS PROVIDED BY  THE  COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND  ANY  EXPRESS OR IMPLIED  WARRANTIES,  INCLUDING,  BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES  OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR  PURPOSE ARE DISCLAIMED. IN  NO EVENT SHALL THE COPYRIGHT
 * HOLDERS OR      CONTRIBUTORS  BE LIABLE FOR   ANY    DIRECT, INDIRECT,
 * INCIDENTAL,  SPECIAL,  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF  SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN  CONTRACT, STRICT LIABILITY, OR
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE   OF THIS SOFTWARE, EVEN   IF ADVISED OF   THE POSSIBILITY OF SUCH
 * DAMAGE.
 */


/**
 **  Emission/reception de la requete d'execution icuTestIcuAcquireRFID
 **/

static BOOL icuTestIcuAcquireRFID (TEST_STR *testId, int rqstNum, 
				    int acti, BOOL silent)
{
  
  

  /* Saisie donnes */
  if (!TEST_ACTIVITY_ON(testId, acti)) {
    
  }
  
  /* Emission d'une requete + replique intermediaire 
     ET/OU reception replique finale (selon TEST_ACTIVITY_RQST_ID) */
  if (!testSendAndOrRcvExecRqst(testId, rqstNum, acti, silent))
    return FALSE;

  
  return TRUE;
}



/* 
 * Copyright (c) 2005 LAAS/CNRS
 * All rights reserved.
 *
 * Redistribution and use  in source  and binary  forms,  with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *   1. Redistributions of  source  code must retain the  above copyright
 *      notice, this list of conditions and the following disclaimer.
 *   2. Redistributions in binary form must reproduce the above copyright
 *      notice,  this list of  conditions and the following disclaimer in
 *      the  documentation  and/or  other   materials provided  with  the
 *      distribution.
 *
 * THIS  SOFTWARE IS PROVIDED BY  THE  COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND  ANY  EXPRESS OR IMPLIED  WARRANTIES,  INCLUDING,  BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES  OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR  PURPOSE ARE DISCLAIMED. IN  NO EVENT SHALL THE COPYRIGHT
 * HOLDERS OR      CONTRIBUTORS  BE LIABLE FOR   ANY    DIRECT, INDIRECT,
 * INCIDENTAL,  SPECIAL,  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF  SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN  CONTRACT, STRICT LIABILITY, OR
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE   OF THIS SOFTWARE, EVEN   IF ADVISED OF   THE POSSIBILITY OF SUCH
 * DAMAGE.
 */


/**
 **  Emission/reception de la requete d'execution icuTestIcuAcquireTag
 **/

static BOOL icuTestIcuAcquireTag (TEST_STR *testId, int rqstNum, 
				    int acti, BOOL silent)
{
  int *inputDims = NULL;
  

  /* Saisie donnes */
  if (!TEST_ACTIVITY_ON(testId, acti)) {
    printf ("-- Enter GEN_STRING256 recoTag:\n");
    scan_struct_GEN_STRING256(stdin, stdout, (GEN_STRING256 *)TEST_RQST_INPUT(testId,rqstNum), 1, 0, inputDims);
  }
  
  /* Emission d'une requete + replique intermediaire 
     ET/OU reception replique finale (selon TEST_ACTIVITY_RQST_ID) */
  if (!testSendAndOrRcvExecRqst(testId, rqstNum, acti, silent))
    return FALSE;

  
  return TRUE;
}

