/**
 ** icuRunTaskCodels.c
 **
 ** Codels appeles par la tache icuRunTask
 **
 ** Auteur: Aymeric Yvernes
** Date: 
 **
 **/

#include <portLib.h>
#include <stdio.h>
#include <h2timeLib.h>
#include <time.h>
#include "server/icuHeader.h"

/*this include is used to enable the modules to use POM*/
#include "libeuler/pomEuler.h"
//#include "cameraClient.h"
#include "interface.h"
#include "math.h"
#include "platine/platineConst.h"

#include <viam/viamStruct.h>
/*Partie pour le pilotage de la platine de l EVI et le calcul des positions des personnes*/
#include "evicamera.h"
#define CAMERA_PAN_OFFSET    50
#define CAMERA_TILT_OFFSET   0
#define CAMERA_MATRIX_INTRINSIC_SIZE 9

#define CAMERA_X 0
#define CAMERA_Y 0
#define CAMERA_Z 1.60



/** Lorsque la detection de visage fonctionne on calcule des heuristiques pour valider le suivi */

/** Historique des detections de visage le plus proche du tracker */
static double nearestFaceSize[TEMPORAL_WINDOW_SIZE];

/** Indice de parcours de la fenetre temporelle (0 indique aucune detection de visage depuis un moment) */
static int nearestFaceCpt = 0;

/** Somme des tailles de visage le plus proche */
static double nearestFaceSizeSum = 0;

/** Moyenne des echelles des visages detectes */
static double nearestFaceSizeMean = 0;

/* Compteur d'absence totale */
static int nobodySeen = 0;

static int waitForId[MAX_ACTIVITIES];

static IplImage* tabImg[10];

/***** CONSTANTES UTILISEES POUR SEUILLAGE  *****/

/** Demarrer le motion monitoring apres START_MOTION_MONITORING_AT images sans detections */
#define START_MOTION_MONITORING_AT 50

/** Seuil pour switcher entre le ICU_FACE_TRACKING et le ICU_BODY_TRACKING */
#define MOYENNE_ECHELLE_SEUIL 80

/** Nombre de detection minimum pour switcher dans une des deux modalites */
#define MINIMUM_DETECT_FOR_FACE_TRACKING 6
#define MINIMUM_DETECT_FOR_BODY_TRACKING 4

/** distance a l interieur de laquelle on va chercher des detections pour savoir si le trackeur est a l ouest */
#define DISTANCE_MAX_FOR_TRACKING_VERIFICATION 40.0
#define DISTANCE_MAX_FOR_TRACKING_VERIFICATION_CARRE 1600.0

/** Nombre d images au dela duquel on reinitialise le trackeur (si on a une detection globale) */
#define NB_IMAGE_SANS_DETECTION_LOCALE 20

/** Nombre minimum d images pendant lesquelles la camera ne bouge pas (en mode suivi de la camera) */
/** Pour eviter qu elle recadre a chaque image */
#define MINIMUM_IMAGES_WITHOUT_MOVING 3

/** Compteur du nombre d'images non centrees sur la cible */
static int notCorrect;

/** Flux video issu de la camera */
static IplImage * brut;
static IplImage * deBayer;
static IplImage * flux;

/** Chaine de caractere contenant le nom d'enregistrement de fichiers images */
static char pictureName[20];

/** Bloque le lancement de ProcessExec tant que l'affichage n'est pas initialise */
static int start=0;

/** Police de caractere pour affichage d'informations sur les images */
static CvFont font;


static int previousModality=0;




static double h_minDist = 9999;
static int h_min = -1;
static double h_motionRatio;
static double h_faceRatio;
static double h_faceMeanDist;
static const double h_faceThresholdMax = 0.5;
static const double h_faceThresholdMin = 0.4;
static const double h_motionThresholdMax = 0.5;
static const double h_motionThresholdMin = 0.2;

#define DHR 0.2

static int h_minDistWindowSum=0;
static int h_minDistWindow[TEMPORAL_WINDOW_SIZE];
static int h_minDistWindowId=0;
static const int h_ROI = 25000;
static int h_numberOfFaces=0;
static int h_minDistWindowSize = TEMPORAL_WINDOW_SIZE;
static int h_minDistWindowFull=0;
static int h_motionWindowSum=0;
static int h_motionWindow[TEMPORAL_WINDOW_SIZE];
static int h_motionWindowId=0;
static int h_numberOfMotion=0;
static int h_motionWindowSize=TEMPORAL_WINDOW_SIZE;
static int h_motionWindowFull=0;



/*CAMERA*/
static POSTER_ID VIAM_CAMERA_POSTER_ID;	/* current camera poster */
//static POSTER_ID cameraPosterId;
static ViamImageBank *VIAM_IMAGE;	/* current poster addr */
//static CameraPoster *cameraPoster;

//static CameraBank icuCameraBank;

//OpenCv image struct to buffer image place
static IplImage *IMAGE_SRC_L= NULL; 

// variables to be used when considering stereo system
static int VIAM_CAMERA_LEFT_ID;
static int VIAM_CAMERA_RIGHT_ID;

static char viamPosterName[255];

static unsigned char *bufferedCameraImage;
static int imageWidth, imageHeight;

//static CameraPPBayer bayerState;
double mIntrinsic[CAMERA_MATRIX_INTRINSIC_SIZE];

/*PLATINE*/
static POSTER_ID platinePosterId;
static PLATINE_STATES *platinePoster;
static double tmpPanPlatine[1000];
int iPan = 0;
int platineOn = 0;
int rfOn = 0;

/*RFID*/
static POSTER_ID rfidPosterId;
static RFID_TAG_LIST *rfidPoster;
static RFID_TAG_LIST tagList;

/*------------------------------------------------------------------------
 *
 * icuRunTaskInit  --  Initialization codel (fIDS, ...)
 *
 * Description: 
 * 
 * Returns:    OK or ERROR
 */

STATUS icuRunTaskInit(int *bilan)
{

	printf("--> ICU : in icuRunTaskInit\n");

	/** Initialisation des parametres d'affichage par defaut */
	SDI_F->display.display = 0;
	SDI_F->display.width = ICU_DEFAULT_DISPLAY_WIDTH;
	SDI_F->display.height = ICU_DEFAULT_DISPLAY_HEIGHT;
	SDI_F->display.posx = ICU_DEFAULT_DISPLAY_VERTI;
	SDI_F->display.posy = ICU_DEFAULT_DISPLAY_HORIZ;

	/** Allocation des detecteurs et trackers */
	Process_create(SDI_F->display.width,SDI_F->display.height);

	/** Mise a 0 des elements de calcul des heuristiques */
	nearestFaceCpt = 0;
	nearestFaceSizeSum = 0;
	nearestFaceSizeMean = 0;
	memset(&nearestFaceSize,0,sizeof(double)*TEMPORAL_WINDOW_SIZE);

	/** Activation du poster et mise a 0 des parametres de sortie */
	SDI_F->displayHRDistance = 0;
	memset(&SDI_F->humanPos, 0, sizeof(ICU_HUMAN_POS));
//SDI_F->humanPos.cap = 0;
	memset(&SDI_F->results, 0, sizeof(ICU_TRACKING_VALUES));

	/** Mise a 0 du compteur d'images non centrees sur la cible */
	notCorrect=0;

	/** Initialisation des parametres du process */
	SDI_F->autoSwitchControl.autoSwitchMode = 0;
	SDI_F->autoSwitchControl.isRobotMoving = 0;

	/** Initialisation pour la reconaissance */
	SDI_F->recoId = 0;

/* Laisser a 0 pour ne pas entrer dans l'ajout de visage tant que la requete
	FaceRecAddSomebody n'a pas ete appele */
	SDI_F->recoAddSomebody = 0;

	/* Charge la liste des personnes connues sur disque */
	Facerec_knownList(&(SDI_F->knownPersonList));

	/* Charge la liste des personnes connues en memoire */
	Facerec_whoLoaded(&(SDI_F->loadedPersonList));

	/* flag indiquant un changement dans la base */
	SDI_F->personListChanged = 0;

	/* Seuil de proba pour reco ou absence */
	SDI_F->paramsPersonList.seuilReco = 0.3;

	/* Fenetre d'observation de l'absence */
	SDI_F->paramsPersonList.absenceWindow = TEMPORAL_WINDOW_SIZE;

	/* Fenetre d'observation de la presence */
	SDI_F->paramsPersonList.presenceWindow = TEMPORAL_WINDOW_SIZE;

	/* Ratio A/P de decision */
	SDI_F->paramsPersonList.absenceRatio = 0.8;

	/* Ratio P/A de decision */
	SDI_F->paramsPersonList.presenceRatio = 0.3;


	/** Init autoswitch mode */
	h_minDist = 9999;
	h_min = -1;
	h_minDistWindowSum=0;
	memset(h_minDistWindow,0,sizeof(int)*TEMPORAL_WINDOW_SIZE);
	h_minDistWindowId=0;
	h_numberOfFaces=0;
	h_minDistWindowSize = TEMPORAL_WINDOW_SIZE;
	h_minDistWindowFull=0;
	h_motionWindowSum=0;
	memset(h_motionWindow,0,sizeof(int)*TEMPORAL_WINDOW_SIZE);
	h_motionWindowId=0;
	h_numberOfMotion=0;
	h_motionWindowSize=TEMPORAL_WINDOW_SIZE;
	h_motionWindowFull=0;

// Initialisation de la platine de la camera (controle du mouvement)
/*EviCamera_WBAuto();
*/
	tagList.nbTags = 0;

//cvNamedWindow("Result", 1);
	return OK;
}


/*------------------------------------------------------------------------
 *
 * icuRunTaskEnd  --  Termination codel
 *
 * Description: 
 * 
 * Returns:    OK or ERROR
 */

STATUS icuRunTaskEnd(void)
{
	printf("in icuRunTaskEnd\n");


	/** Liberation de la video */
	//Video_delete();
	/** Destruction des process */
	Process_delete_all();

	/** Destruction de l'afficheur */
	cvDestroyWindow("ICU");
	//Display_delete();

	return OK;
}


/*------------------------------------------------------------------------
 *
 * icuRunTaskExec  --  1st codel of permanent activity (called BEFORE other activities)
 *
 * Description: 
 * 
 * Reports:      OK
 * 
 * Returns:    OK or ERROR
 */

STATUS icuRunTaskExec(int *bilan)
{
	//printf("icuRunTaskExec\n");
	unsigned char *src, *dst;
	/* Teste si le traitement de la tache peut etre realise (Init Display fait) */
	if(!start) return OK;

	/* Get the image */

	//flux = Video_getImage(NO_FLIP);

	printf("viamPosterId : %x\n", VIAM_CAMERA_POSTER_ID);
	
	if(VIAM_CAMERA_POSTER_ID != NULL)
	if (posterTake(VIAM_CAMERA_POSTER_ID, POSTER_READ) != OK)
	{
		*bilan = errnoGet();
		printf("icuRunTaskExec : posterTake\n");
		return ERROR;
	}
	//printf("CameraPoster taken\n");
	//Pour JIDO
	//SDI_F->humanPos.sensorPos = cameraPoster->image.stereo.pomTag[CAMERA_POSTER_LEFT];
	//memcpy(bufferedCameraImage, cameraPoster->image.stereo.pixel, imageWidth*imageHeight*4*sizeof(char));

	//Pour RACKHAM
	SDI_F->humanPos.sensorPos = VIAM_IMAGE->image[0].pos;

	memcpy(deBayer->imageData, VIAM_IMAGE->image[0].data + VIAM_IMAGE->image[0].dataOffset, VIAM_IMAGE->image[0].imageSize);
	
	posterGive(VIAM_CAMERA_POSTER_ID);

	cvResize(deBayer, flux, CV_INTER_LINEAR);//On redimensionne (ou on copie si c'est la même taille)*/
	/*if(bayerState == CAMERA_BAYER_NIL)
	{
		printf("icuRunTaskExec : bayerState\n");
		return ERROR;
	}*/
/*	src = bufferedCameraImage;
	dst = (unsigned char *)deBayer->imageData;
	dst = 	
	for(int i=0; i<imageHeight*imageWidth; i++) 
	{
		//les images opencv prennent du BGR et non pas du RGBA
		*(dst++) = src[2];
		*(dst++) = src[1];
		*(dst++) = src[0];
		src += 4*sizeof(char);
	}*/
	//cvNamedWindow("Test", 0);
	//cvShowImage("Test", deBayer);
	
	
	/******************************************************************/
	/*     Recuperation de la position angulaire de la camera         */
	/******************************************************************/
	double alphaCam = 0.0;
	if(platineOn)
	{
		//printf("platinePoster taken\n");
		if (posterTake(platinePosterId, POSTER_READ) == OK) 
		{
			if(platinePoster->stateDeg.pos.pan >=0)
				alphaCam = 180 - platinePoster->stateDeg.pos.pan;
			else
				alphaCam = -(180 + platinePoster->stateDeg.pos.pan);
			//alphaCam = -platinePoster->stateDeg.pos.pan; //why (-ve) ???
			posterGive(platinePosterId);
		}
	}
	printf(" ALPHACAM = %f \n\n\n\n",alphaCam);
	fflush(stdout);
	/******************************************************************/
	/*     Recuperation des donnees RFID					          */
	/******************************************************************/
	if(rfOn)
	{
		//printf("Detected tags : %d\n", tagList.nbTags);
		for(int i=0;i<tagList.nbTags;i++)
		{
			printf("Tag[%d] : (%lf, %lf, %lf, %lf)\n", i, 
				tagList.tags[i].angleMean, 
				tagList.tags[i].angleCov, 
				tagList.tags[i].distMean, 
				tagList.tags[i].distCov);
		}
	}
	
	/*******************************************/
	/*     Save one image if necessary         */
	/*******************************************/
	if(SDI_F->captureImage==1)
	{
		printf("Saving snapshot.jpg\n");
		Display_saveImage("snapshot.jpg",flux);
		SDI_F->captureImage=0;
	}


	/**********************************************************/
	/*               Process the algorithm                    */
	/**********************************************************/

	SDI_F->results.target.x = 0;
	SDI_F->results.target.y = 0;
	/* Traitement principal du tracker / detecteur */
	//printf("PlatineOn (%d), alphaCam (%lf)\n", platineOn, alphaCam);
	
	clock_t start_time, end_time;
	start_time = clock();
	Process_step(flux,NULL,&SDI_F->results, alphaCam, &tagList);
	end_time = clock();
	double diff = (double)(end_time - start_time) /(double)CLOCKS_PER_SEC; // en s
	printf("icu::Process_step\t processing time : %lf ms\n", diff*1000.0);
	//printf("Process_step OK\n");

	/**********************************************************/
	/*               Learning Somebody face                   */
	/**********************************************************/

	/* Si on est en cours d'apprentissage on poursuit uniquement l'enregistrement 
	des visages dans la base */
	if(SDI_F->recoAddSomebody)
	{
		printf("--> icuRunTaskCodels \t:\t adding face in %s base\n",SDI_F->recoAddName.name);

		/** Ajout de la personne courante dans la base */
		if(Facerec_addSomebody(flux,SDI_F->recoAddName.name,(ICU_NB_FACES_IN_RECO_DATABASE-SDI_F->recoAddSomebody)))
		{
			printf("--> icuRunTaskCodels \t:\t Face number %d added\n",(ICU_NB_FACES_IN_RECO_DATABASE-SDI_F->recoAddSomebody));
	
			/** On passe a l'ajout suivant */
			SDI_F->recoAddSomebody--;
		}
	}

	/*****************************************************************************************************************/
	/* Mise a jour de la table de reco */

	/* On parcours la liste des visages et on increment le compteur associe a la personne reconnue */
	/* Si le compteur atteind un certain seuil (nombre consecutif de reco suffisant) alors on indique la presence */
	/* de la personne on calcule la proba moyenne et on met a jour le flag de mise a jour de la table */
	/* L'idee c de comptabiliser la presence lorsque la personne est consideree absente et comptabiliser l'absence */
	/* losrqu'elle est consideree presente de facon a detecter un changement d'etat */

	//on oublie l etat precedent de tout le monde
	for(int i=0;i<SDI_F->loadedPersonList.nb;i++)
	{
		SDI_F->loadedPersonList.who[i].cptAP-= SDI_F->loadedPersonList.who[i].window[SDI_F->loadedPersonList.who[i].curid];
		SDI_F->loadedPersonList.who[i].cumul-= SDI_F->loadedPersonList.who[i].reliability[SDI_F->loadedPersonList.who[i].curid];
		SDI_F->loadedPersonList.who[i].window[SDI_F->loadedPersonList.who[i].curid] = 0;
		SDI_F->loadedPersonList.who[i].reliability[SDI_F->loadedPersonList.who[i].curid] = 0.0;
	}

	for(int i=0;i<SDI_F->results.nbDetectedFaces;i++)
	{
		//On recupere l identifiant de la personne
		int ind = Facerec_getId(&(SDI_F->loadedPersonList),SDI_F->results.reco[i].name, SDI_F->results.reco[i].state);
		
		// Si elle est reconnue avec une vraisemblance suffisante on compte une presence
		if(SDI_F->results.reco[i].likelihood>SDI_F->paramsPersonList.seuilReco || ind == 0)
		{
			
			// On signale qu elle est presente
			SDI_F->loadedPersonList.who[ind].window[SDI_F->loadedPersonList.who[ind].curid]=1;
			
			// On enregistre la proba de presence
			SDI_F->loadedPersonList.who[ind].reliability[SDI_F->loadedPersonList.who[ind].curid]=(ind==0)?SDI_F->paramsPersonList.seuilReco : SDI_F->results.reco[i].likelihood;
			
			//On signale qu il a ete mis ajour
			SDI_F->loadedPersonList.who[ind].updated=1;
		}
		
		// Calcul de la distance min avec le tracker pour le switch auto
		double h_curDist=0;
		h_min = i;
		if(SDI_F->currentModality!=ICU_AUTO_SWITCH_MODE) 
			h_curDist = pow(SDI_F->results.faces[i].x-SDI_F->results.target.x,2)+ 
				    pow(SDI_F->results.faces[i].y-SDI_F->results.target.y,2);

		// Teste si visage plus proche
		if(h_curDist < h_minDist)
		{
			h_minDist = h_curDist;
			h_min = i;
		}
	}

	for(int i=0;i<SDI_F->loadedPersonList.nb;i++)
	{
		// on ajoute les nouvelles donnees
		SDI_F->loadedPersonList.who[i].cptAP+= SDI_F->loadedPersonList.who[i].window[SDI_F->loadedPersonList.who[i].curid];
		SDI_F->loadedPersonList.who[i].cumul+= SDI_F->loadedPersonList.who[i].reliability[SDI_F->loadedPersonList.who[i].curid];
	}

	//Mis a jour des valeurs de presence et d absence et signalement si necessaire

	// Init du flag de changement
	SDI_F->personListChanged = 0;

	for(int i=0;i<SDI_F->loadedPersonList.nb;i++)
	{
		//On incremente le curseur en tenant compte de la taille de la fenetre en cours (absence ou presence)
		if(SDI_F->loadedPersonList.who[i].present <= 0)
		{
			SDI_F->loadedPersonList.who[i].curid = (SDI_F->loadedPersonList.who[i].curid+1) % SDI_F->paramsPersonList.presenceWindow;
		}
		else
		{
			SDI_F->loadedPersonList.who[i].curid = (SDI_F->loadedPersonList.who[i].curid+1) % SDI_F->paramsPersonList.absenceWindow;
		}
		//Si la personne est presente assez longtemps
#ifdef VERBOSE
		printf("%s : %d, %d, cur:%d\n",SDI_F->loadedPersonList.who[i].name, SDI_F->loadedPersonList.who[i].present, SDI_F->loadedPersonList.who[i].cptAP, SDI_F->loadedPersonList.who[i].curid);
#endif
		if((double)SDI_F->loadedPersonList.who[i].cptAP/(double)SDI_F->paramsPersonList.presenceWindow >= SDI_F->paramsPersonList.presenceRatio && SDI_F->loadedPersonList.who[i].present != 1)
		{
			//On signale sa presence
			SDI_F->loadedPersonList.who[i].present = 1;
			//On mets a jour sa vraissemblance
			SDI_F->loadedPersonList.who[i].likelihood = SDI_F->loadedPersonList.who[i].cumul / SDI_F->paramsPersonList.presenceWindow;
			printf("PersonList : %s is present\n", SDI_F->loadedPersonList.who[i].name);
			strcpy(SDI_F->waitFor.name, SDI_F->loadedPersonList.who[i].name);
			SDI_F->personListChanged = 1;
		}
		//Si la personne est absente assez longtemps
		else 
		{
			if(1.0-((double)SDI_F->loadedPersonList.who[i].cptAP/(double)SDI_F->paramsPersonList.absenceWindow) >= SDI_F->paramsPersonList.absenceRatio && SDI_F->loadedPersonList.who[i].present != -1)
			{
				SDI_F->loadedPersonList.who[i].present = -1;
	
				//On mets a jour sa vraissemblance
				SDI_F->loadedPersonList.who[i].likelihood = SDI_F->loadedPersonList.who[i].cumul / SDI_F->paramsPersonList.absenceWindow;
				printf("PersonList : %s is not present anymore\n", SDI_F->loadedPersonList.who[i].name);
				strcpy(SDI_F->waitFor.name, SDI_F->loadedPersonList.who[i].name);
				SDI_F->personListChanged = 1;
			}
		}
	
		/*if(i!=0)
		{
			cvSetImageROI(flux, cvRect((i-1)*40, 200, 40, 40));
			cvResize(tabImg[i-1], flux, CV_INTER_LINEAR);
			cvResetImageROI(flux);
			if(SDI_F->loadedPersonList.who[i].present == 1)
			{
				cvRectangle(flux, cvPoint((i-1)*40+3, 203), cvPoint(i*40-3, 237), CV_RGB(0, 255, 0), 3);
			}
		}*/
	}

	/* Compte si personne detecte */
	if(!SDI_F->results.motionDetected && !SDI_F->results.nbDetectedFaces)
		nobodySeen++;
	else
		nobodySeen=0;

	/**********************************************************************************************************/
	/*                    N O U V E A U   A U T O S W I T C H    M O D E                                      */
	/**********************************************************************************************************/
	if(SDI_F->autoSwitchControl.autoSwitchMode && Process_usingFaceDetector())
	{
		/* Teste si le visage le plus proche est suffisament proche :
				- si oui on stocke la distance dans la fenetre temporelle et on indique la presence de qqun
				- si non on compte une absence
		*/

		/***********************************************************************************/
		/* Fenetre de visages */
		/***********************************************************************************/

		/* on retire l'ancienne valeur */
		h_minDistWindowSum -= h_minDistWindow[h_minDistWindowId];

		if(SDI_F->results.nbDetectedFaces && h_minDist<h_ROI) 
		{
			/* On ajoute la nouvelle valeur */

			// h_minDistWindow[h_minDistWindowId] = SDI_F->results.faces[h_min].width; 
			/////////////JIDO
			//h_minDistWindow[h_minDistWindowId] = (int)(-0.079*SDI_F->results.faces[h_min].width+4.5);
			/////////////RACKHAM
			h_minDistWindow[h_minDistWindowId] = (int)(-0.0145*SDI_F->results.faces[h_min].width+2.3451);
		
			if(h_numberOfFaces<h_minDistWindowSize)
				h_numberOfFaces++;
		}
		else 
		{
			if(h_minDistWindow[h_minDistWindowId] && h_numberOfFaces)
				h_numberOfFaces--;
			/* On met a 0 la case */
			h_minDistWindow[h_minDistWindowId] = 0;
		}
		
		/* On cumule les tailles */
		h_minDistWindowSum += h_minDistWindow[h_minDistWindowId];

		/* Indique que la fenetre temporelle est remplie */
		if(!h_minDistWindowFull && h_minDistWindowId==h_minDistWindowSize-1)
			h_minDistWindowFull = 1;

		/* On passe a la case suivante */
		h_minDistWindowId = (h_minDistWindowId + 1) %  h_minDistWindowSize;

		/***********************************************************************************/
		/* Fenetre de mouvement */
		/***********************************************************************************/

		if(SDI_F->results.motionDetected) 
		{
			if(h_numberOfMotion<h_motionWindowSize)
			{
				h_numberOfMotion++;
			}
		}
		else 
		{
			if(h_numberOfMotion)
				h_numberOfMotion--;
		}

		/* Indique que la fenetre temporelle de mouvement est remplie */
		if(!h_motionWindowFull && h_motionWindowId==h_motionWindowSize-1)
			h_motionWindowFull = 1;

		/* On passe a la case suivante */
		h_motionWindowId = (h_motionWindowId + 1) %  h_motionWindowSize;

		/***********************************************************************************/
		/* Decisions de switch */
		/***********************************************************************************/

		h_faceRatio = (double)h_numberOfFaces/(double)h_minDistWindowSize;
		h_faceMeanDist = 0;
		if(h_numberOfFaces)
			h_faceMeanDist = (double)h_minDistWindowSum/(double)h_numberOfFaces;
		
		h_motionRatio = (double)h_numberOfMotion/(double)h_motionWindowSize;

		switch(SDI_F->currentModality)
		{
			case ICU_AUTO_SWITCH_MODE: 
			{
				if(h_motionRatio>h_motionThresholdMax) 
				{
					Process_setModality(ICU_MOTION_MONITORING,1);
				}
				break;
			}
			case ICU_FACE_TRACKING: 
			{
				if(h_faceRatio>h_faceThresholdMax) 
				{
					if (h_faceMeanDist>DHR+0.5) 
					{
						Process_setModality(ICU_BODY_TRACKING,1);
					}
				}
				else 
				{
					Process_setModality(ICU_AUTO_SWITCH_MODE,1);
				}
				break;
			}
			case ICU_BODY_TRACKING: 
			{
				if(h_faceRatio>h_faceThresholdMax) 
				{
					if(h_faceMeanDist<DHR-0.5)
						Process_setModality(ICU_FACE_TRACKING,1);
				} 
				//else if(h_motionRatio>h_motionThresholdMax) {
				else 
				{
					if(h_motionRatio>h_motionThresholdMin)
					{
						Process_setModality(ICU_MOTION_MONITORING,1);
					}
					else
					{
						Process_setModality(ICU_AUTO_SWITCH_MODE,1);
					}
				}
				break;
			}
			case ICU_MOTION_MONITORING: 
			{
				if(h_faceRatio>h_faceThresholdMax) 
				{
					Process_setModality(ICU_BODY_TRACKING,1);
				}
				else 
				{
					/* Sinon si la proportion de visages dans la fenetre est trop faible alors on repasse en init */
					if(h_motionRatio<h_motionThresholdMin) 
					{
						Process_setModality(ICU_AUTO_SWITCH_MODE,1);
					}
				}
				break;
			}
		}
	}
	/**********************************************************************************************************/
	/*                   N O U V E A U   A U T O S W I T C H    M O D E   E N D                               */
	/**********************************************************************************************************/


	/**********************************************************/
	/*               Save faces if requested                  */
	/**********************************************************/
	if(SDI_F->save_face==1) 
		Process_saveDetectedFaces(flux,"visage");

	/***************************************************************/
	/*           Mise a jour de la modalite courrante              */
	/***************************************************************/
	SDI_F->humanPos.currentModality = Process_getCurrentModality();
	SDI_F->currentModality = Process_getCurrentModality();

	/** Display if necessary */
	if(SDI_F->display.display) 
		cvShowImage("ICU", flux);
	//cvWaitKey(2);
	//Display_showImage(flux);

	/** Save video if necessary */
	if(SDI_F->captureVideo)
	{
		sprintf(pictureName,"/tmp/capture_%05d.png",SDI_F->pictureNumber);
		printf("Saving %s\n",pictureName);
		cvSaveImage(pictureName,flux);
		//Display_saveImage(pictureName,flux);
		SDI_F->pictureNumber++;
	}

//	SDI_F->cmdPtu.x = SDI_F->results.target.x;
//	SDI_F->cmdPtu.y = SDI_F->results.target.y;
	return OK;
}

/*------------------------------------------------------------------------
 * InitDisplay
 *
 * Description: 
 *
 * Reports:      OK
 *              S_icu_INIT_DISPLAY_FAILED
 */

/* icuInitDisplayExec  -  codel EXEC of InitDisplay
   Returns:  EXEC END ETHER FAIL ZOMBIE */
ACTIVITY_EVENT icuInitDisplayExec (ICU_AFFICHAGE *display, int *bilan)
{
	//char* d = {"--display=:0.0"};
	printf("--> ICU : in icuInitDisplayExec\n");
	//cvInitSystem(1,&d);

	/** Initialisation de la platine de la camera (controle du mouvement) */
	//EVI_Init();
	SDI_F->cameraPos.pan = 0+CAMERA_PAN_OFFSET;
	SDI_F->cameraPos.tilt = 0+CAMERA_TILT_OFFSET;
	SDI_F->cameraPos.zoom = 47;
	//EVI_GoToD70(SDI_F->cameraPos.pan,SDI_F->cameraPos.tilt,SDI_F->cameraPos.zoom,1);

	/** Initialisation de la camera elle meme (controle des images) */
	Video_create(SDI_F->display.width,SDI_F->display.height);
	printf("--> ICU : in Video_create\n");
	/** Creation d'un afficheur en fonction des parametres de display */
	if(SDI_F->display.display)
	{
		printf("Initializing ICU window\n");
		cvInitSystem(0,NULL);
		cvStartWindowThread();
		cvNamedWindow("ICU", 1);
		cvMoveWindow( "ICU", SDI_F->display.posx, SDI_F->display.posy );
	//cvResizeWindow( "ICU", SDI_F->display.width, SDI_F->display.height);
	}
	//Display_create(SDI_F->display.posx,SDI_F->display.posy,SDI_F->display.width,SDI_F->display.height);

	/** Initialisation de la position de la platine camera */

	/** Recuperation des parametres de la camera une fois positionnee */

	/** Initialisation des parametres de capture */
	SDI_F->captureImage=0;
	SDI_F->captureVideo=0;
	SDI_F->pictureNumber=0;

	/** Initialisation de la police de caracteres */
	cvInitFont(&font,CV_FONT_VECTOR0,0.5,0.5,0.0,1,8);

	/** Initialisation de la camera elle meme (controle des images) */

	//icuCameraBank=CAMERA_BANK_A;
	
	//sprintf(cameraPosterName, CAMERA_POSTER_NAME, icuCameraBank);
	
	if (posterFind(SDI_F->viamPosterName.name, &VIAM_CAMERA_POSTER_ID) != OK) {
		printf("ERROR: Camera poster find %s \n", SDI_F->viamPosterName.name);
		h2perror("camera posterFind");
		*bilan = errnoGet();
		return ETHER;
	}

	 if (posterTake(VIAM_CAMERA_POSTER_ID, POSTER_READ) != OK ) {
		h2perror("camera posterTake");
		*bilan = errnoGet();
		return ETHER;
	}

	VIAM_IMAGE = (ViamImageBank *)posterAddr(VIAM_CAMERA_POSTER_ID);
	
	if (VIAM_IMAGE == NULL) { 
		h2perror("no camera poster");
		*bilan = errnoGet();
		return ETHER;
	}

	imageWidth = VIAM_IMAGE->image[0].width;
	imageHeight = VIAM_IMAGE->image[0].height;

	for(int i=0;i<CAMERA_MATRIX_INTRINSIC_SIZE;i++)
	{
		mIntrinsic[i] =  VIAM_IMAGE->image[0].calibration.intrinsic[i];
	}
	
	posterGive(VIAM_CAMERA_POSTER_ID);

	printf("ICU --> bufferedCameraImage size : %d x %d\n",imageWidth, imageHeight);
	brut=cvCreateImage(cvSize(imageWidth,imageHeight),8,1);
	deBayer=cvCreateImage(cvSize(imageWidth,imageHeight),8,3);
	flux=cvCreateImage(cvSize(SDI_F->display.width,SDI_F->display.height),8,3);

	start = 1;

	return ETHER;
}


/*------------------------------------------------------------------------
 * CaptureImage
 *
 * Description: 
 *
 * Reports:      OK
 */

/* icuCaptureImageExec  -  codel EXEC of CaptureImage
   Returns:  EXEC END ETHER FAIL ZOMBIE */
ACTIVITY_EVENT icuCaptureImageExec (int *captureImage, int *bilan)
{
	Process_enableLog();
	return ETHER;
}

/*------------------------------------------------------------------------
 * EnableDisplay
 *
 * Description: 
 *
 * Reports:      OK
 */

/* icuEnableDisplayExec  -  codel EXEC of EnableDisplay
   Returns:  EXEC END ETHER FAIL ZOMBIE */

ACTIVITY_EVENT icuEnableDisplayExec (ICU_AFFICHAGE *display, int *bilan)
{
/*printf("SDI_F->display.display : %d, display->display : %d\n", SDI_F->display.display, display->display);
	if(SDI_F->display.display == 0)
	{
	Display_delete();
}
	else
	{
	Display_create(SDI_F->display.posx,SDI_F->display.posy,SDI_F->display.width,SDI_F->display.height);
}*/
	SDI_F->display.display = display->display;
	if(SDI_F->display.display == 0)
	{
		cvDestroyWindow("ICU");
//Display_delete();
	}
	else
	{
		cvStartWindowThread();
		cvNamedWindow("ICU", 1);
		cvMoveWindow( "ICU", SDI_F->display.posx, SDI_F->display.posy );
	//cvResizeWindow( "ICU", SDI_F->display.width, SDI_F->display.height);
//Display_create(SDI_F->display.posx,SDI_F->display.posy,SDI_F->display.width,SDI_F->display.height);
	}
	/* ... add your code here ... */
	printf("icuEnableDisplayExec end\n");
	return ETHER;
}
/*------------------------------------------------------------------------
 * SetCameraPos
 *
 * Description: 
 *
 * Reports:      OK
 */

/* icuSetCameraPosExec  -  codel EXEC of SetCameraPos
   Returns:  EXEC END ETHER FAIL ZOMBIE */
ACTIVITY_EVENT icuSetCameraPosExec (ICU_CAMERA_POS *cameraPos, int *bilan)
{
	SDI_F->cameraMove = 1;
//Necessaire car EVI_GoToD70 prends une position absolue ...
	//EVI_GetParamD70((float*)&SDI_F->cameraPos.pan,(float*)&SDI_F->cameraPos.tilt,(float*)&SDI_F->cameraPos.zoom);
	SDI_F->cameraPos.pan += cameraPos->pan;
	SDI_F->cameraPos.tilt += cameraPos->tilt;
	SDI_F->cameraPos.zoom += cameraPos->zoom;
	printf("Pan : %f, Tilt : %f, Zoom : %f\n", SDI_F->cameraPos.pan, SDI_F->cameraPos.tilt, SDI_F->cameraPos.zoom);
	//EVI_GoToD70(SDI_F->cameraPos.pan,SDI_F->cameraPos.tilt,SDI_F->cameraPos.zoom,0);
	SDI_F->cameraMove = 0;

	return ETHER;
}

/*------------------------------------------------------------------------
 * TrackIt
 *
 * Description: 
 *
 * Reports:      OK
 *              S_icu_TRACK_IT_FAILED
 */

/* icuTrackItExec  -  codel EXEC of TrackIt
   Returns:  EXEC END ETHER FAIL ZOMBIE */
ACTIVITY_EVENT icuTrackItExec (int * track_it, int *bilan)
{
	return ETHER;
	printf("TrackIt\n");
	if(!(*track_it))
	{
		return ETHER;
	}
	/***********************************************************************/
	/*               Asservissement platine                        		 */
	/* Track if necessary                                  		         */
	/* Track only the detection                    		                 */
	/* Dont track in modality ICU_FACE_TRACKING or ICU_MOTION_MONITORING   */
	/***********************************************************************/
	if (notCorrect>MINIMUM_IMAGES_WITHOUT_MOVING && 
	//SDI_F->results.target.available &&
		   SDI_F->results.nbDetectedFaces && 
		   SDI_F->results.target.x != 0 && SDI_F->results.target.y != 0 &&
		   !Process_isEnabled(ICU_FACE_TRACKING) && 
		   !Process_isEnabled(ICU_MOTION_MONITORING))
	{
		int angle_max_x=48;//open angle of the camera
		int angle_max_y=36;//open angle of the camera

	//Distances from the target to the center of the image
		double distance_centerx,distance_centery;
		distance_centerx=160.0-(SDI_F->results.target.x);
		distance_centery=120.0-(SDI_F->results.target.y);
	//printf("dx : %lf, dy : %lf\n", distance_centerx,distance_centery);

//double pan,tilt,zoom;
//EVI_GetParamD70((float*)&pan,(float*)&tilt,(float*)&zoom);
		//EVI_GetParamD70((float*)&SDI_F->cameraPos.pan,(float*)&SDI_F->cameraPos.tilt,(float*)&SDI_F->cameraPos.zoom);

//if(fabs(distance_centerx)>10.0)
		SDI_F->cameraPos.pan-=(distance_centerx*(angle_max_x)/SDI_F->display.width);

//if(fabs(distance_centery)>10.0)
		SDI_F->cameraPos.tilt+=(distance_centery*(angle_max_y)/SDI_F->display.height);

//if((fabs(distance_centerx)>10.0) || (fabs(distance_centery)>10.0))
	//{
		//printf("camera going to %lf %lf %lf\n",SDI_F->cameraPos.pan,SDI_F->cameraPos.tilt,SDI_F->cameraPos.zoom);
		//EVI_changeSpeed(10,10,2);
		//EVI_GoToD70(SDI_F->cameraPos.pan,SDI_F->cameraPos.tilt,SDI_F->cameraPos.zoom,0);

		// Reinitialisation du tracker au centre de l'image
		//Process_initTrackerPos(SDI_F->display.width/2,SDI_F->display.height/2);
	//}
		notCorrect = 0;
	}
	else  
		notCorrect++;

	return EXEC;
}

/*------------------------------------------------------------------------
 * WaitForSomebody
 *
 * Description: 
 *
 * Reports:      OK
 *              S_icu_NO_PROCESS_RUNNING
 *              S_icu_NOBODY
 *              S_icu_BAD_PARAMETERS
 */

/* icuWaitForSomebodyExec  -  codel EXEC of WaitForSomebody
   Returns:  EXEC END ETHER FAIL ZOMBIE */
ACTIVITY_EVENT
		icuWaitForSomebodyExec(int *nbImageMaxNobody, ICU_WAITFOR *waitFor, int *report)
{
	//printf("icuWaitForSomebodyExec\t:\tnbImg : %d\n", *nbImageMaxNobody);

	if(*nbImageMaxNobody == 0)
	{
		sprintf(waitFor->name, "\0");
		*report = S_icu_NOBODY;
		return ETHER;
	}
	for(int i=1;i<SDI_F->loadedPersonList.nb;i++)
	{
		if(SDI_F->loadedPersonList.who[i].present == 1)
		{
			printf("%s Found! : %d\n", SDI_F->loadedPersonList.who[i].name, SDI_F->loadedPersonList.who[i].present);
			strcpy(waitFor->name, SDI_F->loadedPersonList.who[i].name);
			*report = OK;
			return ETHER; 
		}
	}
	//for Unknown people
	if(SDI_F->loadedPersonList.who[0].present == 1)
	{
		printf("%s Found! : %d\n", SDI_F->loadedPersonList.who[0].name, SDI_F->loadedPersonList.who[0].present);
		strcpy(waitFor->name, SDI_F->loadedPersonList.who[0].name);
		*report = OK;
		return ETHER; 
	}
	if(*nbImageMaxNobody > 0)
	{
		(*nbImageMaxNobody) --;
	}
	return EXEC; 
}


/*------------------------------------------------------------------------
 * WaitForSomeone
 *
 * Description: 
 *
 * Reports:      OK
 *              S_icu_NO_PROCESS_RUNNING
 *              S_icu_NOBODY
 *              S_icu_BAD_PARAMETERS
 */

/* icuWaitForSomeoneStart  -  codel START of WaitForSomeone
   Returns:  START EXEC END ETHER FAIL ZOMBIE */
ACTIVITY_EVENT icuWaitForSomeoneStart (ICU_WAITFOR *waitFor, int *bilan)
{
	printf("WaitForName : %s\n", waitFor->name);
	waitForId[CURRENT_ACTIVITY_NUM(ICU_RUNTASK_NUM)] = Facerec_getId(&(SDI_F->loadedPersonList),waitFor->name, 2);
	if(waitForId[CURRENT_ACTIVITY_NUM(ICU_RUNTASK_NUM)]<=0)
	{
		printf("waitForId = 0\n");
		*bilan = S_icu_BAD_PARAMETERS;
		return ETHER;
	}
	return EXEC;
}

/* icuWaitForSomeoneExec  -  codel EXEC of WaitForSomeone
   Returns:  EXEC END ETHER FAIL ZOMBIE */
ACTIVITY_EVENT icuWaitForSomeoneExec (ICU_WAITFOR *waitFor, int *bilan)
{
	//printf("icuWaitForSomeoneExec\t:\tnbImg : %d\n", waitFor->nbImageMinSomebody);
	if(SDI_F->loadedPersonList.who[waitForId[CURRENT_ACTIVITY_NUM(ICU_RUNTASK_NUM)]].present == 1)	    
	{
		printf("Hello %s !\n",waitFor->name);
		*bilan = OK;
		return ETHER; 
	}
	else
	{
		if(waitFor->nbImageMinSomebody == 0)
		{
			sprintf(waitFor->name, "\0");
			*bilan=S_icu_NOBODY;
			return ETHER;
		}
		if(waitFor->nbImageMinSomebody > 0)
		{
			(waitFor->nbImageMinSomebody) --;
		}
		return EXEC;
	}
}

/*------------------------------------------------------------------------
 * WaitForNobody
 *
 * Description: 
 *
 * Reports:      OK
 *              S_icu_NO_PROCESS_RUNNING
 *              S_icu_SOMEBODY
 *              S_icu_BAD_PARAMETERS
 */

/* icuWaitForNobodyExec  -  codel EXEC of WaitForNobody
   Returns:  EXEC END ETHER FAIL ZOMBIE */
ACTIVITY_EVENT icuWaitForNobodyExec (int* nbImageMaxNobody, int *bilan)
{
	//printf("icuWaitForNobodyExec\t:\tnbImg : %d\n", *nbImageMaxNobody);
	if(*nbImageMaxNobody == 0)
	{
		*bilan = S_icu_SOMEBODY;
		return ETHER;
	}
	else
	{
		//tant qu il y a une personne, on continue
		for(int i=0;i<SDI_F->loadedPersonList.nb;i++)
		{
			if(SDI_F->loadedPersonList.who[i].present == 1)
			{
				if(*nbImageMaxNobody > 0)
				{
					(*nbImageMaxNobody) --;
				}
				return EXEC; 
			}
		}
		*bilan = OK;
		return ETHER;
	}
}

/*------------------------------------------------------------------------
 * WaitForNone
 *
 * Description: 
 *
 * Reports:      OK
 *              S_icu_NO_PROCESS_RUNNING
 *              S_icu_SOMEBODY
 *              S_icu_BAD_PARAMETERS
 */

/* icuWaitForNoneStart  -  codel START of WaitForNone
   Returns:  START EXEC END ETHER FAIL ZOMBIE */
ACTIVITY_EVENT icuWaitForNoneStart (ICU_WAITFOR *waitFor, int *bilan)
{
	waitForId[CURRENT_ACTIVITY_NUM(ICU_RUNTASK_NUM)] = Facerec_getId(&(SDI_F->loadedPersonList),waitFor->name, 2);
	if(waitForId[CURRENT_ACTIVITY_NUM(ICU_RUNTASK_NUM)]<=0)
	{
		printf("waitForId = 0\n");
		*bilan = S_icu_BAD_PARAMETERS;
		return ETHER;
	}
	return EXEC;
}

/* icuWaitForNoneExec  -  codel EXEC of WaitForNone
   Returns:  EXEC END ETHER FAIL ZOMBIE */
ACTIVITY_EVENT icuWaitForNoneExec (ICU_WAITFOR *waitFor, int *bilan)
{
	//printf("icuWaitForSomeoneExec\t:\tnbImg : %d\n", waitFor->nbImageMinSomebody);
	if(SDI_F->loadedPersonList.who[waitForId[CURRENT_ACTIVITY_NUM(ICU_RUNTASK_NUM)]].present == -1)	    
	{
		printf("Good bye %s !\n",waitFor->name);
		*bilan = OK;
		return ETHER; 
	}
	else
	{
		if(waitFor->nbImageMinSomebody == 0)
		{
			sprintf(waitFor->name, "\0");
			*bilan = S_icu_SOMEBODY;
			return ETHER;
		}
		if(waitFor->nbImageMinSomebody > 0)
		{
			(waitFor->nbImageMinSomebody) --;
		}
		return EXEC;
	}
}

/*------------------------------------------------------------------------
 * WaitForNobodyKnown
 *
 * Description: 
 *
 * Reports:      OK
 *              S_icu_NO_PROCESS_RUNNING
 *              S_icu_SOMEBODY
 *              S_icu_BAD_PARAMETERS
 */

/* icuWaitForNobodyKnownExec  -  codel EXEC of WaitForNobodyKnown
   Returns:  EXEC END ETHER FAIL ZOMBIE */
ACTIVITY_EVENT icuWaitForNobodyKnownExec (int* nbImageMaxNobody, int *bilan)
{
	//printf("icuWaitForNobodyKnownExec\t:\tnbImg : %d, available : %d\n", *nbImageMaxNobody, SDI_F->results.target.available);

	if(*nbImageMaxNobody == 0)
	{
		*bilan = S_icu_SOMEBODY;
		return ETHER;
	}
	else
	{
		if(SDI_F->results.target.available == 1)
		{
			if(*nbImageMaxNobody > 0)
			{
				(*nbImageMaxNobody) --;
			}
			return EXEC;
		}
		*bilan = OK;
		return ETHER;

		//tant qu il y a une personne connue (Unknown = 0), on continue
		/*for(int i=1;i<SDI_F->loadedPersonList.nb;i++)
		{
		if(SDI_F->loadedPersonList.who[i].present == 1)
		{
		if(*nbImageMaxNobody > 0)
		{
		(*nbImageMaxNobody) --;
	}
		return EXEC; 
	}
	}
		*bilan = OK;
		return ETHER;*/
	}
}
/*------------------------------------------------------------------------
 * WaitForPersonListChange
 *
 * Description: 
 *
 * Reports:      OK
 *              S_icu_NO_PROCESS_RUNNING
 */

/* icuWaitForPersonListChangeExec  -  codel EXEC of WaitForPersonListChange
   Returns:  EXEC END ETHER FAIL ZOMBIE */
ACTIVITY_EVENT icuWaitForPersonListChangeExec(ICU_PERSON_LIST_PARAMS *p,ICU_WAITFOR *w, int *bilan)
{
	/* On termine la requete si on constate un changement */
	if(SDI_F->personListChanged)
	{
		printf("personList have changed\n %s is present\n",SDI_F->waitFor.name);
		strcpy(w->name, SDI_F->waitFor.name);
		return ETHER;
	}
	return EXEC;
}

/*------------------------------------------------------------------------
 * ProcessSetModality
 *
 * Description: 
 *
 * Reports:      OK
 */

/* icuProcessSetModalityExec  -  codel EXEC of ProcessSetModality
   Returns:  EXEC END ETHER FAIL ZOMBIE */
ACTIVITY_EVENT icuProcessSetModalityExec (ICU_MODALITIES* type_of_process, int *bilan)
{
	/** Changement de modalite explicite si on est pas en mode auto */
	if(!SDI_F->autoSwitchControl.autoSwitchMode) 
		Process_setModality(*type_of_process,1);
	return ETHER;
}
/*------------------------------------------------------------------------
 * FacerecSetWorkingPath
 *
 * Description: 
 *
 * Reports:      OK
 */

/* icuFacerecSetWorkingPathExec  -  codel EXEC of FacerecSetWorkingPath
   Returns:  EXEC END ETHER FAIL ZOMBIE */
ACTIVITY_EVENT icuFacerecSetWorkingPathExec(GEN_STRING256* facerecWorkingPath, int *bilan) 
{
	Facerec_setWorkingPath(facerecWorkingPath->name);
	// Mise a jour des listes de personnes connues et chargees
	Facerec_clearList(&(SDI_F->knownPersonList));
	Facerec_clearList(&(SDI_F->loadedPersonList));
	Facerec_knownList(&(SDI_F->knownPersonList));
	Facerec_whoLoaded(&(SDI_F->loadedPersonList));
	return ETHER;
}
/*------------------------------------------------------------------------
 * FacerecSetParams
 *
 * Description: 
 *
 * Reports:      OK
 */

/*------------------------------------------------------------------------
 * FacerecInit
 *
 * Description: 
 *
 * Reports:      OK
 */

/* icuFacerecInitExec  -  codel EXEC of FacerecInit
   Returns:  EXEC END ETHER FAIL ZOMBIE */
ACTIVITY_EVENT icuFacerecInitExec(int *bilan)
{
	Facerec_init();
	return ETHER;
}
/*------------------------------------------------------------------------
 * FacerecAddSomebody
 *
 * Description: 
 *
 * Reports:      OK
 */

/* icuFacerecAddSomebodyExec  -  codel EXEC of FacerecAddSomebody
   Returns:  EXEC END ETHER FAIL ZOMBIE */
ACTIVITY_EVENT icuFacerecAddSomebodyExec(GEN_STRING256* recoAddName, int *bilan) 
{
	if(SDI_F->recoAddSomebody) {
		printf("recoAddSomebody");
		/* On reste en apprentissage */
		return EXEC;
	}
	else {
		printf("Termine!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!");
		/* Update de la liste de noms */
		Facerec_addToList(&(SDI_F->loadedPersonList),SDI_F->recoAddName.name);
		Facerec_addToList(&(SDI_F->knownPersonList),SDI_F->recoAddName.name);

		/** Si le nombre de visage est atteint on remet l'affichage normal en marche et les parametres par defaut */
		/** On fait aussi l'initialisation de ICU pour la reco du visage appris */
		printf("Facerec_saveBaseList\n");

		/** Finalise l'apprentissage en sauvant sur fichier la liste de bases en memoire */
		Facerec_saveBaseList();

//printf("Facerec_loadBases\n");

		/** Charge les bases d'apres la liste des personnes chargees et ajoutees */
//Facerec_loadSomebody(SDI_F->recoAddName);

		printf("Facerec_reload\n");

		/** Initialisation de la reco de visage a partir de la base en memoire */
//Facerec_init();

		/* Recharge les personnes */
		Facerec_reload(&(SDI_F->loadedPersonList));
		Facerec_init();

		printf("Facerec_setPeople\n");

		/** Selection de la personne qui vient d'etre enregistree comme reference */
		Facerec_setPeople(SDI_F->recoAddName.name);

		printf("Process_enableDisplay\n");

		/** Reactivation de l'affichage normal */
		Process_enableDisplay();

		printf("Process_setModality(NO_PROCESS)\n");	  

		/** Desactivation de la modalite de detection de visage et retablissement de l'affichage */
		Process_setModality(ICU_NO_PROCESS,1);            

		/* Apprentissage termine */
		return ETHER;
	} 
}


/*------------------------------------------------------------------------
 * FacerecSetPeople
 *
 * Description: 
 *
 * Reports:      OK
 */

/* icuFacerecSetPeopleExec  -  codel EXEC of FacerecSetPeople
   Returns:  EXEC END ETHER FAIL ZOMBIE */
/* Set the people to recognize */
ACTIVITY_EVENT icuFacerecSetPeopleExec(GEN_STRING256* who, int *bilan) 
{
	Facerec_setPeople(who->name);
	return ETHER;
}

/*------------------------------------------------------------------------
 * FacerecRecognizeAll
 *
 * Description: 
 *
 * Reports:      OK
 */

/* icuFacerecRecognizeAllExec  -  codel EXEC of FacerecRecognizeAll
   Returns:  EXEC END ETHER FAIL ZOMBIE */
/* Set the recognition to all people in memory */
ACTIVITY_EVENT icuFacerecRecognizeAllExec(int *bilan) 
{
	Facerec_recognizeAll();
	return ETHER;
}


/*------------------------------------------------------------------------
 * FacerecLoadSomebody
 *
 * Description: 
 *
 * Reports:      OK
 */

/* icuFacerecLoadSomebodyExec  -  codel EXEC of FacerecLoadSomebody
   Returns:  EXEC END ETHER FAIL ZOMBIE */
ACTIVITY_EVENT icuFacerecLoadSomebodyExec(GEN_STRING256* who, int *bilan) 
{
	Facerec_loadSomebody(who->name);
	Facerec_clearList(&(SDI_F->loadedPersonList));
	Facerec_whoLoaded(&(SDI_F->loadedPersonList));
	Facerec_init();
	int j=0;
	char s[256];
	for(int i=1;i<SDI_F->loadedPersonList.nb;i++)
	{
		sprintf(s, "%s/facerec/%s.jpg",ICUCONFIGPATH, SDI_F->loadedPersonList.who[i].name);
		tabImg[j++] = cvLoadImage(s, 1);
	}
	return ETHER;
}

/*------------------------------------------------------------------------
 * FacerecUnloadSomebody
 *
 * Description: 
 *
 * Reports:      OK
 */

/* icuFacerecUnloadSomebodyExec  -  codel EXEC of FacerecUnloadSomebody
   Returns:  EXEC END ETHER FAIL ZOMBIE */
ACTIVITY_EVENT icuFacerecUnloadSomebodyExec(GEN_STRING256* who, int *bilan) 
{
	Facerec_unloadSomebody(who->name);
	Facerec_clearList(&(SDI_F->loadedPersonList));
	Facerec_whoLoaded(&(SDI_F->loadedPersonList));
	Facerec_init();
	return ETHER;
}

/*------------------------------------------------------------------------
 * FacerecLoadEverybody
 *
 * Description: 
 *
 * Reports:      OK
 */

/* icuFacerecLoadEverybodyExec  -  codel EXEC of FacerecLoadEverybody
   Returns:  EXEC END ETHER FAIL ZOMBIE */
/* Load everybody to the memory */
ACTIVITY_EVENT icuFacerecLoadEverybodyExec(int *bilan) 
{
	Facerec_loadEverybody();
	Facerec_clearList(&(SDI_F->loadedPersonList));
	Facerec_whoLoaded(&(SDI_F->loadedPersonList));
	Facerec_init();
	return ETHER;
}
/*------------------------------------------------------------------------
 * FacerecUnloadEverybody
 *
 * Description: 
 *
 * Reports:      OK
 */

/* icuFacerecUnloadEverybodyExec  -  codel EXEC of FacerecUnloadEverybody
   Returns:  EXEC END ETHER FAIL ZOMBIE */
/* Unload all person from the memory */
ACTIVITY_EVENT icuFacerecUnloadEverybodyExec(int *bilan) 
{
	Facerec_unloadEverybody();
	Facerec_clearList(&(SDI_F->loadedPersonList));
	Facerec_whoLoaded(&(SDI_F->loadedPersonList));
	Facerec_init();
	return ETHER;
}
/*------------------------------------------------------------------------
 * WaitForModalityChange
 *
 * Description: 
 *
 * Reports:      OK
 */

/* icuWaitForModalityChangeExec  -  codel EXEC of WaitForModalityChange
   Returns:  EXEC END ETHER FAIL ZOMBIE */
ACTIVITY_EVENT icuWaitForModalityChangeExec(int *bilan)
{
	if(SDI_F->currentModality!=previousModality)
	{
		printf("Change to modality %d\n",SDI_F->currentModality);
		previousModality = SDI_F->currentModality;
		return ETHER;
	}
	return EXEC;
}

/*------------------------------------------------------------------------
 * LaunchPlatine
 *
 * Description: 
 *
 * Reports:      OK
 */

/* icuLaunchPlatineExecStart  -  codel START of LaunchPlatine
   Returns:  START EXEC END ETHER FAIL ZOMBIE */
ACTIVITY_EVENT icuLaunchPlatineExecStart(ICU_PLATINE_PARAMS *platineParams, int *bilan)
{
	iPan = 0;
	char platinePosterName[256];
	sprintf(platinePosterName, "platineState");
	if(posterFind(platinePosterName, (POSTER_ID *)&platinePosterId) == ERROR)
	{
		h2perror("platine posterFind");
		*bilan = errnoGet();
		printf("icuLaunchPlatineExecStart : posterFind\n");
		return ETHER;
	}
	if (posterTake(platinePosterId, POSTER_READ) != OK) 
	{
		h2perror("platine posterTake");
		*bilan = errnoGet();
		printf("icuLaunchPlatineExecStart : posterTake\n");
		return ETHER;
	}
	platinePoster = (PLATINE_STATES *)posterAddr(platinePosterId);
	if (platinePoster == NULL) 
	{ 
		h2perror("no platine poster");
		*bilan = errnoGet();
		printf("icuLaunchPlatineExecStart : posterAddr\n");
		return ETHER;
	}
	posterGive(platinePosterId);
	platineOn = 1;
	return EXEC;
}
/* icuLaunchPlatineExec  -  codel EXEC of LaunchPlatine
   Returns:  EXEC END ETHER FAIL ZOMBIE */
ACTIVITY_EVENT icuLaunchPlatineExec(ICU_PLATINE_PARAMS *platineParams, int *bilan)
{
	if (posterTake(platinePosterId, POSTER_READ) != OK) 
	{
		*bilan = errnoGet();
		return ETHER;
	}
	double pan = platinePoster->stateDeg.pos.pan;
//printf("Pan : %lf\tTilt : %lf\t || min : %lf\tmax : %lf\tvit : %lf\n",platinePoster->stateDeg.pos.pan, platinePoster->stateDeg.pos.tilt, platineParams->posMin, platineParams->posMax, platineParams->vitesse);
	tmpPanPlatine[iPan++] = platinePoster->stateDeg.pos.pan;
	posterGive(platinePosterId);
	if((pan > platineParams->posMax && platineParams->vitesse > 0) || (pan < platineParams->posMin && platineParams->vitesse < 0))
	{
		SDI_F->cmdPlatine.pan = 0.0;
		SDI_F->cmdPlatine.tilt = 0.0;
		for(int i=0;i<iPan;i++)
		{
			printf("%d : %lf\n", i, tmpPanPlatine[i]);
		}
		return ETHER;
	}
	double p = 0.0;
	if(platineParams->vitesse > 0)
	{
		p = (platineParams->posMax - pan)/(platineParams->posMax - platineParams->posMin);
	}
	else
	{
		p = (pan - platineParams->posMin)/(platineParams->posMax - platineParams->posMin);
	}
	SDI_F->cmdPlatine.pan = platineParams->vitesse * MAX(p, 0.05);
	SDI_F->cmdPlatine.tilt = 0.0;
	return EXEC;
}

/*------------------------------------------------------------------------
 * CmdPtu
 *
 * Description: 
 *
 * Reports:      OK
 */

/* icuCmdPtu  -  codel EXEC of CmdPtu
   Returns:  EXEC END ETHER FAIL ZOMBIE */
ACTIVITY_EVENT icuCmdPtu(int *bilan)
{

	double pan = 0.0, tilt = 0.0;
	SDI_F->Data.u = SDI_F->results.target.x;
	SDI_F->Data.v = SDI_F->results.target.y; 
	SDI_F->Data.scale = SDI_F->results.target.scale;

	if(tagList.nbTags != 0)
	{
	  double alphaP = 0.0;
	  if(tagList.tags[0].angleMean != -1.0)
	  {
		      SDI_F->Data.rfidZ= tagList.tags[0].distMean;
		    /*if(tagList.tags[0].angleMean<180.0)
		    {
		      SDI_F->Data.rfidT = -tagList.tags[0].angleMean;
		    }
		  else
		    {
		      SDI_F->Data.rfidT = 540-tagList.tags[0].angleMean;
		    }*/
		    SDI_F->Data.rfidT = 180-tagList.tags[0].angleMean;
	  }
	  if (posterTake(platinePosterId, POSTER_READ) == OK) 
	    {
	      pan = platinePoster->stateDeg.pos.pan;
	      tilt = platinePoster->stateDeg.pos.tilt;
	      posterGive(platinePosterId);
	    }
	  //SDI_F->Data.theta -= alphaP;
	}
	else
	{
		SDI_F->Data.rfidT = -1.0;
		SDI_F->Data.rfidZ= -1.0;
	}
	
	
	      SDI_F->Data.pan = pan;
	      SDI_F->Data.tilt = tilt;

	SDI_F->Data.x[0] = SDI_F->results.target.x+(9*SDI_F->results.target.scale);
	SDI_F->Data.y[0] = SDI_F->results.target.y-(9*SDI_F->results.target.scale);
	SDI_F->Data.z[0] = -0.0145*SDI_F->results.target.scale*18+ 2.3451;

	SDI_F->Data.x[1] = SDI_F->results.target.x+(9*SDI_F->results.target.scale);
	SDI_F->Data.y[1] = SDI_F->results.target.y+(9*SDI_F->results.target.scale);
	SDI_F->Data.z[1] = SDI_F->Data.z[0];

	SDI_F->Data.x[2] = SDI_F->results.target.x-(9*SDI_F->results.target.scale);
	SDI_F->Data.y[2] = SDI_F->results.target.y+(9*SDI_F->results.target.scale);
	SDI_F->Data.z[2] = SDI_F->Data.z[0];

	SDI_F->Data.x[3] = SDI_F->results.target.x-(9*SDI_F->results.target.scale);
	SDI_F->Data.y[3] = SDI_F->results.target.y-(9*SDI_F->results.target.scale);
	SDI_F->Data.z[3] = SDI_F->Data.z[0];
	
//if (SDI_F->results.target.x!=0) 
//	printf("icu:u=%d |v=%d |s=%.3f |z=%.2f\n",SDI_F->results.target.x,SDI_F->results.target.y,SDI_F->results.target.scale,SDI_F->Data.z[0]);
	return EXEC;
}

/*------------------------------------------------------------------------
 * IcuInitRFID
 *
 * Description: 
 *
 * Reports:      OK
 */

/* icuInitRFIDExec  -  codel EXEC of IcuInitRFID
   Returns:  EXEC END ETHER FAIL ZOMBIE */
ACTIVITY_EVENT icuInitRFIDExec(int *bilan)
{
	char rfidPosterName[256];
	sprintf(rfidPosterName, "rfidTagList");
	if(posterFind(rfidPosterName, (POSTER_ID *)&rfidPosterId) == ERROR)
	{
		h2perror("rfid posterFind");
		*bilan = errnoGet();
		printf("icuInitRFIDExec :: posterFind\n");
		return ETHER;
	}
	if (posterTake(rfidPosterId, POSTER_READ) != OK) 
	{
		h2perror("rfid posterTake");
		*bilan = errnoGet();
		printf("icuInitRFIDExec :: posterTake\n");
		return ETHER;
	}
	rfidPoster = (RFID_TAG_LIST *)posterAddr(rfidPosterId);
	if (rfidPoster == NULL) 
	{ 
		h2perror("no rfid poster");
		*bilan = errnoGet();
		printf("icuInitRFIDExec :: posterAddr\n");
		return ETHER;
	}
	posterGive(rfidPosterId);
	initializeRFID();
	rfOn = 1;
	return ETHER;
}

/*------------------------------------------------------------------------
 * IcuCloseRFID
 *
 * Description: 
 *
 * Reports:      OK
 */

/* icuCloseRFIDExec  -  codel EXEC of IcuCloseRFID
   Returns:  EXEC END ETHER FAIL ZOMBIE */
ACTIVITY_EVENT icuCloseRFIDExec(int *bilan)
{
	rfOn = 0;
	closeRFID();
  return ETHER;
}

/*------------------------------------------------------------------------
 * IcuAcquireRFID
 *
 * Description: 
 *
 * Reports:      OK
 */

/* icuAcquireRFIDExec  -  codel EXEC of IcuAcquireRFID
   Returns:  EXEC END ETHER FAIL ZOMBIE */
ACTIVITY_EVENT icuAcquireRFIDExec(int *bilan)
{
	if(rfOn)
	{
		if (posterTake(rfidPosterId, POSTER_READ) != OK) 
		{
			h2perror("rfid posterTake");
			*bilan = errnoGet();
			return ETHER;
		}
		
		tagList.nbTags = rfidPoster->nbTags;
		for(int i=0;i<tagList.nbTags;i++)
		{
			sprintf(tagList.tags[i].tagId, "%16s", rfidPoster->tags[i].tagId);
			tagList.tags[i].angleMean = rfidPoster->tags[i].angleMean;
			tagList.tags[i].angleCov = rfidPoster->tags[i].angleCov;
			tagList.tags[i].distMean = rfidPoster->tags[i].distMean;
			tagList.tags[i].distCov = rfidPoster->tags[i].distCov;
		}
		posterGive(rfidPosterId);
	}
  	return EXEC;
}

/*------------------------------------------------------------------------
 * IcuAcquireTag
 *
 * Description: 
 *
 * Reports:      OK
 */

/* icuAcquireTagExec  -  codel EXEC of IcuAcquireTag
   Returns:  EXEC END ETHER FAIL ZOMBIE */
ACTIVITY_EVENT icuAcquireTagExec(GEN_STRING256* recoTag, int *bilan)
{	
	tagList.nbTags = 0;
	//On ne recupere que le tag que l on veut
	if(rfOn)
	{
		if (posterTake(rfidPosterId, POSTER_READ) != OK) 
		{
			h2perror("rfid posterTake");
			*bilan = errnoGet();
			printf("icuAcquireTagExec :: posterTake\n");
			return ETHER;
		}

		char idTag[16];
		//sprintf(idTag, "e00400007cd7fc01\0"); idTag[16] = '\0';
		sprintf(idTag, "e0040000c1b2fd01\0"); idTag[16] = '\0';
		for(int i=0;i<rfidPoster->nbTags;i++)
		{
			if(!strncmp(rfidPoster->tags[i].tagId, idTag, 16))
			{
				sprintf(tagList.tags[0].tagId, "%s", rfidPoster->tags[i].tagId);
				tagList.tags[0].Tag_MEMORY = NULL;
				//if(rfidPoster->tags[i].angleMean > 180)
				//	tagList.tags[0].angleMean = rfidPoster->tags[i].angleMean;// -180;
				//else
					tagList.tags[0].angleMean = rfidPoster->tags[i].angleMean; // + 180;
					
				//tagList.tags[0].angleMean = 360-rfidPoster->tags[i].angleMean;
				tagList.tags[0].angleCov = rfidPoster->tags[i].angleCov;
				tagList.tags[0].distMean = rfidPoster->tags[i].distMean;
				tagList.tags[0].distCov = rfidPoster->tags[i].distCov;
				tagList.nbTags = 1;
			}
		}
		posterGive(rfidPosterId);
	}
  	return EXEC;
}

