/* Copyright (c) 2006 CNRS/LAAS
 *
 * Permission to use, copy, modify, and distribute this software for any
 * purpose with or without fee is hereby granted, provided that the above
 * copyright notice and this permission notice appear in all copies.
 *
 * THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
 * WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
 * ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
 * ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
 * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */
#ifndef _icuSTRUCT_H
#define _icuSTRUCT_H

#include "icuConst.h"
#include "pom/pomStruct.h"

/**Modality flag definition*/
typedef enum ICU_MODALITIES {
  ICU_NO_PROCESS,
  ICU_AUTO_SWITCH_MODE,
  ICU_FACE_DETECTION,
  ICU_MOTION_DETECTION,
  ICU_FACE_TRACKING,
  ICU_BODY_TRACKING,
  ICU_MOTION_MONITORING,
  ICU_FACE_RECOGNITION,
  ICU_NB_MODALITY_MAX
} ICU_MODALITIES;

typedef struct str {
	char name[ICU_STR_SZ];
}str;
typedef struct ICU_WAITFOR {
  /* Name of the person to wait for */
  char name[256];
  /* Number of face detections to wait before saying that name have been detected */ 
  int nbImageMinSomebody;
} ICU_WAITFOR;

typedef struct ICU_PERSON_POS {
  double x; /* robot frame */
  double y;
  double z;
  int id;
  char name[64];
} ICU_PERSON_POS;

#define ICU_NB_MAX_DETECT 64

typedef struct ICU_HUMAN_POS {
  POM_SENSOR_POS sensorPos;
/*   double cap; */
  int currentModality;
  int number_of_detection;
  ICU_PERSON_POS personPos[ICU_NB_MAX_DETECT];  /* camera frame */
} ICU_HUMAN_POS;


typedef struct ICU_AFFICHAGE {
  int display;
  int width;
  int height;
  int posx;
  int posy;
  int unused;
} ICU_AFFICHAGE;

typedef struct ICU_PAN_TILT_LIMITS {
  int pan_min;
  int pan_max;
  int tilt_min;
  int tilt_max;
} ICU_PAN_TILT_LIMITS;

typedef struct ICU_CAMERA_POS {
  float pan;   // Les parametres de la camera tels qu ils sont stockes dans la camera 
  float tilt;  // (sans prise en compte d'un quelconque offset donc)
  float zoom;
} ICU_CAMERA_POS;

typedef struct ICU_AUTOSWITCH_CONTROL {
  int autoSwitchMode;
  int isRobotMoving;
} ICU_AUTOSWITCH_CONTROL;



#define NCP 20
#define NDP 20

typedef struct ICU_TRACKING_STATE_VECTOR
{
  int available;
  int x;
  int y;
  double theta;
  double scale;

}ICU_TRACKING_STATE_VECTOR;


typedef struct ICU_TRACKING_FACE_ROI
{
  int x;
  int y;
  int width;
  int height;

}ICU_TRACKING_FACE_ROI;

typedef struct ICU_RECOGNITION_VALUE
{
  int id;
  int state;         // etat de la reco
  char name[256]; // nom de la personne
  double likelihood; //ressemblance avec la personne
} ICU_RECOGNITION_VALUE;

#define ICU_NB_MAX_TEMPORAL_WINDOW 56

/* Attention dans cette structure on manipule les elements a partir */
/* de l'indice 1 l'indice 0 etant reserve pour le champs unknown    */
typedef struct ICU_RECOGNITION_WHO
{
  char name[256];
  int present;
  double likelihood;

  /* Compteurs internes et fenetres temporelles */
  double reliability[ICU_NB_MAX_TEMPORAL_WINDOW];
  int window[ICU_NB_MAX_TEMPORAL_WINDOW];  
  double cumul;
  int cptAP;
  int curid;
  int updated;
} ICU_RECOGNITION_WHO;


/* Nombre maximum de personnes connues par le robot */
#define ICU_NB_MAX_KNOWN_PERSON 24

typedef struct ICU_RECOGNITION_WHOLIST
{
  int nb;
  int unused;
  ICU_RECOGNITION_WHO who[ICU_NB_MAX_KNOWN_PERSON];
} ICU_RECOGNITION_WHOLIST;


typedef struct ICU_RFID_TAG
{
	char tagId[16];
	double angleMean;
	double angleCov;
	double distMean;
	double distCov;
} ICU_RFID_TAG;

typedef struct ICU_TRACKING_VALUES
{  
  /** Modalite qui a mis a jour cette structure */
  int modality;

  /** Vecteur d'etat estime */
  ICU_TRACKING_STATE_VECTOR target;

  /** Flag indiquant si du mouvement a ete detecte */
  int motionDetected;

  /** Liste des visages detectes */
  int nbDetectedFaces;
  int nbDetectedFacesUpdated;
  ICU_TRACKING_FACE_ROI faces[ICU_NB_MAX_DETECT];
  ICU_RECOGNITION_VALUE reco[ICU_NB_MAX_DETECT];
  ICU_RFID_TAG tag;

}ICU_TRACKING_VALUES;


typedef struct ICU_PERSON_LIST_PARAMS
{
  /* Seuil de proba pour reco ou absence */
  double seuilReco;
  
  /* Fenetre d'observation de l'absence */
  int absenceWindow;
  
  /* Fenetre d'observation de la presence */
  int presenceWindow;
  
  /* Ratio A/P de decision */
  double absenceRatio;
  
  /* Ratio P/A de decision */
  double presenceRatio;
} ICU_PERSON_LIST_PARAMS;

typedef struct ICU_PLATINE_PARAMS
{
  double posMin;
  double posMax;
  double vitesse;
} ICU_PLATINE_PARAMS;

typedef struct ICU_VISUAL_DATA
{
  double u;   		
  double v;
  double scale;
  double rfidT; 
  double rfidZ;
  double pan;
  double tilt;
  double x[4];   	
  double y[4];   	
  double z[4];  
  int nbSpots;    
  int numRef;   	
} ICU_VISUAL_DATA;

#endif
