#include "evicamera.h"
#include "math.h"

#define DEFAULT_NO_OF_LABELS 5

//var EVI-D31
signed short  EVI_pan,EVI_tilt;
int EVI_dpan=1,EVI_dtilt=1;
int EVI_ImgCenter[2];

int EVI_error[3];
int EVI_perror[3];
int EVI_errorSpd[3];
int EVI_perrorSpd[3];

int EVI_BoxSize[2];
int EVI_panEps=10,EVI_tiltEps=10,EVI_zoomEps=10;
int EVI_MOVE = 0;
int EVI_diagBox;
int EVI_diagImg;

short EVI_IsZoomActive=0;
short  EVI_FirtsIt;

int EVI_CamState;
int EVI_ZoomState;
int NoOfPanLabels,NoOfTiltLabels,NoOfZoomLabels,NoOfPanRules,NoOfTiltRules;

double  EVI_PanVal[]={-4,-1,0,1,4};
double  EVI_TiltVal[]={-4,-1,0,1,4};

unsigned char EVI_panSpd=5, EVI_tiltSpd=5, EVI_zoomSpd=7;
unsigned char EVI_PANSPEED=PAN_SPD_MAX;
unsigned char EVI_TILTSPEED=TILT_SPD_MAX;
unsigned char EVI_ZOOMSPEED=ZOOM_SPD_MAX;

short RMtx[]={4,4,3,1,1,
	      4,3,3,1,1,
	      3,3,2,1,1,
	      3,3,1,1,0,
	      3,3,1,0,0};

int EVI_Init()
{
  printf("Debut init\n");
  EviCamera_CreateRS232Link();
  printf("apres RS232 Link\n");
  EviCamera_InitializeCamera();
  printf("apres InitializeCamera\n");
}

int EVI_ControlParam(int width,int height,int options,double *fuzzySet)
{
  int i;
  int l[]={5,5,5};

  EVI_ImgCenter[0]=width/2;
  EVI_ImgCenter[1]=height/2;
  EVI_IsZoomActive=1;
  EVI_CamState=0;
  EVI_FirtsIt=1;
  EVI_ZoomState=0;
  EVI_diagImg=(int)(sqrt(width*width+height*height)*0.4);

  NoOfPanLabels=DEFAULT_NO_OF_LABELS;
  NoOfTiltLabels=DEFAULT_NO_OF_LABELS;
  NoOfPanRules=DEFAULT_NO_OF_LABELS;
  NoOfTiltRules=DEFAULT_NO_OF_LABELS;

  return 1;
}

int EVI_ChangeProperty(int prop,int val)
{
  EVI_IsZoomActive=val;
  return prop;
}


void EVI_Stop()
{  
  EviCamera_SendCommand2(CamPTDStop);
  if(EVI_IsZoomActive) EviCamera_SendCommand2(CamZoomStop);
}


void EVI_Close()
{
  EviCamera_CloseRS232Link();
}

void EVI_GoHome()
{
#ifdef  DEBUG
  printf("EVI-D31 Camera going home\n");
#endif
  EviCamera_SendCommand2(CamPTDHome);
  EviCamera_SendCommand2(CamZoomWideS);
}

int EVI_changeSpeed(int panSpd,int tiltSpd,int zoomSpd)
{
  int r=1;
  if ((panSpd>=1) && (panSpd<=24)) EVI_PANSPEED=panSpd;
  else printf("Error Pan Speed Value\n");

  if ((tiltSpd>=1) && (tiltSpd<=20)) EVI_TILTSPEED=tiltSpd;
  else printf("Error Tilt Speed Value\n");

  if ((zoomSpd>=2) && (zoomSpd<=7)) {
    EVI_ZOOMSPEED=zoomSpd;
  }  else printf("Error Zoom Speed Value\n");

  return 1;
}

int EVI_GoTo(float pan,float tilt,float zoom)
{
  int ip,it,iz;
 
  if ((pan>100) || (pan<-100) ||
      (tilt>25) || (tilt<-25) ||
      (zoom>47) || (zoom<5)) {
    printf("EVI_GoTo ERROR -> Not a valid parameter\n");
    return 0;
  }
  ip=(int)(pan*8.8);
  it=(int)(tilt*12);
  iz=(int)((47-zoom)*24.35);

#ifdef  DEBUG
  fprintf(stderr,"EVI camera going to pan %d,tilt %d,zoom %d\n",pan,tilt,zoom);
#endif

 EviCamera_PTDAbsPos(EVI_PANSPEED,EVI_TILTSPEED,ip,it);
 EviCamera_ZoomDirect(iz,FALSE);
 
  return 1;
}
int EVI_GoToD70(float pan,float tilt,float zoom, int zoomEnable)
{
  int ip,it,iz;
 
  if ((pan>170) || (pan<-170) ||
      (tilt>30) || (tilt<-90) ||
      (zoom>47) || (zoom<0)) {
    printf("EVI_GoTo ERROR -> Not a valid parameter\n");
    return 0;
  }
  ip=(int)(pan*13.333);
  it=(int)(tilt*13.333);
  iz=(int)((47.0-zoom)*668.596);

#ifdef  DEBUG
  fprintf(stderr,"EVI camera going to pan %f,tilt %f,zoom %f\n",pan,tilt,zoom);
#endif

  
  EviCamera_PTDAbsPos(EVI_PANSPEED,EVI_TILTSPEED,ip,it);
  //Rajoute par Mathias FONTMARTY 12 - 10 - 2005
  //Pour la prise en charge du zoom sur la EVI_D70
  if (zoomEnable) EviCamera_ZoomDirect(iz,FALSE);
  //En fait ceci affecte enormement la reactivite de la camera.

  return 1;
}

int EVI_GetParam(int *pan,int *tilt,int *zoom)
{
  short PanPos,TiltPos;
  unsigned short ZoomPos;

  EviCamera_PanTiltPosInq(&PanPos,&TiltPos );
  ZoomPos=EviCamera_ZoomPosInq();

  *pan= ((int)(PanPos)/8.8);
  *tilt=((TiltPos)/12.0);

 *zoom=(47-ZoomPos/24.35);
  
  return 1;
}
int EVI_GetParamD70(float *pan,float *tilt,float *zoom)
{
  short PanPos,TiltPos;
  unsigned short ZoomPos;

  EviCamera_PanTiltPosInq(&PanPos,&TiltPos );
  ZoomPos=EviCamera_ZoomPosInq();

  *pan= ((float)(PanPos)*0.075);
  *tilt= ((float)(TiltPos)*0.075);
  *zoom= (47.0 - (float)ZoomPos/668.596);
  
  return 1;
}
int EVI_Control(int *boundBox)
{
  int d,t;

  EVI_error[0]=boundBox[8]-EVI_ImgCenter[0];
  EVI_error[1]=boundBox[9]-EVI_ImgCenter[1];

  d=abs(EVI_error[0]/2);
  if (d>20) d = 5; 
  else if (d==0) d=1;

  EVI_panSpd=d;
  EVI_tiltSpd=d;

  if (abs(EVI_error[0])>EVI_panEps) {
    t=EVI_error[0]/abs(EVI_error[0]);
    EVI_pan=t*EVI_dpan;
    EVI_MOVE=TRUE;
  } else EVI_pan=0;

  if (abs(EVI_error[1])>EVI_tiltEps) {
    t=EVI_error[1]/abs(EVI_error[1]);
    EVI_tilt=-t*EVI_dtilt;
    EVI_MOVE=TRUE;
  } else EVI_tilt=0;

  if (EVI_MOVE) EviCamera_PTDRelPos( EVI_panSpd, EVI_tiltSpd, EVI_pan, EVI_tilt); 

  EVI_MOVE=FALSE;

  return 1;

}


int EVI_Zoom(int *box)
{
  unsigned char *mesg;
  int diag;
  int var;

  EVI_BoxSize[0]=box[2]-box[0];
  EVI_BoxSize[1]=box[5]-box[1];

  diag=(int) sqrt(EVI_BoxSize[0]*EVI_BoxSize[0]+EVI_BoxSize[1]*EVI_BoxSize[1]);
  EVI_error[2]=EVI_diagImg-diag;

  if (EVI_error[2]>EVI_zoomEps) {
    var=10;  
    mesg=CamZoomTeleV;
  } else if (EVI_error[2]<-EVI_zoomEps) {
    var=100;
    mesg=CamZoomWideV;
  } else {
    var=0;
    mesg=CamZoomStop;
  }

    EviCamera_SendCommandZoom(var,EVI_zoomSpd);
 
  return 1;
}

/*-------------------------------------------------------------------
 *  function: EVI_calcErrorVect
 *
 *  param   : int *boundBox
 *
 *  return  : double  evaluation
 *
 *------------------------------------------------------------------*/

int EVI_calcErrorVect(int *boundBox)
{

  EVI_perror[0]=EVI_error[0];
  EVI_perror[1]=EVI_error[1];

  EVI_error[0]=boundBox[8]-EVI_ImgCenter[0];
  EVI_error[1]=boundBox[9]-EVI_ImgCenter[1];

  if (EVI_FirtsIt) {
    EVI_FirtsIt=0;
    EVI_errorSpd[0]=0;
    EVI_errorSpd[1]=0;
  } else {
    EVI_errorSpd[0]=(EVI_error[0]-EVI_perror[0]);
    EVI_errorSpd[1]=(EVI_error[1]-EVI_perror[1]);
    EVI_perrorSpd[0]=EVI_errorSpd[0];///EVI_dtime;
    EVI_perrorSpd[1]=EVI_errorSpd[1];///EVI_dtime;
  }


  return 1;
}



int EVI_CenterBox(int *box)
{
  unsigned char *mesg;
  int var=0;

  EVI_error[0]=box[8]-EVI_ImgCenter[0];
  EVI_error[1]=box[9]-EVI_ImgCenter[1];

  if (EVI_error[0]>EVI_panEps)        var=100;
  else if (EVI_error[0]<-EVI_panEps)  var=200;
  else                                  var=0;
  
  if (EVI_error[1]>EVI_tiltEps)       var+=10; 
  else if (EVI_error[1]<-EVI_tiltEps) var+=20;
  else                                 var+=0;
 

  
  if (var!=EVI_CamState) {
    EviCamera_SendCommandPanTilt(var,EVI_panSpd,EVI_tiltSpd);
    EVI_CamState=var;
  }

  if (EVI_IsZoomActive) EVI_Zoom(box);

  return var;

}


