/*
	-----------------------------------------------------------------------------

	               --- CAEN SpA - Computing Systems Division ---

	-----------------------------------------------------------------------------

	CAENRFIDLib.c

	-----------------------------------------------------------------------------

	Created: May 2007
	Rev: Aug 2007

	Version 3.2.5.2-Linux-LAAS

  	-----------------------------------------------------------------------------
*/
#define CAENRFID
#define BMF_HEADER					10
#define AVP_COMMAND_SIZE			8
#define FALSE						0
#define TRUE						1
#define MAX_CHAR_DESCRIPTION		30
#define TIMEOUTCOMM					6000
#define SEL_UNSEL_TAG_VALUE_LENGTH	8
#define DWORD unsigned long int 

#include <termios.h>
#include <sys/ioctl.h>

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/timeb.h>
#include <time.h>

#include "CAENRFIDLib.h"
#include "Protocol.h"
#include "IOLib.h"
#include "includes_rfid.h"

#include <sys/fcntl.h>
#include <fcntl.h>
#include <errno.h>
#include <unistd.h>
/*
		Structure nedeed for library

*/

typedef struct {
	short			Protocol;
	char			*arg[MAX_ARG_SIZE];
	CAENRFIDHandle		handle;
} Lib_Struct_t;

typedef int             Boolean;

typedef union {
	double	dd;
	long	ll[2];
} udouble;

const char *LibraryRelease = "3.2.5.2L";

static Lib_Struct_t LibDescriptor[MAX_CLIENT_SUPPORTED];


unsigned short T11xVer = 0x8001;
unsigned long VendorID = 21336;
static int init_done=0;
static int minus=0;

int getnbTags(){
	return minus;
}

/********************************************************************************
*	
*				SendReceivePacket
*
********************************************************************************/
static int SendReceivePacket(CAENRFIDHandle Handle, IOBuffer_t *OBuffer, IOBuffer_t *IBuffer ) {

	int result;
	int countbyteread = 0;
	int byteread = 0;
	int size;
	int index=0;
	short Command;
	short count;
	int val;

	byteread = 0;

	FixPacketLength(OBuffer);
	count = GetCommandID(OBuffer);
	if (count == -1) return CAENRFID_CommunicationError;
	Command = GetCommand(OBuffer);

	if (Command == -1) return CAENRFID_CommunicationError;
	result = write (LibDescriptor[Handle].handle,OBuffer->memory,OBuffer->size);
	if ( result < OBuffer->size) {
		return CAENRFID_CommunicationError;
	}

	countbyteread = 0;
	IBuffer->memory = (char *)malloc(BMF_HEADER*sizeof(char));
	while ( (countbyteread < BMF_HEADER)) {
		byteread = read(LibDescriptor[Handle].handle,&(IBuffer->memory)[index],(BMF_HEADER-countbyteread));
		if (byteread<1) break;
		countbyteread += byteread;
		index = countbyteread;
	}
	
	
	if (result == 0 || !byteread) {
	return CAENRFID_CommunicationError;
	}
	IBuffer->rpos = 0;
	IBuffer->wpos = BMF_HEADER;
	IBuffer->size = BMF_HEADER;
	if ((size = checkHeader(IBuffer,count)) < BMF_HEADER) {
		if (IBuffer->memory != NULL) free(IBuffer->memory);
		IBuffer->memory = NULL;
		return CAENRFID_CommunicationError;
	}
	if (size >= BMF_HEADER) {
		IBuffer->memory = (char *)realloc(IBuffer->memory,size*sizeof(char));
		if (IBuffer->memory == NULL) return CAENRFID_OutOfMemoryError;

		byteread =0;
		countbyteread = 0;
		val=(size-BMF_HEADER)-64;
		minus=0;
		if (Command==18 && val>=14){
			minus=(int)(val/14);
		}
//		printf("Minus=%d\n",minus);
/**/		while (countbyteread < (size-BMF_HEADER-minus)) {
			byteread = read(LibDescriptor[Handle].handle,&(IBuffer->memory)[index],(size-BMF_HEADER-countbyteread));
			if (byteread<1) break;
			countbyteread += byteread;
//			printf("countbytereadsofar=%d\n",countbyteread);
			index += byteread;
		}
/*	if (Command==110){
		printf("byteread=%d\n",index);
		for (i=0;i<index;i++){
			buff0=IBuffer->memory[i];
			printf("%x ",buff0);
		}
		printf("\n");
	}
*/
		if (result == 00) {
		return CAENRFID_CommunicationError;
		}
		IBuffer->size = size;
		IBuffer->wpos = size;
		if (CheckCommand(IBuffer,Command)) return CAENRFID_StatusOK; 
		else {
			if (IBuffer->memory != NULL) free(IBuffer->memory);
			IBuffer->memory = NULL;
			return CAENRFID_CommunicationError;
		}
	}
	


	return CAENRFID_StatusOK;
}








CAENRFIDErrorCodes
connectRS232 (int _port, speed_t speed, char parity, tcflag_t bytesize, CAENRFIDHandle *handle,int usb2serial)
{
	char *filename;
	struct termios *termios_p;
	int port=_port-1;

	filename=(char *)malloc(15*sizeof(char));
	if (usb2serial)
		sprintf(filename,"/dev/ttyUSB%d",port);
	else
		sprintf( filename, "/dev/ttyS%d", port);


	if ((*handle=open(filename,O_RDWR | O_NOCTTY | O_NDELAY))==-1){
		return CAENRFID_PortError;
	}
	fcntl(*handle, F_SETFL, 0); /* Si on a rien a lire sur le port, alors on attend*/
	  free(filename);

	/*Ici, on définit les paramètres de notre connection*/

	termios_p=(struct termios *)malloc(sizeof(struct termios));
	tcgetattr(*handle,termios_p);
	cfsetispeed(termios_p,speed);
	cfsetospeed(termios_p,speed);
		
	if (parity=='E'){
		termios_p->c_cflag |= PARENB;
		termios_p->c_cflag &= ~PARODD;
		termios_p->c_iflag |= (INPCK|ISTRIP);
	}
	else
		if(parity=='O'){
			termios_p->c_cflag |= PARENB;
			termios_p->c_cflag |= PARODD;
			termios_p->c_iflag |= (INPCK|ISTRIP);
		}
		else
			termios_p->c_cflag &= ~PARENB;
	termios_p->c_cflag &= ~CSIZE;
	termios_p->c_cflag |= bytesize;

	termios_p->c_cflag |= (CLOCAL | CREAD);

	/*On le set en RAW mode*/
	termios_p->c_lflag &= ~(ICANON | ECHO | ECHOE | ISIG);
	termios_p->c_oflag &= ~OPOST;


	if(tcsetattr(*handle,TCSANOW,termios_p)==-1){
		close(*handle);
		return CAENRFID_PortError;
	}
	free(termios_p);
	return CAENRFID_StatusOK;
}



/****************test_set_RTS*******************/
void
test_set_RTS (CAENRFIDHandle *Handle){
int sercmd, serstat;
 int error;
 extern int errno;
	sercmd = TIOCM_RTS;

	printf("d n'est pas un descripteur de fichier valide.%d\n", EBADF);
	printf("d n'est pas associé avec un fichier spécial en mode caractère.\n    La requête spécifiée ne s'applique pas au type d'objet associé avec le descripteur d.%d\n", ENOTTY);
	printf("La requête ou l'argument argp n'est pas valide.%d\n",EINVAL);


	printf("Setting the RTS pin.\n");
	error=ioctl(*Handle, TIOCMBIS, &sercmd); // Set the RTS pin.
	printf("erreur=%d,%d\n",error,errno);
	printf("sercmd=%d\n",sercmd);
	// Read the RTS pin status.
	ioctl(*Handle, TIOCMGET, &serstat);
	if (serstat & TIOCM_RTS)
		printf("RTS pin is set.\n");
	else
		printf("RTS pin is reset.\n");

	getchar(); // Wait for the return key before continuing.

	printf("Resetting the RTS pin.\n");
	error=ioctl(*Handle, TIOCMBIC, &sercmd); // Reset the RTS pin.
	printf("erreur=%d,%d\n",error,errno);
	printf("sercmd=%d\n",sercmd);

	// Read the RTS pin status.
	ioctl(*Handle, TIOCMGET, &serstat);
	if (serstat & TIOCM_RTS)
		printf("RTS pin is set.\n");
	else
		printf("RTS pin is reset.\n");

}




/****switch_RTS*****/
int switch_RTS (CAENRFIDHandle Handle){
  int sercmd, serstat;
  int error;
  CAENRFIDHandle handle;
  sercmd = TIOCM_RTS;
  handle=LibDescriptor[Handle].handle;
  ioctl(handle, TIOCMGET, &serstat);

  if (serstat & TIOCM_RTS){
    error=ioctl(handle, TIOCMBIC, &sercmd);
  }
  else{
    error=ioctl(handle, TIOCMBIS, &sercmd); // Set the RTS pin high.
  }
  return error;
}

int setRTSHigh(CAENRFIDHandle Handle){ 
  CAENRFIDHandle handle=LibDescriptor[Handle].handle;
  int sercmd=TIOCM_RTS;;
  return(ioctl(handle, TIOCMBIS, &sercmd));
}

int setRTSLow(CAENRFIDHandle Handle){ 
  CAENRFIDHandle handle=LibDescriptor[Handle].handle;
  int sercmd=TIOCM_RTS;;
  return(ioctl(handle, TIOCMBIC, &sercmd));
}

/**********************************************************************\

                CAENRFID_Init

\**********************************************************************/
/*		tcflag_t c_cflag=0;
		c_cflag=parity;
		c_cflag|=CSTOPB;
		c_cflag|=speed|bytesize|CLOCAL //CLOCAL ignore les contrôles de status du modem
 */

CAENRFIDErrorCodes
CAENRFID_Init (CAENRFIDPort CommProtocol, int _port, speed_t speed, char parity, tcflag_t bytesize, CAENRFIDHandle *Handle, CAENRFIDProtocol *Protocol,int usb2serial)
{
	int i;
	CAENRFIDErrorCodes res;
	CAENRFIDHandle handle;

	if(!init_done) /*First Call*/{
		for(i=0 ;i<MAX_CLIENT_SUPPORTED ;i++) LibDescriptor[i].Protocol = -1;
		init_done=1;
	}
	

	for (i = 0; i< MAX_CLIENT_SUPPORTED;i++) {
		if ( LibDescriptor[i].Protocol == -1 ) break;
	}
	if (i == MAX_CLIENT_SUPPORTED) return CAENRFID_GenericError;

	if(CommProtocol == RS232){
		connectRS232 (_port, speed, parity, bytesize, &handle, usb2serial);
		LibDescriptor[i].Protocol = RS232;
		LibDescriptor[i].handle = handle;
		*Handle = i;
		res=CAENRFID_GetProtocol(*Handle,(int *)Protocol);

	
		
		return res;

	}
	ioctl(handle, TIOCMBIC, TIOCM_RTS);

return CAENRFID_PortError;
}


/**********************************************************************\

                CAENRFID_End

\**********************************************************************/

CAENRFIDErrorCodes
CAENRFID_End (CAENRFIDHandle handle)
{
	if ( (handle < 0)  ||
		 (handle >= MAX_CLIENT_SUPPORTED) ) return CAENRFID_InvalidHandleError;
	if ( LibDescriptor[handle].Protocol == -1 ) return CAENRFID_InvalidHandleError;

	if(LibDescriptor[handle].Protocol==RS232) {

		if(!close(LibDescriptor[handle].handle)){
		return CAENRFID_PortError;
		}
		else{
			LibDescriptor[handle].Protocol = -1;
			return CAENRFID_StatusOK;
		}
	}
return CAENRFID_StatusOK;
}

/**********************************************************************\

                SWRelease

\**********************************************************************/

CAENRFIDErrorCodes
CAENRFID_GetSWRelease(char *SwRel)
{
        strcpy(SwRel, LibraryRelease);
        return CAENRFID_StatusOK;
}

/**********************************************************************\

                  FWRelease

\**********************************************************************/

CAENRFIDErrorCodes
CAENRFID_GetFWRelease(CAENRFIDHandle handle,char *FwRel)
{
	IOBuffer_t InBuffer,OutBuffer;
	CAENRFIDErrorCodes err;
	char *lFwRel;

	InitIOBuffer(&InBuffer);
	InitIOBuffer(&OutBuffer);
	if (makeHeaderAVP(&OutBuffer,T11xVer) == 0) return CAENRFID_OutOfMemoryError;
	if (addCommandAVP(&OutBuffer,GETFWRELEASE) == 0) return CAENRFID_OutOfMemoryError;
	if (SendReceivePacket(handle,&OutBuffer,&InBuffer) != CAENRFID_StatusOK ) {
		FreeIOBuffer(&OutBuffer);
		FreeIOBuffer(&InBuffer);
		return CAENRFID_OutOfMemoryError;
	}
	FreeIOBuffer(&OutBuffer);
	lFwRel = getStringAVP(&InBuffer,AVP_GETFWRELEASE,200);
	if (lFwRel == 0) {
		err = (CAENRFIDErrorCodes) getAVPStatus(&InBuffer);
		FreeIOBuffer(&InBuffer);
		return err;
	}
	strncpy(FwRel,lFwRel,200);
	free(lFwRel);
	/*InBuffer.rpos--; patch per correggere un errore nell'implementazione sull'A928*/
	err = (CAENRFIDErrorCodes) getAVPStatus(&InBuffer);
	FreeIOBuffer(&InBuffer);
	return err;
}

/**********************************************************************\

                  Inventory

\**********************************************************************/
CAENRFIDErrorCodes
CAENRFID_Inventory (CAENRFIDHandle handle, char *SourceName, CAENRFIDTag **Receive, int *Size)
{
	IOBuffer_t InBuffer,OutBuffer;
	CAENRFIDErrorCodes err = CAENRFID_StatusOK;
	RFIDTag_t Tag;
	TagArray_t TagList;
	int i;
	*Receive = (CAENRFIDTag *) malloc(58);
	InitIOBuffer(&InBuffer);
	InitIOBuffer(&OutBuffer);
	if (makeHeaderAVP(&OutBuffer,T11xVer) == 0) return CAENRFID_OutOfMemoryError;
	if (addCommandAVP(&OutBuffer,ROWREADID) == 0) return CAENRFID_OutOfMemoryError;
	if (SourceName != NULL) {
		if (addStringAVP(&OutBuffer,SourceName,AVP_SOURCE_NAME) == 0) return CAENRFID_OutOfMemoryError;
	}
	if (SendReceivePacket(handle,&OutBuffer,&InBuffer) != CAENRFID_StatusOK ) {
		FreeIOBuffer(&OutBuffer);
		FreeIOBuffer(&InBuffer);
		return CAENRFID_OutOfMemoryError;
	}

	FreeIOBuffer(&OutBuffer);
	if (minus==0){
		*Receive=NULL;
		*Size=0;
		return CAENRFID_StatusOK;
	}
		
	TagList = getTagsAVP(&InBuffer,SourceName);

	*Size = TagList.size;
	if (TagList.size > 0) {
		*Receive = (CAENRFIDTag *) malloc(TagList.size * sizeof(CAENRFIDTag));
		for (i = 0; i<*Size;i++) {
			Tag = TagArrayRem(&TagList);
			(*Receive)[i].Length=16;
			//(*Receive)[i].Length = Tag.Length;
			memcpy((*Receive)[i].ID,Tag.id,16);
//			printf("tag_id=%s / %s\n",(*Receive)[i].ID,Tag.id);
			strncpy((*Receive)[i].LogicalSource,Tag.LSource,MAX_LOGICAL_SOURCE_NAME);
			strncpy((*Receive)[i].ReadPoint,Tag.ReadPoint,MAX_READPOINT_NAME);
		}
	}
	err = (CAENRFIDErrorCodes) TagList.status;
	FreeIOBuffer(&InBuffer);
	return err;
}

/**********************************************************************\

                  setPower

\**********************************************************************/

CAENRFIDErrorCodes
CAENRFID_SetPower(CAENRFIDHandle handle, unsigned int Power) {

	IOBuffer_t InBuffer,OutBuffer;
	CAENRFIDErrorCodes err;

	InitIOBuffer(&InBuffer);
	InitIOBuffer(&OutBuffer);
	if (makeHeaderAVP(&OutBuffer,T11xVer) == 0)	return CAENRFID_OutOfMemoryError;
	if (addCommandAVP(&OutBuffer,SETPOWER) == 0) return CAENRFID_OutOfMemoryError;
	if (addIntAVP(&OutBuffer,&Power,AVP_POWER) == 0) return CAENRFID_OutOfMemoryError;
	if (SendReceivePacket(handle,&OutBuffer,&InBuffer) != CAENRFID_StatusOK ) {
		FreeIOBuffer(&OutBuffer);
		FreeIOBuffer(&InBuffer);
		return CAENRFID_OutOfMemoryError;
	}
	FreeIOBuffer(&OutBuffer);
	err = (CAENRFIDErrorCodes) getAVPStatus(&InBuffer);
	FreeIOBuffer(&InBuffer);
	return err;
}

/**********************************************************************\

                  Read

\**********************************************************************/


CAENRFIDErrorCodes CAENRFID_Read(CAENRFIDHandle handle, CAENRFIDTag *ID, int Address, int Length, void *Data, int type) {

	IOBuffer_t InBuffer,OutBuffer;
	CAENRFIDErrorCodes err;
	char *lData = NULL;
	unsigned int bouff[8];
	char *str=(char *)malloc(20*sizeof(char));
	
	InitIOBuffer(&InBuffer);
	InitIOBuffer(&OutBuffer);
	if (makeHeaderAVP(&OutBuffer,T11xVer) == 0) return CAENRFID_OutOfMemoryError;
	if (addCommandAVP(&OutBuffer,READTAG) == 0) return CAENRFID_OutOfMemoryError;
	if (addStringAVP(&OutBuffer,(*ID).LogicalSource,AVP_SOURCE_NAME) == 0) return CAENRFID_OutOfMemoryError;
	if (addShortAVP(&OutBuffer,&(*ID).Length,AVP_IDLENGTH) == 0) return CAENRFID_OutOfMemoryError;

	sscanf((*ID).ID,"%02x%02x%02x%02x%02x%02x%02x%02x",&bouff[0],&bouff[1],&bouff[2],&bouff[3],&bouff[4],&bouff[5],&bouff[6],&bouff[7]);
	sprintf(str,"%c%c%c%c%c%c%c%c",bouff[0],bouff[1],bouff[2],bouff[3],bouff[4],bouff[5],bouff[6],bouff[7]);
	if (addDataAVP(&OutBuffer,str,(*ID).Length,AVP_TAGID) == 0) return CAENRFID_OutOfMemoryError;
//	if (addDataAVP(&OutBuffer,(*ID).ID,(*ID).Length,AVP_TAGID) == 0) return CAENRFID_OutOfMemoryError;
	if (addShortAVP(&OutBuffer,&Address,AVP_TAGADDRESS) == 0) return CAENRFID_OutOfMemoryError;
	if (addShortAVP(&OutBuffer,&Length,AVP_LENGTH) == 0) return CAENRFID_OutOfMemoryError;
	
	if (SendReceivePacket(handle,&OutBuffer,&InBuffer) != CAENRFID_StatusOK ) {
		FreeIOBuffer(&OutBuffer);
		FreeIOBuffer(&InBuffer);
		return CAENRFID_CommunicationError;
	}
	FreeIOBuffer(&OutBuffer);
	if (type==0){//mem to read
		if ((lData = getDataAVP_mem(&InBuffer)) == NULL){
			return CAENRFID_CommunicationError;
		}
	}
	else //id to read
		if ((lData = getDataAVP_istag(&InBuffer)) == NULL){
			return CAENRFID_CommunicationError;
		}
		
	if (lData == 0) {
		err = (CAENRFIDErrorCodes) getAVPStatus(&InBuffer);
		FreeIOBuffer(&InBuffer);
		return err;
	}
	memcpy(Data,lData,Length);
	free(lData);
	
	FreeIOBuffer(&InBuffer);
	return CAENRFID_StatusOK;
}


/**********************************************************************\

                  Write

\**********************************************************************/


CAENRFIDErrorCodes
CAENRFID_Write(CAENRFIDHandle handle, CAENRFIDTag *ID, int Address, int Length, void *Data) {

	IOBuffer_t InBuffer,OutBuffer;
	unsigned int bouff[8];
	char *str=(char *)malloc(20*sizeof(char));
	
	InitIOBuffer(&InBuffer);
	InitIOBuffer(&OutBuffer);
	if (makeHeaderAVP(&OutBuffer,T11xVer) == 0) return CAENRFID_OutOfMemoryError;
	if (addCommandAVP(&OutBuffer,WRITETAG) == 0) return CAENRFID_OutOfMemoryError;
	if (addStringAVP(&OutBuffer,(*ID).LogicalSource,AVP_SOURCE_NAME) == 0) return CAENRFID_OutOfMemoryError;
	if (addShortAVP(&OutBuffer,&(*ID).Length,AVP_IDLENGTH) == 0) return CAENRFID_OutOfMemoryError;

	sscanf((*ID).ID,"%02x%02x%02x%02x%02x%02x%02x%02x",&bouff[0],&bouff[1],&bouff[2],&bouff[3],&bouff[4],&bouff[5],&bouff[6],&bouff[7]);
	sprintf(str,"%c%c%c%c%c%c%c%c",bouff[0],bouff[1],bouff[2],bouff[3],bouff[4],bouff[5],bouff[6],bouff[7]);
	if (addDataAVP(&OutBuffer,str,(*ID).Length,AVP_TAGID) == 0) return CAENRFID_OutOfMemoryError;
//	if (addDataAVP(&OutBuffer,(*ID).ID,(*ID).Length,AVP_TAGID) == 0) return CAENRFID_OutOfMemoryError;
	if (addShortAVP(&OutBuffer,&Address,AVP_TAGADDRESS) == 0) return CAENRFID_OutOfMemoryError;
	if (addShortAVP(&OutBuffer,&Length,AVP_LENGTH) == 0) return CAENRFID_OutOfMemoryError;


	if (addDataAVP(&OutBuffer,Data,Length,AVP_TAG_VALUE) == 0) 	return CAENRFID_OutOfMemoryError;
	if (SendReceivePacket(handle,&OutBuffer,&InBuffer) != CAENRFID_StatusOK ) {
		FreeIOBuffer(&OutBuffer);
		FreeIOBuffer(&InBuffer);
		return CAENRFID_CommunicationError;
	}
	FreeIOBuffer(&OutBuffer);
	FreeIOBuffer(&InBuffer);
	return CAENRFID_StatusOK;
}

/**********************************************************************\

                  Lock

\**********************************************************************/


CAENRFIDErrorCodes
CAENRFID_Lock(CAENRFIDHandle handle, CAENRFIDTag *ID, short Address) {

	IOBuffer_t InBuffer,OutBuffer;
	CAENRFIDErrorCodes err;
	InitIOBuffer(&InBuffer);
	InitIOBuffer(&OutBuffer);
	if (makeHeaderAVP(&OutBuffer,T11xVer) == 0) return CAENRFID_OutOfMemoryError;
	if (addCommandAVP(&OutBuffer,LOCKTAG) == 0) return CAENRFID_OutOfMemoryError;
	if (addStringAVP(&OutBuffer,(*ID).LogicalSource,AVP_SOURCE_NAME) == 0) return CAENRFID_OutOfMemoryError;
	if (addShortAVP(&OutBuffer,&(*ID).Length,AVP_IDLENGTH) == 0) return CAENRFID_OutOfMemoryError;
	if (addDataAVP(&OutBuffer,(*ID).ID,(*ID).Length,AVP_TAGID) == 0) return CAENRFID_OutOfMemoryError;
	if (addShortAVP(&OutBuffer,&Address,AVP_TAGADDRESS) == 0) return CAENRFID_OutOfMemoryError;
	if (SendReceivePacket(handle,&OutBuffer,&InBuffer) != CAENRFID_StatusOK ) {
		FreeIOBuffer(&OutBuffer);
		FreeIOBuffer(&InBuffer);
		return CAENRFID_CommunicationError;
	}
	FreeIOBuffer(&OutBuffer);
	err = (CAENRFIDErrorCodes) getAVPStatus(&InBuffer);
	FreeIOBuffer(&InBuffer);
	return err;
}

/**********************************************************************\

                  AddReadPoint

\**********************************************************************/
CAENRFIDErrorCodes
CAENRFID_AddReadPoint(CAENRFIDHandle handle, char *SourceName, char *ReadPoint) {

	IOBuffer_t InBuffer,OutBuffer;
	int err;

	InitIOBuffer(&InBuffer);
	InitIOBuffer(&OutBuffer);
	if (makeHeaderAVP(&OutBuffer,T11xVer) == 0)	return CAENRFID_OutOfMemoryError;
	if (addCommandAVP(&OutBuffer,ADDREADPOINT) == 0) return CAENRFID_OutOfMemoryError;
	if (addStringAVP(&OutBuffer,SourceName,AVP_SOURCE_NAME) == 0) return CAENRFID_OutOfMemoryError;
	if (addStringAVP(&OutBuffer,ReadPoint,AVP_READPOINT_NAME) == 0) return CAENRFID_OutOfMemoryError;
	if (SendReceivePacket(handle,&OutBuffer,&InBuffer) != CAENRFID_StatusOK ) {
		FreeIOBuffer(&OutBuffer);
		FreeIOBuffer(&InBuffer);
		return CAENRFID_OutOfMemoryError;
	}
	FreeIOBuffer(&OutBuffer);
	err = getAVPStatus(&InBuffer);
	FreeIOBuffer(&InBuffer);
	return (CAENRFIDErrorCodes) err;
}


/**********************************************************************\

                  RemoveReadPoint

\**********************************************************************/

CAENRFIDErrorCodes
CAENRFID_RemoveReadPoint(CAENRFIDHandle handle, char *SourceName, char *ReadPoint) {

	IOBuffer_t InBuffer,OutBuffer;
	int err;

	InitIOBuffer(&InBuffer);
	InitIOBuffer(&OutBuffer);
	if (makeHeaderAVP(&OutBuffer,T11xVer) == 0)	return CAENRFID_OutOfMemoryError;
	if (addCommandAVP(&OutBuffer,REMOVEREADPOINT) == 0) return CAENRFID_OutOfMemoryError;
	if (addStringAVP(&OutBuffer,SourceName,AVP_SOURCE_NAME) == 0) return CAENRFID_OutOfMemoryError;
	if (addStringAVP(&OutBuffer,ReadPoint,AVP_READPOINT_NAME) == 0) return CAENRFID_OutOfMemoryError;
	if (SendReceivePacket(handle,&OutBuffer,&InBuffer) != CAENRFID_StatusOK ) {
		FreeIOBuffer(&OutBuffer);
		FreeIOBuffer(&InBuffer);
		return CAENRFID_OutOfMemoryError;
	}
	FreeIOBuffer(&OutBuffer);
	err = getAVPStatus(&InBuffer);
	FreeIOBuffer(&InBuffer);
	return (CAENRFIDErrorCodes) err;
}


/**********************************************************************\

                  getPower

\**********************************************************************/
CAENRFIDErrorCodes
CAENRFID_GetPower (CAENRFIDHandle handle, unsigned int *power)
{
	IOBuffer_t InBuffer,OutBuffer;
	int err;
	int p;

	InitIOBuffer(&InBuffer);
	InitIOBuffer(&OutBuffer);
	if (makeHeaderAVP(&OutBuffer,T11xVer) == 0) return CAENRFID_OutOfMemoryError;
	if (addCommandAVP(&OutBuffer,GETPOWER) == 0) return CAENRFID_OutOfMemoryError;
	if (SendReceivePacket(handle,&OutBuffer,&InBuffer) != CAENRFID_StatusOK ) {
		FreeIOBuffer(&OutBuffer);
		FreeIOBuffer(&InBuffer);
		return CAENRFID_OutOfMemoryError;
	}
	FreeIOBuffer(&OutBuffer);
	p = getIntAVP(&InBuffer,AVP_POWER_GET);
	if (p < 0) {
		FreeIOBuffer(&InBuffer);
		return CAENRFID_GenericError;
	}
	*power = p;
	err = getAVPStatus(&InBuffer);
	FreeIOBuffer(&InBuffer);
	return (CAENRFIDErrorCodes)err;
}


/**********************************************************************\

                  ChangeProtocol

\**********************************************************************/
CAENRFIDErrorCodes
CAENRFID_SetProtocol (CAENRFIDHandle handle, CAENRFIDProtocol protocol) {

	IOBuffer_t InBuffer,OutBuffer;
	int err;

	InitIOBuffer(&InBuffer);
	InitIOBuffer(&OutBuffer);
	if (makeHeaderAVP(&OutBuffer,T11xVer) == 0)	return CAENRFID_OutOfMemoryError;
	if (addCommandAVP(&OutBuffer,CHANGEPROTOCOL) == 0) return CAENRFID_OutOfMemoryError;
	if (addIntAVP(&OutBuffer,&protocol,AVP_PROTOCOL_NAME) == 0) return CAENRFID_OutOfMemoryError;
	if (SendReceivePacket(handle,&OutBuffer,&InBuffer) != CAENRFID_StatusOK ) {
		FreeIOBuffer(&OutBuffer);
		FreeIOBuffer(&InBuffer);
		return CAENRFID_OutOfMemoryError;
	}
	FreeIOBuffer(&OutBuffer);
	err = getAVPStatus(&InBuffer);
	FreeIOBuffer(&InBuffer);
	return (CAENRFIDErrorCodes)err;
}

/**********************************************************************\

                  CheckReadPointStatus

\**********************************************************************/
CAENRFIDErrorCodes
CAENRFID_GetReadPointStatus(CAENRFIDHandle handle, char *ReadPoint , CAENRFIDReadPointStatus *status) {

	IOBuffer_t InBuffer,OutBuffer;
	int err;

	InitIOBuffer(&InBuffer);
	InitIOBuffer(&OutBuffer);
	if (makeHeaderAVP(&OutBuffer,T11xVer) == 0)	return CAENRFID_OutOfMemoryError;
	if (addCommandAVP(&OutBuffer,CHECKREADPOINTSTATUS) == 0) return CAENRFID_OutOfMemoryError;
	if (addStringAVP(&OutBuffer,ReadPoint,AVP_READPOINT_NAME) == 0) return CAENRFID_OutOfMemoryError;
	if (SendReceivePacket(handle,&OutBuffer,&InBuffer) != CAENRFID_StatusOK ) {
		FreeIOBuffer(&OutBuffer);
		FreeIOBuffer(&InBuffer);
		return CAENRFID_OutOfMemoryError;
	}
	FreeIOBuffer(&OutBuffer);
	*status = (CAENRFIDReadPointStatus)getIntAVP(&InBuffer,AVP_READPOINT_STATUS);
	if (*status < 0) {
		FreeIOBuffer(&InBuffer);
		return CAENRFID_GenericError;
	}
	err = getAVPStatus(&InBuffer);
	FreeIOBuffer(&InBuffer);
	return (CAENRFIDErrorCodes)err;
}


/**********************************************************************\

                  CheckReadPointInSource

\**********************************************************************/

CAENRFIDErrorCodes
CAENRFID_GetReadPointInSource(CAENRFIDHandle handle, char *ReadPoint, char *SourceName, short *isPresent) {

	IOBuffer_t InBuffer,OutBuffer;
	int err;

	InitIOBuffer(&InBuffer);
	InitIOBuffer(&OutBuffer);
	if (makeHeaderAVP(&OutBuffer,T11xVer) == 0)	return CAENRFID_OutOfMemoryError;
	if (addCommandAVP(&OutBuffer,CHECKREADPOINTINSOURCE) == 0) return CAENRFID_OutOfMemoryError;
	if (addStringAVP(&OutBuffer,ReadPoint,AVP_READPOINT_NAME) == 0) return CAENRFID_OutOfMemoryError;
	if (addStringAVP(&OutBuffer,SourceName,AVP_SOURCE_NAME) == 0) return CAENRFID_OutOfMemoryError;
	if (SendReceivePacket(handle,&OutBuffer,&InBuffer) != CAENRFID_StatusOK ) {
		FreeIOBuffer(&OutBuffer);
		FreeIOBuffer(&InBuffer);
		return CAENRFID_OutOfMemoryError;
	}
	FreeIOBuffer(&OutBuffer);
	*isPresent = getShortAVP(&InBuffer,AVP_BOOLEAN);
	if (*isPresent < 0) {
		FreeIOBuffer(&InBuffer);
		return CAENRFID_GenericError;
	}
	err = getAVPStatus(&InBuffer);
	FreeIOBuffer(&InBuffer);
	return (CAENRFIDErrorCodes)err;
}


/**********************************************************************\

                  GetProtocol

\**********************************************************************/

CAENRFIDErrorCodes
CAENRFID_GetProtocol(CAENRFIDHandle handle, int *Proto) {

	IOBuffer_t InBuffer,OutBuffer;
	int err;

	InitIOBuffer(&InBuffer);
	InitIOBuffer(&OutBuffer);
	if (makeHeaderAVP(&OutBuffer,T11xVer) == 0) return CAENRFID_OutOfMemoryError;
	if (addCommandAVP(&OutBuffer,GETPROTOCOL) == 0) return CAENRFID_OutOfMemoryError;
	if (SendReceivePacket(handle,&OutBuffer,&InBuffer) != CAENRFID_StatusOK ) {
		FreeIOBuffer(&OutBuffer);
		FreeIOBuffer(&InBuffer);
		return CAENRFID_OutOfMemoryError;
	}
	FreeIOBuffer(&OutBuffer);
	*Proto = getIntAVP(&InBuffer,AVP_PROTOCOL_NAME);
	if (*Proto < 0) {
		FreeIOBuffer(&InBuffer);
		return CAENRFID_GenericError;
	}
	err = getAVPStatus(&InBuffer);
	return (CAENRFIDErrorCodes)err;
}




/**********************************************************************\

                  GETMODULATION

\**********************************************************************/
CAENRFIDErrorCodes
CAENRFID_GetModulation(CAENRFIDHandle handle, unsigned short *TxRx) {

	IOBuffer_t InBuffer,OutBuffer;
	int err;

	InitIOBuffer(&InBuffer);
	InitIOBuffer(&OutBuffer);
	if (makeHeaderAVP(&OutBuffer,T11xVer) == 0) return CAENRFID_OutOfMemoryError;
	if (addCommandAVP(&OutBuffer,GETMODULATION) == 0) return CAENRFID_OutOfMemoryError;
	if (SendReceivePacket(handle,&OutBuffer,&InBuffer) != CAENRFID_StatusOK ) {
		FreeIOBuffer(&OutBuffer);
		FreeIOBuffer(&InBuffer);
		return CAENRFID_OutOfMemoryError;
	}
	FreeIOBuffer(&OutBuffer);
	*TxRx = getShortAVP(&InBuffer,AVP_MODULATION);
	err = getAVPStatus(&InBuffer);
	return (CAENRFIDErrorCodes)err;
}

/**********************************************************************\

                  BlockWrite

\**********************************************************************/


CAENRFIDErrorCodes
CAENRFID_BlockWrite(CAENRFIDHandle handle, CAENRFIDTag *ID, int Address, int Length, void *Data) {

	IOBuffer_t InBuffer,OutBuffer;
	int err;

	InitIOBuffer(&InBuffer);
	InitIOBuffer(&OutBuffer);
	if (makeHeaderAVP(&OutBuffer,T11xVer) == 0) return CAENRFID_OutOfMemoryError;
	if (addCommandAVP(&OutBuffer,BLOCKWRITETAG) == 0) return CAENRFID_OutOfMemoryError;
	if (addStringAVP(&OutBuffer,(*ID).LogicalSource,AVP_SOURCE_NAME) == 0) return CAENRFID_OutOfMemoryError;
	if (addShortAVP(&OutBuffer,&(*ID).Length,AVP_IDLENGTH) == 0) return CAENRFID_OutOfMemoryError;
	if (addDataAVP(&OutBuffer,(*ID).ID,(*ID).Length,AVP_TAGID) == 0) return CAENRFID_OutOfMemoryError;
	if (addShortAVP(&OutBuffer,&Address,AVP_TAGADDRESS) == 0) return CAENRFID_OutOfMemoryError;
	if (addShortAVP(&OutBuffer,&Length,AVP_LENGTH) == 0) return CAENRFID_OutOfMemoryError;
	if (addDataAVP(&OutBuffer,Data,Length,AVP_TAG_VALUE) == 0) 	return CAENRFID_OutOfMemoryError;
	if (SendReceivePacket(handle,&OutBuffer,&InBuffer) != CAENRFID_StatusOK ) {
		FreeIOBuffer(&OutBuffer);
		FreeIOBuffer(&InBuffer);
		return CAENRFID_CommunicationError;
	}
	FreeIOBuffer(&OutBuffer);
	err = getAVPStatus(&InBuffer);
	FreeIOBuffer(&InBuffer);
	return (CAENRFIDErrorCodes)err;
}

/**********************************************************************\

                  SetRS232

\**********************************************************************/

CAENRFIDErrorCodes
CAENRFID_SetRS232(CAENRFIDHandle handle, unsigned long baud, unsigned long datab, unsigned long stopb, CAENRFID_RS232_Parity parity, CAENRFID_RS232_FlowControl flowc) {

	
	IOBuffer_t InBuffer,OutBuffer;
	int err;

	InitIOBuffer(&InBuffer);
	InitIOBuffer(&OutBuffer);
	if (makeHeaderAVP(&OutBuffer,T11xVer) == 0)	return CAENRFID_OutOfMemoryError;
	if (addCommandAVP(&OutBuffer,SETRS232) == 0) return CAENRFID_OutOfMemoryError;
	if (addIntAVP(&OutBuffer,&baud,AVP_BAUDRATE) == 0) return CAENRFID_OutOfMemoryError;
	if (addIntAVP(&OutBuffer,&datab,AVP_DATABIT) == 0) return CAENRFID_OutOfMemoryError;
	if (addIntAVP(&OutBuffer,&stopb,AVP_STOPBITS) == 0) return CAENRFID_OutOfMemoryError;
	if (addIntAVP(&OutBuffer,&parity,AVP_PARITY) == 0) return CAENRFID_OutOfMemoryError;
	if (addIntAVP(&OutBuffer,&flowc,AVP_FLOWCONTROL) == 0) return CAENRFID_OutOfMemoryError;
	if (SendReceivePacket(handle,&OutBuffer,&InBuffer) != CAENRFID_StatusOK ) {
		FreeIOBuffer(&OutBuffer);
		FreeIOBuffer(&InBuffer);
		return CAENRFID_OutOfMemoryError;
	}
	FreeIOBuffer(&OutBuffer);
	err = getAVPStatus(&InBuffer);
	FreeIOBuffer(&InBuffer);
	return (CAENRFIDErrorCodes)err;
}
/**********************************************************************\

                  SetDateTime

\**********************************************************************/

CAENRFIDErrorCodes
CAENRFID_SetDateTime(CAENRFIDHandle handle, char *time) {

	IOBuffer_t InBuffer,OutBuffer;
	int err;

	InitIOBuffer(&InBuffer);
	InitIOBuffer(&OutBuffer);
	if (makeHeaderAVP(&OutBuffer,T11xVer) == 0)	return CAENRFID_OutOfMemoryError;
	if (addCommandAVP(&OutBuffer,SETDATETIME) == 0) return CAENRFID_OutOfMemoryError;
	if (addDataAVP(&OutBuffer,time,TIMELENGTH,AVP_DATETIME) == 0) return CAENRFID_OutOfMemoryError;
	if (SendReceivePacket(handle,&OutBuffer,&InBuffer) != CAENRFID_StatusOK ) {
		FreeIOBuffer(&OutBuffer);
		FreeIOBuffer(&InBuffer);
		return CAENRFID_OutOfMemoryError;
	}
	FreeIOBuffer(&OutBuffer);
	err = getAVPStatus(&InBuffer);
	FreeIOBuffer(&InBuffer);
	return (CAENRFIDErrorCodes)err;
}



/**********************************************************************\

                  GetIO

\**********************************************************************/

CAENRFIDErrorCodes
CAENRFID_GetIO(CAENRFIDHandle handle, unsigned int *IORegister) {

	IOBuffer_t InBuffer,OutBuffer;
	int err;

	InitIOBuffer(&InBuffer);
	InitIOBuffer(&OutBuffer);
	if (makeHeaderAVP(&OutBuffer,T11xVer) == 0) return CAENRFID_OutOfMemoryError;
	if (addCommandAVP(&OutBuffer,GETIO) == 0) return CAENRFID_OutOfMemoryError;
	if (SendReceivePacket(handle,&OutBuffer,&InBuffer) != CAENRFID_StatusOK ) {
		FreeIOBuffer(&OutBuffer);
		FreeIOBuffer(&InBuffer);
		return CAENRFID_OutOfMemoryError;
	}
	FreeIOBuffer(&OutBuffer);
	*IORegister = getIntAVP(&InBuffer,AVP_IOREGISTER);
	err = getAVPStatus(&InBuffer);
	return (CAENRFIDErrorCodes)err;
}

/**********************************************************************\

                  SetIO

\**********************************************************************/

CAENRFIDErrorCodes
CAENRFID_SetIO(CAENRFIDHandle handle, unsigned int IORegister) {

	IOBuffer_t InBuffer,OutBuffer;
	int err;

	InitIOBuffer(&InBuffer);
	InitIOBuffer(&OutBuffer);
	if (makeHeaderAVP(&OutBuffer,T11xVer) == 0)	return CAENRFID_OutOfMemoryError;
	if (addCommandAVP(&OutBuffer,SETIO) == 0) return CAENRFID_OutOfMemoryError;
	if (addIntAVP(&OutBuffer,&IORegister,AVP_IOREGISTER) == 0) return CAENRFID_OutOfMemoryError;
	if (SendReceivePacket(handle,&OutBuffer,&InBuffer) != CAENRFID_StatusOK ) {
		FreeIOBuffer(&OutBuffer);
		FreeIOBuffer(&InBuffer);
		return CAENRFID_OutOfMemoryError;
	}
	FreeIOBuffer(&OutBuffer);
	err = getAVPStatus(&InBuffer);
	FreeIOBuffer(&InBuffer);
	return (CAENRFIDErrorCodes)err;
}


/**********************************************************************\

                  GetIODirection

\**********************************************************************/

CAENRFIDErrorCodes
CAENRFID_GetIODirection(CAENRFIDHandle handle, unsigned int *IODirection) {

	IOBuffer_t InBuffer,OutBuffer;
	int err;

	InitIOBuffer(&InBuffer);
	InitIOBuffer(&OutBuffer);
	if (makeHeaderAVP(&OutBuffer,T11xVer) == 0) return CAENRFID_OutOfMemoryError;
	if (addCommandAVP(&OutBuffer,GETIODIRECTION) == 0) return CAENRFID_OutOfMemoryError;
	if (SendReceivePacket(handle,&OutBuffer,&InBuffer) != CAENRFID_StatusOK ) {
		FreeIOBuffer(&OutBuffer);
		FreeIOBuffer(&InBuffer);
		return CAENRFID_OutOfMemoryError;
	}
	FreeIOBuffer(&OutBuffer);
	*IODirection = getIntAVP(&InBuffer,AVP_IOREGISTER);
	err = getAVPStatus(&InBuffer);  
	return (CAENRFIDErrorCodes)err; 
}

/**********************************************************************\

                  SetIODirection

\**********************************************************************/

CAENRFIDErrorCodes
CAENRFID_SetIODirection(CAENRFIDHandle handle, unsigned int IODirection) {

	IOBuffer_t InBuffer,OutBuffer;
	int err;

	InitIOBuffer(&InBuffer);
	InitIOBuffer(&OutBuffer);
	if (makeHeaderAVP(&OutBuffer,T11xVer) == 0)	return CAENRFID_OutOfMemoryError;
	if (addCommandAVP(&OutBuffer,SETIODIRECTION) == 0) return CAENRFID_OutOfMemoryError;
	if (addIntAVP(&OutBuffer,&IODirection,AVP_IOREGISTER) == 0) return CAENRFID_OutOfMemoryError;
	if (SendReceivePacket(handle,&OutBuffer,&InBuffer) != CAENRFID_StatusOK ) {
		FreeIOBuffer(&OutBuffer);
		FreeIOBuffer(&InBuffer);
		return CAENRFID_OutOfMemoryError;
	}
	FreeIOBuffer(&OutBuffer);
	err = getAVPStatus(&InBuffer);
	FreeIOBuffer(&InBuffer);
	return (CAENRFIDErrorCodes)err;
}

/**********************************************************************\

                  CAENRFID_SetSourceConfiguration

\**********************************************************************/


CAENRFIDErrorCodes
CAENRFID_SetSourceConfiguration(CAENRFIDHandle handle, char *SourceName, CAENRFID_SOURCE_Parameter parameter, int pvalue){

	IOBuffer_t InBuffer,OutBuffer;
	int err;

	InitIOBuffer(&InBuffer);
	InitIOBuffer(&OutBuffer);
	if (makeHeaderAVP(&OutBuffer,T11xVer) == 0)	return CAENRFID_OutOfMemoryError;
	if (addCommandAVP(&OutBuffer,SETSOURCECONFIGURATION) == 0) return CAENRFID_OutOfMemoryError;
	if (addStringAVP(&OutBuffer,SourceName,AVP_SOURCE_NAME) == 0) return CAENRFID_OutOfMemoryError;
	if (addIntAVP(&OutBuffer,&parameter,AVP_CONFIGPARAMETER) == 0) return CAENRFID_OutOfMemoryError;
	if (addIntAVP(&OutBuffer,&pvalue,AVP_CONFIGVALUE) == 0) return CAENRFID_OutOfMemoryError;
	if (SendReceivePacket(handle,&OutBuffer,&InBuffer) != CAENRFID_StatusOK ) {
		FreeIOBuffer(&OutBuffer);
		FreeIOBuffer(&InBuffer);
		return CAENRFID_OutOfMemoryError;
	}
	FreeIOBuffer(&OutBuffer);
	err = getAVPStatus(&InBuffer);
	FreeIOBuffer(&InBuffer);
	return (CAENRFIDErrorCodes)err;
}


/**********************************************************************\

                  CAENRFID_GetSourceConfiguration

\**********************************************************************/


CAENRFIDErrorCodes
CAENRFID_GetSourceConfiguration(CAENRFIDHandle handle, char *SourceName, CAENRFID_SOURCE_Parameter parameter, int *pvalue){

	IOBuffer_t InBuffer,OutBuffer;
	int err;

	InitIOBuffer(&InBuffer);
	InitIOBuffer(&OutBuffer);
	if (makeHeaderAVP(&OutBuffer,T11xVer) == 0)	return CAENRFID_OutOfMemoryError;
	if (addCommandAVP(&OutBuffer,GETSOURCECONFIGURATION) == 0) return CAENRFID_OutOfMemoryError;
	if (addStringAVP(&OutBuffer,SourceName,AVP_SOURCE_NAME) == 0) return CAENRFID_OutOfMemoryError;
	if (addIntAVP(&OutBuffer,&parameter,AVP_CONFIGPARAMETER) == 0) return CAENRFID_OutOfMemoryError;
	if (SendReceivePacket(handle,&OutBuffer,&InBuffer) != CAENRFID_StatusOK ) {
		FreeIOBuffer(&OutBuffer);
		FreeIOBuffer(&InBuffer);
		return CAENRFID_OutOfMemoryError;
	}
	FreeIOBuffer(&OutBuffer);
	*pvalue = getIntAVP(&InBuffer,AVP_CONFIGVALUE);
	if (*pvalue < 0) {
		FreeIOBuffer(&InBuffer);
		return CAENRFID_GenericError;
	}
	err = getAVPStatus(&InBuffer);
	return (CAENRFIDErrorCodes)err;
}


/**********************************************************************\

                  SetEventMode

\**********************************************************************/

CAENRFIDErrorCodes
CAENRFID_SetEventMode(CAENRFIDHandle handle, CAENRFID_EventMode EMode) {

	IOBuffer_t InBuffer,OutBuffer;
	int err;

	InitIOBuffer(&InBuffer);
	InitIOBuffer(&OutBuffer);
	if (makeHeaderAVP(&OutBuffer,T11xVer) == 0)	return CAENRFID_OutOfMemoryError;
	if (addCommandAVP(&OutBuffer,SETEVENTMODE) == 0) return CAENRFID_OutOfMemoryError;
	if (addShortAVP(&OutBuffer,&EMode,AVP_EVENTMODE) == 0) return CAENRFID_OutOfMemoryError;
	if (SendReceivePacket(handle,&OutBuffer,&InBuffer) != CAENRFID_StatusOK ) {
		FreeIOBuffer(&OutBuffer);
		FreeIOBuffer(&InBuffer);
		return CAENRFID_OutOfMemoryError;
	}
	FreeIOBuffer(&OutBuffer);
	err = getAVPStatus(&InBuffer);
	FreeIOBuffer(&InBuffer);
	return (CAENRFIDErrorCodes)err;
}


/**********************************************************************\

                  GetEventMode

\**********************************************************************/

CAENRFIDErrorCodes
CAENRFID_GetEventMode(CAENRFIDHandle handle, CAENRFID_EventMode *EMode) {

	IOBuffer_t InBuffer,OutBuffer;
	int err;

	InitIOBuffer(&InBuffer);
	InitIOBuffer(&OutBuffer);
	if (makeHeaderAVP(&OutBuffer,T11xVer) == 0) return CAENRFID_OutOfMemoryError;
	if (addCommandAVP(&OutBuffer,GETEVENTMODE) == 0) return CAENRFID_OutOfMemoryError;
	if (SendReceivePacket(handle,&OutBuffer,&InBuffer) != CAENRFID_StatusOK ) {
		FreeIOBuffer(&OutBuffer);
		FreeIOBuffer(&InBuffer);
		return CAENRFID_OutOfMemoryError;
	}
	FreeIOBuffer(&OutBuffer);
	*EMode = (CAENRFID_EventMode)getShortAVP(&InBuffer,AVP_EVENTMODE);
	if (*EMode < 0) {
		FreeIOBuffer(&InBuffer);
		return CAENRFID_GenericError;
	}
	err = getAVPStatus(&InBuffer);
	return (CAENRFIDErrorCodes)err;
}

/**********************************************************************\

                  CAENRFID_GetReaderInfo

\**********************************************************************/

CAENRFIDErrorCodes
CAENRFID_GetReaderInfo(CAENRFIDHandle handle,char *Model, char *SerialNum)
{
	IOBuffer_t InBuffer,OutBuffer;
	int err;
	char *lInfo;
	int i = 0;

	InitIOBuffer(&InBuffer);
	InitIOBuffer(&OutBuffer);
	if (makeHeaderAVP(&OutBuffer,T11xVer) == 0) return CAENRFID_OutOfMemoryError;
	if (addCommandAVP(&OutBuffer,GETREADERINFO) == 0) return CAENRFID_OutOfMemoryError;
	if (SendReceivePacket(handle,&OutBuffer,&InBuffer) != CAENRFID_StatusOK ) {
		FreeIOBuffer(&OutBuffer);
		FreeIOBuffer(&InBuffer);
		return CAENRFID_OutOfMemoryError;
	}
	FreeIOBuffer(&OutBuffer);
	lInfo = getStringAVP(&InBuffer,AVP_READERINFO,MAX_CHAR_DESCRIPTION);
	if (lInfo == 0) {
		err = getAVPStatus(&InBuffer);
		FreeIOBuffer(&InBuffer);
		return (CAENRFIDErrorCodes)err;
	}
	/*InBuffer.rpos--; patch per correggere errore on firmware A928*/
	err = getAVPStatus(&InBuffer);
	i = strcspn (lInfo," ");
	if (i != 0) {
		strncpy(Model,lInfo,i);
		Model[i] = 0;
		strncpy(SerialNum,lInfo+i+1,strlen(lInfo)-i);
	}
	free(lInfo);
	FreeIOBuffer(&InBuffer);
	return (CAENRFIDErrorCodes)err;
}

/**********************************************************************\

                  CAENRFID_FreeTagMemory

\**********************************************************************/

void
CAENRFID_FreeTagsMemory(CAENRFIDTag **Tags) {
	
	if (*Tags != NULL)  free(*Tags);
}

/**********************************************************************\

                  CAENRFID_FreeNotifyMemory

\**********************************************************************/

void
CAENRFID_FreeNotifyMemory(CAENRFIDNotify **ItemsNotified) {
	
	if (*ItemsNotified != NULL)  free(*ItemsNotified);
}


/**********************************************************************\

                  GETLBTMode

\**********************************************************************/
CAENRFIDErrorCodes
CAENRFID_GetLBTMode(CAENRFIDHandle handle, unsigned short *LBTMode) {

	IOBuffer_t InBuffer,OutBuffer;
	int err;

	InitIOBuffer(&InBuffer);
	InitIOBuffer(&OutBuffer);
	if (makeHeaderAVP(&OutBuffer,T11xVer) == 0) return CAENRFID_OutOfMemoryError;
	if (addCommandAVP(&OutBuffer,GETLBTMODE) == 0) return CAENRFID_OutOfMemoryError;
	if (SendReceivePacket(handle,&OutBuffer,&InBuffer) != CAENRFID_StatusOK ) {
		FreeIOBuffer(&OutBuffer);
		FreeIOBuffer(&InBuffer);
		return CAENRFID_OutOfMemoryError;
	}
	FreeIOBuffer(&OutBuffer);
	*LBTMode = getShortAVP(&InBuffer,AVP_BOOLEAN);
	err = getAVPStatus(&InBuffer);
	return (CAENRFIDErrorCodes)err;
}

/**********************************************************************\

                  setLBTMode

\**********************************************************************/

CAENRFIDErrorCodes
CAENRFID_SetLBTMode(CAENRFIDHandle handle, unsigned short LBTMode) {

	IOBuffer_t InBuffer,OutBuffer;
	int err;

	InitIOBuffer(&InBuffer);
	InitIOBuffer(&OutBuffer);
	if (makeHeaderAVP(&OutBuffer,T11xVer) == 0)	return CAENRFID_OutOfMemoryError;
	if (addCommandAVP(&OutBuffer,SETLBTMODE) == 0) return CAENRFID_OutOfMemoryError;
	if (addShortAVP(&OutBuffer,&LBTMode,AVP_BOOLEAN) == 0) return CAENRFID_OutOfMemoryError;
	if (SendReceivePacket(handle,&OutBuffer,&InBuffer) != CAENRFID_StatusOK ) {
		FreeIOBuffer(&OutBuffer);
		FreeIOBuffer(&InBuffer);
		return CAENRFID_OutOfMemoryError;
	}
	FreeIOBuffer(&OutBuffer);
	err = getAVPStatus(&InBuffer);
	FreeIOBuffer(&InBuffer);
	return (CAENRFIDErrorCodes)err;
}

/**********************************************************************\

                  GetRFChannel

\**********************************************************************/
CAENRFIDErrorCodes
CAENRFID_GetRFChannel(CAENRFIDHandle handle, unsigned short *RFChannel) {

	IOBuffer_t InBuffer,OutBuffer;
	int err;

	InitIOBuffer(&InBuffer);
	InitIOBuffer(&OutBuffer);
	if (makeHeaderAVP(&OutBuffer,T11xVer) == 0) return CAENRFID_OutOfMemoryError;
	if (addCommandAVP(&OutBuffer,GETCHANNEL) == 0) return CAENRFID_OutOfMemoryError;
	if (SendReceivePacket(handle,&OutBuffer,&InBuffer) != CAENRFID_StatusOK ) {
		FreeIOBuffer(&OutBuffer);
		FreeIOBuffer(&InBuffer);
		return CAENRFID_OutOfMemoryError;
	}
	FreeIOBuffer(&OutBuffer);
	*RFChannel = getShortAVP(&InBuffer,AVP_RFCHANNEL);
	err = getAVPStatus(&InBuffer);
	return (CAENRFIDErrorCodes)err;
}

/**********************************************************************\

                  SetRFChannel

\**********************************************************************/

CAENRFIDErrorCodes
CAENRFID_SetRFChannel(CAENRFIDHandle handle, unsigned short RFChannel) {

	IOBuffer_t InBuffer,OutBuffer;
	int err;

	InitIOBuffer(&InBuffer);
	InitIOBuffer(&OutBuffer);
	if (makeHeaderAVP(&OutBuffer,T11xVer) == 0)	return CAENRFID_OutOfMemoryError;
	if (addCommandAVP(&OutBuffer,SETCHANNEL) == 0) return CAENRFID_OutOfMemoryError;
	if (addShortAVP(&OutBuffer,&RFChannel,AVP_RFCHANNEL) == 0) return CAENRFID_OutOfMemoryError;
	if (SendReceivePacket(handle,&OutBuffer,&InBuffer) != CAENRFID_StatusOK ) {
		FreeIOBuffer(&OutBuffer);
		FreeIOBuffer(&InBuffer);
		return CAENRFID_OutOfMemoryError;
	}
	FreeIOBuffer(&OutBuffer);
	err = getAVPStatus(&InBuffer);
	FreeIOBuffer(&InBuffer);
	return (CAENRFIDErrorCodes)err;
}


/**********************************************************************\

                  GetRFRegulation

\**********************************************************************/
CAENRFIDErrorCodes
CAENRFID_GetRFRegulation(CAENRFIDHandle handle, unsigned short *RFRegulation) {

	IOBuffer_t InBuffer,OutBuffer;
	int err;

	InitIOBuffer(&InBuffer);
	InitIOBuffer(&OutBuffer);
	if (makeHeaderAVP(&OutBuffer,T11xVer) == 0) return CAENRFID_OutOfMemoryError;
	if (addCommandAVP(&OutBuffer,GETRFREGULATION) == 0) return CAENRFID_OutOfMemoryError;
	if (SendReceivePacket(handle,&OutBuffer,&InBuffer) != CAENRFID_StatusOK ) {
		FreeIOBuffer(&OutBuffer);
		FreeIOBuffer(&InBuffer);
		return CAENRFID_OutOfMemoryError;
	}
	FreeIOBuffer(&OutBuffer);
	*RFRegulation = getShortAVP(&InBuffer,AVP_RFREGULATION);
	err = getAVPStatus(&InBuffer);
	return (CAENRFIDErrorCodes)err;
}

/**********************************************************************\

                  SetRFRegulation

\**********************************************************************/

CAENRFIDErrorCodes
CAENRFID_SetRFRegulation(CAENRFIDHandle handle, unsigned short RFRegulation) {

	IOBuffer_t InBuffer,OutBuffer;
	int err;

	InitIOBuffer(&InBuffer);
	InitIOBuffer(&OutBuffer);
	if (makeHeaderAVP(&OutBuffer,T11xVer) == 0)	return CAENRFID_OutOfMemoryError;
	if (addCommandAVP(&OutBuffer,SETRFREGULATION) == 0) return CAENRFID_OutOfMemoryError;
	if (addShortAVP(&OutBuffer,&RFRegulation,AVP_RFREGULATION) == 0) return CAENRFID_OutOfMemoryError;
	if (SendReceivePacket(handle,&OutBuffer,&InBuffer) != CAENRFID_StatusOK ) {
		FreeIOBuffer(&OutBuffer);
		FreeIOBuffer(&InBuffer);
		return CAENRFID_OutOfMemoryError;
	}
	FreeIOBuffer(&OutBuffer);
	err = getAVPStatus(&InBuffer);
	FreeIOBuffer(&InBuffer);
	return (CAENRFIDErrorCodes)err;
}

/**********************************************************************\

                  ResetSession

\**********************************************************************/

CAENRFIDErrorCodes
CAENRFID_ResetSession(CAENRFIDHandle handle, char *SourceName) {

	IOBuffer_t InBuffer,OutBuffer;
	int err;

	InitIOBuffer(&InBuffer);
	InitIOBuffer(&OutBuffer);
	if (makeHeaderAVP(&OutBuffer,T11xVer) == 0)	return CAENRFID_OutOfMemoryError;
	if (addCommandAVP(&OutBuffer,RESETSESSION) == 0) return CAENRFID_OutOfMemoryError;
	if (addStringAVP(&OutBuffer,SourceName,AVP_SOURCE_NAME) == 0) return CAENRFID_OutOfMemoryError;
	if (SendReceivePacket(handle,&OutBuffer,&InBuffer) != CAENRFID_StatusOK ) {
		FreeIOBuffer(&OutBuffer);
		FreeIOBuffer(&InBuffer);
		return CAENRFID_OutOfMemoryError;
	}
	FreeIOBuffer(&OutBuffer);
	err = getAVPStatus(&InBuffer);
	FreeIOBuffer(&InBuffer);
	return (CAENRFIDErrorCodes)err;
}

