#include "applipath.h"
#include "def_paramslist.h"

#include "constparams.h"
#include "displayutils.h"
#include "videoimgseq.h"

#include "ColorBase.h"
#include "ColorModel.h"
#include "skincolormodel.h"
#include "skincolorsegmentation.h"
#include "skinregionsegmentation.h"

void test_SkinRegionSegmentation()
{
  SkinRegionSegmentation skinRegions;
  IplImage* img;

  /* Lecture d'une sequence d'images */
  videoimgseq myvid("/home/lbrethes/1pmobileET1pmobile/","seq01","jpg",524,700,5);
  myvid.init();
  IplImage* imgIn = myvid.next();
  if(!imgIn)
    {
      cout << "Erreur de lecture de l'image\n";
      return;
    }
  
  while(imgIn)
    {
      img = skinRegions.processRegionsMask(imgIn);
      Show("IN",imgIn,0,0,0);
      DispFcol(img,"Regions Mask");
      img = skinRegions.getImgRegions();
      DispFcol(img,"Regions");
      cvWaitKey(500);
      imgIn = myvid.next();
    }
}

void test_SkinColorSegmentation()
{
  SkinColorSegmentation scs;
  IplImage* imgProba;

  /* Lecture d'une sequence d'images */
  videoimgseq myvid("/home/lbrethes/1pmobileET1pmobile/","seq01","jpg",524,700,5);
  myvid.init();
  IplImage* imgIn = myvid.next();
  if(!imgIn)
    {
      cout << "Erreur de lecture de l'image\n";
      return;
    }
  
  /* Init seuillage */
  scs.setThreshold(50);

  while(imgIn)
    {
      Show("RGB",imgIn,0,0,0);
      imgProba = scs.process(imgIn);
      Show("Proba RGB",imgProba,330,0,0);
      imgProba = scs.processThresh(imgIn);
      Show("Proba seuille RGB",imgProba,660,0,0);
      cvWaitKey(500);
      imgIn = myvid.next();
    }
}

void test_SkinColorModel()
{

  /**** Apprentissage du modele de couleur ****/
  
  //  cout << "Apprentissage RGB...\n";
  
  //   SkinColorModel scm("RGB");
  //   scm.learn("/home/lbrethes/DEVEL/NewTrack2d/track2d/config/color/data/skin-32-32-32.hist.lud","/home/lbrethes/DEVEL/NewTrack2d/track2d/config/color/data/nonskin-32-32-32.hist.lud");
  //   scm.save("skincolormodel_RGB");
  
  //   cout << "Apprentissage Irg...\n";
  
  //   SkinColorModel scm2("Irg");
  //   scm2.learn("/home/lbrethes/DEVEL/NewTrack2d/track2d/config/color/data/skin-32-32-32.hist.lud","/home/lbrethes/DEVEL/NewTrack2d/track2d/config/color/data/nonskin-32-32-32.hist.lud");
  //   scm2.save("skincolormodel_Irg");
  
  //   cout << "Apprentissage done !\n";
  
  /**** Apprentissage OK ****/


  /* Lecture d'une sequence d'images */
  videoimgseq myvid("/home/lbrethes/1pmobileET1pmobile/","seq01","jpg",524,700,5);
  myvid.init();
  IplImage* imgIn = myvid.next();
  if(!imgIn)
    {
      cout << "Erreur de lecture de l'image\n";
      return;
    }

  IplImage* proba = cvCreateImage(cvGetSize(imgIn),IPL_DEPTH_8U,1);
  IplImage* imgP1 = cvCreateImage(cvGetSize(imgIn),IPL_DEPTH_8U,1);
  IplImage* imgP2 = cvCreateImage(cvGetSize(imgIn),IPL_DEPTH_8U,1);
  IplImage* imgP3 = cvCreateImage(cvGetSize(imgIn),IPL_DEPTH_8U,1);
  IplImage* image[] = {imgP1,imgP2,imgP3};

  /* Test segmentation avec une autre base (Irg) */
  ParamsList base = ParamsList(installpath+"config/color/ColorBaseList.txt");
  int fromBase = base.getIndex("BGR");
  int toBase = base.getIndex("Irg");
  CvtColor* colorconverter = CvtColorAlloc(fromBase,toBase,320,240);
  IplImage* img2 = cvCreateImage(cvGetSize(imgIn),IPL_DEPTH_8U,3);
  IplImage* image2[] = {imgP2,imgP3};

  /* Test du chargement */
  SkinColorModel scmRGB;
  SkinColorModel scmIrg;

  scmRGB.load("skincolormodel_RGB");
  scmIrg.load("skincolormodel_Irg");

  char name[50];
  int cpt=0;
  int disp=1;
  while(imgIn)
    {      
      cout << "Traitement image " << cpt << endl; 
      cvSplit(imgIn,imgP3,imgP2,imgP1,NULL);
      cvCalcBackProject(image,proba,scmRGB.hist);
      
      if(disp)
	{
	  Show("RGB",imgIn,0,0,0);
	  Show("Proba RGB",proba,330,0,0);
	}

      sprintf(name,"proba%05d.jpg",cpt);
      cvSaveImage(name,proba);

      scmRGB.threshold(100);
      cvCalcBackProject(image,proba,scmRGB.hist_thresh);
      if(disp) Show("Proba seuille RGB",proba,660,0,0);

      sprintf(name,"probaSeuille%05d.jpg",cpt);
      cvSaveImage(name,proba);
     
      /* Conversion */
      colorconverter->convert(imgIn,img2);
      
      /* Segmentation */
      cvSplit(img2,imgP1,imgP2,imgP3,NULL);
      cvCalcBackProject(image2,proba,scmIrg.hist);

      sprintf(name,"autre_proba%05d.jpg",cpt);
      cvSaveImage(name,proba);

      if(disp)
	{
	  Show("Irg",img2,0,260,0);
	  Show("Proba Irg",proba,330,260,1);
	}

      scmIrg.threshold(200);
      cvCalcBackProject(image2,proba,scmIrg.hist_thresh);
      
      if(disp)
	{
	  Show("Proba seuille Irg",proba,660,260,1);
	  cvWaitKey(200);
	}

      sprintf(name,"autre_probaSeuille%05d.jpg",cpt++);
      cvSaveImage(name,proba);

      /* Image suivante */
      imgIn = myvid.next();
    }
  
}

void test_ColorBase()
{
  ColorBase cb1;
  ColorBase cb2("YCrCb.txt");

  cb1.disp();
  cb2.disp();

  cb1.load("HSV.txt");

  cb1.disp();
  cb2.disp(); 

  cb1.load("RGB.txt");

  cb1.disp();
  cb2.disp();

}

void test_ColorModel()
{
  ColorModel cm0;
  ColorModel cm3(3);

  cm0.disp();
  cm3.disp();
}

void prob2fcol(IplImage* imgproba, IplImage* imgfcol)
{
  uchar* ptprob = (uchar*)imgproba->imageData;
  uchar* ptfcol = (uchar*)imgfcol->imageData;
  CvScalar fcol;
  for(int i=0;i<imgproba->imageSize;i++)
    {
      fcol = SCalcFcol(*ptprob++);
      *ptfcol++ = (uchar)fcol.val[0];
      *ptfcol++ = (uchar)fcol.val[1];
      *ptfcol++ = (uchar)fcol.val[2];
    }
}


int main(int argc, char** argv)
{
  //test_ColorBase(); 
  //test_ColorModel();
  //test_SkinColorModel();
  //test_SkinColorSegmentation();
  test_SkinRegionSegmentation();
}
