#include "classifier.h"

//Declaration des constantes de classe
const int classifier::APP = 0;
const int classifier::DET = 1;

//Construsteur
classifier::classifier(int etat)
{
  this->state = etat;
  this->workingBase = -1;
  imgtmp = cvCreateImage( cvSize(30,30), IPL_DEPTH_8U, 1 );
  this->weight[0] = 2;
  this->weight[1] = 0.25;

  // Chemin des fichier classifieur par defaut
  workingPath = icuConfigPath + "/facerec/";
}

//Destructeur
classifier::~classifier(void)
{
}


int mustBeLoaded(char* name, int nbNameList, char* nameList[256]) {
  if(nbNameList==-1) return 1;
  for(int i=0;i<nbNameList;i++)
    {
      if(strcmp(nameList[i],name)==0) 
	return 1;
    }
  return 0;
}

//Permet de charger l integralite des bases en memoire a partir du fichier BaseList.txt
//Cree dynamiquement chaque nouvelle base puis, charge chaque base
int classifier::load()
{
  char name[256];
  int nBase;
  
  //Nettoyage de l ancien vecteur de base
  this->base.clear();

  string baseListFileName = workingPath + "/baseList.txt";   
  FILE* baseList = fopen(baseListFileName.c_str(), "r");
  printf("--> Classifier \t:\t Opening base list from %s\n",baseListFileName.c_str());

  /** Si le fichier n'existe pas on sort */
  if(!baseList)
    {
      cout << "|WARNING|--> Classifier \t:\t base list file doesn't exist\n";
      return 0;
    }

  fscanf(baseList,"%d\n", &nBase);

  if(!nBase)
    {
      cout << "|WARNING|--> Classifier \t:\t empty base list\n";
      return 0;
    }

  for(int i=0;i<nBase;i++)
    {
      int nbObj, nbB;
      fscanf(baseList, "%s %d %d", name, &nbObj, &nbB);       
      this->loadSomebody(name);
    }
  fclose(baseList);
  return 1;
}

/* Charge une personne a partir du disque */
int classifier::loadSomebody(char* name)
{
  int nBase;
  string baseListFileName = workingPath + "/baseList.txt"; 
	printf("%s\n", baseListFileName.c_str());  
  FILE* baseList = fopen(baseListFileName.c_str(), "r");
  int nbObj, nbB;
  char nameTmp[256];
  int i;

  /** Si le fichier n'existe pas on sort */
  if(!baseList)
    {
      cout << "|WARNING|--> Classifier \t:\t base list file doesn't exist\n";
      return 0;
    }

   fscanf(baseList,"%d\n", &nBase);

   if(!nBase)
     {
       cout << "|WARNING|--> Classifier \t:\t empty base list\n";
       return 0;
     }

  /* Teste si la base n'est pas chargee */
  for(i=0;i<this->base.size();i++)
    {
      if(strcmp(name,this->base[i]->getName())==0)
	{
	  cout << "|WARNING|--> Classifier \t:\t base " << name << " already loaded\n";
	  return 0;
	}
    }

  for(i=0;i<nBase;i++)
    {
      fscanf(baseList, "%s %d %d", nameTmp, &nbObj, &nbB);
      if(strcmp(name,nameTmp)==0)
	break;
    }
  fclose(baseList);
  
 //  if(i>=nBase) {
//     printf("--> Classifier \t:\t %s not found\n", name);
//     return 0;
//   }

  printf("--> Classifier \t:\t Loading eigenbase %s (%d,%d)\n", name,nbObj,nbB);
	
  //Creation d une nouvelle base et ajout au vecteur
  this->base.push_back(new eigenbase());
  
  //Chargement de la base
  this->base[this->base.size()-1]->load(workingPath.c_str(), name, nbObj, nbB);
  printf("--> Classifier \t:\t Eigenbase number %d loaded\n",i);	

  return 1;
}

/* Decharge une personne a partir du disque */
int classifier::unloadSomebody(char* name)
{
  vector<eigenbase*>::iterator it = this->base.begin();
  for(int i=0;i<this->base.size();i++,it++)
    {
      if(strcmp(name,this->base[i]->getName())==0)
	{
	  printf("Unload %s\n",name);
	  delete (*it);
	  printf("del ok\n");
	  this->base.erase(it);
	  printf("erase ok\n");
	  return 1;
	}
    }
  cout << "|WARNING|--> Classifier \t:\t base " << name << " not loaded\n";
  return 0;
}



//Si le cassifieur est en mode "detection", initialisation de chaque base
void classifier::init()
{
  //Tableau des effectifs Classe/nonClasse pour les distance de 0 a 100 (pas 1)
  double eff[100][2];
 	for(int i=0;i<100;i++)
	{
		eff[i][0] = 0;
		eff[i][1] = 0;
	}
  //Tableau des effectifs Classe/nonClasse pour les distance de 0 a 100 (pas 4)
  double effR[25][2];
 	for(int i=0;i<25;i++)
	{
		effR[i][0] = 0;
		effR[i][1] = 0;
	}
  //Distance de l echantillon j de la base i a la base k
  int distC;
  int C=0,nonC=0;

  switch(this->state)
    {
    case classifier::APP:
      break;
    case classifier::DET:
      for(unsigned int i=0;i<this->base.size();i++)
	{
	  this->base[i]->init();	  
	}
	for(unsigned int i=0;i<this->base.size();i++)
	  {
	    for(int j=0;j<this->base[i]->getNApp();j++)
	      {
		//Projection de l individu j de la base i dans la base k
		for(int k=0;k<this->base.size();k++)
		  {
		    distC = (int)(this->base[k]->computeLikelihood(this->base[i]->getAppr()[j]));
		    if(k==i)
		      {
			eff[distC][0]++;
			C++;
		      }
		    else
		      {
			eff[distC][1]++;
			nonC++;
		      }
		  }
	      }
	  }
	for(int i=0;i<100;i=i+4)
	  {
	    effR[i/4][0] = eff[i][0]/C + eff[i+1][0]/C + eff[i+2][0]/C + eff[i+3][0]/C;
	    effR[i/4][1] = eff[i][1]/nonC + eff[i+1][1]/nonC + eff[i+2][1]/nonC + eff[i+3][1]/nonC;
	  }
	int i;
	for(i=0;i<25 && this->weight[0]*effR[i][0] >= this->weight[1]*effR[i][1];i++);
	int j;
	for(j=(i-1)*4;j<(i+1)*4 && this->weight[0]*eff[j][0]/C >= this->weight[1]*eff[j][1]/nonC;j++);
	this->thresh =(double) j/100.0;
	printf("--> Classifier \t:\tfound better thresh : %lf\n", this->thresh);
      break;
    default:
      ;
    }
}

//Retourne une chaine de caractere correspondant a la personne la plus vraissemblante
// V/X nomDeLaPersonne vraissemblance
void classifier::whoIsIt(IplImage* imgIn)
{
  double dist;
  double* prob = new double[this->base.size()];
  int minClass=-1, sum=0;
  double first = 0;
 
  if(this->base.empty())
  {
	recoState = -1;
	recoId = -1;
	recoLikelihood = -1;
  }
  else
  {
  	//CReation d une image temporaire en memoire, image de travail
  	cvCopyImage(imgIn,imgtmp);
  
  	//Pertraitement
  	eigenbase::preTraiter(imgtmp);
  
  	//Si on travaille sur une seule personne
  	if(this->workingBase != -1)
  	  {
  	    int i = this->workingBase;
  	    dist = base[i]->computeLikelihood(imgtmp);
  	    prob[i] = exp(-pow(dist - this->base[i]->mean,2)/(2*pow(this->base[i]->std,2)));
   	    sum = 1;
   	    minClass = this->workingBase;
   	    //printf("%lf, %lf\n", dist,prob[i]);
  	  }
  	else
  	  {
  	    //pour chaque base, calculer la distance
  	    for(unsigned int i =0;i<this->base.size();i++)
		{
		  //Calcul de la distance
		  dist = base[i]->computeLikelihood(imgtmp);
		  prob[i] = exp(-pow(dist - this->base[i]->mean,2)/(2*pow(this->base[i]->std,2)));
		  //printf("%lf, %lf\t",dist, prob[i]);
		  if(prob[i] - first > 0)
		    {
		      first = prob[i];
		      minClass = i;
		      if(prob[i] - this->thresh >=0)
			{
			  sum ++;
			}
		    }
		}
    	  //printf("\n");
  	  }
  	switch (sum)
  	  {
  	    //P(C0|img) = 1 et P(Cn|img)=0 si pour tout n, Dn > thresh
  	  case 0:
  	    {
		  recoState = 1;
		  recoId = -1;
		  recoLikelihood = 0;
	  //sprintf(who, "X %s %3.2lf", this->base[minClass]->getName(), 0);
 	     }
  	    break;
      
      //P(Cm|img) = 1 et pour tout n!=m P(Cn|img)=0 si il existe un seul m < thresh
 	   case 1:
 	     {
		recoState = 2;
		recoId = minClass;
		recoLikelihood = prob[minClass];
	//sprintf(who, "V1 %s %3.2lf", this->base[minClass]->getName(), prob[minClass]);
  	    }
   	    break;

   	   //(C0|img) = 0 et pou tout n P(Cn|img) = h(thresh - Dn)/sump(h(thresh - Dp)
   	 default:
   	   {
		recoState = 3;
		recoId = minClass;
		recoLikelihood = (double)(1.0/sum);
		//sprintf(who, "V unknown %3.2lf", double(100/sum));
   	   }
   	 }
   }
}
char* classifier::whoIsItName() 
{
   if(recoId == -1)
	return "none";
   else
	return this->base[recoId]->getName(); 
}

//Ajoute l image fournie en entree a la base de nom "name"
// si la base "name" n existe pas, on ajoute cette nouvelle base au vecteur
void classifier::addBase(IplImage* imgIn, char* name, int where)
{
  string basePath = workingPath + "/";
  basePath += name;

  //Recherche de l existance de la base "name"
  bool find = false;
  unsigned int i;
  for(i=0;i<this->base.size() && !find;i++)
    {
      if(strcmp(this->base[i]->getName(), name)==0)
	{
	  find = true;
	  i--;
	}
    }

  //SI on ne l a pas trouvee
  if(!find)
    {
      char cmd[256];

      //Creationdu repertoire sur le disque
      sprintf(cmd, "mkdir %s",basePath.c_str());
      //printf("%s\n",cmd);
      system(cmd);

      //Creation de la nouvelle base en memoire
      this->base.push_back(new eigenbase());
      this->base[i]->setName(name);
    }
  
  this->base[i]->setState(eigenbase::CHNG);

  //Sauvegarde de la nouvelle image sur le disque
  char path[256];
  switch(where)
    {
    case 0:
      {
	sprintf(path, "%s/acp-image%0.3d.jpg", basePath.c_str() , this->base[i]->getNObjects());
	cvSaveImage(path, imgIn);
	this->base[i]->addObject(imgIn);
      }
      break;
    case 1:
      {
	sprintf(path, "%s/app-image%0.3d.jpg", basePath.c_str() , this->base[i]->getNApp());
	cvSaveImage(path, imgIn);
	this->base[i]->addApp(imgIn);
      }
      break;
    default:
      {
	printf("|ERROR|--> Classifier \t:\taddBase, wrong where choice\n");
      }
    }

  //printf("Base size : (%d,%d)\n",this->base[i]->getNObjects(),this->base[i]->getNApp());
}

//Sauvegarde les bases en memoire dans le fichier BaseList.txt
void classifier::end()
{
  string baseListFileName = workingPath + "/baseList.txt";   

  //Charge la liste existante pour ne pas modifier l'existant
  int nBase;
  FILE* baseList = fopen(baseListFileName.c_str(), "r");
  int *nbObj, *nbB;
  char **nmTmp;
  if(!baseList) {
    cout << "|WARNING|--> Classifier \t:\t base list file doesn't exist\n";
    return;
  }
  fscanf(baseList,"%d\n", &nBase);
  cout << nBase << "Already saved\n";
  nbObj = new int[nBase];
  nbB = new int[nBase];
  nmTmp = new char*[nBase];
  int cptCommon = 0;
  int* baseAlreadyOnDisk = new int[this->base.size()];
  memset(baseAlreadyOnDisk,0,sizeof(int)*this->base.size());
  for(int i=0;i<nBase;i++)
    {
      nmTmp[i] = new char[256];
      fscanf(baseList, "%s %d %d", nmTmp[i], nbObj+i, nbB+i);
      //Compte le nombre de bases en memoire qui sont sur disque
      // et met a jour le flag indiquant que la base en memoire est deja sur disque
      for(int j=0;j<this->base.size();j++) {
	if(strcmp(nmTmp[i],this->base[j]->getName())==0) {
	  cptCommon++;
	  baseAlreadyOnDisk[j] = 1;
	  break;
	}
      }
    }
  fclose(baseList);

  //Ecrit toutes les bases en memoires qui ne sont pas sur disque
  FILE* outfile = fopen(baseListFileName.c_str(), "w");
  printf("--> Classifier \t:\t Saving base list in %s\n",baseListFileName.c_str());
  fprintf(outfile, "%d", this->base.size()+nBase-cptCommon);

  for(int i=0;i<nBase;i++)
    {
      cout << nmTmp[i] << " was already on disk\n";
      fprintf(outfile, "\n%s %d %d", nmTmp[i],nbObj[i],nbB[i]);
      delete [] nmTmp[i];
    }

  for(int i=0;i<this->base.size();i++)
    {
      if(!baseAlreadyOnDisk[i]) {
	fprintf(outfile, "\n%s %d %d", this->base[i]->getName(),this->base[i]->getNObjects(), this->base[i]->getNApp());
	cout << this->base[i]->getName()  << " added\n";
      }      
    }
  fclose(outfile);

  delete [] nbObj;
  delete [] nbB;
  delete [] nmTmp;
  delete [] baseAlreadyOnDisk;
}

//Modifie l etat du classifieur : 
// APP = apprentissage
// DET = detection
void classifier::setState(int etat)
{
  this->state = etat;
}

//Specifie la personne que l on veut reconnaitre
void classifier::setWorkingPeople(char* name)
{
  //Recherche de la personne dans la base
  bool find = false;
  unsigned int i;
  for(i=0;i<this->base.size() && !find;i++)
    {
      if(strcmp(this->base[i]->getName(), name)==0)
	{
	  find = true;
	  i--;
	}
    }
  //Si elle n existe pas, on travaille sur toutes les bases
  if(find)
    {
      this->workingBase = i;
    }
  else
    {
      this->workingBase = -1;
    }
  printf("--> Classifier \t:\t setWorkingPeople.workingBase : %d\n",this->workingBase);
}

//"Oublie" que l on veut reconnaitre une seule personne
void classifier::eraseWorkingPeople()
{
  this->workingBase = -1;
}
