#ifndef FACEREC_EIGENBASE_H
#define FACEREC_EIGENBASE_H

#include "externParams.h"

#include "cv.h"
#include "cvaux.h"
#include "cxcore.h"
//#include "cvcam.h"
#include "highgui.h"

#include <stdio.h>
#include <stdlib.h>
//#include <conio.h>
#include <string.h>
#include <assert.h>
#include <math.h>
#include <float.h>
#include <limits.h>
#include <time.h>
#include <ctype.h>
#include <vector>
using namespace std;

#ifndef LEAVE_FILES_BEHIND
#define LEAVE_FILES_BEHIND 0
#endif


#define NBMAXPEOPLE 50

class eigenbase
{
private:
  //Nom de la personne represetee par cette base
  char name[256];
  //Type de reconnaissance (DFFS, CLASS, EUCLIDE, ...)
  int type;
  //Etat de la base (OK = utilisable, CHNG = en cours de modification)
  int state;

  //Taille des images de la base : fixee a 30x30
  CvSize size;

  //Vecteur des images de la base
  vector<IplImage*> objects;
  vector<IplImage*> baseAppr;
  //>Nombre d images de la base
  int nObjects;
  int nApp;

  //Images propres de la base (vecteurs propres, eigenfaces)
  IplImage** eigenObjects;
  //Nombre de vecteurs propres prise en compte (pour couvrir 90% d information)
  int nbValP;
  //Tableau des valeurs propres
  float* eigVals;

  //Image moyenne
  IplImage* avg;
		
  //Matrice gaussienne pour application a l image difference
  //Une fois l image difference calculee, on applique un masque gaussien
  //car un bruit sur un bord de l image n est pas aussi important qu un 
  //bruit au milieu de limage
  double gauss[30][30];

  //Couleur du cadre; pas utilisee
  CvScalar color;

  string workingPath;

public:
  //Moyenne des distance d apprentissage
  double mean;
  //Ecart type des distance d apprentissage
  double std;
  //Nombre total de bases en memoire
  static int nBase;
  //Type de classification
  static const int DFFS;
  //Etats de la base
  static const int OK;
  static const int CHNG;

  //Constructeur
  eigenbase(int typ = eigenbase::DFFS);
  //Destructeur
  ~eigenbase(void);
  //Initialise la base en realisant une ACP
  void init();
  //Charge la base d'images
  void load(const char* dir,char* path, int nbObj, int nbApp);
  //Sauvegarde la base
  void save(char* path);
  //Retourne les coefficients de decomposition de imgIn projetee dans la base
  float* decompose(IplImage* imgIn);
  //Retourne la reconstruction de imgIn dans la base
  IplImage* project(IplImage* imgIn);
  //Accesseurs en lecture
  vector<IplImage*> getObjects();
  vector<IplImage*> getAppr();
  IplImage* getObject(int ind);
  int getNObjects();
  int getNApp();
  IplImage** getEigenObjects();
  IplImage* getAvg();
  char* getName();
  CvSize getSize();
  float* getEigVals();
  int getType();
		
  //Accesseurs en ecriture
  void setObjects(IplImage** obj);
  void setEigenObjects(vector<IplImage*> eigenObj);
  void setAvg(IplImage* moy);
  void setName(char* nom);
  void setSize(CvSize taille);
  void setEigVals(float* valP);
  void setType(int t);
  void setState(int etat);
  
  //Calcule la vraissemblance de l image par rapport a la base
  double computeLikelihood(IplImage* imgIn);
  //Calcule l image difference caracteristique de la DFFS
  IplImage* computeDFFS(IplImage* imgIn);

  //Ajoute l image a la base
  void addObject(IplImage* imgIn);
  void addApp(IplImage* imgIn);

  //Realise un pretraitement a l image d entree
  static void preTraiter(IplImage* imgIn, int choix = 1);
  //Egalise l histogramme de l image d entree
  static void equalize(IplImage* imgIn);
};

#endif
