#include <cv.h>
#include <stdio.h>
#include "applipath.h"
#include "def_paramslist.h"

#include "watershed.h"
#include "imageutils.h"
#include "displayutils.h"
#include "histogrammes.h"
#include "ColorBase.h"
#include "cvtcoloralloc.h"
#include "paramslist.h"
#include "skincolormodel.h"
#include "skincolorsegmentation.h"
#include "temps_cpu.h"

#include <deque>
#include <vector>

void convtest(IplImage* imRGB, IplImage* imBase)
{
  IplImage* rgbSum = cvCreateImage(cvGetSize(imRGB),IPL_DEPTH_32F,1);

  IplImage* pR = cvCreateImage(cvGetSize(imRGB),IPL_DEPTH_8U,1);
  IplImage* pG = cvCreateImage(cvGetSize(imRGB),IPL_DEPTH_8U,1);
  IplImage* pB = cvCreateImage(cvGetSize(imRGB),IPL_DEPTH_8U,1);

  IplImage* pRF = cvCreateImage(cvGetSize(imRGB),IPL_DEPTH_32F,1);
  IplImage* pGF = cvCreateImage(cvGetSize(imRGB),IPL_DEPTH_32F,1);
  IplImage* pBF = cvCreateImage(cvGetSize(imRGB),IPL_DEPTH_32F,1);

  IplImage* resNorm = cvCreateImage(cvGetSize(imRGB),IPL_DEPTH_32F,1);

  IplImage* I = cvCreateImage(cvGetSize(imRGB),IPL_DEPTH_8U,1);
  IplImage* r = cvCreateImage(cvGetSize(imRGB),IPL_DEPTH_8U,1);
  IplImage* g = cvCreateImage(cvGetSize(imRGB),IPL_DEPTH_8U,1);
  IplImage* b = cvCreateImage(cvGetSize(imRGB),IPL_DEPTH_8U,1);


  cvSplit(imRGB,pB,pG,pR,NULL);

  cvConvertScale(pR,pRF);
  cvConvertScale(pG,pGF);
  cvConvertScale(pB,pBF);
  
  /* R + G + B */
  cvAdd(pRF,pGF,rgbSum);
  cvAdd(pBF,rgbSum,rgbSum);

  /* divise par 3 pour obtenir l'intensite */
  double scale = 1.0/3.0;
  cvConvertScale(rgbSum,I,scale);

  /* Normalisation de la partie rouge */
  cvDiv(pRF,rgbSum,resNorm);
  cvConvertScale(resNorm,r,255.0);

  /* Normalisation de la partie verte */
  cvDiv(pGF,rgbSum,resNorm);
  cvConvertScale(resNorm,g,255.0);

  /* Normalisation de la partie bleu */
  cvDiv(pBF,rgbSum,resNorm);
  cvConvertScale(resNorm,b,255.0);

  Show("func I",I,0);
  Show("func r",r,0);
  Show("func g",g,0);
  Show("func b",b,0);


}


void NouvHist2DDyn(CvHistogram *histo, int nbbins, float ech)
{
  float min = 9999;
  float max = 0;
  CvMat* bins = (CvMat*)histo->bins;
  
  /* Recherche min et max */
  for(int i=0;i<nbbins;i++)
    {
      if(bins->data.fl[i])
	{
	  if(bins->data.fl[i]<min) min = bins->data.fl[i];
	  if(bins->data.fl[i]>max) max = bins->data.fl[i];
	}
    }
  
  double delta = max - min;
  for(int i=0;i<nbbins;i++)    
    if(bins->data.fl[i])
      bins->data.fl[i] = 255.0 * (bins->data.fl[i] - min) / delta;
}

void NouvHist2DDisplay(CvHistogram *histo,const char *fen,int scale)
{
  int h,s;
  float max_value;
  int hist_size[2];

  cvGetDims( histo->bins, hist_size );

  IplImage* hist_img = cvCreateImage( cvSize(hist_size[0]*scale,hist_size[1]*scale), 8, 3 );

  cvGetMinMaxHistValue( histo, 0, &max_value, 0, 0 );
  cout << "MAX = " << max_value << endl;

  cvZero( hist_img );
  for( h = 0; h < hist_size[0]; h++ )
    {
      for( s = 0; s < hist_size[1]; s++ )
	{
	  float bin_val = cvQueryHistValue_2D( histo, h, s );
	  if(bin_val)
	    {
	      int intensity = cvRound(bin_val*255/max_value);
	      //cout << "bin_val = " << bin_val << "  intensity = " << intensity << endl; 
	      cvRectangle( hist_img, cvPoint( h*scale, s*scale ),cvPoint( (h+1)*scale - 1, (s+1)*scale - 1),CV_RGB(intensity,intensity,intensity),CV_FILLED );
	    }	  
	}
    }
  cvNamedWindow(fen,0);
  cvShowImage(fen,hist_img);

  cvReleaseImage(&hist_img);
  return;
}



IplImage* filtre(IplImage* imgLabels, int nbreg, IplImage* imgChromClust)
{
  IplImage* imgMask = cvCreateImage(cvGetSize(imgLabels),IPL_DEPTH_8U,1); 
  IplImage* imgTMP = cvCreateImage(cvGetSize(imgLabels),IPL_DEPTH_8U,3);  
  char nm[50];
  CvMemStorage* storage = cvCreateMemStorage(0);
  CvSeq* contour = 0;
  CvPoint * pt;
  int seuilNbpts = 50;
  int nbregFinal=0;

  cvZero(imgMask);

  for(int i=1;i<=nbreg;i++)
    {
      /*
	Creation d'un masque de pixels de la region i
	elimine les classes > a 2
	mask = 0 si imres>classe
      */
      cvThreshold(imgLabels,imgMask,i,0,CV_THRESH_TOZERO_INV);
            
      /*
	ne conserve que la classe 2
	mask = imres si imres>classe-1
      */
      //cvThreshold(imgMask,imgMask,i-1,0,CV_THRESH_TOZERO);
      cvThreshold(imgMask,imgMask,i-1,255,CV_THRESH_BINARY);

      sprintf(nm,"Region %d",i);
      Show(nm,imgMask,0,0,0);

      /**** BOUCHAGE DES TROUS ET FILTRAGE DES PETITES REGIONS ****/
      
      /* recherche des contours des rgions connexes */
      cvFindContours(imgMask,storage,&contour,sizeof(CvContour),CV_RETR_EXTERNAL,CV_CHAIN_APPROX_SIMPLE);
      
      /* trac des rgions dans l'image rsultat avec remplissage */

      CvScalar couleur;
      for( ; contour != 0; contour = contour->h_next )
	{
	  /* Filtrage des petites regions en meme temps */
	  if(contour->total>seuilNbpts)
	    {
	      /* Region conservee */
	      nbregFinal++;

	      /* rcupration d'un point pour obtenir la couleur de la rgion */
	      pt = CV_GET_SEQ_ELEM(CvPoint,contour,0);
	      
	      /* recup du label de la rgion */
	      //int color = (int) cvGetReal2D(imgLabels,pt->y,pt->x);
	      int color = (int)imgLabels->imageData[pt->y*imgLabels->widthStep+pt->x];

	      cout << "Filtrage : color = " << color << endl;
	      
	      /* trac de la rgion */
	      cvDrawContours(imgChromClust,contour,cvScalar(color),cvScalar(color),-1,CV_FILLED,8);
	      //cvDrawContours(imgMask,contour,cvScalar(255),cvScalar(255),-1,CV_FILLED,8);

	      couleur = SCalcFcol(nbregFinal);
	      cvDrawContours(imgTMP,contour,couleur,couleur,-1,CV_FILLED,8);
	    }
	    
	}
        
      //sprintf(nm,"Region ss trous %d",i);
      //Show(nm,imgMask,360,0,1);

    }

  Show("TMP fcol",imgTMP,0,0,1);

  return imgTMP;

}

void myDilate(float* in, float* out, int nbbins)
{
  int vois[24];
  int pos;

  /* table pour acceder rapidement aux voisins */
  int cpt = 0;
  for(int i=-2;i<=2;i++)
    for(int j=-2;j<=2;j++)
      if(i || j) 
	vois[cpt++] = i*256+j; 

  get_time_start();
  for(int i=0;i<nbbins;i++)
    {

      //cout << "pixel " << i << " = " << in[i] << endl;

      /* Recopie de la valeur du centre */
      out[i] = in[i];

      /* Parcours des voisins */
      for(int v=0;v<25;v++)
 	{
 	  pos = i + vois[v];
	  
	  /* Si voisin valide et max dans la region */
 	  if(pos>=0 && pos < nbbins && out[i] < in[pos])
 	    out[i] = in[pos];
 	}
    }
  get_time_stop();
}


void myDilateBis(float* in, float* out, int nbbins)
{
  int pos;
  float tmp[256*256];
  int nbl;

  get_time_start();

  /* Propagation horizontale */
  for(int i=0,j=-1,k=-2,l=1,m=2,cpt=0;i<nbbins;i++,j++,k++,l++,m++,cpt++)
      {
	/* Recopie de la valeur du centre */
	tmp[i] = in[i];
	
	/* Traitement predecesseurs */ 
	if(cpt>0 && tmp[i] < in[j]) tmp[i] = in[j];       
	if(cpt>1 && tmp[i] < in[k]) tmp[i] = in[k]; 
      
	/* Traitement des suivants */ 
	if(cpt<255 && tmp[i] < in[l]) tmp[i] = in[l]; 
	if(cpt<254 && tmp[i] < in[m]) tmp[i] = in[m]; 

	/* remise a 0 de cpt */
	if(cpt>255) cpt = 0;
      }
    

  /* Propagation verticale */
  for(int i=0,j=-256,k=-512,l=256,m=512;i<nbbins;i++,j++,k++,l++,m++)
    {
      /* Recopie de la valeur du centre */
      out[i] = tmp[i];
      
      /* Traitement predecesseurs */ 
      if(j>=0 && out[i] < tmp[j]) out[i] = tmp[j]; 
      if(k>=0 && out[i] < tmp[k]) out[i] = tmp[k]; 

      /* Traitement des suivants */ 
      if(l<nbbins && out[i] < tmp[l]) out[i] = tmp[l];       
      if(m<nbbins && out[i] < tmp[m]) out[i] = tmp[m];        
    }
  get_time_stop();
}

void test2()
{
  ParamsList base(icuConfigPath+"./objectslist/ColorBaseList.txt");
  base = "Irg";

  ColorBase colBase;
  colBase.load("Irg.txt");

  /* Charge une image */
  IplImage* imgIn = cvLoadImage("cap000.jpg",-1);
  CvSize sz = cvGetSize(imgIn);
  int w = sz.width;
  int h = sz.height;
  IplImage* imgBase = cvCreateImage(sz,IPL_DEPTH_8U,3);
  IplImage* imgP1 = cvCreateImage(sz,IPL_DEPTH_8U,1);
  IplImage* imgP2 = cvCreateImage(sz,IPL_DEPTH_8U,1);
  IplImage* imgP3 = cvCreateImage(sz,IPL_DEPTH_8U,1);
  IplImage* planChrom[] = {imgP2,imgP3};
  CvtColor* colconv = CvtColorAlloc(base.getIndex("BGR"),base.i(),320,240);

  //Allocation des Histogrammes
  CvHistogram* histoChrom;
  CvHistogram* histoChromClust;

  int hist_sizeChrom[2];
  float rangeC2[2];
  float rangeC3[2];  
  float* ranges[] = {rangeC2,rangeC3};
  
  /* Definition des ranges et size sur la chrominance pour la base courante */
  hist_sizeChrom[0] = colBase.getdimChrom1();
  hist_sizeChrom[1] = colBase.getdimChrom2();
  rangeC2[0] = colBase.getChrom1Rmin();
  rangeC2[1] = colBase.getChrom1Rmax();
  rangeC3[0] = colBase.getChrom2Rmin();
  rangeC3[1] = colBase.getChrom2Rmax();

  cout << "hist size = " << hist_sizeChrom[0] << " , " << hist_sizeChrom[1] << endl;
  cout << "hist range C2 = " << rangeC2[0] << " - " << rangeC2[1] << endl;
  cout << "hist range C3 = " << rangeC3[0] << " - " << rangeC3[1] << endl;

  /* Creation de l'histogramme de chrominance */
  histoChrom = cvCreateHist(2,hist_sizeChrom,CV_HIST_ARRAY,ranges, 1 );
  histoChromClust = cvCreateHist(2,hist_sizeChrom,CV_HIST_ARRAY,ranges, 1 );

  cvClearHist(histoChrom);
  cvClearHist(histoChromClust);

  /* conversion en couleur */
  colconv->convert(imgIn,imgBase);

  Show("IN",imgIn,0);
  Show("Base",imgBase,0);

  /* recup plan chromatique */
  cvSplit(imgBase,imgP1,imgP2,imgP3,NULL);

  Show("Base P1",imgP1,0);
  Show("Base P2",imgP2,0);
  Show("Base P3",imgP3,0);

  /* Projection couleur peau pour masquer le calcul d'histogramme */
  IplImage* imgProba;
  IplImage* imgProbaThresh = cvCreateImage(sz,IPL_DEPTH_8U,1);
  SkinColorSegmentation scs;
  scs.setThreshold(150);
  imgProba=scs.processThresh(imgIn);
  //cvThreshold(imgProba,imgProbaThresh,127,255,CV_THRESH_BINARY);


  exportImg2Matlab("proba.m",imgProba);

  Show("imgProba",imgProba,0);
  Show("imgProbaThresh",imgProbaThresh,0);

  /* Calcul histogramme */
  cvCalcHist(planChrom,histoChrom,0,imgProba);  

  CvMat* tabHist = (CvMat*)histoChrom->bins;
  CvMat* tabHistClustered = (CvMat*)histoChromClust->bins;

  float min,max;
  cvGetMinMaxHistValue(histoChrom,&min,&max);
  cout << "histo min = " << min << " et max = " << max << endl;
  
  NouvHist2DDyn(histoChrom,256*256,255);

  NouvHist2DDisplay(histoChrom,"Chrom av",4);

  //CvMat* matTmp = cvCreateMatHeader(256,256,CV_32F);
  //cvSetData(matTmp,histoChrom->bins,256*256*sizeof(float));
  
  //IplConvKernel* B_test=NULL;
  // int taille_se = 5;
  // B_test = cvCreateStructuringElementEx(taille_se,taille_se,taille_se/2,taille_se/2,CV_SHAPE_RECT,NULL);
  // cvDilate(matTmp,matTmp,B_test);
  
  /* Ma version de dilatation */

  //myDilate(tabHist->data.fl,tabHistClustered->data.fl,256*256);
  //NouvHist2DDisplay(histoChromClust,"Chrom Lud1",4);

  myDilateBis(tabHist->data.fl,tabHist->data.fl,256*256);
  //NouvHist2DDisplay(histoChromClust,"Chrom LudBis",4);

  //get_time_start();
  //Hist2DDilate(histoChrom,5);
  //get_time_stop();

  NouvHist2DDisplay(histoChrom,"Chrom",4);
  
  /* Alloc du watershed */
  queue<int> picList;
  int nbpics=0;
  double seuil=0.6; 
  Watershed2d<float> ws(256,256);
  float tabClustered[256*256];
  //int tab[256*256];
 


  /* Affichage des bins pour matlab */
  ofstream ficout("tmp.m",ios::out|ios::trunc);
  ficout << "histo = [";
  for(int i=0;i<256;i++)
    {
      for(int j=0;j<256;j++)
	ficout << tabHist->data.fl[i*256+j] << " ";
      
      ficout << ";\n";
    }

  ficout << "];" << endl;

  ficout.close();

  //memset(tab,0,256*256*sizeof(int));
  memset(tabClustered,0,256*256*sizeof(float));

  int nbbins = 256*256;
  //for(int i=0;i<nbbins;i++)
  //{
  //  tab[i] = tabHist->data.fl[i];
      //cout << "Recopie : " << tab[i] << endl;
  //}

  //NouvHist2DDisplay(histoChrom,"Chrom av Clust",4);
  //NouvHist2DDisplay(histoChromClust,"ChromClust av Clust",4);

  nbpics = ws.ExtractMax(tabHist->data.fl, tabHistClustered->data.fl, picList, seuil,127);
  cout << nbpics << " marqueurs : \n";

  //NouvHist2DDisplay(histoChrom,"Chrom ap Clust",4);
  //NouvHist2DDisplay(histoChromClust,"ChromClust ap Clust",4);


  IplImage* imgview = cvCreateImage(cvSize(256,256),IPL_DEPTH_8U,3);
  IplImage* imgview2 = cvCreateImage(sz,IPL_DEPTH_8U,3);
  
  //convtest(imgIn,imgBase);
  
  // while(!picList.empty())
//     {
//       cout << "pos " << picList.front() << endl;
//       picList.pop();
      
//     }
  
// /* Calcul histogramme non seuille pour la clusterisation */
//   cvCalcHist(planChrom,histoChrom,0,imgProba);
//   NouvHist2DDyn(histoChrom,256*256,255);
//   Hist2DDilate(histoChrom,5);
//   NouvHist2DDisplay(histoChrom,"Chrom non thresh",4);
// /* Affichage des bins pour matlab */
//   ficout.open("tmp2.m",ios::out|ios::trunc);
//   ficout << "histo = [";
//   for(int i=0;i<256;i++)
//     {
//       for(int j=0;j<256;j++)
// 	ficout << tabHist->data.fl[i*256+j] << " ";
      
//       ficout << ";\n";
//     }

//   ficout << "];" << endl;

//   ficout.close();  


  /* Execution du partage */
  ws.Process(tabHist->data.fl, tabHistClustered->data.fl, picList, 1);

  
  int pos=0;
  for(int i=0;i<256;i++)
    for(int j=0;j<256;j++)
      {
	//cout << "tabClustered = " << (int)(tabClustered[pos]) << endl;
	CvScalar couleur = SCalcFcol((int)(tabHistClustered->data.fl[pos++]));
 	cvSet2D(imgview,i,j,couleur);
      }
  Show("RES",imgview,0);

  IplImage* imgChrom = cvCreateImage(sz,IPL_DEPTH_8U,1);
  cvCalcBackProject(planChrom,imgChrom,histoChromClust);

  /* Affichage en fausse couleur */
  int val;
  for(int i=0;i<h;i++)
    for(int j=0;j<w;j++)
      {
	//cout << i << "," << j << endl;
	//cout << "1\n";
	val = (int)(cvGetReal2D(imgChrom,i,j));
	//cout << "2\n";
	//cout << "Val = " << val << endl;
	CvScalar couleur = SCalcFcol((int)(val));
	//cout << "3\n";
	cvSet2D(imgview2,i,j,couleur);
      }

  Show("RES2",imgview2,0);

  IplImage* imgChromClust = cvCreateImage(sz,IPL_DEPTH_8U,1);


  /*** FILTRAGE ***/
  IplImage* imgFiltree = filtre(imgChrom,nbpics,imgChromClust);


  IplImage* imgDil = cvCreateImage(cvSize(w,h),IPL_DEPTH_8U,3);
  IplImage* imgEro = cvCreateImage(cvSize(w,h),IPL_DEPTH_8U,3);
  IplImage* imgEro1 = cvCreateImage(cvSize(w,h),IPL_DEPTH_8U,1);
  IplImage* imgEro2 = cvCreateImage(cvSize(w,h),IPL_DEPTH_8U,1);
  IplImage* imgEro3 = cvCreateImage(cvSize(w,h),IPL_DEPTH_8U,1);
  IplImage* imgDil1 = cvCreateImage(cvSize(w,h),IPL_DEPTH_8U,1);
  IplImage* imgDil2 = cvCreateImage(cvSize(w,h),IPL_DEPTH_8U,1);
  IplImage* imgDil3 = cvCreateImage(cvSize(w,h),IPL_DEPTH_8U,1);
  IplImage* imgF1 = cvCreateImage(cvSize(w,h),IPL_DEPTH_8U,1);
  IplImage* imgF2 = cvCreateImage(cvSize(w,h),IPL_DEPTH_8U,1);
  IplImage* imgF3 = cvCreateImage(cvSize(w,h),IPL_DEPTH_8U,1);

  /* Separation de l'image filtree */
  cvSplit(imgFiltree,imgF1,imgF2,imgF3,NULL);


  IplConvKernel* B = cvCreateStructuringElementEx(3,3,1,1,CV_SHAPE_RECT,NULL);
  cvDilate(imgFiltree,imgDil,B,5);
  cvErode(imgFiltree,imgEro,B,2);
  Show("Regions Dil",imgDil,0);
  Show("Regions Ero",imgEro,1);

  
  /* Superposition des contours */
  IplImage* imgGRAY = cvCreateImage(cvSize(w,h),IPL_DEPTH_8U,1);
  IplImage* imgMEDIUM = cvCreateImage(cvSize(w/2,h/2),IPL_DEPTH_8U,1);
  IplImage* imgEDGE = cvCreateImage(cvSize(w,h),IPL_DEPTH_8U,1);
  IplImage* imgEDGE2 = cvCreateImage(cvSize(w,h),IPL_DEPTH_8U,3);
 
  /* conversion BGR vers gris */
  cvCvtColor(imgIn,imgGRAY,CV_BGR2GRAY);

  /* Lissage de l'image */
  cvPyrDown(imgGRAY,imgMEDIUM,IPL_GAUSSIAN_5x5);
  cvPyrUp(imgMEDIUM,imgGRAY,IPL_GAUSSIAN_5x5);

  /* calcul des contours */
  cvCanny(imgGRAY,imgEDGE,20,40);

  /* Ajout des contours */
  cvMerge(imgEDGE,imgEDGE,imgEDGE,NULL,imgEDGE2);
  cvCopy(imgEDGE2,imgFiltree,imgEDGE);

  Show("Contours",imgEDGE,0);
  Show("ContoursAvecRegions",imgFiltree,1);


  cvSplit(imgEro,imgEro1,imgEro2,imgEro3,NULL);
  cvSet(imgDil,cvScalar(0,0,0),imgEro1);
  Show("Regions Dil Test",imgDil,1);

  cvZero(imgFiltree);
  cvSplit(imgDil,imgDil1,imgDil2,imgDil3,NULL);
  cvCopy(imgEDGE2,imgFiltree,imgDil1);
  Show("Contours Conser",imgFiltree,1);


  /******************************* WATERSHED FINAL **************************************/

  cout << "DEBUT WATERSHED\n";

  /* Construction de la liste de marqueurs et calcul des stats des regions */
  int nbp = w*h;
  int regnum;

  float sumR[50];
  float sumG[50];
  float sumB[50];
  int nbpix[50];

  /* Files d'attentes hierarchiques */
  queue<int> niveaux[256];

  memset(sumR,0,50*sizeof(float));
  memset(sumG,0,50*sizeof(float));
  memset(sumB,0,50*sizeof(float));
  memset(nbpix,0,50*sizeof(int));

  /* Pour obtenir les trois plans */
 cvSplit(imgIn,imgP3,imgP2,imgP1,NULL);

  /* Parcours des pixels de l'image des regions */
  for(int i=0;i<nbp;i++)
    {
      regnum = (int)imgChromClust->imageData[i];
      if(regnum<0) cout << "ATTENTION !!! regnum = " << regnum << endl;

      /* Si le pixel appartient a une region, ses coordonnees sont empilees et sa couleur contribue aux stats couleur de la region */
      if(regnum)
	{
	  niveaux[0].push(i);
	  
	  nbpix[regnum]++;
	  sumR[regnum]+=imgP1->imageData[i];
	  sumG[regnum]+=imgP2->imageData[i];
	  sumB[regnum]+=imgP3->imageData[i];
	}      
    }

  /* Calcul du watershed couleur */
  int pt,vois;
  int lbl,lblvois;
  int voisList[8];
  int r,g,b;
  int niv;
  float mR,mG,mB;
  float coldist,dr,dg,db;
  float seuilColdist=50;

  voisList[0] = -w-1;
  voisList[1] = -w;
  voisList[2] = -w+1;
  voisList[3] = -1;
  voisList[4] = 1;
  voisList[5] = w-1;
  voisList[6] = w;
  voisList[7] = w+1;

  DispFcol(imgChromClust,"ChromClust");
  cvWaitKey(0);
  
  float mincoldist=99999;
  float maxcoldist = 0;

  /* Calcul d'un sobel */
  IplImage* imgSOBEL = cvCreateImage(sz,IPL_DEPTH_16S,1);
  IplImage* imgSOBELview = cvCreateImage(sz,IPL_DEPTH_8U,1);
  cvSobel(imgGRAY,imgSOBEL,1,1);
  double mmax,mmin;
  cvMinMaxLoc(imgSOBEL,&mmin,&mmax);
  cvConvertScale(imgSOBEL,imgSOBELview,255.0/mmax);
  Show("SOBEL",imgSOBELview,1);


  for(int i=0;i<256;i++)
    {
      //inondation
      while(!niveaux[i].empty())
	{
	  //tant que file[i] non vide
	  //Lire un point dans la file
	  pt = niveaux[i].front();
	  niveaux[i].pop();
  
	  //recuperation du label du pixel
	  lbl = (int)imgChromClust->imageData[pt];

	  //cout << "Traitement pixel de la region " << lbl << endl;

	  //on traite ses voisins
	  for(int v=0;v<8;v++)
	    {
	      /* On calcule l'indice du voisin */
	      vois = pt + voisList[v];
	      
	      //cout << "Son voisin " << vois << "\n";

	      //si voisin pas trait alors 
	      //      elargir l'image label 
	      //             et
	      //      mettre les coordonnes du voisin dans la file :
	      //         file[max(gradient,i)]


	      /* Teste si on est dans le tableau, si le voisin n'est pas deja traite et si ce n'est pas un point de contour */
	      if(vois>=0 && vois<nbp && !((int)imgChromClust->imageData[vois]) && !((int)imgEDGE->imageData[vois]) )
		{

		  /* Recup de la proba du pixel */
		  lblvois = (int)cvGetReal1D(imgProba,vois);
		  
		  /* On calcule la distance colorimetrique avec la region a laquelle il pourrait appartenir */
		  mR = sumR[lbl] / nbpix[lbl];
		  mG = sumG[lbl] / nbpix[lbl];
		  mB = sumB[lbl] / nbpix[lbl];

		  dr = mR - (int)(imgP1->imageData[vois]); 
		  dg = mG - (int)(imgP2->imageData[vois]); 
		  db = mB - (int)(imgP3->imageData[vois]); 

		  coldist = sqrt(dr*dr+dg*dg+db*db);

		  //cout << "dist = " << coldist <<endl;
		  //if(coldist>maxcoldist) maxcoldist = coldist;
		  //if(coldist<mincoldist) mincoldist = coldist;

		  /* Si la couleur est proche alors on empile le voisin selon sa probabilite couleur peau */
		  if(coldist<seuilColdist || lblvois>100)
		    {
		      //cout << "\test ajoute a la region\n";

		      /* Mise a jour stat couleur de la region */
		      sumR[lbl] += (int)(imgP1->imageData[vois]);
		      sumG[lbl] += (int)(imgP2->imageData[vois]);
		      sumB[lbl] += (int)(imgP3->imageData[vois]);
		      nbpix[lbl]++;
		      
		      /* Labelisation du voisin */
		      imgChromClust->imageData[vois] = (uchar)lbl;

		      /* On place le voisin dans la liste selon sa probabilite de couleur peau */
		      /* On peut aussi ajouter la valeur du contour mais pas dans le cas d'un contour binaire */		     
		      niv = 255 - lblvois;

		      //cout << "sobel = " << (int)cvGetReal1D(imgSOBELview,vois) << endl;
		      //niv = (int)cvGetReal1D(imgSOBELview,vois);
		      
		      //cout << "Proba du voisin : " << lblvois << " --> au niveau " << niv << endl;

		      niveaux[niv].push(vois);
		    }
		}
	    }
	}
    }

  cout << "FIN\n";
  
  DispFcol(imgChromClust,"WATERSHED");

  cout << "Coldist min = " << mincoldist << " max = " << maxcoldist << endl; 

  cvWaitKey(0);
  
}

int test1(void)
{
  int w = 320;
  int h = 240;
  int size = w*h;

  double seuil=0.4; 

  /* Alloc du watershed */
  Watershed2d<int> ws(w,h);

  /* tableau */
  int tab[w*h];
  int tabClustered[w*h];

  /* Liste des pic retenus */
  queue<int> picList;

  /* Nombre de pics */
  int nbpics=0;

  /* remplissage */
  for(int i=0;i<h;i++)
    for(int j=0;j<w;j++)
      tab[i*w+j] = 0;

  tab[10*w+10] = 230;
  tab[10*w+11] = 200;
  tab[10*w+12] = 200;
  tab[10*w+13] = 200;
  tab[10*w+14] = 210;
  tab[10*w+15] = 200;
  tab[10*w+16] = 200;
  tab[10*w+17] = 220;
  tab[10*w+18] = 200;
  tab[10*w+19] = 200;
  tab[10*w+20] = 200;
  

  tab[100*w+10] = 180;
  tab[100*w+11] = 120;
  tab[100*w+12] = 100;

  nbpics = ws.ExtractMax(tab, tabClustered, picList, seuil);
  cout << nbpics << " marqueurs : \n";
  while(!picList.empty())
    {
      cout << "pos " << picList.front() << endl;
      picList.pop();
    }

  /* Execution du partage */
  ws.Process(tab, tabClustered, picList, 0);

  /* Affichage en fausse couleur */
 //  IplImage* imgview = cvCreateImage(cvSize(w,h),IPL_DEPTH_8U,3);
//   IplImage* imgview2 = cvCreateImage(cvSize(w,h),IPL_DEPTH_8U,3);
  
//   int pos=0;
//   for(int i=0;i<h;i++)
//     for(int j=0;j<w;j++)
//       {
// 	CvScalar couleur = SCalcFcol(tabClustered[pos++]);
// 	cvSet2D(imgview,i,j,couleur);
//       }

//   cvSetImageROI(imgview,cvRect(5,5,10,20));
//   cvResize(imgview,imgview2);

//   Show("RES",imgview2,1);
  //ws.Normalize(tab);
  //nbpics = ws.ExtractMax(tab, tabClustered, picList, seuil);
  //cout << nbpics << " marqueurs\n";
}

void lumdil(CvMat* histBins)
{
  int pos;
  float tmp[3];

  /* Propagation horizontale */
  for(int i=0,j=-1,k=-2,l=1,m=2;i<256;i++,j++,k++,l++,m++)
      {
	cout << "Centre es = " << i << endl; 
	pos = i % 3;

	cout << "i = " << i << "  pos = " << pos << endl;

	/* Recopie des elements stockes en temporaire */
	if(i>2)
	  {
	    cout << "recopie de la valeur temporaire tmp[" << pos << "] dans bins[" << k-1 << "]" << endl;
	    histBins->data.fl[k-1]=tmp[pos];
	  }

	/* Recopie de la valeur du centre */
	tmp[pos] = histBins->data.fl[i];
	
	/* Traitement predecesseurs */ 
	if(i>0 && tmp[pos] < histBins->data.fl[j]) tmp[pos] = histBins->data.fl[j];       
	if(i>1 && tmp[pos] < histBins->data.fl[k]) tmp[pos] = histBins->data.fl[k]; 
      
	/* Traitement des suivants */ 
	if(i<255 && tmp[pos] < histBins->data.fl[l]) tmp[pos] = histBins->data.fl[l]; 
	if(i<254 && tmp[pos] < histBins->data.fl[m]) tmp[pos] = histBins->data.fl[m]; 
      }
  
  /* Recopie des dernieres valeurs */
  histBins->data.fl[253]=tmp[0];
  histBins->data.fl[254]=tmp[1];
  histBins->data.fl[255]=tmp[2];
}

int test3(void)
{
  int size = 30;
  double seuil=0.1; 

  /* Alloc du watershed */
  Watershed1d<int> ws(size);

  /* vecteur */
  int vect[size];
  int vectClustered[size];

  /* Liste des pic retenus */
  queue<int> picList;

  /* Nombre de pics */
  int nbpics=0;

  /* remplissage */
  for(int i=0;i<size;i++)
      vect[i] = 0;

  int deb=20;
  for(int i=0;i<5;i++) vect[i] = deb--;
  for(int i=5;i<10;i++) { vect[i] = deb; deb+=10; }
  for(int i=10;i<15;i++) { vect[i] = deb; deb--; }
  for(int i=15;i<25;i++) { vect[i] = deb; deb+=2; }
  for(int i=25;i<30;i++) { vect[i] = deb--; }

  //vect[15] = 0;


  for(int i=0;i<size;i++)
    cout << vect[i] << " ";

  cout << endl;

  //nbpics = ws.ExtractMax(vect, vectClustered, picList, seuil,0);

  /* Alloc du watershed */
  Watershed1d<float> ws2(256);
  float vect2Clustered[256];
  int hist_sizeLum[1];
  float rangeC1[2]; 
  float* rangesLum[] = {rangeC1};
  hist_sizeLum[0] = 256;
  rangeC1[0] = 0;
  rangeC1[1] = 255;
 
  CvHistogram* histo_lum = cvCreateHist(1,hist_sizeLum,CV_HIST_ARRAY,rangesLum,1);
  

  IplImage* img_in = cvLoadImage("cap000.jpg",-1);
  IplImage* imgP1 = cvCreateImage(cvGetSize(img_in),IPL_DEPTH_8U,1);
  IplImage* imgP2 = cvCreateImage(cvGetSize(img_in),IPL_DEPTH_8U,1);
  IplImage* imgP3 = cvCreateImage(cvGetSize(img_in),IPL_DEPTH_8U,1);

  cvSplit(img_in,imgP1,imgP2,imgP3,NULL);
  cvCalcHist(&imgP1,histo_lum);

  

  CvMat* bins = (CvMat*)histo_lum->bins;
  float max=0,min=9999;
  for(int i=0;i<256;i++)
    {
      if(bins->data.fl[i]>max) max = bins->data.fl[i];
      if(bins->data.fl[i]<min) min = bins->data.fl[i];

      cout << bins->data.fl[i] << " ";
    }
  cout << endl;


  /* Amelioration */
  for(int i=0;i<256;i++)
    {
      bins->data.fl[i] = (bins->data.fl[i]-min)/(max-min)*255.0;
    }

  //Hist1DDilate(histo_lum,5);
  lumdil(bins);

  cout << "Apres amelioration\n";
  for(int i=0;i<256;i++)
    {
      cout << bins->data.fl[i] << " ";
    }
  cout << endl;

  getchar();

  nbpics = ws2.ExtractMax(bins->data.fl, vect2Clustered, picList, 0.5,20);
  cout << nbpics << " marqueurs : \n";
  while(!picList.empty())
    {
      cout << "pos " << picList.front() << endl;
      picList.pop();
    }

  /* Execution du partage */
  //ws.Process(tab, tabClustered, picList, 0);
}

int main(void)
{
  test2();
  //test1();
  //test3();
}
