/***************************************************************************
                          imgdisplay.cpp  -  description
                             -------------------
    begin                : Fri Apr  2 10:28:34 CET 2004
    copyright            : (C) 2004 by Paulo Menezes
    email                : paulo@isr.uc.pt
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/


#include "imgdisplay.h"
#include <iostream>
#include <sys/types.h>
#include <linux/types.h>
#include <stdint.h>
#include <stdlib.h>

#include <sys/ipc.h>
#include <sys/shm.h>
//#include <stream.h>
#define UNSET (-1U)

#ifndef BYTE_ORDER
# error "Aiee: BYTE_ORDER not defined\n";
#endif

#define SWAP2(x) (((x>>8) & 0x00ff) |\
                  ((x<<8) & 0xff00))

#define SWAP4(x) (((x>>24) & 0x000000ff) |\
                  ((x>>8)  & 0x0000ff00) |\
                  ((x<<8)  & 0x00ff0000) |\
                  ((x<<24) & 0xff000000))

using namespace std;
/**
   Destructor 
*/
ImgDisplay::~ImgDisplay()
{
  // if(window and xdisplay)
	XDestroyWindow(xdisplay,window);
  
}
/**
   Constructor   perform some initialisations
*/

ImgDisplay::ImgDisplay(int width,int height,char * dsp,int xpos, int ypos ){

  xv_im_port=UNSET;
  xdisplay_bits=0;
  xdisplay_bytes=0;
  xpixmap_bytes=0;
  xbyteswap=0;
  no_mitshm=false;
  imgwidth=0;
  imgheight=0;
  npixels=0;
  xbuffer=NULL;
  videobuffer=NULL;

  buttonrelease=false;
  lutinited=0;
  unsigned int  vfmt_dpth[] = {
    0,               /* unused   */
    8,               /* RGB8     */
    8,               /* GRAY8    */
    16,              /* RGB15 LE */
    16,              /* RGB16 LE */
    16,              /* RGB15 BE */
    16,              /* RGB16 BE */
    24,              /* BGR24    */
    32,              /* BGR32    */
    24,              /* RGB24    */
    32,              /* RGB32    */
    16,              /* LUT2     */
    32,              /* LUT4     */
    16,		     /* YUYV     */
    16,		     /* YUV422P  */
    12,		     /* YUV420P  */
    0,		     /* MJPEG    */
    0,		     /* JPEG     */
    16,		     /* UYVY     */
  };
 vfmt_depth=vfmt_dpth;
 char * vfmt_dsc[] = {
    "none",
    "8 bit PseudoColor (dithering)",
    "8 bit StaticGray",
    "15 bit TrueColor (LE)",
    "16 bit TrueColor (LE)",
    "15 bit TrueColor (BE)",
    "16 bit TrueColor (BE)",
    "24 bit TrueColor (LE: bgr)",
    "32 bit TrueColor (LE: bgr-)",
    "24 bit TrueColor (BE: rgb)",
    "32 bit TrueColor (BE: -rgb)",
    "16 bit TrueColor (lut)",
    "32 bit TrueColor (lut)",
    "16 bit YUV 4:2:2 (packed, YUYV)",
    "16 bit YUV 4:2:2 (planar)",
    "12 bit YUV 4:2:0 (planar)",
    "MJPEG (AVI)",
    "JPEG (JFIF)",
    "16 bit YUV 4:2:2 (packed, UYVY)",
   };
 vfmt_desc=vfmt_dsc;
 struct SEARCHFORMAT fmts[]= {
   { 2, MSBFirst, 0x7c00,     0x03e0,     0x001f,     VIDEO_RGB15_BE },
   { 2, MSBFirst, 0xf800,     0x07e0,     0x001f,     VIDEO_RGB16_BE },
   { 2, LSBFirst, 0x7c00,     0x03e0,     0x001f,     VIDEO_RGB15_LE },
   { 2, LSBFirst, 0xf800,     0x07e0,     0x001f,     VIDEO_RGB16_LE },
   
   { 3, LSBFirst, 0x00ff0000, 0x0000ff00, 0x000000ff, VIDEO_BGR24    },
   { 3, LSBFirst, 0x000000ff, 0x0000ff00, 0x00ff0000, VIDEO_RGB24    },
   { 3, MSBFirst, 0x00ff0000, 0x0000ff00, 0x000000ff, VIDEO_RGB24    },
   { 3, MSBFirst, 0x000000ff, 0x0000ff00, 0x00ff0000, VIDEO_BGR24    },
   
   { 4, LSBFirst, 0x00ff0000, 0x0000ff00, 0x000000ff, VIDEO_BGR32    },
   { 4, LSBFirst, 0x0000ff00, 0x00ff0000, 0xff000000, VIDEO_RGB32    },
   { 4, MSBFirst, 0x00ff0000, 0x0000ff00, 0x000000ff, VIDEO_RGB32    },
   { 4, MSBFirst, 0x0000ff00, 0x00ff0000, 0xff000000, VIDEO_BGR32    },
   
   { 2, -1,       0,          0,          0,          VIDEO_LUT2     },
   { 0 /* END OF LIST */ },
 };
 fmt=fmts;

 // now we'll create the window and perform all necessary setup
 

  if (!(xdisplay=XOpenDisplay(dsp))){
        perror("XOpenDisplay");
        exit(1);
  }

  XGCValues gcval;

  root=RootWindow(xdisplay,screen=DefaultScreen(xdisplay));

  fgcolor=BlackPixel(xdisplay,screen);
  bgcolor=WhitePixel(xdisplay,screen);
  windoww=width;
  windowh=height;
  window=XCreateSimpleWindow(xdisplay,root,xpos,ypos,width,height,2,fgcolor,bgcolor);
  cmap=DefaultColormap(xdisplay,screen);
  XAllocNamedColor(xdisplay,cmap,"red",&color,&ignore);
  fgcolor=color.pixel;
  gcval.foreground=fgcolor;
  gcval.background=bgcolor;
  gc = XCreateGC(xdisplay,window,GCForeground|GCBackground,&gcval);

  eventmask=ExposureMask|ButtonPressMask|ButtonReleaseMask;
  
  XSelectInput(xdisplay,window,eventmask);

  XWMHints wm_hints;
  XClassHint class_hints;
  XSizeHints size_hints;
  XTextProperty windowName,iconName;
  char * window_name="ImageDisplay";
  char * icon_name="imgdsp";
  size_hints.flags=PPosition|PSize|PMinSize;
  size_hints.x=xpos;
  size_hints.y=ypos;
  size_hints.width=width;
  size_hints.height=height;
  size_hints.min_width=160;
  size_hints.min_height=120;
  if (XStringListToTextProperty(&window_name,1,&windowName)==0){
	cerr <<"error allocating structure for windowName\n";
  }
  
   if (XStringListToTextProperty(&icon_name,1,&iconName)==0){
	cerr <<"error allocating structure for iconName\n";
  }
  
  class_hints.res_name = "isy";
  class_hints.res_class="imgdisplay";
  wm_hints.initial_state=NormalState;
  wm_hints.input=True;
  //	wm_hints.icon_pixmap=icon_pixmap;
  wm_hints.flags=StateHint|IconPixmapHint|InputHint;
  XSetWMProperties(xdisplay,window,&windowName,&iconName,NULL,0,&size_hints,
					 &wm_hints, &class_hints);
  
  
  XMapWindow(xdisplay,window);

  
  
  FindVisual();
  InitVisual();
  XvImageInit();
  imgwidth=width;
  imgheight=height;
  if (im_formats[VIDEO_YUY2]){
	vfmtid=VIDEO_YUY2;
	printf("Using YUY2\n");
  }
  else
   if (im_formats[VIDEO_YUV420P]){
	vfmtid=VIDEO_YUV420P;
	printf("Using YUV420p\n");
  }
  else {
	printf("Pixmap format not (yet) supported\n");
	abort();
  }
  
  xvimage = XvCreateXimage(imgwidth,imgheight,im_formats[vfmtid], &shm);
}


/**
   Searches for a visual that can be used for displaying the images.

   Currently we are only able to deal with visuals of class TrueColor
   whose depth is greater or equal to 15 bits, or of class StaticGray
   using 8 bit depth.
  
 */
Visual * ImgDisplay::FindVisual(){
  XVisualInfo  *info, templat;
  Visual*      vi = CopyFromParent;
  int          found,i;
  char         *classe;

  templat.screen = XDefaultScreen(xdisplay);
  info = XGetVisualInfo(xdisplay, VisualScreenMask,&templat,&found);
  for (i = 0; i < found; i++) {

	switch (info[i].c_class) {
	case StaticGray:   classe = "StaticGray";  break;
	case GrayScale:    classe = "GrayScale";   break;
	case StaticColor:  classe = "StaticColor"; break;
	case PseudoColor:  classe=  "PseudoColor"; break;
	case TrueColor:    classe = "TrueColor";   break;
	case DirectColor:  classe = "DirectColor"; break;
	default:           classe = "UNKNOWN";     break;
	}
#ifdef DEBUG
	  fprintf(stderr,"visual: id=0x%lx class=%d (%s), depth=%d\n",
			  info[i].visualid,info[i].c_class,classe,info[i].depth);
#endif
  }
  for (i = 0; vi == CopyFromParent && i < found; i++)
	if (info[i].c_class == TrueColor && info[i].depth >= 15){
	  vi = info[i].visual;
	  xvisualinfo=&info[i];
	}
  for (i = 0; vi == CopyFromParent && i < found; i++)
	if (info[i].c_class == StaticGray && info[i].depth == 8){
	  vi = info[i].visual;
	  xvisualinfo=&info[i]; 
	}
  xvisual = vi;
  
  return vi;
}


void ImgDisplay::ImageToI420(unsigned char * dest, IplImage * src){
  int nlines = src->height;
  int ncols = src->width;

  unsigned char * yp,*yp1, *vp, *up;
  unsigned char   *rp, *rp1;//, *rp2, *rp3;
  unsigned char   *gp, *gp1;//, *gp2, *gp3;
  unsigned char   *bp, *bp1;//, *bp2, *bp3;
  int  rm, bm, gm;
  int uvstep,iuv;
  unsigned char * ustart, * vstart;


	iuv=0;	
	ustart = dest + (xbufwidth*xbufheight);
	
	//	  uvstart+= (x%8)
	vstart =  ustart+(xbufwidth*xbufheight)/4;
	
	uvstep=(xbufwidth)/2;
	
	//	printf("uvstep=%d\n",uvstep);
	if (src->nChannels==3){ 
	  for (int i=0; i < nlines;i+=2){
		yp = dest+i*xbufwidth;
		yp1 = dest+(i+1)*xbufwidth;
		
		up = ustart+uvstep*iuv;
		vp = vstart+uvstep*iuv;
		iuv++;
			
		bp = (unsigned char *)src->imageData+(i*src->widthStep);
		gp=bp+1;
		rp=bp+2;

		bp1=(unsigned char *)src->imageData+((i+1)*src->widthStep);
		gp1=bp1+1;
		rp1=bp1+2;
	  
		for (int j=0; j<ncols; j+=2){
		  *yp =      (int)((0.257 * (*rp)) + (0.504 * (*gp)) + (0.098 * (*bp)) + 16);
		 
		  
		  rm=*rp;
		  gm=*gp;
		  bm=*bp;
		  yp++;rp+=3;gp+=3;bp+=3;
		
		  *yp  =      (int)((0.257 * (*rp)) + (0.504 * (*gp)) + (0.098 * (*bp)) + 16);
		  rm+=*rp;
		  gm+=*gp;
		  bm+=*bp;
		  yp++;rp+=3;gp+=3;bp+=3;
		
		
		  *yp1  =      (int)((0.257 * (*rp1)) + (0.504 * (*gp1)) + (0.098 * (*bp1)) + 16);
		  rm+=*rp1;
		  gm+=*gp1;
		  bm+=*bp1;
		  yp1++;rp1+=3;gp1+=3;bp1+=3;



		  *yp1  =      (int)((0.257 * (*rp1)) + (0.504 * (*gp1)) + (0.098 * (*bp1)) + 16);
		  rm+=*rp1;
		  gm+=*gp1;
		  bm+=*bp1;
		  yp1++;rp1+=3;gp1+=3;bp1+=3;
		
		  rm/=4;
		  gm/=4; 
		  bm/=4;
		  *vp =  (int)((0.439 * (rm)) - (0.368 * (gm)) - (0.071 * (bm)) + 128);
		  
		  *up=(int)(-(0.148 * (rm)) - (0.291 * (gm)) + (0.439 * (bm)) + 128);

		  vp++;
		  up++;
		}
	  }
	} else { // bw images
	  

	  
	  for (int i=0; i < nlines;i+=2){
		yp = dest+i*xbufwidth;
		yp1 = dest+(i+1)*xbufwidth;
		  
		up = ustart+uvstep*iuv;
		vp = vstart+uvstep*iuv;
		iuv++;
		  
		bp = (unsigned char *)src->imageData+(i*src->widthStep);
		  
		bp1=(unsigned char *)src->imageData+((i+1)*src->widthStep);
		
	  
		for (int j=0; j<ncols; j+=2){
		  *yp =   (int)((0.257 * (*bp)) + (0.504 * (*bp)) + (0.098 * (*bp)) + 16);
			
		  bm=*bp;
		  yp++;bp++;
		
		  *yp  =      (int)((0.257 * (*bp)) + (0.504 * (*bp)) + (0.098 * (*bp)) + 16);
		  bm+=*bp;
		  yp++;bp++;
		
		
		  *yp1  =      (int)((0.257 * (*bp1)) + (0.504 * (*bp1)) + (0.098 * (*bp1)) + 16);

		  bm+=*bp1;
		  yp1++;bp1++;



		  *yp1  =      (int)((0.257 * (*bp1)) + (0.504 * (*bp1)) + (0.098 * (*bp1)) + 16);

		  bm+=*bp1;
		  yp1++;bp1++;
		
		
		  bm/=4;
		  *vp =  (int)((0.439 * (bm)) - (0.368 * (bm)) - (0.071 * (bm)) + 128);


		  *up=(int)(-(0.148 * (bm)) - (0.291 * (bm)) + (0.439 * (bm)) + 128);
		  
		  vp++;
		  up++;
		}


		
	}
  }
}

/**
   Convert an  IplImage into  an YUY2 pixmap usable by xvideo

   This format belong to the YUV 4:2:2 family

   \f[\begin{array}{|c|c|c|}
   & Horizontal & Verical\\\hline
   Y sample period & 1 & 1 \\\hline
   V sample period & 2 & 1 \\\hline
   U sample period & 2 & 1 \\\hline
   \end{array}\f]


*/


void ImgDisplay::ImageToYUY2(unsigned char * dest, IplImage * src){
  int nlines = src->height;
  int ncols = src->width;

  unsigned char *yp, *vp, *up;
  unsigned char   *rp;
  unsigned char   *gp;
  unsigned char   *bp;
  int  rm, bm, gm;
  register int aux;

	
	if (src->nChannels==3){ 
	  for (int i=0; i < nlines;i++){
		yp = dest+i*xbufwidth*2;
		
		up = yp+1;
		vp = yp+3;
			
		bp = (unsigned char *)src->imageData+(i*src->widthStep);
		gp=bp+1;
		rp=bp+2;
	  
		for (int j=0; j<ncols; j+=2){
		  //		  *yp =      (int)((0.257 * (*rp)) + (0.504 * (*gp)) + (0.098 * (*bp)) + 16);
		  aux= (257* (*rp)+504*(*gp)+98*(*bp))/1000+16;
		  *yp=aux;
		  rm=*rp;
		  gm=*gp;
		  bm=*bp;
		  yp+=2;rp+=3;gp+=3;bp+=3;
		
		  //		  *yp  =      (int)((0.257 * (*rp)) + (0.504 * (*gp)) + (0.098 * (*bp)) + 16);
		  aux= (257* (*rp)+504*(*gp)+98*(*bp))/1000+16;
		  *yp=aux;

		  rm+=*rp;
		  gm+=*gp;
		  bm+=*bp;
		  yp+=2;rp+=3;gp+=3;bp+=3;
		
		
		  rm/=2;
		  gm/=2; 
		  bm/=2;
		  //		  *vp =  (int)((0.439 * (rm)) - (0.368 * (gm)) - (0.071 * (bm)) + 128);
		  aux=(439*rm -368*gm -71*bm)/1000+128;
		  *vp=aux;
		  
		  //*up=(int)(-(0.148 * (rm)) - (0.291 * (gm)) + (0.439 * (bm)) + 128);
		  aux=(-148*rm - 291 * gm + 439 * bm)/1000+128;
		  *up=aux;
		  // *vp=0;
		  //*up=0;
		  
		  vp+=4;
		  up+=4;
		}
	  }
	} else { // bw images
	  

		  *vp =  (int)((0.439 * (bm)) - (0.368 * (bm)) - (0.071 * (bm)) + 128);


		  *up=(int)(-(0.148 * (bm)) - (0.291 * (bm)) + (0.439 * (bm)) + 128);
		  
		  vp++;
		  up++;
		  
	}
}




void ImgDisplay::Rgb24ToLUT2(unsigned char* dest, unsigned char* src, int p)
{
    uint16_t*  d = (uint16_t*)dest;

    while (p-- > 0) {
	*(d++) = lut_red[src[0]] | lut_green[src[1]] |
	    lut_blue[src[2]];
	src += 3;
    }
}

void ImgDisplay::Bgr24ToLUT2(unsigned char*  dest, unsigned char*  src,
	      int p)
{
    uint16_t*  d = (uint16_t*)dest;

    while (p-- > 0) {
	*(d++) = lut_red[src[2]] | lut_green[src[1]] |
	    lut_blue[src[0]];
	src += 3;
    }
}

 void ImgDisplay::GrayToLUT2(unsigned char*  dest, unsigned char*  src, int p)
{
    uint16_t*  d = (uint16_t*)dest;

    while (p-- > 0) {
	*(d++) = lut_red[*src] | lut_green[*src] | lut_blue[*src];
	src++;
    }
}

void ImgDisplay::Rgb24ToLUT4(unsigned char*  dest, unsigned char*  src, int p)
{
    unsigned int*  d = (unsigned int*)dest;

    while (p-- > 0) {
	*(d++) = lut_red[src[0]] | lut_green[src[1]] |
	    lut_blue[src[2]];
	src += 3;
    }
}

void ImgDisplay::Bgr24ToLUT4(unsigned char*  dest, unsigned char*  src,int p)
{
    unsigned int*  d = (unsigned int*)dest;

    while (p-- > 0) {
	*(d++) = lut_red[src[2]] | lut_green[src[1]] |
	    lut_blue[src[0]];
	src += 3;
    }
}

void ImgDisplay::GrayToLUT4(unsigned char*  dest, unsigned char*  src, int p)
{
    unsigned int*  d = (unsigned int*)dest;

    while (p-- > 0) {
	*(d++) = lut_red[*src] | lut_green[*src] | lut_blue[*src];
	src++;
    }
}

/**
   Initialise Look up tables
*/

void ImgDisplay::LUTinit(unsigned long red_mask, unsigned long green_mask,
						 unsigned long blue_mask, unsigned int fmtid, int swap){

    static int      once=0;
    int             rgb_red_bits = 0;
    int             rgb_red_shift = 0;
    int             rgb_green_bits = 0;
    int             rgb_green_shift = 0;
    int             rgb_blue_bits = 0;
    int             rgb_blue_shift = 0;
    unsigned int    i;
    unsigned int    mask;

    if (lutinited++) {
	fprintf(stderr,"panic: lut_init called twice\n");
	exit(1);
    }

    for (i = 0; i < 32; i++) {
        mask = (1 << i);
        if (red_mask & mask)
            rgb_red_bits++;
        else if (!rgb_red_bits)
            rgb_red_shift++;
        if (green_mask & mask)
            rgb_green_bits++;
        else if (!rgb_green_bits)
            rgb_green_shift++;
        if (blue_mask & mask)
            rgb_blue_bits++;
        else if (!rgb_blue_bits)
            rgb_blue_shift++;
    }
#if 0
    printf("color: bits shift\n");
    printf("red  : %04i %05i\n", rgb_red_bits, rgb_red_shift);
    printf("green: %04i %05i\n", rgb_green_bits, rgb_green_shift);
    printf("blue : %04i %05i\n", rgb_blue_bits, rgb_blue_shift);
#endif
    
    if (rgb_red_bits > 8)
	for (i = 0; i < 256; i++)
	    lut_red[i] = (i << (rgb_red_bits + rgb_red_shift - 8));
    else
	for (i = 0; i < 256; i++)
	    lut_red[i] = (i >> (8 - rgb_red_bits)) << rgb_red_shift;
    
    if (rgb_green_bits > 8)
	for (i = 0; i < 256; i++)
	    lut_green[i] = (i << (rgb_green_bits + rgb_green_shift - 8));
    else
	for (i = 0; i < 256; i++)
	    lut_green[i] = (i >> (8 - rgb_green_bits)) << rgb_green_shift;
    
    if (rgb_blue_bits > 8)
	for (i = 0; i < 256; i++)
	    lut_blue[i] = (i << (rgb_blue_bits + rgb_blue_shift - 8));
    else
	for (i = 0; i < 256; i++)
	    lut_blue[i] = (i >> (8 - rgb_blue_bits)) << rgb_blue_shift;

    switch (vfmt_depth[fmtid]) {
    case 16:
	if (swap) {
	    for (i = 0; i < 256; i++) {
		lut_red[i] = SWAP2(lut_red[i]);
		lut_green[i] = SWAP2(lut_green[i]);
		lut_blue[i] = SWAP2(lut_blue[i]);
	    }
	}
// 	for (i = 0; i < nconv2; i++)
// 	    lut2_list[i].fmtid_out = fmtid;
// 	ng_conv_register(NG_PLUGIN_MAGIC,"built-in",lut2_list,nconv2);
	break;
    case 32:
	if (swap) {
	    for (i = 0; i < 256; i++) {
		lut_red[i] = SWAP4(lut_red[i]);
		lut_green[i] = SWAP4(lut_green[i]);
		lut_blue[i] = SWAP4(lut_blue[i]);
	    }
	}
// 	for (i = 0; i < nconv4; i++)
// 	    lut4_list[i].fmtid_out = fmtid;
// 	ng_conv_register(NG_PLUGIN_MAGIC,"built-in",lut4_list,nconv4);
	break;
    }



}

/**
   Initialises many things that depend on the visual


   - It queries for the support of shared memory
   - it queries for the supported pixmap    formats
   - finds if byte swapping is needed or not
   - Initializes a Look Up Table for fast conversion of pixel values
   

*/

void ImgDisplay::InitVisual()
{
  XPixmapFormatValues *pf;
  int                  i,n;
  int                  format = 0;

  // start by testing if we can use shared memory 
  if (!XShmQueryExtension(xdisplay))
	no_mitshm = true;

  xdisplay_bits = xvisualinfo->depth;
  xdisplay_bytes = (xdisplay_bits+7)/8;

  pf = XListPixmapFormats(xdisplay,&n);
  for (i = 0; i < n; i++)
	if (pf[i].depth == xdisplay_bits)
	  xpixmap_bytes = pf[i].bits_per_pixel/8;

#ifdef DEBUG
	fprintf(stderr,"x11: color depth: "
			"%d bits, %d bytes - pixmap: %d bytes\n",
			xdisplay_bits,xdisplay_bytes,xpixmap_bytes);
	
	if (xvisualinfo->c_class == TrueColor || xvisualinfo->c_class == DirectColor)
	  fprintf(stderr, "x11: color masks: "
			  "red=0x%08lx green=0x%08lx blue=0x%08lx\n",
			  xvisualinfo->red_mask, xvisualinfo->green_mask, xvisualinfo->blue_mask);
	
	fprintf(stderr,"x11: server byte order: %s\n",
			ImageByteOrder(xdisplay)==LSBFirst ? "little endian":"big endian");
	
	fprintf(stderr,"x11: client byte order: %s\n",
			BYTE_ORDER==LITTLE_ENDIAN ? "little endian":"big endian");
 #endif
  
  if (ImageByteOrder(xdisplay)==LSBFirst && BYTE_ORDER!=LITTLE_ENDIAN)
	xbyteswap=true;
	
  if (ImageByteOrder(xdisplay)==MSBFirst && BYTE_ORDER!=BIG_ENDIAN)
	xbyteswap=true;
	
  if (xvisualinfo->c_class == TrueColor /* || xvisualinfo->class == DirectColor */) {
	/* pixmap format */
	// after the following cycle xdisplayfmtid will have the id of the
	// last format that will have the depth and masks in
	// correspondance with those of the xserver's visual
	for (i = 0; fmt[i].depth > 0; i++) {
	  if (fmt[i].depth  == (unsigned int)xpixmap_bytes                               &&
		  (fmt[i].order == ImageByteOrder(xdisplay) || fmt[i].order == -1) &&
		  (fmt[i].red   == xvisualinfo->red_mask     || fmt[i].red   == 0)  &&
		  (fmt[i].green == xvisualinfo->green_mask   || fmt[i].green == 0)  &&
		  (fmt[i].blue  == xvisualinfo->blue_mask    || fmt[i].blue  == 0)) {
		xdisplayfmtid = fmt[i].format;
		break;
	  }
	}
	if (fmt[i].depth == 0) {
	  fprintf(stderr, "Huh?\n");
	  exit(1);
	}

	// initialise the LUT given the masks and the format id
	LUTinit(xvisualinfo->red_mask, xvisualinfo->green_mask, xvisualinfo->blue_mask,
				xdisplayfmtid,xbyteswap);
	
	/* guess physical screen format */
	if (ImageByteOrder(xdisplay) == MSBFirst) {
	  switch (xpixmap_bytes) {
	  case 2: format = (xdisplay_bits==15) ?
				VIDEO_RGB15_BE : VIDEO_RGB16_BE; break;
	  case 3: format = VIDEO_RGB24; break;
	  case 4: format = VIDEO_RGB32; break;
	  }
	} else {
	  switch (xpixmap_bytes) {
	  case 2: format = (xdisplay_bits==15) ?
				VIDEO_RGB15_LE : VIDEO_RGB16_LE; break;
	  case 3: format = VIDEO_BGR24; break;
	  case 4: format = VIDEO_BGR32; break;
	  }
	}
  }
  
  if (xvisualinfo->c_class == StaticGray && xvisualinfo->depth == 8) {
	format = VIDEO_GRAY;
  }
  
  if (0 == format) {
	if (xvisualinfo->c_class == PseudoColor && xvisualinfo->depth == 8) {
	  fprintf(stderr,
			  "\n"
			  "8-bit Pseudocolor Visual (256 colors) is *not* supported.\n"
			  "You can startup X11 either with 15 bpp (or more)...\n"
			  "	xinit -- -bpp 16\n"
			  "... or with StaticGray visual:\n"
			  "	xinit -- -cc StaticGray\n"
			  );
	} else {
	  fprintf(stderr, "Sorry, I can't handle your strange display\n");
	}
	abort();
  }
  xdisplayfmtid = format;
}



#define HAVE_LIBXV
#ifdef HAVE_LIBXV
/**
   Determines if the X server has Xvideo extension support and then
   determines which adapter and port to use as well as the image
   format to be used
*/
void ImgDisplay::XvImageInit()
{
  int i;

  if (Success != XvQueryExtension(xdisplay,&xvver,&xvrel,&xvreq,&xvev,&xverr)) {
#ifdef DEBUG
	fprintf(stderr,"Xvideo: Server has no Xvideo extention support\n");
#endif
	return;
  }
  if (Success != XvQueryAdaptors(xdisplay,DefaultRootWindow(xdisplay),(unsigned int *)&adaptors,&ai)) {
	fprintf(stderr,"Xvideo: XvQueryAdaptors failed");
	return;
  }
  for (i = 0; i < adaptors; i++) {
	// find one that has Input and Image capabilities
	if ((ai[i].type & XvInputMask) &&
	    (ai[i].type & XvImageMask) &&
	    (xv_im_port == UNSET)) {
	  xv_im_port = ai[i].base_id;
	  xv_im_adaptor = i;
	}
  }
  if (UNSET == xv_im_port)
	return;

  fo = XvListImageFormats(xdisplay, xv_im_port, &formats);
#ifdef DEBUG
  fprintf(stderr,"  image format list for port %d\n",xv_im_port);
#endif
  for(i = 0; i < formats; i++) {
#ifdef DEBUG

	  fprintf(stderr, "    0x%x (%c%c%c%c) %s",
			  fo[i].id,
			  (fo[i].id)       & 0xff,
			  (fo[i].id >>  8) & 0xff,
			  (fo[i].id >> 16) & 0xff,
			  (fo[i].id >> 24) & 0xff,
			  (fo[i].format == XvPacked) ? "packed" : "planar");
#endif
	if (0x32595559 == fo[i].id) {
	  im_formats[VIDEO_YUY2] = fo[i].id;
#ifdef DEBUG
	  fprintf(stderr," [ok: %s]",vfmt_desc[VIDEO_YUY2]);
#endif
	}
	if (0x59565955 == fo[i].id) {
	  im_formats[VIDEO_UYVY] = fo[i].id;
#ifdef DEBUG
	  fprintf(stderr," [ok: %s]",vfmt_desc[VIDEO_UYVY]);
#endif
	}
	if (0x30323449 == fo[i].id) {
	  im_formats[VIDEO_YUV420P] = fo[i].id;
#ifdef DEBUG
	  fprintf(stderr," [ok: %s]",vfmt_desc[VIDEO_YUV420P]);
#endif
	}
#ifdef DEBUG
	fprintf(stderr,"\n");
#endif
  }
}

/**   
	  Creates an appropriate XvImage to be used with the XV extension

	  \param width image width
	  \param height image height
	  \param format image format
	  \param shm handle to return the shared memory segment in the case MITSHM exists
	  \return pointer to the created XvImage structure
	  
	  The method creates the XvImage taking into account the existence
	  of shared memory support or not.
*/
XvImage* ImgDisplay::XvCreateXimage( int width, int height, int format,
									 XShmSegmentInfo **shm)
{
  XvImage         *xvimage = NULL;
  unsigned char   *ximage_data;
  XShmSegmentInfo *shminfo = NULL;
  //  void            *old_handler;
  if (width%8)
	width+=8-width%8;
  if (height%8)
	height+=8-height%8; // force an even number of lines and cols
	
  if (no_mitshm)
	goto no_mitshm;
    
  //  old_handler = XSetErrorHandler(catch_no_mitshm);
  shminfo = (XShmSegmentInfo *)malloc(sizeof(XShmSegmentInfo));
  memset(shminfo, 0, sizeof(XShmSegmentInfo));
  xvimage = XvShmCreateImage(xdisplay, xv_im_port, format, 0,
							 width, height, shminfo);
  xbufwidth=width;
  xbufheight=height;
  
  if (NULL == xvimage)
	goto shm_error;
  shminfo->shmid = shmget(IPC_PRIVATE, xvimage->data_size,
						  IPC_CREAT | 0777);
  
  if (-1 == shminfo->shmid) {
	perror("shmget [xv]");
	goto shm_error;
  }
  shminfo->shmaddr = (char *) shmat(shminfo->shmid, 0, 0);
  if ((void *)-1 == shminfo->shmaddr) {
	perror("shmat");
	goto shm_error;
  }
  xvimage->data = shminfo->shmaddr;
  shminfo->readOnly = False;
    
  XShmAttach(xdisplay, shminfo);
  XSync(xdisplay, False);
  if (no_mitshm)
	goto shm_error;
  shmctl(shminfo->shmid, IPC_RMID, 0);
  //  XSetErrorHandler(old_handler);
  *shm = shminfo;
  return xvimage;

 shm_error:
  if (xvimage) {
	XFree(xvimage);
	xvimage = NULL;
  }
  if ((void *)-1 != shminfo->shmaddr  &&  NULL != shminfo->shmaddr)
	shmdt(shminfo->shmaddr);
  free(shminfo);
  //  XSetErrorHandler(old_handler);
  no_mitshm = true;

 no_mitshm:
  *shm = NULL;
  if (NULL == (ximage_data = (unsigned char *)malloc(width * height * 2))) {
	fprintf(stderr,"out of memory\n");
	abort();
  }
  xvimage = XvCreateImage(xdisplay, xv_im_port, format, (char *)ximage_data,
						  width, height);
  return xvimage;
}


/**
   destroys a XvImage for bot the cases where it uses shm or not
*/
void ImgDisplay::XvDestroyXimage( XvImage * xvimage, XShmSegmentInfo *shm)
{
  if (shm && !no_mitshm) {
	XShmDetach(xdisplay, shm);
	XFree(xvimage);
	shmdt(shm->shmaddr);
	free(shm);
  } else
	XFree(xvimage);
}

/**
   Renders an image to the screen
*/ 
void ImgDisplay::XvBlit( Drawable dr, GC gc, XvImage *xi, int srcx, int srcy, int srcw, int srch, int dstx, int dsty, int dstw, int dsth)
{
  if (no_mitshm)
	XvPutImage(xdisplay,xv_im_port,dr,gc,xi,srcx,srcy,srcw,srch,dstx,dsty,dstw,dsth);
  else
	XvShmPutImage(xdisplay,xv_im_port,dr,gc,xi,srcx,srcy,srcw,srch,dstx,dsty,dstw,dsth,False);
  //  printf("put done\n");
     XFlush( xdisplay );
  // XSync(xdisplay, False);

}
#endif



#if 0 // to be used later

XImage*
x11_create_ximage(Display *xdisplay, XVisualInfo *vinfo,
		  int width, int height, XShmSegmentInfo **shm)
{
    XImage          *ximage = NULL;
    unsigned char   *ximage_data;
    XShmSegmentInfo *shminfo = NULL;
    void            *old_handler;
    
    if (no_mitshm)
	goto no_mitshm;

    assert(width > 0 && height > 0);
    
    old_handler = XSetErrorHandler(catch_no_mitshm);
    shminfo = malloc(sizeof(XShmSegmentInfo));
    memset(shminfo, 0, sizeof(XShmSegmentInfo));
    ximage = XShmCreateImage(xdisplay,vinfo->visual,vinfo->depth,
			     ZPixmap, NULL,
			     shminfo, width, height);
    if (NULL == ximage)
	goto shm_error;
    shminfo->shmid = shmget(IPC_PRIVATE,
			    ximage->bytes_per_line * ximage->height,
			    IPC_CREAT | 0777);
    if (-1 == shminfo->shmid) {
	  perror("shmget [x11]");
	  goto shm_error;
    }
    shminfo->shmaddr = (char *) shmat(shminfo->shmid, 0, 0);
    if ((void *)-1 == shminfo->shmaddr) {
	  perror("shmat");
	  goto shm_error;
    }
    ximage->data = shminfo->shmaddr;
    shminfo->readOnly = False;
    
    XShmAttach(xdisplay, shminfo);
    XSync(xdisplay, False);
    if (no_mitshm)
	  goto shm_error;
    shmctl(shminfo->shmid, IPC_RMID, 0);
    XSetErrorHandler(old_handler);
    *shm = shminfo;
    return ximage;

 shm_error:
    if (ximage) {
	  XDestroyImage(ximage);
	  ximage = NULL;
    }
    if ((void *)-1 != shminfo->shmaddr  &&  NULL != shminfo->shmaddr)
	  shmdt(shminfo->shmaddr);
    free(shminfo);
    XSetErrorHandler(old_handler);
    no_mitshm = 1;

 no_mitshm:
    *shm = NULL;
    if (NULL == (ximage_data = malloc(width * height * pixmap_bytes))) {
	fprintf(stderr,"out of memory\n");
	abort();
    }
    ximage = XCreateImage(xdisplay, vinfo->visual, vinfo->depth,
			  ZPixmap, 0, ximage_data,
			  width, height,
			  8, 0);
    memset(ximage->data, 0, ximage->bytes_per_line * ximage->height);
    return ximage;
}

void
x11_destroy_ximage(Display *xdisplay, XImage *ximage, XShmSegmentInfo *shm)
{
    if (shm && !no_mitshm) {
	XShmDetach(xdisplay, shm);
	XDestroyImage(ximage);
	shmdt(shm->shmaddr);
	free(shm);
    } else
	XDestroyImage(ximage);
}

void x11_blit(Display *xdisplay, Drawable dr, GC gc, XImage *xi,
	      int a, int b, int c, int d, int w, int h)
{
    if (no_mitshm)
	XPutImage(xdisplay,dr,gc,xi,a,b,c,d,w,h);
    else
	XShmPutImage(xdisplay,dr,gc,xi,a,b,c,d,w,h,True);
}

Pixmap
x11_create_pixmap(Display *xdisplay, XVisualInfo *vinfo, struct ng_video_buf *buf)
{
    Pixmap          pixmap;
    XImage          *ximage;
    GC              gc;
    XShmSegmentInfo *shm;
    Screen          *scr = DefaultScreenOfDisplay(xdisplay);

    pixmap = XCreatePixmap(xdisplay,RootWindowOfScreen(scr),
                           buf->fmt.width, buf->fmt.height, vinfo->depth);

    gc = XCreateGC(xdisplay, pixmap, 0, NULL);

    if (NULL == (ximage = x11_create_ximage(xdisplay, vinfo, buf->fmt.width,
					    buf->fmt.height, &shm))) {
	XFreePixmap(xdisplay, pixmap);
        XFreeGC(xdisplay, gc);
        return 0;
    }
    memcpy(ximage->data,buf->data,buf->size);
    x11_blit(xdisplay, pixmap, gc, ximage, 0, 0, 0, 0,
	     buf->fmt.width, buf->fmt.height);
    x11_destroy_ximage(xdisplay, ximage, shm);
    XFreeGC(xdisplay, gc);
    return pixmap;
}
#endif // 0


void ImgDisplay::PutImage(IplImage * img){

  int npixels;
  if ((!img) ||(!img->imageData)){
	printf("error Invalid image or image data\n");
	abort();
  }
  //allow the flip the image 
  // cvFlip(img,NULL,1);
#warning add test for detect oversized images
  if ((imgwidth!=img->width)||(imgheight==img->height)){
	delete [] xbuffer;
	xbuffer=NULL;
	delete [] videobuffer;
	videobuffer=NULL;
	XvDestroyXimage(xvimage,shm);
	imgwidth=img->width;
	imgheight=img->height;
	xvimage = XvCreateXimage(imgwidth,imgheight,im_formats[vfmtid], &shm);
	npixels=imgwidth*imgheight;
  }
  
  
  if (xbuffer==NULL){
	// create xbuffer
	//xbuffersize=img->width*img->height*xpixmap_bytes;
	xbuffersize=xvimage->data_size;
	xbuffer = new char[xbuffersize];
  }
  
//   if (videobuffer==NULL){
// 	// create videobuffer
// 	videobuffersize=img->width*img->height*img->nChannels;
// 	videobuffer = new char[videobuffersize];
//   }

  /* colocar aqui a conversao para I420 */
  switch (vfmtid){
  case VIDEO_YUV420P:
	ImageToI420((unsigned char *)xbuffer,img);
	break;
  case VIDEO_YUY2:
	ImageToYUY2((unsigned char *)xbuffer,img);
	break;
  default:
	printf("Format unknown\n");
	abort();
	break;
  }
  
  memcpy(xvimage->data,xbuffer,xvimage->data_size);
 
  XvBlit( window,gc, xvimage,  0,0, xbufwidth, xbufheight, 0, 0, windoww, windowh);
  ProcEvents();
}

/**
   Internal method to test events and act accordingly
*/
long ImgDisplay::ProcEvents(){
  
  XEvent ev;
  XButtonEvent * bep=(XButtonEvent *)&ev;
  char c;
  
  //XNextEvent (xdisplay, &ev);
 XCheckWindowEvent (xdisplay,window,eventmask, &ev);
  
  switch (ev.type) {
  case Expose :
	//	cout <<"should redisplay\n";
	break;
  case ButtonPress:	
	//cout<< "mouse press on "<<bep->x<<","<<bep->y<<"\n";
	break;
  case ButtonRelease:
	buttonrelease=true;
	buttonreleasex=bep->x;
	buttonreleasey=bep->y;
   	//cout <<"mouse release on "<<bep->x<<","<<bep->y<<"\n";
  default :
	      break;
	    
	}
}


/**
   verifies if the mouse has been released in the display window
   and if true returns the position
   \param x address of the int to return the x coordinate
   \param y address of the int to return the y coordinate
   \return true or false if the mouse has been released or not
*/
bool ImgDisplay::MouseReleased(int * x, int * y){
  if (buttonrelease){
	buttonrelease=false;
	*x = buttonreleasex;
	*y = buttonreleasey;
	return true;
  }
  return false;
}
