/***************************************************************************
                          imgdisplay.h  -  description
                             -------------------
    begin                : Fri Apr  2 10:28:34 CET 2004
    copyright            : (C) 2004 by Paulo Menezes
    email                : paulo@isr.uc.pt
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <X11/X.h>
#include <X11/Xlib.h>
#include <stdio.h>

#include <string.h>
#include <X11/Xlib.h>
#include <X11/Xatom.h>
#include <X11/Intrinsic.h>
#include <X11/Shell.h>
#include <X11/extensions/XShm.h>
#include <X11/extensions/Xv.h>
#include <X11/extensions/Xvlib.h>
#include <X11/Xutil.h>

#include <cv.h>
#include "highgui.h"

#define VIDEOFMTNR 19

/**
   Class to display images on a Xwindows screen

   Does not rely on any toolkit other than xwindows itself and it
   Xvideo extension. Currently only Xvideo mode is supported but other
   modes of displaying images will be supported in the future.

   \author Paulo Menezes
*/
class ImgDisplay{
 
  Display *  xdisplay;
  Visual * xvisual;
  int screen;
  Window window,root;
  Colormap cmap;
  XColor color,ignore;
  long fgcolor,bgcolor;
  XVisualInfo * xvisualinfo;
  int xdisplayfmtid; //< ID of the display format
  int xdisplay_bits;
  int xdisplay_bytes;
  int xpixmap_bytes;
  int imgwidth;
  int imgheight;
  int npixels;
  int lutinited;
  bool xbyteswap;

  /* X */
  Drawable drawable;
  GC  gc; // graphics context

  XvImage * xvimage;
  enum {STATUS_UNKNOWN,
		STATUS_BROKEN,
		STATUS_CONVERT,
		STATUS_XVIDEO}  blit_status;

  int windowx,windowy,windoww,windowh;
  XShmSegmentInfo *shm;
  char * xbuffer;
  int xbuffersize;
  int xbufwidth, xbufheight;
  
  char * videobuffer;
  int videobuffersize;
  
  /* XV related */
  unsigned int xv_im_port;
  unsigned int xv_im_adaptor;
  unsigned int xvver, xvrel, xvreq,xvev,xverr;
  int formats;
  int adaptors;
  XvImageFormatValues  *fo;
  XvAdaptorInfo        *ai;

  long eventmask;

  bool no_mitshm;

 struct SEARCHFORMAT {
    unsigned int   depth;
    int            order;
    unsigned long  red;
    unsigned long  green;
    unsigned long  blue;
    unsigned int   format;
} *fmt; 
  
 unsigned int im_formats[VIDEOFMTNR];
 

  
  char** vfmt_desc;
  unsigned int *vfmt_depth;
  
  enum{
	VIDEO_NONE,      
	  VIDEO_RGB08,      /* bt848 dithered */
	  VIDEO_GRAY,     
	  VIDEO_RGB15_LE,   /* 15 bpp little endian */
	  VIDEO_RGB16_LE,   /* 16 bpp little endian */
	  VIDEO_RGB15_BE,   /* 15 bpp big endian */
	  VIDEO_RGB16_BE,   /* 16 bpp big endian */
	  VIDEO_BGR24,      /* bgrbgrbgrbgr (LE) */
	  VIDEO_BGR32,      /* bgr-bgr-bgr- (LE) */
	  VIDEO_RGB24,      /* rgbrgbrgbrgb (BE) */
	  VIDEO_RGB32,      /* -rgb-rgb-rgb (BE) */
	  VIDEO_LUT2,       /* lookup-table 2 byte depth */
	  VIDEO_LUT4,       /* lookup-table 4 byte depth */
	  VIDEO_YUY2,	      /* 4:2:2 */
	  VIDEO_YUV422P,    /* YUV 4:2:2 (planar) */
	  VIDEO_YUV420P,	  /* YUV 4:2:0 (planar) */
	  VIDEO_MJPEG,	  /* MJPEG (AVI) */
	  VIDEO_JPEG,	      /* JPEG (JFIF) */
	  VIDEO_UYVY,	      /* 4:2:2 */
	  VIDEO_FMT_COUNT} vfmtid;
  unsigned long lut_red[256];
  unsigned long lut_green[256];
  unsigned long lut_blue[256];

  
  bool buttonrelease;
  int buttonreleasex, buttonreleasey;

  void InitVisual();

  void ImageToI420(unsigned char * dest, IplImage * src);
  void ImageToYUY2(unsigned char * dest, IplImage * src);
  void Rgb24ToLUT2(unsigned char*  dest, unsigned char*  src, int p);
  void Bgr24ToLUT2(unsigned char*  dest, unsigned char*  src,int p);
  void GrayToLUT2(unsigned char*  dest, unsigned char*  src, int p);
  void Rgb24ToLUT4(unsigned char*  dest, unsigned char*  src, int p);
  void Bgr24ToLUT4(unsigned char*  dest, unsigned char*  src,int p);
  void GrayToLUT4(unsigned char*  dest, unsigned char*  src, int p);
  void LUTinit(unsigned long red_mask, unsigned long green_mask,
			   unsigned long blue_mask, unsigned int fmtid, int swap);
  
  void XvImageInit();
  XvImage* XvCreateXimage( int width, int height, int format, XShmSegmentInfo **shm);
  void XvDestroyXimage( XvImage * xvimage, XShmSegmentInfo *shm);
  void XvBlit( Drawable dr, GC gc, XvImage *xi, int srcx, int srcy, int srcw, int srch, int dstx, int dsty, int dstw, int dsth);
  Visual * FindVisual();  
 public:
  /**
	 Class constructor
	 \param width width of the displaying window
	 \param height height of the displaying window
	 \param dsp display name
	 \param xpos x coordinate of the NE corner if no WM is used
	 \param ypos y coordinate of the NE corner if no WM is used

	 \Note: all the input parameters have default values
  */
  ImgDisplay(int width=100, int height=100, char * dsp=":0",int xpos=0,int ypos=0);

  /**
	 Puts the given image in the displaying window
	 
  */
  void PutImage(IplImage * img);
 
  long  ProcEvents();
  bool MouseReleased(int * x, int * y);
  ~ImgDisplay();
};
