/*
	-----------------------------------------------------------------------------

	               --- CAEN SpA - Computing Systems Division --- 

	-----------------------------------------------------------------------------

	CAENRFIDTypes.h

	-----------------------------------------------------------------------------

	Created: Oct 2006
	Rev: Aug 2007

	Version 3.2L

  	-----------------------------------------------------------------------------
*/

#define MAX_ID_LENGTH							16
#define MAX_ARG_SIZE							50
#define MAX_CLIENT_SUPPORTED					20
#define MAX_READPOINT_NAME						5
#define MAX_LOGICAL_SOURCE_NAME					30
#define evUnknown                               0
#define evGlimpsed                              1
#define EvNew                                   2
#define evObserved                              3
#define evLost                                  4
#define evPurged                                5
#define TWOBITE									2
#define TIMELENGTH								19

#include <sys/time.h>
#include "includes_rfid.h"

typedef int CAENRFIDHandle;


/*
        Communication ports enum
*/

typedef enum { 
	RS232   = 0,
	RS485   = 1,
	TCP     = 2, 
	USB     = 3 
} CAENRFIDPort;


/*
        Tag identification struct: for each tag it contains 
        the ID, the length of the ID and the antenna used to 
        identify the tag.
*/

typedef struct {
    char               ID[MAX_ID_LENGTH];
    short               Length;
    char		LogicalSource[MAX_LOGICAL_SOURCE_NAME];
    char		ReadPoint[MAX_READPOINT_NAME];
    CAENRFIDProtocol	Type;
} CAENRFIDTag;


typedef struct {
    short               Length;
    byte                ID[MAX_ID_LENGTH];
    int			EvtType;
    CAENRFIDProtocol	Type;
    char		LogicalSource[MAX_LOGICAL_SOURCE_NAME];
    struct timeval	TimeStamp;
} CAENRFIDNotify;


typedef enum {
	Good		= 0,
	Poor		= 1,
	Bad		= 2
} CAENRFIDReadPointStatus;

typedef enum {
        CAENRS232_Parity_None      = 0,
        CAENRS232_Parity_Odd       = 1,
        CAENRS232_Parity_Even      = 2     
} CAENRFID_RS232_Parity;

typedef enum {

        CAENRFID_RS232_FlowControl_XonXoff    = 2,
        CAENRFID_RS232_FlowControl_Hardware   = 1,
        CAENRFID_RS232_FlowControl_None       = 0
}CAENRFID_RS232_FlowControl;

typedef enum {
		SEL_EQUAL			=	0x00,    /* comparazione di uguaglianza */
		SEL_NOT_EQUAL		=	0x01,    /* comparazione di disuguaglianza */
		SEL_GREATER_THAN	=	0x02,    /* comparazione di maggioranza */
		SEL_LOWER_THAN		=	0x03,    /* comparazione di maggioranza */
		UNS_EQUAL			=	0x04,    /* comparazione di uguaglianza */
		UNS_NOT_EQUAL		=	0x05,    /* comparazione di disuguaglianza */
		UNS_GREATER_THAN	=	0x06,    /* comparazione di maggioranza */
		UNS_LOWER_THAN		=	0x07    /* comparazione di maggioranza */
} CAENRFID_SelUnsel_Op;

typedef enum {
		CONFIG_READCYCLE			= 0,
		CONFIG_OBSERVEDTHRESHOLD	= 1,
		CONFIG_LOSTTHRESHOLD		= 2,
		CONFIG_G2_Q_VALUE			= 3,
		CONFIG_G2_SESSION			= 4,
		CONFIG_G2_TARGET			= 5,
		CONFIG_G2_SELECTED			= 6,
		CONFIG_ISO18006B_DESB		= 7
} CAENRFID_SOURCE_Parameter;

typedef enum {
		READCYCLE_MODE				= 0,
		TIME_MODE					= 1,
		NOEVENT_MODE				= 2 
} CAENRFID_EventMode;

typedef enum {
		RFID_TFTP					= 1
} CAENRFID_FWUpgradeType;
