#ifndef OPTICALFLOW_H
#define OPTICALFLOW_H


#include <cv.h>
#include <iostream>
#include <string>
using namespace std;


class OpticalFlow
{
 public:

  /* image ancienne */
  IplImage* oldimage;

  /* image masque flot */
  IplImage* imgflowmask;
  
  /* image derivee en x et en y */
  IplImage* gimgx,* gimgy;

  /* dimensions des images */
  CvSize imgSize;
  int width, height;

  /* parametres divers */
  CvTermCriteria criteria;
  double lambda;
  int xremainstep,yremainstep,mremainstep;

  OpticalFlow(int w=320, int h=240, IplImage* imgOLD=NULL);
  ~OpticalFlow();

  /* methode de calcul du flot a partir de deux images */
  inline void computeflow(IplImage* imgbw,IplImage* oldimage) { cvCalcOpticalFlowHS(imgbw, oldimage, 0, this->gimgx, this->gimgy, this->lambda, this->criteria); };

  /* methode de calcul du flot a partir d'une image et qui memorise l'image comme etant la precedente */
  inline void computeflow(IplImage* imgbw) { this->computeflow(imgbw,this->oldimage); cvCopyImage(imgbw,this->oldimage); };

  /* methode qui calcule et retourne le masque des pixels en mouvement */
  IplImage* flowmask();

  /* methode qui retourne le masque */
  inline IplImage* getflowmask() { return this->imgflowmask; };

  /* meme methode mais qui fait en meme temps le calcul du flot */
  inline IplImage* flowmask(IplImage* imgbw) { this->computeflow(imgbw); return flowmask(); };
  inline IplImage* flowmask(IplImage* imgbw,IplImage* oldimage) { this->computeflow(imgbw,oldimage); return flowmask(); };

  /* methode qui filtre une image passee en parametre par le masque */
  inline void applyflowmask(IplImage* imgInOut) { this->applyflowmask(imgInOut,imgInOut); };
  inline void applyflowmask(IplImage* imgIn, IplImage* imgOut) { cvCopy(imgIn,imgOut,this->imgflowmask); };

  /* methode qui filtre une image et calcule le flot en meme temps */
  void computeandapplyflowmask(IplImage* imgbw,IplImage* oldimage, IplImage* imgIn, IplImage* imgOut);
  inline void computeandapplyflowmask(IplImage* imgbw, IplImage* imgIn, IplImage* imgOut) { this->computeandapplyflowmask(imgbw,this->oldimage,imgIn,imgOut); cvCopyImage(imgbw,this->oldimage); };

};
#endif
