#ifndef SIR_H
#define SIR_H

#include <string.h>
#include <cv.h>
#include "ParticleFilter.h"

using std::string;

/*! \class SIRFilter
 *  \brief Class that implements SIR Particle Filter
 *  \author Ludovic Brthes
 *  \date 2002-2005
 *
 *  This class is inherited from ParticleFilter and implement the classical 
 *  Sampling Important Resampling algorithm (Condensation)
 */
class SIRFilter: public ParticleFilter
{
 public:

  /* Pour tester les detecteurs */
  FILE* fichier;

  /* Image temporaire */
  //IplImage* imgTmp;

  /*! \fn SIRFilter(TrackingContext *context)
   *  \brief The filter constructor 
   *  \param nbSpl the number of samples to use in the filter
   *  \param target_type is the type of target to load in the filter
   *  \param nomfic is teh filename containing the target parameters
   */
  SIRFilter(string nomfic, ImgProducer* imgproducer=0, twoDDetector* detector=0, Prior* prior=0, 
	    ContDynamicModel* ContDynamic=0, DiscDynamicModel** DiscDynamic=0);

  /*! \fn ~SIRFilter()
   *  \brief The filter destructor 
   */
  ~SIRFilter();

  /*! \fn int step()
   *  \brief This is the main processing step. This method correspond to a step of the
   *  algorithm. Calling step for each image achieve a Sampling Important Resampling filtering. 
   */
  int step(IplImage* obs);
};

#endif
