#ifndef FACEREC_CLASSIFIER_H
#define FACEREC_CLASSIFIER_H

#include "externParams.h"
#include "eigenbase.h"
#include <vector>
#include <iostream>
#include <string>
using namespace std;

class classifier
{
private:
  //Etat du classifieur : APP / DET
  int state;

  //Base que l on veut reconnaitre : -1 = toutes les bases
  int workingBase;

  //Image temporaire pour pre traiter les imagettes de test
  IplImage* imgtmp;

  //Seuil de classification
  double thresh;

  //poids pour le seuillage
  double weight[2];

  //chemin de stockage des images pour classifieur
  string workingPath;

public:
  //Vecteur contenant l ensemble des bases de travail
  vector<eigenbase*> base;

  //Etat d apprentissage du classifieur
  static const int APP;

  //Etat de detection du classifieur
  static const int DET;

  // Resultats de la reco
  int recoState;
  int recoId;
  double recoLikelihood;

  /* Recuperation du nom de la personne reconnue */
  char* whoIsItName();

  /* Recuperation de la proba de la personne reconnue */
  inline double whoIsItLikelihood() { return this->recoLikelihood; };

  /* Recuperation de l'etat de la reco (VX , X , V1 ou V) */
  inline int whoIsItState() { return this->recoState; };

  /* Recuperation de l'indice dans la liste des bases */
  inline int whoIsItId() { return this->recoId; };

  inline void setWorkingPath(string str) { workingPath = str; };
  inline string getWorkingPath() { return workingPath; };

  //Constructeur
  classifier(int etat = classifier::DET);

  //Destructeur
  ~classifier(void);

  //Permet de charger l integralite des bases en memoire a partir du fichier BaseList.txt
  //Cree dynamiquement chaque nouvelle base puis, charge chaque base
  int load();

  //Si le cassifieur est en mode "detection", initialisation de chaque base
  void init();

  //Ajoute l image fournie en entree a la base de nom "name"
  // si la base "name" n existe pas, on ajoute cette nouvelle base au vecteur
  void addBase(IplImage* imgIn, char* name, int where = 0);

  //Retourne une chaine de caractere correspondant a la personne la plus vraissemblante
  // V/X nomDeLaPersonne vraissemblance
  inline void whoIsIt(IplImage* imgIn, int* state, char* name, double* likelihood)
  {
    whoIsIt(imgIn);
    *state = recoState;
    *likelihood = recoLikelihood;
    strcpy(name,this->base[recoId]->getName());
  };
  inline void whoIsIt(IplImage* imgIn, char* who)
  {
    whoIsIt(imgIn);
    sprintf(who,"%d %s %3.2lf",recoState,this->base[recoId]->getName(),recoLikelihood);
  };
  void whoIsIt(IplImage* imgIn);

  //Specifie la personne que l on veut reconnaitre
  void setWorkingPeople(char* name);

  //"Oublie" que l on veut reconnaitre une seule personne
  void eraseWorkingPeople();

  //Modifie l etat du classifieur : 
  // APP = apprentissage
  // DET = detection
  void setState(int etat);

  //Sauvegarde les bases en memoire dans le fichier BaseList.txt
  void end();

  /* Charge une personne a partir du disque */
  int loadSomebody(char* name);

  /* Decharge une personne */
  int unloadSomebody(char* name);
};

#endif
