/*
 * Copyright (c) 2006 CNRS/LAAS
 *
 * Permission to use, copy, modify, and distribute this software for any
 * purpose with or without fee is hereby granted, provided that the above
 * copyright notice and this permission notice appear in all copies.
 *
 * THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
 * WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
 * ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
 * ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
 * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 * 
 * author  : Aymeric Yvernes	     
 * Date : february-August 2004 			
 * Updated : Mathias Fontmarty (september-december 2005)
 * Updated : Ludovic Brethes (june-august 2006)
 */

/*Interface.h*/
#ifndef ICU_INTERFACE_H
#define ICU_INTERFACE_H
#include <cv.h>
#include <highgui.h>  /* also from opencv */
#include "rfid/rfidStruct.h"
#include "icuStruct.h"


#define NO_FLIP 1
#define FLIP 0

/** Taille de la fenetre temporelle d'observation des heuristiques */
#define TEMPORAL_WINDOW_SIZE 20
#define TEMPORAL_WINDOW_SIZE_MOINS_UN 19

/** Liste des retours de fonction */
#define FUNCTION_OK 0
#define ERROR_IMG_NULL 1

/** Modes d'activite des process */
#define DISABLED_PROCESS 0
#define INIT_PROCESS 1
#define RUNNING_PROCESS 2

/**  On fait une detection de visage sur NB_IMAGE_FACE_DETECTION */
#define NB_IMAGE_TO_FACE_DETECTION 0

/** On fait une detection de mouvement sur NB_IMAGE_TO_MOTION_DETECTION */
#define NB_IMAGE_TO_MOTION_DETECTION 0

/** nombre de detections de mvt necessaires au declenchement du filtre de MOTION_MONITORING */
#define NB_MOTION_DETECTION 3

/** Seuil de similarite pour afficher d'une couleur differente la personne reconnue d'une autre */
#define FACEREC_SIMILARITY 0.35f

/* #ifdef __cplusplus */
/* extern "C" */
/* { */
/* #endif */

  /***************************************************************************/
  /*  I n t e r f a c e s   p o u r   l ' a c q u i s i t i o n   v i d e o  */
  /***************************************************************************/

  /** Initialisation de l'acquisition */
  void Video_create(int w, int h);

  /** Capture d'une image */
  IplImage *  Video_getImage(int flip_or_no_flip);

  /** Liberation de la video */
  void Video_delete();

  /***********************************************************/
  /*  I n t e r f a c e s   p o u r   l ' a f f i c h a g e  */
  /***********************************************************/
  
  /** Creation afficheur */
  void Display_create(int x,int y, int w, int h);

  /** Destruction afficheur */
  void Display_delete();

  /** Affichage d'une image dans l'afficheur */
  void Display_showImage(IplImage *in);

  /** Enregistrement de l'image en parametre */
  void Display_saveImage(char* filename, IplImage *in);

  /*****************************************************/
  /*  I n t e r f a c e s   d e   t r a i t e m e n t  */
  /*****************************************************/

  /** Active l'affichage */
  void Process_enableDisplay();

  /** Desactive l'affichage */
  void Process_disableDisplay();

  /** Creation / Allocation des objets de traitement */
  void Process_create(int w, int h);

  /** Activation d'un traitement (modalite) */
  void Process_enableProcess(int processType);

  /** Cette fonction active la modalite de traitement */
  void Process_setModality(int modality, int display);

  /** Retourne la modalite courante */
  int Process_getCurrentModality();

  /** Destruction d'une modalite */
  void Process_delete(int processType);

  /** Destruction de toutes les modalites */
  void Process_delete_all();

  /** Etape d'execution d'un traitement */
  int Process_step(IplImage * in , IplImage * out , ICU_TRACKING_VALUES * params_out, double alphaCam = 0.0, RFID_TAG_LIST *tagList = NULL);

  /** Initialisation de la modalite de suivi autour d'un etat en parametre */
  void Process_initTracker(double* stateVector);

  /** Initialisation de la modalite de suivi a une position donnee */
  void Process_initTrackerPos(int x, int y);
  
  /** Teste si une modalite est activee */
  int Process_isEnabled(int processType);

  /** Teste si le detecteur de visage est utilise */
  int Process_usingFaceDetector();

  /** Teste si le detecteur de mouvement est utilise */
  int Process_usingMotionDetector();

  /** Enregistrement des visages detectes */
  void Process_saveDetectedFaces(IplImage* img,char* basenm);

  /** Recherche l'indice du visage precedent le plus proche */
  int Process_getNearestFace(int isFirst);

  void Process_updateModalityBodyTracking(IplImage* in, ICU_RFID_TAG* tag, double alphaCam = 0.0);
  void Process_updateModalityFaceTracking(IplImage* in, ICU_RECOGNITION_VALUE* reco);
  void Process_updateModalityMotionMonitoring(IplImage* in);
  void Process_enableLog();
  void Process_disableLog();

  /** Initialisation du classifieur avec les bases chargees */
  void Facerec_init();

  /** Ajout d'une personne dans la liste des bases */
  int Facerec_addSomebody(IplImage* img, char* name, int isFirst);

  /** Selectionne une personne d'interet */
  void Facerec_setPeople(char* name);
  
  /** Selectionne la reconaissance de toute les personnes */
  void Facerec_recognizeAll();
  
  /** Reconaissance */
  void Facerec_whoIsIt(IplImage* imgIn, int* state, char* name, double* likelihood);
  
  /** Reconaissance avec formatage du resultat dans une chaine de caractere */
  void Facerec_whoIsItString(IplImage* imgIn, char* who);

  /** Teste une personne a reconnaitre */
  void Facerec_isIt(IplImage* imgIn, char* name, char* who);

  void Facerec_enable();
  void Facerec_disable();
  int Facerec_isEnabled();

  /** Enregistrement des bases dans la liste de bases */
  void Facerec_saveBaseList();

  /* Ajout d'une personne dans une liste et init des autres parametres */
  void Facerec_addToList(ICU_RECOGNITION_WHOLIST *whoList, char* nm);

  /* Recuperation de la liste des personnes connues sur disque */
  int Facerec_knownList(ICU_RECOGNITION_WHOLIST *whoList);

  /* Recuperation de la liste des personnes connues en memoire */
  int Facerec_whoLoaded(ICU_RECOGNITION_WHOLIST *whoLoaded); 

  /* Chargement et dechargement d'une personne */
  int Facerec_loadSomebody(char *name);
  int Facerec_unloadSomebody(char *name);

  /* Charge toute les personnes du disque */
  int Facerec_loadEverybody();

  /* Decharge toute les personnes de la memoire */
  int Facerec_unloadEverybody();

  void Facerec_clearList(ICU_RECOGNITION_WHOLIST *whoList);

  void Facerec_setWorkingPath(char workingPath[256]);

  int Facerec_getId(ICU_RECOGNITION_WHOLIST *whoList, char name[256], int state);

  void Facerec_reload(ICU_RECOGNITION_WHOLIST *whoList);

/* #ifdef __cplusplus */
/* } */
/* #endif */

/*Fonction RFID*/
void initializeRFID();
void closeRFID();
void acquisitionZone();
void acquisitionTag(char *tag);

#endif
