/********************************************************/
/*							*/
/*							*/
/*	author  : Aymeric Yvernes			*/
/*	Company : LAAS-CNRS				*/
/*	Date : february-August 2004 			*/
/*							*/
/*							*/
/********************************************************/

/**********************************************************
By : Aymeric Yvernes
updated by Ludovic Brethes and Thierry Germa
Order By :CNRS-LAAS- RIA
**********************************************************/



//#include "pom/pomStruct.h"
#include "../../icuStruct.h"

using namespace std;
#include <stdio.h>
#include <unistd.h>
#include <iostream>
#include <signal.h>


#include "applipath.h"
#include "def_paramslist.h"

/** En tetes d'acquisition et d'affichage */
#include "video.h"
#include "video4linux.h"
#include "highgui.h"
#include "interface.h"
//#include "imgdisplay.h"

/** En tete producteur d'images */
#include "ImgProducer.h"

/** En tetes detecteurs 2D */
#include "faceDetector.h"
//#include "motionDetector.h"
#include "motionBlobDetector.h"


/** En tetes filtres particulaires */
#include "ParticleFilter.h"
#include "SIR.h"
#include "ICOND.h"

/** Classifieur pour la reconaissance de visages */
#include "classifier.h"

#include "externParams.h"

#include "includes_rfid.h"
#define NBANT 8
#define ANGLERES 10

#define ANGLE_DISC 10
#define DIST_DISC 25
int rfidOn = 0;
//externalTag* Tag;
//angle horizontal d ouverture de la camera
double theta = 45.0;
int nbDetectedTags = 0;
//CvPoint zone_vis1[2+ANGLERES];

/** Afficheur rapide utilisant X */
//ImgDisplay* my_display = 0;

/** Acquisition des images */
video* my_video = 0;

/** Generateur d'images de traitement */
ImgProducer* imgBank;

/** Liste des detecteurs utilises dans ICU */
faceDetector* detectorF = 0;
//motionDetector* detectorM = 0;
motionBlobDetector* detectorM = 0;

/** Flags indiquant si les detecteurs sont necessaires */
int needFaceDetection = 0;
int needMotionDetection = 0;

/** Indique si la detection de visage doit etre filtree (eliminer les detections trop proches) */
int cleanFaceDetection = 0;

/** Filtres particulaires mis en oeuvre dans ICU (un par modalite) */
ParticleFilter* faceTrackingPF = 0;
ParticleFilter* bodyTrackingPF = 0;
ParticleFilter* motionMonitoringPF = 0;

/** Flag indiquant pour chaque modalite si elle est active */
int enabledProcess[ICU_NB_MODALITY_MAX];

/** Modalite courante */
int currentModality = ICU_NO_PROCESS;

/** Parametre utilise pour n'utiliser la detection qu'une fois sur NB_IMAGE_TO_FACE_DETECTION */
int nbImageToFaceDetection;

/** Parametre utilise pour n'utiliser la detection qu'une fois sur NB_IMAGE_TO_MOTION_DETECTION */
int nbImageToMotionDetection;

/** Image RGB temporaire pour flip de l'original */
IplImage* imgFlipped;

/** Image d'entree */
IplImage* imgInput;

/** Flag indiquant l'utilisation de la reco de visages */
int needFaceRecognition = 0;

/** Classifieur pour la reconaissance de visages */
classifier* cl;

double SeuilDecision = 0.25;

/** On enregistre une image sur NB_TO_APP dans la base d'apprentissage. Le reste sert a l'ACP */
#define NB_TO_APP 3

/** Compteur pour separer des images d'apprentissage et celles de l'ACP */
int nbToAPP = NB_TO_APP;

/** Imagette correspondant a un visage */
IplImage* resizeImage;

/** Imagette en niveaux de gris */
IplImage* greyImage;

/** Police de caractere pour affichage de la reco */
CvFont police;

/** Vecteur temporaire de proba */
double coeff[ICU_NB_MAX_DETECT];
double coeffsum;

/** Position du visage precedent dans l'ajout pour apprentissage */
int nearestFacePrecX = -1;
int nearestFacePrecY = -1;

/** Distance max au dela de laquelle le visage detecte est considere comme trop loin */
double nearestFacePrecDistMax = 999999;

/** Flag pour affichage des resultats */
int enabledDisplay = 1;

#define CONFIDENCE_ZONE 10

int cptEchec=0;
int cptUpdate = 0;
int nbDetectedBeforeUpdate=0;
int nbDetectedBeforeUpdateMotion=0;
int cptImg = 0;
bool captureVideo=false;
bool spatialFilter = false;
int cptDetectBeforeInit = TEMPORAL_WINDOW_SIZE/4;
int nbFaceTrackingMatching = 0;



/** Active l'affichage */
void Process_enableDisplay()
{
  enabledDisplay = 1;
}

/** Desactive l'affichage */
void Process_disableDisplay()
{
  enabledDisplay = 0;
}

/** Teste si le detecteur de visage est utilise */
int Process_usingFaceDetector()
{
  return needFaceDetection;
}

/** Teste si le detecteur de visage est utilise */
int Process_usingMotionDetector()
{
  return needMotionDetection;
}

/*The constructors of the different classes*/
void Process_create(int w, int h)
{
  string filename;

  printf("\n--> ICU Process_create(%d,%d)\n",w,h);

  /** Allocation du generateur d'images de traitement commun a tous les process */
  imgBank = new ImgProducer(w,h);

  printf("\n--> ICU Process_create \t:\t ImgProducer allocation done\n");

  /** Allocation des detecteurs */  
  filename = icuConfigPath+"/detectors/faceDetector.txt";
  detectorF = new faceDetector(filename.c_str(),imgBank);

  printf("\n--> ICU Process_create \t:\t faceDetector allocation done\n");

  //filename = icuConfigPath+"/detectors/motionDetector.txt";
  //detectorM = new motionDetector(filename.c_str(),imgBank);
  filename = icuConfigPath+"/detectors/motionBlobDetector.txt";
  detectorM = new motionBlobDetector(filename.c_str(),imgBank);

  printf("\n--> ICU Process_create \t:\t motionDetector allocation done\n");

  /** Allocation des filtres particulaires (un par modalite de suivi) */
  filename = icuConfigPath+"/filters/sir.txt";
  faceTrackingPF = new SIRFilter(filename.c_str(),imgBank,detectorF);
  //faceTrackingPF = new ICONDFilter(filename.c_str(),imgBank,detectorF);

  printf("\n--> ICU Process_create \t:\t faceTrackingPF allocation done\n");

  filename = icuConfigPath+"/filters/body_tracking_icond.txt";
  bodyTrackingPF = new ICONDFilter(filename.c_str(),imgBank,detectorF);

  printf("\n--> ICU Process_create \t:\t bodyTracking allocation done\n");

  filename = icuConfigPath+"/filters/motion_monitoring_icond.txt";
  motionMonitoringPF = new ICONDFilter(filename.c_str(),imgBank,detectorM);
  //motionMonitoringPF = new SIRFilter(filename.c_str(),imgBank,detectorM);

  printf("\n--> ICU Process_create \t:\t motionMonitoring allocation done\n");

  /** Desactivation des traitements */
  memset(enabledProcess,0,sizeof(int)*ICU_NB_MODALITY_MAX);  

  /** Init des parametres de suil */
  nbImageToFaceDetection = NB_IMAGE_TO_FACE_DETECTION;
  nbImageToMotionDetection = NB_IMAGE_TO_MOTION_DETECTION;

  /** Alloc image temporaire */
  imgFlipped = cvCreateImage(cvSize(w,h),IPL_DEPTH_8U,3);

  /** Imagettes pour la reco de visages */
  resizeImage =  cvCreateImage(cvSize(30,30), IPL_DEPTH_8U, 3);
  greyImage =  cvCreateImage(cvSize(30,30), IPL_DEPTH_8U, 1);

  /** Init de la police de caractere */
  cvInitFont(&police, CV_FONT_HERSHEY_SIMPLEX, 0.5, 0.5, 0, 1);

  printf("\n--> ICU Process_create \t:\t default parameters allocation done\n");

  /** Allocation du classifieur */
  cl = new classifier();

  printf("\n--> ICU Process_create \t:\t Classifier allocation done\n");

  /** Chargement des bases dans le classifieur */
  //cl->load();

  printf("\n--> ICU Process_create \t:\t Classifier bases loaded\n");

  /** Initialisation du classifieur (calcul ACP) */
  cl->init();

  printf("\n--> ICU Process_create \t:\t Classifier initialization done\n");
  printf("--> ICU Process_create DONE\n");


  captureVideo = false;
}

void Video_create(int w, int h)
{
  printf("\n--> ICU Video_create(%d,%d)\n",w,h);

  /** Allocation de l'objet d'acquisition video */
  my_video = new video4linux(w,h,true);
  my_video->doOpen(1);

  printf("\n--> ICU Video_create DONE\n");
}


/*void Display_create(int x , int y , int w , int h)
{
  printf("ICU Display_create(%d,%d,%d,%d)\n",x,y,w,h);
  //if (!my_display)
	my_display = new ImgDisplay(w,h,":0.0",x,y);
  printf("ICU Display_create DONE\n");
}*/



IplImage *  Video_getImage(int flip_or_no_flip)
{
  if(flip_or_no_flip!=FLIP) imgInput = my_video->getImage();
  else
    {
      cvFlip(my_video->getImage(),imgFlipped);
      imgInput = imgFlipped;
    }

  return imgInput;
}
/*void Display_showImage(IplImage * in)
{
  if(my_display != NULL)
    my_display->PutImage(in);
}*/

void Display_saveImage(char* filename, IplImage *in)
{
  cvSaveImage(filename,in);
}

int Process_isEnabled(int processType)
{
  return enabledProcess[processType];
}

void Process_enableProcess(int processType)
{
	int switchMod = currentModality;
	currentModality = processType;
  enabledProcess[currentModality] = INIT_PROCESS;
  enabledProcess[switchMod] = 0;

  switch(processType)
    {
    case ICU_NO_PROCESS :
      {
	/** Active la detection de visage dans Process_step */
	needFaceDetection = 0;     

	/** Active la detection de mouvement dans Process_step */
	needMotionDetection = 0; 

	/** Filtre les visages superposes */
        cleanFaceDetection = 0; 

	/** Active la reconaissance de visage dans Process_step */
	needFaceRecognition = 0;     
  
	/** Desactive le filtre spatial */
	spatialFilter = false;
	printf("enable modality %d : ICU_NO_PROCESS\n",processType);
	
      }
      break;
    case ICU_AUTO_SWITCH_MODE :
      {
	/** Active la detection de visage dans Process_step */
	needFaceDetection = 1;     

	/** Active la detection de mouvement dans Process_step */
	needMotionDetection = 1; 

	/** Filtre les visages superposes */
        cleanFaceDetection = 0; 

	/** Active la reconaissance de visage dans Process_step */
	needFaceRecognition = 0;     
  
	/** Desactive le filtre spatial */
	spatialFilter = false;

	printf("enable modality %d : ICU_AUTO_SWITCH_MODE\n",processType);
      }
      break;
    case ICU_FACE_RECOGNITION :
      {
	/** Active la detection de visage dans Process_step */
	needFaceDetection = 1;     

	/** Active la detection de mouvement dans Process_step */
	needMotionDetection = 0; 

	/** Filtre les visages superposes */
        cleanFaceDetection = 1; 

	/** Active la reconaissance de visage dans Process_step */
	needFaceRecognition = 1;     
  
	/** Desactive le filtre spatial */
	spatialFilter = false;

	printf("enable modality %d : ICU_FACE_RECOGNITION\n",processType);
      }
      break;
    case ICU_FACE_DETECTION :
      {
	/** Active la detection de visage dans Process_step */
	needFaceDetection = 1;     

	/** Active la detection de mouvement dans Process_step */
	needMotionDetection = 0; 

	/** Filtre les visages superposes */
        cleanFaceDetection = 0; 

	/** Active la reconaissance de visage dans Process_step */
	needFaceRecognition = 0;     
  
	/** Desactive le filtre spatial */
	spatialFilter = false;

	printf("enable modality %d : ICU_FACE_DETECTION\n",processType);
      }
      break;
    case ICU_MOTION_DETECTION :
      {
	/** Active la detection de visage dans Process_step */
	needFaceDetection = 0;     

	/** Active la detection de mouvement dans Process_step */
	needMotionDetection = 1; 

	/** Filtre les visages superposes */
        cleanFaceDetection = 0; 

	/** Active la reconaissance de visage dans Process_step */
	needFaceRecognition = 0;
  
	/** Desactive le filtre spatial */
	spatialFilter = false;
  
	printf("enable modality %d : ICU_MOTION_DETECTION\n",processType);  
      }
      break;
    case ICU_FACE_TRACKING :
      {
	/** Active la detection de visage dans Process_step */
	needFaceDetection = 1;     

	/** Active la detection de mouvement dans Process_step */
	needMotionDetection = 0; 

	/** Filtre les visages superposes */
        cleanFaceDetection = 0; 

	/** Active la reconaissance de visage dans Process_step */
	needFaceRecognition = 1;     
  
	/** Desactive le filtre spatial */
	spatialFilter = false;

	cptDetectBeforeInit = TEMPORAL_WINDOW_SIZE/4;
	printf("enable modality %d : ICU_FACE_TRACKING\n",processType);
      }
      break;
    case ICU_BODY_TRACKING :
      {
	/** Active la detection de visage dans Process_step */
	needFaceDetection = 1;     

	/** Active la detection de mouvement dans Process_step */
	needMotionDetection = 1; 

	/** Filtre les visages superposes */
        cleanFaceDetection = 0; 

	/** Active la reconaissance de visage dans Process_step */
	needFaceRecognition = 1;     
  
	/** Desactive le filtre spatial */
	cptDetectBeforeInit = TEMPORAL_WINDOW_SIZE/4;
	spatialFilter = false;

	printf("enable modality %d : ICU_BODY_TRACKING\n",processType);
      }
      break;
    case ICU_MOTION_MONITORING :
      {
	/** Active la detection de visage dans Process_step */
	needFaceDetection = 1;     

	/** Active la detection de mouvement dans Process_step */
	needMotionDetection = 1; 

	/** Filtre les visages superposes */
        cleanFaceDetection = 0; 

	/** Active la reconaissance de visage dans Process_step */
	needFaceRecognition = 0;     
  
	/** Desactive le filtre spatial */
	cptDetectBeforeInit = TEMPORAL_WINDOW_SIZE/10;
	spatialFilter = false;

	printf("enable modality %d : ICU_MOTION_MONITORING\n",processType);
      }
      break;
    default : 
      {
	cout << "ERROR bad processType to enable !\n";
      }
    }
}

/** Cette fonction active la modalite de traitement */
void Process_setModality(int modality, int display)
{
  enabledDisplay = display;

  /** Activation du process de la nouvelle modalite */
  Process_enableProcess(modality);

  /** Passage dans la modalite modality */
  currentModality = modality;

  printf("--> ICU Process_setModality(%d)\n",modality);
}

/** Retourne la modalite courante */
int Process_getCurrentModality()
{
  return currentModality;
}

  int Process_step(IplImage * in , IplImage * out , ICU_TRACKING_VALUES * params_out, double alphaCam, RFID_TAG_LIST *tagList)
{
  if(!in) 
    {
      cout << "ERROR image in is NULL in Process_step\n";
      return ERROR_IMG_NULL;
    }

  if(captureVideo) {
    logout << "-----------------------" << endl;
    logout << " Process_step img " << cptImg << endl;
  }

  // Pour chaque acquisition on positionne l'image dans le producteur 
  imgBank->set(in);

 
  /**********************************************************************************/
  /*                      D  E  T  E  C  T  I  O  N  S                              */
  /**********************************************************************************/

  /** Detection de visage si modalite ICU_FACE_DETECTION ou ICU_FACE_TRACKING ou ICU_BODY_TRACKING */
	if (detectorF && needFaceDetection)
	{
		//printf("Detection de visage\n");
		/** Ici, on effectue une detection sur NB_IMAGE_TO_FACE_DETECTION */
		if(!nbImageToFaceDetection)
		{
			nbImageToFaceDetection=NB_IMAGE_TO_FACE_DETECTION;
			detectorF->process();
			if(detectorF->nbdetected && cleanFaceDetection)
			{
				detectorF->cleanSuperposedFaces();
			}

			/** Recopie des detections de visage dans la structure de sortie 
			et reconaissance de visage si besoin */
			coeffsum = 0;

			int nbOverSeuil = 0;
			params_out->nbDetectedFaces = detectorF->nbdetected;
			for(int n=0;n<detectorF->nbdetected;n++)
			{
				//printf("\n--> ICU Process_step() : export face %d %d\n",n,enabledProcess[ICU_FACE_TRACKING]);
				params_out->faces[n].x = detectorF->detectedROI[n].x;
				params_out->faces[n].y = detectorF->detectedROI[n].y;
				params_out->faces[n].width = detectorF->detectedROI[n].width;
				params_out->faces[n].height = detectorF->detectedROI[n].height;

				/** Si besoin de la reco on teste les visages */
				if(needFaceRecognition)
				{
					/** Recuperation de l'imagette */
					cvSetImageROI(in,detectorF->detectedROI[n]);
					cvResize(in,resizeImage,CV_INTER_LINEAR);
					cvCvtColor(resizeImage,greyImage,CV_BGR2GRAY);

					/** Execution de la reco */
					cl->whoIsIt(greyImage);

					/** Recuperation du resultat de la reco */
					strcpy(params_out->reco[n].name,cl->whoIsItName());
					params_out->reco[n].state = cl->whoIsItState();
					params_out->reco[n].likelihood = cl->whoIsItLikelihood();
					params_out->reco[n].id = cl->whoIsItId();

					/** Mise a jour des poids du melange de gaussienne */
					coeff[n] = params_out->reco[n].likelihood;
					if(coeff[n]<SeuilDecision) coeff[n] = 0.0;
					else nbOverSeuil++;

				}

				/* Ponderation spatiale pour filtrer les detections trop loin de l'estimation precedente */
				double dx,dy;
				if(spatialFilter) 
				{
					dx = params_out->faces[n].x-params_out->target.x;
					dy = params_out->faces[n].y-params_out->target.y;
					coeff[n] *= exp(-0.5*(dx*dx*0.2/params_out->target.scale+dy*dy*0.2/params_out->target.scale));
				}

				coeffsum+=coeff[n];
			}

			/** On enleve la ROI de l'image */
			cvResetImageROI(in);

			nbDetectedBeforeUpdate = detectorF->nbdetected;

			if(needFaceRecognition && detectorF->nbdetected /*&& spatialFilter*/)
			{	      
				detectorF->nbdetected = detectorF->gm->updateMixtureCoeff(coeff,coeffsum);
			}

			/* Il est important de placer cette recopie apres l'eventuel cleanSuperposedFaces 
			ET la mise a jour liee a la reco pour que l'ensemble des heuristiques tiennent 
			compte de la selection de la personne d'interet */
			params_out->nbDetectedFacesUpdated = detectorF->nbdetected;
		}
		else nbImageToFaceDetection--;
	}
    
  /** Detection de visage si modalite ICU_MOTION_DETECTION ou ICU_MOTION_MONITORING */
	if(detectorM && needMotionDetection)
	{
		/** Ici, on effectue une detection sur NB_IMAGE_TO_MOTION_DETECTION */
		if(!nbImageToMotionDetection)
		{
			nbImageToMotionDetection=NB_IMAGE_TO_MOTION_DETECTION;

			/** Detection */
			detectorM->process();


			nbDetectedBeforeUpdateMotion = detectorM->nbdetected;

			/* Ponderation spatiale pour filtrer les detections trop loin de l'estimation precedente */
			coeffsum = 0;
			double dx,dy;
			for(int n=0;n<detectorM->nbdetected;n++) 
			{
				/**Si la region d'interet est plus large que haute, on l elimine*/
				if((double)detectorM->detectedROI[n].width/(double)detectorM->detectedROI[n].height > 0.3) 
				{
					coeff[n]=0;
				}
				else 
				{
					if(spatialFilter) 
					{
						dx = detectorM->detectedCENTER[n].x-params_out->target.x;
						dy = detectorM->detectedCENTER[n].y-params_out->target.y;
						coeff[n] = exp(-0.5*(dx*dx*0.1/params_out->target.scale+dy*dy*0.1/params_out->target.scale));
						coeffsum+=coeff[n];
					}
				}
			}
			/* Mise a jour de la mixture */
			if(spatialFilter) 
			{  
				detectorM->nbdetected = detectorM->gm->updateMixtureCoeff(coeff,coeffsum);
			}	
			/* Mise a jour de la sortie */
			params_out->motionDetected = detectorM->nbdetected;
		}
		else nbImageToMotionDetection--;
	}
 
 	//printf("rfidOn : %d\n", rfidOn);
	if(rfidOn)
    {
		/*printf("Detected tags : %d\n", tagList->nbTags);
		for(int i=0;i<tagList->nbTags;i++)
		{
			printf("Tag[%d] : (%lf, %lf, %lf, %lf)\n", i, 
				tagList->tags[i].angleMean, 
				tagList->tags[i].angleCov, 
				tagList->tags[i].distMean, 
				tagList->tags[i].distCov);
		}*/
		
		nbDetectedTags = tagList->nbTags;
		if(tagList->nbTags == 1)
		{
			params_out->tag.angleMean = tagList->tags[0].angleMean;
			params_out->tag.angleCov = tagList->tags[0].angleCov;
			params_out->tag.distMean = tagList->tags[0].distMean;
			params_out->tag.distCov = tagList->tags[0].distCov;
			
	    	/*Ponderer aussi suivant la detection rfid*/
			//Determiner l'angle de l utilisateur
			double alpha = 0.0;
			double beta = 0.0;
			//Position angulaire de l utilisateur dans rCam
			//beta = ((in->width/2)-bodyTrackingPF->pfState->ContinusP[0])*(theta/in->width);
			beta = ((in->width/2)-params_out->target.x)*(theta/in->width);
			//Position angulaire de rCam dans rRobot
			//updatealphaCam = 0.0;
			//Position angulaire de l utilisateur dans rRobot
			alpha = alphaCam - beta;
			//Si le tag est dans l'image
			/*if(params_out->tag.angleMean-alphaCam < theta/2 && detectorF->nbdetected == 0)
			{
				coeff[0] = exp(-0.5*pow(((alpha-params_out->tag.angleMean)/params_out->tag.angleCov),2));
				coeffsum = coeff[0];
				//Mise a jour de la gaussienne
				detectorF->gm->updateMixtureCoeff(coeff,coeffsum);
				
				//Position moyenne du tag dans l'image
				detectorF->gm->glist[0]->mean[0] = in->width/2 + (params_out->tag.angleMean-alphaCam)/(theta/in->width);
      			detectorF->gm->glist[0]->mean[1] = in->height/2;
			}*/
			
			
		}
		/*else
		{
			printf("No tag was found !!!!\n");
		}*/
    }
 
  /**********************************************************************************/
  /*                M O D A L I T I E S   E X E C U T I O N S                       */
  /**********************************************************************************/

  char text[256];
  /** Memorisation de la modalite qui sort les resultats */
  params_out->modality = currentModality;
  switch(currentModality)
    {
    case ICU_NO_PROCESS :
      {
	sprintf(text, "ICU_NO_PROCESS");
	cvPutText(in,text,cvPoint(5, 15),&police,CV_RGB(0,0,255));
	//printf("NO PROCESS\n");
      }
      break;
    case ICU_AUTO_SWITCH_MODE :
      {
	sprintf(text, "ICU_AUTO_SWITCH_MODE");
	cvPutText(in,text,cvPoint(5, 15),&police,CV_RGB(0,0,255));
	//printf("ICU_AUTO_SWITCH_MODE\n");
        if(enabledDisplay) 
	{
	  detectorF->dispROI(in, 2, CV_RGB(0,255,255));
	  detectorM->dispROI(in, 2, CV_RGB(0,0,255));
	}
      }
      break;
    case ICU_FACE_RECOGNITION :
      {
	sprintf(text, "ICU_FACE_RECOGNITION");
	cvPutText(in,text,cvPoint(5, 15),&police,CV_RGB(0,0,255));
	//printf("ICU_FACE_RECOGNITION\n");

	/* Si pas d'affichage */
	if(!enabledDisplay) 
	  break;

	/** Affichage du resultat de la reco et de la detection */
	CvPoint pt;	
	
	for(int i=0;i<nbDetectedBeforeUpdate;i++)
	  {
	    /** Positionnement du texte */
	    pt.x = detectorF->detectedROI[i].x;
	    pt.y = detectorF->detectedROI[i].y + detectorF->detectedROI[i].height + 15;

	    /** Formatage du texte */
	    sprintf(text,"%s %3.2lf%%",params_out->reco[i].name,params_out->reco[i].likelihood*100);
	    //printf("%s\n",text);
	    
	    /** Affichage */
	    if(params_out->reco[i].likelihood==-1)
	      {
		//cvPutText(in,text,pt,&police,CV_RGB(0,0,255));
		cvRectangle(in,cvPoint(detectorF->detectedROI[i].x,detectorF->detectedROI[i].y),
			    cvPoint(detectorF->detectedROI[i].x+detectorF->detectedROI[i].width,
				    detectorF->detectedROI[i].y+detectorF->detectedROI[i].height),
			    CV_RGB(0,0,255),3,8,0); 
	      }
	    else if(params_out->reco[i].likelihood>FACEREC_SIMILARITY)
	      {
		cvPutText(in,text,pt,&police,CV_RGB(0,255,0));
		cvRectangle(in,cvPoint(detectorF->detectedROI[i].x,detectorF->detectedROI[i].y),
			    cvPoint(detectorF->detectedROI[i].x+detectorF->detectedROI[i].width,
				    detectorF->detectedROI[i].y+detectorF->detectedROI[i].height),
			    CV_RGB(0,255,0),3,8,0); 
	      }
	    else
	      {
		//cvPutText(in,text,pt,&police,CV_RGB(0,255,0));
		cvRectangle(in,cvPoint(detectorF->detectedROI[i].x,detectorF->detectedROI[i].y),
			    cvPoint(detectorF->detectedROI[i].x+detectorF->detectedROI[i].width,
				    detectorF->detectedROI[i].y+detectorF->detectedROI[i].height),
			    CV_RGB(255,0,0),3,8,0); 
	      }
	  }      
      }
      break;
    case ICU_FACE_DETECTION :
      {
	sprintf(text, "ICU_FACE_DETECTION");
	cvPutText(in,text,cvPoint(5, 15),&police,CV_RGB(0,0,255));
	//printf("ICU_FACE_DETECTION\n");
	/* Si affichage */
	if(enabledDisplay) 
	  detectorF->disp(in, 2, CV_RGB(0,255,255), 2, CV_RGB(0,255,255));
      }
      break;
    case ICU_MOTION_DETECTION :
      {
	sprintf(text, "ICU_MOTION_DETECTION");
	cvPutText(in,text,cvPoint(5, 15),&police,CV_RGB(0,0,255));
	//printf("ICU_MOTION_DETECTION\n");
	/* Si affichage */
	if(enabledDisplay) 
	  detectorM->disp(in, 2, CV_RGB(0,0,255), 2, CV_RGB(0,0,255));
      }
      break;
    case ICU_FACE_TRACKING :
      {
	sprintf(text, "ICU_FACE_TRACKING");
	cvPutText(in,text,cvPoint(5, 15),&police,CV_RGB(0,0,255));
	//printf("ICU_FACE_TRACKING : %d\n",enabledProcess[ICU_FACE_TRACKING]);
	switch(enabledProcess[ICU_FACE_TRACKING])
	  {
	  case INIT_PROCESS :
	    {
			params_out->target.available = 0;
	      /** Init du filtre selon la detection */
	      if(faceTrackingPF->InitSamplesFromDetection()) {
		cptDetectBeforeInit--;
	      	//printf("%d before init\n", cptDetectBeforeInit);
	      }
		
	      if(!cptDetectBeforeInit) {
		/** Passage en mode running */
		enabledProcess[ICU_FACE_TRACKING] = RUNNING_PROCESS;

		cptEchec = 0;
		cptDetectBeforeInit = TEMPORAL_WINDOW_SIZE/4;

		/* On active le filtrage spatial */
		spatialFilter = true;
	      }
	      else
		break;
	    }
	    /* Attention pas de break volontairement pour faire un filtrage des l'init sur l'image courante */
	  case RUNNING_PROCESS :
	    { 
			params_out->target.available = 1;
	      /** Tracking */
	      if(!faceTrackingPF->step(in)) {
		//printf("Echec tracker\n");
		/* Echec du filtre on revient en init */
		enabledProcess[ICU_FACE_TRACKING] = INIT_PROCESS;
		/* On desactive le filtrage spatial */
		spatialFilter = false;
		break;
	      }
		/*sprintf(text, "target --> %d, %d, %f", (int)(faceTrackingPF->pfState->ContinusP[0]), (int)(faceTrackingPF->pfState->ContinusP[1]),faceTrackingPF->pfState->ContinusP[2]);
		cvPutText(in,text,cvPoint(5, 30),&police,CV_RGB(0,0,255));*/
	      /** Recopie des resultats vers la sortie */
	      params_out->target.x = (int)(faceTrackingPF->pfState->ContinusP[0]);
	      params_out->target.y= (int)(faceTrackingPF->pfState->ContinusP[1]);
	      params_out->target.theta = 0;
	      params_out->target.scale = faceTrackingPF->pfState->ContinusP[2];	 

	      /* Mise a jour de la modalite */
	      Process_updateModalityFaceTracking(in, params_out->reco);

	      /** Affichage du resultat */
	      if(enabledDisplay) faceTrackingPF->DispState(in);     

	    }
	    break;
	  default : 
	    {
	      cout << "Unknown process mode\n";
	    }
	  }
      }
      if(enabledDisplay) 
	detectorF->dispROI(in, 2, CV_RGB(0,255,255));
      break;
    case ICU_BODY_TRACKING :
      {
	sprintf(text, "ICU_BODY_TRACKING");
	cvPutText(in,text,cvPoint(5, 15),&police,CV_RGB(0,0,255));
	//printf("ICU_BODY_TRACKING\n");
	switch(enabledProcess[ICU_BODY_TRACKING])
	  {
	  case INIT_PROCESS :
	    {
			params_out->target.available = 0;
	      /** Init du filtre selon la detection */
	      if(bodyTrackingPF->InitSamplesFromDetection()) {
		cptDetectBeforeInit--;
	      }
	      //printf("%d before init\n", cptDetectBeforeInit);
	      if(!cptDetectBeforeInit) 
	      {
	      	
			/** Passage en mode running */
			enabledProcess[ICU_BODY_TRACKING] = RUNNING_PROCESS;

			cptEchec = 0;
			cptUpdate = 0;
			cptDetectBeforeInit = TEMPORAL_WINDOW_SIZE/4;

			/* On active le filtrage spatial */
			spatialFilter = true;
	      }
	      else
		break;
	    }
	    /* Attention pas de break volontairement pour faire un filtrage des l'init sur l'image courante */
	  case RUNNING_PROCESS :
	    {
			params_out->target.available = 1;
	      //printf("ICU_BODY_TRACKING : running\n");
	      /** Tracking */
	      if(!bodyTrackingPF->step(in)) 
	      {
			printf("Echec tracker\n");
			/* Echec du filtre on revient en init */
			enabledProcess[ICU_BODY_TRACKING] = INIT_PROCESS;
			/* On desactive le filtrage spatial */
			spatialFilter = false;
			break;
	      }
	      
		/*sprintf(text, "target --> %d, %d, %f", (int)(bodyTrackingPF->pfState->ContinusP[0]), (int)(bodyTrackingPF->pfState->ContinusP[1]),bodyTrackingPF->pfState->ContinusP[2]);
		cvPutText(in,text,cvPoint(5, 30),&police,CV_RGB(0,0,255));*/
	      /** Recopie des resultats vers la sortie */
	      params_out->target.x = (int)(bodyTrackingPF->pfState->ContinusP[0]);
	      params_out->target.y= (int)(bodyTrackingPF->pfState->ContinusP[1]);
	      /*ATTENTION : POSITION ANGULAIRE DE LA TARGET PAR RAPPORT AU ROBOT*/
	      params_out->target.theta = 0;
	      params_out->target.scale = bodyTrackingPF->pfState->ContinusP[2];	      

	      /* Mise a jour de la modalite */
	      Process_updateModalityBodyTracking(in, &(params_out->tag), alphaCam);
	    }
	    break;
	  default : 
	    {
	      cout << "Unknown process mode\n";
	    }
	  }	
      }
      if(enabledDisplay) 
	detectorF->dispROI(in, 2, CV_RGB(0,255,255));
      break;
    case ICU_MOTION_MONITORING :
      {
	sprintf(text, "ICU_MOTION_MONITORING");
	cvPutText(in,text,cvPoint(5, 15),&police,CV_RGB(0,0,255));
	//printf("ICU_MOTION_MONITORING\n");
	switch(enabledProcess[ICU_MOTION_MONITORING])
	  {
	  case INIT_PROCESS :
	    {
			params_out->target.available = 0;
	      /** Init du filtre selon la detection */
	      if(motionMonitoringPF->InitSamplesFromDetection()) {
		cptDetectBeforeInit--;
	      }
	      
	      /** Passage en mode running au bout de NB_MOTION_DETECTION detections */
	      if(!cptDetectBeforeInit) {
		cptDetectBeforeInit = TEMPORAL_WINDOW_SIZE/10;
		enabledProcess[ICU_MOTION_MONITORING] = RUNNING_PROCESS;
		cptEchec = 0;
		/* On active le filtrage spatial */
		spatialFilter = true;
	      }
	      else
		break;
	    }
	    /* Attention pas de break volontairement pour faire un filtrage des l'init sur l'image courante */
	  case RUNNING_PROCESS :
	    {
			params_out->target.available = 1;
	      /** Tracking */
	      if(!motionMonitoringPF->step(in)) {
		/* Echec du filtre on revient en init */
		enabledProcess[ICU_MOTION_MONITORING] = INIT_PROCESS;
		/* On desactive le filtrage spatial */
		spatialFilter = false;
		break;
	      }
		/*sprintf(text, "target --> %d, %d, %f", (int)(motionMonitoringPF->pfState->ContinusP[0]), (int)(motionMonitoringPF->pfState->ContinusP[1]),motionMonitoringPF->pfState->ContinusP[2]);
		cvPutText(in,text,cvPoint(5, 30),&police,CV_RGB(0,0,255));*/
	      /** Recopie des resultats vers la sortie */
	      params_out->target.x = (int)(motionMonitoringPF->pfState->ContinusP[0]);
	      params_out->target.y = (int)(motionMonitoringPF->pfState->ContinusP[1]);
	      params_out->target.theta = 0;
	      params_out->target.scale = motionMonitoringPF->pfState->ContinusP[2];	      
 
	      /* Mise a jour de la modalite courante */
	      Process_updateModalityMotionMonitoring(in);

	      /** Affichage du resultat */
	      if(enabledDisplay) motionMonitoringPF->DispState(in);
	      
	     
	    }
	    break;
	  default : 
	    {
	      cout << "Unknown process mode\n";
	    }
	  }
      }
      if(enabledDisplay) 
      {
	detectorF->dispROI(in, 2, CV_RGB(0,255,255));
	detectorM->dispROI(in, 2, CV_RGB(0,0,255));
      }
      break;    
    } // Fin du switch currentModality


  if(captureVideo) {
    cptImg++;
  }
  
  return 1;
}

/* Gestion des switchs pour sequence */
void Process_updateModalityBodyTracking(IplImage* in, ICU_RFID_TAG* tag, double alphaCam) {
  //Process_enableDisplay(); 
  bodyTrackingPF->target->setColor(CV_RGB(0,255,0));

  /** On parcours les visages detectes                                  */
  /** On identifie le  + proche de l'estimee dans une zone de confiance */
  /** --> si aucun visage un incremente un echec                        */
  /** --> si visage :                                                   */
  /**        - c la personne on remet le compteur d'echec a 0           */
  /**        - c pas la personne on desactive l'affichage du tracker    */
  /** Si ne nb d'echecs depasse un seuil on passe en INIT               */
  double mindist=CONFIDENCE_ZONE*bodyTrackingPF->pfState->ContinusP[2];
  double dist;
  int minid=-1;

	//Si le RFID est en marche, on monitore l echec sur le RFID
	if(rfidOn)
	{
		if(nbDetectedTags != 0)
		{
			//Si la distance entre l estimee du tracker et l estimee du tag est trop 
			// grande, on repasse en INIT
			double alpha = 0.0;
			double beta = 0.0;
			//Position angulaire de l utilisateur dans rCam
			beta = ((in->width/2)-bodyTrackingPF->pfState->ContinusP[0])*(theta/in->width);
			//Position angulaire de l utilisateur dans rRobot
			double angleTag = tag->angleMean;
			if(angleTag > 180.0)
				angleTag -= 360.0;
			alpha = alphaCam - beta;
			double conf = exp(-0.5*(pow((alpha-angleTag)/tag->angleCov,2)));
			printf("Alpha : %lf = %lf + %lf, angleMean : %lf, angleCov : %lf, confidence : %lf\n", alpha, alphaCam, beta, angleTag, tag->angleCov, conf);
			
			if(conf<0.1)
			{
				cptEchec ++;
			}
			else
			{
				cptEchec = 0;
				cptUpdate ++;
			}
		}
		else
		{
			cptUpdate ++;
		}
	}
	//Sinon, on monitore sur la reco
	else
	{
		if(nbDetectedBeforeUpdate) 
		{
			for(int i=0;i<nbDetectedBeforeUpdate;i++) 
			{
			    dist = sqrt(pow(((int)(bodyTrackingPF->pfState->ContinusP[0]-detectorF->detectedCENTER[i].x)),2)+
				  pow(((int)(bodyTrackingPF->pfState->ContinusP[1]-detectorF->detectedCENTER[i].y)),2));
			    if(dist<mindist) 
			    {
			    	mindist = dist;
			    	minid = i;
			    }
			} 
		}   

	    /** --> si aucun visage on incremente un echec                        */
		if(minid==-1) 
		{
			bodyTrackingPF->target->setColor(CV_RGB(255,0,0));
			cptEchec++;
			//printf("Echec : pas de dectection ou pas de visage proche du tracker\n");
		}
		else 
		{
			//printf("coeff = %f\n",coeff[minid]);
			if(coeff[minid]) 
			{
				//printf("Pas d'Echec : visage reconnu proche du tracker\n");
				cptEchec = 0; 
				cptUpdate ++;  
				if(coeff[minid]>0.4) 
				{
					double mm[3];
					mm[0] = detectorF->detectedCENTER[minid].x;
					mm[1] = detectorF->detectedCENTER[minid].y;
					mm[2] = bodyTrackingPF->pfState->ContinusP[2];
					//bodyTrackingPF->target->InitModelFromPrior(mm);
					bodyTrackingPF->InitSamplesFromPrior(mm);
				}
			}
			else 
			{
				//Process_disableDisplay();
				//printf("Pas d'Echec : visage pas reconnu proche du tracker\n");
				bodyTrackingPF->target->setColor(CV_RGB(255,0,0));      
			}
		}
		
	}
	if(cptEchec>10 || cptUpdate == 0) 
	{
		enabledProcess[ICU_BODY_TRACKING] = INIT_PROCESS;

		// On desactive le filtrage spatial
		spatialFilter = false;
	}
    /** Affichage du resultat */
    if(enabledDisplay && enabledProcess[ICU_BODY_TRACKING] == RUNNING_PROCESS) 
    	bodyTrackingPF->DispState(in);
	
}



/* Gestion des switchs pour sequence */
void Process_updateModalityFaceTracking(IplImage* in, ICU_RECOGNITION_VALUE* reco) {
  faceTrackingPF->target->setColor(CV_RGB(0,255,0));

  /** On parcours les visages detectes                                  */
  /** On identifie le  + proche de l'estimee dans une zone de confiance */
  /** --> si aucun visage un incremente un echec                        */
  /** --> si visage on remet le compteur d'echec a 0                    */
  /** Si ne nb d'echecs depasse un seuil on passe en INIT               */
  double mindist=CONFIDENCE_ZONE*faceTrackingPF->pfState->ContinusP[2];
  double dist;
  int minid=-1;
  char text[256];
  if(nbDetectedBeforeUpdate) {
    for(int i=0;i<nbDetectedBeforeUpdate;i++) {
      dist = sqrt(pow(((int)(faceTrackingPF->pfState->ContinusP[0]-detectorF->detectedCENTER[i].x)),2)+
		  pow(((int)(faceTrackingPF->pfState->ContinusP[1]-detectorF->detectedCENTER[i].y)),2));
      if(dist<mindist) {
	mindist = dist;
	minid = i;
      }
    }
  }    

  for(int i=0;i<nbDetectedBeforeUpdate;i++) {
    if(i!=minid) {
      cvRectangle(in,cvPoint(detectorF->detectedROI[i].x,detectorF->detectedROI[i].y),
		    cvPoint((detectorF->detectedROI[i].x+detectorF->detectedROI[i].width),
			    (detectorF->detectedROI[i].y+detectorF->detectedROI[i].height)),CV_RGB(0,0,255),2);	
		  
      /** Execution de la reco */
      sprintf(text,"%s, %.2f%%", reco[i].name, reco[i].likelihood*100);
      cvPutText(in,
		text,
		cvPoint(detectorF->detectedROI[i].x,detectorF->detectedROI[i].y+detectorF->detectedROI[i].height+15),
		&police,
		CV_RGB(0,0,255));
      }
    }

    /** --> si aucun visage on incremente un echec                        */
    if(minid==-1) {
      faceTrackingPF->target->setColor(CV_RGB(255,0,0));
      cptEchec++;
      /* On remet a 0 le compteur de matching */
      nbFaceTrackingMatching = 0;
    }
    else {
      cptEchec = 0;   

      sprintf(text,"%s, %.2f%%", reco[minid].name, reco[minid].likelihood*100);
      cvPutText(in,
		text,
		cvPoint(detectorF->detectedROI[minid].x,detectorF->detectedROI[minid].y+detectorF->detectedROI[minid].height+15),
		&police,
		CV_RGB(0,255,0));
      /* On incremente un compteur de matching */
      nbFaceTrackingMatching++;

      /* Si on a matche suffisament on reinitialise le tracker sur la detection */
      if(nbFaceTrackingMatching>TEMPORAL_WINDOW_SIZE) {
	//printf("INIT FORCE\n");
	double mm[3];
	mm[0] = detectorF->detectedCENTER[minid].x;
	mm[1] = detectorF->detectedCENTER[minid].y;
	mm[2] = faceTrackingPF->pfState->ContinusP[2];
	faceTrackingPF->InitSamplesFromPrior(mm);
	nbFaceTrackingMatching = 0;
      }
    }

    if(cptEchec>TEMPORAL_WINDOW_SIZE) {
    enabledProcess[ICU_FACE_TRACKING] = INIT_PROCESS;

    /* On desactive le filtrage spatial */
    spatialFilter = false;
  }
}

/* Gestion des switchs pour sequence */
void Process_updateModalityMotionMonitoring(IplImage* in) {
  motionMonitoringPF->target->setColor(CV_RGB(0,255,0));

  /** On parcours les blobs detectes                                    */
  /** On identifie le  + proche de l'estimee dans une zone de confiance */
  /** --> si aucun blob un incremente un echec                          */
  /** --> si blob on remet le compteur d'echec a 0                      */
  /** Si ne nb d'echecs depasse un seuil on passe en INIT               */
  double mindist=CONFIDENCE_ZONE*motionMonitoringPF->pfState->ContinusP[2];
  double dist;
  int minid=-1;

  if(nbDetectedBeforeUpdateMotion) {
    for(int i=0;i<nbDetectedBeforeUpdateMotion;i++) {
      dist = sqrt(pow(((int)(motionMonitoringPF->pfState->ContinusP[0]-detectorM->detectedCENTER[i].x)),2)+
		  pow(((int)(motionMonitoringPF->pfState->ContinusP[1]-detectorM->detectedCENTER[i].y)),2));
      if(dist<mindist) {
	mindist = dist;
	minid = i;
      }
    }    

    /*for(int i=0;i<nbDetectedBeforeUpdateMotion;i++) {
      if(i!=minid) { 
	cvRectangle(in,cvPoint(detectorM->detectedROI[i].x,detectorM->detectedROI[i].y),
		    cvPoint((detectorM->detectedROI[i].x+detectorM->detectedROI[i].width),
			    (detectorM->detectedROI[i].y+detectorM->detectedROI[i].height)),CV_RGB(0,0,255),2);	
      }
    }*/
  }
  
  /** --> si aucun blob on incremente un echec                        */
  if(minid==-1) {
    motionMonitoringPF->target->setColor(CV_RGB(255,0,0));
    cptEchec++;
    //printf("Echec : pas de dectection ou pas de blob proche du tracker\n");
  }
  else {
    //printf("Pas d'Echec : blob proche du tracker\n");
    cptEchec = 0;   
  }
  
  if(cptEchec>TEMPORAL_WINDOW_SIZE) {
    enabledProcess[ICU_MOTION_MONITORING] = INIT_PROCESS;

    /* On desactive le filtrage spatial */
    spatialFilter = false;
  }
}

void Process_enableLog() {
  logout.open(ficlog.c_str(),ofstream::out | ofstream::app);
  captureVideo = true;
}

void Process_disableLog() {
  if(logout.is_open())
    logout.close();
  captureVideo = false;
}



/** Initialisation de la modalite de suivi a une position donnee */
void Process_initTrackerPos(int x, int y)
{
  double initVector[4];
  double* priormean;

  /** On place la position */
  initVector[0] = x;
  initVector[1] = y;


  /** On fait l'init */
  switch(currentModality)
    {
    case ICU_FACE_TRACKING :
      priormean = faceTrackingPF->target->prior->getMean();
      initVector[2] = priormean[2];
      initVector[3] = priormean[3];
      faceTrackingPF->InitSamplesFromPrior(initVector);
      break;
    case ICU_BODY_TRACKING :
      priormean = bodyTrackingPF->target->prior->getMean();
      initVector[2] = priormean[2];
      bodyTrackingPF->InitSamplesFromPrior(initVector);
      break;
    case ICU_MOTION_MONITORING :
      priormean = motionMonitoringPF->target->prior->getMean();
      initVector[2] = priormean[2];
      motionMonitoringPF->InitSamplesFromPrior(initVector);
      break;
    }
}

/** Initialisation de la modalite de suivi autour d'un etat en parametre */
void Process_initTracker(double* stateVector)
{
  switch(currentModality)
    {
    case ICU_FACE_TRACKING :
      faceTrackingPF->InitSamplesFromPrior(stateVector);
      break;
    case ICU_BODY_TRACKING :
      bodyTrackingPF->InitSamplesFromPrior(stateVector);
      break;
    case ICU_MOTION_MONITORING :
      motionMonitoringPF->InitSamplesFromPrior(stateVector);
      break;
    }
}


/** The destructors */
/*void Display_delete()
{
  cout<<"Delete display\n";
  if (my_display)
  {
    delete my_display;
    my_display = NULL;
  }
}*/

void  Process_delete(int processType)
{
  switch(processType)
    {
    case ICU_NO_PROCESS:
      {

      }
      break;

    case ICU_AUTO_SWITCH_MODE:
      {

      }
      break;

      /* On ne detruit le detecteur de visage que si il existe et que les trackers l'utilisant n'existent plus */
    case ICU_FACE_DETECTION :
      {
	if(detectorF && !faceTrackingPF && !bodyTrackingPF)
	  {
	    printf("delete Face detector\n");
	    enabledProcess[ICU_FACE_DETECTION] = DISABLED_PROCESS;
	    delete detectorF;
	    detectorF = NULL;
	  }
	else printf("cannot destroy Face detector\n");
      }
      break;

    case ICU_MOTION_DETECTION:
      {
	if(detectorM && !motionMonitoringPF)
	  {
	    printf("delete Motion detection\n");
	    enabledProcess[ICU_MOTION_DETECTION] = DISABLED_PROCESS;
	    delete detectorM;
	    detectorM = NULL;
	  }
	else printf("cannot destroy Motion detector\n");
      }
      break;

    case ICU_FACE_TRACKING:
      {
	if (faceTrackingPF)
	  {
	    printf("delete faceTracking particle filter\n");
	    enabledProcess[ICU_FACE_TRACKING] = DISABLED_PROCESS;
	    delete faceTrackingPF;
	    faceTrackingPF=NULL;
	  }
	else printf("cannot destroy faceTracking particle filter\n");
      }
      break;

    case ICU_BODY_TRACKING:
      {
	if(bodyTrackingPF)
	  {
	    printf("delete bodyTracking particle filter\n");
	    enabledProcess[ICU_BODY_TRACKING] = DISABLED_PROCESS;
	    delete bodyTrackingPF;
	    bodyTrackingPF=NULL;
	  }
	else printf("cannot destroy bodyTracking particle filter\n");
      }
      break;

    case ICU_MOTION_MONITORING:
      {
	if(motionMonitoringPF)
	  {
	    printf("delete motionMonitoring particle filter\n");
	    enabledProcess[ICU_MOTION_MONITORING] = DISABLED_PROCESS;
	    delete motionMonitoringPF;
	    motionMonitoringPF=NULL;
	  }
	else printf("cannot destroy motionMonitoring particle filter\n");
      }
      break;

    }
}

void Process_delete_all()
{
  if(logout.is_open())
    logout.close();

  cout << "Destroying all process" << endl;
  Process_delete(ICU_MOTION_MONITORING);
  Process_delete(ICU_BODY_TRACKING);
  Process_delete(ICU_FACE_TRACKING);
  Process_delete(ICU_FACE_DETECTION);
  Process_delete(ICU_MOTION_DETECTION);  

  /** Desactivation des traitements */
  memset(enabledProcess,0,sizeof(int)*ICU_NB_MODALITY_MAX);  

  cout << "Destroying all process OK" << endl;
}


void Video_delete()
{
  cout<<"Delete video\n";
  delete my_video;
  my_video = 0;
}

/** Enregistrement des visages detectes */
void Process_saveDetectedFaces(IplImage* img,char* basenm)
{
  char fileName[256];
  int i=0;

  /** Sauvegarde des visages */
  for(i=0;i<detectorF->nbdetected;i++)
    {
      /** Formatage du nom */
      sprintf(fileName,"%s%d.jpg",basenm,i);

      /** On positionne une region d'interet sur l'image */
      cvSetImageROI(img,detectorF->detectedROI[i]);
      
      /** Enregistrement de l'image */
      Display_saveImage(fileName,img);
    }
  
  /** Remise a 0 de la zone d'interet */
  cvResetImageROI(img);
}

void Facerec_setWorkingPath(char workingPath[256]) {
  cl->setWorkingPath(workingPath);
  cout << "Switching to new Facerec workingPath : " << cl->getWorkingPath().c_str() << endl;
}

/** Initialisation du classifieur avec les bases chargees */
void Facerec_init()
{
  cl->init();
}

/** Enregistrement des bases dans la liste de bases */
void Facerec_saveBaseList()
{
  cl->end();
}

/** Fonction qui retourne le premier visage detecte par defaut
    et qui retourne le plus proche du precedent a partir de l'appel suivant */
int Process_getNearestFace(int isFirst)
{
  /** Si il y a eu detection */
  if(!nbImageToFaceDetection && detectorF->nbdetected) 
    {
      /** Si ce n'est pas le premier visage de la base on recherche le plus proche */
      if(!isFirst)
	{
	  int idmin = 0;
	  double dist, distmin;

	  distmin = pow(nearestFacePrecX-detectorF->detectedCENTER[0].x,2) +
	            pow(nearestFacePrecY-detectorF->detectedCENTER[0].y,2);

	  /** Recherche du visage le plus proche du precedent */
	  for(int i=1;i<detectorF->nbdetected;i++)
	    {
	      dist = pow(nearestFacePrecX-detectorF->detectedCENTER[i].x,2) +
	             pow(nearestFacePrecY-detectorF->detectedCENTER[i].y,2);

	      if(dist<distmin)
		{
		  distmin = dist;
		  idmin = i;
		}
	    }

	  /** Si la distance est acceptable on retourne l'indice du visage */
	  if(distmin<nearestFacePrecDistMax)
	    {
	      nearestFacePrecX = detectorF->detectedCENTER[idmin].x;
	      nearestFacePrecY = detectorF->detectedCENTER[idmin].y;
	      nearestFacePrecDistMax = pow(detectorF->detectedROI[idmin].width,2) +
	                               pow(detectorF->detectedROI[idmin].height,2);
	      return idmin;
	    }
	  else
	    {
	      return -1;	
	    }    
	}
      
      /** Sinon c'est le premier ajout dans la base, on prend le premier visage de la liste */
      else
	{
	  nearestFacePrecX = detectorF->detectedCENTER[0].x;
	  nearestFacePrecY = detectorF->detectedCENTER[0].y;
	  nearestFacePrecDistMax = pow(detectorF->detectedROI[0].width,2) +
	                           pow(detectorF->detectedROI[0].height,2);
	  return 0;
	}
    }
  
  /** Sinon on retourne l'indice -1 */
  else 
    {
      return -1; 
    }   
}

/** Ajout d'une personne dans la liste des bases */
int Facerec_addSomebody(IplImage* img, char* name, int isFirst)
{
  /** Recuperation du visage le plus proche du precedent (si il y en a) */
  int nearestFace = Process_getNearestFace(isFirst);

  if(nearestFace == -1) return 0;

  /** Ajout d'une region d'interet dans l'image correspondant au visage */
  cvSetImageROI(img,detectorF->detectedROI[nearestFace]);

  /** Redimentionnement de l'imagette */
  cvResize(img,resizeImage,CV_INTER_LINEAR);

  /** Conversion en niveaux de gris */
  cvCvtColor(resizeImage,greyImage,CV_BGR2GRAY);

  /** Ajout de l'imagette dans la base */
  if(nbToAPP==0) {
    cl->addBase(greyImage,name,1);
    nbToAPP = NB_TO_APP;
  }
  else {
    cl->addBase(greyImage,name,0);
    nbToAPP--;
  }

  /** Reset de la ROI */
  cvResetImageROI(img);

  /** Trace du visage ajoute */
  cvRectangle(img,cvPoint(detectorF->detectedROI[nearestFace].x,detectorF->detectedROI[nearestFace].y),
	      cvPoint(detectorF->detectedROI[nearestFace].x+detectorF->detectedROI[nearestFace].width,
		      detectorF->detectedROI[nearestFace].y+detectorF->detectedROI[nearestFace].height),
	      CV_RGB(0,0,255),3,8,0);

  return 1;
}

/** Selectionne une personne d'interet */
void Facerec_setPeople(char* name)
{
  cl->setWorkingPeople(name);
}

/** Selectionne la reconaissance de toute les personnes */
void Facerec_recognizeAll()
{
  cl->eraseWorkingPeople();
}

/** Reconaissance */
void Facerec_whoIsIt(IplImage* imgIn, int* state, char* name, double* likelihood)
{
  cl->whoIsIt(imgIn,state,name,likelihood);
}

/** Reconaissance avec formatage du resultat dans une chaine de caractere */
void Facerec_whoIsItString(IplImage* imgIn, char* who)
{
  cl->whoIsIt(imgIn,who);
}

void Facerec_isIt(IplImage* imgIn, char* name, char* who)
{
  Facerec_setPeople(name);
  Facerec_whoIsItString(imgIn,who);
}

void Facerec_enable()
{
  needFaceRecognition = 1;
}

void Facerec_disable()
{
  needFaceRecognition = 0;
}

int Facerec_isEnabled()
{
  return needFaceRecognition;
}

/* Ajout d'une personne dans une liste et init des autres parametres */
void Facerec_addToList(ICU_RECOGNITION_WHOLIST *whoList, char* nm)
{
  //printf(" Add to list \n");
  strcpy(whoList->who[whoList->nb].name,nm);
  whoList->who[whoList->nb].present = -1;
  whoList->who[whoList->nb].likelihood = 0;
  for(int k=0;k<ICU_NB_MAX_TEMPORAL_WINDOW;k++)
    {
      whoList->who[whoList->nb].window[k] = 0;
      whoList->who[whoList->nb].reliability[k] = 0;
    }
  whoList->who[whoList->nb].curid = 0;
  whoList->who[whoList->nb].updated = 0;
  whoList->who[whoList->nb].cptAP = 0;
  whoList->who[whoList->nb].cumul = 0;
  whoList->nb++;
}

/* Vidage de la liste */
void Facerec_clearList(ICU_RECOGNITION_WHOLIST *whoList)
{
  int j;
  
  for(j=0;j<whoList->nb;j++)
    {
      strcpy(whoList->who[j].name,"");
      whoList->who[j].present = -1;
      whoList->who[j].likelihood = 0;
      for(int k=0;k<ICU_NB_MAX_TEMPORAL_WINDOW;k++)
	{
	  whoList->who[j].window[k] = 0;
	  whoList->who[j].reliability[k] = 0;
	}
      whoList->who[j].curid = 0;
      whoList->who[j].updated = 0;
      whoList->who[j].cptAP = 0;
      whoList->who[j].cumul = 0;
    }
  whoList->nb = 0;
}

/* Recuperqtion de la liste des personnes connues sur disque */
int Facerec_knownList(ICU_RECOGNITION_WHOLIST *whoList) 
{
  char nm[256];
  int nBase;
  string baseListFileName = cl->getWorkingPath() + "/baseList.txt";   
  FILE* baseList = fopen(baseListFileName.c_str(), "r");
  
  /** Si le fichier n'existe pas on sort */
  if(!baseList) {
    cout << "|WARNING|--> Classifier \t:\t base list file doesn't exist\n";
    return 0;
  }

  fscanf(baseList,"%d\n",&nBase);

  if(!nBase) {
    cout << "|WARNING|--> Classifier \t:\t empty base list\n";
    return 0;
  }
  for(int i=0;i<nBase;i++) {
    int nbObj, nbB;
    fscanf(baseList, "%s %d %d",nm, &nbObj, &nbB);
    printf("Facerec_knownList : %s\n",nm);
    Facerec_addToList(whoList,nm);
  }
  return 1;
}

/* Recuperation de la liste des personnes connues en memoire */
int Facerec_whoLoaded(ICU_RECOGNITION_WHOLIST *whoLoaded) 
{
  /* Ajoute le champs Unknown si c le premier appel */
  if(!whoLoaded->nb)
    Facerec_addToList(whoLoaded,"Unknown");

  for(int i=0;i<(int)cl->base.size();i++) {
    Facerec_addToList(whoLoaded,cl->base[i]->getName());
    printf("Facerec_whoLoaded : %s\n",whoLoaded->who[whoLoaded->nb-1].name);
  }
  return 1;
}

int Facerec_loadSomebody(char *name) 
{
  cl->loadSomebody(name);
  return 1;
}

int Facerec_unloadSomebody(char *name) 
{
  cl->unloadSomebody(name);
  return 1;
}

/* Charge toute les personnes du disque */
int Facerec_loadEverybody()
{
  cl->load();
  return 1;
}

/* Decharge toute les personnes de la memoire */
int Facerec_unloadEverybody() 
{
  /* On vide completement le classifieur */
  delete cl;
  cl = new classifier();
  return 1;
}

int Facerec_getId(ICU_RECOGNITION_WHOLIST *whoList, char name[256], int state) {
  int i;
  //if(state == 2) return 0;
  for(i=0;i<whoList->nb;i++)
    if(strcmp(whoList->who[i].name,name)==0)
      return i;
  return 0;
}


void Facerec_reload(ICU_RECOGNITION_WHOLIST *whoList) {
  /* On vide completement le classifieur */
  delete cl;
  cl = new classifier();
  
  /* On recharge la liste de personnes */
  for(int i=0;i<whoList->nb;i++) {
    if(strcmp(whoList->who[i].name,"Unknown")!=0)
      cl->loadSomebody(whoList->who[i].name);
  }
}



/*Fonction RFID*/
void initializeRFID()
{
	/*CAENRFIDErrorCodes error = init_RFID(1);
	printf("initializeRFID termine\n");
    if(error!=CAENRFID_StatusOK)
		printf("Initialisation raté\n\n\n");*/
	
	
	rfidOn = 1;
		
}

void closeRFID()
{
	/*CAENRFIDErrorCodes error=close_RFID();
    if(error!=CAENRFID_StatusOK)
		printf("Fermeture raté\n\n\n");*/
	
	rfidOn = 0;
}

void acquisitionZone()
{
    //CAENRFIDErrorCodes error;
	int i=0;
	int j;
	externalTag *ordA;
	internalTag *ord;
	acquisition();
        ord = get_Internal_Tag_by_pos(i);

	printf("\n");
        while (ord!=NULL) 
	{
       		ordA=getTagInfo(ord);
       		
                printf("( tag %s :",ord->Tag_ID);
		for (j=0;j<NBANT;j++)
		{
                        printf("%d;",ord->last_detec[j]);

                }// end for j
                printf(") ");

                printf("angle : %lf",ordA->angle);
                 printf(";");
                 printf("ecart type angle : %lf )\n",ordA->ecartTypeAngle);
                 printf("longueur : %lf",ordA->longueur);
                 printf(";");
                 printf("ecart type long : %lf )\n",ordA->ecartTypeLongueur);
                 free(ordA);
                i++;
                ord=get_Internal_Tag_by_pos(i);
        }
}

void acquisitionTag(char *tag)
{
	/*//CAENRFIDErrorCodes error;
	//int j;
	internalTag *ord;
	acquisition();
    ord = get_Internal_Tag_by_Id(tag);
    if(ord == NULL)
    {
    	Tag = NULL;
    }
    else
    {
	    Tag=getTagInfo(ord);
	}*/
}
