#include <cv.h>
#include <highgui.h>

#include "applipath.h"
#include "def_paramslist.h"
#include "MotionDistribution.h"
#include "displayutils.h"
#include "videoimgseq.h"

void prob2fcol(IplImage* imgproba, IplImage* imgfcol)
{
  uchar* ptprob = (uchar*)imgproba->imageData;
  uchar* ptfcol = (uchar*)imgfcol->imageData;
  CvScalar fcol;
  for(int i=0;i<imgproba->imageSize;i++)
    {
      fcol = SCalcFcol(*ptprob++);
      *ptfcol++ = (uchar)fcol.val[0];
      *ptfcol++ = (uchar)fcol.val[1];
      *ptfcol++ = (uchar)fcol.val[2];
    }
}

template<class T>
void Permut(T** pt1, T** pt2)
{
  T* tmp = *pt2;
  *pt2 = *pt1;
  *pt1 = tmp; 
}

int test_MotionColorDistribution()
{
  char nm[50];
  int cpt=0;

  IplImage* buff[2];
  buff[0] = cvCreateImage(cvSize(320,240),IPL_DEPTH_8U,1);
  buff[1] = cvCreateImage(cvSize(320,240),IPL_DEPTH_8U,1);
  IplImage* silh = cvCreateImage(cvSize(320,240),IPL_DEPTH_8U,1);

  /* pointeur sur image courante et precedente */
  IplImage* curimg=buff[0];
  IplImage* previmg=buff[1];

  /* Lecture d'une sequence d'images */
  videoimgseq myvid("/home/lbrethes/1pmobileET1pmobile/","seq01","jpg",524,700,5);
  myvid.init();
  IplImage* imgIn = myvid.next();
  if(!imgIn)
    {
      cout << "Erreur de lecture de l'image\n";
      return 0;
    }

  /* Region de l'image a considerer */
  CvRect region = cvRect(0,0,10,10);
  int nbPix = region.width*region.height;

  /* Distribution du mouvement */
  MotionDistribution* md = new MotionDistribution(8,1,320,240,&nbPix);
  //md->Disp();

  IplImage* imgdist = cvCreateImage(cvSize(320,240),IPL_DEPTH_8U,1);
  IplImage* imgdistview = cvCreateImage(cvSize(320,240),IPL_DEPTH_8U,1);
  IplImage* imgdistviewCol = cvCreateImage(cvSize(320,240),IPL_DEPTH_8U,3);

  char k=0;
  while(k!=27)
    {
      /* Permutation des images */
      Permut<IplImage>(&curimg,&previmg);

      /* Chargement image courante */
      if(!myvid.grab()) return 0;

      /* Calcul de la difference absolue */
      cvCvtColor(myvid.get(), curimg, CV_BGR2GRAY );
      cvAbsDiff( curimg, previmg, silh );

      /* Calcul de la distrubution du mouvement dans une zone */
      md->InitMask();
      md->CalcFromRect(0,silh,region);
      
      /* Calcul de la distance */
      float distance,max=0;

      cvZero(imgdist);
      cvZero(imgdistview);
      cvZero(imgdistviewCol);
     
      int regW = 10;
      int regH = 10;
      int demiregW = regW/2;
      int demiregH = regH/2;
      int pas = 10;
      int demipas = pas/2;
      int imgH = img_height - demiregH;
      int imgW = img_width - demiregW;
      CvPoint pt1,pt2;

      for(int i=demipas;i<img_height;i+=pas)
	{
	  for(int j=demipas;j<img_width;j+=pas)
	    {
	      /* determine la region */	      
	      pt1.x = (j > demiregW) ? (j - demiregW) : 0;
	      pt1.y = (i > demiregH) ? (i - demiregH) : 0;
	      pt2.x = (j < imgW) ? (j + demiregW) : img_width;
	      pt2.y = (i < imgH) ? (i + demiregH) : img_height;

	      /* calcule l'histo */
	      md->CalcFromRect(0,silh,pt1,pt2);
	      
	      /* calcule la distance */
	      distance = md->BhattaDistance();
	      cout << " dist = " << distance << endl; 
	      //distance=1-distance;
	      //if(distance>max) max = distance;

	      double coul = 255.0*(1.0-distance);
	      //if(distance<0.1)
		cvRectangle(imgdistviewCol,pt1,pt2,cvScalar(coul,coul,coul),-1);
	      
	      //cout << i << "  " << j << endl;
		//cvCircle(imgdistviewCol,cvPoint(i,j),2,CV_RGB(255,0,0),-1);

	      
	    }

	}

      //cout << "max = " << max << endl;
      Show("dist",imgdist,0);
      sprintf(nm,"motion_imgdist%03d.jpg",cpt);
      cvSaveImage(nm,imgdist);

      Show("dist 2",imgdistview,0);
      sprintf(nm,"motion_imgdistview%03d.jpg",cpt);
      cvSaveImage(nm,imgdistview);


      //prob2fcol(imgdist,imgdistviewCol);
      Show("dist fcol",imgdistviewCol,0);
      sprintf(nm,"motion_imgdistviewfcol%03d.jpg",cpt++);
      cvSaveImage(nm,imgdistviewCol);

      /* Affichage de la distribution */
      md->Disp("Histo",0);

      Show("cur",curimg,0,0,0);
      Show("prev",previmg,330,0,0);
      cvRectangle(silh,cvPoint(region.x,region.y),cvPoint(region.x+region.width,region.y+region.height),cvScalar(255));
      Show("AbsDiff",silh,660,0,0);

      k=cvWaitKey(50);
    }
  
  
  return 1;
}


int main(void)
{
  test_MotionColorDistribution();
}
