#include "OpticalFlow.h"


OpticalFlow::OpticalFlow(int w, int h, IplImage* imgOLD)
{
  /* recopie des dimensions */
  this->imgSize.width = w;
  this->imgSize.height = h;
  this->width=w;
  this->height=h;

  /* Alloc des images utiles */
  if(!imgOLD)  this->oldimage = cvCreateImage(this->imgSize,IPL_DEPTH_8U,1);
  else this->oldimage = imgOLD;

  this->imgflowmask = cvCreateImage(this->imgSize,IPL_DEPTH_8U,1);
  this->gimgx = cvCreateImage(this->imgSize,IPL_DEPTH_32F,1);
  this->gimgy = cvCreateImage(this->imgSize,IPL_DEPTH_32F,1);

  /* Init de constantes */  
  this->xremainstep=this->gimgx->widthStep-this->gimgx->width*4;
  this->yremainstep=this->gimgy->widthStep-this->gimgy->width*4;
  this->mremainstep=this->imgflowmask->widthStep-this->imgflowmask->width;

  /* Init de certains parametres */
  this->lambda = 0.01;
  this->criteria.type = CV_TERMCRIT_ITER;
  this->criteria.max_iter = 3;
}

OpticalFlow::~OpticalFlow()
{
  cvReleaseImage(&this->oldimage);
  cvReleaseImage(&this->imgflowmask);
  cvReleaseImage(&this->gimgx);
  cvReleaseImage(&this->gimgy);
}

/* methode qui calcule et retourne le masque des pixels en mouvement */
IplImage* OpticalFlow::flowmask()
{
  register int i,j;
  unsigned char  *mask;
  float * fx, * fy;
  fx=(float *)this->gimgx->imageData;
  fy=(float *)this->gimgy->imageData;
  mask=(unsigned char *)this->imgflowmask->imageData;

  for(j=0;j<this->height;j++)
    {
      for(i=0;i<this->width;i++)
	{
	  if((fabs(*fx)>0.5)||(fabs(*fy)>0.5)) *mask=255;
	  else *mask=0;
	  fx++;
	  fy++;
	  mask++;
	}
      fx+=xremainstep;
      fy+=yremainstep;
      mask+=mremainstep;
    }
  return this->imgflowmask;
}

/* methode qui filtre une image et calcule le flot en meme temps */
void OpticalFlow::computeandapplyflowmask(IplImage* imgbw,IplImage* oldimage, IplImage* imgIn, IplImage* imgOut)
{
  register int i,j;
  unsigned char  *mask, *in, *out;
  float * fx, * fy;
  fx=(float *)this->gimgx->imageData;
  fy=(float *)this->gimgy->imageData;
  mask=(unsigned char *)this->imgflowmask->imageData;
  in=(unsigned char *)imgIn->imageData;
  out=(unsigned char *)imgOut->imageData;

  /* calcul du flot */
  computeflow(imgbw,oldimage);
  
  /* filtrage */
  for(j=0;j<this->height;j++)
    {
      for(i=0;i<this->width;i++)
	{
	  if((fabs(*fx)>0.5)||(fabs(*fy)>0.5)) 
	    {
	      *mask=255;
	      *out = *in;
	    }
	  else
	    {
	      *mask=0;
	      *out = 0;
	    }
	  fx++;
	  fy++;
	  mask++;
	  in++;
	  out++;
	}
      fx+=xremainstep;
      fy+=yremainstep;
      mask+=mremainstep;
    }
}
