#include "SIR.h"

/**
   Consruction du filtre SIR
*/
SIRFilter::SIRFilter(string nomfic, ImgProducer* imgproducer, 
		     twoDDetector* detector, Prior* prior, 
		     ContDynamicModel* ContDynamic, DiscDynamicModel** DiscDynamic):ParticleFilter(nomfic,imgproducer,
												   detector,prior,
												   ContDynamic,DiscDynamic)
{
  cout << "--> SIRFilter (Constructor) \t:\t New Sampling Importance Resampling filter (SIR) ....." << endl;
  
  cout << "********************************" << endl;
  cout << "*    LOADING NEW SIR FILTER    *" << endl;
  cout << "********************************" << endl;
  
  /* On colle le nom du filtre */
  this->nmfiltre = "SIR";

  cout << "--> SIRFilter (Constructor) \t:\t New SIR filter.....OK construction done" << endl;
  
  cout << "*******************************" << endl;
  cout << "*      SIR FILTER LOADED      *" << endl;
  cout << "*******************************" << endl;
}

/**
   Destructeur du filtre SIR
*/
SIRFilter::~SIRFilter()
{

}

/*! \fn int step()
 *  \brief This is the main processing step. This method correspond to a step of the
 *  algorithm. Calling step for each image achieve a SIR filtering. To run rigth the image measure
 *  building method must have been called before
 */
int SIRFilter::step(IplImage* obs)
{
  int i;

  /* Init des poids */
  this->curw->ResetVector();

  /* Connexion des images de traitement dans la cible pour limiter les appels de fonctions dans la boucle */
  this->target->LinkImages();

  /* Filtrage */  
  for( i=0 ; i < this->nbSamples ; i++ )
    {
      /* Prediction des parametres discrets des particules */
      for(int j=0;j<this->nDP;j++)
	this->target->DiscDynamic[j]->DiscDrawFromDyn(this->sps[i],this->sps[i]);
      
      /* Prediction des parametres continus des particules */
      this->target->ContDynamic->ContDrawFromDynWithNoise(this->sps[i],this->sps[i]);

      /* mesure des particules */
      this->curw->setW(i,this->target->CalcLikelihood(this->sps[i]));
    }

  /* Normalisation des poids */
  if(!this->curw->NormVector()) {cout << "Weights NULL" << endl; return 0;}
  //this->curw->NormVector();
  /* Selection de la particule moyenne */
  this->Select();

  /* N efficace */
  //cout << "Neff = " << this->CalcNeff() << " %" << endl << endl;

  /* resampling des particules */
  this->Resample();

  return 1;
}
