/***************************************************************************
                          video4linux.cpp  -  description
 	$Id: video4linux.cpp,v 1.1 2007/06/19 14:40:40 tgerma Exp $	
                             -------------------
    begin                : Mon May 13 2002
    copyright            : (C) 2002 by Paulo Menezes
    email                : pm@deec.uc.pt
 ***************************************************************************/


/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
#ifdef linux
#include "video4linux.h"
//#include "pwc-ioctl.h"
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <sys/ioctl.h>
#include <stdlib.h>
#include <stdio.h>
#include <unistd.h>
#include <sys/mman.h>
#include <errno.h>
//#define RGBINVERT

video4linux::video4linux(int w,int h, bool optimus)
{
  // here we must go out and count cameras
  // get properties   etc
  // so for each camera we do
  width=w;
  height=h;
  image=NULL;
  optimal=optimus;
  
  camfd=open("/dev/video0",O_RDWR);
  if (camfd<0)
    {
      perror("V4L open");
      abort();
    }

  if (ioctl(camfd,VIDIOCGCAP,&vcap)==-1)
    {
      perror("CGCAP");
      abort();
    }

  if (vcap.type & VID_TYPE_CAPTURE)
    {
      printf("can capture to memory\n");
    }

  if (vcap.type & VID_TYPE_OVERLAY)
    {
      printf("can overlay image in framebuffer\n");
    }
  
  if (vcap.type & VID_TYPE_FRAMERAM)
    {
      printf("overlay overwrites frame buffer memory\n");
    }

  if (vcap.type & VID_TYPE_SUBCAPTURE)
    {
      printf("can capture a subimage\n");
    }
  
  //#define VIDIOCPWCSAGC           _IOW('v', 200, int)	
  /* ganho automtico */
  //  int val = -1;
  //   ioctl(camfd, VIDIOCPWCSAGC, &val);
  //   ioctl(camfd,  VIDIOCPWCSSHUTTER,&agc); // automatic shutter
  //   struct pwc_whitebalance wb; // white balance
  //   wb.mode=PWC_WB_AUTO;
  //   ioctl(camfd,VIDIOCPWCSAWB,&wb);
  
  //  ioctl(camfd,VIDIOCPWCFACTORY,&val); // restore factory defaults

  //  struct pwc_leds LED;
  /* Blinking LED, 1 second interval, 20% duty cycle */
  // LED.led_on = 200;
  //  LED.led_off = 800;
  //  ioctl(camfd, VIDIOCPWCSLED, &LED);
  
  for (int i=0;i<vcap.channels;i++)
    {
      Channel.channel=i;
      if ((ioctl(camfd, VIDIOCGCHAN, &Channel) != -1))
	{
	
	  Channel.norm = 1;
	  Channel.channel = i;
	
	  if ((ioctl(camfd,VIDIOCSCHAN, &Channel) != -1)&&
	      (ioctl(camfd, VIDIOCGCAP, &vcap)    != -1))
	    {
	      if (Channel.type&VIDEO_TYPE_CAMERA)
		{
		  printf("Camera: %s, channel %s\n",vcap.name,Channel.name);
		}    
	    }			
	}
    }
  
#ifdef RGBINVERT
  rgbimg=cvCreateImage(cvSize(width,height),8,3);
#endif
  printf("Contructor done\n");

}

video4linux::~video4linux()
{

  if ((image)&&(image->imageData))
    cvReleaseImage(&image);
  if ((yuvimg)&&(yuvimg->imageData))
    cvReleaseImage(&yuvimg);
  for (curbuf=0;curbuf<maxbufs;curbuf++)
    if (ioctl(camfd,VIDIOCSYNC,&curbuf)==-1)
      {
	// perror("SYNC");
      }
  
  //cvReleaseImage(&rgbimg);
  munmap(memorymap,mbuf.size);
  close(camfd);
  printf("video closed\n");
}



/** No descriptions */
bool video4linux::doOpen(int inputnr)
{
  
  CvSize size;
  Channel.channel=inputnr;
  
  Channel.norm =  VIDEO_MODE_PAL;
  if(ioctl(camfd, VIDIOCSCHAN, &Channel) < 0)
    {
      perror("VIDIOCSCHAN");
      //    abort();
    }
  
  if (ioctl(camfd,VIDIOCGPICT,&picture)==-1)
    perror("GPICT");
  printf("Palete=%d\n",  picture.palette);
  picture.palette=VIDEO_PALETTE_YUV420P;
  picture.depth=24;
  //   picture.brightness=30000;
  //   picture.hue=30000;
  //   picture.colour=30000;
  //   picture.contrast=30000;
  //   picture.whiteness=30000;
  picture.palette=VIDEO_PALETTE_RGB24;

  if (ioctl(camfd,VIDIOCSPICT,&picture)==-1)
    {
      perror("SPICT");
      picture.palette=VIDEO_PALETTE_YUV420P;
      if (ioctl(camfd,VIDIOCSPICT,&picture)==-1)
	{
	  perror("SPICT2");
	  abort();
	}
    }
  
  vmmap.frame=0;
  vmmap.width=width;
  vmmap.height=height;
  vmmap.format=picture.palette;
  vmmap.format= picture.palette;
  size.width=width;
  size.height=height;
  
  
  
  if(ioctl(camfd, VIDIOCGMBUF, &mbuf) == -1)
    {
      perror("mbuf");
      abort();
    }

  if (optimal)
    {
      maxbufs=mbuf.frames;
    }
  else
    {
      maxbufs=1;
    }
  
  printf("Number of frames %d\n",mbuf.frames);
  memorymap = (char*)mmap(0,
			 mbuf.size,
			 PROT_READ|PROT_WRITE, MAP_SHARED,
			 camfd,
			 0);
 
  vmmap.frame=0;

  for (curbuf=0;curbuf<maxbufs;curbuf++)
    if (ioctl(camfd,VIDIOCSYNC,&curbuf)==-1)
      {
	//	 perror("SYNC");
      }
  

  curbuf=0;
  if(ioctl(camfd, VIDIOCMCAPTURE, &vmmap) == -1)
    {	  
      printf("CMCAP FAILLED\n");
    }
  

  image =cvCreateImageHeader(size,IPL_DEPTH_8U,3);
  width=size.width;
  height=size.height;
  yuvimg=cvCreateImage(cvSize(image->width,image->height),IPL_DEPTH_8U,3);
  rgbimg=cvCreateImage(cvSize(image->width,image->height),IPL_DEPTH_8U,3);

  return true;
}


/** No descriptions */
IplImage * video4linux::getImage()
{
  int err;
  vmmap.frame=(curbuf+1)%maxbufs;
  
  while(err=ioctl(camfd,VIDIOCMCAPTURE,&vmmap))
    {
      switch(errno)
	{
	case EINTR:
	  break;
	default:
	  perror("CAPTURE");
	  abort();
	}
    }
  
  while(err=ioctl(camfd,VIDIOCSYNC,&curbuf))
    {
      switch(errno){
      case EINTR:
	break;
      default:
	perror("SYNC");
	sleep(1);
	//abort();
      }
    }
  
  
  
  if ( vmmap.format==VIDEO_PALETTE_RGB24)
    {
#ifdef RGBINVERT
      int nbytes;
      char * aux;
      char * dest;
      nbytes=width*height*3-3;
      aux=(char *)(memorymap+mbuf.offsets[curbuf]+nbytes);
      for (int i=0;i<height;i++)
	{
	  dest=rgbimg->imageData+rgbimg->widthStep*i;
	  for (int j=0;j<width;j++){
	    *dest++ = *aux++;
	    *dest++ = *aux++;
	    *dest++ = *aux++;
	    aux-=6;
	  }
	
	  cvSetImageData(image,rgbimg->imageData,rgbimg->widthStep);
	}
#else
      cvSetImageData(image,(char *)(memorymap+mbuf.offsets[curbuf]),image->widthStep);
  
#endif
    }
  else
    SetRGBfromYUV420(image,(void *)(memorymap+mbuf.offsets[curbuf]));
  
  curbuf=(curbuf+1)%maxbufs;
  //printf("end getImage\n");
  return image;
  
}

#define ONRACKHAM
#ifndef ONRACKHAM /* Version qui marche avec webcam */
int video4linux::SetRGBfromYUV420(IplImage * image, void*memorymap)
{
  unsigned char * rgbdata,*srcdata;
  
  if (image ==NULL){
    printf("image ==NULL\n");
    return -1;
  }
  if (memorymap == NULL){
    printf("mmap==NULL");
    return -1;
  }

 //Conversion 
 rgbdata=(unsigned char*)this->rgbimg->imageData;
 srcdata=(unsigned char *) memorymap;
 YUV2BGR420(srcdata,rgbdata,width,height);

 //Recopie dans l'image
 cvSetImageData(this->image,this->rgbimg->imageData,this->rgbimg->widthStep);
 return 0; /* ok */
}


#else /* Version sur rackham (semble y avoir un pb de conversion notamment avec le rouge */

#define SAT(c)  if (c & (~255)) { if (c < 0) c = 0; else c = 255; }

int video4linux::SetRGBfromYUV420(IplImage * image, void*memorymap)
{
  unsigned char * rgbdata,*yp1,*yp2,*up,*vp,*l1,*l2;
  int i,j;
 
  int r,g,b,cr,cg,cb,y;
 
  if (image ==NULL)
    {
      printf("image ==NULL\n");
      return -1;
    }
  if (memorymap == NULL)
    {
      printf("mmap==NULL\n");
      return -1;
    }


  rgbdata=(unsigned char*)rgbimg->imageData;
  yp1=(unsigned char *) memorymap;
  yp2=yp1+image->width;
  up=yp1+image->width*image->height;
  vp=up+(image->width*image->height)/4;
  l1=rgbdata;
  l2=rgbdata+image->widthStep;

  for (j=image->height/2;j;j--)
    {
      for (i=image->width/2;i;i--)
	{
	  cb=((*up-128)*454)>>8;
	  cr=((*vp-128)*359)>>8;
	  cg=((*vp-128)*183+(*up-128)*88)>>8;
	  
	  y=*(yp1++);
	  r=y+cr;
	  g=y+cg;
	  b=y+cb;
	  SAT(r);
	  SAT(g);
	  SAT(b);
	  *(l1++)=b;
	  *(l1++)=g;
	  *(l1++)=r;
	  
	  y=*(yp1++);
	  r=y+cr;
	  g=y+cg;
	  b=y+cb;
	  SAT(r);
	  SAT(g);
	  SAT(b);
	  *(l1++)=b;
	  *(l1++)=g;
	  *(l1++)=r;
	  
	  y=*(yp2++);
	  r=y+cr;
	  g=y+cg;
	  b=y+cb;
	  SAT(r);
	  SAT(g);
	  SAT(b);
	  *(l2++)=b;
	  *(l2++)=g;
	  *(l2++)=r;
	  
	  y=*(yp2++);
	  r=y+cr;
	  g=y+cg;
	  b=y+cb;
	  SAT(r);
	  SAT(g);
	  SAT(b);
	  *(l2++)=b;
	  *(l2++)=g;
	  *(l2++)=r;
	  up++;
	  vp++;
	}
      l1=l2;
      l2+=image->widthStep;
      yp1=yp2;
      yp2+=image->width;
    }
  cvSetImageData(image,rgbimg->imageData,rgbimg->widthStep);
  return 0; /* ok */
}
#endif


#endif
