
/* 
 * Copyright (c) 1993-2005 LAAS/CNRS
 * All rights reserved.
 *
 * Redistribution and use  in source  and binary  forms,  with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *   1. Redistributions of  source  code must retain the  above copyright
 *      notice, this list of conditions and the following disclaimer.
 *   2. Redistributions in binary form must reproduce the above copyright
 *      notice,  this list of  conditions and the following disclaimer in
 *      the  documentation  and/or  other   materials provided  with  the
 *      distribution.
 *
 * THIS  SOFTWARE IS PROVIDED BY  THE  COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND  ANY  EXPRESS OR IMPLIED  WARRANTIES,  INCLUDING,  BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES  OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR  PURPOSE ARE DISCLAIMED. IN  NO EVENT SHALL THE COPYRIGHT
 * HOLDERS OR      CONTRIBUTORS  BE LIABLE FOR   ANY    DIRECT, INDIRECT,
 * INCIDENTAL,  SPECIAL,  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF  SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN  CONTRACT, STRICT LIABILITY, OR
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE   OF THIS SOFTWARE, EVEN   IF ADVISED OF   THE POSSIBILITY OF SUCH
 * DAMAGE.
 */

/**
 ** This file was automagically generated by GenoM.
 ** Manual editing not recommended. Changes will be overridden.
 **/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <tcl.h>

#include <portLib.h>
#include <errnoLib.h>
#include <h2errorLib.h>

#include <genom/tclServ.h>
#include <genom/tcl_cs.h>

#include "icuMsgLib.h"
#include "icuPosterLib.h"

int $Module$_Init(Tcl_Interp *);

struct ModuleInfo {
   char *name;
   CLIENT_ID cid;
};

static int icuInstall(Tcl_Interp *interp,
			   struct ModuleInfo *, int install);
static int icuClientInitCb(ClientData clientData, Tcl_Interp *interp,
				int objc, Tcl_Obj *const objv[]);
static int icuClientEndCb(ClientData clientData, Tcl_Interp *interp,
			       int objc, Tcl_Obj *const objv[]);


#define TEST_BAD_USAGE(b)					\
{								\
   if (b) {							\
      Tcl_SetResult(interp, "wrong # args", TCL_STATIC);	\
      return TCL_ERROR;						\
   }								\
}

/*
 * ----------------------------------------------------------------------
 * icu connection / disconnection
 */

static int
icuClientInitCb(ClientData clientData, Tcl_Interp *interp,
		     int objc, Tcl_Obj *const objv[])
{
   char *mboxName;
   struct ModuleInfo *m;
   STATUS status;
   char strerr[64];

   icuRecordH2errMsgs();

   m = malloc(sizeof(struct ModuleInfo));
   if (m == NULL) {
      Tcl_SetResult(interp, "Not enough memory", TCL_STATIC);
      return TCL_ERROR;
   }

   if (objc > 1) {
      mboxName = Tcl_GetStringFromObj(objv[1], NULL);
      if (mboxName == NULL) return TCL_ERROR;
   } else {
      mboxName = ICU_MBOX_NAME;
   }

   m->name = malloc(strlen(mboxName)+1);
   if (m->name == NULL) {
      free(m);
      Tcl_SetResult(interp, "Not enough memory", TCL_STATIC);
      return TCL_ERROR;
   }
   strcpy(m->name, mboxName);

   status = csClientInit(mboxName, ICU_MAX_RQST_SIZE,
			 ICU_MAX_INTERMED_REPLY_SIZE, 
			 ICU_MAX_REPLY_SIZE, 
			 &m->cid);

   if (status == ERROR) {
      Tcl_SetResult(interp, h2getErrMsg(errnoGet(), strerr, 64), TCL_VOLATILE);
      Tcl_AppendResult(interp, ": \"", mboxName, "\"", NULL);
      return TCL_ERROR;
   }

   Tcl_SetResult(interp, "connected", TCL_STATIC);
   return icuInstall(interp, m, 1);
}

static int
icuClientEndCb(ClientData clientData, Tcl_Interp *interp,
		    int objc, Tcl_Obj *const objv[])
{
   struct ModuleInfo *m = (struct ModuleInfo *)clientData;
   char strerr[64];

   if (csClientEnd(m->cid) == ERROR) {
      Tcl_SetResult(interp, h2getErrMsg(errnoGet(), strerr, 64), TCL_VOLATILE);
      return TCL_ERROR;
   }

   icuInstall(interp, m, 0);
   free(m->name);
   free(m);

   Tcl_SetResult(interp, "disconnected", TCL_STATIC);
   return TCL_OK;
}


/*
 * ----------------------------------------------------------------------
 * icu abort request
 */

static int icuAbortRqstSendCb(ClientData, Tcl_Interp *, int, Tcl_Obj *const []); 
static int icuAbortReplyRcvCb(ClientData, Tcl_Interp *, int, Tcl_Obj * const []);

static int
icuAbortRqstSendCb(ClientData data, Tcl_Interp *interp,
			int objc, Tcl_Obj *const objv[])
{
   struct ModuleInfo *m = (struct ModuleInfo *)data;
   int rqstId, bilan;
   char buf[10];
   int activity;	/* input */
   char strerr[64];

   TEST_BAD_USAGE(objc != 2);
		 
   if (Tcl_GetIntFromObj(interp, objv[1], &activity) != TCL_OK)
      return TCL_ERROR;

   if (tclServRqstSend(interp, Tcl_GetStringFromObj(objv[0], NULL),
		       m->cid, ICU_ABORT_RQST, 
		       &activity, sizeof(int),
		       &rqstId, &bilan) == ERROR) {

      Tcl_SetResult(interp, h2getErrMsg(bilan, strerr, 64), TCL_VOLATILE);
      return TCL_ERROR;
   }
    
   sprintf(buf, " %d", rqstId);
   Tcl_SetResult(interp, "SENT", TCL_STATIC);
   Tcl_AppendResult(interp, buf, NULL);
   return TCL_OK;
}

static int
icuAbortReplyRcvCb(ClientData clientData, Tcl_Interp *interp,
			int objc, Tcl_Obj * const objv[])
{
   struct ModuleInfo *m = (struct ModuleInfo *)clientData;
   int ret;
   int rqstId, bilan, activity, status;
   Tcl_Obj *result;
   char strerr[64];

   TEST_BAD_USAGE(
      objc != 2 ||
      Tcl_GetIntFromObj(interp, objv[1], &rqstId) != TCL_OK
   );

   status = tclServReplyRcv(interp, m->cid, rqstId, NO_BLOCK,
			    &activity, NULL, 0, &bilan);

   switch (status) {
      case WAITING_INTERMED_REPLY:
	 Tcl_SetResult(interp, "SENT", TCL_STATIC);
	 return TCL_OK;

      case WAITING_FINAL_REPLY: {
	 char tmp[50];
	 sprintf(tmp, "ACK %d", activity);
	 Tcl_SetResult(interp, tmp, TCL_VOLATILE);
	 return TCL_OK;
      }

      case FINAL_REPLY_OK:
	 break;

      default:
	 Tcl_SetResult(interp, h2getErrMsg(bilan, strerr, 64), TCL_VOLATILE);
	 return TCL_ERROR;
   }

   result = Tcl_NewListObj(0, NULL);
   if (result == NULL) return TCL_ERROR;

   ret = Tcl_ListObjAppendElement(interp, result,
				  Tcl_NewStringObj("TERM", -1));
   if (ret != TCL_OK) return TCL_ERROR;

   if (bilan == OK)
      ret = Tcl_ListObjAppendElement(interp, result,
				     Tcl_NewStringObj("OK", -1));
   else
      ret = Tcl_ListObjAppendElement(interp, result,
				     Tcl_NewStringObj(
					h2getErrMsg(bilan, strerr, 64), -1));
   if (ret != TCL_OK) return TCL_ERROR;

   Tcl_SetObjResult(interp, result);
   return TCL_OK;
}

/* --- FILE GENERATED WITH GENOM, DO NOT EDIT BY HAND ------------------ */

/* 
 * Copyright (c) 1993-2005 LAAS/CNRS
 * All rights reserved.
 *
 * Redistribution and use  in source  and binary  forms,  with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *   1. Redistributions of  source  code must retain the  above copyright
 *      notice, this list of conditions and the following disclaimer.
 *   2. Redistributions in binary form must reproduce the above copyright
 *      notice,  this list of  conditions and the following disclaimer in
 *      the  documentation  and/or  other   materials provided  with  the
 *      distribution.
 *
 * THIS  SOFTWARE IS PROVIDED BY  THE  COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND  ANY  EXPRESS OR IMPLIED  WARRANTIES,  INCLUDING,  BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES  OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR  PURPOSE ARE DISCLAIMED. IN  NO EVENT SHALL THE COPYRIGHT
 * HOLDERS OR      CONTRIBUTORS  BE LIABLE FOR   ANY    DIRECT, INDIRECT,
 * INCIDENTAL,  SPECIAL,  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF  SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN  CONTRACT, STRICT LIABILITY, OR
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE   OF THIS SOFTWARE, EVEN   IF ADVISED OF   THE POSSIBILITY OF SUCH
 * DAMAGE.
 */

/*
 * ----------------------------------------------------------------------
 * InitDisplay Request
 */

static int icuInitDisplayRqstSendCb(ClientData, Tcl_Interp *, int, Tcl_Obj *const []); 
static int icuInitDisplayReplyRcvCb(ClientData, Tcl_Interp *, int, Tcl_Obj * const []);

static int
icuInitDisplayRqstSendCb(ClientData data, Tcl_Interp *interp,
				int objc, Tcl_Obj *const objv[])
{
   struct ModuleInfo *tclModuleInfo = (struct ModuleInfo *)data;
   int rqstId, bilan;
   char buf[10];
   static ICU_AFFICHAGE display;	/* input */
#if (6 > 0)
   int ret;
   int curObjc = 0;
#endif
   char strerr[64];

   TEST_BAD_USAGE(objc != 6+1);
		 
   ret = Tcl_GetIntFromObj(interp, objv[++curObjc],    	&display.display);
   if (ret != TCL_OK) return TCL_ERROR;
   ret = Tcl_GetIntFromObj(interp, objv[++curObjc],    	&display.width);
   if (ret != TCL_OK) return TCL_ERROR;
   ret = Tcl_GetIntFromObj(interp, objv[++curObjc],    	&display.height);
   if (ret != TCL_OK) return TCL_ERROR;
   ret = Tcl_GetIntFromObj(interp, objv[++curObjc],    	&display.posx);
   if (ret != TCL_OK) return TCL_ERROR;
   ret = Tcl_GetIntFromObj(interp, objv[++curObjc],    	&display.posy);
   if (ret != TCL_OK) return TCL_ERROR;
   ret = Tcl_GetIntFromObj(interp, objv[++curObjc],    	&display.unused);
   if (ret != TCL_OK) return TCL_ERROR;


   if (tclServRqstSend(interp, Tcl_GetStringFromObj(objv[0], NULL),
		       tclModuleInfo->cid, ICU_INITDISPLAY_RQST,
		       &display, sizeof(display),
		       &rqstId, &bilan) == ERROR) {

     Tcl_SetResult(interp, h2getErrMsg(bilan, strerr, 64), TCL_VOLATILE);
      return TCL_ERROR;
   }
    
   sprintf(buf, " %d", rqstId);
   Tcl_SetResult(interp, "SENT", TCL_STATIC);
   Tcl_AppendResult(interp, buf, NULL);
   return TCL_OK;
}

static int
icuInitDisplayReplyRcvCb(ClientData data, Tcl_Interp *interp,
				int objc, Tcl_Obj * const objv[])
{
   struct ModuleInfo *tclModuleInfo = (struct ModuleInfo *)data;
   int ret;
   int rqstId, bilan, activity;
   /* no output */ 	/* output */
   Tcl_Obj *my_own_private_unique_result;
   char strerr[64];

   TEST_BAD_USAGE(
      objc != 2 ||
      Tcl_GetIntFromObj(interp, objv[1], &rqstId) != TCL_OK
   );

   switch (
      /* c'est laid */
      tclServReplyRcv(interp,
		      tclModuleInfo->cid, rqstId, NO_BLOCK,
		      &activity, NULL, 0,
		      &bilan)
      ) {
      case WAITING_INTERMED_REPLY:
	 Tcl_SetResult(interp, "SENT", TCL_STATIC);
	 return TCL_OK;

      case WAITING_FINAL_REPLY: {
	 char tmp[50];
	 sprintf(tmp, "ACK %d", activity);
	 Tcl_SetResult(interp, tmp, TCL_VOLATILE);
	 return TCL_OK;
      }

      case FINAL_REPLY_OK:
	 break;

      default:
	 Tcl_SetResult(interp, h2getErrMsg(bilan, strerr, 64), TCL_VOLATILE);
	 return TCL_ERROR;
   }

   my_own_private_unique_result = Tcl_NewListObj(0, NULL);
   if (my_own_private_unique_result == NULL) return TCL_ERROR;

   ret = Tcl_ListObjAppendElement(interp, my_own_private_unique_result,
				  Tcl_NewStringObj("TERM", -1));
   if (ret != TCL_OK) return TCL_ERROR;

   if (bilan == OK)
      ret = Tcl_ListObjAppendElement(interp, my_own_private_unique_result,
				     Tcl_NewStringObj("OK", -1));
   else
      ret = Tcl_ListObjAppendElement(interp, my_own_private_unique_result,
				     Tcl_NewStringObj(
					h2getErrMsg(bilan, strerr, 64), -1));
   if (ret != TCL_OK) return TCL_ERROR;



   Tcl_SetObjResult(interp, my_own_private_unique_result);
   return TCL_OK;
}

/* --- FILE GENERATED WITH GENOM, DO NOT EDIT BY HAND ------------------ */

/* 
 * Copyright (c) 1993-2005 LAAS/CNRS
 * All rights reserved.
 *
 * Redistribution and use  in source  and binary  forms,  with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *   1. Redistributions of  source  code must retain the  above copyright
 *      notice, this list of conditions and the following disclaimer.
 *   2. Redistributions in binary form must reproduce the above copyright
 *      notice,  this list of  conditions and the following disclaimer in
 *      the  documentation  and/or  other   materials provided  with  the
 *      distribution.
 *
 * THIS  SOFTWARE IS PROVIDED BY  THE  COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND  ANY  EXPRESS OR IMPLIED  WARRANTIES,  INCLUDING,  BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES  OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR  PURPOSE ARE DISCLAIMED. IN  NO EVENT SHALL THE COPYRIGHT
 * HOLDERS OR      CONTRIBUTORS  BE LIABLE FOR   ANY    DIRECT, INDIRECT,
 * INCIDENTAL,  SPECIAL,  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF  SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN  CONTRACT, STRICT LIABILITY, OR
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE   OF THIS SOFTWARE, EVEN   IF ADVISED OF   THE POSSIBILITY OF SUCH
 * DAMAGE.
 */

/*
 * ----------------------------------------------------------------------
 * PomTagging Request
 */

static int icuPomTaggingRqstSendCb(ClientData, Tcl_Interp *, int, Tcl_Obj *const []); 
static int icuPomTaggingReplyRcvCb(ClientData, Tcl_Interp *, int, Tcl_Obj * const []);

static int
icuPomTaggingRqstSendCb(ClientData data, Tcl_Interp *interp,
				int objc, Tcl_Obj *const objv[])
{
   struct ModuleInfo *tclModuleInfo = (struct ModuleInfo *)data;
   int rqstId, bilan;
   char buf[10];
   static int tag_on_off;	/* input */
#if (1 > 0)
   int ret;
   int curObjc = 0;
#endif
   char strerr[64];

   TEST_BAD_USAGE(objc != 1+1);
		 
   ret = Tcl_GetIntFromObj(interp, objv[++curObjc],    	&tag_on_off);
   if (ret != TCL_OK) return TCL_ERROR;


   if (tclServRqstSend(interp, Tcl_GetStringFromObj(objv[0], NULL),
		       tclModuleInfo->cid, ICU_POMTAGGING_RQST,
		       &tag_on_off, sizeof(tag_on_off),
		       &rqstId, &bilan) == ERROR) {

     Tcl_SetResult(interp, h2getErrMsg(bilan, strerr, 64), TCL_VOLATILE);
      return TCL_ERROR;
   }
    
   sprintf(buf, " %d", rqstId);
   Tcl_SetResult(interp, "SENT", TCL_STATIC);
   Tcl_AppendResult(interp, buf, NULL);
   return TCL_OK;
}

static int
icuPomTaggingReplyRcvCb(ClientData data, Tcl_Interp *interp,
				int objc, Tcl_Obj * const objv[])
{
   struct ModuleInfo *tclModuleInfo = (struct ModuleInfo *)data;
   int ret;
   int rqstId, bilan, activity;
   /* no output */ 	/* output */
   Tcl_Obj *my_own_private_unique_result;
   char strerr[64];

   TEST_BAD_USAGE(
      objc != 2 ||
      Tcl_GetIntFromObj(interp, objv[1], &rqstId) != TCL_OK
   );

   switch (
      /* c'est laid */
      tclServReplyRcv(interp,
		      tclModuleInfo->cid, rqstId, NO_BLOCK,
		      &activity, NULL, 0,
		      &bilan)
      ) {
      case WAITING_INTERMED_REPLY:
	 Tcl_SetResult(interp, "SENT", TCL_STATIC);
	 return TCL_OK;

      case WAITING_FINAL_REPLY: {
	 char tmp[50];
	 sprintf(tmp, "ACK %d", activity);
	 Tcl_SetResult(interp, tmp, TCL_VOLATILE);
	 return TCL_OK;
      }

      case FINAL_REPLY_OK:
	 break;

      default:
	 Tcl_SetResult(interp, h2getErrMsg(bilan, strerr, 64), TCL_VOLATILE);
	 return TCL_ERROR;
   }

   my_own_private_unique_result = Tcl_NewListObj(0, NULL);
   if (my_own_private_unique_result == NULL) return TCL_ERROR;

   ret = Tcl_ListObjAppendElement(interp, my_own_private_unique_result,
				  Tcl_NewStringObj("TERM", -1));
   if (ret != TCL_OK) return TCL_ERROR;

   if (bilan == OK)
      ret = Tcl_ListObjAppendElement(interp, my_own_private_unique_result,
				     Tcl_NewStringObj("OK", -1));
   else
      ret = Tcl_ListObjAppendElement(interp, my_own_private_unique_result,
				     Tcl_NewStringObj(
					h2getErrMsg(bilan, strerr, 64), -1));
   if (ret != TCL_OK) return TCL_ERROR;



   Tcl_SetObjResult(interp, my_own_private_unique_result);
   return TCL_OK;
}

/* --- FILE GENERATED WITH GENOM, DO NOT EDIT BY HAND ------------------ */

/* 
 * Copyright (c) 1993-2005 LAAS/CNRS
 * All rights reserved.
 *
 * Redistribution and use  in source  and binary  forms,  with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *   1. Redistributions of  source  code must retain the  above copyright
 *      notice, this list of conditions and the following disclaimer.
 *   2. Redistributions in binary form must reproduce the above copyright
 *      notice,  this list of  conditions and the following disclaimer in
 *      the  documentation  and/or  other   materials provided  with  the
 *      distribution.
 *
 * THIS  SOFTWARE IS PROVIDED BY  THE  COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND  ANY  EXPRESS OR IMPLIED  WARRANTIES,  INCLUDING,  BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES  OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR  PURPOSE ARE DISCLAIMED. IN  NO EVENT SHALL THE COPYRIGHT
 * HOLDERS OR      CONTRIBUTORS  BE LIABLE FOR   ANY    DIRECT, INDIRECT,
 * INCIDENTAL,  SPECIAL,  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF  SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN  CONTRACT, STRICT LIABILITY, OR
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE   OF THIS SOFTWARE, EVEN   IF ADVISED OF   THE POSSIBILITY OF SUCH
 * DAMAGE.
 */

/*
 * ----------------------------------------------------------------------
 * AutoMode Request
 */

static int icuAutoModeRqstSendCb(ClientData, Tcl_Interp *, int, Tcl_Obj *const []); 
static int icuAutoModeReplyRcvCb(ClientData, Tcl_Interp *, int, Tcl_Obj * const []);

static int
icuAutoModeRqstSendCb(ClientData data, Tcl_Interp *interp,
				int objc, Tcl_Obj *const objv[])
{
   struct ModuleInfo *tclModuleInfo = (struct ModuleInfo *)data;
   int rqstId, bilan;
   char buf[10];
   static ICU_AUTOSWITCH_CONTROL autoSwitchControl;	/* input */
#if (2 > 0)
   int ret;
   int curObjc = 0;
#endif
   char strerr[64];

   TEST_BAD_USAGE(objc != 2+1);
		 
   ret = Tcl_GetIntFromObj(interp, objv[++curObjc],    	&autoSwitchControl.autoSwitchMode);
   if (ret != TCL_OK) return TCL_ERROR;
   ret = Tcl_GetIntFromObj(interp, objv[++curObjc],    	&autoSwitchControl.isRobotMoving);
   if (ret != TCL_OK) return TCL_ERROR;


   if (tclServRqstSend(interp, Tcl_GetStringFromObj(objv[0], NULL),
		       tclModuleInfo->cid, ICU_AUTOMODE_RQST,
		       &autoSwitchControl, sizeof(autoSwitchControl),
		       &rqstId, &bilan) == ERROR) {

     Tcl_SetResult(interp, h2getErrMsg(bilan, strerr, 64), TCL_VOLATILE);
      return TCL_ERROR;
   }
    
   sprintf(buf, " %d", rqstId);
   Tcl_SetResult(interp, "SENT", TCL_STATIC);
   Tcl_AppendResult(interp, buf, NULL);
   return TCL_OK;
}

static int
icuAutoModeReplyRcvCb(ClientData data, Tcl_Interp *interp,
				int objc, Tcl_Obj * const objv[])
{
   struct ModuleInfo *tclModuleInfo = (struct ModuleInfo *)data;
   int ret;
   int rqstId, bilan, activity;
   /* no output */ 	/* output */
   Tcl_Obj *my_own_private_unique_result;
   char strerr[64];

   TEST_BAD_USAGE(
      objc != 2 ||
      Tcl_GetIntFromObj(interp, objv[1], &rqstId) != TCL_OK
   );

   switch (
      /* c'est laid */
      tclServReplyRcv(interp,
		      tclModuleInfo->cid, rqstId, NO_BLOCK,
		      &activity, NULL, 0,
		      &bilan)
      ) {
      case WAITING_INTERMED_REPLY:
	 Tcl_SetResult(interp, "SENT", TCL_STATIC);
	 return TCL_OK;

      case WAITING_FINAL_REPLY: {
	 char tmp[50];
	 sprintf(tmp, "ACK %d", activity);
	 Tcl_SetResult(interp, tmp, TCL_VOLATILE);
	 return TCL_OK;
      }

      case FINAL_REPLY_OK:
	 break;

      default:
	 Tcl_SetResult(interp, h2getErrMsg(bilan, strerr, 64), TCL_VOLATILE);
	 return TCL_ERROR;
   }

   my_own_private_unique_result = Tcl_NewListObj(0, NULL);
   if (my_own_private_unique_result == NULL) return TCL_ERROR;

   ret = Tcl_ListObjAppendElement(interp, my_own_private_unique_result,
				  Tcl_NewStringObj("TERM", -1));
   if (ret != TCL_OK) return TCL_ERROR;

   if (bilan == OK)
      ret = Tcl_ListObjAppendElement(interp, my_own_private_unique_result,
				     Tcl_NewStringObj("OK", -1));
   else
      ret = Tcl_ListObjAppendElement(interp, my_own_private_unique_result,
				     Tcl_NewStringObj(
					h2getErrMsg(bilan, strerr, 64), -1));
   if (ret != TCL_OK) return TCL_ERROR;



   Tcl_SetObjResult(interp, my_own_private_unique_result);
   return TCL_OK;
}

/* --- FILE GENERATED WITH GENOM, DO NOT EDIT BY HAND ------------------ */

/* 
 * Copyright (c) 1993-2005 LAAS/CNRS
 * All rights reserved.
 *
 * Redistribution and use  in source  and binary  forms,  with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *   1. Redistributions of  source  code must retain the  above copyright
 *      notice, this list of conditions and the following disclaimer.
 *   2. Redistributions in binary form must reproduce the above copyright
 *      notice,  this list of  conditions and the following disclaimer in
 *      the  documentation  and/or  other   materials provided  with  the
 *      distribution.
 *
 * THIS  SOFTWARE IS PROVIDED BY  THE  COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND  ANY  EXPRESS OR IMPLIED  WARRANTIES,  INCLUDING,  BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES  OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR  PURPOSE ARE DISCLAIMED. IN  NO EVENT SHALL THE COPYRIGHT
 * HOLDERS OR      CONTRIBUTORS  BE LIABLE FOR   ANY    DIRECT, INDIRECT,
 * INCIDENTAL,  SPECIAL,  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF  SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN  CONTRACT, STRICT LIABILITY, OR
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE   OF THIS SOFTWARE, EVEN   IF ADVISED OF   THE POSSIBILITY OF SUCH
 * DAMAGE.
 */

/*
 * ----------------------------------------------------------------------
 * SaveFace Request
 */

static int icuSaveFaceRqstSendCb(ClientData, Tcl_Interp *, int, Tcl_Obj *const []); 
static int icuSaveFaceReplyRcvCb(ClientData, Tcl_Interp *, int, Tcl_Obj * const []);

static int
icuSaveFaceRqstSendCb(ClientData data, Tcl_Interp *interp,
				int objc, Tcl_Obj *const objv[])
{
   struct ModuleInfo *tclModuleInfo = (struct ModuleInfo *)data;
   int rqstId, bilan;
   char buf[10];
   static int save_face;	/* input */
#if (1 > 0)
   int ret;
   int curObjc = 0;
#endif
   char strerr[64];

   TEST_BAD_USAGE(objc != 1+1);
		 
   ret = Tcl_GetIntFromObj(interp, objv[++curObjc],    	&save_face);
   if (ret != TCL_OK) return TCL_ERROR;


   if (tclServRqstSend(interp, Tcl_GetStringFromObj(objv[0], NULL),
		       tclModuleInfo->cid, ICU_SAVEFACE_RQST,
		       &save_face, sizeof(save_face),
		       &rqstId, &bilan) == ERROR) {

     Tcl_SetResult(interp, h2getErrMsg(bilan, strerr, 64), TCL_VOLATILE);
      return TCL_ERROR;
   }
    
   sprintf(buf, " %d", rqstId);
   Tcl_SetResult(interp, "SENT", TCL_STATIC);
   Tcl_AppendResult(interp, buf, NULL);
   return TCL_OK;
}

static int
icuSaveFaceReplyRcvCb(ClientData data, Tcl_Interp *interp,
				int objc, Tcl_Obj * const objv[])
{
   struct ModuleInfo *tclModuleInfo = (struct ModuleInfo *)data;
   int ret;
   int rqstId, bilan, activity;
   /* no output */ 	/* output */
   Tcl_Obj *my_own_private_unique_result;
   char strerr[64];

   TEST_BAD_USAGE(
      objc != 2 ||
      Tcl_GetIntFromObj(interp, objv[1], &rqstId) != TCL_OK
   );

   switch (
      /* c'est laid */
      tclServReplyRcv(interp,
		      tclModuleInfo->cid, rqstId, NO_BLOCK,
		      &activity, NULL, 0,
		      &bilan)
      ) {
      case WAITING_INTERMED_REPLY:
	 Tcl_SetResult(interp, "SENT", TCL_STATIC);
	 return TCL_OK;

      case WAITING_FINAL_REPLY: {
	 char tmp[50];
	 sprintf(tmp, "ACK %d", activity);
	 Tcl_SetResult(interp, tmp, TCL_VOLATILE);
	 return TCL_OK;
      }

      case FINAL_REPLY_OK:
	 break;

      default:
	 Tcl_SetResult(interp, h2getErrMsg(bilan, strerr, 64), TCL_VOLATILE);
	 return TCL_ERROR;
   }

   my_own_private_unique_result = Tcl_NewListObj(0, NULL);
   if (my_own_private_unique_result == NULL) return TCL_ERROR;

   ret = Tcl_ListObjAppendElement(interp, my_own_private_unique_result,
				  Tcl_NewStringObj("TERM", -1));
   if (ret != TCL_OK) return TCL_ERROR;

   if (bilan == OK)
      ret = Tcl_ListObjAppendElement(interp, my_own_private_unique_result,
				     Tcl_NewStringObj("OK", -1));
   else
      ret = Tcl_ListObjAppendElement(interp, my_own_private_unique_result,
				     Tcl_NewStringObj(
					h2getErrMsg(bilan, strerr, 64), -1));
   if (ret != TCL_OK) return TCL_ERROR;



   Tcl_SetObjResult(interp, my_own_private_unique_result);
   return TCL_OK;
}

/* --- FILE GENERATED WITH GENOM, DO NOT EDIT BY HAND ------------------ */

/* 
 * Copyright (c) 1993-2005 LAAS/CNRS
 * All rights reserved.
 *
 * Redistribution and use  in source  and binary  forms,  with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *   1. Redistributions of  source  code must retain the  above copyright
 *      notice, this list of conditions and the following disclaimer.
 *   2. Redistributions in binary form must reproduce the above copyright
 *      notice,  this list of  conditions and the following disclaimer in
 *      the  documentation  and/or  other   materials provided  with  the
 *      distribution.
 *
 * THIS  SOFTWARE IS PROVIDED BY  THE  COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND  ANY  EXPRESS OR IMPLIED  WARRANTIES,  INCLUDING,  BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES  OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR  PURPOSE ARE DISCLAIMED. IN  NO EVENT SHALL THE COPYRIGHT
 * HOLDERS OR      CONTRIBUTORS  BE LIABLE FOR   ANY    DIRECT, INDIRECT,
 * INCIDENTAL,  SPECIAL,  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF  SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN  CONTRACT, STRICT LIABILITY, OR
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE   OF THIS SOFTWARE, EVEN   IF ADVISED OF   THE POSSIBILITY OF SUCH
 * DAMAGE.
 */

/*
 * ----------------------------------------------------------------------
 * CaptureImage Request
 */

static int icuCaptureImageRqstSendCb(ClientData, Tcl_Interp *, int, Tcl_Obj *const []); 
static int icuCaptureImageReplyRcvCb(ClientData, Tcl_Interp *, int, Tcl_Obj * const []);

static int
icuCaptureImageRqstSendCb(ClientData data, Tcl_Interp *interp,
				int objc, Tcl_Obj *const objv[])
{
   struct ModuleInfo *tclModuleInfo = (struct ModuleInfo *)data;
   int rqstId, bilan;
   char buf[10];
   static int captureImage;	/* input */
#if (1 > 0)
   int ret;
   int curObjc = 0;
#endif
   char strerr[64];

   TEST_BAD_USAGE(objc != 1+1);
		 
   ret = Tcl_GetIntFromObj(interp, objv[++curObjc],    	&captureImage);
   if (ret != TCL_OK) return TCL_ERROR;


   if (tclServRqstSend(interp, Tcl_GetStringFromObj(objv[0], NULL),
		       tclModuleInfo->cid, ICU_CAPTUREIMAGE_RQST,
		       &captureImage, sizeof(captureImage),
		       &rqstId, &bilan) == ERROR) {

     Tcl_SetResult(interp, h2getErrMsg(bilan, strerr, 64), TCL_VOLATILE);
      return TCL_ERROR;
   }
    
   sprintf(buf, " %d", rqstId);
   Tcl_SetResult(interp, "SENT", TCL_STATIC);
   Tcl_AppendResult(interp, buf, NULL);
   return TCL_OK;
}

static int
icuCaptureImageReplyRcvCb(ClientData data, Tcl_Interp *interp,
				int objc, Tcl_Obj * const objv[])
{
   struct ModuleInfo *tclModuleInfo = (struct ModuleInfo *)data;
   int ret;
   int rqstId, bilan, activity;
   /* no output */ 	/* output */
   Tcl_Obj *my_own_private_unique_result;
   char strerr[64];

   TEST_BAD_USAGE(
      objc != 2 ||
      Tcl_GetIntFromObj(interp, objv[1], &rqstId) != TCL_OK
   );

   switch (
      /* c'est laid */
      tclServReplyRcv(interp,
		      tclModuleInfo->cid, rqstId, NO_BLOCK,
		      &activity, NULL, 0,
		      &bilan)
      ) {
      case WAITING_INTERMED_REPLY:
	 Tcl_SetResult(interp, "SENT", TCL_STATIC);
	 return TCL_OK;

      case WAITING_FINAL_REPLY: {
	 char tmp[50];
	 sprintf(tmp, "ACK %d", activity);
	 Tcl_SetResult(interp, tmp, TCL_VOLATILE);
	 return TCL_OK;
      }

      case FINAL_REPLY_OK:
	 break;

      default:
	 Tcl_SetResult(interp, h2getErrMsg(bilan, strerr, 64), TCL_VOLATILE);
	 return TCL_ERROR;
   }

   my_own_private_unique_result = Tcl_NewListObj(0, NULL);
   if (my_own_private_unique_result == NULL) return TCL_ERROR;

   ret = Tcl_ListObjAppendElement(interp, my_own_private_unique_result,
				  Tcl_NewStringObj("TERM", -1));
   if (ret != TCL_OK) return TCL_ERROR;

   if (bilan == OK)
      ret = Tcl_ListObjAppendElement(interp, my_own_private_unique_result,
				     Tcl_NewStringObj("OK", -1));
   else
      ret = Tcl_ListObjAppendElement(interp, my_own_private_unique_result,
				     Tcl_NewStringObj(
					h2getErrMsg(bilan, strerr, 64), -1));
   if (ret != TCL_OK) return TCL_ERROR;



   Tcl_SetObjResult(interp, my_own_private_unique_result);
   return TCL_OK;
}

/* --- FILE GENERATED WITH GENOM, DO NOT EDIT BY HAND ------------------ */

/* 
 * Copyright (c) 1993-2005 LAAS/CNRS
 * All rights reserved.
 *
 * Redistribution and use  in source  and binary  forms,  with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *   1. Redistributions of  source  code must retain the  above copyright
 *      notice, this list of conditions and the following disclaimer.
 *   2. Redistributions in binary form must reproduce the above copyright
 *      notice,  this list of  conditions and the following disclaimer in
 *      the  documentation  and/or  other   materials provided  with  the
 *      distribution.
 *
 * THIS  SOFTWARE IS PROVIDED BY  THE  COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND  ANY  EXPRESS OR IMPLIED  WARRANTIES,  INCLUDING,  BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES  OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR  PURPOSE ARE DISCLAIMED. IN  NO EVENT SHALL THE COPYRIGHT
 * HOLDERS OR      CONTRIBUTORS  BE LIABLE FOR   ANY    DIRECT, INDIRECT,
 * INCIDENTAL,  SPECIAL,  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF  SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN  CONTRACT, STRICT LIABILITY, OR
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE   OF THIS SOFTWARE, EVEN   IF ADVISED OF   THE POSSIBILITY OF SUCH
 * DAMAGE.
 */

/*
 * ----------------------------------------------------------------------
 * CaptureVideo Request
 */

static int icuCaptureVideoRqstSendCb(ClientData, Tcl_Interp *, int, Tcl_Obj *const []); 
static int icuCaptureVideoReplyRcvCb(ClientData, Tcl_Interp *, int, Tcl_Obj * const []);

static int
icuCaptureVideoRqstSendCb(ClientData data, Tcl_Interp *interp,
				int objc, Tcl_Obj *const objv[])
{
   struct ModuleInfo *tclModuleInfo = (struct ModuleInfo *)data;
   int rqstId, bilan;
   char buf[10];
   static int captureVideo;	/* input */
#if (1 > 0)
   int ret;
   int curObjc = 0;
#endif
   char strerr[64];

   TEST_BAD_USAGE(objc != 1+1);
		 
   ret = Tcl_GetIntFromObj(interp, objv[++curObjc],    	&captureVideo);
   if (ret != TCL_OK) return TCL_ERROR;


   if (tclServRqstSend(interp, Tcl_GetStringFromObj(objv[0], NULL),
		       tclModuleInfo->cid, ICU_CAPTUREVIDEO_RQST,
		       &captureVideo, sizeof(captureVideo),
		       &rqstId, &bilan) == ERROR) {

     Tcl_SetResult(interp, h2getErrMsg(bilan, strerr, 64), TCL_VOLATILE);
      return TCL_ERROR;
   }
    
   sprintf(buf, " %d", rqstId);
   Tcl_SetResult(interp, "SENT", TCL_STATIC);
   Tcl_AppendResult(interp, buf, NULL);
   return TCL_OK;
}

static int
icuCaptureVideoReplyRcvCb(ClientData data, Tcl_Interp *interp,
				int objc, Tcl_Obj * const objv[])
{
   struct ModuleInfo *tclModuleInfo = (struct ModuleInfo *)data;
   int ret;
   int rqstId, bilan, activity;
   /* no output */ 	/* output */
   Tcl_Obj *my_own_private_unique_result;
   char strerr[64];

   TEST_BAD_USAGE(
      objc != 2 ||
      Tcl_GetIntFromObj(interp, objv[1], &rqstId) != TCL_OK
   );

   switch (
      /* c'est laid */
      tclServReplyRcv(interp,
		      tclModuleInfo->cid, rqstId, NO_BLOCK,
		      &activity, NULL, 0,
		      &bilan)
      ) {
      case WAITING_INTERMED_REPLY:
	 Tcl_SetResult(interp, "SENT", TCL_STATIC);
	 return TCL_OK;

      case WAITING_FINAL_REPLY: {
	 char tmp[50];
	 sprintf(tmp, "ACK %d", activity);
	 Tcl_SetResult(interp, tmp, TCL_VOLATILE);
	 return TCL_OK;
      }

      case FINAL_REPLY_OK:
	 break;

      default:
	 Tcl_SetResult(interp, h2getErrMsg(bilan, strerr, 64), TCL_VOLATILE);
	 return TCL_ERROR;
   }

   my_own_private_unique_result = Tcl_NewListObj(0, NULL);
   if (my_own_private_unique_result == NULL) return TCL_ERROR;

   ret = Tcl_ListObjAppendElement(interp, my_own_private_unique_result,
				  Tcl_NewStringObj("TERM", -1));
   if (ret != TCL_OK) return TCL_ERROR;

   if (bilan == OK)
      ret = Tcl_ListObjAppendElement(interp, my_own_private_unique_result,
				     Tcl_NewStringObj("OK", -1));
   else
      ret = Tcl_ListObjAppendElement(interp, my_own_private_unique_result,
				     Tcl_NewStringObj(
					h2getErrMsg(bilan, strerr, 64), -1));
   if (ret != TCL_OK) return TCL_ERROR;



   Tcl_SetObjResult(interp, my_own_private_unique_result);
   return TCL_OK;
}

/* --- FILE GENERATED WITH GENOM, DO NOT EDIT BY HAND ------------------ */

/* 
 * Copyright (c) 1993-2005 LAAS/CNRS
 * All rights reserved.
 *
 * Redistribution and use  in source  and binary  forms,  with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *   1. Redistributions of  source  code must retain the  above copyright
 *      notice, this list of conditions and the following disclaimer.
 *   2. Redistributions in binary form must reproduce the above copyright
 *      notice,  this list of  conditions and the following disclaimer in
 *      the  documentation  and/or  other   materials provided  with  the
 *      distribution.
 *
 * THIS  SOFTWARE IS PROVIDED BY  THE  COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND  ANY  EXPRESS OR IMPLIED  WARRANTIES,  INCLUDING,  BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES  OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR  PURPOSE ARE DISCLAIMED. IN  NO EVENT SHALL THE COPYRIGHT
 * HOLDERS OR      CONTRIBUTORS  BE LIABLE FOR   ANY    DIRECT, INDIRECT,
 * INCIDENTAL,  SPECIAL,  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF  SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN  CONTRACT, STRICT LIABILITY, OR
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE   OF THIS SOFTWARE, EVEN   IF ADVISED OF   THE POSSIBILITY OF SUCH
 * DAMAGE.
 */

/*
 * ----------------------------------------------------------------------
 * SetViamPosterName Request
 */

static int icuSetViamPosterNameRqstSendCb(ClientData, Tcl_Interp *, int, Tcl_Obj *const []); 
static int icuSetViamPosterNameReplyRcvCb(ClientData, Tcl_Interp *, int, Tcl_Obj * const []);

static int
icuSetViamPosterNameRqstSendCb(ClientData data, Tcl_Interp *interp,
				int objc, Tcl_Obj *const objv[])
{
   struct ModuleInfo *tclModuleInfo = (struct ModuleInfo *)data;
   int rqstId, bilan;
   char buf[10];
   static str viamPosterName;	/* input */
#if (1 > 0)
   int ret;
   int curObjc = 0;
#endif
   char strerr[64];

   TEST_BAD_USAGE(objc != 1+1);
		 
   strncpy(viamPosterName.name, 
   	Tcl_GetStringFromObj(objv[++curObjc], NULL),    	255);
   ret = TCL_OK;
   if (ret != TCL_OK) return TCL_ERROR;


   if (tclServRqstSend(interp, Tcl_GetStringFromObj(objv[0], NULL),
		       tclModuleInfo->cid, ICU_SETVIAMPOSTERNAME_RQST,
		       &viamPosterName, sizeof(viamPosterName),
		       &rqstId, &bilan) == ERROR) {

     Tcl_SetResult(interp, h2getErrMsg(bilan, strerr, 64), TCL_VOLATILE);
      return TCL_ERROR;
   }
    
   sprintf(buf, " %d", rqstId);
   Tcl_SetResult(interp, "SENT", TCL_STATIC);
   Tcl_AppendResult(interp, buf, NULL);
   return TCL_OK;
}

static int
icuSetViamPosterNameReplyRcvCb(ClientData data, Tcl_Interp *interp,
				int objc, Tcl_Obj * const objv[])
{
   struct ModuleInfo *tclModuleInfo = (struct ModuleInfo *)data;
   int ret;
   int rqstId, bilan, activity;
   /* no output */ 	/* output */
   Tcl_Obj *my_own_private_unique_result;
   char strerr[64];

   TEST_BAD_USAGE(
      objc != 2 ||
      Tcl_GetIntFromObj(interp, objv[1], &rqstId) != TCL_OK
   );

   switch (
      /* c'est laid */
      tclServReplyRcv(interp,
		      tclModuleInfo->cid, rqstId, NO_BLOCK,
		      &activity, NULL, 0,
		      &bilan)
      ) {
      case WAITING_INTERMED_REPLY:
	 Tcl_SetResult(interp, "SENT", TCL_STATIC);
	 return TCL_OK;

      case WAITING_FINAL_REPLY: {
	 char tmp[50];
	 sprintf(tmp, "ACK %d", activity);
	 Tcl_SetResult(interp, tmp, TCL_VOLATILE);
	 return TCL_OK;
      }

      case FINAL_REPLY_OK:
	 break;

      default:
	 Tcl_SetResult(interp, h2getErrMsg(bilan, strerr, 64), TCL_VOLATILE);
	 return TCL_ERROR;
   }

   my_own_private_unique_result = Tcl_NewListObj(0, NULL);
   if (my_own_private_unique_result == NULL) return TCL_ERROR;

   ret = Tcl_ListObjAppendElement(interp, my_own_private_unique_result,
				  Tcl_NewStringObj("TERM", -1));
   if (ret != TCL_OK) return TCL_ERROR;

   if (bilan == OK)
      ret = Tcl_ListObjAppendElement(interp, my_own_private_unique_result,
				     Tcl_NewStringObj("OK", -1));
   else
      ret = Tcl_ListObjAppendElement(interp, my_own_private_unique_result,
				     Tcl_NewStringObj(
					h2getErrMsg(bilan, strerr, 64), -1));
   if (ret != TCL_OK) return TCL_ERROR;



   Tcl_SetObjResult(interp, my_own_private_unique_result);
   return TCL_OK;
}

/* --- FILE GENERATED WITH GENOM, DO NOT EDIT BY HAND ------------------ */

/* 
 * Copyright (c) 1993-2005 LAAS/CNRS
 * All rights reserved.
 *
 * Redistribution and use  in source  and binary  forms,  with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *   1. Redistributions of  source  code must retain the  above copyright
 *      notice, this list of conditions and the following disclaimer.
 *   2. Redistributions in binary form must reproduce the above copyright
 *      notice,  this list of  conditions and the following disclaimer in
 *      the  documentation  and/or  other   materials provided  with  the
 *      distribution.
 *
 * THIS  SOFTWARE IS PROVIDED BY  THE  COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND  ANY  EXPRESS OR IMPLIED  WARRANTIES,  INCLUDING,  BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES  OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR  PURPOSE ARE DISCLAIMED. IN  NO EVENT SHALL THE COPYRIGHT
 * HOLDERS OR      CONTRIBUTORS  BE LIABLE FOR   ANY    DIRECT, INDIRECT,
 * INCIDENTAL,  SPECIAL,  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF  SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN  CONTRACT, STRICT LIABILITY, OR
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE   OF THIS SOFTWARE, EVEN   IF ADVISED OF   THE POSSIBILITY OF SUCH
 * DAMAGE.
 */

/*
 * ----------------------------------------------------------------------
 * EnableDisplay Request
 */

static int icuEnableDisplayRqstSendCb(ClientData, Tcl_Interp *, int, Tcl_Obj *const []); 
static int icuEnableDisplayReplyRcvCb(ClientData, Tcl_Interp *, int, Tcl_Obj * const []);

static int
icuEnableDisplayRqstSendCb(ClientData data, Tcl_Interp *interp,
				int objc, Tcl_Obj *const objv[])
{
   struct ModuleInfo *tclModuleInfo = (struct ModuleInfo *)data;
   int rqstId, bilan;
   char buf[10];
   static ICU_AFFICHAGE display;	/* input */
#if (6 > 0)
   int ret;
   int curObjc = 0;
#endif
   char strerr[64];

   TEST_BAD_USAGE(objc != 6+1);
		 
   ret = Tcl_GetIntFromObj(interp, objv[++curObjc],    	&display.display);
   if (ret != TCL_OK) return TCL_ERROR;
   ret = Tcl_GetIntFromObj(interp, objv[++curObjc],    	&display.width);
   if (ret != TCL_OK) return TCL_ERROR;
   ret = Tcl_GetIntFromObj(interp, objv[++curObjc],    	&display.height);
   if (ret != TCL_OK) return TCL_ERROR;
   ret = Tcl_GetIntFromObj(interp, objv[++curObjc],    	&display.posx);
   if (ret != TCL_OK) return TCL_ERROR;
   ret = Tcl_GetIntFromObj(interp, objv[++curObjc],    	&display.posy);
   if (ret != TCL_OK) return TCL_ERROR;
   ret = Tcl_GetIntFromObj(interp, objv[++curObjc],    	&display.unused);
   if (ret != TCL_OK) return TCL_ERROR;


   if (tclServRqstSend(interp, Tcl_GetStringFromObj(objv[0], NULL),
		       tclModuleInfo->cid, ICU_ENABLEDISPLAY_RQST,
		       &display, sizeof(display),
		       &rqstId, &bilan) == ERROR) {

     Tcl_SetResult(interp, h2getErrMsg(bilan, strerr, 64), TCL_VOLATILE);
      return TCL_ERROR;
   }
    
   sprintf(buf, " %d", rqstId);
   Tcl_SetResult(interp, "SENT", TCL_STATIC);
   Tcl_AppendResult(interp, buf, NULL);
   return TCL_OK;
}

static int
icuEnableDisplayReplyRcvCb(ClientData data, Tcl_Interp *interp,
				int objc, Tcl_Obj * const objv[])
{
   struct ModuleInfo *tclModuleInfo = (struct ModuleInfo *)data;
   int ret;
   int rqstId, bilan, activity;
   /* no output */ 	/* output */
   Tcl_Obj *my_own_private_unique_result;
   char strerr[64];

   TEST_BAD_USAGE(
      objc != 2 ||
      Tcl_GetIntFromObj(interp, objv[1], &rqstId) != TCL_OK
   );

   switch (
      /* c'est laid */
      tclServReplyRcv(interp,
		      tclModuleInfo->cid, rqstId, NO_BLOCK,
		      &activity, NULL, 0,
		      &bilan)
      ) {
      case WAITING_INTERMED_REPLY:
	 Tcl_SetResult(interp, "SENT", TCL_STATIC);
	 return TCL_OK;

      case WAITING_FINAL_REPLY: {
	 char tmp[50];
	 sprintf(tmp, "ACK %d", activity);
	 Tcl_SetResult(interp, tmp, TCL_VOLATILE);
	 return TCL_OK;
      }

      case FINAL_REPLY_OK:
	 break;

      default:
	 Tcl_SetResult(interp, h2getErrMsg(bilan, strerr, 64), TCL_VOLATILE);
	 return TCL_ERROR;
   }

   my_own_private_unique_result = Tcl_NewListObj(0, NULL);
   if (my_own_private_unique_result == NULL) return TCL_ERROR;

   ret = Tcl_ListObjAppendElement(interp, my_own_private_unique_result,
				  Tcl_NewStringObj("TERM", -1));
   if (ret != TCL_OK) return TCL_ERROR;

   if (bilan == OK)
      ret = Tcl_ListObjAppendElement(interp, my_own_private_unique_result,
				     Tcl_NewStringObj("OK", -1));
   else
      ret = Tcl_ListObjAppendElement(interp, my_own_private_unique_result,
				     Tcl_NewStringObj(
					h2getErrMsg(bilan, strerr, 64), -1));
   if (ret != TCL_OK) return TCL_ERROR;



   Tcl_SetObjResult(interp, my_own_private_unique_result);
   return TCL_OK;
}

/* --- FILE GENERATED WITH GENOM, DO NOT EDIT BY HAND ------------------ */

/* 
 * Copyright (c) 1993-2005 LAAS/CNRS
 * All rights reserved.
 *
 * Redistribution and use  in source  and binary  forms,  with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *   1. Redistributions of  source  code must retain the  above copyright
 *      notice, this list of conditions and the following disclaimer.
 *   2. Redistributions in binary form must reproduce the above copyright
 *      notice,  this list of  conditions and the following disclaimer in
 *      the  documentation  and/or  other   materials provided  with  the
 *      distribution.
 *
 * THIS  SOFTWARE IS PROVIDED BY  THE  COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND  ANY  EXPRESS OR IMPLIED  WARRANTIES,  INCLUDING,  BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES  OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR  PURPOSE ARE DISCLAIMED. IN  NO EVENT SHALL THE COPYRIGHT
 * HOLDERS OR      CONTRIBUTORS  BE LIABLE FOR   ANY    DIRECT, INDIRECT,
 * INCIDENTAL,  SPECIAL,  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF  SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN  CONTRACT, STRICT LIABILITY, OR
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE   OF THIS SOFTWARE, EVEN   IF ADVISED OF   THE POSSIBILITY OF SUCH
 * DAMAGE.
 */

/*
 * ----------------------------------------------------------------------
 * SetCameraPos Request
 */

static int icuSetCameraPosRqstSendCb(ClientData, Tcl_Interp *, int, Tcl_Obj *const []); 
static int icuSetCameraPosReplyRcvCb(ClientData, Tcl_Interp *, int, Tcl_Obj * const []);

static int
icuSetCameraPosRqstSendCb(ClientData data, Tcl_Interp *interp,
				int objc, Tcl_Obj *const objv[])
{
   struct ModuleInfo *tclModuleInfo = (struct ModuleInfo *)data;
   int rqstId, bilan;
   char buf[10];
   static ICU_CAMERA_POS cameraPos;	/* input */
#if (3 > 0)
   int ret;
   int curObjc = 0;
#endif
   char strerr[64];

   TEST_BAD_USAGE(objc != 3+1);
		 
   {
      double tmpDouble;
      ret = Tcl_GetDoubleFromObj(interp, objv[++curObjc],       	&tmpDouble);
      cameraPos.pan = (float)tmpDouble;
   }
   if (ret != TCL_OK) return TCL_ERROR;
   {
      double tmpDouble;
      ret = Tcl_GetDoubleFromObj(interp, objv[++curObjc],       	&tmpDouble);
      cameraPos.tilt = (float)tmpDouble;
   }
   if (ret != TCL_OK) return TCL_ERROR;
   {
      double tmpDouble;
      ret = Tcl_GetDoubleFromObj(interp, objv[++curObjc],       	&tmpDouble);
      cameraPos.zoom = (float)tmpDouble;
   }
   if (ret != TCL_OK) return TCL_ERROR;


   if (tclServRqstSend(interp, Tcl_GetStringFromObj(objv[0], NULL),
		       tclModuleInfo->cid, ICU_SETCAMERAPOS_RQST,
		       &cameraPos, sizeof(cameraPos),
		       &rqstId, &bilan) == ERROR) {

     Tcl_SetResult(interp, h2getErrMsg(bilan, strerr, 64), TCL_VOLATILE);
      return TCL_ERROR;
   }
    
   sprintf(buf, " %d", rqstId);
   Tcl_SetResult(interp, "SENT", TCL_STATIC);
   Tcl_AppendResult(interp, buf, NULL);
   return TCL_OK;
}

static int
icuSetCameraPosReplyRcvCb(ClientData data, Tcl_Interp *interp,
				int objc, Tcl_Obj * const objv[])
{
   struct ModuleInfo *tclModuleInfo = (struct ModuleInfo *)data;
   int ret;
   int rqstId, bilan, activity;
   /* no output */ 	/* output */
   Tcl_Obj *my_own_private_unique_result;
   char strerr[64];

   TEST_BAD_USAGE(
      objc != 2 ||
      Tcl_GetIntFromObj(interp, objv[1], &rqstId) != TCL_OK
   );

   switch (
      /* c'est laid */
      tclServReplyRcv(interp,
		      tclModuleInfo->cid, rqstId, NO_BLOCK,
		      &activity, NULL, 0,
		      &bilan)
      ) {
      case WAITING_INTERMED_REPLY:
	 Tcl_SetResult(interp, "SENT", TCL_STATIC);
	 return TCL_OK;

      case WAITING_FINAL_REPLY: {
	 char tmp[50];
	 sprintf(tmp, "ACK %d", activity);
	 Tcl_SetResult(interp, tmp, TCL_VOLATILE);
	 return TCL_OK;
      }

      case FINAL_REPLY_OK:
	 break;

      default:
	 Tcl_SetResult(interp, h2getErrMsg(bilan, strerr, 64), TCL_VOLATILE);
	 return TCL_ERROR;
   }

   my_own_private_unique_result = Tcl_NewListObj(0, NULL);
   if (my_own_private_unique_result == NULL) return TCL_ERROR;

   ret = Tcl_ListObjAppendElement(interp, my_own_private_unique_result,
				  Tcl_NewStringObj("TERM", -1));
   if (ret != TCL_OK) return TCL_ERROR;

   if (bilan == OK)
      ret = Tcl_ListObjAppendElement(interp, my_own_private_unique_result,
				     Tcl_NewStringObj("OK", -1));
   else
      ret = Tcl_ListObjAppendElement(interp, my_own_private_unique_result,
				     Tcl_NewStringObj(
					h2getErrMsg(bilan, strerr, 64), -1));
   if (ret != TCL_OK) return TCL_ERROR;



   Tcl_SetObjResult(interp, my_own_private_unique_result);
   return TCL_OK;
}

/* --- FILE GENERATED WITH GENOM, DO NOT EDIT BY HAND ------------------ */

/* 
 * Copyright (c) 1993-2005 LAAS/CNRS
 * All rights reserved.
 *
 * Redistribution and use  in source  and binary  forms,  with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *   1. Redistributions of  source  code must retain the  above copyright
 *      notice, this list of conditions and the following disclaimer.
 *   2. Redistributions in binary form must reproduce the above copyright
 *      notice,  this list of  conditions and the following disclaimer in
 *      the  documentation  and/or  other   materials provided  with  the
 *      distribution.
 *
 * THIS  SOFTWARE IS PROVIDED BY  THE  COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND  ANY  EXPRESS OR IMPLIED  WARRANTIES,  INCLUDING,  BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES  OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR  PURPOSE ARE DISCLAIMED. IN  NO EVENT SHALL THE COPYRIGHT
 * HOLDERS OR      CONTRIBUTORS  BE LIABLE FOR   ANY    DIRECT, INDIRECT,
 * INCIDENTAL,  SPECIAL,  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF  SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN  CONTRACT, STRICT LIABILITY, OR
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE   OF THIS SOFTWARE, EVEN   IF ADVISED OF   THE POSSIBILITY OF SUCH
 * DAMAGE.
 */

/*
 * ----------------------------------------------------------------------
 * TrackIt Request
 */

static int icuTrackItRqstSendCb(ClientData, Tcl_Interp *, int, Tcl_Obj *const []); 
static int icuTrackItReplyRcvCb(ClientData, Tcl_Interp *, int, Tcl_Obj * const []);

static int
icuTrackItRqstSendCb(ClientData data, Tcl_Interp *interp,
				int objc, Tcl_Obj *const objv[])
{
   struct ModuleInfo *tclModuleInfo = (struct ModuleInfo *)data;
   int rqstId, bilan;
   char buf[10];
   static int track_it;	/* input */
#if (1 > 0)
   int ret;
   int curObjc = 0;
#endif
   char strerr[64];

   TEST_BAD_USAGE(objc != 1+1);
		 
   ret = Tcl_GetIntFromObj(interp, objv[++curObjc],    	&track_it);
   if (ret != TCL_OK) return TCL_ERROR;


   if (tclServRqstSend(interp, Tcl_GetStringFromObj(objv[0], NULL),
		       tclModuleInfo->cid, ICU_TRACKIT_RQST,
		       &track_it, sizeof(track_it),
		       &rqstId, &bilan) == ERROR) {

     Tcl_SetResult(interp, h2getErrMsg(bilan, strerr, 64), TCL_VOLATILE);
      return TCL_ERROR;
   }
    
   sprintf(buf, " %d", rqstId);
   Tcl_SetResult(interp, "SENT", TCL_STATIC);
   Tcl_AppendResult(interp, buf, NULL);
   return TCL_OK;
}

static int
icuTrackItReplyRcvCb(ClientData data, Tcl_Interp *interp,
				int objc, Tcl_Obj * const objv[])
{
   struct ModuleInfo *tclModuleInfo = (struct ModuleInfo *)data;
   int ret;
   int rqstId, bilan, activity;
   /* no output */ 	/* output */
   Tcl_Obj *my_own_private_unique_result;
   char strerr[64];

   TEST_BAD_USAGE(
      objc != 2 ||
      Tcl_GetIntFromObj(interp, objv[1], &rqstId) != TCL_OK
   );

   switch (
      /* c'est laid */
      tclServReplyRcv(interp,
		      tclModuleInfo->cid, rqstId, NO_BLOCK,
		      &activity, NULL, 0,
		      &bilan)
      ) {
      case WAITING_INTERMED_REPLY:
	 Tcl_SetResult(interp, "SENT", TCL_STATIC);
	 return TCL_OK;

      case WAITING_FINAL_REPLY: {
	 char tmp[50];
	 sprintf(tmp, "ACK %d", activity);
	 Tcl_SetResult(interp, tmp, TCL_VOLATILE);
	 return TCL_OK;
      }

      case FINAL_REPLY_OK:
	 break;

      default:
	 Tcl_SetResult(interp, h2getErrMsg(bilan, strerr, 64), TCL_VOLATILE);
	 return TCL_ERROR;
   }

   my_own_private_unique_result = Tcl_NewListObj(0, NULL);
   if (my_own_private_unique_result == NULL) return TCL_ERROR;

   ret = Tcl_ListObjAppendElement(interp, my_own_private_unique_result,
				  Tcl_NewStringObj("TERM", -1));
   if (ret != TCL_OK) return TCL_ERROR;

   if (bilan == OK)
      ret = Tcl_ListObjAppendElement(interp, my_own_private_unique_result,
				     Tcl_NewStringObj("OK", -1));
   else
      ret = Tcl_ListObjAppendElement(interp, my_own_private_unique_result,
				     Tcl_NewStringObj(
					h2getErrMsg(bilan, strerr, 64), -1));
   if (ret != TCL_OK) return TCL_ERROR;



   Tcl_SetObjResult(interp, my_own_private_unique_result);
   return TCL_OK;
}

/* --- FILE GENERATED WITH GENOM, DO NOT EDIT BY HAND ------------------ */

/* 
 * Copyright (c) 1993-2005 LAAS/CNRS
 * All rights reserved.
 *
 * Redistribution and use  in source  and binary  forms,  with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *   1. Redistributions of  source  code must retain the  above copyright
 *      notice, this list of conditions and the following disclaimer.
 *   2. Redistributions in binary form must reproduce the above copyright
 *      notice,  this list of  conditions and the following disclaimer in
 *      the  documentation  and/or  other   materials provided  with  the
 *      distribution.
 *
 * THIS  SOFTWARE IS PROVIDED BY  THE  COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND  ANY  EXPRESS OR IMPLIED  WARRANTIES,  INCLUDING,  BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES  OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR  PURPOSE ARE DISCLAIMED. IN  NO EVENT SHALL THE COPYRIGHT
 * HOLDERS OR      CONTRIBUTORS  BE LIABLE FOR   ANY    DIRECT, INDIRECT,
 * INCIDENTAL,  SPECIAL,  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF  SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN  CONTRACT, STRICT LIABILITY, OR
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE   OF THIS SOFTWARE, EVEN   IF ADVISED OF   THE POSSIBILITY OF SUCH
 * DAMAGE.
 */

/*
 * ----------------------------------------------------------------------
 * WaitForSomebody Request
 */

static int icuWaitForSomebodyRqstSendCb(ClientData, Tcl_Interp *, int, Tcl_Obj *const []); 
static int icuWaitForSomebodyReplyRcvCb(ClientData, Tcl_Interp *, int, Tcl_Obj * const []);

static int
icuWaitForSomebodyRqstSendCb(ClientData data, Tcl_Interp *interp,
				int objc, Tcl_Obj *const objv[])
{
   struct ModuleInfo *tclModuleInfo = (struct ModuleInfo *)data;
   int rqstId, bilan;
   char buf[10];
   static int nbImageMaxNobody;	/* input */
#if (1 > 0)
   int ret;
   int curObjc = 0;
#endif
   char strerr[64];

   TEST_BAD_USAGE(objc != 1+1);
		 
   ret = Tcl_GetIntFromObj(interp, objv[++curObjc],    	&nbImageMaxNobody);
   if (ret != TCL_OK) return TCL_ERROR;


   if (tclServRqstSend(interp, Tcl_GetStringFromObj(objv[0], NULL),
		       tclModuleInfo->cid, ICU_WAITFORSOMEBODY_RQST,
		       &nbImageMaxNobody, sizeof(nbImageMaxNobody),
		       &rqstId, &bilan) == ERROR) {

     Tcl_SetResult(interp, h2getErrMsg(bilan, strerr, 64), TCL_VOLATILE);
      return TCL_ERROR;
   }
    
   sprintf(buf, " %d", rqstId);
   Tcl_SetResult(interp, "SENT", TCL_STATIC);
   Tcl_AppendResult(interp, buf, NULL);
   return TCL_OK;
}

static int
icuWaitForSomebodyReplyRcvCb(ClientData data, Tcl_Interp *interp,
				int objc, Tcl_Obj * const objv[])
{
   struct ModuleInfo *tclModuleInfo = (struct ModuleInfo *)data;
   int ret;
   int rqstId, bilan, activity;
   static ICU_WAITFOR waitFor;	/* output */
   Tcl_Obj *my_own_private_unique_result;
   char strerr[64];

   TEST_BAD_USAGE(
      objc != 2 ||
      Tcl_GetIntFromObj(interp, objv[1], &rqstId) != TCL_OK
   );

   switch (
      /* c'est laid */
      tclServReplyRcv(interp,
		      tclModuleInfo->cid, rqstId, NO_BLOCK,
		      &activity, &waitFor, sizeof(waitFor),
		      &bilan)
      ) {
      case WAITING_INTERMED_REPLY:
	 Tcl_SetResult(interp, "SENT", TCL_STATIC);
	 return TCL_OK;

      case WAITING_FINAL_REPLY: {
	 char tmp[50];
	 sprintf(tmp, "ACK %d", activity);
	 Tcl_SetResult(interp, tmp, TCL_VOLATILE);
	 return TCL_OK;
      }

      case FINAL_REPLY_OK:
	 break;

      default:
	 Tcl_SetResult(interp, h2getErrMsg(bilan, strerr, 64), TCL_VOLATILE);
	 return TCL_ERROR;
   }

   my_own_private_unique_result = Tcl_NewListObj(0, NULL);
   if (my_own_private_unique_result == NULL) return TCL_ERROR;

   ret = Tcl_ListObjAppendElement(interp, my_own_private_unique_result,
				  Tcl_NewStringObj("TERM", -1));
   if (ret != TCL_OK) return TCL_ERROR;

   if (bilan == OK)
      ret = Tcl_ListObjAppendElement(interp, my_own_private_unique_result,
				     Tcl_NewStringObj("OK", -1));
   else
      ret = Tcl_ListObjAppendElement(interp, my_own_private_unique_result,
				     Tcl_NewStringObj(
					h2getErrMsg(bilan, strerr, 64), -1));
   if (ret != TCL_OK) return TCL_ERROR;

   ret = Tcl_ListObjAppendElement(interp, my_own_private_unique_result,
	Tcl_NewStringObj(waitFor.name, -1));
   if (ret != TCL_OK) return TCL_ERROR;
   ret = Tcl_ListObjAppendElement(interp, my_own_private_unique_result,
	Tcl_NewIntObj(waitFor.nbImageMinSomebody));
   if (ret != TCL_OK) return TCL_ERROR;


   Tcl_SetObjResult(interp, my_own_private_unique_result);
   return TCL_OK;
}

/* --- FILE GENERATED WITH GENOM, DO NOT EDIT BY HAND ------------------ */

/* 
 * Copyright (c) 1993-2005 LAAS/CNRS
 * All rights reserved.
 *
 * Redistribution and use  in source  and binary  forms,  with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *   1. Redistributions of  source  code must retain the  above copyright
 *      notice, this list of conditions and the following disclaimer.
 *   2. Redistributions in binary form must reproduce the above copyright
 *      notice,  this list of  conditions and the following disclaimer in
 *      the  documentation  and/or  other   materials provided  with  the
 *      distribution.
 *
 * THIS  SOFTWARE IS PROVIDED BY  THE  COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND  ANY  EXPRESS OR IMPLIED  WARRANTIES,  INCLUDING,  BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES  OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR  PURPOSE ARE DISCLAIMED. IN  NO EVENT SHALL THE COPYRIGHT
 * HOLDERS OR      CONTRIBUTORS  BE LIABLE FOR   ANY    DIRECT, INDIRECT,
 * INCIDENTAL,  SPECIAL,  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF  SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN  CONTRACT, STRICT LIABILITY, OR
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE   OF THIS SOFTWARE, EVEN   IF ADVISED OF   THE POSSIBILITY OF SUCH
 * DAMAGE.
 */

/*
 * ----------------------------------------------------------------------
 * WaitForSomeone Request
 */

static int icuWaitForSomeoneRqstSendCb(ClientData, Tcl_Interp *, int, Tcl_Obj *const []); 
static int icuWaitForSomeoneReplyRcvCb(ClientData, Tcl_Interp *, int, Tcl_Obj * const []);

static int
icuWaitForSomeoneRqstSendCb(ClientData data, Tcl_Interp *interp,
				int objc, Tcl_Obj *const objv[])
{
   struct ModuleInfo *tclModuleInfo = (struct ModuleInfo *)data;
   int rqstId, bilan;
   char buf[10];
   static ICU_WAITFOR waitFor;	/* input */
#if (2 > 0)
   int ret;
   int curObjc = 0;
#endif
   char strerr[64];

   TEST_BAD_USAGE(objc != 2+1);
		 
   strncpy(waitFor.name, 
   	Tcl_GetStringFromObj(objv[++curObjc], NULL),    	256);
   ret = TCL_OK;
   if (ret != TCL_OK) return TCL_ERROR;
   ret = Tcl_GetIntFromObj(interp, objv[++curObjc],    	&waitFor.nbImageMinSomebody);
   if (ret != TCL_OK) return TCL_ERROR;


   if (tclServRqstSend(interp, Tcl_GetStringFromObj(objv[0], NULL),
		       tclModuleInfo->cid, ICU_WAITFORSOMEONE_RQST,
		       &waitFor, sizeof(waitFor),
		       &rqstId, &bilan) == ERROR) {

     Tcl_SetResult(interp, h2getErrMsg(bilan, strerr, 64), TCL_VOLATILE);
      return TCL_ERROR;
   }
    
   sprintf(buf, " %d", rqstId);
   Tcl_SetResult(interp, "SENT", TCL_STATIC);
   Tcl_AppendResult(interp, buf, NULL);
   return TCL_OK;
}

static int
icuWaitForSomeoneReplyRcvCb(ClientData data, Tcl_Interp *interp,
				int objc, Tcl_Obj * const objv[])
{
   struct ModuleInfo *tclModuleInfo = (struct ModuleInfo *)data;
   int ret;
   int rqstId, bilan, activity;
   /* no output */ 	/* output */
   Tcl_Obj *my_own_private_unique_result;
   char strerr[64];

   TEST_BAD_USAGE(
      objc != 2 ||
      Tcl_GetIntFromObj(interp, objv[1], &rqstId) != TCL_OK
   );

   switch (
      /* c'est laid */
      tclServReplyRcv(interp,
		      tclModuleInfo->cid, rqstId, NO_BLOCK,
		      &activity, NULL, 0,
		      &bilan)
      ) {
      case WAITING_INTERMED_REPLY:
	 Tcl_SetResult(interp, "SENT", TCL_STATIC);
	 return TCL_OK;

      case WAITING_FINAL_REPLY: {
	 char tmp[50];
	 sprintf(tmp, "ACK %d", activity);
	 Tcl_SetResult(interp, tmp, TCL_VOLATILE);
	 return TCL_OK;
      }

      case FINAL_REPLY_OK:
	 break;

      default:
	 Tcl_SetResult(interp, h2getErrMsg(bilan, strerr, 64), TCL_VOLATILE);
	 return TCL_ERROR;
   }

   my_own_private_unique_result = Tcl_NewListObj(0, NULL);
   if (my_own_private_unique_result == NULL) return TCL_ERROR;

   ret = Tcl_ListObjAppendElement(interp, my_own_private_unique_result,
				  Tcl_NewStringObj("TERM", -1));
   if (ret != TCL_OK) return TCL_ERROR;

   if (bilan == OK)
      ret = Tcl_ListObjAppendElement(interp, my_own_private_unique_result,
				     Tcl_NewStringObj("OK", -1));
   else
      ret = Tcl_ListObjAppendElement(interp, my_own_private_unique_result,
				     Tcl_NewStringObj(
					h2getErrMsg(bilan, strerr, 64), -1));
   if (ret != TCL_OK) return TCL_ERROR;



   Tcl_SetObjResult(interp, my_own_private_unique_result);
   return TCL_OK;
}

/* --- FILE GENERATED WITH GENOM, DO NOT EDIT BY HAND ------------------ */

/* 
 * Copyright (c) 1993-2005 LAAS/CNRS
 * All rights reserved.
 *
 * Redistribution and use  in source  and binary  forms,  with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *   1. Redistributions of  source  code must retain the  above copyright
 *      notice, this list of conditions and the following disclaimer.
 *   2. Redistributions in binary form must reproduce the above copyright
 *      notice,  this list of  conditions and the following disclaimer in
 *      the  documentation  and/or  other   materials provided  with  the
 *      distribution.
 *
 * THIS  SOFTWARE IS PROVIDED BY  THE  COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND  ANY  EXPRESS OR IMPLIED  WARRANTIES,  INCLUDING,  BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES  OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR  PURPOSE ARE DISCLAIMED. IN  NO EVENT SHALL THE COPYRIGHT
 * HOLDERS OR      CONTRIBUTORS  BE LIABLE FOR   ANY    DIRECT, INDIRECT,
 * INCIDENTAL,  SPECIAL,  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF  SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN  CONTRACT, STRICT LIABILITY, OR
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE   OF THIS SOFTWARE, EVEN   IF ADVISED OF   THE POSSIBILITY OF SUCH
 * DAMAGE.
 */

/*
 * ----------------------------------------------------------------------
 * WaitForNobody Request
 */

static int icuWaitForNobodyRqstSendCb(ClientData, Tcl_Interp *, int, Tcl_Obj *const []); 
static int icuWaitForNobodyReplyRcvCb(ClientData, Tcl_Interp *, int, Tcl_Obj * const []);

static int
icuWaitForNobodyRqstSendCb(ClientData data, Tcl_Interp *interp,
				int objc, Tcl_Obj *const objv[])
{
   struct ModuleInfo *tclModuleInfo = (struct ModuleInfo *)data;
   int rqstId, bilan;
   char buf[10];
   static int nbImageMaxNobody;	/* input */
#if (1 > 0)
   int ret;
   int curObjc = 0;
#endif
   char strerr[64];

   TEST_BAD_USAGE(objc != 1+1);
		 
   ret = Tcl_GetIntFromObj(interp, objv[++curObjc],    	&nbImageMaxNobody);
   if (ret != TCL_OK) return TCL_ERROR;


   if (tclServRqstSend(interp, Tcl_GetStringFromObj(objv[0], NULL),
		       tclModuleInfo->cid, ICU_WAITFORNOBODY_RQST,
		       &nbImageMaxNobody, sizeof(nbImageMaxNobody),
		       &rqstId, &bilan) == ERROR) {

     Tcl_SetResult(interp, h2getErrMsg(bilan, strerr, 64), TCL_VOLATILE);
      return TCL_ERROR;
   }
    
   sprintf(buf, " %d", rqstId);
   Tcl_SetResult(interp, "SENT", TCL_STATIC);
   Tcl_AppendResult(interp, buf, NULL);
   return TCL_OK;
}

static int
icuWaitForNobodyReplyRcvCb(ClientData data, Tcl_Interp *interp,
				int objc, Tcl_Obj * const objv[])
{
   struct ModuleInfo *tclModuleInfo = (struct ModuleInfo *)data;
   int ret;
   int rqstId, bilan, activity;
   /* no output */ 	/* output */
   Tcl_Obj *my_own_private_unique_result;
   char strerr[64];

   TEST_BAD_USAGE(
      objc != 2 ||
      Tcl_GetIntFromObj(interp, objv[1], &rqstId) != TCL_OK
   );

   switch (
      /* c'est laid */
      tclServReplyRcv(interp,
		      tclModuleInfo->cid, rqstId, NO_BLOCK,
		      &activity, NULL, 0,
		      &bilan)
      ) {
      case WAITING_INTERMED_REPLY:
	 Tcl_SetResult(interp, "SENT", TCL_STATIC);
	 return TCL_OK;

      case WAITING_FINAL_REPLY: {
	 char tmp[50];
	 sprintf(tmp, "ACK %d", activity);
	 Tcl_SetResult(interp, tmp, TCL_VOLATILE);
	 return TCL_OK;
      }

      case FINAL_REPLY_OK:
	 break;

      default:
	 Tcl_SetResult(interp, h2getErrMsg(bilan, strerr, 64), TCL_VOLATILE);
	 return TCL_ERROR;
   }

   my_own_private_unique_result = Tcl_NewListObj(0, NULL);
   if (my_own_private_unique_result == NULL) return TCL_ERROR;

   ret = Tcl_ListObjAppendElement(interp, my_own_private_unique_result,
				  Tcl_NewStringObj("TERM", -1));
   if (ret != TCL_OK) return TCL_ERROR;

   if (bilan == OK)
      ret = Tcl_ListObjAppendElement(interp, my_own_private_unique_result,
				     Tcl_NewStringObj("OK", -1));
   else
      ret = Tcl_ListObjAppendElement(interp, my_own_private_unique_result,
				     Tcl_NewStringObj(
					h2getErrMsg(bilan, strerr, 64), -1));
   if (ret != TCL_OK) return TCL_ERROR;



   Tcl_SetObjResult(interp, my_own_private_unique_result);
   return TCL_OK;
}

/* --- FILE GENERATED WITH GENOM, DO NOT EDIT BY HAND ------------------ */

/* 
 * Copyright (c) 1993-2005 LAAS/CNRS
 * All rights reserved.
 *
 * Redistribution and use  in source  and binary  forms,  with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *   1. Redistributions of  source  code must retain the  above copyright
 *      notice, this list of conditions and the following disclaimer.
 *   2. Redistributions in binary form must reproduce the above copyright
 *      notice,  this list of  conditions and the following disclaimer in
 *      the  documentation  and/or  other   materials provided  with  the
 *      distribution.
 *
 * THIS  SOFTWARE IS PROVIDED BY  THE  COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND  ANY  EXPRESS OR IMPLIED  WARRANTIES,  INCLUDING,  BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES  OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR  PURPOSE ARE DISCLAIMED. IN  NO EVENT SHALL THE COPYRIGHT
 * HOLDERS OR      CONTRIBUTORS  BE LIABLE FOR   ANY    DIRECT, INDIRECT,
 * INCIDENTAL,  SPECIAL,  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF  SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN  CONTRACT, STRICT LIABILITY, OR
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE   OF THIS SOFTWARE, EVEN   IF ADVISED OF   THE POSSIBILITY OF SUCH
 * DAMAGE.
 */

/*
 * ----------------------------------------------------------------------
 * WaitForNone Request
 */

static int icuWaitForNoneRqstSendCb(ClientData, Tcl_Interp *, int, Tcl_Obj *const []); 
static int icuWaitForNoneReplyRcvCb(ClientData, Tcl_Interp *, int, Tcl_Obj * const []);

static int
icuWaitForNoneRqstSendCb(ClientData data, Tcl_Interp *interp,
				int objc, Tcl_Obj *const objv[])
{
   struct ModuleInfo *tclModuleInfo = (struct ModuleInfo *)data;
   int rqstId, bilan;
   char buf[10];
   static ICU_WAITFOR waitFor;	/* input */
#if (2 > 0)
   int ret;
   int curObjc = 0;
#endif
   char strerr[64];

   TEST_BAD_USAGE(objc != 2+1);
		 
   strncpy(waitFor.name, 
   	Tcl_GetStringFromObj(objv[++curObjc], NULL),    	256);
   ret = TCL_OK;
   if (ret != TCL_OK) return TCL_ERROR;
   ret = Tcl_GetIntFromObj(interp, objv[++curObjc],    	&waitFor.nbImageMinSomebody);
   if (ret != TCL_OK) return TCL_ERROR;


   if (tclServRqstSend(interp, Tcl_GetStringFromObj(objv[0], NULL),
		       tclModuleInfo->cid, ICU_WAITFORNONE_RQST,
		       &waitFor, sizeof(waitFor),
		       &rqstId, &bilan) == ERROR) {

     Tcl_SetResult(interp, h2getErrMsg(bilan, strerr, 64), TCL_VOLATILE);
      return TCL_ERROR;
   }
    
   sprintf(buf, " %d", rqstId);
   Tcl_SetResult(interp, "SENT", TCL_STATIC);
   Tcl_AppendResult(interp, buf, NULL);
   return TCL_OK;
}

static int
icuWaitForNoneReplyRcvCb(ClientData data, Tcl_Interp *interp,
				int objc, Tcl_Obj * const objv[])
{
   struct ModuleInfo *tclModuleInfo = (struct ModuleInfo *)data;
   int ret;
   int rqstId, bilan, activity;
   /* no output */ 	/* output */
   Tcl_Obj *my_own_private_unique_result;
   char strerr[64];

   TEST_BAD_USAGE(
      objc != 2 ||
      Tcl_GetIntFromObj(interp, objv[1], &rqstId) != TCL_OK
   );

   switch (
      /* c'est laid */
      tclServReplyRcv(interp,
		      tclModuleInfo->cid, rqstId, NO_BLOCK,
		      &activity, NULL, 0,
		      &bilan)
      ) {
      case WAITING_INTERMED_REPLY:
	 Tcl_SetResult(interp, "SENT", TCL_STATIC);
	 return TCL_OK;

      case WAITING_FINAL_REPLY: {
	 char tmp[50];
	 sprintf(tmp, "ACK %d", activity);
	 Tcl_SetResult(interp, tmp, TCL_VOLATILE);
	 return TCL_OK;
      }

      case FINAL_REPLY_OK:
	 break;

      default:
	 Tcl_SetResult(interp, h2getErrMsg(bilan, strerr, 64), TCL_VOLATILE);
	 return TCL_ERROR;
   }

   my_own_private_unique_result = Tcl_NewListObj(0, NULL);
   if (my_own_private_unique_result == NULL) return TCL_ERROR;

   ret = Tcl_ListObjAppendElement(interp, my_own_private_unique_result,
				  Tcl_NewStringObj("TERM", -1));
   if (ret != TCL_OK) return TCL_ERROR;

   if (bilan == OK)
      ret = Tcl_ListObjAppendElement(interp, my_own_private_unique_result,
				     Tcl_NewStringObj("OK", -1));
   else
      ret = Tcl_ListObjAppendElement(interp, my_own_private_unique_result,
				     Tcl_NewStringObj(
					h2getErrMsg(bilan, strerr, 64), -1));
   if (ret != TCL_OK) return TCL_ERROR;



   Tcl_SetObjResult(interp, my_own_private_unique_result);
   return TCL_OK;
}

/* --- FILE GENERATED WITH GENOM, DO NOT EDIT BY HAND ------------------ */

/* 
 * Copyright (c) 1993-2005 LAAS/CNRS
 * All rights reserved.
 *
 * Redistribution and use  in source  and binary  forms,  with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *   1. Redistributions of  source  code must retain the  above copyright
 *      notice, this list of conditions and the following disclaimer.
 *   2. Redistributions in binary form must reproduce the above copyright
 *      notice,  this list of  conditions and the following disclaimer in
 *      the  documentation  and/or  other   materials provided  with  the
 *      distribution.
 *
 * THIS  SOFTWARE IS PROVIDED BY  THE  COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND  ANY  EXPRESS OR IMPLIED  WARRANTIES,  INCLUDING,  BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES  OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR  PURPOSE ARE DISCLAIMED. IN  NO EVENT SHALL THE COPYRIGHT
 * HOLDERS OR      CONTRIBUTORS  BE LIABLE FOR   ANY    DIRECT, INDIRECT,
 * INCIDENTAL,  SPECIAL,  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF  SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN  CONTRACT, STRICT LIABILITY, OR
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE   OF THIS SOFTWARE, EVEN   IF ADVISED OF   THE POSSIBILITY OF SUCH
 * DAMAGE.
 */

/*
 * ----------------------------------------------------------------------
 * WaitForNobodyKnown Request
 */

static int icuWaitForNobodyKnownRqstSendCb(ClientData, Tcl_Interp *, int, Tcl_Obj *const []); 
static int icuWaitForNobodyKnownReplyRcvCb(ClientData, Tcl_Interp *, int, Tcl_Obj * const []);

static int
icuWaitForNobodyKnownRqstSendCb(ClientData data, Tcl_Interp *interp,
				int objc, Tcl_Obj *const objv[])
{
   struct ModuleInfo *tclModuleInfo = (struct ModuleInfo *)data;
   int rqstId, bilan;
   char buf[10];
   static int nbImageMaxNobody;	/* input */
#if (1 > 0)
   int ret;
   int curObjc = 0;
#endif
   char strerr[64];

   TEST_BAD_USAGE(objc != 1+1);
		 
   ret = Tcl_GetIntFromObj(interp, objv[++curObjc],    	&nbImageMaxNobody);
   if (ret != TCL_OK) return TCL_ERROR;


   if (tclServRqstSend(interp, Tcl_GetStringFromObj(objv[0], NULL),
		       tclModuleInfo->cid, ICU_WAITFORNOBODYKNOWN_RQST,
		       &nbImageMaxNobody, sizeof(nbImageMaxNobody),
		       &rqstId, &bilan) == ERROR) {

     Tcl_SetResult(interp, h2getErrMsg(bilan, strerr, 64), TCL_VOLATILE);
      return TCL_ERROR;
   }
    
   sprintf(buf, " %d", rqstId);
   Tcl_SetResult(interp, "SENT", TCL_STATIC);
   Tcl_AppendResult(interp, buf, NULL);
   return TCL_OK;
}

static int
icuWaitForNobodyKnownReplyRcvCb(ClientData data, Tcl_Interp *interp,
				int objc, Tcl_Obj * const objv[])
{
   struct ModuleInfo *tclModuleInfo = (struct ModuleInfo *)data;
   int ret;
   int rqstId, bilan, activity;
   /* no output */ 	/* output */
   Tcl_Obj *my_own_private_unique_result;
   char strerr[64];

   TEST_BAD_USAGE(
      objc != 2 ||
      Tcl_GetIntFromObj(interp, objv[1], &rqstId) != TCL_OK
   );

   switch (
      /* c'est laid */
      tclServReplyRcv(interp,
		      tclModuleInfo->cid, rqstId, NO_BLOCK,
		      &activity, NULL, 0,
		      &bilan)
      ) {
      case WAITING_INTERMED_REPLY:
	 Tcl_SetResult(interp, "SENT", TCL_STATIC);
	 return TCL_OK;

      case WAITING_FINAL_REPLY: {
	 char tmp[50];
	 sprintf(tmp, "ACK %d", activity);
	 Tcl_SetResult(interp, tmp, TCL_VOLATILE);
	 return TCL_OK;
      }

      case FINAL_REPLY_OK:
	 break;

      default:
	 Tcl_SetResult(interp, h2getErrMsg(bilan, strerr, 64), TCL_VOLATILE);
	 return TCL_ERROR;
   }

   my_own_private_unique_result = Tcl_NewListObj(0, NULL);
   if (my_own_private_unique_result == NULL) return TCL_ERROR;

   ret = Tcl_ListObjAppendElement(interp, my_own_private_unique_result,
				  Tcl_NewStringObj("TERM", -1));
   if (ret != TCL_OK) return TCL_ERROR;

   if (bilan == OK)
      ret = Tcl_ListObjAppendElement(interp, my_own_private_unique_result,
				     Tcl_NewStringObj("OK", -1));
   else
      ret = Tcl_ListObjAppendElement(interp, my_own_private_unique_result,
				     Tcl_NewStringObj(
					h2getErrMsg(bilan, strerr, 64), -1));
   if (ret != TCL_OK) return TCL_ERROR;



   Tcl_SetObjResult(interp, my_own_private_unique_result);
   return TCL_OK;
}

/* --- FILE GENERATED WITH GENOM, DO NOT EDIT BY HAND ------------------ */

/* 
 * Copyright (c) 1993-2005 LAAS/CNRS
 * All rights reserved.
 *
 * Redistribution and use  in source  and binary  forms,  with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *   1. Redistributions of  source  code must retain the  above copyright
 *      notice, this list of conditions and the following disclaimer.
 *   2. Redistributions in binary form must reproduce the above copyright
 *      notice,  this list of  conditions and the following disclaimer in
 *      the  documentation  and/or  other   materials provided  with  the
 *      distribution.
 *
 * THIS  SOFTWARE IS PROVIDED BY  THE  COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND  ANY  EXPRESS OR IMPLIED  WARRANTIES,  INCLUDING,  BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES  OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR  PURPOSE ARE DISCLAIMED. IN  NO EVENT SHALL THE COPYRIGHT
 * HOLDERS OR      CONTRIBUTORS  BE LIABLE FOR   ANY    DIRECT, INDIRECT,
 * INCIDENTAL,  SPECIAL,  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF  SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN  CONTRACT, STRICT LIABILITY, OR
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE   OF THIS SOFTWARE, EVEN   IF ADVISED OF   THE POSSIBILITY OF SUCH
 * DAMAGE.
 */

/*
 * ----------------------------------------------------------------------
 * WaitForPersonListChange Request
 */

static int icuWaitForPersonListChangeRqstSendCb(ClientData, Tcl_Interp *, int, Tcl_Obj *const []); 
static int icuWaitForPersonListChangeReplyRcvCb(ClientData, Tcl_Interp *, int, Tcl_Obj * const []);

static int
icuWaitForPersonListChangeRqstSendCb(ClientData data, Tcl_Interp *interp,
				int objc, Tcl_Obj *const objv[])
{
   struct ModuleInfo *tclModuleInfo = (struct ModuleInfo *)data;
   int rqstId, bilan;
   char buf[10];
   static ICU_PERSON_LIST_PARAMS paramsPersonList;	/* input */
#if (5 > 0)
   int ret;
   int curObjc = 0;
#endif
   char strerr[64];

   TEST_BAD_USAGE(objc != 5+1);
		 
   ret = Tcl_GetDoubleFromObj(interp, objv[++curObjc],    	&paramsPersonList.seuilReco);
   if (ret != TCL_OK) return TCL_ERROR;
   ret = Tcl_GetIntFromObj(interp, objv[++curObjc],    	&paramsPersonList.absenceWindow);
   if (ret != TCL_OK) return TCL_ERROR;
   ret = Tcl_GetIntFromObj(interp, objv[++curObjc],    	&paramsPersonList.presenceWindow);
   if (ret != TCL_OK) return TCL_ERROR;
   ret = Tcl_GetDoubleFromObj(interp, objv[++curObjc],    	&paramsPersonList.absenceRatio);
   if (ret != TCL_OK) return TCL_ERROR;
   ret = Tcl_GetDoubleFromObj(interp, objv[++curObjc],    	&paramsPersonList.presenceRatio);
   if (ret != TCL_OK) return TCL_ERROR;


   if (tclServRqstSend(interp, Tcl_GetStringFromObj(objv[0], NULL),
		       tclModuleInfo->cid, ICU_WAITFORPERSONLISTCHANGE_RQST,
		       &paramsPersonList, sizeof(paramsPersonList),
		       &rqstId, &bilan) == ERROR) {

     Tcl_SetResult(interp, h2getErrMsg(bilan, strerr, 64), TCL_VOLATILE);
      return TCL_ERROR;
   }
    
   sprintf(buf, " %d", rqstId);
   Tcl_SetResult(interp, "SENT", TCL_STATIC);
   Tcl_AppendResult(interp, buf, NULL);
   return TCL_OK;
}

static int
icuWaitForPersonListChangeReplyRcvCb(ClientData data, Tcl_Interp *interp,
				int objc, Tcl_Obj * const objv[])
{
   struct ModuleInfo *tclModuleInfo = (struct ModuleInfo *)data;
   int ret;
   int rqstId, bilan, activity;
   static ICU_WAITFOR waitFor;	/* output */
   Tcl_Obj *my_own_private_unique_result;
   char strerr[64];

   TEST_BAD_USAGE(
      objc != 2 ||
      Tcl_GetIntFromObj(interp, objv[1], &rqstId) != TCL_OK
   );

   switch (
      /* c'est laid */
      tclServReplyRcv(interp,
		      tclModuleInfo->cid, rqstId, NO_BLOCK,
		      &activity, &waitFor, sizeof(waitFor),
		      &bilan)
      ) {
      case WAITING_INTERMED_REPLY:
	 Tcl_SetResult(interp, "SENT", TCL_STATIC);
	 return TCL_OK;

      case WAITING_FINAL_REPLY: {
	 char tmp[50];
	 sprintf(tmp, "ACK %d", activity);
	 Tcl_SetResult(interp, tmp, TCL_VOLATILE);
	 return TCL_OK;
      }

      case FINAL_REPLY_OK:
	 break;

      default:
	 Tcl_SetResult(interp, h2getErrMsg(bilan, strerr, 64), TCL_VOLATILE);
	 return TCL_ERROR;
   }

   my_own_private_unique_result = Tcl_NewListObj(0, NULL);
   if (my_own_private_unique_result == NULL) return TCL_ERROR;

   ret = Tcl_ListObjAppendElement(interp, my_own_private_unique_result,
				  Tcl_NewStringObj("TERM", -1));
   if (ret != TCL_OK) return TCL_ERROR;

   if (bilan == OK)
      ret = Tcl_ListObjAppendElement(interp, my_own_private_unique_result,
				     Tcl_NewStringObj("OK", -1));
   else
      ret = Tcl_ListObjAppendElement(interp, my_own_private_unique_result,
				     Tcl_NewStringObj(
					h2getErrMsg(bilan, strerr, 64), -1));
   if (ret != TCL_OK) return TCL_ERROR;

   ret = Tcl_ListObjAppendElement(interp, my_own_private_unique_result,
	Tcl_NewStringObj(waitFor.name, -1));
   if (ret != TCL_OK) return TCL_ERROR;
   ret = Tcl_ListObjAppendElement(interp, my_own_private_unique_result,
	Tcl_NewIntObj(waitFor.nbImageMinSomebody));
   if (ret != TCL_OK) return TCL_ERROR;


   Tcl_SetObjResult(interp, my_own_private_unique_result);
   return TCL_OK;
}

/* --- FILE GENERATED WITH GENOM, DO NOT EDIT BY HAND ------------------ */

/* 
 * Copyright (c) 1993-2005 LAAS/CNRS
 * All rights reserved.
 *
 * Redistribution and use  in source  and binary  forms,  with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *   1. Redistributions of  source  code must retain the  above copyright
 *      notice, this list of conditions and the following disclaimer.
 *   2. Redistributions in binary form must reproduce the above copyright
 *      notice,  this list of  conditions and the following disclaimer in
 *      the  documentation  and/or  other   materials provided  with  the
 *      distribution.
 *
 * THIS  SOFTWARE IS PROVIDED BY  THE  COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND  ANY  EXPRESS OR IMPLIED  WARRANTIES,  INCLUDING,  BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES  OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR  PURPOSE ARE DISCLAIMED. IN  NO EVENT SHALL THE COPYRIGHT
 * HOLDERS OR      CONTRIBUTORS  BE LIABLE FOR   ANY    DIRECT, INDIRECT,
 * INCIDENTAL,  SPECIAL,  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF  SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN  CONTRACT, STRICT LIABILITY, OR
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE   OF THIS SOFTWARE, EVEN   IF ADVISED OF   THE POSSIBILITY OF SUCH
 * DAMAGE.
 */

/*
 * ----------------------------------------------------------------------
 * ProcessSetModality Request
 */

static int icuProcessSetModalityRqstSendCb(ClientData, Tcl_Interp *, int, Tcl_Obj *const []); 
static int icuProcessSetModalityReplyRcvCb(ClientData, Tcl_Interp *, int, Tcl_Obj * const []);

static int
icuProcessSetModalityRqstSendCb(ClientData data, Tcl_Interp *interp,
				int objc, Tcl_Obj *const objv[])
{
   struct ModuleInfo *tclModuleInfo = (struct ModuleInfo *)data;
   int rqstId, bilan;
   char buf[10];
   static ICU_MODALITIES setModality;	/* input */
#if (1 > 0)
   int ret;
   int curObjc = 0;
#endif
   char strerr[64];

   TEST_BAD_USAGE(objc != 1+1);
		 
   ret = Tcl_GetEnumFromObj(interp, objv[++curObjc],    	(int *)&setModality,"ICU_NB_MODALITY_MAX", 8, "ICU_FACE_RECOGNITION", 7, "ICU_MOTION_MONITORING", 6, "ICU_BODY_TRACKING", 5, "ICU_FACE_TRACKING", 4, "ICU_MOTION_DETECTION", 3, "ICU_FACE_DETECTION", 2, "ICU_AUTO_SWITCH_MODE", 1, "ICU_NO_PROCESS", 0, NULL);
   if (ret != TCL_OK) return TCL_ERROR;


   if (tclServRqstSend(interp, Tcl_GetStringFromObj(objv[0], NULL),
		       tclModuleInfo->cid, ICU_PROCESSSETMODALITY_RQST,
		       &setModality, sizeof(setModality),
		       &rqstId, &bilan) == ERROR) {

     Tcl_SetResult(interp, h2getErrMsg(bilan, strerr, 64), TCL_VOLATILE);
      return TCL_ERROR;
   }
    
   sprintf(buf, " %d", rqstId);
   Tcl_SetResult(interp, "SENT", TCL_STATIC);
   Tcl_AppendResult(interp, buf, NULL);
   return TCL_OK;
}

static int
icuProcessSetModalityReplyRcvCb(ClientData data, Tcl_Interp *interp,
				int objc, Tcl_Obj * const objv[])
{
   struct ModuleInfo *tclModuleInfo = (struct ModuleInfo *)data;
   int ret;
   int rqstId, bilan, activity;
   /* no output */ 	/* output */
   Tcl_Obj *my_own_private_unique_result;
   char strerr[64];

   TEST_BAD_USAGE(
      objc != 2 ||
      Tcl_GetIntFromObj(interp, objv[1], &rqstId) != TCL_OK
   );

   switch (
      /* c'est laid */
      tclServReplyRcv(interp,
		      tclModuleInfo->cid, rqstId, NO_BLOCK,
		      &activity, NULL, 0,
		      &bilan)
      ) {
      case WAITING_INTERMED_REPLY:
	 Tcl_SetResult(interp, "SENT", TCL_STATIC);
	 return TCL_OK;

      case WAITING_FINAL_REPLY: {
	 char tmp[50];
	 sprintf(tmp, "ACK %d", activity);
	 Tcl_SetResult(interp, tmp, TCL_VOLATILE);
	 return TCL_OK;
      }

      case FINAL_REPLY_OK:
	 break;

      default:
	 Tcl_SetResult(interp, h2getErrMsg(bilan, strerr, 64), TCL_VOLATILE);
	 return TCL_ERROR;
   }

   my_own_private_unique_result = Tcl_NewListObj(0, NULL);
   if (my_own_private_unique_result == NULL) return TCL_ERROR;

   ret = Tcl_ListObjAppendElement(interp, my_own_private_unique_result,
				  Tcl_NewStringObj("TERM", -1));
   if (ret != TCL_OK) return TCL_ERROR;

   if (bilan == OK)
      ret = Tcl_ListObjAppendElement(interp, my_own_private_unique_result,
				     Tcl_NewStringObj("OK", -1));
   else
      ret = Tcl_ListObjAppendElement(interp, my_own_private_unique_result,
				     Tcl_NewStringObj(
					h2getErrMsg(bilan, strerr, 64), -1));
   if (ret != TCL_OK) return TCL_ERROR;



   Tcl_SetObjResult(interp, my_own_private_unique_result);
   return TCL_OK;
}

/* --- FILE GENERATED WITH GENOM, DO NOT EDIT BY HAND ------------------ */

/* 
 * Copyright (c) 1993-2005 LAAS/CNRS
 * All rights reserved.
 *
 * Redistribution and use  in source  and binary  forms,  with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *   1. Redistributions of  source  code must retain the  above copyright
 *      notice, this list of conditions and the following disclaimer.
 *   2. Redistributions in binary form must reproduce the above copyright
 *      notice,  this list of  conditions and the following disclaimer in
 *      the  documentation  and/or  other   materials provided  with  the
 *      distribution.
 *
 * THIS  SOFTWARE IS PROVIDED BY  THE  COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND  ANY  EXPRESS OR IMPLIED  WARRANTIES,  INCLUDING,  BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES  OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR  PURPOSE ARE DISCLAIMED. IN  NO EVENT SHALL THE COPYRIGHT
 * HOLDERS OR      CONTRIBUTORS  BE LIABLE FOR   ANY    DIRECT, INDIRECT,
 * INCIDENTAL,  SPECIAL,  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF  SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN  CONTRACT, STRICT LIABILITY, OR
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE   OF THIS SOFTWARE, EVEN   IF ADVISED OF   THE POSSIBILITY OF SUCH
 * DAMAGE.
 */

/*
 * ----------------------------------------------------------------------
 * FacerecSetWorkingPath Request
 */

static int icuFacerecSetWorkingPathRqstSendCb(ClientData, Tcl_Interp *, int, Tcl_Obj *const []); 
static int icuFacerecSetWorkingPathReplyRcvCb(ClientData, Tcl_Interp *, int, Tcl_Obj * const []);

static int
icuFacerecSetWorkingPathRqstSendCb(ClientData data, Tcl_Interp *interp,
				int objc, Tcl_Obj *const objv[])
{
   struct ModuleInfo *tclModuleInfo = (struct ModuleInfo *)data;
   int rqstId, bilan;
   char buf[10];
   static GEN_STRING256 facerecWorkingPath;	/* input */
#if (1 > 0)
   int ret;
   int curObjc = 0;
#endif
   char strerr[64];

   TEST_BAD_USAGE(objc != 1+1);
		 
   strncpy(facerecWorkingPath.name, 
   	Tcl_GetStringFromObj(objv[++curObjc], NULL),    	256);
   ret = TCL_OK;
   if (ret != TCL_OK) return TCL_ERROR;


   if (tclServRqstSend(interp, Tcl_GetStringFromObj(objv[0], NULL),
		       tclModuleInfo->cid, ICU_FACERECSETWORKINGPATH_RQST,
		       &facerecWorkingPath, sizeof(facerecWorkingPath),
		       &rqstId, &bilan) == ERROR) {

     Tcl_SetResult(interp, h2getErrMsg(bilan, strerr, 64), TCL_VOLATILE);
      return TCL_ERROR;
   }
    
   sprintf(buf, " %d", rqstId);
   Tcl_SetResult(interp, "SENT", TCL_STATIC);
   Tcl_AppendResult(interp, buf, NULL);
   return TCL_OK;
}

static int
icuFacerecSetWorkingPathReplyRcvCb(ClientData data, Tcl_Interp *interp,
				int objc, Tcl_Obj * const objv[])
{
   struct ModuleInfo *tclModuleInfo = (struct ModuleInfo *)data;
   int ret;
   int rqstId, bilan, activity;
   /* no output */ 	/* output */
   Tcl_Obj *my_own_private_unique_result;
   char strerr[64];

   TEST_BAD_USAGE(
      objc != 2 ||
      Tcl_GetIntFromObj(interp, objv[1], &rqstId) != TCL_OK
   );

   switch (
      /* c'est laid */
      tclServReplyRcv(interp,
		      tclModuleInfo->cid, rqstId, NO_BLOCK,
		      &activity, NULL, 0,
		      &bilan)
      ) {
      case WAITING_INTERMED_REPLY:
	 Tcl_SetResult(interp, "SENT", TCL_STATIC);
	 return TCL_OK;

      case WAITING_FINAL_REPLY: {
	 char tmp[50];
	 sprintf(tmp, "ACK %d", activity);
	 Tcl_SetResult(interp, tmp, TCL_VOLATILE);
	 return TCL_OK;
      }

      case FINAL_REPLY_OK:
	 break;

      default:
	 Tcl_SetResult(interp, h2getErrMsg(bilan, strerr, 64), TCL_VOLATILE);
	 return TCL_ERROR;
   }

   my_own_private_unique_result = Tcl_NewListObj(0, NULL);
   if (my_own_private_unique_result == NULL) return TCL_ERROR;

   ret = Tcl_ListObjAppendElement(interp, my_own_private_unique_result,
				  Tcl_NewStringObj("TERM", -1));
   if (ret != TCL_OK) return TCL_ERROR;

   if (bilan == OK)
      ret = Tcl_ListObjAppendElement(interp, my_own_private_unique_result,
				     Tcl_NewStringObj("OK", -1));
   else
      ret = Tcl_ListObjAppendElement(interp, my_own_private_unique_result,
				     Tcl_NewStringObj(
					h2getErrMsg(bilan, strerr, 64), -1));
   if (ret != TCL_OK) return TCL_ERROR;



   Tcl_SetObjResult(interp, my_own_private_unique_result);
   return TCL_OK;
}

/* --- FILE GENERATED WITH GENOM, DO NOT EDIT BY HAND ------------------ */

/* 
 * Copyright (c) 1993-2005 LAAS/CNRS
 * All rights reserved.
 *
 * Redistribution and use  in source  and binary  forms,  with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *   1. Redistributions of  source  code must retain the  above copyright
 *      notice, this list of conditions and the following disclaimer.
 *   2. Redistributions in binary form must reproduce the above copyright
 *      notice,  this list of  conditions and the following disclaimer in
 *      the  documentation  and/or  other   materials provided  with  the
 *      distribution.
 *
 * THIS  SOFTWARE IS PROVIDED BY  THE  COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND  ANY  EXPRESS OR IMPLIED  WARRANTIES,  INCLUDING,  BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES  OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR  PURPOSE ARE DISCLAIMED. IN  NO EVENT SHALL THE COPYRIGHT
 * HOLDERS OR      CONTRIBUTORS  BE LIABLE FOR   ANY    DIRECT, INDIRECT,
 * INCIDENTAL,  SPECIAL,  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF  SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN  CONTRACT, STRICT LIABILITY, OR
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE   OF THIS SOFTWARE, EVEN   IF ADVISED OF   THE POSSIBILITY OF SUCH
 * DAMAGE.
 */

/*
 * ----------------------------------------------------------------------
 * FacerecSetParams Request
 */

static int icuFacerecSetParamsRqstSendCb(ClientData, Tcl_Interp *, int, Tcl_Obj *const []); 
static int icuFacerecSetParamsReplyRcvCb(ClientData, Tcl_Interp *, int, Tcl_Obj * const []);

static int
icuFacerecSetParamsRqstSendCb(ClientData data, Tcl_Interp *interp,
				int objc, Tcl_Obj *const objv[])
{
   struct ModuleInfo *tclModuleInfo = (struct ModuleInfo *)data;
   int rqstId, bilan;
   char buf[10];
   static ICU_PERSON_LIST_PARAMS paramsPersonList;	/* input */
#if (5 > 0)
   int ret;
   int curObjc = 0;
#endif
   char strerr[64];

   TEST_BAD_USAGE(objc != 5+1);
		 
   ret = Tcl_GetDoubleFromObj(interp, objv[++curObjc],    	&paramsPersonList.seuilReco);
   if (ret != TCL_OK) return TCL_ERROR;
   ret = Tcl_GetIntFromObj(interp, objv[++curObjc],    	&paramsPersonList.absenceWindow);
   if (ret != TCL_OK) return TCL_ERROR;
   ret = Tcl_GetIntFromObj(interp, objv[++curObjc],    	&paramsPersonList.presenceWindow);
   if (ret != TCL_OK) return TCL_ERROR;
   ret = Tcl_GetDoubleFromObj(interp, objv[++curObjc],    	&paramsPersonList.absenceRatio);
   if (ret != TCL_OK) return TCL_ERROR;
   ret = Tcl_GetDoubleFromObj(interp, objv[++curObjc],    	&paramsPersonList.presenceRatio);
   if (ret != TCL_OK) return TCL_ERROR;


   if (tclServRqstSend(interp, Tcl_GetStringFromObj(objv[0], NULL),
		       tclModuleInfo->cid, ICU_FACERECSETPARAMS_RQST,
		       &paramsPersonList, sizeof(paramsPersonList),
		       &rqstId, &bilan) == ERROR) {

     Tcl_SetResult(interp, h2getErrMsg(bilan, strerr, 64), TCL_VOLATILE);
      return TCL_ERROR;
   }
    
   sprintf(buf, " %d", rqstId);
   Tcl_SetResult(interp, "SENT", TCL_STATIC);
   Tcl_AppendResult(interp, buf, NULL);
   return TCL_OK;
}

static int
icuFacerecSetParamsReplyRcvCb(ClientData data, Tcl_Interp *interp,
				int objc, Tcl_Obj * const objv[])
{
   struct ModuleInfo *tclModuleInfo = (struct ModuleInfo *)data;
   int ret;
   int rqstId, bilan, activity;
   /* no output */ 	/* output */
   Tcl_Obj *my_own_private_unique_result;
   char strerr[64];

   TEST_BAD_USAGE(
      objc != 2 ||
      Tcl_GetIntFromObj(interp, objv[1], &rqstId) != TCL_OK
   );

   switch (
      /* c'est laid */
      tclServReplyRcv(interp,
		      tclModuleInfo->cid, rqstId, NO_BLOCK,
		      &activity, NULL, 0,
		      &bilan)
      ) {
      case WAITING_INTERMED_REPLY:
	 Tcl_SetResult(interp, "SENT", TCL_STATIC);
	 return TCL_OK;

      case WAITING_FINAL_REPLY: {
	 char tmp[50];
	 sprintf(tmp, "ACK %d", activity);
	 Tcl_SetResult(interp, tmp, TCL_VOLATILE);
	 return TCL_OK;
      }

      case FINAL_REPLY_OK:
	 break;

      default:
	 Tcl_SetResult(interp, h2getErrMsg(bilan, strerr, 64), TCL_VOLATILE);
	 return TCL_ERROR;
   }

   my_own_private_unique_result = Tcl_NewListObj(0, NULL);
   if (my_own_private_unique_result == NULL) return TCL_ERROR;

   ret = Tcl_ListObjAppendElement(interp, my_own_private_unique_result,
				  Tcl_NewStringObj("TERM", -1));
   if (ret != TCL_OK) return TCL_ERROR;

   if (bilan == OK)
      ret = Tcl_ListObjAppendElement(interp, my_own_private_unique_result,
				     Tcl_NewStringObj("OK", -1));
   else
      ret = Tcl_ListObjAppendElement(interp, my_own_private_unique_result,
				     Tcl_NewStringObj(
					h2getErrMsg(bilan, strerr, 64), -1));
   if (ret != TCL_OK) return TCL_ERROR;



   Tcl_SetObjResult(interp, my_own_private_unique_result);
   return TCL_OK;
}

/* --- FILE GENERATED WITH GENOM, DO NOT EDIT BY HAND ------------------ */

/* 
 * Copyright (c) 1993-2005 LAAS/CNRS
 * All rights reserved.
 *
 * Redistribution and use  in source  and binary  forms,  with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *   1. Redistributions of  source  code must retain the  above copyright
 *      notice, this list of conditions and the following disclaimer.
 *   2. Redistributions in binary form must reproduce the above copyright
 *      notice,  this list of  conditions and the following disclaimer in
 *      the  documentation  and/or  other   materials provided  with  the
 *      distribution.
 *
 * THIS  SOFTWARE IS PROVIDED BY  THE  COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND  ANY  EXPRESS OR IMPLIED  WARRANTIES,  INCLUDING,  BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES  OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR  PURPOSE ARE DISCLAIMED. IN  NO EVENT SHALL THE COPYRIGHT
 * HOLDERS OR      CONTRIBUTORS  BE LIABLE FOR   ANY    DIRECT, INDIRECT,
 * INCIDENTAL,  SPECIAL,  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF  SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN  CONTRACT, STRICT LIABILITY, OR
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE   OF THIS SOFTWARE, EVEN   IF ADVISED OF   THE POSSIBILITY OF SUCH
 * DAMAGE.
 */

/*
 * ----------------------------------------------------------------------
 * FacerecInit Request
 */

static int icuFacerecInitRqstSendCb(ClientData, Tcl_Interp *, int, Tcl_Obj *const []); 
static int icuFacerecInitReplyRcvCb(ClientData, Tcl_Interp *, int, Tcl_Obj * const []);

static int
icuFacerecInitRqstSendCb(ClientData data, Tcl_Interp *interp,
				int objc, Tcl_Obj *const objv[])
{
   struct ModuleInfo *tclModuleInfo = (struct ModuleInfo *)data;
   int rqstId, bilan;
   char buf[10];
   /* no input */ 	/* input */
#if (0 > 0)
   int ret;
   int curObjc = 0;
#endif
   char strerr[64];

   TEST_BAD_USAGE(objc != 0+1);
		 


   if (tclServRqstSend(interp, Tcl_GetStringFromObj(objv[0], NULL),
		       tclModuleInfo->cid, ICU_FACERECINIT_RQST,
		       NULL, 0,
		       &rqstId, &bilan) == ERROR) {

     Tcl_SetResult(interp, h2getErrMsg(bilan, strerr, 64), TCL_VOLATILE);
      return TCL_ERROR;
   }
    
   sprintf(buf, " %d", rqstId);
   Tcl_SetResult(interp, "SENT", TCL_STATIC);
   Tcl_AppendResult(interp, buf, NULL);
   return TCL_OK;
}

static int
icuFacerecInitReplyRcvCb(ClientData data, Tcl_Interp *interp,
				int objc, Tcl_Obj * const objv[])
{
   struct ModuleInfo *tclModuleInfo = (struct ModuleInfo *)data;
   int ret;
   int rqstId, bilan, activity;
   /* no output */ 	/* output */
   Tcl_Obj *my_own_private_unique_result;
   char strerr[64];

   TEST_BAD_USAGE(
      objc != 2 ||
      Tcl_GetIntFromObj(interp, objv[1], &rqstId) != TCL_OK
   );

   switch (
      /* c'est laid */
      tclServReplyRcv(interp,
		      tclModuleInfo->cid, rqstId, NO_BLOCK,
		      &activity, NULL, 0,
		      &bilan)
      ) {
      case WAITING_INTERMED_REPLY:
	 Tcl_SetResult(interp, "SENT", TCL_STATIC);
	 return TCL_OK;

      case WAITING_FINAL_REPLY: {
	 char tmp[50];
	 sprintf(tmp, "ACK %d", activity);
	 Tcl_SetResult(interp, tmp, TCL_VOLATILE);
	 return TCL_OK;
      }

      case FINAL_REPLY_OK:
	 break;

      default:
	 Tcl_SetResult(interp, h2getErrMsg(bilan, strerr, 64), TCL_VOLATILE);
	 return TCL_ERROR;
   }

   my_own_private_unique_result = Tcl_NewListObj(0, NULL);
   if (my_own_private_unique_result == NULL) return TCL_ERROR;

   ret = Tcl_ListObjAppendElement(interp, my_own_private_unique_result,
				  Tcl_NewStringObj("TERM", -1));
   if (ret != TCL_OK) return TCL_ERROR;

   if (bilan == OK)
      ret = Tcl_ListObjAppendElement(interp, my_own_private_unique_result,
				     Tcl_NewStringObj("OK", -1));
   else
      ret = Tcl_ListObjAppendElement(interp, my_own_private_unique_result,
				     Tcl_NewStringObj(
					h2getErrMsg(bilan, strerr, 64), -1));
   if (ret != TCL_OK) return TCL_ERROR;



   Tcl_SetObjResult(interp, my_own_private_unique_result);
   return TCL_OK;
}

/* --- FILE GENERATED WITH GENOM, DO NOT EDIT BY HAND ------------------ */

/* 
 * Copyright (c) 1993-2005 LAAS/CNRS
 * All rights reserved.
 *
 * Redistribution and use  in source  and binary  forms,  with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *   1. Redistributions of  source  code must retain the  above copyright
 *      notice, this list of conditions and the following disclaimer.
 *   2. Redistributions in binary form must reproduce the above copyright
 *      notice,  this list of  conditions and the following disclaimer in
 *      the  documentation  and/or  other   materials provided  with  the
 *      distribution.
 *
 * THIS  SOFTWARE IS PROVIDED BY  THE  COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND  ANY  EXPRESS OR IMPLIED  WARRANTIES,  INCLUDING,  BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES  OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR  PURPOSE ARE DISCLAIMED. IN  NO EVENT SHALL THE COPYRIGHT
 * HOLDERS OR      CONTRIBUTORS  BE LIABLE FOR   ANY    DIRECT, INDIRECT,
 * INCIDENTAL,  SPECIAL,  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF  SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN  CONTRACT, STRICT LIABILITY, OR
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE   OF THIS SOFTWARE, EVEN   IF ADVISED OF   THE POSSIBILITY OF SUCH
 * DAMAGE.
 */

/*
 * ----------------------------------------------------------------------
 * FacerecAddSomebody Request
 */

static int icuFacerecAddSomebodyRqstSendCb(ClientData, Tcl_Interp *, int, Tcl_Obj *const []); 
static int icuFacerecAddSomebodyReplyRcvCb(ClientData, Tcl_Interp *, int, Tcl_Obj * const []);

static int
icuFacerecAddSomebodyRqstSendCb(ClientData data, Tcl_Interp *interp,
				int objc, Tcl_Obj *const objv[])
{
   struct ModuleInfo *tclModuleInfo = (struct ModuleInfo *)data;
   int rqstId, bilan;
   char buf[10];
   static GEN_STRING256 recoAddName;	/* input */
#if (1 > 0)
   int ret;
   int curObjc = 0;
#endif
   char strerr[64];

   TEST_BAD_USAGE(objc != 1+1);
		 
   strncpy(recoAddName.name, 
   	Tcl_GetStringFromObj(objv[++curObjc], NULL),    	256);
   ret = TCL_OK;
   if (ret != TCL_OK) return TCL_ERROR;


   if (tclServRqstSend(interp, Tcl_GetStringFromObj(objv[0], NULL),
		       tclModuleInfo->cid, ICU_FACERECADDSOMEBODY_RQST,
		       &recoAddName, sizeof(recoAddName),
		       &rqstId, &bilan) == ERROR) {

     Tcl_SetResult(interp, h2getErrMsg(bilan, strerr, 64), TCL_VOLATILE);
      return TCL_ERROR;
   }
    
   sprintf(buf, " %d", rqstId);
   Tcl_SetResult(interp, "SENT", TCL_STATIC);
   Tcl_AppendResult(interp, buf, NULL);
   return TCL_OK;
}

static int
icuFacerecAddSomebodyReplyRcvCb(ClientData data, Tcl_Interp *interp,
				int objc, Tcl_Obj * const objv[])
{
   struct ModuleInfo *tclModuleInfo = (struct ModuleInfo *)data;
   int ret;
   int rqstId, bilan, activity;
   /* no output */ 	/* output */
   Tcl_Obj *my_own_private_unique_result;
   char strerr[64];

   TEST_BAD_USAGE(
      objc != 2 ||
      Tcl_GetIntFromObj(interp, objv[1], &rqstId) != TCL_OK
   );

   switch (
      /* c'est laid */
      tclServReplyRcv(interp,
		      tclModuleInfo->cid, rqstId, NO_BLOCK,
		      &activity, NULL, 0,
		      &bilan)
      ) {
      case WAITING_INTERMED_REPLY:
	 Tcl_SetResult(interp, "SENT", TCL_STATIC);
	 return TCL_OK;

      case WAITING_FINAL_REPLY: {
	 char tmp[50];
	 sprintf(tmp, "ACK %d", activity);
	 Tcl_SetResult(interp, tmp, TCL_VOLATILE);
	 return TCL_OK;
      }

      case FINAL_REPLY_OK:
	 break;

      default:
	 Tcl_SetResult(interp, h2getErrMsg(bilan, strerr, 64), TCL_VOLATILE);
	 return TCL_ERROR;
   }

   my_own_private_unique_result = Tcl_NewListObj(0, NULL);
   if (my_own_private_unique_result == NULL) return TCL_ERROR;

   ret = Tcl_ListObjAppendElement(interp, my_own_private_unique_result,
				  Tcl_NewStringObj("TERM", -1));
   if (ret != TCL_OK) return TCL_ERROR;

   if (bilan == OK)
      ret = Tcl_ListObjAppendElement(interp, my_own_private_unique_result,
				     Tcl_NewStringObj("OK", -1));
   else
      ret = Tcl_ListObjAppendElement(interp, my_own_private_unique_result,
				     Tcl_NewStringObj(
					h2getErrMsg(bilan, strerr, 64), -1));
   if (ret != TCL_OK) return TCL_ERROR;



   Tcl_SetObjResult(interp, my_own_private_unique_result);
   return TCL_OK;
}

/* --- FILE GENERATED WITH GENOM, DO NOT EDIT BY HAND ------------------ */

/* 
 * Copyright (c) 1993-2005 LAAS/CNRS
 * All rights reserved.
 *
 * Redistribution and use  in source  and binary  forms,  with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *   1. Redistributions of  source  code must retain the  above copyright
 *      notice, this list of conditions and the following disclaimer.
 *   2. Redistributions in binary form must reproduce the above copyright
 *      notice,  this list of  conditions and the following disclaimer in
 *      the  documentation  and/or  other   materials provided  with  the
 *      distribution.
 *
 * THIS  SOFTWARE IS PROVIDED BY  THE  COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND  ANY  EXPRESS OR IMPLIED  WARRANTIES,  INCLUDING,  BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES  OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR  PURPOSE ARE DISCLAIMED. IN  NO EVENT SHALL THE COPYRIGHT
 * HOLDERS OR      CONTRIBUTORS  BE LIABLE FOR   ANY    DIRECT, INDIRECT,
 * INCIDENTAL,  SPECIAL,  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF  SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN  CONTRACT, STRICT LIABILITY, OR
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE   OF THIS SOFTWARE, EVEN   IF ADVISED OF   THE POSSIBILITY OF SUCH
 * DAMAGE.
 */

/*
 * ----------------------------------------------------------------------
 * FacerecSetPeople Request
 */

static int icuFacerecSetPeopleRqstSendCb(ClientData, Tcl_Interp *, int, Tcl_Obj *const []); 
static int icuFacerecSetPeopleReplyRcvCb(ClientData, Tcl_Interp *, int, Tcl_Obj * const []);

static int
icuFacerecSetPeopleRqstSendCb(ClientData data, Tcl_Interp *interp,
				int objc, Tcl_Obj *const objv[])
{
   struct ModuleInfo *tclModuleInfo = (struct ModuleInfo *)data;
   int rqstId, bilan;
   char buf[10];
   static GEN_STRING256 recoName;	/* input */
#if (1 > 0)
   int ret;
   int curObjc = 0;
#endif
   char strerr[64];

   TEST_BAD_USAGE(objc != 1+1);
		 
   strncpy(recoName.name, 
   	Tcl_GetStringFromObj(objv[++curObjc], NULL),    	256);
   ret = TCL_OK;
   if (ret != TCL_OK) return TCL_ERROR;


   if (tclServRqstSend(interp, Tcl_GetStringFromObj(objv[0], NULL),
		       tclModuleInfo->cid, ICU_FACERECSETPEOPLE_RQST,
		       &recoName, sizeof(recoName),
		       &rqstId, &bilan) == ERROR) {

     Tcl_SetResult(interp, h2getErrMsg(bilan, strerr, 64), TCL_VOLATILE);
      return TCL_ERROR;
   }
    
   sprintf(buf, " %d", rqstId);
   Tcl_SetResult(interp, "SENT", TCL_STATIC);
   Tcl_AppendResult(interp, buf, NULL);
   return TCL_OK;
}

static int
icuFacerecSetPeopleReplyRcvCb(ClientData data, Tcl_Interp *interp,
				int objc, Tcl_Obj * const objv[])
{
   struct ModuleInfo *tclModuleInfo = (struct ModuleInfo *)data;
   int ret;
   int rqstId, bilan, activity;
   /* no output */ 	/* output */
   Tcl_Obj *my_own_private_unique_result;
   char strerr[64];

   TEST_BAD_USAGE(
      objc != 2 ||
      Tcl_GetIntFromObj(interp, objv[1], &rqstId) != TCL_OK
   );

   switch (
      /* c'est laid */
      tclServReplyRcv(interp,
		      tclModuleInfo->cid, rqstId, NO_BLOCK,
		      &activity, NULL, 0,
		      &bilan)
      ) {
      case WAITING_INTERMED_REPLY:
	 Tcl_SetResult(interp, "SENT", TCL_STATIC);
	 return TCL_OK;

      case WAITING_FINAL_REPLY: {
	 char tmp[50];
	 sprintf(tmp, "ACK %d", activity);
	 Tcl_SetResult(interp, tmp, TCL_VOLATILE);
	 return TCL_OK;
      }

      case FINAL_REPLY_OK:
	 break;

      default:
	 Tcl_SetResult(interp, h2getErrMsg(bilan, strerr, 64), TCL_VOLATILE);
	 return TCL_ERROR;
   }

   my_own_private_unique_result = Tcl_NewListObj(0, NULL);
   if (my_own_private_unique_result == NULL) return TCL_ERROR;

   ret = Tcl_ListObjAppendElement(interp, my_own_private_unique_result,
				  Tcl_NewStringObj("TERM", -1));
   if (ret != TCL_OK) return TCL_ERROR;

   if (bilan == OK)
      ret = Tcl_ListObjAppendElement(interp, my_own_private_unique_result,
				     Tcl_NewStringObj("OK", -1));
   else
      ret = Tcl_ListObjAppendElement(interp, my_own_private_unique_result,
				     Tcl_NewStringObj(
					h2getErrMsg(bilan, strerr, 64), -1));
   if (ret != TCL_OK) return TCL_ERROR;



   Tcl_SetObjResult(interp, my_own_private_unique_result);
   return TCL_OK;
}

/* --- FILE GENERATED WITH GENOM, DO NOT EDIT BY HAND ------------------ */

/* 
 * Copyright (c) 1993-2005 LAAS/CNRS
 * All rights reserved.
 *
 * Redistribution and use  in source  and binary  forms,  with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *   1. Redistributions of  source  code must retain the  above copyright
 *      notice, this list of conditions and the following disclaimer.
 *   2. Redistributions in binary form must reproduce the above copyright
 *      notice,  this list of  conditions and the following disclaimer in
 *      the  documentation  and/or  other   materials provided  with  the
 *      distribution.
 *
 * THIS  SOFTWARE IS PROVIDED BY  THE  COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND  ANY  EXPRESS OR IMPLIED  WARRANTIES,  INCLUDING,  BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES  OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR  PURPOSE ARE DISCLAIMED. IN  NO EVENT SHALL THE COPYRIGHT
 * HOLDERS OR      CONTRIBUTORS  BE LIABLE FOR   ANY    DIRECT, INDIRECT,
 * INCIDENTAL,  SPECIAL,  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF  SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN  CONTRACT, STRICT LIABILITY, OR
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE   OF THIS SOFTWARE, EVEN   IF ADVISED OF   THE POSSIBILITY OF SUCH
 * DAMAGE.
 */

/*
 * ----------------------------------------------------------------------
 * FacerecRecognizeAll Request
 */

static int icuFacerecRecognizeAllRqstSendCb(ClientData, Tcl_Interp *, int, Tcl_Obj *const []); 
static int icuFacerecRecognizeAllReplyRcvCb(ClientData, Tcl_Interp *, int, Tcl_Obj * const []);

static int
icuFacerecRecognizeAllRqstSendCb(ClientData data, Tcl_Interp *interp,
				int objc, Tcl_Obj *const objv[])
{
   struct ModuleInfo *tclModuleInfo = (struct ModuleInfo *)data;
   int rqstId, bilan;
   char buf[10];
   /* no input */ 	/* input */
#if (0 > 0)
   int ret;
   int curObjc = 0;
#endif
   char strerr[64];

   TEST_BAD_USAGE(objc != 0+1);
		 


   if (tclServRqstSend(interp, Tcl_GetStringFromObj(objv[0], NULL),
		       tclModuleInfo->cid, ICU_FACERECRECOGNIZEALL_RQST,
		       NULL, 0,
		       &rqstId, &bilan) == ERROR) {

     Tcl_SetResult(interp, h2getErrMsg(bilan, strerr, 64), TCL_VOLATILE);
      return TCL_ERROR;
   }
    
   sprintf(buf, " %d", rqstId);
   Tcl_SetResult(interp, "SENT", TCL_STATIC);
   Tcl_AppendResult(interp, buf, NULL);
   return TCL_OK;
}

static int
icuFacerecRecognizeAllReplyRcvCb(ClientData data, Tcl_Interp *interp,
				int objc, Tcl_Obj * const objv[])
{
   struct ModuleInfo *tclModuleInfo = (struct ModuleInfo *)data;
   int ret;
   int rqstId, bilan, activity;
   /* no output */ 	/* output */
   Tcl_Obj *my_own_private_unique_result;
   char strerr[64];

   TEST_BAD_USAGE(
      objc != 2 ||
      Tcl_GetIntFromObj(interp, objv[1], &rqstId) != TCL_OK
   );

   switch (
      /* c'est laid */
      tclServReplyRcv(interp,
		      tclModuleInfo->cid, rqstId, NO_BLOCK,
		      &activity, NULL, 0,
		      &bilan)
      ) {
      case WAITING_INTERMED_REPLY:
	 Tcl_SetResult(interp, "SENT", TCL_STATIC);
	 return TCL_OK;

      case WAITING_FINAL_REPLY: {
	 char tmp[50];
	 sprintf(tmp, "ACK %d", activity);
	 Tcl_SetResult(interp, tmp, TCL_VOLATILE);
	 return TCL_OK;
      }

      case FINAL_REPLY_OK:
	 break;

      default:
	 Tcl_SetResult(interp, h2getErrMsg(bilan, strerr, 64), TCL_VOLATILE);
	 return TCL_ERROR;
   }

   my_own_private_unique_result = Tcl_NewListObj(0, NULL);
   if (my_own_private_unique_result == NULL) return TCL_ERROR;

   ret = Tcl_ListObjAppendElement(interp, my_own_private_unique_result,
				  Tcl_NewStringObj("TERM", -1));
   if (ret != TCL_OK) return TCL_ERROR;

   if (bilan == OK)
      ret = Tcl_ListObjAppendElement(interp, my_own_private_unique_result,
				     Tcl_NewStringObj("OK", -1));
   else
      ret = Tcl_ListObjAppendElement(interp, my_own_private_unique_result,
				     Tcl_NewStringObj(
					h2getErrMsg(bilan, strerr, 64), -1));
   if (ret != TCL_OK) return TCL_ERROR;



   Tcl_SetObjResult(interp, my_own_private_unique_result);
   return TCL_OK;
}

/* --- FILE GENERATED WITH GENOM, DO NOT EDIT BY HAND ------------------ */

/* 
 * Copyright (c) 1993-2005 LAAS/CNRS
 * All rights reserved.
 *
 * Redistribution and use  in source  and binary  forms,  with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *   1. Redistributions of  source  code must retain the  above copyright
 *      notice, this list of conditions and the following disclaimer.
 *   2. Redistributions in binary form must reproduce the above copyright
 *      notice,  this list of  conditions and the following disclaimer in
 *      the  documentation  and/or  other   materials provided  with  the
 *      distribution.
 *
 * THIS  SOFTWARE IS PROVIDED BY  THE  COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND  ANY  EXPRESS OR IMPLIED  WARRANTIES,  INCLUDING,  BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES  OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR  PURPOSE ARE DISCLAIMED. IN  NO EVENT SHALL THE COPYRIGHT
 * HOLDERS OR      CONTRIBUTORS  BE LIABLE FOR   ANY    DIRECT, INDIRECT,
 * INCIDENTAL,  SPECIAL,  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF  SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN  CONTRACT, STRICT LIABILITY, OR
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE   OF THIS SOFTWARE, EVEN   IF ADVISED OF   THE POSSIBILITY OF SUCH
 * DAMAGE.
 */

/*
 * ----------------------------------------------------------------------
 * FacerecLoadSomebody Request
 */

static int icuFacerecLoadSomebodyRqstSendCb(ClientData, Tcl_Interp *, int, Tcl_Obj *const []); 
static int icuFacerecLoadSomebodyReplyRcvCb(ClientData, Tcl_Interp *, int, Tcl_Obj * const []);

static int
icuFacerecLoadSomebodyRqstSendCb(ClientData data, Tcl_Interp *interp,
				int objc, Tcl_Obj *const objv[])
{
   struct ModuleInfo *tclModuleInfo = (struct ModuleInfo *)data;
   int rqstId, bilan;
   char buf[10];
   static GEN_STRING256 recoName;	/* input */
#if (1 > 0)
   int ret;
   int curObjc = 0;
#endif
   char strerr[64];

   TEST_BAD_USAGE(objc != 1+1);
		 
   strncpy(recoName.name, 
   	Tcl_GetStringFromObj(objv[++curObjc], NULL),    	256);
   ret = TCL_OK;
   if (ret != TCL_OK) return TCL_ERROR;


   if (tclServRqstSend(interp, Tcl_GetStringFromObj(objv[0], NULL),
		       tclModuleInfo->cid, ICU_FACERECLOADSOMEBODY_RQST,
		       &recoName, sizeof(recoName),
		       &rqstId, &bilan) == ERROR) {

     Tcl_SetResult(interp, h2getErrMsg(bilan, strerr, 64), TCL_VOLATILE);
      return TCL_ERROR;
   }
    
   sprintf(buf, " %d", rqstId);
   Tcl_SetResult(interp, "SENT", TCL_STATIC);
   Tcl_AppendResult(interp, buf, NULL);
   return TCL_OK;
}

static int
icuFacerecLoadSomebodyReplyRcvCb(ClientData data, Tcl_Interp *interp,
				int objc, Tcl_Obj * const objv[])
{
   struct ModuleInfo *tclModuleInfo = (struct ModuleInfo *)data;
   int ret;
   int rqstId, bilan, activity;
   /* no output */ 	/* output */
   Tcl_Obj *my_own_private_unique_result;
   char strerr[64];

   TEST_BAD_USAGE(
      objc != 2 ||
      Tcl_GetIntFromObj(interp, objv[1], &rqstId) != TCL_OK
   );

   switch (
      /* c'est laid */
      tclServReplyRcv(interp,
		      tclModuleInfo->cid, rqstId, NO_BLOCK,
		      &activity, NULL, 0,
		      &bilan)
      ) {
      case WAITING_INTERMED_REPLY:
	 Tcl_SetResult(interp, "SENT", TCL_STATIC);
	 return TCL_OK;

      case WAITING_FINAL_REPLY: {
	 char tmp[50];
	 sprintf(tmp, "ACK %d", activity);
	 Tcl_SetResult(interp, tmp, TCL_VOLATILE);
	 return TCL_OK;
      }

      case FINAL_REPLY_OK:
	 break;

      default:
	 Tcl_SetResult(interp, h2getErrMsg(bilan, strerr, 64), TCL_VOLATILE);
	 return TCL_ERROR;
   }

   my_own_private_unique_result = Tcl_NewListObj(0, NULL);
   if (my_own_private_unique_result == NULL) return TCL_ERROR;

   ret = Tcl_ListObjAppendElement(interp, my_own_private_unique_result,
				  Tcl_NewStringObj("TERM", -1));
   if (ret != TCL_OK) return TCL_ERROR;

   if (bilan == OK)
      ret = Tcl_ListObjAppendElement(interp, my_own_private_unique_result,
				     Tcl_NewStringObj("OK", -1));
   else
      ret = Tcl_ListObjAppendElement(interp, my_own_private_unique_result,
				     Tcl_NewStringObj(
					h2getErrMsg(bilan, strerr, 64), -1));
   if (ret != TCL_OK) return TCL_ERROR;



   Tcl_SetObjResult(interp, my_own_private_unique_result);
   return TCL_OK;
}

/* --- FILE GENERATED WITH GENOM, DO NOT EDIT BY HAND ------------------ */

/* 
 * Copyright (c) 1993-2005 LAAS/CNRS
 * All rights reserved.
 *
 * Redistribution and use  in source  and binary  forms,  with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *   1. Redistributions of  source  code must retain the  above copyright
 *      notice, this list of conditions and the following disclaimer.
 *   2. Redistributions in binary form must reproduce the above copyright
 *      notice,  this list of  conditions and the following disclaimer in
 *      the  documentation  and/or  other   materials provided  with  the
 *      distribution.
 *
 * THIS  SOFTWARE IS PROVIDED BY  THE  COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND  ANY  EXPRESS OR IMPLIED  WARRANTIES,  INCLUDING,  BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES  OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR  PURPOSE ARE DISCLAIMED. IN  NO EVENT SHALL THE COPYRIGHT
 * HOLDERS OR      CONTRIBUTORS  BE LIABLE FOR   ANY    DIRECT, INDIRECT,
 * INCIDENTAL,  SPECIAL,  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF  SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN  CONTRACT, STRICT LIABILITY, OR
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE   OF THIS SOFTWARE, EVEN   IF ADVISED OF   THE POSSIBILITY OF SUCH
 * DAMAGE.
 */

/*
 * ----------------------------------------------------------------------
 * FacerecUnloadSomebody Request
 */

static int icuFacerecUnloadSomebodyRqstSendCb(ClientData, Tcl_Interp *, int, Tcl_Obj *const []); 
static int icuFacerecUnloadSomebodyReplyRcvCb(ClientData, Tcl_Interp *, int, Tcl_Obj * const []);

static int
icuFacerecUnloadSomebodyRqstSendCb(ClientData data, Tcl_Interp *interp,
				int objc, Tcl_Obj *const objv[])
{
   struct ModuleInfo *tclModuleInfo = (struct ModuleInfo *)data;
   int rqstId, bilan;
   char buf[10];
   static GEN_STRING256 recoName;	/* input */
#if (1 > 0)
   int ret;
   int curObjc = 0;
#endif
   char strerr[64];

   TEST_BAD_USAGE(objc != 1+1);
		 
   strncpy(recoName.name, 
   	Tcl_GetStringFromObj(objv[++curObjc], NULL),    	256);
   ret = TCL_OK;
   if (ret != TCL_OK) return TCL_ERROR;


   if (tclServRqstSend(interp, Tcl_GetStringFromObj(objv[0], NULL),
		       tclModuleInfo->cid, ICU_FACERECUNLOADSOMEBODY_RQST,
		       &recoName, sizeof(recoName),
		       &rqstId, &bilan) == ERROR) {

     Tcl_SetResult(interp, h2getErrMsg(bilan, strerr, 64), TCL_VOLATILE);
      return TCL_ERROR;
   }
    
   sprintf(buf, " %d", rqstId);
   Tcl_SetResult(interp, "SENT", TCL_STATIC);
   Tcl_AppendResult(interp, buf, NULL);
   return TCL_OK;
}

static int
icuFacerecUnloadSomebodyReplyRcvCb(ClientData data, Tcl_Interp *interp,
				int objc, Tcl_Obj * const objv[])
{
   struct ModuleInfo *tclModuleInfo = (struct ModuleInfo *)data;
   int ret;
   int rqstId, bilan, activity;
   /* no output */ 	/* output */
   Tcl_Obj *my_own_private_unique_result;
   char strerr[64];

   TEST_BAD_USAGE(
      objc != 2 ||
      Tcl_GetIntFromObj(interp, objv[1], &rqstId) != TCL_OK
   );

   switch (
      /* c'est laid */
      tclServReplyRcv(interp,
		      tclModuleInfo->cid, rqstId, NO_BLOCK,
		      &activity, NULL, 0,
		      &bilan)
      ) {
      case WAITING_INTERMED_REPLY:
	 Tcl_SetResult(interp, "SENT", TCL_STATIC);
	 return TCL_OK;

      case WAITING_FINAL_REPLY: {
	 char tmp[50];
	 sprintf(tmp, "ACK %d", activity);
	 Tcl_SetResult(interp, tmp, TCL_VOLATILE);
	 return TCL_OK;
      }

      case FINAL_REPLY_OK:
	 break;

      default:
	 Tcl_SetResult(interp, h2getErrMsg(bilan, strerr, 64), TCL_VOLATILE);
	 return TCL_ERROR;
   }

   my_own_private_unique_result = Tcl_NewListObj(0, NULL);
   if (my_own_private_unique_result == NULL) return TCL_ERROR;

   ret = Tcl_ListObjAppendElement(interp, my_own_private_unique_result,
				  Tcl_NewStringObj("TERM", -1));
   if (ret != TCL_OK) return TCL_ERROR;

   if (bilan == OK)
      ret = Tcl_ListObjAppendElement(interp, my_own_private_unique_result,
				     Tcl_NewStringObj("OK", -1));
   else
      ret = Tcl_ListObjAppendElement(interp, my_own_private_unique_result,
				     Tcl_NewStringObj(
					h2getErrMsg(bilan, strerr, 64), -1));
   if (ret != TCL_OK) return TCL_ERROR;



   Tcl_SetObjResult(interp, my_own_private_unique_result);
   return TCL_OK;
}

/* --- FILE GENERATED WITH GENOM, DO NOT EDIT BY HAND ------------------ */

/* 
 * Copyright (c) 1993-2005 LAAS/CNRS
 * All rights reserved.
 *
 * Redistribution and use  in source  and binary  forms,  with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *   1. Redistributions of  source  code must retain the  above copyright
 *      notice, this list of conditions and the following disclaimer.
 *   2. Redistributions in binary form must reproduce the above copyright
 *      notice,  this list of  conditions and the following disclaimer in
 *      the  documentation  and/or  other   materials provided  with  the
 *      distribution.
 *
 * THIS  SOFTWARE IS PROVIDED BY  THE  COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND  ANY  EXPRESS OR IMPLIED  WARRANTIES,  INCLUDING,  BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES  OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR  PURPOSE ARE DISCLAIMED. IN  NO EVENT SHALL THE COPYRIGHT
 * HOLDERS OR      CONTRIBUTORS  BE LIABLE FOR   ANY    DIRECT, INDIRECT,
 * INCIDENTAL,  SPECIAL,  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF  SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN  CONTRACT, STRICT LIABILITY, OR
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE   OF THIS SOFTWARE, EVEN   IF ADVISED OF   THE POSSIBILITY OF SUCH
 * DAMAGE.
 */

/*
 * ----------------------------------------------------------------------
 * FacerecLoadEverybody Request
 */

static int icuFacerecLoadEverybodyRqstSendCb(ClientData, Tcl_Interp *, int, Tcl_Obj *const []); 
static int icuFacerecLoadEverybodyReplyRcvCb(ClientData, Tcl_Interp *, int, Tcl_Obj * const []);

static int
icuFacerecLoadEverybodyRqstSendCb(ClientData data, Tcl_Interp *interp,
				int objc, Tcl_Obj *const objv[])
{
   struct ModuleInfo *tclModuleInfo = (struct ModuleInfo *)data;
   int rqstId, bilan;
   char buf[10];
   /* no input */ 	/* input */
#if (0 > 0)
   int ret;
   int curObjc = 0;
#endif
   char strerr[64];

   TEST_BAD_USAGE(objc != 0+1);
		 


   if (tclServRqstSend(interp, Tcl_GetStringFromObj(objv[0], NULL),
		       tclModuleInfo->cid, ICU_FACERECLOADEVERYBODY_RQST,
		       NULL, 0,
		       &rqstId, &bilan) == ERROR) {

     Tcl_SetResult(interp, h2getErrMsg(bilan, strerr, 64), TCL_VOLATILE);
      return TCL_ERROR;
   }
    
   sprintf(buf, " %d", rqstId);
   Tcl_SetResult(interp, "SENT", TCL_STATIC);
   Tcl_AppendResult(interp, buf, NULL);
   return TCL_OK;
}

static int
icuFacerecLoadEverybodyReplyRcvCb(ClientData data, Tcl_Interp *interp,
				int objc, Tcl_Obj * const objv[])
{
   struct ModuleInfo *tclModuleInfo = (struct ModuleInfo *)data;
   int ret;
   int rqstId, bilan, activity;
   /* no output */ 	/* output */
   Tcl_Obj *my_own_private_unique_result;
   char strerr[64];

   TEST_BAD_USAGE(
      objc != 2 ||
      Tcl_GetIntFromObj(interp, objv[1], &rqstId) != TCL_OK
   );

   switch (
      /* c'est laid */
      tclServReplyRcv(interp,
		      tclModuleInfo->cid, rqstId, NO_BLOCK,
		      &activity, NULL, 0,
		      &bilan)
      ) {
      case WAITING_INTERMED_REPLY:
	 Tcl_SetResult(interp, "SENT", TCL_STATIC);
	 return TCL_OK;

      case WAITING_FINAL_REPLY: {
	 char tmp[50];
	 sprintf(tmp, "ACK %d", activity);
	 Tcl_SetResult(interp, tmp, TCL_VOLATILE);
	 return TCL_OK;
      }

      case FINAL_REPLY_OK:
	 break;

      default:
	 Tcl_SetResult(interp, h2getErrMsg(bilan, strerr, 64), TCL_VOLATILE);
	 return TCL_ERROR;
   }

   my_own_private_unique_result = Tcl_NewListObj(0, NULL);
   if (my_own_private_unique_result == NULL) return TCL_ERROR;

   ret = Tcl_ListObjAppendElement(interp, my_own_private_unique_result,
				  Tcl_NewStringObj("TERM", -1));
   if (ret != TCL_OK) return TCL_ERROR;

   if (bilan == OK)
      ret = Tcl_ListObjAppendElement(interp, my_own_private_unique_result,
				     Tcl_NewStringObj("OK", -1));
   else
      ret = Tcl_ListObjAppendElement(interp, my_own_private_unique_result,
				     Tcl_NewStringObj(
					h2getErrMsg(bilan, strerr, 64), -1));
   if (ret != TCL_OK) return TCL_ERROR;



   Tcl_SetObjResult(interp, my_own_private_unique_result);
   return TCL_OK;
}

/* --- FILE GENERATED WITH GENOM, DO NOT EDIT BY HAND ------------------ */

/* 
 * Copyright (c) 1993-2005 LAAS/CNRS
 * All rights reserved.
 *
 * Redistribution and use  in source  and binary  forms,  with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *   1. Redistributions of  source  code must retain the  above copyright
 *      notice, this list of conditions and the following disclaimer.
 *   2. Redistributions in binary form must reproduce the above copyright
 *      notice,  this list of  conditions and the following disclaimer in
 *      the  documentation  and/or  other   materials provided  with  the
 *      distribution.
 *
 * THIS  SOFTWARE IS PROVIDED BY  THE  COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND  ANY  EXPRESS OR IMPLIED  WARRANTIES,  INCLUDING,  BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES  OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR  PURPOSE ARE DISCLAIMED. IN  NO EVENT SHALL THE COPYRIGHT
 * HOLDERS OR      CONTRIBUTORS  BE LIABLE FOR   ANY    DIRECT, INDIRECT,
 * INCIDENTAL,  SPECIAL,  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF  SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN  CONTRACT, STRICT LIABILITY, OR
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE   OF THIS SOFTWARE, EVEN   IF ADVISED OF   THE POSSIBILITY OF SUCH
 * DAMAGE.
 */

/*
 * ----------------------------------------------------------------------
 * FacerecUnloadEverybody Request
 */

static int icuFacerecUnloadEverybodyRqstSendCb(ClientData, Tcl_Interp *, int, Tcl_Obj *const []); 
static int icuFacerecUnloadEverybodyReplyRcvCb(ClientData, Tcl_Interp *, int, Tcl_Obj * const []);

static int
icuFacerecUnloadEverybodyRqstSendCb(ClientData data, Tcl_Interp *interp,
				int objc, Tcl_Obj *const objv[])
{
   struct ModuleInfo *tclModuleInfo = (struct ModuleInfo *)data;
   int rqstId, bilan;
   char buf[10];
   /* no input */ 	/* input */
#if (0 > 0)
   int ret;
   int curObjc = 0;
#endif
   char strerr[64];

   TEST_BAD_USAGE(objc != 0+1);
		 


   if (tclServRqstSend(interp, Tcl_GetStringFromObj(objv[0], NULL),
		       tclModuleInfo->cid, ICU_FACERECUNLOADEVERYBODY_RQST,
		       NULL, 0,
		       &rqstId, &bilan) == ERROR) {

     Tcl_SetResult(interp, h2getErrMsg(bilan, strerr, 64), TCL_VOLATILE);
      return TCL_ERROR;
   }
    
   sprintf(buf, " %d", rqstId);
   Tcl_SetResult(interp, "SENT", TCL_STATIC);
   Tcl_AppendResult(interp, buf, NULL);
   return TCL_OK;
}

static int
icuFacerecUnloadEverybodyReplyRcvCb(ClientData data, Tcl_Interp *interp,
				int objc, Tcl_Obj * const objv[])
{
   struct ModuleInfo *tclModuleInfo = (struct ModuleInfo *)data;
   int ret;
   int rqstId, bilan, activity;
   /* no output */ 	/* output */
   Tcl_Obj *my_own_private_unique_result;
   char strerr[64];

   TEST_BAD_USAGE(
      objc != 2 ||
      Tcl_GetIntFromObj(interp, objv[1], &rqstId) != TCL_OK
   );

   switch (
      /* c'est laid */
      tclServReplyRcv(interp,
		      tclModuleInfo->cid, rqstId, NO_BLOCK,
		      &activity, NULL, 0,
		      &bilan)
      ) {
      case WAITING_INTERMED_REPLY:
	 Tcl_SetResult(interp, "SENT", TCL_STATIC);
	 return TCL_OK;

      case WAITING_FINAL_REPLY: {
	 char tmp[50];
	 sprintf(tmp, "ACK %d", activity);
	 Tcl_SetResult(interp, tmp, TCL_VOLATILE);
	 return TCL_OK;
      }

      case FINAL_REPLY_OK:
	 break;

      default:
	 Tcl_SetResult(interp, h2getErrMsg(bilan, strerr, 64), TCL_VOLATILE);
	 return TCL_ERROR;
   }

   my_own_private_unique_result = Tcl_NewListObj(0, NULL);
   if (my_own_private_unique_result == NULL) return TCL_ERROR;

   ret = Tcl_ListObjAppendElement(interp, my_own_private_unique_result,
				  Tcl_NewStringObj("TERM", -1));
   if (ret != TCL_OK) return TCL_ERROR;

   if (bilan == OK)
      ret = Tcl_ListObjAppendElement(interp, my_own_private_unique_result,
				     Tcl_NewStringObj("OK", -1));
   else
      ret = Tcl_ListObjAppendElement(interp, my_own_private_unique_result,
				     Tcl_NewStringObj(
					h2getErrMsg(bilan, strerr, 64), -1));
   if (ret != TCL_OK) return TCL_ERROR;



   Tcl_SetObjResult(interp, my_own_private_unique_result);
   return TCL_OK;
}

/* --- FILE GENERATED WITH GENOM, DO NOT EDIT BY HAND ------------------ */

/* 
 * Copyright (c) 1993-2005 LAAS/CNRS
 * All rights reserved.
 *
 * Redistribution and use  in source  and binary  forms,  with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *   1. Redistributions of  source  code must retain the  above copyright
 *      notice, this list of conditions and the following disclaimer.
 *   2. Redistributions in binary form must reproduce the above copyright
 *      notice,  this list of  conditions and the following disclaimer in
 *      the  documentation  and/or  other   materials provided  with  the
 *      distribution.
 *
 * THIS  SOFTWARE IS PROVIDED BY  THE  COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND  ANY  EXPRESS OR IMPLIED  WARRANTIES,  INCLUDING,  BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES  OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR  PURPOSE ARE DISCLAIMED. IN  NO EVENT SHALL THE COPYRIGHT
 * HOLDERS OR      CONTRIBUTORS  BE LIABLE FOR   ANY    DIRECT, INDIRECT,
 * INCIDENTAL,  SPECIAL,  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF  SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN  CONTRACT, STRICT LIABILITY, OR
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE   OF THIS SOFTWARE, EVEN   IF ADVISED OF   THE POSSIBILITY OF SUCH
 * DAMAGE.
 */

/*
 * ----------------------------------------------------------------------
 * WaitForModalityChange Request
 */

static int icuWaitForModalityChangeRqstSendCb(ClientData, Tcl_Interp *, int, Tcl_Obj *const []); 
static int icuWaitForModalityChangeReplyRcvCb(ClientData, Tcl_Interp *, int, Tcl_Obj * const []);

static int
icuWaitForModalityChangeRqstSendCb(ClientData data, Tcl_Interp *interp,
				int objc, Tcl_Obj *const objv[])
{
   struct ModuleInfo *tclModuleInfo = (struct ModuleInfo *)data;
   int rqstId, bilan;
   char buf[10];
   /* no input */ 	/* input */
#if (0 > 0)
   int ret;
   int curObjc = 0;
#endif
   char strerr[64];

   TEST_BAD_USAGE(objc != 0+1);
		 


   if (tclServRqstSend(interp, Tcl_GetStringFromObj(objv[0], NULL),
		       tclModuleInfo->cid, ICU_WAITFORMODALITYCHANGE_RQST,
		       NULL, 0,
		       &rqstId, &bilan) == ERROR) {

     Tcl_SetResult(interp, h2getErrMsg(bilan, strerr, 64), TCL_VOLATILE);
      return TCL_ERROR;
   }
    
   sprintf(buf, " %d", rqstId);
   Tcl_SetResult(interp, "SENT", TCL_STATIC);
   Tcl_AppendResult(interp, buf, NULL);
   return TCL_OK;
}

static int
icuWaitForModalityChangeReplyRcvCb(ClientData data, Tcl_Interp *interp,
				int objc, Tcl_Obj * const objv[])
{
   struct ModuleInfo *tclModuleInfo = (struct ModuleInfo *)data;
   int ret;
   int rqstId, bilan, activity;
   /* no output */ 	/* output */
   Tcl_Obj *my_own_private_unique_result;
   char strerr[64];

   TEST_BAD_USAGE(
      objc != 2 ||
      Tcl_GetIntFromObj(interp, objv[1], &rqstId) != TCL_OK
   );

   switch (
      /* c'est laid */
      tclServReplyRcv(interp,
		      tclModuleInfo->cid, rqstId, NO_BLOCK,
		      &activity, NULL, 0,
		      &bilan)
      ) {
      case WAITING_INTERMED_REPLY:
	 Tcl_SetResult(interp, "SENT", TCL_STATIC);
	 return TCL_OK;

      case WAITING_FINAL_REPLY: {
	 char tmp[50];
	 sprintf(tmp, "ACK %d", activity);
	 Tcl_SetResult(interp, tmp, TCL_VOLATILE);
	 return TCL_OK;
      }

      case FINAL_REPLY_OK:
	 break;

      default:
	 Tcl_SetResult(interp, h2getErrMsg(bilan, strerr, 64), TCL_VOLATILE);
	 return TCL_ERROR;
   }

   my_own_private_unique_result = Tcl_NewListObj(0, NULL);
   if (my_own_private_unique_result == NULL) return TCL_ERROR;

   ret = Tcl_ListObjAppendElement(interp, my_own_private_unique_result,
				  Tcl_NewStringObj("TERM", -1));
   if (ret != TCL_OK) return TCL_ERROR;

   if (bilan == OK)
      ret = Tcl_ListObjAppendElement(interp, my_own_private_unique_result,
				     Tcl_NewStringObj("OK", -1));
   else
      ret = Tcl_ListObjAppendElement(interp, my_own_private_unique_result,
				     Tcl_NewStringObj(
					h2getErrMsg(bilan, strerr, 64), -1));
   if (ret != TCL_OK) return TCL_ERROR;



   Tcl_SetObjResult(interp, my_own_private_unique_result);
   return TCL_OK;
}

/* --- FILE GENERATED WITH GENOM, DO NOT EDIT BY HAND ------------------ */

/* 
 * Copyright (c) 1993-2005 LAAS/CNRS
 * All rights reserved.
 *
 * Redistribution and use  in source  and binary  forms,  with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *   1. Redistributions of  source  code must retain the  above copyright
 *      notice, this list of conditions and the following disclaimer.
 *   2. Redistributions in binary form must reproduce the above copyright
 *      notice,  this list of  conditions and the following disclaimer in
 *      the  documentation  and/or  other   materials provided  with  the
 *      distribution.
 *
 * THIS  SOFTWARE IS PROVIDED BY  THE  COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND  ANY  EXPRESS OR IMPLIED  WARRANTIES,  INCLUDING,  BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES  OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR  PURPOSE ARE DISCLAIMED. IN  NO EVENT SHALL THE COPYRIGHT
 * HOLDERS OR      CONTRIBUTORS  BE LIABLE FOR   ANY    DIRECT, INDIRECT,
 * INCIDENTAL,  SPECIAL,  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF  SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN  CONTRACT, STRICT LIABILITY, OR
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE   OF THIS SOFTWARE, EVEN   IF ADVISED OF   THE POSSIBILITY OF SUCH
 * DAMAGE.
 */

/*
 * ----------------------------------------------------------------------
 * LaunchPlatine Request
 */

static int icuLaunchPlatineRqstSendCb(ClientData, Tcl_Interp *, int, Tcl_Obj *const []); 
static int icuLaunchPlatineReplyRcvCb(ClientData, Tcl_Interp *, int, Tcl_Obj * const []);

static int
icuLaunchPlatineRqstSendCb(ClientData data, Tcl_Interp *interp,
				int objc, Tcl_Obj *const objv[])
{
   struct ModuleInfo *tclModuleInfo = (struct ModuleInfo *)data;
   int rqstId, bilan;
   char buf[10];
   static ICU_PLATINE_PARAMS platineParams;	/* input */
#if (3 > 0)
   int ret;
   int curObjc = 0;
#endif
   char strerr[64];

   TEST_BAD_USAGE(objc != 3+1);
		 
   ret = Tcl_GetDoubleFromObj(interp, objv[++curObjc],    	&platineParams.posMin);
   if (ret != TCL_OK) return TCL_ERROR;
   ret = Tcl_GetDoubleFromObj(interp, objv[++curObjc],    	&platineParams.posMax);
   if (ret != TCL_OK) return TCL_ERROR;
   ret = Tcl_GetDoubleFromObj(interp, objv[++curObjc],    	&platineParams.vitesse);
   if (ret != TCL_OK) return TCL_ERROR;


   if (tclServRqstSend(interp, Tcl_GetStringFromObj(objv[0], NULL),
		       tclModuleInfo->cid, ICU_LAUNCHPLATINE_RQST,
		       &platineParams, sizeof(platineParams),
		       &rqstId, &bilan) == ERROR) {

     Tcl_SetResult(interp, h2getErrMsg(bilan, strerr, 64), TCL_VOLATILE);
      return TCL_ERROR;
   }
    
   sprintf(buf, " %d", rqstId);
   Tcl_SetResult(interp, "SENT", TCL_STATIC);
   Tcl_AppendResult(interp, buf, NULL);
   return TCL_OK;
}

static int
icuLaunchPlatineReplyRcvCb(ClientData data, Tcl_Interp *interp,
				int objc, Tcl_Obj * const objv[])
{
   struct ModuleInfo *tclModuleInfo = (struct ModuleInfo *)data;
   int ret;
   int rqstId, bilan, activity;
   /* no output */ 	/* output */
   Tcl_Obj *my_own_private_unique_result;
   char strerr[64];

   TEST_BAD_USAGE(
      objc != 2 ||
      Tcl_GetIntFromObj(interp, objv[1], &rqstId) != TCL_OK
   );

   switch (
      /* c'est laid */
      tclServReplyRcv(interp,
		      tclModuleInfo->cid, rqstId, NO_BLOCK,
		      &activity, NULL, 0,
		      &bilan)
      ) {
      case WAITING_INTERMED_REPLY:
	 Tcl_SetResult(interp, "SENT", TCL_STATIC);
	 return TCL_OK;

      case WAITING_FINAL_REPLY: {
	 char tmp[50];
	 sprintf(tmp, "ACK %d", activity);
	 Tcl_SetResult(interp, tmp, TCL_VOLATILE);
	 return TCL_OK;
      }

      case FINAL_REPLY_OK:
	 break;

      default:
	 Tcl_SetResult(interp, h2getErrMsg(bilan, strerr, 64), TCL_VOLATILE);
	 return TCL_ERROR;
   }

   my_own_private_unique_result = Tcl_NewListObj(0, NULL);
   if (my_own_private_unique_result == NULL) return TCL_ERROR;

   ret = Tcl_ListObjAppendElement(interp, my_own_private_unique_result,
				  Tcl_NewStringObj("TERM", -1));
   if (ret != TCL_OK) return TCL_ERROR;

   if (bilan == OK)
      ret = Tcl_ListObjAppendElement(interp, my_own_private_unique_result,
				     Tcl_NewStringObj("OK", -1));
   else
      ret = Tcl_ListObjAppendElement(interp, my_own_private_unique_result,
				     Tcl_NewStringObj(
					h2getErrMsg(bilan, strerr, 64), -1));
   if (ret != TCL_OK) return TCL_ERROR;



   Tcl_SetObjResult(interp, my_own_private_unique_result);
   return TCL_OK;
}

/* --- FILE GENERATED WITH GENOM, DO NOT EDIT BY HAND ------------------ */

/* 
 * Copyright (c) 1993-2005 LAAS/CNRS
 * All rights reserved.
 *
 * Redistribution and use  in source  and binary  forms,  with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *   1. Redistributions of  source  code must retain the  above copyright
 *      notice, this list of conditions and the following disclaimer.
 *   2. Redistributions in binary form must reproduce the above copyright
 *      notice,  this list of  conditions and the following disclaimer in
 *      the  documentation  and/or  other   materials provided  with  the
 *      distribution.
 *
 * THIS  SOFTWARE IS PROVIDED BY  THE  COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND  ANY  EXPRESS OR IMPLIED  WARRANTIES,  INCLUDING,  BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES  OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR  PURPOSE ARE DISCLAIMED. IN  NO EVENT SHALL THE COPYRIGHT
 * HOLDERS OR      CONTRIBUTORS  BE LIABLE FOR   ANY    DIRECT, INDIRECT,
 * INCIDENTAL,  SPECIAL,  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF  SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN  CONTRACT, STRICT LIABILITY, OR
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE   OF THIS SOFTWARE, EVEN   IF ADVISED OF   THE POSSIBILITY OF SUCH
 * DAMAGE.
 */

/*
 * ----------------------------------------------------------------------
 * CmdPtu Request
 */

static int icuCmdPtuRqstSendCb(ClientData, Tcl_Interp *, int, Tcl_Obj *const []); 
static int icuCmdPtuReplyRcvCb(ClientData, Tcl_Interp *, int, Tcl_Obj * const []);

static int
icuCmdPtuRqstSendCb(ClientData data, Tcl_Interp *interp,
				int objc, Tcl_Obj *const objv[])
{
   struct ModuleInfo *tclModuleInfo = (struct ModuleInfo *)data;
   int rqstId, bilan;
   char buf[10];
   /* no input */ 	/* input */
#if (0 > 0)
   int ret;
   int curObjc = 0;
#endif
   char strerr[64];

   TEST_BAD_USAGE(objc != 0+1);
		 


   if (tclServRqstSend(interp, Tcl_GetStringFromObj(objv[0], NULL),
		       tclModuleInfo->cid, ICU_CMDPTU_RQST,
		       NULL, 0,
		       &rqstId, &bilan) == ERROR) {

     Tcl_SetResult(interp, h2getErrMsg(bilan, strerr, 64), TCL_VOLATILE);
      return TCL_ERROR;
   }
    
   sprintf(buf, " %d", rqstId);
   Tcl_SetResult(interp, "SENT", TCL_STATIC);
   Tcl_AppendResult(interp, buf, NULL);
   return TCL_OK;
}

static int
icuCmdPtuReplyRcvCb(ClientData data, Tcl_Interp *interp,
				int objc, Tcl_Obj * const objv[])
{
   struct ModuleInfo *tclModuleInfo = (struct ModuleInfo *)data;
   int ret;
   int rqstId, bilan, activity;
   /* no output */ 	/* output */
   Tcl_Obj *my_own_private_unique_result;
   char strerr[64];

   TEST_BAD_USAGE(
      objc != 2 ||
      Tcl_GetIntFromObj(interp, objv[1], &rqstId) != TCL_OK
   );

   switch (
      /* c'est laid */
      tclServReplyRcv(interp,
		      tclModuleInfo->cid, rqstId, NO_BLOCK,
		      &activity, NULL, 0,
		      &bilan)
      ) {
      case WAITING_INTERMED_REPLY:
	 Tcl_SetResult(interp, "SENT", TCL_STATIC);
	 return TCL_OK;

      case WAITING_FINAL_REPLY: {
	 char tmp[50];
	 sprintf(tmp, "ACK %d", activity);
	 Tcl_SetResult(interp, tmp, TCL_VOLATILE);
	 return TCL_OK;
      }

      case FINAL_REPLY_OK:
	 break;

      default:
	 Tcl_SetResult(interp, h2getErrMsg(bilan, strerr, 64), TCL_VOLATILE);
	 return TCL_ERROR;
   }

   my_own_private_unique_result = Tcl_NewListObj(0, NULL);
   if (my_own_private_unique_result == NULL) return TCL_ERROR;

   ret = Tcl_ListObjAppendElement(interp, my_own_private_unique_result,
				  Tcl_NewStringObj("TERM", -1));
   if (ret != TCL_OK) return TCL_ERROR;

   if (bilan == OK)
      ret = Tcl_ListObjAppendElement(interp, my_own_private_unique_result,
				     Tcl_NewStringObj("OK", -1));
   else
      ret = Tcl_ListObjAppendElement(interp, my_own_private_unique_result,
				     Tcl_NewStringObj(
					h2getErrMsg(bilan, strerr, 64), -1));
   if (ret != TCL_OK) return TCL_ERROR;



   Tcl_SetObjResult(interp, my_own_private_unique_result);
   return TCL_OK;
}

/* --- FILE GENERATED WITH GENOM, DO NOT EDIT BY HAND ------------------ */

/* 
 * Copyright (c) 1993-2005 LAAS/CNRS
 * All rights reserved.
 *
 * Redistribution and use  in source  and binary  forms,  with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *   1. Redistributions of  source  code must retain the  above copyright
 *      notice, this list of conditions and the following disclaimer.
 *   2. Redistributions in binary form must reproduce the above copyright
 *      notice,  this list of  conditions and the following disclaimer in
 *      the  documentation  and/or  other   materials provided  with  the
 *      distribution.
 *
 * THIS  SOFTWARE IS PROVIDED BY  THE  COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND  ANY  EXPRESS OR IMPLIED  WARRANTIES,  INCLUDING,  BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES  OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR  PURPOSE ARE DISCLAIMED. IN  NO EVENT SHALL THE COPYRIGHT
 * HOLDERS OR      CONTRIBUTORS  BE LIABLE FOR   ANY    DIRECT, INDIRECT,
 * INCIDENTAL,  SPECIAL,  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF  SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN  CONTRACT, STRICT LIABILITY, OR
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE   OF THIS SOFTWARE, EVEN   IF ADVISED OF   THE POSSIBILITY OF SUCH
 * DAMAGE.
 */

/*
 * ----------------------------------------------------------------------
 * IcuInitRFID Request
 */

static int icuIcuInitRFIDRqstSendCb(ClientData, Tcl_Interp *, int, Tcl_Obj *const []); 
static int icuIcuInitRFIDReplyRcvCb(ClientData, Tcl_Interp *, int, Tcl_Obj * const []);

static int
icuIcuInitRFIDRqstSendCb(ClientData data, Tcl_Interp *interp,
				int objc, Tcl_Obj *const objv[])
{
   struct ModuleInfo *tclModuleInfo = (struct ModuleInfo *)data;
   int rqstId, bilan;
   char buf[10];
   /* no input */ 	/* input */
#if (0 > 0)
   int ret;
   int curObjc = 0;
#endif
   char strerr[64];

   TEST_BAD_USAGE(objc != 0+1);
		 


   if (tclServRqstSend(interp, Tcl_GetStringFromObj(objv[0], NULL),
		       tclModuleInfo->cid, ICU_ICUINITRFID_RQST,
		       NULL, 0,
		       &rqstId, &bilan) == ERROR) {

     Tcl_SetResult(interp, h2getErrMsg(bilan, strerr, 64), TCL_VOLATILE);
      return TCL_ERROR;
   }
    
   sprintf(buf, " %d", rqstId);
   Tcl_SetResult(interp, "SENT", TCL_STATIC);
   Tcl_AppendResult(interp, buf, NULL);
   return TCL_OK;
}

static int
icuIcuInitRFIDReplyRcvCb(ClientData data, Tcl_Interp *interp,
				int objc, Tcl_Obj * const objv[])
{
   struct ModuleInfo *tclModuleInfo = (struct ModuleInfo *)data;
   int ret;
   int rqstId, bilan, activity;
   /* no output */ 	/* output */
   Tcl_Obj *my_own_private_unique_result;
   char strerr[64];

   TEST_BAD_USAGE(
      objc != 2 ||
      Tcl_GetIntFromObj(interp, objv[1], &rqstId) != TCL_OK
   );

   switch (
      /* c'est laid */
      tclServReplyRcv(interp,
		      tclModuleInfo->cid, rqstId, NO_BLOCK,
		      &activity, NULL, 0,
		      &bilan)
      ) {
      case WAITING_INTERMED_REPLY:
	 Tcl_SetResult(interp, "SENT", TCL_STATIC);
	 return TCL_OK;

      case WAITING_FINAL_REPLY: {
	 char tmp[50];
	 sprintf(tmp, "ACK %d", activity);
	 Tcl_SetResult(interp, tmp, TCL_VOLATILE);
	 return TCL_OK;
      }

      case FINAL_REPLY_OK:
	 break;

      default:
	 Tcl_SetResult(interp, h2getErrMsg(bilan, strerr, 64), TCL_VOLATILE);
	 return TCL_ERROR;
   }

   my_own_private_unique_result = Tcl_NewListObj(0, NULL);
   if (my_own_private_unique_result == NULL) return TCL_ERROR;

   ret = Tcl_ListObjAppendElement(interp, my_own_private_unique_result,
				  Tcl_NewStringObj("TERM", -1));
   if (ret != TCL_OK) return TCL_ERROR;

   if (bilan == OK)
      ret = Tcl_ListObjAppendElement(interp, my_own_private_unique_result,
				     Tcl_NewStringObj("OK", -1));
   else
      ret = Tcl_ListObjAppendElement(interp, my_own_private_unique_result,
				     Tcl_NewStringObj(
					h2getErrMsg(bilan, strerr, 64), -1));
   if (ret != TCL_OK) return TCL_ERROR;



   Tcl_SetObjResult(interp, my_own_private_unique_result);
   return TCL_OK;
}

/* --- FILE GENERATED WITH GENOM, DO NOT EDIT BY HAND ------------------ */

/* 
 * Copyright (c) 1993-2005 LAAS/CNRS
 * All rights reserved.
 *
 * Redistribution and use  in source  and binary  forms,  with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *   1. Redistributions of  source  code must retain the  above copyright
 *      notice, this list of conditions and the following disclaimer.
 *   2. Redistributions in binary form must reproduce the above copyright
 *      notice,  this list of  conditions and the following disclaimer in
 *      the  documentation  and/or  other   materials provided  with  the
 *      distribution.
 *
 * THIS  SOFTWARE IS PROVIDED BY  THE  COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND  ANY  EXPRESS OR IMPLIED  WARRANTIES,  INCLUDING,  BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES  OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR  PURPOSE ARE DISCLAIMED. IN  NO EVENT SHALL THE COPYRIGHT
 * HOLDERS OR      CONTRIBUTORS  BE LIABLE FOR   ANY    DIRECT, INDIRECT,
 * INCIDENTAL,  SPECIAL,  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF  SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN  CONTRACT, STRICT LIABILITY, OR
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE   OF THIS SOFTWARE, EVEN   IF ADVISED OF   THE POSSIBILITY OF SUCH
 * DAMAGE.
 */

/*
 * ----------------------------------------------------------------------
 * IcuCloseRFID Request
 */

static int icuIcuCloseRFIDRqstSendCb(ClientData, Tcl_Interp *, int, Tcl_Obj *const []); 
static int icuIcuCloseRFIDReplyRcvCb(ClientData, Tcl_Interp *, int, Tcl_Obj * const []);

static int
icuIcuCloseRFIDRqstSendCb(ClientData data, Tcl_Interp *interp,
				int objc, Tcl_Obj *const objv[])
{
   struct ModuleInfo *tclModuleInfo = (struct ModuleInfo *)data;
   int rqstId, bilan;
   char buf[10];
   /* no input */ 	/* input */
#if (0 > 0)
   int ret;
   int curObjc = 0;
#endif
   char strerr[64];

   TEST_BAD_USAGE(objc != 0+1);
		 


   if (tclServRqstSend(interp, Tcl_GetStringFromObj(objv[0], NULL),
		       tclModuleInfo->cid, ICU_ICUCLOSERFID_RQST,
		       NULL, 0,
		       &rqstId, &bilan) == ERROR) {

     Tcl_SetResult(interp, h2getErrMsg(bilan, strerr, 64), TCL_VOLATILE);
      return TCL_ERROR;
   }
    
   sprintf(buf, " %d", rqstId);
   Tcl_SetResult(interp, "SENT", TCL_STATIC);
   Tcl_AppendResult(interp, buf, NULL);
   return TCL_OK;
}

static int
icuIcuCloseRFIDReplyRcvCb(ClientData data, Tcl_Interp *interp,
				int objc, Tcl_Obj * const objv[])
{
   struct ModuleInfo *tclModuleInfo = (struct ModuleInfo *)data;
   int ret;
   int rqstId, bilan, activity;
   /* no output */ 	/* output */
   Tcl_Obj *my_own_private_unique_result;
   char strerr[64];

   TEST_BAD_USAGE(
      objc != 2 ||
      Tcl_GetIntFromObj(interp, objv[1], &rqstId) != TCL_OK
   );

   switch (
      /* c'est laid */
      tclServReplyRcv(interp,
		      tclModuleInfo->cid, rqstId, NO_BLOCK,
		      &activity, NULL, 0,
		      &bilan)
      ) {
      case WAITING_INTERMED_REPLY:
	 Tcl_SetResult(interp, "SENT", TCL_STATIC);
	 return TCL_OK;

      case WAITING_FINAL_REPLY: {
	 char tmp[50];
	 sprintf(tmp, "ACK %d", activity);
	 Tcl_SetResult(interp, tmp, TCL_VOLATILE);
	 return TCL_OK;
      }

      case FINAL_REPLY_OK:
	 break;

      default:
	 Tcl_SetResult(interp, h2getErrMsg(bilan, strerr, 64), TCL_VOLATILE);
	 return TCL_ERROR;
   }

   my_own_private_unique_result = Tcl_NewListObj(0, NULL);
   if (my_own_private_unique_result == NULL) return TCL_ERROR;

   ret = Tcl_ListObjAppendElement(interp, my_own_private_unique_result,
				  Tcl_NewStringObj("TERM", -1));
   if (ret != TCL_OK) return TCL_ERROR;

   if (bilan == OK)
      ret = Tcl_ListObjAppendElement(interp, my_own_private_unique_result,
				     Tcl_NewStringObj("OK", -1));
   else
      ret = Tcl_ListObjAppendElement(interp, my_own_private_unique_result,
				     Tcl_NewStringObj(
					h2getErrMsg(bilan, strerr, 64), -1));
   if (ret != TCL_OK) return TCL_ERROR;



   Tcl_SetObjResult(interp, my_own_private_unique_result);
   return TCL_OK;
}

/* --- FILE GENERATED WITH GENOM, DO NOT EDIT BY HAND ------------------ */

/* 
 * Copyright (c) 1993-2005 LAAS/CNRS
 * All rights reserved.
 *
 * Redistribution and use  in source  and binary  forms,  with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *   1. Redistributions of  source  code must retain the  above copyright
 *      notice, this list of conditions and the following disclaimer.
 *   2. Redistributions in binary form must reproduce the above copyright
 *      notice,  this list of  conditions and the following disclaimer in
 *      the  documentation  and/or  other   materials provided  with  the
 *      distribution.
 *
 * THIS  SOFTWARE IS PROVIDED BY  THE  COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND  ANY  EXPRESS OR IMPLIED  WARRANTIES,  INCLUDING,  BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES  OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR  PURPOSE ARE DISCLAIMED. IN  NO EVENT SHALL THE COPYRIGHT
 * HOLDERS OR      CONTRIBUTORS  BE LIABLE FOR   ANY    DIRECT, INDIRECT,
 * INCIDENTAL,  SPECIAL,  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF  SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN  CONTRACT, STRICT LIABILITY, OR
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE   OF THIS SOFTWARE, EVEN   IF ADVISED OF   THE POSSIBILITY OF SUCH
 * DAMAGE.
 */

/*
 * ----------------------------------------------------------------------
 * IcuAcquireRFID Request
 */

static int icuIcuAcquireRFIDRqstSendCb(ClientData, Tcl_Interp *, int, Tcl_Obj *const []); 
static int icuIcuAcquireRFIDReplyRcvCb(ClientData, Tcl_Interp *, int, Tcl_Obj * const []);

static int
icuIcuAcquireRFIDRqstSendCb(ClientData data, Tcl_Interp *interp,
				int objc, Tcl_Obj *const objv[])
{
   struct ModuleInfo *tclModuleInfo = (struct ModuleInfo *)data;
   int rqstId, bilan;
   char buf[10];
   /* no input */ 	/* input */
#if (0 > 0)
   int ret;
   int curObjc = 0;
#endif
   char strerr[64];

   TEST_BAD_USAGE(objc != 0+1);
		 


   if (tclServRqstSend(interp, Tcl_GetStringFromObj(objv[0], NULL),
		       tclModuleInfo->cid, ICU_ICUACQUIRERFID_RQST,
		       NULL, 0,
		       &rqstId, &bilan) == ERROR) {

     Tcl_SetResult(interp, h2getErrMsg(bilan, strerr, 64), TCL_VOLATILE);
      return TCL_ERROR;
   }
    
   sprintf(buf, " %d", rqstId);
   Tcl_SetResult(interp, "SENT", TCL_STATIC);
   Tcl_AppendResult(interp, buf, NULL);
   return TCL_OK;
}

static int
icuIcuAcquireRFIDReplyRcvCb(ClientData data, Tcl_Interp *interp,
				int objc, Tcl_Obj * const objv[])
{
   struct ModuleInfo *tclModuleInfo = (struct ModuleInfo *)data;
   int ret;
   int rqstId, bilan, activity;
   /* no output */ 	/* output */
   Tcl_Obj *my_own_private_unique_result;
   char strerr[64];

   TEST_BAD_USAGE(
      objc != 2 ||
      Tcl_GetIntFromObj(interp, objv[1], &rqstId) != TCL_OK
   );

   switch (
      /* c'est laid */
      tclServReplyRcv(interp,
		      tclModuleInfo->cid, rqstId, NO_BLOCK,
		      &activity, NULL, 0,
		      &bilan)
      ) {
      case WAITING_INTERMED_REPLY:
	 Tcl_SetResult(interp, "SENT", TCL_STATIC);
	 return TCL_OK;

      case WAITING_FINAL_REPLY: {
	 char tmp[50];
	 sprintf(tmp, "ACK %d", activity);
	 Tcl_SetResult(interp, tmp, TCL_VOLATILE);
	 return TCL_OK;
      }

      case FINAL_REPLY_OK:
	 break;

      default:
	 Tcl_SetResult(interp, h2getErrMsg(bilan, strerr, 64), TCL_VOLATILE);
	 return TCL_ERROR;
   }

   my_own_private_unique_result = Tcl_NewListObj(0, NULL);
   if (my_own_private_unique_result == NULL) return TCL_ERROR;

   ret = Tcl_ListObjAppendElement(interp, my_own_private_unique_result,
				  Tcl_NewStringObj("TERM", -1));
   if (ret != TCL_OK) return TCL_ERROR;

   if (bilan == OK)
      ret = Tcl_ListObjAppendElement(interp, my_own_private_unique_result,
				     Tcl_NewStringObj("OK", -1));
   else
      ret = Tcl_ListObjAppendElement(interp, my_own_private_unique_result,
				     Tcl_NewStringObj(
					h2getErrMsg(bilan, strerr, 64), -1));
   if (ret != TCL_OK) return TCL_ERROR;



   Tcl_SetObjResult(interp, my_own_private_unique_result);
   return TCL_OK;
}

/* --- FILE GENERATED WITH GENOM, DO NOT EDIT BY HAND ------------------ */

/* 
 * Copyright (c) 1993-2005 LAAS/CNRS
 * All rights reserved.
 *
 * Redistribution and use  in source  and binary  forms,  with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *   1. Redistributions of  source  code must retain the  above copyright
 *      notice, this list of conditions and the following disclaimer.
 *   2. Redistributions in binary form must reproduce the above copyright
 *      notice,  this list of  conditions and the following disclaimer in
 *      the  documentation  and/or  other   materials provided  with  the
 *      distribution.
 *
 * THIS  SOFTWARE IS PROVIDED BY  THE  COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND  ANY  EXPRESS OR IMPLIED  WARRANTIES,  INCLUDING,  BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES  OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR  PURPOSE ARE DISCLAIMED. IN  NO EVENT SHALL THE COPYRIGHT
 * HOLDERS OR      CONTRIBUTORS  BE LIABLE FOR   ANY    DIRECT, INDIRECT,
 * INCIDENTAL,  SPECIAL,  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF  SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN  CONTRACT, STRICT LIABILITY, OR
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE   OF THIS SOFTWARE, EVEN   IF ADVISED OF   THE POSSIBILITY OF SUCH
 * DAMAGE.
 */

/*
 * ----------------------------------------------------------------------
 * IcuAcquireTag Request
 */

static int icuIcuAcquireTagRqstSendCb(ClientData, Tcl_Interp *, int, Tcl_Obj *const []); 
static int icuIcuAcquireTagReplyRcvCb(ClientData, Tcl_Interp *, int, Tcl_Obj * const []);

static int
icuIcuAcquireTagRqstSendCb(ClientData data, Tcl_Interp *interp,
				int objc, Tcl_Obj *const objv[])
{
   struct ModuleInfo *tclModuleInfo = (struct ModuleInfo *)data;
   int rqstId, bilan;
   char buf[10];
   static GEN_STRING256 recoTag;	/* input */
#if (1 > 0)
   int ret;
   int curObjc = 0;
#endif
   char strerr[64];

   TEST_BAD_USAGE(objc != 1+1);
		 
   strncpy(recoTag.name, 
   	Tcl_GetStringFromObj(objv[++curObjc], NULL),    	256);
   ret = TCL_OK;
   if (ret != TCL_OK) return TCL_ERROR;


   if (tclServRqstSend(interp, Tcl_GetStringFromObj(objv[0], NULL),
		       tclModuleInfo->cid, ICU_ICUACQUIRETAG_RQST,
		       &recoTag, sizeof(recoTag),
		       &rqstId, &bilan) == ERROR) {

     Tcl_SetResult(interp, h2getErrMsg(bilan, strerr, 64), TCL_VOLATILE);
      return TCL_ERROR;
   }
    
   sprintf(buf, " %d", rqstId);
   Tcl_SetResult(interp, "SENT", TCL_STATIC);
   Tcl_AppendResult(interp, buf, NULL);
   return TCL_OK;
}

static int
icuIcuAcquireTagReplyRcvCb(ClientData data, Tcl_Interp *interp,
				int objc, Tcl_Obj * const objv[])
{
   struct ModuleInfo *tclModuleInfo = (struct ModuleInfo *)data;
   int ret;
   int rqstId, bilan, activity;
   /* no output */ 	/* output */
   Tcl_Obj *my_own_private_unique_result;
   char strerr[64];

   TEST_BAD_USAGE(
      objc != 2 ||
      Tcl_GetIntFromObj(interp, objv[1], &rqstId) != TCL_OK
   );

   switch (
      /* c'est laid */
      tclServReplyRcv(interp,
		      tclModuleInfo->cid, rqstId, NO_BLOCK,
		      &activity, NULL, 0,
		      &bilan)
      ) {
      case WAITING_INTERMED_REPLY:
	 Tcl_SetResult(interp, "SENT", TCL_STATIC);
	 return TCL_OK;

      case WAITING_FINAL_REPLY: {
	 char tmp[50];
	 sprintf(tmp, "ACK %d", activity);
	 Tcl_SetResult(interp, tmp, TCL_VOLATILE);
	 return TCL_OK;
      }

      case FINAL_REPLY_OK:
	 break;

      default:
	 Tcl_SetResult(interp, h2getErrMsg(bilan, strerr, 64), TCL_VOLATILE);
	 return TCL_ERROR;
   }

   my_own_private_unique_result = Tcl_NewListObj(0, NULL);
   if (my_own_private_unique_result == NULL) return TCL_ERROR;

   ret = Tcl_ListObjAppendElement(interp, my_own_private_unique_result,
				  Tcl_NewStringObj("TERM", -1));
   if (ret != TCL_OK) return TCL_ERROR;

   if (bilan == OK)
      ret = Tcl_ListObjAppendElement(interp, my_own_private_unique_result,
				     Tcl_NewStringObj("OK", -1));
   else
      ret = Tcl_ListObjAppendElement(interp, my_own_private_unique_result,
				     Tcl_NewStringObj(
					h2getErrMsg(bilan, strerr, 64), -1));
   if (ret != TCL_OK) return TCL_ERROR;



   Tcl_SetObjResult(interp, my_own_private_unique_result);
   return TCL_OK;
}

/* --- FILE GENERATED WITH GENOM, DO NOT EDIT BY HAND ------------------ */

/* 
 * Copyright (c) 1993-2005 LAAS/CNRS
 * All rights reserved.
 *
 * Redistribution and use  in source  and binary  forms,  with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *   1. Redistributions of  source  code must retain the  above copyright
 *      notice, this list of conditions and the following disclaimer.
 *   2. Redistributions in binary form must reproduce the above copyright
 *      notice,  this list of  conditions and the following disclaimer in
 *      the  documentation  and/or  other   materials provided  with  the
 *      distribution.
 *
 * THIS  SOFTWARE IS PROVIDED BY  THE  COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND  ANY  EXPRESS OR IMPLIED  WARRANTIES,  INCLUDING,  BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES  OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR  PURPOSE ARE DISCLAIMED. IN  NO EVENT SHALL THE COPYRIGHT
 * HOLDERS OR      CONTRIBUTORS  BE LIABLE FOR   ANY    DIRECT, INDIRECT,
 * INCIDENTAL,  SPECIAL,  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF  SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN  CONTRACT, STRICT LIABILITY, OR
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE   OF THIS SOFTWARE, EVEN   IF ADVISED OF   THE POSSIBILITY OF SUCH
 * DAMAGE.
 */

/*
 * ----------------------------------------------------------------------
 * Results Poster
 */

static int icuResultshumanPosPosterTclRead(ClientData, Tcl_Interp *, int, Tcl_Obj *const []); 

static int
icuResultshumanPosPosterTclRead(ClientData data, Tcl_Interp *interp,
			 int objc, Tcl_Obj *const objv[])
{
   int ret;
   static struct ICU_HUMAN_POS *_posterData;	/* data */
   Tcl_Obj *my_own_private_unique_result;
   char strerr[64];

   TEST_BAD_USAGE(objc != 1);
		 
  if ((_posterData = malloc(sizeof(struct ICU_HUMAN_POS))) == NULL) {
      Tcl_SetResult(interp, h2getErrMsg(errnoGet(), strerr, 64), TCL_VOLATILE);
      return TCL_ERROR;
  }

   if (icuResultshumanPosPosterRead(_posterData) != OK) {
      Tcl_SetResult(interp, h2getErrMsg(errnoGet(), strerr, 64), TCL_VOLATILE);
      free(_posterData);
      return TCL_ERROR;
   }

   my_own_private_unique_result = Tcl_NewListObj(0, NULL);
   if (my_own_private_unique_result == NULL) {
     free(_posterData);
     return TCL_ERROR;
   }

   ret = Tcl_ListObjAppendElement(interp, my_own_private_unique_result,
	Tcl_NewIntObj((*_posterData).sensorPos.date));
   if (ret != TCL_OK) {free(_posterData); return TCL_ERROR;}
   ret = Tcl_ListObjAppendElement(interp, my_own_private_unique_result,
	Tcl_NewIntObj((*_posterData).sensorPos.pad));
   if (ret != TCL_OK) {free(_posterData); return TCL_ERROR;}
   ret = Tcl_ListObjAppendElement(interp, my_own_private_unique_result,
	Tcl_NewDoubleObj((*_posterData).sensorPos.sensorToMain.euler.yaw));
   if (ret != TCL_OK) {free(_posterData); return TCL_ERROR;}
   ret = Tcl_ListObjAppendElement(interp, my_own_private_unique_result,
	Tcl_NewDoubleObj((*_posterData).sensorPos.sensorToMain.euler.pitch));
   if (ret != TCL_OK) {free(_posterData); return TCL_ERROR;}
   ret = Tcl_ListObjAppendElement(interp, my_own_private_unique_result,
	Tcl_NewDoubleObj((*_posterData).sensorPos.sensorToMain.euler.roll));
   if (ret != TCL_OK) {free(_posterData); return TCL_ERROR;}
   ret = Tcl_ListObjAppendElement(interp, my_own_private_unique_result,
	Tcl_NewDoubleObj((*_posterData).sensorPos.sensorToMain.euler.x));
   if (ret != TCL_OK) {free(_posterData); return TCL_ERROR;}
   ret = Tcl_ListObjAppendElement(interp, my_own_private_unique_result,
	Tcl_NewDoubleObj((*_posterData).sensorPos.sensorToMain.euler.y));
   if (ret != TCL_OK) {free(_posterData); return TCL_ERROR;}
   ret = Tcl_ListObjAppendElement(interp, my_own_private_unique_result,
	Tcl_NewDoubleObj((*_posterData).sensorPos.sensorToMain.euler.z));
   if (ret != TCL_OK) {free(_posterData); return TCL_ERROR;}
   ret = Tcl_ListObjAppendElement(interp, my_own_private_unique_result,
	Tcl_NewDoubleObj((*_posterData).sensorPos.sensorToMain.var.voo));
   if (ret != TCL_OK) {free(_posterData); return TCL_ERROR;}
   ret = Tcl_ListObjAppendElement(interp, my_own_private_unique_result,
	Tcl_NewDoubleObj((*_posterData).sensorPos.sensorToMain.var.vop));
   if (ret != TCL_OK) {free(_posterData); return TCL_ERROR;}
   ret = Tcl_ListObjAppendElement(interp, my_own_private_unique_result,
	Tcl_NewDoubleObj((*_posterData).sensorPos.sensorToMain.var.vpp));
   if (ret != TCL_OK) {free(_posterData); return TCL_ERROR;}
   ret = Tcl_ListObjAppendElement(interp, my_own_private_unique_result,
	Tcl_NewDoubleObj((*_posterData).sensorPos.sensorToMain.var.vor));
   if (ret != TCL_OK) {free(_posterData); return TCL_ERROR;}
   ret = Tcl_ListObjAppendElement(interp, my_own_private_unique_result,
	Tcl_NewDoubleObj((*_posterData).sensorPos.sensorToMain.var.vpr));
   if (ret != TCL_OK) {free(_posterData); return TCL_ERROR;}
   ret = Tcl_ListObjAppendElement(interp, my_own_private_unique_result,
	Tcl_NewDoubleObj((*_posterData).sensorPos.sensorToMain.var.vrr));
   if (ret != TCL_OK) {free(_posterData); return TCL_ERROR;}
   ret = Tcl_ListObjAppendElement(interp, my_own_private_unique_result,
	Tcl_NewDoubleObj((*_posterData).sensorPos.sensorToMain.var.vox));
   if (ret != TCL_OK) {free(_posterData); return TCL_ERROR;}
   ret = Tcl_ListObjAppendElement(interp, my_own_private_unique_result,
	Tcl_NewDoubleObj((*_posterData).sensorPos.sensorToMain.var.vpx));
   if (ret != TCL_OK) {free(_posterData); return TCL_ERROR;}
   ret = Tcl_ListObjAppendElement(interp, my_own_private_unique_result,
	Tcl_NewDoubleObj((*_posterData).sensorPos.sensorToMain.var.vrx));
   if (ret != TCL_OK) {free(_posterData); return TCL_ERROR;}
   ret = Tcl_ListObjAppendElement(interp, my_own_private_unique_result,
	Tcl_NewDoubleObj((*_posterData).sensorPos.sensorToMain.var.vxx));
   if (ret != TCL_OK) {free(_posterData); return TCL_ERROR;}
   ret = Tcl_ListObjAppendElement(interp, my_own_private_unique_result,
	Tcl_NewDoubleObj((*_posterData).sensorPos.sensorToMain.var.voy));
   if (ret != TCL_OK) {free(_posterData); return TCL_ERROR;}
   ret = Tcl_ListObjAppendElement(interp, my_own_private_unique_result,
	Tcl_NewDoubleObj((*_posterData).sensorPos.sensorToMain.var.vpy));
   if (ret != TCL_OK) {free(_posterData); return TCL_ERROR;}
   ret = Tcl_ListObjAppendElement(interp, my_own_private_unique_result,
	Tcl_NewDoubleObj((*_posterData).sensorPos.sensorToMain.var.vry));
   if (ret != TCL_OK) {free(_posterData); return TCL_ERROR;}
   ret = Tcl_ListObjAppendElement(interp, my_own_private_unique_result,
	Tcl_NewDoubleObj((*_posterData).sensorPos.sensorToMain.var.vxy));
   if (ret != TCL_OK) {free(_posterData); return TCL_ERROR;}
   ret = Tcl_ListObjAppendElement(interp, my_own_private_unique_result,
	Tcl_NewDoubleObj((*_posterData).sensorPos.sensorToMain.var.vyy));
   if (ret != TCL_OK) {free(_posterData); return TCL_ERROR;}
   ret = Tcl_ListObjAppendElement(interp, my_own_private_unique_result,
	Tcl_NewDoubleObj((*_posterData).sensorPos.sensorToMain.var.voz));
   if (ret != TCL_OK) {free(_posterData); return TCL_ERROR;}
   ret = Tcl_ListObjAppendElement(interp, my_own_private_unique_result,
	Tcl_NewDoubleObj((*_posterData).sensorPos.sensorToMain.var.vpz));
   if (ret != TCL_OK) {free(_posterData); return TCL_ERROR;}
   ret = Tcl_ListObjAppendElement(interp, my_own_private_unique_result,
	Tcl_NewDoubleObj((*_posterData).sensorPos.sensorToMain.var.vrz));
   if (ret != TCL_OK) {free(_posterData); return TCL_ERROR;}
   ret = Tcl_ListObjAppendElement(interp, my_own_private_unique_result,
	Tcl_NewDoubleObj((*_posterData).sensorPos.sensorToMain.var.vxz));
   if (ret != TCL_OK) {free(_posterData); return TCL_ERROR;}
   ret = Tcl_ListObjAppendElement(interp, my_own_private_unique_result,
	Tcl_NewDoubleObj((*_posterData).sensorPos.sensorToMain.var.vyz));
   if (ret != TCL_OK) {free(_posterData); return TCL_ERROR;}
   ret = Tcl_ListObjAppendElement(interp, my_own_private_unique_result,
	Tcl_NewDoubleObj((*_posterData).sensorPos.sensorToMain.var.vzz));
   if (ret != TCL_OK) {free(_posterData); return TCL_ERROR;}
   ret = Tcl_ListObjAppendElement(interp, my_own_private_unique_result,
	Tcl_NewDoubleObj((*_posterData).sensorPos.mainToBase.euler.yaw));
   if (ret != TCL_OK) {free(_posterData); return TCL_ERROR;}
   ret = Tcl_ListObjAppendElement(interp, my_own_private_unique_result,
	Tcl_NewDoubleObj((*_posterData).sensorPos.mainToBase.euler.pitch));
   if (ret != TCL_OK) {free(_posterData); return TCL_ERROR;}
   ret = Tcl_ListObjAppendElement(interp, my_own_private_unique_result,
	Tcl_NewDoubleObj((*_posterData).sensorPos.mainToBase.euler.roll));
   if (ret != TCL_OK) {free(_posterData); return TCL_ERROR;}
   ret = Tcl_ListObjAppendElement(interp, my_own_private_unique_result,
	Tcl_NewDoubleObj((*_posterData).sensorPos.mainToBase.euler.x));
   if (ret != TCL_OK) {free(_posterData); return TCL_ERROR;}
   ret = Tcl_ListObjAppendElement(interp, my_own_private_unique_result,
	Tcl_NewDoubleObj((*_posterData).sensorPos.mainToBase.euler.y));
   if (ret != TCL_OK) {free(_posterData); return TCL_ERROR;}
   ret = Tcl_ListObjAppendElement(interp, my_own_private_unique_result,
	Tcl_NewDoubleObj((*_posterData).sensorPos.mainToBase.euler.z));
   if (ret != TCL_OK) {free(_posterData); return TCL_ERROR;}
   ret = Tcl_ListObjAppendElement(interp, my_own_private_unique_result,
	Tcl_NewDoubleObj((*_posterData).sensorPos.mainToBase.var.voo));
   if (ret != TCL_OK) {free(_posterData); return TCL_ERROR;}
   ret = Tcl_ListObjAppendElement(interp, my_own_private_unique_result,
	Tcl_NewDoubleObj((*_posterData).sensorPos.mainToBase.var.vop));
   if (ret != TCL_OK) {free(_posterData); return TCL_ERROR;}
   ret = Tcl_ListObjAppendElement(interp, my_own_private_unique_result,
	Tcl_NewDoubleObj((*_posterData).sensorPos.mainToBase.var.vpp));
   if (ret != TCL_OK) {free(_posterData); return TCL_ERROR;}
   ret = Tcl_ListObjAppendElement(interp, my_own_private_unique_result,
	Tcl_NewDoubleObj((*_posterData).sensorPos.mainToBase.var.vor));
   if (ret != TCL_OK) {free(_posterData); return TCL_ERROR;}
   ret = Tcl_ListObjAppendElement(interp, my_own_private_unique_result,
	Tcl_NewDoubleObj((*_posterData).sensorPos.mainToBase.var.vpr));
   if (ret != TCL_OK) {free(_posterData); return TCL_ERROR;}
   ret = Tcl_ListObjAppendElement(interp, my_own_private_unique_result,
	Tcl_NewDoubleObj((*_posterData).sensorPos.mainToBase.var.vrr));
   if (ret != TCL_OK) {free(_posterData); return TCL_ERROR;}
   ret = Tcl_ListObjAppendElement(interp, my_own_private_unique_result,
	Tcl_NewDoubleObj((*_posterData).sensorPos.mainToBase.var.vox));
   if (ret != TCL_OK) {free(_posterData); return TCL_ERROR;}
   ret = Tcl_ListObjAppendElement(interp, my_own_private_unique_result,
	Tcl_NewDoubleObj((*_posterData).sensorPos.mainToBase.var.vpx));
   if (ret != TCL_OK) {free(_posterData); return TCL_ERROR;}
   ret = Tcl_ListObjAppendElement(interp, my_own_private_unique_result,
	Tcl_NewDoubleObj((*_posterData).sensorPos.mainToBase.var.vrx));
   if (ret != TCL_OK) {free(_posterData); return TCL_ERROR;}
   ret = Tcl_ListObjAppendElement(interp, my_own_private_unique_result,
	Tcl_NewDoubleObj((*_posterData).sensorPos.mainToBase.var.vxx));
   if (ret != TCL_OK) {free(_posterData); return TCL_ERROR;}
   ret = Tcl_ListObjAppendElement(interp, my_own_private_unique_result,
	Tcl_NewDoubleObj((*_posterData).sensorPos.mainToBase.var.voy));
   if (ret != TCL_OK) {free(_posterData); return TCL_ERROR;}
   ret = Tcl_ListObjAppendElement(interp, my_own_private_unique_result,
	Tcl_NewDoubleObj((*_posterData).sensorPos.mainToBase.var.vpy));
   if (ret != TCL_OK) {free(_posterData); return TCL_ERROR;}
   ret = Tcl_ListObjAppendElement(interp, my_own_private_unique_result,
	Tcl_NewDoubleObj((*_posterData).sensorPos.mainToBase.var.vry));
   if (ret != TCL_OK) {free(_posterData); return TCL_ERROR;}
   ret = Tcl_ListObjAppendElement(interp, my_own_private_unique_result,
	Tcl_NewDoubleObj((*_posterData).sensorPos.mainToBase.var.vxy));
   if (ret != TCL_OK) {free(_posterData); return TCL_ERROR;}
   ret = Tcl_ListObjAppendElement(interp, my_own_private_unique_result,
	Tcl_NewDoubleObj((*_posterData).sensorPos.mainToBase.var.vyy));
   if (ret != TCL_OK) {free(_posterData); return TCL_ERROR;}
   ret = Tcl_ListObjAppendElement(interp, my_own_private_unique_result,
	Tcl_NewDoubleObj((*_posterData).sensorPos.mainToBase.var.voz));
   if (ret != TCL_OK) {free(_posterData); return TCL_ERROR;}
   ret = Tcl_ListObjAppendElement(interp, my_own_private_unique_result,
	Tcl_NewDoubleObj((*_posterData).sensorPos.mainToBase.var.vpz));
   if (ret != TCL_OK) {free(_posterData); return TCL_ERROR;}
   ret = Tcl_ListObjAppendElement(interp, my_own_private_unique_result,
	Tcl_NewDoubleObj((*_posterData).sensorPos.mainToBase.var.vrz));
   if (ret != TCL_OK) {free(_posterData); return TCL_ERROR;}
   ret = Tcl_ListObjAppendElement(interp, my_own_private_unique_result,
	Tcl_NewDoubleObj((*_posterData).sensorPos.mainToBase.var.vxz));
   if (ret != TCL_OK) {free(_posterData); return TCL_ERROR;}
   ret = Tcl_ListObjAppendElement(interp, my_own_private_unique_result,
	Tcl_NewDoubleObj((*_posterData).sensorPos.mainToBase.var.vyz));
   if (ret != TCL_OK) {free(_posterData); return TCL_ERROR;}
   ret = Tcl_ListObjAppendElement(interp, my_own_private_unique_result,
	Tcl_NewDoubleObj((*_posterData).sensorPos.mainToBase.var.vzz));
   if (ret != TCL_OK) {free(_posterData); return TCL_ERROR;}
   ret = Tcl_ListObjAppendElement(interp, my_own_private_unique_result,
	Tcl_NewDoubleObj((*_posterData).sensorPos.mainToOrigin.euler.yaw));
   if (ret != TCL_OK) {free(_posterData); return TCL_ERROR;}
   ret = Tcl_ListObjAppendElement(interp, my_own_private_unique_result,
	Tcl_NewDoubleObj((*_posterData).sensorPos.mainToOrigin.euler.pitch));
   if (ret != TCL_OK) {free(_posterData); return TCL_ERROR;}
   ret = Tcl_ListObjAppendElement(interp, my_own_private_unique_result,
	Tcl_NewDoubleObj((*_posterData).sensorPos.mainToOrigin.euler.roll));
   if (ret != TCL_OK) {free(_posterData); return TCL_ERROR;}
   ret = Tcl_ListObjAppendElement(interp, my_own_private_unique_result,
	Tcl_NewDoubleObj((*_posterData).sensorPos.mainToOrigin.euler.x));
   if (ret != TCL_OK) {free(_posterData); return TCL_ERROR;}
   ret = Tcl_ListObjAppendElement(interp, my_own_private_unique_result,
	Tcl_NewDoubleObj((*_posterData).sensorPos.mainToOrigin.euler.y));
   if (ret != TCL_OK) {free(_posterData); return TCL_ERROR;}
   ret = Tcl_ListObjAppendElement(interp, my_own_private_unique_result,
	Tcl_NewDoubleObj((*_posterData).sensorPos.mainToOrigin.euler.z));
   if (ret != TCL_OK) {free(_posterData); return TCL_ERROR;}
   ret = Tcl_ListObjAppendElement(interp, my_own_private_unique_result,
	Tcl_NewDoubleObj((*_posterData).sensorPos.mainToOrigin.var.voo));
   if (ret != TCL_OK) {free(_posterData); return TCL_ERROR;}
   ret = Tcl_ListObjAppendElement(interp, my_own_private_unique_result,
	Tcl_NewDoubleObj((*_posterData).sensorPos.mainToOrigin.var.vop));
   if (ret != TCL_OK) {free(_posterData); return TCL_ERROR;}
   ret = Tcl_ListObjAppendElement(interp, my_own_private_unique_result,
	Tcl_NewDoubleObj((*_posterData).sensorPos.mainToOrigin.var.vpp));
   if (ret != TCL_OK) {free(_posterData); return TCL_ERROR;}
   ret = Tcl_ListObjAppendElement(interp, my_own_private_unique_result,
	Tcl_NewDoubleObj((*_posterData).sensorPos.mainToOrigin.var.vor));
   if (ret != TCL_OK) {free(_posterData); return TCL_ERROR;}
   ret = Tcl_ListObjAppendElement(interp, my_own_private_unique_result,
	Tcl_NewDoubleObj((*_posterData).sensorPos.mainToOrigin.var.vpr));
   if (ret != TCL_OK) {free(_posterData); return TCL_ERROR;}
   ret = Tcl_ListObjAppendElement(interp, my_own_private_unique_result,
	Tcl_NewDoubleObj((*_posterData).sensorPos.mainToOrigin.var.vrr));
   if (ret != TCL_OK) {free(_posterData); return TCL_ERROR;}
   ret = Tcl_ListObjAppendElement(interp, my_own_private_unique_result,
	Tcl_NewDoubleObj((*_posterData).sensorPos.mainToOrigin.var.vox));
   if (ret != TCL_OK) {free(_posterData); return TCL_ERROR;}
   ret = Tcl_ListObjAppendElement(interp, my_own_private_unique_result,
	Tcl_NewDoubleObj((*_posterData).sensorPos.mainToOrigin.var.vpx));
   if (ret != TCL_OK) {free(_posterData); return TCL_ERROR;}
   ret = Tcl_ListObjAppendElement(interp, my_own_private_unique_result,
	Tcl_NewDoubleObj((*_posterData).sensorPos.mainToOrigin.var.vrx));
   if (ret != TCL_OK) {free(_posterData); return TCL_ERROR;}
   ret = Tcl_ListObjAppendElement(interp, my_own_private_unique_result,
	Tcl_NewDoubleObj((*_posterData).sensorPos.mainToOrigin.var.vxx));
   if (ret != TCL_OK) {free(_posterData); return TCL_ERROR;}
   ret = Tcl_ListObjAppendElement(interp, my_own_private_unique_result,
	Tcl_NewDoubleObj((*_posterData).sensorPos.mainToOrigin.var.voy));
   if (ret != TCL_OK) {free(_posterData); return TCL_ERROR;}
   ret = Tcl_ListObjAppendElement(interp, my_own_private_unique_result,
	Tcl_NewDoubleObj((*_posterData).sensorPos.mainToOrigin.var.vpy));
   if (ret != TCL_OK) {free(_posterData); return TCL_ERROR;}
   ret = Tcl_ListObjAppendElement(interp, my_own_private_unique_result,
	Tcl_NewDoubleObj((*_posterData).sensorPos.mainToOrigin.var.vry));
   if (ret != TCL_OK) {free(_posterData); return TCL_ERROR;}
   ret = Tcl_ListObjAppendElement(interp, my_own_private_unique_result,
	Tcl_NewDoubleObj((*_posterData).sensorPos.mainToOrigin.var.vxy));
   if (ret != TCL_OK) {free(_posterData); return TCL_ERROR;}
   ret = Tcl_ListObjAppendElement(interp, my_own_private_unique_result,
	Tcl_NewDoubleObj((*_posterData).sensorPos.mainToOrigin.var.vyy));
   if (ret != TCL_OK) {free(_posterData); return TCL_ERROR;}
   ret = Tcl_ListObjAppendElement(interp, my_own_private_unique_result,
	Tcl_NewDoubleObj((*_posterData).sensorPos.mainToOrigin.var.voz));
   if (ret != TCL_OK) {free(_posterData); return TCL_ERROR;}
   ret = Tcl_ListObjAppendElement(interp, my_own_private_unique_result,
	Tcl_NewDoubleObj((*_posterData).sensorPos.mainToOrigin.var.vpz));
   if (ret != TCL_OK) {free(_posterData); return TCL_ERROR;}
   ret = Tcl_ListObjAppendElement(interp, my_own_private_unique_result,
	Tcl_NewDoubleObj((*_posterData).sensorPos.mainToOrigin.var.vrz));
   if (ret != TCL_OK) {free(_posterData); return TCL_ERROR;}
   ret = Tcl_ListObjAppendElement(interp, my_own_private_unique_result,
	Tcl_NewDoubleObj((*_posterData).sensorPos.mainToOrigin.var.vxz));
   if (ret != TCL_OK) {free(_posterData); return TCL_ERROR;}
   ret = Tcl_ListObjAppendElement(interp, my_own_private_unique_result,
	Tcl_NewDoubleObj((*_posterData).sensorPos.mainToOrigin.var.vyz));
   if (ret != TCL_OK) {free(_posterData); return TCL_ERROR;}
   ret = Tcl_ListObjAppendElement(interp, my_own_private_unique_result,
	Tcl_NewDoubleObj((*_posterData).sensorPos.mainToOrigin.var.vzz));
   if (ret != TCL_OK) {free(_posterData); return TCL_ERROR;}
   ret = Tcl_ListObjAppendElement(interp, my_own_private_unique_result,
	Tcl_NewDoubleObj((*_posterData).sensorPos.VLocal.euler.yaw));
   if (ret != TCL_OK) {free(_posterData); return TCL_ERROR;}
   ret = Tcl_ListObjAppendElement(interp, my_own_private_unique_result,
	Tcl_NewDoubleObj((*_posterData).sensorPos.VLocal.euler.pitch));
   if (ret != TCL_OK) {free(_posterData); return TCL_ERROR;}
   ret = Tcl_ListObjAppendElement(interp, my_own_private_unique_result,
	Tcl_NewDoubleObj((*_posterData).sensorPos.VLocal.euler.roll));
   if (ret != TCL_OK) {free(_posterData); return TCL_ERROR;}
   ret = Tcl_ListObjAppendElement(interp, my_own_private_unique_result,
	Tcl_NewDoubleObj((*_posterData).sensorPos.VLocal.euler.x));
   if (ret != TCL_OK) {free(_posterData); return TCL_ERROR;}
   ret = Tcl_ListObjAppendElement(interp, my_own_private_unique_result,
	Tcl_NewDoubleObj((*_posterData).sensorPos.VLocal.euler.y));
   if (ret != TCL_OK) {free(_posterData); return TCL_ERROR;}
   ret = Tcl_ListObjAppendElement(interp, my_own_private_unique_result,
	Tcl_NewDoubleObj((*_posterData).sensorPos.VLocal.euler.z));
   if (ret != TCL_OK) {free(_posterData); return TCL_ERROR;}
   ret = Tcl_ListObjAppendElement(interp, my_own_private_unique_result,
	Tcl_NewDoubleObj((*_posterData).sensorPos.VLocal.var.voo));
   if (ret != TCL_OK) {free(_posterData); return TCL_ERROR;}
   ret = Tcl_ListObjAppendElement(interp, my_own_private_unique_result,
	Tcl_NewDoubleObj((*_posterData).sensorPos.VLocal.var.vop));
   if (ret != TCL_OK) {free(_posterData); return TCL_ERROR;}
   ret = Tcl_ListObjAppendElement(interp, my_own_private_unique_result,
	Tcl_NewDoubleObj((*_posterData).sensorPos.VLocal.var.vpp));
   if (ret != TCL_OK) {free(_posterData); return TCL_ERROR;}
   ret = Tcl_ListObjAppendElement(interp, my_own_private_unique_result,
	Tcl_NewDoubleObj((*_posterData).sensorPos.VLocal.var.vor));
   if (ret != TCL_OK) {free(_posterData); return TCL_ERROR;}
   ret = Tcl_ListObjAppendElement(interp, my_own_private_unique_result,
	Tcl_NewDoubleObj((*_posterData).sensorPos.VLocal.var.vpr));
   if (ret != TCL_OK) {free(_posterData); return TCL_ERROR;}
   ret = Tcl_ListObjAppendElement(interp, my_own_private_unique_result,
	Tcl_NewDoubleObj((*_posterData).sensorPos.VLocal.var.vrr));
   if (ret != TCL_OK) {free(_posterData); return TCL_ERROR;}
   ret = Tcl_ListObjAppendElement(interp, my_own_private_unique_result,
	Tcl_NewDoubleObj((*_posterData).sensorPos.VLocal.var.vox));
   if (ret != TCL_OK) {free(_posterData); return TCL_ERROR;}
   ret = Tcl_ListObjAppendElement(interp, my_own_private_unique_result,
	Tcl_NewDoubleObj((*_posterData).sensorPos.VLocal.var.vpx));
   if (ret != TCL_OK) {free(_posterData); return TCL_ERROR;}
   ret = Tcl_ListObjAppendElement(interp, my_own_private_unique_result,
	Tcl_NewDoubleObj((*_posterData).sensorPos.VLocal.var.vrx));
   if (ret != TCL_OK) {free(_posterData); return TCL_ERROR;}
   ret = Tcl_ListObjAppendElement(interp, my_own_private_unique_result,
	Tcl_NewDoubleObj((*_posterData).sensorPos.VLocal.var.vxx));
   if (ret != TCL_OK) {free(_posterData); return TCL_ERROR;}
   ret = Tcl_ListObjAppendElement(interp, my_own_private_unique_result,
	Tcl_NewDoubleObj((*_posterData).sensorPos.VLocal.var.voy));
   if (ret != TCL_OK) {free(_posterData); return TCL_ERROR;}
   ret = Tcl_ListObjAppendElement(interp, my_own_private_unique_result,
	Tcl_NewDoubleObj((*_posterData).sensorPos.VLocal.var.vpy));
   if (ret != TCL_OK) {free(_posterData); return TCL_ERROR;}
   ret = Tcl_ListObjAppendElement(interp, my_own_private_unique_result,
	Tcl_NewDoubleObj((*_posterData).sensorPos.VLocal.var.vry));
   if (ret != TCL_OK) {free(_posterData); return TCL_ERROR;}
   ret = Tcl_ListObjAppendElement(interp, my_own_private_unique_result,
	Tcl_NewDoubleObj((*_posterData).sensorPos.VLocal.var.vxy));
   if (ret != TCL_OK) {free(_posterData); return TCL_ERROR;}
   ret = Tcl_ListObjAppendElement(interp, my_own_private_unique_result,
	Tcl_NewDoubleObj((*_posterData).sensorPos.VLocal.var.vyy));
   if (ret != TCL_OK) {free(_posterData); return TCL_ERROR;}
   ret = Tcl_ListObjAppendElement(interp, my_own_private_unique_result,
	Tcl_NewDoubleObj((*_posterData).sensorPos.VLocal.var.voz));
   if (ret != TCL_OK) {free(_posterData); return TCL_ERROR;}
   ret = Tcl_ListObjAppendElement(interp, my_own_private_unique_result,
	Tcl_NewDoubleObj((*_posterData).sensorPos.VLocal.var.vpz));
   if (ret != TCL_OK) {free(_posterData); return TCL_ERROR;}
   ret = Tcl_ListObjAppendElement(interp, my_own_private_unique_result,
	Tcl_NewDoubleObj((*_posterData).sensorPos.VLocal.var.vrz));
   if (ret != TCL_OK) {free(_posterData); return TCL_ERROR;}
   ret = Tcl_ListObjAppendElement(interp, my_own_private_unique_result,
	Tcl_NewDoubleObj((*_posterData).sensorPos.VLocal.var.vxz));
   if (ret != TCL_OK) {free(_posterData); return TCL_ERROR;}
   ret = Tcl_ListObjAppendElement(interp, my_own_private_unique_result,
	Tcl_NewDoubleObj((*_posterData).sensorPos.VLocal.var.vyz));
   if (ret != TCL_OK) {free(_posterData); return TCL_ERROR;}
   ret = Tcl_ListObjAppendElement(interp, my_own_private_unique_result,
	Tcl_NewDoubleObj((*_posterData).sensorPos.VLocal.var.vzz));
   if (ret != TCL_OK) {free(_posterData); return TCL_ERROR;}
   ret = Tcl_ListObjAppendElement(interp, my_own_private_unique_result,
	Tcl_NewIntObj((*_posterData).currentModality));
   if (ret != TCL_OK) {free(_posterData); return TCL_ERROR;}
   ret = Tcl_ListObjAppendElement(interp, my_own_private_unique_result,
	Tcl_NewIntObj((*_posterData).number_of_detection));
   if (ret != TCL_OK) {free(_posterData); return TCL_ERROR;}
   {
      int loop1[1];
      for(loop1[0]=0; loop1[0]<64; loop1[0]++) {
   ret = Tcl_ListObjAppendElement(interp, my_own_private_unique_result,
	Tcl_NewDoubleObj((*_posterData).personPos[loop1[0]].x));
   if (ret != TCL_OK) {free(_posterData); return TCL_ERROR;}
   ret = Tcl_ListObjAppendElement(interp, my_own_private_unique_result,
	Tcl_NewDoubleObj((*_posterData).personPos[loop1[0]].y));
   if (ret != TCL_OK) {free(_posterData); return TCL_ERROR;}
   ret = Tcl_ListObjAppendElement(interp, my_own_private_unique_result,
	Tcl_NewDoubleObj((*_posterData).personPos[loop1[0]].z));
   if (ret != TCL_OK) {free(_posterData); return TCL_ERROR;}
   ret = Tcl_ListObjAppendElement(interp, my_own_private_unique_result,
	Tcl_NewIntObj((*_posterData).personPos[loop1[0]].id));
   if (ret != TCL_OK) {free(_posterData); return TCL_ERROR;}
   ret = Tcl_ListObjAppendElement(interp, my_own_private_unique_result,
	Tcl_NewStringObj((*_posterData).personPos[loop1[0]].name, -1));
   if (ret != TCL_OK) {free(_posterData); return TCL_ERROR;}
      }
   }


   Tcl_SetObjResult(interp, my_own_private_unique_result);
   free(_posterData);
   return TCL_OK;
}

/* --- FILE GENERATED WITH GENOM, DO NOT EDIT BY HAND ------------------ */

/* 
 * Copyright (c) 1993-2005 LAAS/CNRS
 * All rights reserved.
 *
 * Redistribution and use  in source  and binary  forms,  with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *   1. Redistributions of  source  code must retain the  above copyright
 *      notice, this list of conditions and the following disclaimer.
 *   2. Redistributions in binary form must reproduce the above copyright
 *      notice,  this list of  conditions and the following disclaimer in
 *      the  documentation  and/or  other   materials provided  with  the
 *      distribution.
 *
 * THIS  SOFTWARE IS PROVIDED BY  THE  COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND  ANY  EXPRESS OR IMPLIED  WARRANTIES,  INCLUDING,  BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES  OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR  PURPOSE ARE DISCLAIMED. IN  NO EVENT SHALL THE COPYRIGHT
 * HOLDERS OR      CONTRIBUTORS  BE LIABLE FOR   ANY    DIRECT, INDIRECT,
 * INCIDENTAL,  SPECIAL,  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF  SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN  CONTRACT, STRICT LIABILITY, OR
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE   OF THIS SOFTWARE, EVEN   IF ADVISED OF   THE POSSIBILITY OF SUCH
 * DAMAGE.
 */

/*
 * ----------------------------------------------------------------------
 * HumanPresence Poster
 */

static int icuHumanPresenceloadedPersonListPosterTclRead(ClientData, Tcl_Interp *, int, Tcl_Obj *const []); 

static int
icuHumanPresenceloadedPersonListPosterTclRead(ClientData data, Tcl_Interp *interp,
			 int objc, Tcl_Obj *const objv[])
{
   int ret;
   static struct ICU_RECOGNITION_WHOLIST *_posterData;	/* data */
   Tcl_Obj *my_own_private_unique_result;
   char strerr[64];

   TEST_BAD_USAGE(objc != 1);
		 
  if ((_posterData = malloc(sizeof(struct ICU_RECOGNITION_WHOLIST))) == NULL) {
      Tcl_SetResult(interp, h2getErrMsg(errnoGet(), strerr, 64), TCL_VOLATILE);
      return TCL_ERROR;
  }

   if (icuHumanPresenceloadedPersonListPosterRead(_posterData) != OK) {
      Tcl_SetResult(interp, h2getErrMsg(errnoGet(), strerr, 64), TCL_VOLATILE);
      free(_posterData);
      return TCL_ERROR;
   }

   my_own_private_unique_result = Tcl_NewListObj(0, NULL);
   if (my_own_private_unique_result == NULL) {
     free(_posterData);
     return TCL_ERROR;
   }

   ret = Tcl_ListObjAppendElement(interp, my_own_private_unique_result,
	Tcl_NewIntObj((*_posterData).nb));
   if (ret != TCL_OK) {free(_posterData); return TCL_ERROR;}
   ret = Tcl_ListObjAppendElement(interp, my_own_private_unique_result,
	Tcl_NewIntObj((*_posterData).unused));
   if (ret != TCL_OK) {free(_posterData); return TCL_ERROR;}
   {
      int loop1[1];
      for(loop1[0]=0; loop1[0]<24; loop1[0]++) {
   ret = Tcl_ListObjAppendElement(interp, my_own_private_unique_result,
	Tcl_NewStringObj((*_posterData).who[loop1[0]].name, -1));
   if (ret != TCL_OK) {free(_posterData); return TCL_ERROR;}
   ret = Tcl_ListObjAppendElement(interp, my_own_private_unique_result,
	Tcl_NewIntObj((*_posterData).who[loop1[0]].present));
   if (ret != TCL_OK) {free(_posterData); return TCL_ERROR;}
   ret = Tcl_ListObjAppendElement(interp, my_own_private_unique_result,
	Tcl_NewDoubleObj((*_posterData).who[loop1[0]].likelihood));
   if (ret != TCL_OK) {free(_posterData); return TCL_ERROR;}
   {
      int loop2[1];
      for(loop2[0]=0; loop2[0]<56; loop2[0]++) {
         ret = Tcl_ListObjAppendElement(interp, my_own_private_unique_result,
	Tcl_NewDoubleObj((*_posterData).who[loop1[0]].reliability[loop2[0]]));
         if (ret != TCL_OK) {free(_posterData); return TCL_ERROR;}
      }
   }
   {
      int loop2[1];
      for(loop2[0]=0; loop2[0]<56; loop2[0]++) {
         ret = Tcl_ListObjAppendElement(interp, my_own_private_unique_result,
	Tcl_NewIntObj((*_posterData).who[loop1[0]].window[loop2[0]]));
         if (ret != TCL_OK) {free(_posterData); return TCL_ERROR;}
      }
   }
   ret = Tcl_ListObjAppendElement(interp, my_own_private_unique_result,
	Tcl_NewDoubleObj((*_posterData).who[loop1[0]].cumul));
   if (ret != TCL_OK) {free(_posterData); return TCL_ERROR;}
   ret = Tcl_ListObjAppendElement(interp, my_own_private_unique_result,
	Tcl_NewIntObj((*_posterData).who[loop1[0]].cptAP));
   if (ret != TCL_OK) {free(_posterData); return TCL_ERROR;}
   ret = Tcl_ListObjAppendElement(interp, my_own_private_unique_result,
	Tcl_NewIntObj((*_posterData).who[loop1[0]].curid));
   if (ret != TCL_OK) {free(_posterData); return TCL_ERROR;}
   ret = Tcl_ListObjAppendElement(interp, my_own_private_unique_result,
	Tcl_NewIntObj((*_posterData).who[loop1[0]].updated));
   if (ret != TCL_OK) {free(_posterData); return TCL_ERROR;}
      }
   }


   Tcl_SetObjResult(interp, my_own_private_unique_result);
   free(_posterData);
   return TCL_OK;
}

/* --- FILE GENERATED WITH GENOM, DO NOT EDIT BY HAND ------------------ */

/* 
 * Copyright (c) 1993-2005 LAAS/CNRS
 * All rights reserved.
 *
 * Redistribution and use  in source  and binary  forms,  with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *   1. Redistributions of  source  code must retain the  above copyright
 *      notice, this list of conditions and the following disclaimer.
 *   2. Redistributions in binary form must reproduce the above copyright
 *      notice,  this list of  conditions and the following disclaimer in
 *      the  documentation  and/or  other   materials provided  with  the
 *      distribution.
 *
 * THIS  SOFTWARE IS PROVIDED BY  THE  COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND  ANY  EXPRESS OR IMPLIED  WARRANTIES,  INCLUDING,  BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES  OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR  PURPOSE ARE DISCLAIMED. IN  NO EVENT SHALL THE COPYRIGHT
 * HOLDERS OR      CONTRIBUTORS  BE LIABLE FOR   ANY    DIRECT, INDIRECT,
 * INCIDENTAL,  SPECIAL,  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF  SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN  CONTRACT, STRICT LIABILITY, OR
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE   OF THIS SOFTWARE, EVEN   IF ADVISED OF   THE POSSIBILITY OF SUCH
 * DAMAGE.
 */

/*
 * ----------------------------------------------------------------------
 * PlatineCmd Poster
 */

static int icuPlatineCmdcmdPlatinePosterTclRead(ClientData, Tcl_Interp *, int, Tcl_Obj *const []); 

static int
icuPlatineCmdcmdPlatinePosterTclRead(ClientData data, Tcl_Interp *interp,
			 int objc, Tcl_Obj *const objv[])
{
   int ret;
   static struct PLATINE_AXIS_STR *_posterData;	/* data */
   Tcl_Obj *my_own_private_unique_result;
   char strerr[64];

   TEST_BAD_USAGE(objc != 1);
		 
  if ((_posterData = malloc(sizeof(struct PLATINE_AXIS_STR))) == NULL) {
      Tcl_SetResult(interp, h2getErrMsg(errnoGet(), strerr, 64), TCL_VOLATILE);
      return TCL_ERROR;
  }

   if (icuPlatineCmdcmdPlatinePosterRead(_posterData) != OK) {
      Tcl_SetResult(interp, h2getErrMsg(errnoGet(), strerr, 64), TCL_VOLATILE);
      free(_posterData);
      return TCL_ERROR;
   }

   my_own_private_unique_result = Tcl_NewListObj(0, NULL);
   if (my_own_private_unique_result == NULL) {
     free(_posterData);
     return TCL_ERROR;
   }

   ret = Tcl_ListObjAppendElement(interp, my_own_private_unique_result,
	Tcl_NewDoubleObj((*_posterData).pan));
   if (ret != TCL_OK) {free(_posterData); return TCL_ERROR;}
   ret = Tcl_ListObjAppendElement(interp, my_own_private_unique_result,
	Tcl_NewDoubleObj((*_posterData).tilt));
   if (ret != TCL_OK) {free(_posterData); return TCL_ERROR;}


   Tcl_SetObjResult(interp, my_own_private_unique_result);
   free(_posterData);
   return TCL_OK;
}

/* --- FILE GENERATED WITH GENOM, DO NOT EDIT BY HAND ------------------ */

/* 
 * Copyright (c) 1993-2005 LAAS/CNRS
 * All rights reserved.
 *
 * Redistribution and use  in source  and binary  forms,  with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *   1. Redistributions of  source  code must retain the  above copyright
 *      notice, this list of conditions and the following disclaimer.
 *   2. Redistributions in binary form must reproduce the above copyright
 *      notice,  this list of  conditions and the following disclaimer in
 *      the  documentation  and/or  other   materials provided  with  the
 *      distribution.
 *
 * THIS  SOFTWARE IS PROVIDED BY  THE  COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND  ANY  EXPRESS OR IMPLIED  WARRANTIES,  INCLUDING,  BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES  OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR  PURPOSE ARE DISCLAIMED. IN  NO EVENT SHALL THE COPYRIGHT
 * HOLDERS OR      CONTRIBUTORS  BE LIABLE FOR   ANY    DIRECT, INDIRECT,
 * INCIDENTAL,  SPECIAL,  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF  SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN  CONTRACT, STRICT LIABILITY, OR
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE   OF THIS SOFTWARE, EVEN   IF ADVISED OF   THE POSSIBILITY OF SUCH
 * DAMAGE.
 */

/*
 * ----------------------------------------------------------------------
 * VisualData Poster
 */

static int icuVisualDataDataPosterTclRead(ClientData, Tcl_Interp *, int, Tcl_Obj *const []); 

static int
icuVisualDataDataPosterTclRead(ClientData data, Tcl_Interp *interp,
			 int objc, Tcl_Obj *const objv[])
{
   int ret;
   static struct ICU_VISUAL_DATA *_posterData;	/* data */
   Tcl_Obj *my_own_private_unique_result;
   char strerr[64];

   TEST_BAD_USAGE(objc != 1);
		 
  if ((_posterData = malloc(sizeof(struct ICU_VISUAL_DATA))) == NULL) {
      Tcl_SetResult(interp, h2getErrMsg(errnoGet(), strerr, 64), TCL_VOLATILE);
      return TCL_ERROR;
  }

   if (icuVisualDataDataPosterRead(_posterData) != OK) {
      Tcl_SetResult(interp, h2getErrMsg(errnoGet(), strerr, 64), TCL_VOLATILE);
      free(_posterData);
      return TCL_ERROR;
   }

   my_own_private_unique_result = Tcl_NewListObj(0, NULL);
   if (my_own_private_unique_result == NULL) {
     free(_posterData);
     return TCL_ERROR;
   }

   ret = Tcl_ListObjAppendElement(interp, my_own_private_unique_result,
	Tcl_NewDoubleObj((*_posterData).u));
   if (ret != TCL_OK) {free(_posterData); return TCL_ERROR;}
   ret = Tcl_ListObjAppendElement(interp, my_own_private_unique_result,
	Tcl_NewDoubleObj((*_posterData).v));
   if (ret != TCL_OK) {free(_posterData); return TCL_ERROR;}
   ret = Tcl_ListObjAppendElement(interp, my_own_private_unique_result,
	Tcl_NewDoubleObj((*_posterData).scale));
   if (ret != TCL_OK) {free(_posterData); return TCL_ERROR;}
   ret = Tcl_ListObjAppendElement(interp, my_own_private_unique_result,
	Tcl_NewDoubleObj((*_posterData).rfidT));
   if (ret != TCL_OK) {free(_posterData); return TCL_ERROR;}
   ret = Tcl_ListObjAppendElement(interp, my_own_private_unique_result,
	Tcl_NewDoubleObj((*_posterData).rfidZ));
   if (ret != TCL_OK) {free(_posterData); return TCL_ERROR;}
   ret = Tcl_ListObjAppendElement(interp, my_own_private_unique_result,
	Tcl_NewDoubleObj((*_posterData).pan));
   if (ret != TCL_OK) {free(_posterData); return TCL_ERROR;}
   ret = Tcl_ListObjAppendElement(interp, my_own_private_unique_result,
	Tcl_NewDoubleObj((*_posterData).tilt));
   if (ret != TCL_OK) {free(_posterData); return TCL_ERROR;}
   {
      int loop1[1];
      for(loop1[0]=0; loop1[0]<4; loop1[0]++) {
         ret = Tcl_ListObjAppendElement(interp, my_own_private_unique_result,
	Tcl_NewDoubleObj((*_posterData).x[loop1[0]]));
         if (ret != TCL_OK) {free(_posterData); return TCL_ERROR;}
      }
   }
   {
      int loop1[1];
      for(loop1[0]=0; loop1[0]<4; loop1[0]++) {
         ret = Tcl_ListObjAppendElement(interp, my_own_private_unique_result,
	Tcl_NewDoubleObj((*_posterData).y[loop1[0]]));
         if (ret != TCL_OK) {free(_posterData); return TCL_ERROR;}
      }
   }
   {
      int loop1[1];
      for(loop1[0]=0; loop1[0]<4; loop1[0]++) {
         ret = Tcl_ListObjAppendElement(interp, my_own_private_unique_result,
	Tcl_NewDoubleObj((*_posterData).z[loop1[0]]));
         if (ret != TCL_OK) {free(_posterData); return TCL_ERROR;}
      }
   }
   ret = Tcl_ListObjAppendElement(interp, my_own_private_unique_result,
	Tcl_NewIntObj((*_posterData).nbSpots));
   if (ret != TCL_OK) {free(_posterData); return TCL_ERROR;}
   ret = Tcl_ListObjAppendElement(interp, my_own_private_unique_result,
	Tcl_NewIntObj((*_posterData).numRef));
   if (ret != TCL_OK) {free(_posterData); return TCL_ERROR;}


   Tcl_SetObjResult(interp, my_own_private_unique_result);
   free(_posterData);
   return TCL_OK;
}

/* 
 * Copyright (c) 1993-2003 LAAS/CNRS
 * All rights reserved.
 *
 * Redistribution and use  in source  and binary  forms,  with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *   1. Redistributions of  source  code must retain the  above copyright
 *      notice, this list of conditions and the following disclaimer.
 *   2. Redistributions in binary form must reproduce the above copyright
 *      notice,  this list of  conditions and the following disclaimer in
 *      the  documentation  and/or  other   materials provided  with  the
 *      distribution.
 *
 * THIS  SOFTWARE IS PROVIDED BY  THE  COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND  ANY  EXPRESS OR IMPLIED  WARRANTIES,  INCLUDING,  BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES  OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR  PURPOSE ARE DISCLAIMED. IN  NO EVENT SHALL THE COPYRIGHT
 * HOLDERS OR      CONTRIBUTORS  BE LIABLE FOR   ANY    DIRECT, INDIRECT,
 * INCIDENTAL,  SPECIAL,  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF  SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN  CONTRACT, STRICT LIABILITY, OR
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE   OF THIS SOFTWARE, EVEN   IF ADVISED OF   THE POSSIBILITY OF SUCH
 * DAMAGE.
 */

/*
 * ----------------------------------------------------------------------
 * icu installation
 */

int
Icu_Init(Tcl_Interp *interp)
{
   char buf[20];
   int code;

   code = Tcl_PkgProvide(interp, "Icu", "1.0");
   if (code != TCL_OK) return code;

   Tcl_CreateObjCommand(interp, "::icu::connect",
			icuClientInitCb, NULL, NULL);

   sprintf(buf, "%d", (int)ICU_CLIENT_MBOX_REPLY_SIZE);
   Tcl_SetVar(interp, "::icu::mboxSize", buf, TCL_GLOBAL_ONLY);

   Tcl_SetVar(interp, "::icu::datadir", DATADIR, TCL_GLOBAL_ONLY);

   return TCL_OK;
}

static int
icuInstall(Tcl_Interp *interp, struct ModuleInfo *m, int install)
{
   int offset;
   char name[73];

   strcpy(name, "::");
   strcat(name, m->name);
   strcat(name, "::");
   offset = strlen(name);

   if (install) {
      strcpy(name+offset, "disconnect");
      Tcl_CreateObjCommand(interp, name, icuClientEndCb, m, NULL);

      strcpy(name+offset, "AbortSend");
      Tcl_CreateObjCommand(interp, name, icuAbortRqstSendCb, m, NULL);
      strcpy(name+offset, "AbortRcv");
      Tcl_CreateObjCommand(interp, name, icuAbortReplyRcvCb, m, NULL);
      strcpy(name+offset, "InitDisplaySend");
      Tcl_CreateObjCommand(interp, name, icuInitDisplayRqstSendCb, m, NULL);
      strcpy(name+offset, "InitDisplayRcv");
      Tcl_CreateObjCommand(interp, name, icuInitDisplayReplyRcvCb, m, NULL);
      strcpy(name+offset, "PomTaggingSend");
      Tcl_CreateObjCommand(interp, name, icuPomTaggingRqstSendCb, m, NULL);
      strcpy(name+offset, "PomTaggingRcv");
      Tcl_CreateObjCommand(interp, name, icuPomTaggingReplyRcvCb, m, NULL);
      strcpy(name+offset, "AutoModeSend");
      Tcl_CreateObjCommand(interp, name, icuAutoModeRqstSendCb, m, NULL);
      strcpy(name+offset, "AutoModeRcv");
      Tcl_CreateObjCommand(interp, name, icuAutoModeReplyRcvCb, m, NULL);
      strcpy(name+offset, "SaveFaceSend");
      Tcl_CreateObjCommand(interp, name, icuSaveFaceRqstSendCb, m, NULL);
      strcpy(name+offset, "SaveFaceRcv");
      Tcl_CreateObjCommand(interp, name, icuSaveFaceReplyRcvCb, m, NULL);
      strcpy(name+offset, "CaptureImageSend");
      Tcl_CreateObjCommand(interp, name, icuCaptureImageRqstSendCb, m, NULL);
      strcpy(name+offset, "CaptureImageRcv");
      Tcl_CreateObjCommand(interp, name, icuCaptureImageReplyRcvCb, m, NULL);
      strcpy(name+offset, "CaptureVideoSend");
      Tcl_CreateObjCommand(interp, name, icuCaptureVideoRqstSendCb, m, NULL);
      strcpy(name+offset, "CaptureVideoRcv");
      Tcl_CreateObjCommand(interp, name, icuCaptureVideoReplyRcvCb, m, NULL);
      strcpy(name+offset, "SetViamPosterNameSend");
      Tcl_CreateObjCommand(interp, name, icuSetViamPosterNameRqstSendCb, m, NULL);
      strcpy(name+offset, "SetViamPosterNameRcv");
      Tcl_CreateObjCommand(interp, name, icuSetViamPosterNameReplyRcvCb, m, NULL);
      strcpy(name+offset, "EnableDisplaySend");
      Tcl_CreateObjCommand(interp, name, icuEnableDisplayRqstSendCb, m, NULL);
      strcpy(name+offset, "EnableDisplayRcv");
      Tcl_CreateObjCommand(interp, name, icuEnableDisplayReplyRcvCb, m, NULL);
      strcpy(name+offset, "SetCameraPosSend");
      Tcl_CreateObjCommand(interp, name, icuSetCameraPosRqstSendCb, m, NULL);
      strcpy(name+offset, "SetCameraPosRcv");
      Tcl_CreateObjCommand(interp, name, icuSetCameraPosReplyRcvCb, m, NULL);
      strcpy(name+offset, "TrackItSend");
      Tcl_CreateObjCommand(interp, name, icuTrackItRqstSendCb, m, NULL);
      strcpy(name+offset, "TrackItRcv");
      Tcl_CreateObjCommand(interp, name, icuTrackItReplyRcvCb, m, NULL);
      strcpy(name+offset, "WaitForSomebodySend");
      Tcl_CreateObjCommand(interp, name, icuWaitForSomebodyRqstSendCb, m, NULL);
      strcpy(name+offset, "WaitForSomebodyRcv");
      Tcl_CreateObjCommand(interp, name, icuWaitForSomebodyReplyRcvCb, m, NULL);
      strcpy(name+offset, "WaitForSomeoneSend");
      Tcl_CreateObjCommand(interp, name, icuWaitForSomeoneRqstSendCb, m, NULL);
      strcpy(name+offset, "WaitForSomeoneRcv");
      Tcl_CreateObjCommand(interp, name, icuWaitForSomeoneReplyRcvCb, m, NULL);
      strcpy(name+offset, "WaitForNobodySend");
      Tcl_CreateObjCommand(interp, name, icuWaitForNobodyRqstSendCb, m, NULL);
      strcpy(name+offset, "WaitForNobodyRcv");
      Tcl_CreateObjCommand(interp, name, icuWaitForNobodyReplyRcvCb, m, NULL);
      strcpy(name+offset, "WaitForNoneSend");
      Tcl_CreateObjCommand(interp, name, icuWaitForNoneRqstSendCb, m, NULL);
      strcpy(name+offset, "WaitForNoneRcv");
      Tcl_CreateObjCommand(interp, name, icuWaitForNoneReplyRcvCb, m, NULL);
      strcpy(name+offset, "WaitForNobodyKnownSend");
      Tcl_CreateObjCommand(interp, name, icuWaitForNobodyKnownRqstSendCb, m, NULL);
      strcpy(name+offset, "WaitForNobodyKnownRcv");
      Tcl_CreateObjCommand(interp, name, icuWaitForNobodyKnownReplyRcvCb, m, NULL);
      strcpy(name+offset, "WaitForPersonListChangeSend");
      Tcl_CreateObjCommand(interp, name, icuWaitForPersonListChangeRqstSendCb, m, NULL);
      strcpy(name+offset, "WaitForPersonListChangeRcv");
      Tcl_CreateObjCommand(interp, name, icuWaitForPersonListChangeReplyRcvCb, m, NULL);
      strcpy(name+offset, "ProcessSetModalitySend");
      Tcl_CreateObjCommand(interp, name, icuProcessSetModalityRqstSendCb, m, NULL);
      strcpy(name+offset, "ProcessSetModalityRcv");
      Tcl_CreateObjCommand(interp, name, icuProcessSetModalityReplyRcvCb, m, NULL);
      strcpy(name+offset, "FacerecSetWorkingPathSend");
      Tcl_CreateObjCommand(interp, name, icuFacerecSetWorkingPathRqstSendCb, m, NULL);
      strcpy(name+offset, "FacerecSetWorkingPathRcv");
      Tcl_CreateObjCommand(interp, name, icuFacerecSetWorkingPathReplyRcvCb, m, NULL);
      strcpy(name+offset, "FacerecSetParamsSend");
      Tcl_CreateObjCommand(interp, name, icuFacerecSetParamsRqstSendCb, m, NULL);
      strcpy(name+offset, "FacerecSetParamsRcv");
      Tcl_CreateObjCommand(interp, name, icuFacerecSetParamsReplyRcvCb, m, NULL);
      strcpy(name+offset, "FacerecInitSend");
      Tcl_CreateObjCommand(interp, name, icuFacerecInitRqstSendCb, m, NULL);
      strcpy(name+offset, "FacerecInitRcv");
      Tcl_CreateObjCommand(interp, name, icuFacerecInitReplyRcvCb, m, NULL);
      strcpy(name+offset, "FacerecAddSomebodySend");
      Tcl_CreateObjCommand(interp, name, icuFacerecAddSomebodyRqstSendCb, m, NULL);
      strcpy(name+offset, "FacerecAddSomebodyRcv");
      Tcl_CreateObjCommand(interp, name, icuFacerecAddSomebodyReplyRcvCb, m, NULL);
      strcpy(name+offset, "FacerecSetPeopleSend");
      Tcl_CreateObjCommand(interp, name, icuFacerecSetPeopleRqstSendCb, m, NULL);
      strcpy(name+offset, "FacerecSetPeopleRcv");
      Tcl_CreateObjCommand(interp, name, icuFacerecSetPeopleReplyRcvCb, m, NULL);
      strcpy(name+offset, "FacerecRecognizeAllSend");
      Tcl_CreateObjCommand(interp, name, icuFacerecRecognizeAllRqstSendCb, m, NULL);
      strcpy(name+offset, "FacerecRecognizeAllRcv");
      Tcl_CreateObjCommand(interp, name, icuFacerecRecognizeAllReplyRcvCb, m, NULL);
      strcpy(name+offset, "FacerecLoadSomebodySend");
      Tcl_CreateObjCommand(interp, name, icuFacerecLoadSomebodyRqstSendCb, m, NULL);
      strcpy(name+offset, "FacerecLoadSomebodyRcv");
      Tcl_CreateObjCommand(interp, name, icuFacerecLoadSomebodyReplyRcvCb, m, NULL);
      strcpy(name+offset, "FacerecUnloadSomebodySend");
      Tcl_CreateObjCommand(interp, name, icuFacerecUnloadSomebodyRqstSendCb, m, NULL);
      strcpy(name+offset, "FacerecUnloadSomebodyRcv");
      Tcl_CreateObjCommand(interp, name, icuFacerecUnloadSomebodyReplyRcvCb, m, NULL);
      strcpy(name+offset, "FacerecLoadEverybodySend");
      Tcl_CreateObjCommand(interp, name, icuFacerecLoadEverybodyRqstSendCb, m, NULL);
      strcpy(name+offset, "FacerecLoadEverybodyRcv");
      Tcl_CreateObjCommand(interp, name, icuFacerecLoadEverybodyReplyRcvCb, m, NULL);
      strcpy(name+offset, "FacerecUnloadEverybodySend");
      Tcl_CreateObjCommand(interp, name, icuFacerecUnloadEverybodyRqstSendCb, m, NULL);
      strcpy(name+offset, "FacerecUnloadEverybodyRcv");
      Tcl_CreateObjCommand(interp, name, icuFacerecUnloadEverybodyReplyRcvCb, m, NULL);
      strcpy(name+offset, "WaitForModalityChangeSend");
      Tcl_CreateObjCommand(interp, name, icuWaitForModalityChangeRqstSendCb, m, NULL);
      strcpy(name+offset, "WaitForModalityChangeRcv");
      Tcl_CreateObjCommand(interp, name, icuWaitForModalityChangeReplyRcvCb, m, NULL);
      strcpy(name+offset, "LaunchPlatineSend");
      Tcl_CreateObjCommand(interp, name, icuLaunchPlatineRqstSendCb, m, NULL);
      strcpy(name+offset, "LaunchPlatineRcv");
      Tcl_CreateObjCommand(interp, name, icuLaunchPlatineReplyRcvCb, m, NULL);
      strcpy(name+offset, "CmdPtuSend");
      Tcl_CreateObjCommand(interp, name, icuCmdPtuRqstSendCb, m, NULL);
      strcpy(name+offset, "CmdPtuRcv");
      Tcl_CreateObjCommand(interp, name, icuCmdPtuReplyRcvCb, m, NULL);
      strcpy(name+offset, "IcuInitRFIDSend");
      Tcl_CreateObjCommand(interp, name, icuIcuInitRFIDRqstSendCb, m, NULL);
      strcpy(name+offset, "IcuInitRFIDRcv");
      Tcl_CreateObjCommand(interp, name, icuIcuInitRFIDReplyRcvCb, m, NULL);
      strcpy(name+offset, "IcuCloseRFIDSend");
      Tcl_CreateObjCommand(interp, name, icuIcuCloseRFIDRqstSendCb, m, NULL);
      strcpy(name+offset, "IcuCloseRFIDRcv");
      Tcl_CreateObjCommand(interp, name, icuIcuCloseRFIDReplyRcvCb, m, NULL);
      strcpy(name+offset, "IcuAcquireRFIDSend");
      Tcl_CreateObjCommand(interp, name, icuIcuAcquireRFIDRqstSendCb, m, NULL);
      strcpy(name+offset, "IcuAcquireRFIDRcv");
      Tcl_CreateObjCommand(interp, name, icuIcuAcquireRFIDReplyRcvCb, m, NULL);
      strcpy(name+offset, "IcuAcquireTagSend");
      Tcl_CreateObjCommand(interp, name, icuIcuAcquireTagRqstSendCb, m, NULL);
      strcpy(name+offset, "IcuAcquireTagRcv");
      Tcl_CreateObjCommand(interp, name, icuIcuAcquireTagReplyRcvCb, m, NULL);
      strcpy(name+offset, "ResultshumanPosPosterRead");
      Tcl_CreateObjCommand(interp, name, icuResultshumanPosPosterTclRead, m, NULL);
      strcpy(name+offset, "HumanPresenceloadedPersonListPosterRead");
      Tcl_CreateObjCommand(interp, name, icuHumanPresenceloadedPersonListPosterTclRead, m, NULL);
      strcpy(name+offset, "PlatineCmdcmdPlatinePosterRead");
      Tcl_CreateObjCommand(interp, name, icuPlatineCmdcmdPlatinePosterTclRead, m, NULL);
      strcpy(name+offset, "VisualDataDataPosterRead");
      Tcl_CreateObjCommand(interp, name, icuVisualDataDataPosterTclRead, m, NULL);

   } else {
      strcpy(name+offset, "disconnect");
      Tcl_DeleteCommand(interp, name);

      strcpy(name+offset, "AbortSend");
      Tcl_DeleteCommand(interp, name);
      strcpy(name+offset, "AbortRcv");
      Tcl_DeleteCommand(interp, name);
      strcpy(name+offset, "InitDisplaySend");
      Tcl_DeleteCommand(interp, name);
      strcpy(name+offset, "InitDisplayRcv");
      Tcl_DeleteCommand(interp, name);
      strcpy(name+offset, "PomTaggingSend");
      Tcl_DeleteCommand(interp, name);
      strcpy(name+offset, "PomTaggingRcv");
      Tcl_DeleteCommand(interp, name);
      strcpy(name+offset, "AutoModeSend");
      Tcl_DeleteCommand(interp, name);
      strcpy(name+offset, "AutoModeRcv");
      Tcl_DeleteCommand(interp, name);
      strcpy(name+offset, "SaveFaceSend");
      Tcl_DeleteCommand(interp, name);
      strcpy(name+offset, "SaveFaceRcv");
      Tcl_DeleteCommand(interp, name);
      strcpy(name+offset, "CaptureImageSend");
      Tcl_DeleteCommand(interp, name);
      strcpy(name+offset, "CaptureImageRcv");
      Tcl_DeleteCommand(interp, name);
      strcpy(name+offset, "CaptureVideoSend");
      Tcl_DeleteCommand(interp, name);
      strcpy(name+offset, "CaptureVideoRcv");
      Tcl_DeleteCommand(interp, name);
      strcpy(name+offset, "SetViamPosterNameSend");
      Tcl_DeleteCommand(interp, name);
      strcpy(name+offset, "SetViamPosterNameRcv");
      Tcl_DeleteCommand(interp, name);
      strcpy(name+offset, "EnableDisplaySend");
      Tcl_DeleteCommand(interp, name);
      strcpy(name+offset, "EnableDisplayRcv");
      Tcl_DeleteCommand(interp, name);
      strcpy(name+offset, "SetCameraPosSend");
      Tcl_DeleteCommand(interp, name);
      strcpy(name+offset, "SetCameraPosRcv");
      Tcl_DeleteCommand(interp, name);
      strcpy(name+offset, "TrackItSend");
      Tcl_DeleteCommand(interp, name);
      strcpy(name+offset, "TrackItRcv");
      Tcl_DeleteCommand(interp, name);
      strcpy(name+offset, "WaitForSomebodySend");
      Tcl_DeleteCommand(interp, name);
      strcpy(name+offset, "WaitForSomebodyRcv");
      Tcl_DeleteCommand(interp, name);
      strcpy(name+offset, "WaitForSomeoneSend");
      Tcl_DeleteCommand(interp, name);
      strcpy(name+offset, "WaitForSomeoneRcv");
      Tcl_DeleteCommand(interp, name);
      strcpy(name+offset, "WaitForNobodySend");
      Tcl_DeleteCommand(interp, name);
      strcpy(name+offset, "WaitForNobodyRcv");
      Tcl_DeleteCommand(interp, name);
      strcpy(name+offset, "WaitForNoneSend");
      Tcl_DeleteCommand(interp, name);
      strcpy(name+offset, "WaitForNoneRcv");
      Tcl_DeleteCommand(interp, name);
      strcpy(name+offset, "WaitForNobodyKnownSend");
      Tcl_DeleteCommand(interp, name);
      strcpy(name+offset, "WaitForNobodyKnownRcv");
      Tcl_DeleteCommand(interp, name);
      strcpy(name+offset, "WaitForPersonListChangeSend");
      Tcl_DeleteCommand(interp, name);
      strcpy(name+offset, "WaitForPersonListChangeRcv");
      Tcl_DeleteCommand(interp, name);
      strcpy(name+offset, "ProcessSetModalitySend");
      Tcl_DeleteCommand(interp, name);
      strcpy(name+offset, "ProcessSetModalityRcv");
      Tcl_DeleteCommand(interp, name);
      strcpy(name+offset, "FacerecSetWorkingPathSend");
      Tcl_DeleteCommand(interp, name);
      strcpy(name+offset, "FacerecSetWorkingPathRcv");
      Tcl_DeleteCommand(interp, name);
      strcpy(name+offset, "FacerecSetParamsSend");
      Tcl_DeleteCommand(interp, name);
      strcpy(name+offset, "FacerecSetParamsRcv");
      Tcl_DeleteCommand(interp, name);
      strcpy(name+offset, "FacerecInitSend");
      Tcl_DeleteCommand(interp, name);
      strcpy(name+offset, "FacerecInitRcv");
      Tcl_DeleteCommand(interp, name);
      strcpy(name+offset, "FacerecAddSomebodySend");
      Tcl_DeleteCommand(interp, name);
      strcpy(name+offset, "FacerecAddSomebodyRcv");
      Tcl_DeleteCommand(interp, name);
      strcpy(name+offset, "FacerecSetPeopleSend");
      Tcl_DeleteCommand(interp, name);
      strcpy(name+offset, "FacerecSetPeopleRcv");
      Tcl_DeleteCommand(interp, name);
      strcpy(name+offset, "FacerecRecognizeAllSend");
      Tcl_DeleteCommand(interp, name);
      strcpy(name+offset, "FacerecRecognizeAllRcv");
      Tcl_DeleteCommand(interp, name);
      strcpy(name+offset, "FacerecLoadSomebodySend");
      Tcl_DeleteCommand(interp, name);
      strcpy(name+offset, "FacerecLoadSomebodyRcv");
      Tcl_DeleteCommand(interp, name);
      strcpy(name+offset, "FacerecUnloadSomebodySend");
      Tcl_DeleteCommand(interp, name);
      strcpy(name+offset, "FacerecUnloadSomebodyRcv");
      Tcl_DeleteCommand(interp, name);
      strcpy(name+offset, "FacerecLoadEverybodySend");
      Tcl_DeleteCommand(interp, name);
      strcpy(name+offset, "FacerecLoadEverybodyRcv");
      Tcl_DeleteCommand(interp, name);
      strcpy(name+offset, "FacerecUnloadEverybodySend");
      Tcl_DeleteCommand(interp, name);
      strcpy(name+offset, "FacerecUnloadEverybodyRcv");
      Tcl_DeleteCommand(interp, name);
      strcpy(name+offset, "WaitForModalityChangeSend");
      Tcl_DeleteCommand(interp, name);
      strcpy(name+offset, "WaitForModalityChangeRcv");
      Tcl_DeleteCommand(interp, name);
      strcpy(name+offset, "LaunchPlatineSend");
      Tcl_DeleteCommand(interp, name);
      strcpy(name+offset, "LaunchPlatineRcv");
      Tcl_DeleteCommand(interp, name);
      strcpy(name+offset, "CmdPtuSend");
      Tcl_DeleteCommand(interp, name);
      strcpy(name+offset, "CmdPtuRcv");
      Tcl_DeleteCommand(interp, name);
      strcpy(name+offset, "IcuInitRFIDSend");
      Tcl_DeleteCommand(interp, name);
      strcpy(name+offset, "IcuInitRFIDRcv");
      Tcl_DeleteCommand(interp, name);
      strcpy(name+offset, "IcuCloseRFIDSend");
      Tcl_DeleteCommand(interp, name);
      strcpy(name+offset, "IcuCloseRFIDRcv");
      Tcl_DeleteCommand(interp, name);
      strcpy(name+offset, "IcuAcquireRFIDSend");
      Tcl_DeleteCommand(interp, name);
      strcpy(name+offset, "IcuAcquireRFIDRcv");
      Tcl_DeleteCommand(interp, name);
      strcpy(name+offset, "IcuAcquireTagSend");
      Tcl_DeleteCommand(interp, name);
      strcpy(name+offset, "IcuAcquireTagRcv");
      Tcl_DeleteCommand(interp, name);
      strcpy(name+offset, "ResultshumanPosPosterRead");
      Tcl_DeleteCommand(interp, name);
      strcpy(name+offset, "HumanPresenceloadedPersonListPosterRead");
      Tcl_DeleteCommand(interp, name);
      strcpy(name+offset, "PlatineCmdcmdPlatinePosterRead");
      Tcl_DeleteCommand(interp, name);
      strcpy(name+offset, "VisualDataDataPosterRead");
      Tcl_DeleteCommand(interp, name);

   }

   return TCL_OK;
}
